package com.intuit.sb.cdm.util.v3;


import org.junit.Test;
import java.util.Date;;
import static org.junit.Assert.assertEquals;

public class DateAdapterTest {
    private Date invalidDate = new Date("Sun Dec 31 00:00:00 PST 9999");
    private Date correctDate = new Date("Wed Nov 15 00:00:00 PST 2017");

    @Test
    public void testNull() {
        Date test = new DateAdapter().unmarshal(null);
        assertEquals(test, null);
    }

    @Test
    public void testInvalidDate() {
        Date test = new DateAdapter().unmarshal("2022-32-98");
        assertEquals(invalidDate, test);
    }

    @Test
    public void testShort() {
        Date test = new DateAdapter().unmarshal("2017-02-1");
        assertEquals(invalidDate, test);
    }

    @Test
    public void testLongInvalid() {
        Date test = new DateAdapter().unmarshal("2017-11-345");
        assertEquals(invalidDate, test);
    }

    @Test
    public void testLongValid() {
        Date test = new DateAdapter().unmarshal("2017-11-150");
        assertEquals(correctDate, test);
    }

    @Test
    public void testWithSlashes() {
        Date test = new DateAdapter().unmarshal("2017/11/15");
        assertEquals(correctDate, test);
    }

    @Test
    public void testWithDashes() {
        Date test = new DateAdapter().unmarshal("2017-11-15");
        assertEquals(correctDate, test);
    }

    @Test
    public void testWithDots() {
        Date test = new DateAdapter().unmarshal("2017.11.15");
        assertEquals(correctDate, test);
    }

    @Test
    public void testWithMixOfDashesAndSlashes() {
        Date test = new DateAdapter().unmarshal("2017-11/15");
        assertEquals(correctDate, test);
    }

    @Test
    public void testWithMixOfDashesAndDots() {
        Date test = new DateAdapter().unmarshal("2017-11.15");
        assertEquals(correctDate, test);
    }

    @Test
    public void testWithMixOfSlashesAndDots() {
        Date test = new DateAdapter().unmarshal("2017/11.15");
        assertEquals(correctDate, test);
    }





}
