package com.intuit.sb.payroll.ca.workerpayment.impl;

import java.util.Date;

import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.ca.workerpayment.CanadaWorkerPaymentService;
import com.intuit.sb.payroll.capability.ca.client.dto.PayrollScheduleFrequency;
import com.intuit.sb.payroll.capability.ca.client.mappers.PayrollFrequencyMapper;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.Worker;

@Remote(CanadaWorkerPaymentService.class)
@Stateless(name = "CanadaWorkerPaymentService")
public class CanadaWorkerPaymentEngine implements CanadaWorkerPaymentService
{
	public static final String JNDI_NAME = "CanadaWorkerPaymentService/remote";
	
	private static Logger sLogger = PayrollLoggerFactory.makeLogger(CanadaWorkerPaymentEngine.class);
	
	@EJB(name="PaycheckCalculatorCanada")
	private PaycheckCalculator paycheckCalculator;

	public Money getMaxCPPLimit(Date date) {
		return new Money("CAD", "2118.60");
	}

	public Money getMaxEILimit(Date date) {
		return new Money("CAD", "731.79");
	}

	public Paycheck calculatePaycheck(Worker worker, Employer employer,
			Paycheck paycheck, PayrollScheduleFrequency frequencyDto)
			throws ValidationException, ServiceFailureException 
	{
		
		Paycheck calculated = null;
		try{
			PayrollFrequencyEnum frequency = PayrollFrequencyMapper.mapFromEIToDEntity(frequencyDto);
			calculated = paycheckCalculator.calculatePaycheck(worker, employer, paycheck, frequency);			
		}catch(RuntimeException e)
		{
			sLogger.error("Unexpected Runtime Exception in Service", e);
			throw new ServiceFailureException("Unexpected Error in Service", e);
		}

		return calculated;
		
		/*setupCompensationDetailsInPaycheck(paycheck, worker.getCompensationDtls(), paycheck.getPayPeriod());
		
		Paycheck result = new Paycheck();
		result.setDistributionMethod(paycheck.getDistributionMethod());
		result.setPayPeriod(paycheck.getPayPeriod());
		result.setPayDate(paycheck.getPayDate());
		//Hourly
		PaycheckLineItem hourlyLineItem = new PaycheckLineItem();
		hourlyLineItem.setLawId(CanadaLawID.Regular_Hourly.getLawId());
		hourlyLineItem.setValue(new Money("CAD", "2000.00"));
		PtdDatumType ptdDatumType = new PtdDatumType();
		ptdDatumType.setMoneyValue(new Money("CAD", "2000.00"));
		hourlyLineItem.setPtdRate(ptdDatumType);
		hourlyLineItem.setIncomeSubjectToTax(new Money("CAD", "2000.00"));
		hourlyLineItem.setWageBase(new Money("CAD", "2000.00"));
		result.addPaycheckLineItem(hourlyLineItem);
		
		//EI
		PaycheckLineItem eiLineItem = new PaycheckLineItem();
		eiLineItem.setLawId(CanadaLawID.EI_EE.getLawId());
		eiLineItem.setValue(new Money("CAD", "34.60"));
		PtdDatumType eiptdDatumType = new PtdDatumType();
		eiptdDatumType.setMoneyValue(new Money("CAD", "34.60"));
		eiLineItem.setPtdRate(eiptdDatumType);
		eiLineItem.setIncomeSubjectToTax(new Money("CAD", "2000.00"));
		eiLineItem.setWageBase(new Money("CAD", "2000.00"));
		result.addPaycheckLineItem(eiLineItem);
		
		//CPP
		PaycheckLineItem cppLineItem = new PaycheckLineItem();
		cppLineItem.setLawId(CanadaLawID.CPP_EE.getLawId());
		cppLineItem.setValue(new Money("CAD", "92.37"));
		PtdDatumType cppptdDatumValue = new PtdDatumType();
		cppptdDatumValue.setMoneyValue(new Money("CAD", "92.37"));
		cppLineItem.setPtdRate(cppptdDatumValue);
		cppLineItem.setIncomeSubjectToTax(new Money("CAD", "2000.00"));
		cppLineItem.setWageBase(new Money("CAD", "2000.00"));
		result.addPaycheckLineItem(cppLineItem);
		
		//Fed Tax		
		PaycheckLineItem fedTaxLineItem = new PaycheckLineItem();
		fedTaxLineItem.setLawId(CanadaLawID.FED_TAX.getLawId());
		fedTaxLineItem.setValue(new Money("CAD", "254.72"));
		PtdDatumType fedTaxPtdDatum = new PtdDatumType();
		fedTaxPtdDatum.setMoneyValue(new Money("CAD", "254.72"));
		fedTaxLineItem.setPtdRate(fedTaxPtdDatum);
		fedTaxLineItem.setIncomeSubjectToTax(new Money("CAD", "2000.00"));
		fedTaxLineItem.setWageBase(new Money("CAD", "2000.00"));
		result.addPaycheckLineItem(fedTaxLineItem);
		
		//ON Tax
		PaycheckLineItem onTaxLineItem = new PaycheckLineItem();
		onTaxLineItem.setLawId(CanadaLawID.PROV_TAX_ON.getLawId());
		onTaxLineItem.setValue(new Money("CAD", "134.86"));
		PtdDatumType onTaxPtdDatum = new PtdDatumType();
		onTaxPtdDatum.setMoneyValue(new Money("CAD", "134.86"));
		onTaxLineItem.setPtdRate(onTaxPtdDatum);
		onTaxLineItem.setIncomeSubjectToTax(new Money("CAD", "2000.00"));
		onTaxLineItem.setWageBase(new Money("CAD", "2000.00"));
		result.addPaycheckLineItem(onTaxLineItem);
		return result;*/
	}
	
//	private void setupCompensationDetailsInPaycheck(Paycheck paycheck,
//			Set<CompensationDtl> compensationDetails, DateRange payPeriod) 
//	{
//        for(CompensationDtl compendationDtl : compensationDetails)
//        {
//            PaycheckLineItem lineItem = new PaycheckLineItem();
//            lineItem.setSourcePayrollItemDtl(compendationDtl);
//            lineItem.setLawId(compendationDtl.getPayrollItem().getLawId());
//            lineItem.setName(compendationDtl.getDescription());
//            lineItem.setPayPeriod(payPeriod);
//    		PayrollItemDtl linkedPayrollItemDtl = compendationDtl.getLinkedPayrollItemDtl();
//    		if (linkedPayrollItemDtl != null) {
//    			CompensationDtl linkedCompensationDtl = (CompensationDtl)linkedPayrollItemDtl;
//    			Datum linkedPIRate = linkedCompensationDtl.getRate();    		
//    			BigDecimal linkedRate = new BigDecimal(linkedPIRate.getValue1AsUSDMoneyAmount().toString());
//    			Double inputRate = compendationDtl.getRate().getValue1AsQuantity();
//    			BigDecimal moneyValue = linkedRate.multiply(new BigDecimal(inputRate));    			
//    			Datum rate = new Datum();
//    			rate.setValue1AsUSDMoneyAmount(moneyValue);
//    		    rate.setType2Value2(linkedPIRate.getType2(), linkedPIRate.getValue2());
//    		    lineItem.setRate(rate); 
//    		} else {
//    		    lineItem.setRate(compendationDtl.getRate());     
//    		}
//    		//Quantity
//    		lineItem.setQuantity(compendationDtl.getQuantity());
//    		
//    		//Add the Line Item to Paycheck
//    		paycheck.addPaycheckLineItem(lineItem);
//        	
//        }
//	}

}
