package com.intuit.sb.payroll.capability.ca.client.mappers;

import com.intuit.sb.payroll.capability.ca.client.dto.PayrollScheduleFrequency;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;


/**
 * Converts Pojo and Services Payroll Frequencies.
 *
 * Tested by the PayrollFrequencyMapperTest in the same package.
 *
 * $Author: pszeto $
 * $Revision: #2 $
 * $DateTime: 2009/01/12 15:06:37 $
 *
 */
public class PayrollFrequencyMapper {

    private PayrollFrequencyMapper() {
    }

    /**
     * Maps a Data Entity PayrollFrequency to Entity Interface PayrollScheduleFrequency
     *
     * @param prFrequency
     * @return  Entity Interface PayrollScheduleFrequency
     */
    public static PayrollScheduleFrequency mapFromDEntityToEI(
    				PayrollFrequencyEnum prFrequency) {
        if (prFrequency == null) {
            return null;
        }
        if (prFrequency == PayrollFrequencyEnum.NOTSET) {
            return PayrollScheduleFrequency.NotSet;
        }
        if (prFrequency == PayrollFrequencyEnum.WEEKLY) {
            return PayrollScheduleFrequency.Weekly;
        }
        if (prFrequency == PayrollFrequencyEnum.BIWEEKLY) {
            return PayrollScheduleFrequency.Biweekly;
        }
        if (prFrequency == PayrollFrequencyEnum.MONTHLY) {
            return PayrollScheduleFrequency.Monthly;
        }
        if (prFrequency == PayrollFrequencyEnum.SEMIMONTHLY) {
            return PayrollScheduleFrequency.Semimonthly;
        }
        if (prFrequency == PayrollFrequencyEnum.DAILY) {
            return PayrollScheduleFrequency.Daily;
        }
        return PayrollScheduleFrequency.NotSet;
    }

    /**
     * Maps Entity Interface PayrollScheduleFrequency to Data Entity PayrollFrequency
     *
     * @param eiFrequency Entity Interface PayrollScheduleFrequency to map from
     * @return PayrollFrequency
     */
    public static PayrollFrequencyEnum mapFromEIToDEntity(PayrollScheduleFrequency eiFrequency) {
        if (null == eiFrequency) {
            return null;
        }
        if (eiFrequency.equals(PayrollScheduleFrequency.Biweekly)) {
            return PayrollFrequencyEnum.BIWEEKLY;
        }
        if (eiFrequency.equals(PayrollScheduleFrequency.Monthly)) {
            return PayrollFrequencyEnum.MONTHLY;
        }
        if (eiFrequency.equals(PayrollScheduleFrequency.Semimonthly)) {
            return PayrollFrequencyEnum.SEMIMONTHLY;
        }
        if (eiFrequency.equals(PayrollScheduleFrequency.Weekly)) {
            return PayrollFrequencyEnum.WEEKLY;
        } 
        if (eiFrequency.equals(PayrollScheduleFrequency.Daily)) {
            return PayrollFrequencyEnum.DAILY;
        } 
        else 
        {
            return PayrollFrequencyEnum.NOTSET;
        }
    }
}
