package com.intuit.sb.payroll.capability.ca.workerpayment;

import java.math.BigDecimal;
import java.util.Currency;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.junit.Ignore;

import com.intuit.sb.payroll.ca.workerpayment.CanadaLawID;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.ca.model.TD1Detail;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDocTypeEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.LawEntityTypeEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.PtdDatum;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.model.Tax;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.TaxTypeEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.utils.PtdUtils;

@Ignore
public class BasePaycheckTest {

	protected static final Map<PayrollFrequencyEnum, Integer> frequencyToDaysPerPeriod = new HashMap<PayrollFrequencyEnum, Integer>();
	
	static
	{
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.ANNUALLY, 365);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.SEMIANNUALLY, 365 / 2);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.SEMIMONTHLY, 365 / 13);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.BIWEEKLY, 365 / 26);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.MONTHLY, 365 / 12);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.QUARTERLY, 365 / 4);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.WEEKLY, 7);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.DAILY, 1);
	}
	
	private static final Money sZeroMoney = new Money(Currency.getInstance("CAD").toString(), "0.00");
	
	protected Paycheck createPaycheck(Date payDate, Worker worker,
			DateRange payPeriod, CanadaLawID provincialTaxLaw,
			BigDecimal federalTD1Claim, BigDecimal provincialTD1Claim) {
		Paycheck paycheck = new Paycheck();	    
	    paycheck.setPayPeriod(payPeriod);
        paycheck.setPayDate(payDate);
        
		paycheck.setWorker(worker); 
		
		setupCompensationDetailsInPaycheck(paycheck, worker.getCompensationDtls(), payPeriod);
		
        setupTD1ForWorker(worker, federalTD1Claim, provincialTD1Claim, provincialTaxLaw);
		return paycheck;
	}

	protected Worker createEmployee(Date birthDate, CanadaLawID provincialTaxLaw, String province) {
		
		return createEmployee(birthDate, provincialTaxLaw, province, null, null, null);
	}
	
	protected Worker createEmployee(Date birthDate, CanadaLawID provincialTaxLaw, String province, Date payDate, BigDecimal cppYTD, BigDecimal eiYTD) {
		Worker worker = new Employee();
		PersonName personName = new PersonName();
		personName.setFirstName("Joe");
		personName.setLastName("ThePlumber");
		personName.setTitle("Dr.");
		
		worker.setPersonName(personName);
		worker.setDateOfBirth(birthDate);
		
		setupWorkLocationForWorker(worker, province);
		
        setupTaxDetailForWorker(worker, provincialTaxLaw, payDate, cppYTD, eiYTD);
		
		return worker;
	}
	
	
	protected void setupTaxDetailForWorker(Worker worker, CanadaLawID provincialTaxLaw, Date payDate, 
			BigDecimal cppYTD, BigDecimal eiYTD) {
		
		TaxDtl federalTax = new TaxDtl();
		Tax tax = new Tax();
		tax.setSourceEntity(LawEntityTypeEnum.WORKER);
		tax.setLawId(CanadaLawID.FED_TAX.getLawId());
		tax.setTaxType(TaxTypeEnum.FEDERALTAX);
		federalTax.setPayrollItem(tax);
		worker.addTaxDtl(federalTax);
		
		TaxDtl provincialTax = new TaxDtl();
		tax = new Tax();
		tax.setSourceEntity(LawEntityTypeEnum.WORKER);
		tax.setLawId(provincialTaxLaw.getLawId());
		tax.setTaxType(TaxTypeEnum.STATETAX);
		provincialTax.setPayrollItem(tax);
		worker.addTaxDtl(provincialTax);
		
		TaxDtl eiTax = new TaxDtl();
		tax = new Tax();
		tax.setSourceEntity(LawEntityTypeEnum.WORKER);
		tax.setLawId(CanadaLawID.EI_EE.getLawId());
		tax.setTaxType(TaxTypeEnum.FEDERALTAX);
		eiTax.setPayrollItem(tax);
		
		String currency = LocaleGuru.getInstance().getCurrency();
		
		if (null != eiYTD) {
			PtdDatum yearToDate = new PtdDatum();
			
			// this is just heinous: EI YTD is specified in terms of AMOUNT, not WAGE BASE - EI calcs need WAGE BASE not AMOUNT.
			BigDecimal reverseCalculatedWageBase = eiYTD.divide(new BigDecimal("0.0173"), BigDecimal.ROUND_HALF_UP);
			BigDecimal reverseCalculatedISTT = eiYTD.divide(new BigDecimal("0.0173"), BigDecimal.ROUND_HALF_UP);
			
			PtdDatumType datumType = PtdUtils.newPtdDatumType(PtdTypeEnum.QUARTER1, payDate, new Money(currency, eiYTD.toString()), new Money(currency, reverseCalculatedISTT.toString()), new Money(currency, reverseCalculatedWageBase.toString()), 0.0);
			
			yearToDate.setDatum(datumType);			
			eiTax.getPtdDatums().add(yearToDate);
		}
		
		worker.addTaxDtl(eiTax);
		
		
		TaxDtl cppTax = new TaxDtl();
		tax = new Tax();
		tax.setSourceEntity(LawEntityTypeEnum.WORKER);
		tax.setLawId(CanadaLawID.CPP_EE.getLawId());
		tax.setTaxType(TaxTypeEnum.FEDERALTAX);
		cppTax.setPayrollItem(tax);
		
		if (null != cppYTD) {
			PtdDatum yearToDate = new PtdDatum();
			
			PtdDatumType datumType = PtdUtils.newPtdDatumType(PtdTypeEnum.QUARTER1, payDate, new Money(currency, cppYTD.toString()), sZeroMoney, sZeroMoney, 0.0);
			
			yearToDate.setDatum(datumType);
			cppTax.getPtdDatums().add(yearToDate);
		}
		
		worker.addTaxDtl(cppTax);
	}


	protected void setupTD1ForWorker(Worker worker, BigDecimal federalClaim, BigDecimal provincialClaim, CanadaLawID provincialLawId) 
	{		
		createTD1(worker, federalClaim, BigDecimal.ZERO, 
				CanadaLawID.FED_TAX, ComplianceDocTypeEnum.TD1_CRA);
		
		createTD1(worker, provincialClaim, BigDecimal.ZERO, 
				provincialLawId, ComplianceDocTypeEnum.TD1_PROV);
	}


	protected void createTD1(Worker worker, BigDecimal totalClaimValue, BigDecimal additionalAmountValue, CanadaLawID law, ComplianceDocTypeEnum complianceDocType) {
		Currency CAD = Currency.getInstance(Locale.CANADA); 
        TD1Detail td1 = new TD1Detail();       
        td1.setLawId(law.getLawId());
        td1.setComplianceDocType(complianceDocType);
        
        Money totalClaimAmt = new Money();
        totalClaimAmt.setCurrencyCode(CAD);       
        totalClaimAmt.setAmount(totalClaimValue);
        td1.setTotalClaimAmount(totalClaimAmt);       
        
        Money additionalTax = new Money();
        additionalTax.setCurrencyCode(CAD);
        additionalTax.setAmount(additionalAmountValue);
        td1.setAdditionalAmount(additionalTax);
        worker.addWorkerComplianceDocDtl(td1);
	}


 
	protected void setupCompensationDetailsInPaycheck(Paycheck paycheck,
			Set<CompensationDtl> compensationDetails, DateRange payPeriod) 
	{
        for(CompensationDtl compendationDtl : compensationDetails)
        {
            PaycheckLineItem lineItem = new PaycheckLineItem();
            lineItem.setSourcePayrollItemDtl(compendationDtl);
            lineItem.setLawId(compendationDtl.getPayrollItem().getLawId());
            lineItem.setName(compendationDtl.getDescription());
            lineItem.setPayPeriod(payPeriod);
            
    		PayrollItemDtl linkedPayrollItemDtl = compendationDtl.getLinkedPayrollItemDtl();
    		
    		if (linkedPayrollItemDtl != null) {
    			CompensationDtl linkedCompensationDtl = (CompensationDtl)linkedPayrollItemDtl;
    			Datum linkedPIRate = linkedCompensationDtl.getRate();
    			
    			BigDecimal linkedRate = new BigDecimal(linkedPIRate.getValue1AsCADMoneyAmount().toString());
    			Double inputRate = compendationDtl.getRate().getValue1AsQuantity();
    			BigDecimal moneyValue = linkedRate.multiply(new BigDecimal(inputRate));    			
    			
    			Datum rate = new Datum();
    			rate.setValue1AsCADMoneyAmount(moneyValue);
    		    rate.setType2Value2(linkedPIRate.getType2(), linkedPIRate.getValue2());
    		    
    		    lineItem.setRate(rate); 
    		} else {
    		    lineItem.setRate(compendationDtl.getRate());     
    		}
    		
    		//Quantity
    		lineItem.setQuantity(compendationDtl.getQuantity());
    		
    		//Add the Line Item to Paycheck
    		paycheck.addPaycheckLineItem(lineItem);
        }
	}


	protected void setupHourlyCompensationDetailForWorker(Worker worker, BigDecimal rate, double quantity)
	{
		CompensationDtl hourlyCompensation = new CompensationDtl();
		hourlyCompensation.setCompensationType(CompensationTypeEnum.HOURLYREG);
		hourlyCompensation.setName("Regular Hourly Wages");
		
		Compensation hComp = new Compensation();
		hComp.setLawId(CanadaLawID.Regular_Hourly.getLawId());
		hComp.setCompensationType(CompensationTypeEnum.HOURLYREG);
		hourlyCompensation.setPayrollItem(hComp);
	
		//Set the Rate
		Datum hourlyRate = new Datum();
		hourlyRate.setValue1AsCADMoneyAmount(rate);
		hourlyRate.setType2(DatumType2Enum.HOUR);
		hourlyRate.setValue2(new Double(1));		
		hourlyCompensation.setRate(hourlyRate);
		
		//Now set the Actual Hours
		Datum quantityDatum = new Datum();
	    quantityDatum.setValue1AsQuantity(quantity);
	    quantityDatum.setType2(DatumType2Enum.NOTAPPLICABLE);	    
	    hourlyCompensation.setQuantity(quantityDatum);	    
	    worker.addCompensationDtl(hourlyCompensation);
		
	}
	
	protected void setupHourlyOvertimeCompensationDetailForWorker(Worker worker, BigDecimal rate, double quantity)
	{
		CompensationDtl overtimeCompensation = new CompensationDtl();
		overtimeCompensation.setCompensationType(CompensationTypeEnum.HOURLYOVT);
		overtimeCompensation.setName("Overtime Hourly Wages");
		
		Compensation hComp = new Compensation();
		hComp.setLawId(CanadaLawID.Overtime.getLawId());
		hComp.setCompensationType(CompensationTypeEnum.HOURLYOVT);
		overtimeCompensation.setPayrollItem(hComp);
	
		//Set the Rate
		Datum hourlyRate = new Datum();
		hourlyRate.setValue1AsCADMoneyAmount(rate);
		hourlyRate.setType2(DatumType2Enum.HOUR);
		hourlyRate.setValue2(new Double(1));		
		overtimeCompensation.setRate(hourlyRate);
		
		//Now set the Actual Hours
		Datum quantityDatum = new Datum();
	    quantityDatum.setValue1AsQuantity(quantity);
	    quantityDatum.setType2(DatumType2Enum.NOTAPPLICABLE);	    
	    overtimeCompensation.setQuantity(quantityDatum);
	    
	    worker.addCompensationDtl(overtimeCompensation);
		
	}
	
	protected void setupSalaryCompensationDetailForWorker(Worker worker, BigDecimal salary) {
		
		CompensationDtl salaryCompensation = new CompensationDtl();
		salaryCompensation.setCompensationType(CompensationTypeEnum.SALARYREG);
		salaryCompensation.setName("Regular Salary");
		
		Compensation hComp = new Compensation();
		hComp.setLawId(CanadaLawID.Regular_Salary.getLawId());
		hComp.setCompensationType(CompensationTypeEnum.SALARYREG);
		salaryCompensation.setPayrollItem(hComp);
	
		//Set the Rate
		Datum hourlyRate = new Datum();
		hourlyRate.setValue1AsCADMoneyAmount(salary);
		hourlyRate.setType2(DatumType2Enum.YEAR);
		hourlyRate.setValue2(new Double(1));		
		salaryCompensation.setRate(hourlyRate);
		
		worker.addCompensationDtl(salaryCompensation);
	}
	
	protected void setupWorkLocationForWorker(Worker worker, String province)
	{
		WorkLocation workLocation = new WorkLocation();
		
		GlobalPostalAddress taxAddress = new GlobalPostalAddress();
		taxAddress.setCountryCode(Locale.CANADA.getCountry());
		taxAddress.setCountrySubDivisionCode(province);
		
		workLocation.setTaxResidenceAddress(taxAddress);
		
		worker.setWorkLocation(workLocation);		
		worker.setTaxResidenceAddress(taxAddress);
	}
}
