package com.intuit.sb.payroll.capability.ca.workerpayment;


import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Currency;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import junit.framework.Assert;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.junit.Test;

import com.intuit.sb.payroll.ca.workerpayment.CanadaLawID;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QBOEPaycheckCalculatorBean;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.ca.model.TD1Detail;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDocTypeEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.LawEntityTypeEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.Tax;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.TaxTypeEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;

@SuppressWarnings({"unused"})
public class CalculateHourlyPaycheckNoDeductionsTest extends BasePaycheckTest
{
	private static Logger logger = LogManager.getLogger(CalculateHourlyPaycheckNoDeductionsTest.class);
	
	@Test
	public void test_OntarioHourlyWithoutDeductions()
	{
		BigDecimal amount = new BigDecimal("2000.00");
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), new BigDecimal("2000.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("92.34"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("34.60"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("254.72"), amount, amount),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("134.86"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("25.00"), 
				80.0, PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems); 
	}
	
	@Test
	public void test_OntarioHourlyWithSmallPay()
	{
		BigDecimal amount = new BigDecimal("100.00");
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), amount, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("0.00"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("1.73"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("0.00"), amount, amount),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("0.00"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("1.00"), 
				100.0, PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems); 
	}
	
	@Test
	public void test_AlbertaHourlyWithoutDeductions()
	{
		BigDecimal amount = new BigDecimal("2000.00");
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), new BigDecimal("2000.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("92.34"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("34.60"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("254.72"), amount, amount),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_AB.getLawId(), new BigDecimal("124.52"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "AB", new BigDecimal("25.00"), 
				80.0, PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_AB, new BigDecimal("10100.00"), 
				new BigDecimal("16775.00"), lineItems); 
	}
	
	@Test
	public void test_SaskatchewanHourlyWithoutDeductions()
	{
		BigDecimal amount = new BigDecimal("2000.00");
		
		CanadaLawID provLaw = CanadaLawID.PROV_TAX_SK;
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), new BigDecimal("2000.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("92.34"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("34.60"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("254.72"), amount, amount),//Fed Tax
				new LineItemAmount(provLaw.getLawId(), new BigDecimal("160.96"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "SK", new BigDecimal("25.00"), 
				80.0, PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), provLaw, new BigDecimal("10100.00"), 
				new BigDecimal("13269.00"), lineItems); 
	}
	
	@Test
	public void test_ManitobaHourlyWithoutDeductions()
	{
		BigDecimal amount = new BigDecimal("2000.00");
		
		CanadaLawID provLaw = CanadaLawID.PROV_TAX_MB;
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), new BigDecimal("2000.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("92.34"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("34.60"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("254.72"), amount, amount),//Fed Tax
				new LineItemAmount(provLaw.getLawId(), new BigDecimal("186.10"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "MB", new BigDecimal("25.00"), 
				80.0, PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), provLaw, new BigDecimal("10100.00"), 
				new BigDecimal("8134.00"), lineItems); 
	}
	
	@Test
	public void test_BCHourlyWithoutDeductions()
	{
		BigDecimal amount = new BigDecimal("2000.00");
		
		CanadaLawID provLaw = CanadaLawID.PROV_TAX_BC;
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), new BigDecimal("2000.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("92.34"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("34.60"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("254.72"), amount, amount),//Fed Tax
				new LineItemAmount(provLaw.getLawId(), new BigDecimal("93.94"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "BC", new BigDecimal("25.00"), 
				80.0, PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), provLaw, new BigDecimal("10100.00"), 
				new BigDecimal("9373.00"), lineItems); 
	}

	private void doTest(Date birthDate, String province, BigDecimal hourlyRate, 
			double hoursWorked, PayrollFrequencyEnum payFrequency, Date payDate, 
			CanadaLawID provincialTaxLaw, BigDecimal federalTD1Claim, 
			BigDecimal provincialTD1Claim, LineItemAmount[] lineItems) {
		
		Worker worker = createEmployee(birthDate, provincialTaxLaw, province);
		
		setupHourlyCompensationDetailForWorker(worker, hourlyRate, hoursWorked);
		
		Employer employer = new Employer();	
	    employer.setFirstPayrollDate(birthDate);
	    
	    int daysPerPeriod = frequencyToDaysPerPeriod.get(payFrequency);
		
	    Date payPeriodStartDate = DateUtil.getNextDate(payDate, -1 * daysPerPeriod);
	    
	    DateRange payPeriod = new DateRange(payPeriodStartDate, payDate);
	    
	    Paycheck paycheck = createPaycheck(payDate, worker, payPeriod,
				provincialTaxLaw, federalTD1Claim, provincialTD1Claim);
        
        PaycheckCalculator paycheckCalculator = new QBOEPaycheckCalculatorBean();
        
	    try {
			Paycheck calculated = paycheckCalculator.calculatePaycheck(worker, employer, paycheck, payFrequency);
			
			// verify paycheck line items
			Assert.assertTrue("Expected lineItem value mismatch!", 
					PaycheckUtils.verifyLineItems(calculated.getPaycheckLineItems(),
                    lineItems, "paycheck line items"));
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());			
		}
	}

}
