package com.intuit.sb.payroll.capability.ca.workerpayment;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;

import junit.framework.Assert;

import org.junit.Test;

import com.intuit.sb.payroll.ca.workerpayment.CanadaLawID;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QBOEPaycheckCalculatorBean;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.Worker;

public class CalculateSalaryWithVacationTest extends BasePaycheckTest {

	@Test
	public void test_AlbertaSalaryWithVacation() throws Exception {
		BigDecimal vacPay = new BigDecimal("130.77");
		BigDecimal salary = new BigDecimal("3269.23");
		
		BigDecimal gross = salary.add(vacPay);
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Salary.getLawId(), salary, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.VacationPay.getLawId(), vacPay, new BigDecimal("0.00"), new BigDecimal("0.00")),//Vacation
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("161.64"), gross, gross),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("58.82"), gross, gross),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("579.22"), gross, gross),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("313.65"), gross, gross)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("85000.00"), "4%", PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems); 
	}
	
	private void doTest(Date birthDate, String province, BigDecimal salary, String vacationRate,
			PayrollFrequencyEnum payFrequency, Date payDate, 
			CanadaLawID provincialTaxLaw, BigDecimal federalTD1Claim, 
			BigDecimal provincialTD1Claim, LineItemAmount[] lineItems) {
		
		Worker worker = createEmployee(birthDate, provincialTaxLaw, province);
		
		setupSalaryCompensationDetailForWorker(worker, salary);
		
		CompensationDtl vacDtl = new CompensationDtl();
		vacDtl.setCompensationType(CompensationTypeEnum.VACATIONPAY);
		vacDtl.setName("Vacation Payout");
		Compensation vac = new Compensation();
		vac.setLawId(CanadaLawID.VacationPay.getLawId());
		vac.setCompensationType(CompensationTypeEnum.VACATIONPAY);
		vacDtl.setPayrollItem(vac);
		Datum rate = new Datum();
		rate.setValue1AsPercentage(4.0);
		vacDtl.setRate(rate);
		worker.addCompensationDtl(vacDtl);
		
		
		Employer employer = new Employer();	
	    employer.setFirstPayrollDate(birthDate);
	    
	    int daysPerPeriod = frequencyToDaysPerPeriod.get(payFrequency);
		
	    Date payPeriodStartDate = DateUtil.getNextDate(payDate, -1 * daysPerPeriod);
	    
	    DateRange payPeriod = new DateRange(payPeriodStartDate, payDate);
	    
	    Paycheck paycheck = createPaycheck(payDate, worker, payPeriod,
				provincialTaxLaw, federalTD1Claim, provincialTD1Claim);
        
        PaycheckCalculator paycheckCalculator = new QBOEPaycheckCalculatorBean();
        
	    try {
			Paycheck calculated = paycheckCalculator.calculatePaycheck(worker, employer, paycheck, payFrequency);
			
			// verify paycheck line items
			Assert.assertTrue("Expected lineItem value mismatch!", 
					PaycheckUtils.verifyLineItems(calculated.getPaycheckLineItems(),
                    lineItems, "paycheck line items"));
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());			
		}
	}
}
