package com.intuit.sb.payroll.capability.ca.workerpayment;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;

import junit.framework.Assert;

import org.junit.Test;

import com.intuit.sb.payroll.ca.workerpayment.CanadaLawID;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QBOEPaycheckCalculatorBean;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.Deduction;
import com.intuit.sb.payroll.entity.model.DeductionDtl;
import com.intuit.sb.payroll.entity.model.DeductionTypeEnum;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.HealthBenefit;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.OtherBenefit;
import com.intuit.sb.payroll.entity.model.OtherBenefitDtl;
import com.intuit.sb.payroll.entity.model.OtherBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.RetirementBenefitDtl;
import com.intuit.sb.payroll.entity.model.RetirementBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;

public class TaxableBenefitsTest extends BasePaycheckTest {

	@Test
	public void test_OntarioWithCoPaidHealth() throws Exception {
		BigDecimal grossPay = new BigDecimal("1000.00");
		final BigDecimal benefitAmt = new BigDecimal("100.00");
				
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Salary.getLawId(), grossPay, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CompanyPaidGroupHealth.getLawId(), benefitAmt, new BigDecimal("0.00"), new BigDecimal("0.00")),//Benefit
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("47.79"), grossPay.add(benefitAmt), grossPay.add(benefitAmt)),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("17.30"), grossPay, grossPay),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("90.94"), grossPay.add(benefitAmt), grossPay.add(benefitAmt)),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("53.49"), grossPay.add(benefitAmt), grossPay.add(benefitAmt))//ON Tax
		};
		
		// What I really wanted was a closure. Poor Java... :-)
		CustomPayrollItemAdder itemAdder = new CustomPayrollItemAdder() {

			public void addCustomPayrollItemToWorker(Worker worker) {
				HealthBenefitDtl benefitDtl = new HealthBenefitDtl();
				benefitDtl.setBenefitType(HealthBenefitTypeEnum.COMPANYPAIDHEALTH);
				HealthBenefit benefit = new HealthBenefit();
				benefit.setLawId(CanadaLawID.CompanyPaidGroupHealth.getLawId());
				
				Datum rateDatum = new Datum();
				rateDatum.setValue1AsCADMoneyAmount(benefitAmt);
				
				benefitDtl.setRate(rateDatum);
				benefitDtl.setPayrollItem(benefit);
				worker.addHealthBenefitDtl(benefitDtl);
			}
			
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("26000.00"), benefitAmt, 
				PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems, itemAdder); 
	}
	
	@Test
	public void test_OntarioWithRRSPMatching() throws Exception {
		BigDecimal grossPay = new BigDecimal("1000.00");
		final BigDecimal benefitAmt = new BigDecimal("100.00");
				
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Salary.getLawId(), grossPay, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.RRSPMatching.getLawId(), benefitAmt, new BigDecimal("0.00"), new BigDecimal("0.00")),//Benefit
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("47.79"), grossPay.add(benefitAmt), grossPay.add(benefitAmt)),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("19.03"), grossPay.add(benefitAmt), grossPay.add(benefitAmt)),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("75.68"), grossPay, grossPay),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("47.33"), grossPay, grossPay)//ON Tax
		};
		
		// What I really wanted was a closure. Poor Java... :-)
		CustomPayrollItemAdder itemAdder = new CustomPayrollItemAdder() {

			public void addCustomPayrollItemToWorker(Worker worker) {
				RetirementBenefitDtl benefitDtl = new RetirementBenefitDtl();
				benefitDtl.setBenefitType(RetirementBenefitTypeEnum.RRSP);
				HealthBenefit benefit = new HealthBenefit();
				benefit.setLawId(CanadaLawID.RRSPMatching.getLawId());
				
				Datum rateDatum = new Datum();
				rateDatum.setValue1AsCADMoneyAmount(benefitAmt);
				
				benefitDtl.setRate(rateDatum);
				benefitDtl.setPayrollItem(benefit);
				worker.addRetirementBenefitDtl(benefitDtl);
			}
			
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("26000.00"), benefitAmt, 
				PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems, itemAdder); 
	}
	
	@Test
	public void test_OntarioWithParking() throws Exception {
		BigDecimal grossPay = new BigDecimal("1000.00");
		final BigDecimal benefitAmt = new BigDecimal("100.00");
				
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Salary.getLawId(), grossPay, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.Parking.getLawId(), benefitAmt, new BigDecimal("0.00"), new BigDecimal("0.00")),//Benefit
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("47.79"), grossPay.add(benefitAmt), grossPay.add(benefitAmt)),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("17.30"), grossPay, grossPay),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("90.94"), grossPay.add(benefitAmt), grossPay.add(benefitAmt)),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("53.49"), grossPay.add(benefitAmt), grossPay.add(benefitAmt))//ON Tax
		};
		
		// What I really wanted was a closure. Poor Java... :-)
		CustomPayrollItemAdder itemAdder = new CustomPayrollItemAdder() {

			public void addCustomPayrollItemToWorker(Worker worker) {
				OtherBenefitDtl benefitDtl = new OtherBenefitDtl();
				benefitDtl.setBenefitType(OtherBenefitTypeEnum.PARKING);
				OtherBenefit benefit = new OtherBenefit();
				benefit.setLawId(CanadaLawID.Parking.getLawId());
				
				Datum rateDatum = new Datum();
				rateDatum.setValue1AsCADMoneyAmount(benefitAmt);
				
				benefitDtl.setRate(rateDatum);
				benefitDtl.setPayrollItem(benefit);
				worker.addOtherBenefitDtl(benefitDtl);
			}
			
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("26000.00"), benefitAmt, 
				PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems, itemAdder); 
	}
	
	@Test
	public void test_OntarioWithRRSP() throws Exception {
		BigDecimal grossPay = new BigDecimal("1000.00");
		final BigDecimal benefitAmt = new BigDecimal("100.00");
				
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Salary.getLawId(), grossPay, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.RRSP.getLawId(), benefitAmt, new BigDecimal("0.00"), new BigDecimal("0.00")),//Benefit
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("42.84"), grossPay, grossPay),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("17.30"), grossPay, grossPay),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("61.69"), grossPay.subtract(benefitAmt), grossPay.subtract(benefitAmt)),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("37.99"), grossPay.subtract(benefitAmt), grossPay.subtract(benefitAmt))//ON Tax
		};
		
		// What I really wanted was a closure. Poor Java... :-)
		CustomPayrollItemAdder itemAdder = new CustomPayrollItemAdder() {

			public void addCustomPayrollItemToWorker(Worker worker) {
				
				DeductionDtl deductionDtl = new DeductionDtl();
				deductionDtl.setDeductionType(DeductionTypeEnum.RRSP);
				Deduction deduction = new Deduction();
				deduction.setLawId(CanadaLawID.RRSP.getLawId());
				
				Datum rateDatum = new Datum();
				rateDatum.setValue1AsCADMoneyAmount(benefitAmt);
				
				deductionDtl.setRate(rateDatum);
				
				deductionDtl.setPayrollItem(deduction);
				worker.addDeductionDtl(deductionDtl);
			}
			
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("26000.00"), benefitAmt, 
				PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems, itemAdder); 
	}
	
	@Test
	public void test_OntarioWithAlimony() throws Exception {
		BigDecimal grossPay = new BigDecimal("1000.00");
		final BigDecimal benefitAmt = new BigDecimal("100.00");
				
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Salary.getLawId(), grossPay, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.Alimony.getLawId(), benefitAmt, new BigDecimal("0.00"), new BigDecimal("0.00")),//Benefit
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("42.84"), grossPay, grossPay),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("17.30"), grossPay, grossPay),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("61.69"), grossPay.subtract(benefitAmt), grossPay.subtract(benefitAmt)),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("37.99"), grossPay.subtract(benefitAmt), grossPay.subtract(benefitAmt))//ON Tax
		};
		
		// What I really wanted was a closure. Poor Java... :-)
		CustomPayrollItemAdder itemAdder = new CustomPayrollItemAdder() {

			public void addCustomPayrollItemToWorker(Worker worker) {
				
				DeductionDtl deductionDtl = new DeductionDtl();
				deductionDtl.setDeductionType(DeductionTypeEnum.ALIMONY);
				Deduction deduction = new Deduction();
				deduction.setLawId(CanadaLawID.Alimony.getLawId());
				
				Datum rateDatum = new Datum();
				rateDatum.setValue1AsCADMoneyAmount(benefitAmt);
				
				deductionDtl.setRate(rateDatum);
				
				deductionDtl.setPayrollItem(deduction);
				worker.addDeductionDtl(deductionDtl);
			}
			
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("26000.00"), benefitAmt, 
				PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems, itemAdder); 
	}
	
	@Test
	public void test_OntarioWithSocialFund() throws Exception {
		BigDecimal grossPay = new BigDecimal("1000.00");
		final BigDecimal benefitAmt = new BigDecimal("100.00");
				
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Salary.getLawId(), grossPay, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.SocialFund.getLawId(), benefitAmt, new BigDecimal("0.00"), new BigDecimal("0.00")),//Benefit
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("42.84"), grossPay, grossPay),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("17.30"), grossPay, grossPay),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("76.69"), grossPay, grossPay),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("47.73"), grossPay, grossPay)//ON Tax
		};
		
		// What I really wanted was a closure. Poor Java... :-)
		CustomPayrollItemAdder itemAdder = new CustomPayrollItemAdder() {

			public void addCustomPayrollItemToWorker(Worker worker) {
				
				DeductionDtl deductionDtl = new DeductionDtl();
				deductionDtl.setDeductionType(DeductionTypeEnum.SOCIALFUND);
				Deduction deduction = new Deduction();
				deduction.setLawId(CanadaLawID.SocialFund.getLawId());
				
				Datum rateDatum = new Datum();
				rateDatum.setValue1AsCADMoneyAmount(benefitAmt);
				
				deductionDtl.setRate(rateDatum);
				
				deductionDtl.setPayrollItem(deduction);
				worker.addDeductionDtl(deductionDtl);
			}
			
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("26000.00"), benefitAmt, 
				PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems, itemAdder); 
	}
	
	@Test
	public void test_OntarioWithLoanRepayment() throws Exception {
		BigDecimal grossPay = new BigDecimal("1000.00");
		final BigDecimal benefitAmt = new BigDecimal("100.00");
				
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Salary.getLawId(), grossPay, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.LoanRepayment.getLawId(), benefitAmt, new BigDecimal("0.00"), new BigDecimal("0.00")),//Benefit
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("42.84"), grossPay, grossPay),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("17.30"), grossPay, grossPay),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("76.69"), grossPay, grossPay),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("47.73"), grossPay, grossPay)//ON Tax
		};
		
		// What I really wanted was a closure. Poor Java... :-)
		CustomPayrollItemAdder itemAdder = new CustomPayrollItemAdder() {

			public void addCustomPayrollItemToWorker(Worker worker) {
				
				DeductionDtl deductionDtl = new DeductionDtl();
				deductionDtl.setDeductionType(DeductionTypeEnum.LOANREPAYMENT);
				Deduction deduction = new Deduction();
				deduction.setLawId(CanadaLawID.LoanRepayment.getLawId());
				
				Datum rateDatum = new Datum();
				rateDatum.setValue1AsCADMoneyAmount(benefitAmt);
				
				deductionDtl.setRate(rateDatum);
				
				deductionDtl.setPayrollItem(deduction);
				worker.addDeductionDtl(deductionDtl);
			}
			
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("26000.00"), benefitAmt, 
				PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems, itemAdder); 
	}
	
	private void doTest(Date birthDate, String province, BigDecimal salary, BigDecimal benefitAmt,
			PayrollFrequencyEnum payFrequency, Date payDate, 
			CanadaLawID provincialTaxLaw, BigDecimal federalTD1Claim, 
			BigDecimal provincialTD1Claim, LineItemAmount[] lineItems, CustomPayrollItemAdder itemAdder) {
		
		Worker worker = createEmployee(birthDate, provincialTaxLaw, province);
				
		setupSalaryCompensationDetailForWorker(worker, salary);
		
		itemAdder.addCustomPayrollItemToWorker(worker);
		
		Employer employer = new Employer();	
	    employer.setFirstPayrollDate(birthDate);
	    
	    int daysPerPeriod = frequencyToDaysPerPeriod.get(payFrequency);
		
	    Date payPeriodStartDate = DateUtil.getNextDate(payDate, -1 * daysPerPeriod);
	    
	    DateRange payPeriod = new DateRange(payPeriodStartDate, payDate);
	    
	    Paycheck paycheck = createPaycheck(payDate, worker, payPeriod,
				provincialTaxLaw, federalTD1Claim, provincialTD1Claim);
        
        PaycheckCalculator paycheckCalculator = new QBOEPaycheckCalculatorBean();
        
	    try {
			Paycheck calculated = paycheckCalculator.calculatePaycheck(worker, employer, paycheck, payFrequency);
			
			// verify paycheck line items
			Assert.assertTrue("Expected lineItem value mismatch!", 
					PaycheckUtils.verifyLineItems(calculated.getPaycheckLineItems(),
                    lineItems, "paycheck line items"));
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());			
		}
	}
	
	// I love this interface name...
	private static interface CustomPayrollItemAdder
	{
		public void addCustomPayrollItemToWorker(Worker worker);
	}
}
