package com.intuit.sb.payroll.capability.workerpayments;

import java.util.Set;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollRun;

/**
 * This interface acts as a facade for an underlying calculation provider.  It provides methods for creating individual paychecks (to preview
 * their values) and {@link PayrollRun}s.  
 * 
 * The implementation is expected to be stateless and do no explicit persistence. 
 *
 */
@Local
public interface PaycheckCalculator
{
	@SuppressWarnings("nls")
	public static final String JNDI_NAME = "PaycheckCalculator/local";
    /**
     * Calculates the deductions, net & gross pay for a single paycheck from the supplied paycheck input data. The
     * calculation is completely stateless.
     *  
     * @return The completed paycheck. This is the same object as the {@code paycheck} input parameter. The following
     * data is added to the paycheck as output:
     * <ul> 
     * <li>all line items that appear on the check</li>
     * </ul>
     * @throws ValidationException 
     *  
     * @throws ServiceFailureException Any internal failures that prevent calculation of the paycheck.
     * calculation of the paycheck.
     */
    public Paycheck calculatePaycheck(PayrollRunInputs payRunInputs, PaycheckInputs paycheckInputs) throws ValidationException, ServiceFailureException;
    
    public PayrollRun calculatePayrollRun(PayrollRunInputs payRunInputs, Set<PaycheckInputs> paycheckInputsCollection) throws ValidationException, ServiceFailureException;
    
    public AdjustmentResponse calculateLiabilityAdjustments(AdjustmentRequest request) throws ServiceFailureException;

}