package com.intuit.sb.payroll.capability.workerpayments;

import java.io.Serializable;
import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * A DTO that encapsulates the data needed to create a payroll run. It consists of the workers to be included in the run
 * and the other data that defines the PayrollRun.
 * 
 * The PayrollRun itself should not be created/modified by clients, since the key values in that entity are calculated,
 * and only the domain services should be doing that work.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
public class PayrollRunCreateRequest implements Serializable {

	private static final long serialVersionUID = 8461306420743399793L;

	private List<Long> workerIdList;
	private PayrollRunInfo payrollRunInfo;

	public PayrollRunCreateRequest() {
	}

	public PayrollRunCreateRequest(List<Long> workerIdList, PayrollRunInfo payrollRunInfo) {
		super();
		this.workerIdList = workerIdList;
		this.payrollRunInfo = payrollRunInfo;
	}

	public List<Long> getWorkerIdList() {
		return workerIdList;
	}

	public void setWorkerIdList(List<Long> workerIdList) {
		this.workerIdList = workerIdList;
	}

	public PayrollRunInfo getPayrollRunInfo() {
		return payrollRunInfo;
	}

	public void setPayrollRunInfo(PayrollRunInfo payrollRunInfo) {
		this.payrollRunInfo = payrollRunInfo;
	}

	@Override
	public boolean equals(Object other) {
		return EqualsBuilder.reflectionEquals(this, other);
	}

	@Override
	public int hashCode() {
		return HashCodeBuilder.reflectionHashCode(this);
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}
}
