package com.intuit.sb.payroll.capability.workerpayments;

import java.io.Serializable;
import java.util.Date;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;

/**
 * Seed information required to create a PayrollRun. Used in PayrollRunRequests and also emitted from the PayrollSchedule
 * Component as the result of date calculations.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
public class PayrollRunInfo implements Serializable {

	private static final long serialVersionUID = -4892069223097532560L;
	private DateRange payPeriod;
	private Date payDate;
	private Date payrollRunDate;
	private PayrollFrequencyEnum frequency;
	private boolean partialPayPeriod = false;
	private PayrollRunTypeEnum payrollRunType = PayrollRunTypeEnum.NOTSET;
	private String description;

	public PayrollRunInfo() {
	}

	public PayrollRunInfo(DateRange payPeriod, Date payDate, Date payrollRunDate, PayrollFrequencyEnum frequency,
			PayrollRunTypeEnum payrollRunType) {
		super();
		this.payPeriod = payPeriod;
		this.payDate = payDate;
		this.payrollRunDate = payrollRunDate;
		this.frequency = frequency;
		this.payrollRunType = payrollRunType;
	}
	
	public PayrollRunInfo(DateRange payPeriod, Date payDate, Date payrollRunDate, PayrollFrequencyEnum frequency,
			PayrollRunTypeEnum payrollRunType, boolean isPartialPeriod) {
		super();
		this.payPeriod = payPeriod;
		this.payDate = payDate;
		this.payrollRunDate = payrollRunDate;
		this.frequency = frequency;
		this.payrollRunType = payrollRunType;
		this.partialPayPeriod = isPartialPeriod;
	}

	public Date getPayrollRunDate() {
		return payrollRunDate;
	}

	public void setPayrollRunDate(Date payrollRunDate) {
		this.payrollRunDate = payrollRunDate;
	}

	/**
	 * @return the payDate
	 */
	public Date getPayDate() {
		return payDate;
	}

	/**
	 * @param payDate
	 *            the payDate to set
	 */
	public void setPayDate(Date payDate) {
		this.payDate = payDate;
	}

	public DateRange getPayPeriod() {
		return payPeriod;
	}

	public void setPayPeriod(DateRange payPeriod) {
		this.payPeriod = payPeriod;
	}

	/**
	 * @return Frequency component
	 */
	public PayrollFrequencyEnum getFrequency() {
		return this.frequency;
	}

	/**
	 * @param frequency
	 *            frequency of the payment
	 */
	public void setFrequency(PayrollFrequencyEnum frequency) {
		this.frequency = frequency;
	}

	public void setPayrollRunType(PayrollRunTypeEnum payrollRunType) {
		this.payrollRunType = payrollRunType;
	}

	public PayrollRunTypeEnum getPayrollRunType() {
		return payrollRunType;
	}
	

	public void setPartialPayPeriod(boolean partialPayPeriod) {
		this.partialPayPeriod = partialPayPeriod;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getDescription() {
		return description;
	}

	public boolean isPartialPayPeriod() {
		return partialPayPeriod;
	}
	
	@Override
	public boolean equals(Object other) {
		return EqualsBuilder.reflectionEquals(this, other);
	}

	@Override
	public int hashCode() {
		return HashCodeBuilder.reflectionHashCode(this);
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}
}
