package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.text.MessageFormat;
import java.util.Date;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculationRequest;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;

/**
 * Validates {@link PaycheckCalculationRequest} beans for integrity before processing.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class PaycheckCalculationRequestValidator {
	
	private PaycheckCalculationRequestValidator() {}

	/**
	 * Validates that a PaycheckCalcuationRequest has the minimal information required for processing.
	 * 
	 * @param request
	 * @throws ValidationException
	 */
    static void validatePaycheckCalculationRequest(PaycheckCalculationRequest request) throws ValidationException {
        if (null == request) {
            throw new ValidationException("PaycheckCalculationRequest is null");
        }
        if (null == request.getPaycheckDate()) {
            throw new ValidationException("PaycheckDate in PaycheckCalculationRequest is null");
        }
        if (null == request.getPayPeriod()) {
            throw new ValidationException("PayPeriod in PaycheckCalculationRequest is null");
        }
        
        //Date Validations
        Date startDate = request.getPayPeriod().getStartDate();
		if (null == startDate) {
            throw new ValidationException("PayPeriod StartDate in PaycheckCalculationRequest is null");
        }
        Date endDate = request.getPayPeriod().getEndDate();
		if (null == endDate) {
            throw new ValidationException("PayPeriod EndDate in PaycheckCalculationRequest is null");
        }

		if (startDate.after(endDate)) {
			throw new ValidationException(MessageFormat.format(
					"Paycheck start date {0} must be before the end date {1}.", startDate, endDate));
		}

		if (null == request.getWorkerId()) {
            throw new ValidationException("WorkerId in PaycheckCalculationRequest is null");
        }
        if (0 == request.getWorkerId()) {
            throw new ValidationException("WorkerId in PaycheckCalculationRequest is invalid");
        }
        if (null == request.getFrequency()) {
            throw new ValidationException("PayrollFrequency in PaycheckCalculationRequest is null");
        }
        if (PayrollFrequencyEnum.NOTSET == request.getFrequency()) {
            throw new ValidationException("PayrollFrequency in PaycheckCalculationRequest is not set");
        }
    }
}
