package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunCreateRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;

/**
 * Validates fields in the PayrollRunRequest object.
 * 
 * $Author: pszeto $ 
 * $Revision: #8 $ 
 * $DateTime: 2009/03/06 14:49:56 $
 * 
 */
public class PayrollRunCreateRequestValidator extends AbstractEntityDataValidator<PayrollRunCreateRequest> {
	private static PayrollRunCreateRequestValidator sInstance = new PayrollRunCreateRequestValidator();

	private PayrollRunCreateRequestValidator() {
	}

	public static PayrollRunCreateRequestValidator getInstance() {
		return sInstance;
	}

	@Override
	public void validateEntityCollection(List<PayrollRunCreateRequest> payRunCollection) throws ValidationException {
		for (PayrollRunCreateRequest payrRun : payRunCollection) {
			validateEntityImpl(payrRun);
		}
	}

	@Override
	public void validateEntity(PayrollRunCreateRequest payRun) throws ValidationException {
		validateEntityImpl(payRun);
	}

	public void validateEntityImpl(PayrollRunCreateRequest payRun) throws ValidationException {
		if (payRun == null) {
			throw new ValidationException("PayrollRunCreateRequest can not be null ..");
		}
		
		if(payRun.getWorkerIdList().size() <= 0)
		{
			throw new ValidationException("You should have at least 1 worker in the workerList to create a payroll run");
		}
		
		PayrollRunInfo payrollRunInfo = payRun.getPayrollRunInfo();
		PayrollRunInfoValidator.getInstance().validateEntity(payrollRunInfo);
	}

}
