package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.management.PayrollService;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;

public class PayrollRunInfoValidator extends AbstractEntityDataValidator<PayrollRunInfo> {

	private static PayrollRunInfoValidator sInstance = new PayrollRunInfoValidator();

	public static PayrollRunInfoValidator getInstance() {
		return sInstance;
	}
	@Override
	public void validateEntity(PayrollRunInfo entity) throws ValidationException {
		validateEntityImpl(entity);
	}

	@Override
	public void validateEntityCollection(List<PayrollRunInfo> entityCollection) throws ValidationException {
		for (PayrollRunInfo payrollRunInfo : entityCollection) {
			validateEntityImpl(payrollRunInfo);
		}		
	}

	private void validateEntityImpl(PayrollRunInfo payrollRunInfo) throws ValidationException {
		if (payrollRunInfo == null) {
			throw new ValidationException("PayrollRunInfo can not be null ..");
		}
		
		if ((null == payrollRunInfo.getPayrollRunType()) ||
			(payrollRunInfo.getPayrollRunType() == PayrollRunTypeEnum.NOTSET)) {
			throw new ValidationException("PayrollRuntype must be set");
		}
		
		if (payrollRunInfo.getPayrollRunType() != PayrollRunTypeEnum.TERMINATION)
		{
			validatePayPeriod(payrollRunInfo);
		}

	}

	private void validatePayPeriod(PayrollRunInfo payrollRunInfo) throws ValidationException {
		DateRange period = payrollRunInfo.getPayPeriod();
		if(period == null)
		{
			throw new ValidationException("payPeriod is null");
		}
		Date startDate = period.getStartDate();
		Date endDate = period.getEndDate();

		Date runDate = payrollRunInfo.getPayrollRunDate();
		Date payDate = payrollRunInfo.getPayDate();

		if (startDate == null) {
			throw new ValidationException("periodStartDate is null");
		}
		if (endDate == null) {
			throw new ValidationException("periodEndDate is null");
		}
		if (runDate == null) {
			throw new ValidationException("runDate is null");
		}
		if (payDate == null) {
			throw new ValidationException("paydate is null");
		}

		// end date can not be before begin date
		if (endDate.before(startDate)) {
			throw new ValidationException(
					"Invalid PayrollRunPeriod: Pay period end date can not be before Pay period begin date");
		}

		// paycheck date can not be before today.
		Calendar runDateCal = DateUtil.dateToCalendar(runDate);
		Calendar payDateCal = DateUtil.dateToCalendar(payDate);
		if (DateUtil.isSameDay(runDateCal, payDateCal) || payDateCal.after(runDateCal)) {
			// you may pass
		} else {
			throw new ValidationException("Invalid PayrollRunPeriod: Paycheck date can not be before run date.");
		}
		
		// paycheck date can not be before today.
		Calendar nowDateCal = PayrollService.getInstance().getSystemDate();
		if (DateUtil.isSameDay(nowDateCal, payDateCal) || payDateCal.after(nowDateCal)) {
			// you may pass
		} else {
			throw new ValidationException("Invalid PayrollRunPeriod: Paycheck date can not be before today.");
		}

		// validate the frequency is supported
		PayrollFrequencyEnum payFreq = payrollRunInfo.getFrequency();
		switch (payFreq) {
			case WEEKLY:
			case BIWEEKLY:
			case SEMIMONTHLY:
			case MONTHLY:
			case QUARTERLY:
				break;

			default:
				throw new ValidationException("PayrollScheduleFrequency specified is not a valid one");
		}
	}

}
