package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;

/**
 * Validates PayrollSchedule Data.
 * 
 * This class has been resourced and can pass PMD muster.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
@SuppressWarnings("nls")
public class PayrollScheduleValidator extends AbstractEntityDataValidator<PayrollSchedule> {

	private static PayrollScheduleValidator sInstance = new PayrollScheduleValidator();
	
	private PayrollScheduleValidator() {}
	
	public static PayrollScheduleValidator getInstance() {
		return sInstance;
	}
	
	@Override
	public void validateEntity(PayrollSchedule payrollSchedule) throws ValidationException {

		if (payrollSchedule == null) {
			throw new ValidationException(PayrollScheduleResources.getString("PayrollScheduleValidator.Schedule.Null")); //$NON-NLS-1$
		}
		
		PayrollFrequencyEnum payFrequency = payrollSchedule.getFrequency();
		if (payFrequency == null) {
			throw new ValidationException(PayrollScheduleResources.getString("PayrollScheduleValidator.Frequency.Null")); //$NON-NLS-1$
		}

		if (payFrequency == PayrollFrequencyEnum.NOTSET) {
			throw new ValidationException(PayrollScheduleResources.getString("PayrollScheduleValidator.Frequency.NotSet")); //$NON-NLS-1$
		}

		if (payrollSchedule.getPayDayOffset() < payrollSchedule.getPayrollRunDayOffset()) {
			throw new ValidationException(PayrollScheduleResources.getString("PayrollScheduleValidator.PayRundate.Invalid")); //$NON-NLS-1$
		}
		
		if (payFrequency == PayrollFrequencyEnum.MONTHLY) {
			if (!validatePayPeriodEndDayofMonth(payrollSchedule.getPayPeriodEndDayofMonth1())) {
				throw new ValidationException(
						PayrollScheduleResources.getString("PayrollScheduleValidator.LastDay.Invalid")); //$NON-NLS-1$
			}

		} else if (payFrequency == PayrollFrequencyEnum.SEMIMONTHLY) {
			if (!validatePayPeriodEndDayofMonth(payrollSchedule.getPayPeriodEndDayofMonth1())) {
				throw new ValidationException(
						PayrollScheduleResources.getString("PayrollScheduleValidator.EndDay.Invalid")); //$NON-NLS-1$
			}
			if (!validatePayPeriodEndDayofMonth(payrollSchedule.getPayPeriodEndDayofMonth2())) {
				throw new ValidationException(
						PayrollScheduleResources.getString("PayrollScheduleValidator.EndDay2.Invalid")); //$NON-NLS-1$
			}
			if (payrollSchedule.getPayPeriodEndDayofMonth2() <= payrollSchedule.getPayPeriodEndDayofMonth1()) {
				throw new ValidationException(PayrollScheduleResources.getString("PayrollScheduleValidator.EndDate2.AfterEndDate1")); //$NON-NLS-1$
			}
		}
		// Neither monthly nor semi monthly
		else {
			//TODO Not sure whether or not the following validation is correct, need to investigate 
//			if (payrollSchedule.getPayrollRunDayOffset() + payrollSchedule.getPayDayOffset() < -3) {
//				throw new ValidationException(PayrollScheduleResources.getString("PayrollScheduleValidator.Paydate.Invalid")); //$NON-NLS-1$
//			}
			if (null == payrollSchedule.getNextPayPeriodEndDate()) {
				throw new ValidationException(PayrollScheduleResources.getString("PayrollScheduleValidator.EndDate.NotSet")); //$NON-NLS-1$
			}
		}

	}

	@Override
	public void validateEntityCollection(List<PayrollSchedule> entityCollection) throws ValidationException {
		for (PayrollSchedule payrollSchedule : entityCollection) {
			validateEntity(payrollSchedule);
		}
	}

	private boolean validatePayPeriodEndDayofMonth(int day) {
		if ((day < 1 || day > 28) && day != PayrollSchedule.LAST_DAY_OF_MONTH) {
			return false;
		} else {
			return true;
		}
	}
}
