package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.text.MessageFormat;
import java.util.Set;

import org.apache.commons.lang.math.RandomUtils;

import com.intuit.dataTypes.QBDate;
import com.intuit.dataTypes.Rate;
import com.intuit.payroll.j.CompanyIf;
import com.intuit.payroll.j.PayrollDatumIf;
import com.intuit.payroll.j.PayrollFactory;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.common.management.PayrollService;
import com.intuit.sb.payroll.entity.model.DisabilityInsDocDtl;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerComplianceDocDtl;
import com.intuit.sb.payroll.entity.model.UnemploymentInsDocDtl;

public class CompanyFactory {
	
	static CompanyIf createCompany(PayrollEngineContext context) {
		PayrollItemInfo payrollItemInfo = context.getPayrollItemInfo();
		Employer employer = context.getEmployer();
		CompanyIf qboeCompany = PayrollFactory.createCompany(new QBDate(employer.getFirstPayrollDate()), String.valueOf(RandomUtils.nextInt()), payrollItemInfo, payrollItemInfo);
		// Set the System Date override.
		qboeCompany.setSystemDate(new QBDate(PayrollService.getInstance().getSystemDate().getTime()));
		payrollItemInfo.setUserTaxability(qboeCompany.createUserTaxability());
		//Set the created company on the context - it is used downstream extensively.
		context.setQboCompany(qboeCompany);
		

		return qboeCompany;
	}

	/**
	 * Use this outside the context of a payroll run or calc - this ensures the rates are set 
	 * on the company.  The payroll run/calc process does this implicitly by adding line items to the checks.
	 * 
	 * FIXME find a more elegant way to bring this together. - It's in the handlers baby!
	 * @param context
	 */
	static void applyTaxRatesToCompany(PayrollEngineContext context) {
		Employer employer = context.getEmployer();
	
		//Set the rate on the UI payroll item. 
		Set<EmployerComplianceDocDtl> employerComplianceDocDtls = employer.getEmployerComplianceDocDtls();
		for (EmployerComplianceDocDtl employerComplianceDocDtl : employerComplianceDocDtls) {
			String docId = employerComplianceDocDtl.getLawId().getValue();
			String rate;
			if (employerComplianceDocDtl instanceof UnemploymentInsDocDtl) {
				rate = ((UnemploymentInsDocDtl)employerComplianceDocDtl).getPercentageRate().toString() + "%";
			} else if (employerComplianceDocDtl instanceof DisabilityInsDocDtl) {
				rate = ((DisabilityInsDocDtl)employerComplianceDocDtl).getPercentageRate().toString() + "%";
			} else {
				String mesg = MessageFormat.format("Unexpected company compliance type ''{0}'' (LawId={1}).",
						new Object[] {employerComplianceDocDtl.getClass().getName(), docId});
				throw new IllegalArgumentException(mesg);
			}
			Rate uiRate = new Rate(rate);
			PayrollItemIf payrollItemIf = context.getPayrollItemForLaw(docId);
			PayrollDatumIf qboeRateContainer = payrollItemIf.getDatum(DatumID.RATE);

			if (qboeRateContainer.isEditable()) {
				qboeRateContainer.setValue(uiRate);
			}
		}
	}
}
