package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import com.intuit.dataTypes.QBDate;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.common.i18n.LocaleObjectFactory;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.Worker;

public class EmployeeFactory {

	static EmployeeIf createQboEmployee(PayrollEngineContext context, Worker worker, PayrollFrequencyEnum frequency) {
		EmployeeIf qboeEmployee = context.getQboCompany().createEmployee();
		
		qboeEmployee.setDatumValue(DatumID.PAY_FREQUENCY, PayrollFrequencyMapper.mapToPayFrequency(frequency));
		
		NationalRegionMapper regionMapper = LocaleObjectFactory.getInstance().newInstance(NationalRegionMapper.class);

		final GlobalPostalAddress workAddress = worker.getWorkLocation().getTaxResidenceAddress();
		qboeEmployee.setDatumValue(DatumID.WORK_LOCATION, regionMapper.mapToNationalRegion(workAddress));

		final GlobalPostalAddress liveAddress = worker.getTaxResidenceAddress();
		qboeEmployee.setDatumValue(DatumID.LIVE_LOCATION, regionMapper.mapToNationalRegion(liveAddress));
		
		final DateRange employmentPeriod = worker.getEmploymentPeriod();
		if (null != employmentPeriod) {
			qboeEmployee.setDatumValue(DatumID.HIRE_DATE, new QBDate(employmentPeriod.getStartDate()));
		}
		
		if (null != worker.getDateOfBirth()) {
			qboeEmployee.setDatumValue(DatumID.BIRTH_DATE, new QBDate(worker.getDateOfBirth()));
		}
		
		return qboeEmployee;
	}
}
