package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.text.MessageFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import com.intuit.dataTypes.Money;
import com.intuit.dataTypes.TimeDuration;
import com.intuit.payroll.j.DateRangeIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.j.PayrollItemTotalsCollectionIf;
import com.intuit.payroll.j.PayrollItemTotalsIf;
import com.intuit.payroll.j.PayrollTotalIf;
import com.intuit.payroll.types.PayrollException;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.payroll.types.enumtype.PaycheckSetupType;
import com.intuit.payroll.types.enumtype.TotalsTransactionType;
import com.intuit.sb.payroll.capability.workerpayments.CalculationEngineException;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.DatumType1Enum;
import com.intuit.sb.payroll.entity.model.DeductionDtl;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PaycheckLineItemSortSequenceEnum;
import com.intuit.sb.payroll.entity.model.PaycheckLitemPtdDatum;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PayrollItemDtlPtdDatum;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.model.Tax;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.utils.PtdUtils;

/**
 * Maps Paychecks to and From Domain Services and QBO models.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class PaycheckMapper {
	private static final String US_DOLLAR = "USD"; 
	private static final String ZERO_AMOUNT = "0.00";

	private PaycheckMapper() {}
	//private static Logger sLogger = PayrollLoggerFactory.makeLogger(PaycheckMapper.class);
	
	public static void mapPaycheckResults(PaycheckCalculationContext context, boolean updatePayItemPtd) {
		
		PayrollItemTotalsCollectionIf qboeTotalsCollection = context.getQboeEmployee().getTotalsCollection();
		
		try {
			Paycheck paycheck = context.getPaycheck();
			
			Collection<PaycheckLineIf> qboePaycheckLines = context.getQboePaycheck().getPaycheckLines();
			for (Iterator<PaycheckLineIf> iterator = qboePaycheckLines.iterator(); iterator.hasNext();) {
				PaycheckLineIf qboePaycheckLine = iterator.next();
				PaycheckLineItem paycheckLineItem = context.getPaycheckLineItemMap().get(qboePaycheckLine);
				if(paycheckLineItem == null) {
					continue;
				}
				PayrollItemTotalsIf qboeLineTotals = qboeTotalsCollection.getTotals(qboePaycheckLine.getPayrollItem());
				mapPayCheckLineItem(qboePaycheckLine, paycheckLineItem);
				mapPaycheckLineItemYTD(qboeLineTotals, context.getWorker(), paycheck.getPayDate(), paycheckLineItem);
				if (updatePayItemPtd) {
					// Copy all the PTD datums from the paycheck line to the payroll item dtl
					for(PaycheckLitemPtdDatum ptdDatum : paycheckLineItem.getPtdDatums()) {
						PayrollItemDtlPtdDatum payrollItemPtdDatum = paycheckLineItem.getSourcePayrollItemDtl().getPtdDatumOfType(ptdDatum.getDatum().getPtdType());
						// PTO payrollItems can contain both MONEY and QUANTITY ptdDatums, we don't want to set a
						//   MONEY value into a QUANTITY ptdDatum
						if (DatumType1Enum.QUANTITY != payrollItemPtdDatum.getDatum().getDatumType()) {
							payrollItemPtdDatum.setDatum(ptdDatum.getDatum());
						}
					}
				}
			}
			
			// Gross Pay
			Money grossPay = sumPaycheckLineItems(qboePaycheckLines, PaycheckSetupType.EARNING);
			paycheck.setGrossAmount(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, grossPay.toString()));

			// Net pay amount
			Money netPay = context.getQboePaycheck().getNetAmount();
			if (netPay == null) {
				throw new CalculationEngineException("Unable to obtain net pay from paycheck");
			}
			paycheck.setNetAmount(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, netPay.toString()));

			// Total deductions
			Money totalDeductions = sumPaycheckLineItems(qboePaycheckLines, PaycheckSetupType.DEDUCTION);
			totalDeductions.add(sumPaycheckLineItems(qboePaycheckLines, PaycheckSetupType.EMPLOYEE_PAID_TAX));
			paycheck.setDeductionAmount(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, totalDeductions.toString()));
			
			// Worker Paid Deductions
			Money workerPdBenefits = sumPaycheckLineItems(qboePaycheckLines, PaycheckSetupType.DEDUCTION);
			paycheck.setWorkerPdBenefits(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, workerPdBenefits.toString()));
			
			// Worker Paid Taxes
			Money workerPdTaxes = sumPaycheckLineItems(qboePaycheckLines, PaycheckSetupType.EMPLOYEE_PAID_TAX);
			paycheck.setWorkerPdTaxes(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, workerPdTaxes.toString()));
			
			// Employer Paid Benefits
			Money employerPdBenefits = sumPaycheckLineItems(qboePaycheckLines, PaycheckSetupType.COMPANY_PAID_NON_TAXABLE);
			employerPdBenefits.add(sumPaycheckLineItems(qboePaycheckLines, PaycheckSetupType.COMPANY_PAID_TAXABLE));
			paycheck.setEmployerPdBenefits(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, employerPdBenefits.toString()));
			
			// Employer Paid Taxes
			Money employerPdTaxes = sumPaycheckLineItems(qboePaycheckLines, PaycheckSetupType.COMPANY_PAID_TAX);
			paycheck.setEmployerPdTaxes(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, employerPdTaxes.toString()));
		} catch (PayrollException pe) {
			String mesg = MessageFormat.format("Unable to obtain paycheck results: {0}", pe.getMessage());
			throw new CalculationEngineException(mesg);
		}
	}

	private static void mapPaycheckLineItemYTD(
			PayrollItemTotalsIf qboeLineTotals, Worker worker, Date payDate,
			PaycheckLineItem paycheckLineItem) {
		
		// For each period (Q1-4, YTD), set the corresponding totals to the line item
		copyPtdInfoToPaycheckLine(PtdTypeEnum.QUARTER1, payDate, paycheckLineItem, qboeLineTotals);
		copyPtdInfoToPaycheckLine(PtdTypeEnum.QUARTER2, payDate, paycheckLineItem, qboeLineTotals);
		copyPtdInfoToPaycheckLine(PtdTypeEnum.QUARTER3, payDate, paycheckLineItem, qboeLineTotals);
		copyPtdInfoToPaycheckLine(PtdTypeEnum.QUARTER4, payDate, paycheckLineItem, qboeLineTotals);
		copyPtdInfoToPaycheckLine(PtdTypeEnum.YEAR, payDate, paycheckLineItem, qboeLineTotals);
	}

	private static void copyPtdInfoToPaycheckLine(PtdTypeEnum type,
			Date payDate, PaycheckLineItem paycheckLineItem,
			PayrollItemTotalsIf qboeLineTotals) {
		DateRangeIf period = QBOEConvert.convertGeminiDateRangeToQboeDateRange(type.getDateRange(payDate));
		
		PtdDatumType ptdDatumType = PtdUtils.newPtdDatumType(type, payDate, 
				getMoneyValueFromTotals(qboeLineTotals, period, DatumID.TOTAL_AMOUNT), 
				getMoneyValueFromTotals(qboeLineTotals, period, DatumID.TOTAL_INCOME_SUBJECT_TO_TAX), 
				getMoneyValueFromTotals(qboeLineTotals, period, DatumID.TOTAL_WAGE_BASE), 
				Double.valueOf(getTimeDurationValueFromTotals(qboeLineTotals, period, DatumID.TOTAL_HOURS).getHours()));
				
		paycheckLineItem.getPtdDatumOfType(type).setDatum(ptdDatumType);
	}
	
	private static com.intuit.sb.payroll.entity.model.Money getMoneyValueFromTotals(PayrollItemTotalsIf qboeLineTotals, 
			DateRangeIf period, DatumID datumId) {
		
		com.intuit.sb.payroll.entity.model.Money ytdAmount;
		
		PayrollTotalIf datum = qboeLineTotals.getDatum(datumId, TotalsTransactionType.ALL, period);
		if (null == datum) {
			ytdAmount = new com.intuit.sb.payroll.entity.model.Money(ZERO_AMOUNT);
		} else {
			ytdAmount = new com.intuit.sb.payroll.entity.model.Money(datum.getValue().toString());
		}
		
		return ytdAmount;
	}
	
	private static TimeDuration getTimeDurationValueFromTotals(PayrollItemTotalsIf qboeLineTotals, 
			DateRangeIf period, DatumID datumId) {
		
		TimeDuration rv;
		
		PayrollTotalIf datum = qboeLineTotals.getDatum(datumId, TotalsTransactionType.ALL, period);
		
		if (null == datum) {
			rv = new TimeDuration(0);
		} else {
			rv = (TimeDuration) datum.getValue();
		}
		
		return rv;
	}

	/*
	 * Calculate the sum of the amounts in a collection of paycheck line items.
	 */
	private static Money sumPaycheckLineItems(Collection<PaycheckLineIf> qboePaycheckLines, PaycheckSetupType tableType)
			throws PayrollException {

		Money total = new Money();
		QBOEFilter tableFilter = QBOEPayrollItemFilter.create(tableType);
		for (QBOEFilteredIterator iterator = new QBOEFilteredIterator(qboePaycheckLines.iterator(), tableFilter); iterator
				.hasNext();) {
			PaycheckLineIf qboePaycheckLine = (PaycheckLineIf) iterator.next();
			total = total.add((Money) qboePaycheckLine.getDatumValue(DatumID.AMOUNT));
		}
		return total;
	}
	
	private static PaycheckLineItem mapPayCheckLineItem(PaycheckLineIf qboePaycheckLine, PaycheckLineItem paycheckLineitem)
			throws PayrollException {

		PaycheckLineItemSortSequenceEnum sortSequence = getSortSequence(paycheckLineitem);
		paycheckLineitem.setSortSequence(sortSequence);
		String lawId = new Integer(qboePaycheckLine.getPayrollItem().getDetailType().toInt()).toString();
		paycheckLineitem.getLawId().setValue(lawId);
		paycheckLineitem.setName(paycheckLineitem.getSourcePayrollItemDtl().getDescription());
		
		//Rate
//		Object qboDatum = qboePaycheckLine.getDatumValue(DatumID.RATE);
//		if (qboDatum != null) {
//			try {
//			Datum rate = new Datum();
//			rate.setValue1AsUSDMoneyAmount(new BigDecimal(qboDatum.toString()));
//			paycheckLineitem.setRate(rate);
//			} catch (Exception e) {
//				e.printStackTrace();
//				System.err.println("Unable to set money rate for " + qboePaycheckLine.getDatum(DatumID.RATE));
//				//probably not a money rate?
//			}
//		}
		
		// Amount
		Object amount = qboePaycheckLine.getDatumValue(DatumID.AMOUNT);
		if (amount == null) {
			amount = ZERO_AMOUNT;
		}
		paycheckLineitem.setValue(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, amount.toString()));

		// ISTT
		Object istt = qboePaycheckLine.getDatumValue(DatumID.INCOME_SUBJECT_TO_TAX);
		if (null == istt) {
			istt = ZERO_AMOUNT;
		}
		paycheckLineitem.setIncomeSubjectToTax(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, istt.toString()));

		// Wage Base
		Object wageBase = qboePaycheckLine.getDatumValue(DatumID.WAGE_BASE);
		if (null == wageBase) {
			wageBase = ZERO_AMOUNT;
		}
		paycheckLineitem.setWageBase(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, wageBase.toString()));

		// Exempt Amount
		Object exemptAmount = qboePaycheckLine.getDatumValue(DatumID.EXEMPTION_AMOUNT);
		if (null == exemptAmount) {
			exemptAmount = ZERO_AMOUNT;
		}
		paycheckLineitem.setExemptAmount(new com.intuit.sb.payroll.entity.model.Money(US_DOLLAR, exemptAmount.toString()));

		return paycheckLineitem;
	}

	/**
	 * This is our chance to categorize items as we please.
	 * 
	 * Add the appropriate if's to support your pay items.
	 * 
	 * @param paycheckLineitem
	 * @return the PaycheckLineItemSortSequenceEnum that is relevant.
	 */
	private static PaycheckLineItemSortSequenceEnum getSortSequence(PaycheckLineItem paycheckLineitem) {
		PayrollItemDtl sourcePayrollItemDtl = paycheckLineitem.getSourcePayrollItemDtl();
		if (sourcePayrollItemDtl instanceof CompensationDtl) {
			return PaycheckLineItemSortSequenceEnum.WAGES;
		} else if (sourcePayrollItemDtl instanceof TaxDtl) {
			TaxDtl taxDtl = (TaxDtl) sourcePayrollItemDtl;
			Tax tax = (Tax) taxDtl.getPayrollItem();
			switch (tax.getSourceEntity()) {
				case WORKER:
					return PaycheckLineItemSortSequenceEnum.EMPLOYEEPAIDTAX;
	
				case EMPLOYER:
					return PaycheckLineItemSortSequenceEnum.EMPLOYERPAIDTAX;
				default:
					return PaycheckLineItemSortSequenceEnum.NOTSET;
			}
		} else if (sourcePayrollItemDtl instanceof HealthBenefitDtl) {
			HealthBenefitDtl hbDtl = (HealthBenefitDtl) sourcePayrollItemDtl;
			switch (hbDtl.getBenefitType()) {
				case SEC125HEALTH:
					return PaycheckLineItemSortSequenceEnum.PRETAXDEDUCTION;
				default:
					return PaycheckLineItemSortSequenceEnum.NOTSET;
			}
		} else if (sourcePayrollItemDtl instanceof DeductionDtl) {
			DeductionDtl dd = (DeductionDtl) sourcePayrollItemDtl;
			switch (dd.getDeductionType()) {
				case CASHTIPSOFFSET:
					return PaycheckLineItemSortSequenceEnum.POSTTAXDEDUCTION;
				default:
					return PaycheckLineItemSortSequenceEnum.NOTSET;
			}
		}
		return PaycheckLineItemSortSequenceEnum.NOTSET;
	}
}
