package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.List;
import java.util.Locale;

import com.intuit.payroll.j.EmployeeIf;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Interface defining the contract for participating in the chain.  It is at the discretion
 * of the developer how many PayrollItemDtls are handled by one handler.  If there is a chance
 * that any of the PayrollItems would be used differently in different locales, then it must 
 * be broken out into it's own handler.
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public interface PayrollItemHandler {
	
	/**
	 * Indicates what LawId this handler supports.
	 * @return
	 */
	public String getLawId();
	
	/**
	 * Returns the Locales for which this handler applies.
	 * @return
	 */
	public List<Locale> getLocaleList();
	
	/**
	 * Each handler is expected to create the appropriate entries into the context as required.  The state of
	 * the flow is that empty Paychecks are created, but we may have to create line items if they aren't already there.
	 * 
	 * This will add line items to a paycheck.
	 * 
	 * TODO: rename to make this clear.
	 * 
	 * @param context
	 */
	public void handle(PaycheckCalculationContext context);
	
	/**
	 * Creates a QBO Payroll Item for each Payroll Item on our side for an employee, including totals collection.
	 * @param context contains the current employer, the payrollitem map, qboCallback.
	 */
	public void createQboPayrollItemsFromPayrollItemDtls(PayrollEngineContext context, EmployeeIf employee, Worker worker);
	
}
