package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;

public class PayrollItemHandlerChainStrategyBuilder {

	private static Logger sLogger = PayrollLoggerFactory.makeLogger(QBOEPaycheckCalculatorBean.class);
	
	private Map<Locale, List<PayrollItemHandler>> itemRegistry = new Hashtable<Locale, List<PayrollItemHandler>>();
	
	private static List<String> handlerNames = new ArrayList<String>();
	
	//FIXME this would come from config/rules/whatever - each locale would have it's own?  This makes it safe to share
	// a classloader.
	static {
		//Intl
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.IntlHourlyCompensationHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.IntlHourlyOvertimeCompensationHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.IntlPieceworkCompensationHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.IntlSalaryCompensationHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.IntlCommissionCompensationHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.IntlBonusCompensationHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.IntlTaxHandler");
		//CA
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.CACompanyPaidHealthBenefitHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.CARetirementRRSPHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.CAParkingBenefitHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.CATransitBenefitHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.CAAlimonyDeductionHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.CALoanRepaymentDeductionHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.CARRSPDeductionHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.CASocialFundDeductionHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.CAVacationPayCompensationHandler");
		//US
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.USSec125HealthBenefitHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.USTipsPaycheckCompensationHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.USTipsAllocatedCompensationHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.USReimbursementCompensationHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.USSickPTODisbursementHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.USVacationPTODisbursementHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.USTipsCashCompensationHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.USCAStateWithholdingHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.USFederalWithholdingHandler");
		handlerNames.add("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers.USStateUnemploymentHandler");

	}
	
	@SuppressWarnings("unchecked")
	public PayrollItemHandlerChainStrategyBuilder () {
		//FIXME horribly inefficient means to set up the classes, methinks the strategy builder should call on 
		// some other class that maintains the list?
		for (String handlerName : handlerNames) {
			try {
				Class<PayrollItemHandler> handlerClass = (Class<PayrollItemHandler>) Class.forName(handlerName);
				PayrollItemHandler handlerInstance = handlerClass.newInstance();
				List<Locale> localeList = handlerInstance.getLocaleList();
				for (Locale locale : localeList) {
					if (itemRegistry.get(locale) == null) {
						itemRegistry.put(locale, new ArrayList<PayrollItemHandler>());
					}
					List<PayrollItemHandler> localeHandlerList = itemRegistry.get(locale);
					localeHandlerList.add(handlerInstance);
				}
				
			} catch (Exception e) {
				sLogger.error("Problem loading PayrollItemHandlers", e);
				throw new InfrastructureException("Unable to initialize the Payroll Item Handlers", e);
			}
			
		}
	}
	
	/**
	 * Returns the List of handlers that are appropriate for the current Locale.
	 * @return
	 */
	public List<PayrollItemHandler> getHandlers() {
		Locale locale = LocaleGuru.getInstance().getLocale();
		List<PayrollItemHandler> results = itemRegistry.get(locale);
		return results;
	}
	
	/**
	 * Returns the List of handlers that are appropriate for a given Locale.
	 * WARNING - only use this API if you think you are smarter than LocaleGuru.
	 * 
	 * @param locale
	 * @return
	 */
	public List<PayrollItemHandler> getHandlersForLocale(Locale locale) {
		List<PayrollItemHandler> results = itemRegistry.get(locale);
		return results;
	}
	
}
