package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.Date;

import com.intuit.dataTypes.QBDate;
import com.intuit.payroll.j.DateRangeIf;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Money;

public class QBOEConvert {
	public static Money qboeMoneyToGeminiMoney(com.intuit.dataTypes.Money money) {
		return new Money(LocaleGuru.getInstance().getCurrency(), money.toString());
	}
	
	public static com.intuit.dataTypes.Money geminiMoneyToQboeMoney(Money money) {
		return new com.intuit.dataTypes.Money(money.getAmount().toString());
	}
	
	public static DateRangeIf newQboeDateRange(Date inclusiveStart, Date exclusiveEnd) {
		return new QboeDateRange(new QBDate(inclusiveStart), new QBDate(exclusiveEnd));
	}
	
	public static DateRangeIf newQboeDateRange(QBDate inclusiveStart, QBDate exclusiveEnd) {
		return new QboeDateRange(inclusiveStart, exclusiveEnd);
	}
	
	public static DateRangeIf convertGeminiDateRangeToQboeDateRange(
			DateRange dateRange) {
		
		return newQboeDateRange(dateRange.getStartDate(), dateRange.getEndDate());
	}
	
	/**
	 * Local implementation of the DateRangeIf to support QBOE calls. This class is copied directly 
	 * from the QBOE implementation.
	 * FIXME: [th] remove later, we don't need to implement the interface.
	 * @author gcoulombe
	 */
	public static class QboeDateRange implements DateRangeIf {
 
		private QBDate begin;
	    private QBDate exclusiveEnd;

	    /**
	     * Constructor.
	     * DANGER no safe copy of the parameters.  This is for performance reasons.
	     * Profiling has shown this to be an issue.
	     * @param begin first date of the period (eg 1/1/2004)
	     * @param exclusiveEnd first date <i>after</i> the end of the period (eg 1/1/2005)
	     */
	    public QboeDateRange (QBDate begin, QBDate exclusiveEnd) {
	        this.begin = begin;
	        this.exclusiveEnd = exclusiveEnd;
	    }

	    // Implements DateRangeIf
	    public QBDate getBeginDate() {
	        return begin;
	    }

	    // Implements DateRangeIf
	    public QBDate getEndDate() {
	        return exclusiveEnd;
	    }

	    // Implements DateRangeIf
	    public boolean contains (QBDate date) {
	        if (begin != null && date.before(begin)) {
	            return false;
	        }
	        if (exclusiveEnd != null && ! date.before(exclusiveEnd)) {
	            return false;
	        }
	        return true;
	    }

	    // Implements DateRangeIf
	    public boolean overlaps(DateRangeIf that) {
	        // Identity
	        if (this == that) return true;

	        if (this.getBeginDate() == null && that.getBeginDate() == null) {
	            //         		<------)
	            //              <--------)
	            // etc
	            return true;
	        }

	        if (this.getEndDate() == null && that.getEndDate() == null) {
	            //         		[------>
	            //            [-------->
	            // etc
	            return true;
	        }

	        if (this.getEndDate() != null &&
	                that.getBeginDate() != null &&
	                ! this.getEndDate().after(that.getBeginDate())) {
	            // this       [-----)
	            // that             [-----)
	            //
	            // this       [-----)
	            // that               [-----)
	            //
	            // this     <-------)
	            // that               [------->
	            // etc.
	            return false;
	        }
	        if (that.getEndDate() != null &&
	                this.getBeginDate() != null &&
	                ! that.getEndDate().after(this.getBeginDate())) {
	            // this             [-----)
	            // that       [-----)
	            //
	            // this               [-----)
	            // that       [-----)
	            //
	            // this               [------->
	            // that     <-------)
	            // etc.
	            return false;
	        }
	        // Excluded the non-overlapping cases, so it must overlap in some way.  For example:
	        // this        [--------)
	        // that           [--)
	        //
	        // this        [-------)
	        // that             [-------)
	        //
	        // this             [--------->
	        // that      <---------)
	        //
	        // this          [--)
	        // that      <----------->
	        // etc.
	        return true;
	    }

	    public String toString () {
	        StringBuilder result = new StringBuilder();
	        result.append("[");
	        result.append(begin);
	        result.append(",");
	        result.append(exclusiveEnd);
	        result.append("]");
	        return result.toString();
	    }

		public DateRangeIf overlap(DateRangeIf that) {
			// TODO Auto-generated method stub
			return null;
		}
		
	}
}
