package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.Iterator;
import java.util.NoSuchElementException;

@SuppressWarnings("unchecked")
public class QBOEFilteredIterator implements Iterator {

	private Iterator iterator;
	private QBOEFilter filter;

	// the next object from 'iterator' to return matching the criteria
	private Object nextObj;

	/**
	 * Constructor -- create a filtered iterator based on the existing iterator
	 */
	public QBOEFilteredIterator(Iterator iterator, QBOEFilter filter) {
		this.iterator = iterator;
		this.filter = filter;
	}

	/**
	 * Implements Iterator -- return true if we have not iterated
	 * over an object that matches the criteria.
	 * @see QBOEFilter#include
	 */
	public boolean hasNext() {
		if (nextObj!=null) {
			return true; // already queued up
		}

		while (iterator.hasNext()) {
			Object o = iterator.next();
			if (filter.include(o)) {
				nextObj = o;
				return true;
			}
		}
		return false;
	}

	/**
	 * Implements Iterator - return the next Object that satisfies matching criteria
	 * @see QBOEFilter#include
	 */
	public Object next() throws NoSuchElementException {
		if (!hasNext()) {
			throw new NoSuchElementException();
		}
		Object result = nextObj;
		nextObj = null;
		return result;
	}

	/**
	 * Not supported
	 * @throws UnsupportedOperationException
	 * @throws IllegalStateException
	 */
	public void remove() throws UnsupportedOperationException, IllegalStateException {
		throw new UnsupportedOperationException();
	}

}
