package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.EnumMap;

import com.intuit.dataTypes.TimeDuration;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
/**
 * TODO [impl] Olli sent email to Robert Sage 8/1 to clarify if these conversion factors are correct.
 * 8/22 Robert's response is that QBO doesn't use TimeDurations for month and year. It handles salary completely
 * differently to hourly. Hence, from a QBO POV, the mapping for month & year doesn't make sense. We need to
 * examine this when we support something more than hourly and bonus payments.
 *
 */
public class TimeDurationMapper {

	private static EnumMap<DatumType2Enum, Integer> SecondsPerUnitOfTimeMap = new EnumMap<DatumType2Enum, Integer>(
			DatumType2Enum.class);
	static {
		SecondsPerUnitOfTimeMap.put(DatumType2Enum.HOUR, 60 * 60);
		SecondsPerUnitOfTimeMap.put(DatumType2Enum.DAY, SecondsPerUnitOfTimeMap.get(DatumType2Enum.HOUR) * 24);
		SecondsPerUnitOfTimeMap.put(DatumType2Enum.WEEK, SecondsPerUnitOfTimeMap.get(DatumType2Enum.DAY) * 7);
		SecondsPerUnitOfTimeMap.put(DatumType2Enum.MONTH, SecondsPerUnitOfTimeMap.get(DatumType2Enum.DAY) * 30);
		SecondsPerUnitOfTimeMap.put(DatumType2Enum.YEAR, SecondsPerUnitOfTimeMap.get(DatumType2Enum.DAY) * 365);
	}

	private TimeDurationMapper() {}
	
	public static TimeDuration getTimeWorked(Datum rate, Datum quantity) {
		// Seconds in 1 year = 31536000. It seems reasonable that a payroll period won't exceed this and overflow an int.
		Double secondsPerUnitOfWork = rate.getValue2() * SecondsPerUnitOfTimeMap.get(rate.getType2());
		// The quantity holds the number of units of work
		Double secondsWorked = quantity.getValue1AsQuantity() * secondsPerUnitOfWork;
		// We're throwing away the fractional seconds
		TimeDuration timeWorked = new TimeDuration(secondsWorked.intValue());
		return timeWorked;
	}
	
	/**
	 * Converts our hours to a Time Duration
	 * @param hours
	 * @return
	 */
	public static TimeDuration getTimeDuration(Double hours) {
		Double seconds = hours * (3600) ;
		TimeDuration time = new TimeDuration(seconds.intValue());
		return time;
	}
	
}
