package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import java.math.BigDecimal;

import com.intuit.dataTypes.Money;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.types.enumtype.CalcBasedOn;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Handles default behaviour for Quantity rates (like Piecework).
 * 
 * $Author: $ 
 * $Revision: $
 * $DateTime: $
 *
 */
public abstract class AbstractFlatRateCompensationHandler extends AbstractCompensationHandler {

	protected void createImpl(PayrollEngineContext context, Worker worker, EmployeeIf employee, PayrollItemDtl compensationDtl) {
		final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(employee, compensationDtl);
		
		qboeEmployeePayrollItem.getPayrollItem().setDatumValue(DatumID.CALC_BASED_ON, CalcBasedOn.FLAT_AMOUNT);
		Money fixedCommisionMoneyRate = null;
		if (compensationDtl.getRate() != null) {
			final BigDecimal fixedCommissionRate = compensationDtl.getRate().getValue1AsMoneyAmount();
			if (fixedCommissionRate != null) {
				fixedCommisionMoneyRate = new Money(fixedCommissionRate.toString());
			} else {
				fixedCommisionMoneyRate = new Money(0);
			}
			qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, fixedCommisionMoneyRate);
		}

		// YTD
		createYTDTotals(context, worker, compensationDtl, qboeEmployeePayrollItem);
		
	}
	
	/**
	 * 
	 * @param context
	 * @param compensationDtl
	 */
	protected void handleImpl(PaycheckCalculationContext context, final CompensationDtl compensationDtl) {
		final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(context.getQboeEmployee(), compensationDtl);
		final PaycheckLineItem paycheckLineItem = findPaycheckLineItemForCompensation(context, compensationDtl);
		context.getPaycheck().addPaycheckLineItem(paycheckLineItem);
		
		//Handles fixed and variable rates. Fixed rates come from the CompensationDtl, variable
		//rates come from the line item override.
		qboeEmployeePayrollItem.getPayrollItem().setDatumValue(DatumID.CALC_BASED_ON, CalcBasedOn.FLAT_AMOUNT);
		final Datum lineItemRate = paycheckLineItem.getRate();
		Money fixedCommisionMoneyRate = null;
		// If it's fixed and there's no override
		if (compensationDtl.isFixedRate() && lineItemRate == null) {
			paycheckLineItem.setRate(compensationDtl.getRate().clone());
			final BigDecimal fixedCommissionRate = compensationDtl.getRate().getValue1AsMoneyAmount();
			fixedCommisionMoneyRate = new Money(fixedCommissionRate.toString());
		} else {
			if (lineItemRate != null && lineItemRate.getValue1AsMoneyAmount() != null) {
				fixedCommisionMoneyRate = new Money(lineItemRate.getValue1AsMoneyAmount().toString());
			} else {
				fixedCommisionMoneyRate = new Money(0);
			}
		}
		qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, fixedCommisionMoneyRate);
		final PaycheckLineIf  qboePaycheckLine = context.getQboePaycheck().createLine(qboeEmployeePayrollItem);
		qboePaycheckLine.setDatumValue(DatumID.RATE, fixedCommisionMoneyRate);

		// YTD
		createYTDTotals(context, context.getWorker(), compensationDtl, qboeEmployeePayrollItem);
		
		context.getPaycheckLineItemMap().put(qboePaycheckLine, paycheckLineItem);
	}

}
