package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import java.math.BigDecimal;

import org.apache.commons.lang.math.RandomUtils;

import com.intuit.dataTypes.Money;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.payroll.types.enumtype.CalcBasedOn;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Handles default behaviour for Flat rates.
 * 
 * $Author: $ 
 * $Revision: $
 * $DateTime: $
 *
 */
public abstract class AbstractFlatRateDeductionHandler extends AbstractPayrollItemHandler {

	protected void createImpl(PayrollEngineContext context, Worker worker, EmployeeIf employee, PayrollItemDtl compensationDtl) {
		final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(employee, compensationDtl);
		
		qboeEmployeePayrollItem.getPayrollItem().setDatumValue(DatumID.CALC_BASED_ON, CalcBasedOn.FLAT_AMOUNT);
		Money fixedDeductionRate = null;
		if (compensationDtl.getRate() != null) {
			final BigDecimal fixedCommissionRate = compensationDtl.getRate().getValue1AsMoneyAmount();
			if (fixedCommissionRate != null) {
				fixedDeductionRate = new Money(fixedCommissionRate.toString());
			} else {
				fixedDeductionRate = new Money(0);
			}
			qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, fixedDeductionRate);
		}

		// YTD
		createYTDTotals(context, worker, compensationDtl, qboeEmployeePayrollItem);
		
	}
	
	protected void handleImpl(PaycheckCalculationContext context, final PayrollItemDtl payrollItemDtl) {
		final PayrollItemDetailType payrollItemDetailType = PayrollItemDetailType.parseInt(new Integer(
				payrollItemDtl.getPayrollItem().getLawId().getValue()));
		final PayrollItemIf qboePayrollItem = context.getQboCompany().createPayrollItem(payrollItemDetailType);
		qboePayrollItem.setDatumValue(DatumID.CALC_BASED_ON, CalcBasedOn.FLAT_AMOUNT);
		//FIXME: what happens when the long is longer than the int :)
		int payrollItemId = RandomUtils.nextInt();
		context.getPayrollItemInfo().setPayrollItem(payrollItemId, qboePayrollItem);
		qboePayrollItem.finishInit(payrollItemId, false);
		final EmployeePayrollItemIf qboeEmployeePayrollItem = context.getQboeEmployee().createEmployeePayrollItem(
				qboePayrollItem);
		// FIXME [Model] [Impl] need to really check the isActive() on the
		// compensation - benefits don't require an override
		// if (! healthBenefitDtl.isActive() && effective in current year) {
		// PaycheckLineIf qboePaycheckLine =
		// context.getQboePaycheck().createHistoryLine(qboeEmployeePayrollItem);
		// }
		
		PaycheckLineItem paycheckLineItem = findPaycheckLineItemForCompensation(context, payrollItemDtl);
		if (paycheckLineItem.getRate() == null) {
			Datum rate = payrollItemDtl.getRate();
			paycheckLineItem.setRate(rate);
			Money deductionAmount = new Money(rate.getValue1AsMoneyAmount().toString());
			qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, deductionAmount);
		} else {
			qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, paycheckLineItem.getRate().getValue1AsMoneyAmount().toString());
		}

		PaycheckLineIf qboePaycheckLine = context.getQboePaycheck().createLine(qboeEmployeePayrollItem);

		// YTD
		createYTDTotals(context, context.getWorker(), payrollItemDtl, qboeEmployeePayrollItem);
		
		context.getPaycheck().addPaycheckLineItem(paycheckLineItem);
		context.getPaycheckLineItemMap().put(qboePaycheckLine, paycheckLineItem);
	}

}
