package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import com.intuit.dataTypes.Money;
import com.intuit.dataTypes.TimeDuration;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.TimeDurationMapper;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.Worker;

public class IntlHourlyCompensationHandler extends AbstractCompensationHandler {

	private static List<Locale> localeList = new ArrayList<Locale>();

	static {
		localeList.add(Locale.US);
		localeList.add(Locale.CANADA);
	}

	public String getLawId() {
		return QboeLawIdConstants.sHourlyLawId;
	}

	public List<Locale> getLocaleList() {
		return localeList;
	}
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollItemHandler#createQboPayrollItemsFromPayrollItemDtls(com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext, com.intuit.payroll.j.EmployeeIf, com.intuit.sb.payroll.entity.model.Worker)
	 */
	public void createQboPayrollItemsFromPayrollItemDtls(PayrollEngineContext context, EmployeeIf employee,	Worker worker) {
		Set<CompensationDtl> workerCompensations = worker.getCompensationDtls();
		for (final CompensationDtl compensationDtl : workerCompensations) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYREG) {
				final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(employee, compensationDtl);
				final BigDecimal hourlyRate = compensationDtl.getRate().getValue1AsMoneyAmount();
				qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(hourlyRate.toString()));
				// YTD
				createYTDTotals(context, worker, compensationDtl, qboeEmployeePayrollItem);
			}
		}
	}
	
	/**
	 * Creates a paychecklineitem
	 */
	public void handle(PaycheckCalculationContext context) {
		Set<CompensationDtl> workerCompensations = context.getWorker().getCompensationDtls();
		for (final CompensationDtl compensationDtl : workerCompensations) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYREG) {
				//FIXME don't create, re-use.  We are now creating the payroll items in the method above.
				final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(context.getQboeEmployee(), compensationDtl);
				final PaycheckLineItem paycheckLineItem = findPaycheckLineItemForCompensation(context, compensationDtl);

				// This could be null if we just got into the previous null check.
				// It is assumed there's a rate on the PayrollItem instead.
				final Datum lineItemRate = paycheckLineItem.getRate();
				// Set Employee Data
				final BigDecimal hourlyRate = compensationDtl.getRate().getValue1AsMoneyAmount();
				qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(hourlyRate.toString()));
				final PaycheckLineIf qboePaycheckLine = context.getQboePaycheck().createLine(qboeEmployeePayrollItem);

				// Quantity
				if (lineItemRate != null) {
					final BigDecimal hourlyRateOverride = lineItemRate.getValue1AsMoneyAmount();
					qboePaycheckLine.setDatumValue(DatumID.RATE, new Money(hourlyRateOverride.toString()));
					final TimeDuration timeWorked = TimeDurationMapper.getTimeWorked(lineItemRate, paycheckLineItem
							.getQuantity());
					qboePaycheckLine.setDatumValue(DatumID.HOURS, timeWorked);
				} else {
					// There were no rate overrides, so use the rate from the worker
					Datum datum = paycheckLineItem.getRate();
					if (null == datum) {
						datum = new Datum();
						paycheckLineItem.setRate(datum);
					}
					datum.setValue1AsUSDMoneyAmount(hourlyRate);
					// They didn't work and there were no rate overrides.
					TimeDuration emptyDuration = new TimeDuration(0);
					qboePaycheckLine.setDatumValue(DatumID.HOURS, emptyDuration);
				}
				// YTD
				createYTDTotals(context, context.getWorker(), compensationDtl, qboeEmployeePayrollItem);

				context.getPaycheckLineItemMap().put(qboePaycheckLine, paycheckLineItem);
			}
		}
	}
}
