package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import com.intuit.dataTypes.Money;
import com.intuit.dataTypes.TimeDuration;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.TimeDurationMapper;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.utils.LinkedCompensationUtils;

public class IntlHourlyOvertimeCompensationHandler extends AbstractCompensationHandler {

	private static List<Locale> localeList = new ArrayList<Locale>();

	static {
		localeList.add(Locale.US);
		localeList.add(Locale.CANADA);
	}

	public String getLawId() {
		return QboeLawIdConstants.sHourlyLawId;
	}

	public List<Locale> getLocaleList() {
		return localeList;
	}

	public void createQboPayrollItemsFromPayrollItemDtls(PayrollEngineContext context, EmployeeIf employee,	Worker worker) {
		Set<CompensationDtl> workerCompensations = worker.getCompensationDtls();
		for (final CompensationDtl compensationDtl : workerCompensations) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYOVT ||
					compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYOVT) {
				final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(employee, compensationDtl);
				Datum rate = LinkedCompensationUtils.getMoneyRateFromLinkedPayrollItemDtl(compensationDtl, compensationDtl.getLinkedPayrollItemDtl());
				final BigDecimal overtimeRate = rate.getValue1AsMoneyAmount();
				qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(overtimeRate.toString()));
				// YTD
				createYTDTotals(context, worker, compensationDtl, qboeEmployeePayrollItem);
			}
		}
	}
	
	public void handle(PaycheckCalculationContext context) {
		Set<CompensationDtl> workerCompensations = context.getWorker().getCompensationDtls();
		for (final CompensationDtl compensationDtl : workerCompensations) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYDBL ||
					compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYOVT) {
				//FIXME don't create, re-use.  We are now creating the payroll items in the method above.
				final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(context.getQboeEmployee(), compensationDtl);
				final PaycheckLineItem paycheckLineItem = findPaycheckLineItemForCompensation(context, compensationDtl);
				PaycheckLineIf qboePaycheckLine = null;
				BigDecimal overtimeRateOverride = null;
				// This could be null if we just got into the previous null check.
				// It is assumed there's a rate on the PayrollItem instead.
				final Datum lineItemRate = paycheckLineItem.getRate();
				if (lineItemRate != null) {
				// Note that we grab this from the line item since the linked rate stuff was already done there.
					overtimeRateOverride = lineItemRate.getValue1AsMoneyAmount();
					qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(overtimeRateOverride.toString()));
				} else {
					Datum rate = LinkedCompensationUtils.getMoneyRateFromLinkedPayrollItemDtl(compensationDtl, compensationDtl.getLinkedPayrollItemDtl());
					overtimeRateOverride = rate.getValue1AsMoneyAmount();
					qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(overtimeRateOverride.toString()));
					//The rate was never set.
					paycheckLineItem.setRate(rate);
				}
				
				qboePaycheckLine = context.getQboePaycheck().createLine(qboeEmployeePayrollItem);
				qboePaycheckLine.setDatumValue(DatumID.RATE, new Money(overtimeRateOverride.toString()));

				// Set Paycheck Data
				TimeDuration overtimeWorked = null;
				if (paycheckLineItem.getQuantity() != null) {
					overtimeWorked = TimeDurationMapper.getTimeWorked(lineItemRate, paycheckLineItem.getQuantity());
				} else {
					overtimeWorked = new TimeDuration(0);
				}
				qboePaycheckLine.setDatumValue(DatumID.HOURS, overtimeWorked);

				// YTD
				createYTDTotals(context, context.getWorker(), compensationDtl, qboeEmployeePayrollItem);

				context.getPaycheckLineItemMap().put(qboePaycheckLine, paycheckLineItem);
			}
		}
	}
}
