package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import com.intuit.dataTypes.Money;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.Worker;

public class IntlSalaryCompensationHandler extends AbstractCompensationHandler {

	private static List<Locale> localeList = new ArrayList<Locale>();

	static {
		localeList.add(Locale.US);
		localeList.add(Locale.CANADA);
	}

	public String getLawId() {
		return QboeLawIdConstants.sSalaryLawId;
	}

	public List<Locale> getLocaleList() {
		return localeList;
	}
	
	public void createQboPayrollItemsFromPayrollItemDtls(PayrollEngineContext context, EmployeeIf employee,
			Worker worker) {
	
		Set<CompensationDtl> workerCompensations = worker.getCompensationDtls();
		for (final CompensationDtl compensationDtl : workerCompensations) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.SALARYREG) {
				final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(employee, compensationDtl);

				final BigDecimal yearlyRate = compensationDtl.getRate().getValue1AsMoneyAmount();
				qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(yearlyRate.toString()));

				// YTD
				createYTDTotals(context, worker, compensationDtl, qboeEmployeePayrollItem);
			}
		}
	}

	public void handle(PaycheckCalculationContext context) {
		Set<CompensationDtl> workerCompensations = context.getWorker().getCompensationDtls();
		for (final CompensationDtl compensationDtl : workerCompensations) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.SALARYREG) {
				final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(context.getQboeEmployee(),	compensationDtl);

				final BigDecimal yearlyRate = compensationDtl.getRate().getValue1AsMoneyAmount();
				qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(yearlyRate.toString()));
				final PaycheckLineIf qboePaycheckLine = context.getQboePaycheck().createLine(qboeEmployeePayrollItem);

				// YTD
				createYTDTotals(context, context.getWorker(), compensationDtl, qboeEmployeePayrollItem);

				final PaycheckLineItem paycheckLineItem = findPaycheckLineItemForCompensation(context, compensationDtl);
				// Set the rate on the PaycheckLineItem
				Datum datum = paycheckLineItem.getRate();
				if (null == datum) {
					datum = new Datum();
					paycheckLineItem.setRate(datum);
				}
				datum.setValue1AsUSDMoneyAmount(yearlyRate);
				context.getPaycheckLineItemMap().put(qboePaycheckLine, paycheckLineItem);
			}
		}
	}
}
