package com.intuit.sb.payroll;

import javax.ejb.ApplicationException;


/**
 * Something went wrong that the user can fix.
 */
@ApplicationException(rollback=true)
public class ValidationException extends Exception {

	/**
	 * Serial Id
	 */
	private static final long serialVersionUID = 781644816974070526L;

	/**
	 * Basic constructor.
	 */
	public ValidationException() {
		super();
	}

	/**
	 * Common case constructor.
	 * @param message A text message that provides context on the specific exception case.
	 */
	public ValidationException(String message) {
		super(message);
	}

	/**
	 * Exception capturing a nested cause indicating why validation of the required workflow
	 * could not be completed. Note that this does not necessarily mean the workflow is
	 * incomplete but more likely indicates a problem with the workflow system itself.
	 * This form of the exception should be used with care as nested exceptions rarely
	 * contain any useful information for a user. The user is unable to take action based
	 * on the exception stack trace and is better served by a focused message.
	 * @param cause the nested cause.
	 */
	public ValidationException(Throwable cause) {
		super(cause);
	}

	/**
	 * Exception capturing a message and nested cause indicating why validation of the required workflow
	 * could not be completed. Note that this does not necessarily mean the workflow is
	 * incomplete but more likely indicates a problem with the workflow system itself.
	 * This form of the exception should be used with care as nested exceptions rarely
	 * contain any useful information for a user. The user is unable to take action based
	 * on the exception stack trace and is better served by a simple focused message without
	 * the addition of the nested cause.
	 * @param message A text message that provides context on the specific exception case.
	 * @param cause the nested cause.
	 */
	public ValidationException(String message, Throwable cause) {
		super(message, cause);
	}
}
