package com.intuit.sb.payroll.common.i18n;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

/**
 * This class is the holder for locale profile information. The source of that information can be either the user's
 * profile, the offering configuration or default values.
 * 
 * @see LocaleGuru 
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
@SuppressWarnings("nls")
public class LocaleProfile {
	public static final String DATE_FORMAT_PROPERTY = "date_format";
	public static final String CURRENCY_FORMAT_PROPERTY = "currency_format";
	public static final String TIME_ZONE_PROPERTY = "time_zone";
	public static final String CURRENCY_PROPERTY = "currency";
	public static final String LANGUAGE_PROPERTY = "language";
	public static final String COUNTRY_PROPERTY = "country";

	private String country;
	private String language;
	private String timeZone;
	private String currency;
	private String currencyFormat;
	private String dateFormat;

	public void setCountry(String country) {
		this.country = trim(country);
	}

	public String getCountry() {
		return country;
	}

	public void setLanguage(String language) {
		this.language = trim(language);
	}

	public String getLanguage() {
		return language;
	}

	public void setCurrency(String currency) {
		this.currency = trim(currency);
	}

	public String getCurrency() {
		return currency;
	}

	public void setCurrencyFormat(String currencyFormat) {
		this.currencyFormat = trim(currencyFormat);
	}

	public String getCurrencyFormat() {
		return currencyFormat;
	}

	public void setDateFormat(String dateFormat) {
		this.dateFormat = trim(dateFormat);
	}

	public String getDateFormat() {
		return dateFormat;
	}

	public void setTimeZone(String timeZone) {
		this.timeZone = timeZone;
	}

	public String getTimeZone() {
		return timeZone;
	}

	@Override
	public int hashCode() {
		return HashCodeBuilder.reflectionHashCode(this);
	}

	@Override
	public boolean equals(Object obj) {
		return EqualsBuilder.reflectionEquals(this, obj);
	}

	/**
	 * Trim and return the given string. If the string is null or empty (after trimming), return null.
	 * 
	 * @param s the string to be trimmed.
	 * @return the given string or null if the string is null or empty.
	 */
	private String trim(String s) {
		if (s == null) {
			return null;
		}

		String trimmed = s.trim();

		if (trimmed.length() == 0) {
			return null;
		}

		return trimmed;
	}
}
