package com.intuit.sb.payroll.common.management;

import java.util.Calendar;

import com.intuit.sb.payroll.common.DateUtil;

/**
 * Represents the PayrollService and some of the management facilities it will expose. 
 * 
 * FIXME: this should be a JMX bean, not a singleton.
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public class PayrollService {

	private static PayrollService sInstance;
	
	private static ThreadLocal<SystemDate> tSystemDate = new ThreadLocal<SystemDate>();
	
	private PayrollService() {}

	/**
	 * Returns the instance of the PayrollSerivce.  This may be a singleton or a reference to an MBean.
	 * @return
	 */
	public static synchronized PayrollService getInstance() {
		if (null == sInstance) {
			sInstance = new PayrollService();
		}
		return sInstance;
	}
	
	/**
	 * Override the SystemDate for purposes of business logic testing.
	 * @param dateToSet
	 */
	public void setSystemDate(Calendar dateToSet) {
		tSystemDate.set(new SystemDate(dateToSet));
	}
	
	/**
	 * Returns the System date.  If no date override has been provided for this thread, the current Date will be returned.
	 * @return
	 */
	public Calendar getSystemDate() {
		SystemDate systemDate = tSystemDate.get();
		if (systemDate != null) {
			return systemDate.getDate();
		} else {
			return DateUtil.getNowCalendar();
		}
	}
	
	/**
	 * Call if you want to clear the date before the thread dies a natural death.
	 */
	public void clearSystemDate() {
		tSystemDate.remove();
	}
}
