package com.intuit.sb.payroll.entity.domain;

import java.util.Date;

/**
 * An interface for the information required to audit creation and updates to an entity.
 * <p>
 * All properties in this interface are read only as they're either immutable and
 * automatically managed by the implementation.
 */
public interface Auditable
{
	/**
	 * Gets the date this entity was created.
     * @return the creation date of this entity
     */
	public Date getCreationDate();

	/**
	 * Gets the date this entity was last updated
	 * @return the modification date of this entity
	 */
	public Date getLastModificationDate();
	/**
	 * Gets the creator of this entity.
	 * @return the ID of the account that created the object.
	 */
	public String getCreatedBy();
	/**
	 * Gets the most recent modifier of this entity.
	 * @return the ID of the account that modified the object.
	 */
	public String getLastModifiedBy();
}
