package com.intuit.sb.payroll.entity.domain;

/**
 * Interface for all persisted objects in the domain model. It provides the basic
 * fields and properties needed for all persisted types.
 * <p>
 * All properties in this interface are read only as they're either immutable and
 * automatically managed by the implementation.
 */
public interface Identifiable
{
	/**
	 * Gets the unique ID for this entity. This is only unique within 
	 * the scope of the entire set of entities of the same type.
	 * @return unique identity for the entity
	 */
	public long getId();

	/**
	 * Gets the version of this entity. Each update to this entity
	 * will cause an increment in its version.
     * @return the version of this entity
     */
	public long getVersion();
}
