package com.intuit.sb.payroll.entity.model;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.EntityListeners;
import javax.persistence.Transient;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.employer.EmployerEntityListener;


/**
 * Employer is a business entity which engages in the employment relationship with {@linkplain Worker}s.
 * Every Employer is a business but the reverse is not true. The status of Employer imposes 
 * on a business a set of legal requirements needed to be met to comply with applicable laws..
 */
@EntityListeners(EmployerEntityListener.class)
public class Employer extends EmployerBase {

	private static final long serialVersionUID = 646437545L;


	public Employer() {
	}

	@Transient
	public java.util.Set<OtherBenefit> getOtherBenefitItems() {
		return getItemsOfType(OtherBenefit.class);
	}

	public void addOtherBenefit(OtherBenefit otherBenefit) {
		getPayrollItems().add(otherBenefit);
	}

	public void removeOtherBenefit(OtherBenefit otherBenefit) {
		getPayrollItems().remove(otherBenefit);
	}

	@Transient
	public java.util.Set<HealthBenefit> getHealthBenefitItems() {
		return getItemsOfType(HealthBenefit.class);
	}

	public void addHealthBenefit(HealthBenefit healthBenefit) {
		getPayrollItems().add(healthBenefit);
	}

	public void removeHealthBenefit(HealthBenefit healthBenefit) {
		getPayrollItems().remove(healthBenefit);
	}

	@Transient
	public java.util.Set<RetirementBenefit> getRetirementBenefitItems() {
		return getItemsOfType(RetirementBenefit.class);
	}

	public void addRetirementBenefit(RetirementBenefit retirementBenefit) {
		getPayrollItems().add(retirementBenefit);
	}

	public void removeRetirementBenefit(RetirementBenefit retirementBenefit) {
		getPayrollItems().remove(retirementBenefit);
	}

	@Transient
	public java.util.Set<Addition> getAdditionItems() {
		return getItemsOfType(Addition.class);
	}

	public void addAddition(Addition addition) {
		getPayrollItems().add(addition);
	}

	public void removeAddition(Addition addition) {
		getPayrollItems().remove(addition);
	}

	@Transient
	public java.util.Set<Deduction> getDeductionItems() {
		return getItemsOfType(Deduction.class);
	}

	public void addDeduction(Deduction deduction) {
		getPayrollItems().add(deduction);
	}

	public void removeDeduction(Deduction deduction) {
		getPayrollItems().remove(deduction);
	}

	@Transient
	public java.util.Set<PaidTimeOff> getPaidTimeOffItems() {
		return getItemsOfType(PaidTimeOff.class);
	}
	
	public void addPaidTimeOff(PaidTimeOff paidTimeOff) {
		getPayrollItems().add(paidTimeOff);
	}
	
	public void removePaidTimeOff(PaidTimeOff paidTimeOff) {
		getPayrollItems().remove(paidTimeOff);
	}
	
	@Transient
	public java.util.Set<Tax> getTaxItems() {
		return getItemsOfType(Tax.class);
	}

	public void addTax(Tax tax) {
		getPayrollItems().add(tax);
	}

	public void removeTax(Tax tax) {
		getPayrollItems().remove(tax);
	}

	@Transient
	public java.util.Set<Compensation> getCompensationItems() {
		return getItemsOfType(Compensation.class);
	}

	public void addCompensation(Compensation compensation) {
		getPayrollItems().add(compensation);
	}

	public void removeCompensation(Compensation compensation) {
		getPayrollItems().remove(compensation);
	}
	
	@SuppressWarnings("unchecked")
	private <T> Set<T> getItemsOfType(Class<T> clazz) {
		Set<T> items = new HashSet<T>();
		for (PayrollItem payrollItem : getPayrollItems()) {
			if (clazz.isInstance(payrollItem)) {
				items.add((T)payrollItem);
			}
		}
		return items;
	}

	/**
	 * Returns a Set of EmployerAdjustments for a given year.
	 * 
	 * @param yearFromDate
	 * @return
	 */
	public Set<EmployerAdjustment> getEmployerAdjustmentsForYear(int yearFromDate) {
		Set<EmployerAdjustment> results = new HashSet<EmployerAdjustment>();
		Set<EmployerAdjustment> employerAdjustments = getEmployerAdjustments();
		for (EmployerAdjustment employerAdjustment : employerAdjustments) {
			if (DateUtil.isInCalendarYear(employerAdjustment.getEffectiveDate(), yearFromDate)) {
				results.add(employerAdjustment);
			}
		}
		return results;
	}


}
