package com.intuit.sb.payroll.entity.utils;

import java.math.BigDecimal;

import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;

public class LinkedCompensationUtils {
	
	/**
	 * Assumes that the compensation holds a multiplier in the Qty slot, and that the rate is a function of
	 * this multiplier times the dollar rate in the linked item.
	 * 
	 * @param compensationDtl the payroll item with a multiplier (eg: 1.5, 2.0)
	 * @param linkedPayrollItemDtl the payroll item with a base dollar rate (eg: $10.00)
	 * @return the actual money rate of the compensationDtl (eg: $15.00, $20.00)
	 */
	public static Datum getMoneyRateFromLinkedPayrollItemDtl(CompensationDtl compensationDtl,
			PayrollItemDtl linkedPayrollItemDtl) {
		CompensationDtl linkedCompensationDtl = (CompensationDtl)linkedPayrollItemDtl;
		Datum linkedPIRate = linkedCompensationDtl.getRate();

		BigDecimal linkedRate = new BigDecimal(linkedPIRate.getValue1AsUSDMoneyAmount().toString());
		Double inputRate = compensationDtl.getRate().getValue1AsQuantity();
		BigDecimal moneyValue = linkedRate.multiply(new BigDecimal(inputRate));
		
		Datum rate = new Datum();
		rate.setValue1AsUSDMoneyAmount(moneyValue);
		rate.setType2Value2(linkedPIRate.getType2(), linkedPIRate.getValue2());
		return rate;
	}
}
