CREATE TABLE HISTPAYCHECK (
    PRIMARY KEY(ID),
    ID NUMBER(20,0) NOT NULL, 
    REALM_ID NUMBER(20,0) DEFAULT -1 NOT NULL, 
    TXNDATE TIMESTAMP NULL, 
    TXNNO VARCHAR2(100) NULL, 
    MEMO VARCHAR2(100) NULL, 
    PAYROLLPERIOD_STARTDATE DATE NULL, 
    PAYROLLPERIOD_ENDDATE DATE NULL, 
    GROSSAMOUNT_CUR VARCHAR2(100) NULL, 
    GROSSAMOUNT_AMT NUMBER(19,4) NULL, 
    NETAMOUNT_CUR VARCHAR2(100) NULL, 
    NETAMOUNT_AMT NUMBER(19,4) NULL, 
    DEDUCTIONAMOUNT_CUR VARCHAR2(100) NULL, 
    DEDUCTIONAMOUNT_AMT NUMBER(19,4) NULL, 
    WORKERPAIDTAXES_CUR VARCHAR2(100) NULL, 
    WORKERPAIDTAXES_AMT NUMBER(19,4) NULL, 
    EMPLOYERPAIDTAXES_CUR VARCHAR2(100) NULL, 
    EMPLOYERPAIDTAXES_AMT NUMBER(19,4) NULL, 
    WORKERPAIDBENEFITS_CUR VARCHAR2(100) NULL, 
    WORKERPAIDBENEFITS_AMT NUMBER(19,4) NULL, 
    EMPLOYERPAIDBENEFITS_CUR VARCHAR2(100) NULL, 
    EMPLOYERPAIDBENEFITS_AMT NUMBER(19,4) NULL, 
    CREATIONDATE TIMESTAMP NULL, 
    CREATEDBY VARCHAR2(100) NULL, 
    LASTMODIFICATIONDATE TIMESTAMP NULL, 
    LASTMODIFIEDBY VARCHAR2(100) NULL, 
    VERSION NUMBER(20,0) NOT NULL, 
    UUID VARCHAR2(255) NOT NULL, 
    REALMOWNER NUMBER(20,0) NULL, 
    EMPLOYER NUMBER(20,0) NULL, 
    WORKER NUMBER(20,0) NULL
)
/

CREATE TABLE HISTPAYCHECKLINEITEM (
    PRIMARY KEY(ID),
    ID NUMBER(20,0) NOT NULL, 
    REALM_ID NUMBER(20,0) DEFAULT -1 NOT NULL, 
    NAME VARCHAR2(100) NULL, 
    LAWID_VALUE VARCHAR2(100) NULL, 
    VALUE_CUR VARCHAR2(100) NULL, 
    VALUE_AMT NUMBER(19,4) NULL, 
    INCOMESUBJECTTOTAX_CUR VARCHAR2(100) NULL, 
    INCOMESUBJECTTOTAX_AMT NUMBER(19,4) NULL, 
    WAGEBASE_CUR VARCHAR2(100) NULL, 
    WAGEBASE_AMT NUMBER(19,4) NULL, 
    EXCESSAMOUNT_CUR VARCHAR2(100) NULL, 
    EXCESSAMOUNT_AMT NUMBER(19,4) NULL, 
    EXEMPTAMOUNT_CUR VARCHAR2(100) NULL, 
    EXEMPTAMOUNT_AMT NUMBER(19,4) NULL, 
    CREATIONDATE TIMESTAMP NULL, 
    CREATEDBY VARCHAR2(100) NULL, 
    LASTMODIFICATIONDATE TIMESTAMP NULL, 
    LASTMODIFIEDBY VARCHAR2(100) NULL, 
    VERSION NUMBER(20,0) NOT NULL, 
    UUID VARCHAR2(255) NOT NULL, 
    REALMOWNER NUMBER(20,0) NULL, 
    PAYROLLITEMDTL NUMBER(20,0) NULL, 
    PAYROLLITEM NUMBER(20,0) NULL, 
    HISTPAYCHECK NUMBER(20,0) NULL
)
/

CREATE INDEX HISTPAYCHECK_RX ON HISTPAYCHECK(REALM_ID ASC)
/

CREATE INDEX HISTPAYCHECK_FK_WORKER ON HISTPAYCHECK(WORKER ASC)
/

CREATE INDEX HISTPAYCHECK_FK_EMPLOYER ON HISTPAYCHECK(EMPLOYER ASC)
/

CREATE INDEX HISTPAYCHECK_FK_REALMOWNER ON HISTPAYCHECK(REALMOWNER ASC)
/

ALTER TABLE HISTPAYCHECK ADD CONSTRAINT HISTPAYCHECK_FK_WORKER
    FOREIGN KEY (WORKER)
    REFERENCES WORKER (ID)
/

ALTER TABLE HISTPAYCHECK ADD CONSTRAINT HISTPAYCHECK_FK_EMPLOYER
    FOREIGN KEY (EMPLOYER)
    REFERENCES EMPLOYER (ID)
/

ALTER TABLE HISTPAYCHECK ADD CONSTRAINT HISTPAYCHECK_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE OR REPLACE TRIGGER "HISTPAYCHECK_VT" BEFORE INSERT OR UPDATE ON "HISTPAYCHECK"
    FOR EACH ROW
    BEGIN
     IF USER = UPPER('&&1') THEN RETURN; 
     END IF; 
     IF INSERTING THEN 
         :new.realm_id := SBCORE_VPD_POLICY_PKG.GET_REALM_ID_FROM_CTX; 
     ELSE 
         IF UPDATING THEN 
             IF :old.realm_id != :new.realm_id THEN 
                 :new.realm_id := :old.realm_id; 
             END IF; 
         END IF; 
     END IF; 
    END;
/


CREATE INDEX HISTPAYCHECKLINEITEM_RX ON HISTPAYCHECKLINEITEM(REALM_ID ASC)
/

CREATE INDEX HPCHKLITM_FK_PITMDTL ON HISTPAYCHECKLINEITEM(PAYROLLITEMDTL ASC)
/

CREATE INDEX HPCHK_FK_HPCHKLITM ON HISTPAYCHECKLINEITEM(HISTPAYCHECK ASC)
/

CREATE INDEX HPCHKLITM_FK_PITM ON HISTPAYCHECKLINEITEM(PAYROLLITEM ASC)
/

CREATE INDEX HSTPCHKLITM_FK_ER ON HISTPAYCHECKLINEITEM(REALMOWNER ASC)
/

ALTER TABLE HISTPAYCHECKLINEITEM ADD CONSTRAINT HPCHKLITM_FK_PITMDTL
    FOREIGN KEY (PAYROLLITEMDTL)
    REFERENCES PAYROLLITEMDTL (ID)
/

ALTER TABLE HISTPAYCHECKLINEITEM ADD CONSTRAINT HPCHK_FK_HPCHKLITM
    FOREIGN KEY (HISTPAYCHECK)
    REFERENCES HISTPAYCHECK (ID)
/

ALTER TABLE HISTPAYCHECKLINEITEM ADD CONSTRAINT HPCHKLITM_FK_PITM
    FOREIGN KEY (PAYROLLITEM)
    REFERENCES PAYROLLITEM (ID)
/

ALTER TABLE HISTPAYCHECKLINEITEM ADD CONSTRAINT HSTPCHKLITM_FK_ER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE OR REPLACE TRIGGER "HISTPAYCHECKLINEITEM_VT" BEFORE INSERT OR UPDATE ON "HISTPAYCHECKLINEITEM"
    FOR EACH ROW
    BEGIN
     IF USER = UPPER('&&1') THEN RETURN; 
     END IF; 
     IF INSERTING THEN 
         :new.realm_id := SBCORE_VPD_POLICY_PKG.GET_REALM_ID_FROM_CTX; 
     ELSE 
         IF UPDATING THEN 
             IF :old.realm_id != :new.realm_id THEN 
                 :new.realm_id := :old.realm_id; 
             END IF; 
         END IF; 
     END IF; 
    END;
/


ALTER TABLE WORKERCOMPLIANCEDOCDTL ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE EMPLOYERID ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE EMPLOYER ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE EMPLOYERADJUSTMENT ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PAYMENTLINEITEM ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PTDDATUM ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PTDDATUM ADD (HISTPAYCHECKLINEITEM NUMBER(20,0) NULL)
/

ALTER TABLE WORKLOCATION ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE TASK ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE WORKERADJUSTMENT ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PAYROLLRUN ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE WORKER ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PAYCHECK ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PAYMENTLINEITEMSPLIT ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PAYCHECKLINEITEM ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PAYMENT ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE EMPLOYERADJLINEITEM ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE EMPLOYMENTPERIOD ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE EMPLOYERCOMPLIANCEDOCDTL ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE BANKACCOUNT ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE WORKERADJLINEITEM ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PAYROLLITEMDTL ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE CONTACTPERSON ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PAYROLLSCHEDULE ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE PAYROLLITEM ADD (REALMOWNER NUMBER(20,0) NULL)
/

ALTER TABLE WORKERCOMPLIANCEDOCDTL ADD CONSTRAINT WKRCOMPLDOCDTL_FK_ER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX WKRCOMPLDOCDTL_FK_ER ON WORKERCOMPLIANCEDOCDTL(REALMOWNER ASC)
/

ALTER TABLE EMPLOYERID ADD CONSTRAINT EMPLOYERID_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX EMPLOYERID_FK_REALMOWNER ON EMPLOYERID(REALMOWNER ASC)
/

ALTER TABLE EMPLOYER ADD UNIQUE (REALMOWNER)
/

ALTER TABLE EMPLOYER ADD CONSTRAINT EMPLOYER_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

ALTER TABLE EMPLOYERADJUSTMENT ADD CONSTRAINT ERADJ_FK_ER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX ERADJ_FK_ER ON EMPLOYERADJUSTMENT(REALMOWNER ASC)
/

ALTER TABLE PAYMENTLINEITEM ADD CONSTRAINT PAYMENTLINEITEM_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX PAYMENTLINEITEM_FK_REALMOWNER ON PAYMENTLINEITEM(REALMOWNER ASC)
/

ALTER TABLE PTDDATUM ADD CONSTRAINT HPCHKLITMPTD_FK_HPCHKLITM
    FOREIGN KEY (HISTPAYCHECKLINEITEM)
    REFERENCES HISTPAYCHECKLINEITEM (ID)
/

ALTER TABLE PTDDATUM ADD CONSTRAINT PTDDATUM_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX HPCHKLITMPTD_FK_HPCHKLITM ON PTDDATUM(HISTPAYCHECKLINEITEM ASC)
/

CREATE INDEX PTDDATUM_FK_REALMOWNER ON PTDDATUM(REALMOWNER ASC)
/

ALTER TABLE WORKLOCATION ADD CONSTRAINT WORKLOCATION_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX WORKLOCATION_FK_REALMOWNER ON WORKLOCATION(REALMOWNER ASC)
/

ALTER TABLE TASK ADD CONSTRAINT TASK_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX TASK_FK_REALMOWNER ON TASK(REALMOWNER ASC)
/

ALTER TABLE WORKERADJUSTMENT ADD CONSTRAINT WORKERADJUSTMENT_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX WORKERADJUSTMENT_FK_REALMOWNER ON WORKERADJUSTMENT(REALMOWNER ASC)
/

ALTER TABLE PAYROLLRUN ADD CONSTRAINT PAYROLLRUN_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX PAYROLLRUN_FK_REALMOWNER ON PAYROLLRUN(REALMOWNER ASC)
/

ALTER TABLE WORKER ADD UNIQUE (REALMOWNER)
/

ALTER TABLE WORKER ADD CONSTRAINT WORKER_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

ALTER TABLE PAYCHECK ADD CONSTRAINT PAYCHECK_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX PAYCHECK_FK_REALMOWNER ON PAYCHECK(REALMOWNER ASC)
/

ALTER TABLE PAYMENTLINEITEMSPLIT ADD CONSTRAINT PMTLITMSPLT_FK_ER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX PMTLITMSPLT_FK_ER ON PAYMENTLINEITEMSPLIT(REALMOWNER ASC)
/

ALTER TABLE PAYCHECKLINEITEM ADD CONSTRAINT PAYCHECKLINEITEM_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX PAYCHECKLINEITEM_FK_REALMOWNER ON PAYCHECKLINEITEM(REALMOWNER ASC)
/

ALTER TABLE PAYMENT ADD CONSTRAINT PAYMENT_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX PAYMENT_FK_REALMOWNER ON PAYMENT(REALMOWNER ASC)
/

ALTER TABLE EMPLOYERADJLINEITEM ADD CONSTRAINT ERADJLITM_FK_ER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX ERADJLITM_FK_ER ON EMPLOYERADJLINEITEM(REALMOWNER ASC)
/

ALTER TABLE EMPLOYMENTPERIOD ADD CONSTRAINT EMPLOYMENTPERIOD_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX EMPLOYMENTPERIOD_FK_REALMOWNER ON EMPLOYMENTPERIOD(REALMOWNER ASC)
/

ALTER TABLE EMPLOYERCOMPLIANCEDOCDTL ADD CONSTRAINT ERCOMPLDOCDTL_FK_ER1
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX ERCOMPLDOCDTL_FK_ER1 ON EMPLOYERCOMPLIANCEDOCDTL(REALMOWNER ASC)
/

ALTER TABLE BANKACCOUNT ADD CONSTRAINT BANKACCOUNT_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX BANKACCOUNT_FK_REALMOWNER ON BANKACCOUNT(REALMOWNER ASC)
/

ALTER TABLE WORKERADJLINEITEM ADD CONSTRAINT WKRADJLITEM_FK_ER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX WKRADJLITEM_FK_ER ON WORKERADJLINEITEM(REALMOWNER ASC)
/

ALTER TABLE PAYROLLITEMDTL ADD CONSTRAINT PAYROLLITEMDTL_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX PAYROLLITEMDTL_FK_REALMOWNER ON PAYROLLITEMDTL(REALMOWNER ASC)
/

ALTER TABLE CONTACTPERSON ADD CONSTRAINT CONTACTPERSON_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX CONTACTPERSON_FK_REALMOWNER ON CONTACTPERSON(REALMOWNER ASC)
/

ALTER TABLE PAYROLLSCHEDULE ADD CONSTRAINT PAYROLLSCHEDULE_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX PAYROLLSCHEDULE_FK_REALMOWNER ON PAYROLLSCHEDULE(REALMOWNER ASC)
/

ALTER TABLE PAYROLLITEM ADD CONSTRAINT PAYROLLITEM_FK_REALMOWNER
    FOREIGN KEY (REALMOWNER)
    REFERENCES EMPLOYER (ID)
/

CREATE INDEX PAYROLLITEM_FK_REALMOWNER ON PAYROLLITEM(REALMOWNER ASC)
/

GRANT SELECT, INSERT, UPDATE, DELETE ON HISTPAYCHECK TO &&2
/

GRANT SELECT, INSERT, UPDATE, DELETE ON HISTPAYCHECKLINEITEM TO &&2
/

UPDATE PROJECT_VERSIONS_TABLE SET SCHEMA_VERSION='1.0.28', TIMESTAMP='2009-07-06 10:57:16.595' WHERE PROJECT_ID='_c4GAoJEkEd2zF5tDuPZkGw'
/

