package com.intuit.sb.payroll.entity.utils;

import java.util.Calendar;
import java.util.Date;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;

public class PtdUtilsTest {

	@Test
	public void testGetDateRangeForPeriodType() {
		DateRange expectedRangeForQ1 = new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 1), 
													DateUtil.createDate(2009, Calendar.APRIL, 1));
		DateRange computedRangeForQ1 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER1, 2009);
		Assert.assertEquals(computedRangeForQ1, expectedRangeForQ1);
		
		DateRange expectedRangeForQ2 = new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 1), 
				DateUtil.createDate(2009, Calendar.JULY, 1));
		DateRange computedRangeForQ2 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER2, 2009);
		Assert.assertEquals(computedRangeForQ2, expectedRangeForQ2);

		DateRange expectedRangeForQ3 = new DateRange(DateUtil.createDate(2009, Calendar.JULY, 1), 
				DateUtil.createDate(2009, Calendar.OCTOBER, 1));
		DateRange computedRangeForQ3 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER3, 2009);
		Assert.assertEquals(computedRangeForQ3, expectedRangeForQ3);

		DateRange expectedRangeForQ4 = new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 1), 
				DateUtil.createDate(2010, Calendar.JANUARY, 1));
		DateRange computedRangeForQ4 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER4, 2009);
		Assert.assertEquals(computedRangeForQ4, expectedRangeForQ4);

	}
	
	@Test
	public void testDateInCorrectQuarter() {
		DateRange q1 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER1, 2009);
		DateRange q2 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER2, 2009);
		DateRange q3 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER3, 2009);
		DateRange q4 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER4, 2009);
		
		Date jan1  = DateUtil.createDate(2009, Calendar.JANUARY, 1);
		Date mar31 = DateUtil.createDate(2009, Calendar.MARCH, 30);
		Date apr1  = DateUtil.createDate(2009, Calendar.APRIL, 1);
		Date may15 = DateUtil.createDate(2009, Calendar.MAY, 15);
		Date jul1   = DateUtil.createDate(2009, Calendar.JULY, 1);
		Date dec31 = DateUtil.createDate(2009, Calendar.DECEMBER, 31);
		
		Assert.assertEquals(PtdUtils.getQtrRangeForDate(jan1), q1);
		Assert.assertEquals(PtdUtils.getQtrRangeForDate(mar31), q1);
		Assert.assertEquals(PtdUtils.getQtrRangeForDate(apr1), q2);
		Assert.assertEquals(PtdUtils.getQtrRangeForDate(may15), q2);
		Assert.assertEquals(PtdUtils.getQtrRangeForDate(jul1), q3);
		Assert.assertEquals(PtdUtils.getQtrRangeForDate(dec31), q4);
	}
}
