package com.intuit.sb.payroll.notification;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Set;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;
import javax.xml.ws.Holder;

import org.apache.log4j.Logger;

import com.intuit.ias.notification.ErrorDataType;
import com.intuit.ias.notification.HeaderDataType;
import com.intuit.ias.notification.Fault;
import com.intuit.ias.notification.NotificationPort;
import com.intuit.ias.notification.NotificationService;
import com.intuit.ias.notification.AttributeDataType;
import com.intuit.ias.notification.BasicContentDataType;
import com.intuit.ias.notification.BasicRecipientDataType;
import com.intuit.ias.notification.BasicSenderDataType;
import com.intuit.ias.notification.ContentFormatDataType;
import com.intuit.ias.notification.NotificationDataType;
import com.intuit.ias.notification.NotificationTypeDataType;
import com.intuit.ias.notification.SendRequest;
import com.intuit.ias.notification.SendResponse;
import com.intuit.ias.notification.NotificationDataType.ContentProfile;
import com.intuit.ias.notification.NotificationDataType.DeliveryType;
import com.intuit.ias.notification.NotificationDataType.Destinations;
import com.intuit.ias.notification.NotificationDataType.SenderProfile;
import com.intuit.ias.notification.NotificationDataType.ContentProfile.TemplateProperties;
import com.intuit.ias.notification.NotificationDataType.Destinations.Destination;
import com.intuit.ias.notification.SendResponse.Result;
import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.notification.template.EmailTemplate;

/**
 * This helper service provide the api's to send mail to recipients. Creating a singleton instance of this utility will
 * initialize all the required objects required to make web service request to the Notification Service
 * 
 */

// FIXME: We still use downloaded wsdl for generation of java files, Anuj says that some work is needed on https support
// of wsdl
// FIXME Move these to configuration file once we decide where should key store file go
// FIXME Documentation
public class PayrollNotificationService {

	private static final String serviceURL = "https://ntfsvce2e.ptc.intuit.com/ias/notification/public?wsdl";
	private static final String namespace = "http://www.intuit.com/ias/notification/pub/wsdl";
	private static final String serviceName = "NotificationService";
	private static Logger sLogger = PayrollLoggerFactory.makeLogger(PayrollNotificationService.class);

	private SenderProfile senderProfile = new SenderProfile();

	private NotificationPort notificationPort = null;
	private Holder<HeaderDataType> headerHolder;
	HeaderDataType header = null;

	// Single instance
	private static PayrollNotificationService payrollNotificationService = new PayrollNotificationService();

	public static PayrollNotificationService getInstance() {
		return payrollNotificationService;
	}

	private PayrollNotificationService() {
		sLogger.debug("Creating PayrollNotificationService ");
		setSenderProfile();
		try {
			setClientProfile();
			setSoapHeader();
		} catch (MalformedURLException e) {
			throw new InfrastructureException(e);
		} catch (DatatypeConfigurationException e) {
			throw new InfrastructureException(e);
		}
	}

	/**
	 * Sets the sender profile, this profile id is provided and registered by the Notification Service
	 */
	private void setSenderProfile() {
		senderProfile.setSenderId("SBDPAYROLLGEMINI");
		BasicSenderDataType sender = new BasicSenderDataType();
		sender.setAddress("intuit@intuit.com");
		sender.setName("Intuit");
		senderProfile.setSender(sender);
	}

	/**
	 * Creates NotificationService from the wsdl URL
	 * 
	 * @throws MalformedURLException
	 */
	private void setClientProfile() throws MalformedURLException {
		/*
		 * FIXME check if this is the right way to set ssl keys June found out that this should be command line
		 * parameters
		 * 
		 * The following properties should go to JBoss run.bat/run.sh javax.net.ssl.trustStore
		 * javax.net.ssl.trustStorePassword javax.net.ssl.keyStore javax.net.ssl.keyStorePassword
		 * 
		 * The keystore certificate (.jks) file should be stored in JBoss /server/default/conf folder and used from
		 * there.
		 * 
		 * javax.net.ssl.trustStore=%JBOSS_HOME%\server\default\conf\notificationclient.jks
		 * javax.net.ssl.keyStore=%JBOSS_HOME%\server\default\conf\notificationclient.jks
		 * javax.net.ssl.trustStorePassword=xxxxx javax.net.ssl.keyStorePassword=xxxxxx
		 */

		URL wsdlLocation = new URL(serviceURL);
		QName qName = new QName(namespace, serviceName);
		NotificationService service = new NotificationService(wsdlLocation, qName);
		notificationPort = service.getPort(NotificationPort.class);
	}

	/**
	 * Sets the SOAP header
	 * 
	 * @throws DatatypeConfigurationException
	 */
	private void setSoapHeader() throws DatatypeConfigurationException {
		header = new HeaderDataType();

		// FIXME Include meaningful versions and name

		header.setServiceVersion("1.0");
		// TODO check with Anuj/Fred if we are required to set TransactionId

		header.setTransactionId("test123");
		header.setSourceSystem("TestClientName");
		header.setCallerSystem("TestClientName");
		header.setEndpoint("TestClientName");
		setCreationDateTimeToHeader(header);
		headerHolder = new Holder<HeaderDataType>(header);
	}

	private void setCreationDateTimeToHeader(HeaderDataType header) throws DatatypeConfigurationException {
		DatatypeFactory df = DatatypeFactory.newInstance();

		Calendar calendar = LocaleGuru.getInstance().getCalendar();
		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(calendar.getTime());
		XMLGregorianCalendar createDate = df.newXMLGregorianCalendar(cal);

		header.setCreationDateTime(createDate);
	}

	/**
	 * Create the SOAP Request with information with sender profile, content and notification destinations
	 * 
	 * @param contentProfile
	 * @param destinations
	 * @return
	 */
	private SendRequest createRequest(ContentProfile contentProfile, Destinations destinations) {
		SendRequest request = new SendRequest();
		request.setVersion("1.0");
		NotificationDataType notification = new NotificationDataType();
		// We currently only support email no SMS
		notification.setNotificationType(NotificationTypeDataType.EMAIL);

		DeliveryType deliveryType = new DeliveryType();
		// FIXME [th] determine configuration artifact to govern this.
		deliveryType.setImmediate(new DeliveryType.Immediate());
		notification.setDeliveryType(deliveryType);

		// Set the sender info
		notification.setSenderProfile(senderProfile);

		// Set the content (template info with parameters)
		notification.setContentProfile(contentProfile);

		// Set multiple destinations
		notification.setDestinations(destinations);
		request.setNotification(notification);
		return request;
	}

	private ContentFormatDataType mapToWSDLType(ContentFormatType format) {
		if (format == ContentFormatType.HTML) {
			return ContentFormatDataType.HTML;
		}
		if (format == ContentFormatType.PLAIN) {
			return ContentFormatDataType.TEXT;
		}
		return null;
	}

	/**
	 * Adds an email id to the list of email addresses that need to be sent
	 */

	private Destination createDestination(EmailRecipient emailRecipient) {
		Destination destination = new Destination();
		destination.setRecipientId(emailRecipient.getRecipientId());
		BasicRecipientDataType recipient = new BasicRecipientDataType();
		destination.setRecipient(recipient);
		recipient.setAddress(emailRecipient.getEmailAddress());
		recipient.setName(emailRecipient.getRecipientName());
		destination.setFormat(mapToWSDLType(emailRecipient.getFormat()));
		return destination;
	}

	/**
	 * Utility method of Payroll Services to send Mail based on the template. This utility will create a SOAP request
	 * based on the email recipients and content and send this request to the Notification Service. This utility method
	 * supports only one email recipient per mail (template)
	 * 
	 * @param emailRecipient
	 *            Subjects to whom the email is to be sent
	 * @param emailTemplate
	 *            Content of the email
	 * @throws PayrollException
	 */
	public void sendMail(EmailRecipient emailRecipient, EmailTemplate emailTemplate) {
		sLogger.debug("Sending Mail to " + emailRecipient.getEmailAddress() + " Template used "
				+ emailTemplate.getTemplateName());
		Destinations destinations = new Destinations();

		Destination destination = createDestination(emailRecipient);

		// Add to destinations
		destinations.getDestination().add(destination);

		// Set time with the SOAP header
		try {
			setCreationDateTimeToHeader(header);
		} catch (DatatypeConfigurationException e1) {
			throw new InfrastructureException(e1);
		}

		ContentProfile contentProfile = new ContentProfile();
		BasicContentDataType content = new BasicContentDataType();
		// Just a dummy Container Provider Id
		content.setContentProviderId("test_content_provider_id");
		content.setContentId(emailTemplate.getTemplateName());
		contentProfile.setContent((BasicContentDataType) content);

		TemplateProperties properties = new TemplateProperties();

		List<AttributeDataType> attributes = properties.getAttribute();

		Set<Object> keys = emailTemplate.getProperties().keySet();
		for (Object key : keys) {
			AttributeDataType attribute = new AttributeDataType();
			attribute.setName((String) key);
			attribute.setValue((String) emailTemplate.getProperties().get(key));
			attributes.add(attribute);
		}
		contentProfile.setTemplateProperties(properties);
		SendResponse response;
		try {
			response = notificationPort.send(headerHolder, createRequest(contentProfile, destinations));
		} catch (Fault e) {
			sLogger.error(e.getMessage());
			throw new InfrastructureException(e);
		}
		validateResponse(response);
	}

	/*
	 * Displays SOAP repsonse
	 */
	private void validateResponse(SendResponse response) {
		if (response == null) {
			sLogger.error("Response is null");
			return;
		}
		ErrorDataType error = response.getError();
		if (error != null) {
			displayError(error);
			return;
		}
		List<Result> results = response.getResult();
		for (Result result : results) {
			sLogger.debug("   RecipientId: " + result.getRecipientId());
			if (result.getSuccess() != null) {
				sLogger.debug("   Success");
			} else {
				displayError(result.getError());
			}
		}
	}

	/*
	 * Displays error
	 */
	private void displayError(ErrorDataType error) {
		if (error != null) {
			sLogger.error("   Error Category:    " + error.getCategory());
			sLogger.error("   Error Code:        " + error.getCode());
			sLogger.error("   Error Description: " + error.getDescription());
			sLogger.error("   Error Source:      " + error.getSource());
		}
	}
}