package com.intuit.sb.payroll.notification.template;

import java.util.Properties;

/**
 * Interface to define the Email Template used from the Notification Service. 
 * Notification Services uses Email Templates created/maintained at the Notification
 * Service site. <br>
 * These templates contain a unique identifier (Template Name) that
 * should be used to locate the template to send any mail.<br>
 * The templates along with the name as contain variables that are required to set the 
 * attributes we have defined during the creation of templates.
 *
 */
// TODO: some kind of validation on properties may be required

public interface EmailTemplate 
{
	/**
	 * Set the key value pair corresponding to the attributes defined in the
	 * template. The key name is case-senstive
	 * @param key
	 * @param value
	 */
	public void setProperty(String key, String value);
	
	/**
	 * Get the Properties object containing the key value pair
	 * @return Properties
	 */
	public Properties getProperties();
	
	/**
	 * Get the default template name. This name identifier is defined at the
	 * Notification Service.
	 * @return
	 */
	public String getTemplateName();
	
	/**
	 * Set the template Name. The template name is used as an identifier to locate the
	 * template use to send the notification
	 * @param templateName
	 * @return
	 */
	public void setTemplateName(String templateName);
}
