package com.intuit.sb.payroll.rules.impl;

import java.net.URL;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.core.rules.remote.RuleException;
import com.intuit.sb.core.rules.remote.RuleService;
import com.intuit.sb.payroll.rules.RulesBridge;
import com.intuit.sb.payroll.rules.RulesEngineFactory;
import com.intuit.sb.payroll.rules.impl.drools.DroolsJsr94RulesFacade;
import com.intuit.sb.payroll.rules.impl.drools.DroolsStatelessSession;

@Stateless(name = "RuleService")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class RuleServiceLocalBean implements RuleService
{
	private static DroolsJsr94RulesFacade rulesFacade;
	private String configPath;
	private static boolean initialized = false;

	@SuppressWarnings("unchecked")
	public List runRules(String uri, List request) throws RuleException
	{
		DroolsStatelessSession rulesSession = null;
		try
		{
			rulesSession = rulesFacade.getStatelessRuleSession(uri);
			List outputObjects = rulesSession.executeRules(request);
			return outputObjects;
		} catch (Exception e)
		{
			throw new RuleException(e);
		} finally
		{
			if (rulesSession != null)
			{
				rulesSession.Release();
			}
		}
	}

	public void registerRulesets() throws RuleException
	{
		try {
			URL metaUrl = this.getClass().getResource(RulesBridge.PAYROLL_COMPONENT_RULES_CFG);
			rulesFacade.registerRulesets(metaUrl);
		} catch (Exception e) {
			throw new RuleException(e);
		}
		
	}
	
	public void unRegisterRulesets(final URL configUrl) throws RuleException
	{
		rulesFacade.unRegisterRulesets(configUrl);
	}

	public void initialize() throws RuleException
	{
		try
		{
			rulesFacade = RulesEngineFactory.getInstance().getRulesProvider();
			rulesFacade.initializeRulesRuntime();
		}
		catch(Exception e)
		{
			System.out.println("Failed to start the rules runtime.");
			throw new RuleException(e);
		}
	}

	public String getConfigPath() {
		return configPath;
	}

	public void setConfigPath(String configPath) {
		this.configPath = configPath;
	}
 
	@SuppressWarnings("unchecked")
	public List executeRules(String uri, List arg1, Map arg2)
			throws RuleException {
		if (!initialized){
			initialize();
			registerRulesets();
			initialized = true;
		}
		
		return this.runRules(uri, arg1);
	}
}
