package com.intuit.sb.payroll.rules.impl.drools;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.util.HashMap;
import java.util.LinkedHashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.drools.compiler.PackageBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.rules.RulesException;

// This singleton class provides the helper functionality to load rules
// specified a uri
public class DroolsHelper {
	
	public DroolsHelper()
	{
	}
	
	//initializes the mapping between the uris and the actual drl and dsl files
	public HashMap<String,LinkedHashMap<String,String>> initializeRulesMap(final URL metaUrl) throws RulesException, IOException, SAXException, ParserConfigurationException
	{
        InputStream inStream = null;
        Document doc = null;
        
        try
        {
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			docBuilderFactory.setValidating(true);
		    DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
		    inStream = metaUrl.openStream();
		    doc = docBuilder.parse(inStream);
        }
    	finally
    	{
    		if(inStream != null)
    		{
    			try 
                {
					inStream.close();
				} 
    			catch (IOException e) 
                {
				}
    		}
    	}

        LinkedHashMap<String,String>rulesMap = null;
    	String rulesetName = null;
        HashMap<String,LinkedHashMap<String,String>> rulesetsUriMap = new HashMap<String,LinkedHashMap<String,String>>(); // key is the rulesetName

	    NodeList listRuleset = doc.getElementsByTagName("Ruleset");
	    for(int s=0; s < listRuleset.getLength() ; s++)
	    {
	    	rulesMap = new LinkedHashMap<String,String>(); //holds the uri to actual drl mapping

	    	Node firstRuleSetNode = listRuleset.item(s);
	        if(firstRuleSetNode.getNodeType() == Node.ELEMENT_NODE)
	        {
	            Element firstRulesetElement = (Element)firstRuleSetNode;
	            rulesetName = firstRulesetElement.getAttribute("Name");
	            
	            if(rulesetName == null || rulesetName.trim().length() == 0)
	            {
	            	throw new RulesException("Ruleset name not defined in payroll-rules-cfg.xml");
	            }
	            
	            // get the packages for the RuleSet
	            NodeList rulesetPackages = firstRulesetElement.getElementsByTagName("Package");
	
	            for(int k = 0; k < rulesetPackages.getLength(); k++)
	            {
	            	Node rulesetPackageFirstNode = rulesetPackages.item(k);
	            	
			        if(rulesetPackageFirstNode.getNodeType() == Node.ELEMENT_NODE)
			        {
			            Element firstRuleSetPackageElement = (Element)rulesetPackageFirstNode;
			            NodeList firstRulesetList = firstRuleSetPackageElement.getElementsByTagName("uri");
			            Element uriElement = (Element)firstRulesetList.item(0);
			
			            NodeList textURIList = uriElement.getChildNodes();
			            String uri = ((Node)textURIList.item(0)).getNodeValue().trim();
			
			            // get the url entry for the RuleSet
			            NodeList urlList = firstRuleSetPackageElement.getElementsByTagName("url");
			            Element urlElement = (Element)urlList.item(0);
			
			            NodeList textURLList = urlElement.getChildNodes();
			            String url = ((Node)textURLList.item(0)).getNodeValue().trim();

			            if(uri == null || uri.trim().length() == 0)
			            {
			            	throw new RulesException("uri not defined for ruleset " + rulesetName);
			            }
			            if(url == null || url.trim().length() == 0)
			            {
			            	throw new RulesException("url not defined for ruleset " + rulesetName);
			            }
			            
			            //add the uri and corresponding url to the HashTable
			            rulesMap.put(uri, url);
			        }
	            }
	        }//end of if clause
	        
	        rulesetsUriMap.put(rulesetName, rulesMap);
	    }//end of for loop

		return rulesetsUriMap;
	}
	
    public org.drools.rule.Package ReadRule(final String url) throws Exception
    {
    	//this code may change
        InputStream stream = getClass().getResourceAsStream(url);
        if (null == stream) {
        	throw new InfrastructureException("Cannot load Rule with url " + url.toString());
        }
        try {
        	Reader source = new InputStreamReader(stream);
            PackageBuilder builder = new PackageBuilder( );

            //this wil parse and compile in one step
            //NOTE: There are 2 methods here, the one argument one is for normal DRL.
            // All the action happens in here.....
            try {
            	builder.addPackageFromDrl( source );
            } finally {
            	source.close();
            }
            //get the compiled package (which is serializable)
            org.drools.rule.Package pkg = builder.getPackage();
            return pkg;
        } finally {
        	stream.close();
        }
    }

    /**
     * Returns a string containing the throwable message or, if it's null,
     * the string representation of the throwable (i.e. the exception
     * class name).
     * @param t the throwable
     * @return a non-null message from the throwable
     */
    public static String throwableMessage(Throwable t)
    {
    	return (t.getMessage() != null) ? t.getMessage() : t.toString();
    }

    /**
     * Returns a string containing all the nested messages in the throwable or, if it's null,
     * the string representation of the throwable (i.e. the exception class name).
     * @param t the throwable
     * @return a non-null message from the throwable
     */
    public static String throwableMessageChain(Throwable t)
    {
        StringBuffer mesg = new StringBuffer(throwableMessage(t));
        for (Throwable chainedThrowable = t.getCause(); chainedThrowable != null; chainedThrowable = chainedThrowable.getCause()) {
        	mesg.append(": ").append(throwableMessage(chainedThrowable)); //NORES
        }
    	return mesg.toString();
    }

}
