package com.intuit.sb.payroll.capability.agency.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.payroll.agency.dao.DAOFactory;
import com.intuit.payroll.agency.util.IAgencyHoliday;
import com.intuit.payroll.agency.util.RulesCalendar;
import com.intuit.sb.payroll.capability.agency.AgencyComponent;
import com.intuit.sb.payroll.capability.agency.Holiday;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;

@Stateless(name = "AgencyComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class AgencyComponentBean implements AgencyComponent {

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.agency.AgencyComponent#getHolidaysForPeriod(com.intuit.sb.payroll.entity.model.DateRange)
	 */
	public List<Holiday> getHolidaysForPeriod(DateRange period) {
		//Gets the Holidays from the AgencyRules in the PayrollEngine.
		Iterable<IAgencyHoliday> holidays = DAOFactory.getDAOFactory().getHolidayGroupDAO().getHolidays("FEDERAL");
		List<Holiday> results = new ArrayList<Holiday>();
		for (IAgencyHoliday agencyHoliday : holidays) {
			Date date = toDate(agencyHoliday.getHolidayDate());
			if (DateUtil.isInPeriod(date, period.getStartDate(), period.getEndDate())) {
				Holiday holiday = new Holiday(agencyHoliday.getUIString(), date);
				results.add(holiday);
			}
		}
		return results;
	}

	/**
	 * Converts a RulesCalendar to a Java Date.
	 * @param holidayDate
	 * @return converted Date.
	 */
	private Date toDate(RulesCalendar holidayDate) {
		//Note - Rules Calendar uses base 1 months (dot net style), we require base 0.
		return DateUtil.createDate(holidayDate.getYear(), holidayDate.getMonth() -1, holidayDate.getDay());
	}

}
