package com.intuit.sb.payroll.capability.compensation.impl;

import java.math.BigDecimal;
import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.DatumType1Enum;

/**
 * Validates CompensationDtls.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class CompensationDtlValidator extends AbstractEntityDataValidator<CompensationDtl> {

	private static CompensationDtlValidator sInstance = new CompensationDtlValidator();
	
	private CompensationDtlValidator() {}
	
	public static CompensationDtlValidator getInstance() {
		return sInstance;
	}
	
	@Override
	public void validateEntity(CompensationDtl compensationDtl) throws ValidationException {

		PayrollParamValidator.checkIsNotNull(compensationDtl, "CompensationDtl");
			
		//Validating negative money rates
		if (compensationDtl.getRate()!= null && compensationDtl.getRate().getType1() == DatumType1Enum.MONEY) {
			if (compensationDtl.getRate().getValue1AsUSDMoneyAmount().compareTo(new BigDecimal(0)) < 0)
			{
				//FIXME [I18N] resource translation
				throw new ValidationException("Compensation Rate must not be negative");
			}
		}
		
		// Validate negative units
		if (compensationDtl.getQuantity() != null) {
			if (compensationDtl.getQuantity().getValue1AsQuantity() < 0) {
					//FIXME [I18N] resource translation
					throw new ValidationException("Compensation Quantity must not be negative");
			}
		}
		
		if (compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYOVT ||
				compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYDBL) {
			if (compensationDtl.getLinkedPayrollItemDtl() == null) {
				throw new ValidationException("Overtime and Doubletime require a linked Hourly Item");
			}
		}
	}

	@Override
	public void validateEntityCollection(List<CompensationDtl> entityCollection) throws ValidationException {
		for (CompensationDtl compensationDtl : entityCollection) {
			validateEntity(compensationDtl);
		}

	}
}
