/**
 * Copyright: 2007, Intuit, Inc. All rights reserved.
 *
 */
package com.intuit.sb.payroll.capability.config;

import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;

import org.apache.log4j.Logger;
import org.jboss.mx.util.MBeanProxyExt;
import org.jboss.mx.util.MBeanServerLocator;

import com.intuit.sb.payroll.capability.config.impl.GeminiConfigurationServiceMBean;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;

/**
 * Utility class that provides the caller with a value of the property to 
 * enable/disable "Setup Assisted Task" from showing up
 */
public class GeminiConfigurationUtility {

	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(GeminiConfigurationUtility.class);
	private static GeminiConfigurationService geminiCongifService = null;
	
	/**
	 * Read the user specified boolean via the MBean and return its value
	 * @return value specified to show Tax Payments task after user has done payroll run
	 * This is used to show the "Prepare Quarterly Taxes" TO-DO item and other UI artifacts for Tax Payment Service
	 */
	public static boolean isTaxPaymentsEnabled() {
		
		boolean enableTaxPayments = false;
		GeminiConfigurationService geminiCongifService = getGeminiConfigurationService();
		if (geminiCongifService != null) {
			enableTaxPayments = Boolean.valueOf(geminiCongifService.isTaxPaymentServiceEnabled());
		} else {	//	for tests - embedded container - default
			enableTaxPayments = Boolean.TRUE;
		}
		
		return enableTaxPayments;

	}
	

	/** Get the MBean service object */
	public static GeminiConfigurationService getGeminiConfigurationService() {
		// read JMX runtime - for tests, set the "show tax payments" to true
		//	TODO see if the MBean instance can be retrieved via generic service locator
		if (!ServiceLocator.getInstance().isEmbedded()) {	//	runtime case
			MBeanServer mbeanServer = MBeanServerLocator.locate();
			try {
				sLogger.debug("Locating the GeminiConfigurationService MBean");
				geminiCongifService = (GeminiConfigurationService)MBeanProxyExt.create(GeminiConfigurationService.class, GeminiConfigurationService.JNDI_NAME, mbeanServer);

				if (geminiCongifService == null) {
					sLogger.error("Unable to locate GeminiConfigurationService MBean");
					throw new RuntimeException("Unable to locate GeminiConfigurationService MBean");
				}
			} catch (MalformedObjectNameException e) {
				sLogger.error("Unable to locate GeminiConfigurationService MBean");
				throw new RuntimeException(e);
			}
		}else{
			if (geminiCongifService == null){
				geminiCongifService = new GeminiConfigurationServiceMBean();
				try {
					geminiCongifService.create();
				} catch (Exception e) {
					sLogger.error("Unable to create GeminiConfigurationService MBean");
					throw new RuntimeException(e);
				}
			}
		}
		
		return geminiCongifService;
	}


	public static String getTaxPaymentServiceHost() {
		GeminiConfigurationService geminiConfigService = getGeminiConfigurationService();
		return geminiConfigService.getTaxPaymentServiceHost();
	}
}
