/**
 * Copyright: 2007, Intuit, Inc. All rights reserved.
 *
 */
package com.intuit.sb.payroll.capability.config.impl;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.jboss.annotation.ejb.Management;
import org.jboss.annotation.ejb.Service;
import org.jboss.annotation.javaee.Description;

import com.intuit.sb.payroll.capability.config.GeminiConfigurationService;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;

/**
 * FIXME: [th] move to gemini app at some point.
 * JMX MBean to manage the property based on which we will fire the Setup Assisted Payments task
 *
 */
@Service(objectName = "geminiJMX:service=GeminiConfigurationMBean")
@Management(GeminiConfigurationService.class)
public class GeminiConfigurationServiceMBean implements GeminiConfigurationService {

	/**
	 * Properties containing flag whether showTaxPayments is true or false;
	 * 
	 */
	private static Properties properties = new Properties();
	
	private static final String _MBEAN_CONFIG_FILE = "GeminiConfigurationServiceMBean.properties";
	private static final String _IS_TAX_PAYMENT_SERVICE_ENABLED = "is_tax_payment_service_enabled"; 
	private static final String _TPS_WSDL_URL = "tps_wsdl_url"; 
	
	static private final Logger logger = PayrollLoggerFactory.makeLogger(GeminiConfigurationServiceMBean.class);
	
	private boolean isTaxPaymentServiceEnabled;	//	value of display tax payments boolean - decides if tax payments option should show up
	private String taxPaymentServiceHost = null; 
	
    
    /** Read property file to decide whether to enable/disable "showTaxPayments" */
    static {
    	
		String thisPackage = GeminiConfigurationService.class.getPackage().getName();
		String configFileName = thisPackage.replace('.', '/') + "/" + _MBEAN_CONFIG_FILE;
		InputStream propertyStream = Thread.currentThread().getContextClassLoader().getResourceAsStream(configFileName);
		try {
			properties.load(propertyStream);
		} catch (IOException e) {
			logger.error("Unable to load " + _MBEAN_CONFIG_FILE, e);
		}
    }

    @Description("Host machine for Tax Payment service")
    public String getTaxPaymentServiceHost(){
	   return this.taxPaymentServiceHost;
    }
   
    public void setTaxPaymentServiceHost(String url){
	    this.taxPaymentServiceHost = url;
    }

    public void setTaxPaymentServiceEnabled(boolean isTaxPaymentServiceEnabled) {
		this.isTaxPaymentServiceEnabled = isTaxPaymentServiceEnabled;
	}

	@Description("Boolean to turn Tax Payment Service on/off")
    public boolean isTaxPaymentServiceEnabled() {
        return isTaxPaymentServiceEnabled;
    }

	// Lifecycle methods
    public void create() throws Exception {
    	// initialize showTaxPayments
    	if (properties != null) {
    		isTaxPaymentServiceEnabled = Boolean.valueOf(properties.getProperty(_IS_TAX_PAYMENT_SERVICE_ENABLED));
    		taxPaymentServiceHost = properties.getProperty(_TPS_WSDL_URL);
    	} else {
    		isTaxPaymentServiceEnabled = false;	//  if prop file not specified, default 
    	}
        logger.debug("GeminiConfigurationServiceMBean - Creating");
    }

    public void destroy() {
    	//  TODO - save variable value to persistence location??? [ri]
        logger.debug("ShowHideTaxPaymentsMBean - Destroying");
    }
    
}