package com.intuit.sb.payroll.capability.employer.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.adapter.PTSRequestInvoker;
import com.intuit.sb.payroll.capability.pts.employer.EmployerPTSWorkflow;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.RequiredData;
import com.intuit.sb.payroll.common.ResourceKeyConstants;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.employer.EmployerEAO;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.message.PayrollMessageService;
import com.intuit.sb.payroll.pts.entity.BankAccountState;
import com.intuit.sb.payroll.pts.entity.BankAccountStateEAO;
import com.intuit.sb.payroll.pts.entity.BankAccountStatusEnum;
import com.intuit.sb.payroll.pts.entity.InteractionEnum;
import com.intuit.sb.payroll.pts.entity.PPSPTSInteraction;
import com.intuit.sb.payroll.pts.entity.PPSPTSInteractionEAO;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSWorkflowState;
import com.intuit.sb.payroll.pts.entity.StateIDModified;
import com.intuit.sb.payroll.pts.entity.StateIDModifiedEAO;
import com.intuit.sb.payroll.pts.entity.TaxServiceState;
import com.intuit.sb.payroll.pts.entity.TaxServiceStatusEnum;



/**
 * Implementation of the Employer WorkFlow Management functions for Payroll Tax Service
 * 
 */
@Stateless(name = "EmployerPTSWorkflowComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class EmployerPTSWorkflowComponentBean implements EmployerPTSWorkflowComponent {
	
	private static Logger sLogger = PayrollLoggerFactory.makeLogger(EmployerPTSWorkflowComponentBean.class);
	
	@EJB
	EmployerComponent employerComponent;
	
	@EJB
	EntityService entityService;
	
	@EJB
	TaskManager  taskManager;
	
	@EJB
	PPSPTSInteractionEAO ppsPtsInteractionEAO;
	
	@EJB
	BankAccountStateEAO bankAccountStateEAO;
	
	@EJB
	StateIDModifiedEAO stateIDModifiedEAO;
	
	@EJB 
	EmployerEAO employerEAO;
	
	@EJB
	PayrollMessageService messageService;
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#saveEmployer(java.lang.String)
	 */
	private void createPTSEmployer(Employer employer) throws ServiceFailureException {

		// Create PTS Employer if required
		List<PTSEmployer> ptsEmployerList = entityService.findAll(PTSEmployer.class);

		// If there are no PTS employers in the DB, create one.
		if (ptsEmployerList == null || ptsEmployerList.size() == 0) {
			PTSEmployer ptsEmployer = new PTSEmployer();
			employer = entityService.makeManaged(employer);
			ptsEmployer.setEmployer(employer);
			
			PTSWorkflowState ptsWorkflowState = new PTSWorkflowState();
			TaxServiceState taxServiceState = new TaxServiceState();
			ptsWorkflowState.setTaxServiceState(taxServiceState);
			ptsWorkflowState = entityService.makePersistent(ptsWorkflowState);
			ptsEmployer.setPTSWorkflowState(ptsWorkflowState);
			ptsEmployer.setCurrentTaxServiceStatus(TaxServiceStatusEnum.NOT_SIGNED_FOR_ASSISTED);
			entityService.makeManaged(ptsEmployer);
		}
	}

	public PTSEmployer savePTSEmployer(PTSEmployer employer) throws ValidationException, ServiceFailureException {
		
		try {
			if (entityService.notPersistent(employer)) {
				employer = entityService.makeManaged(employer);
			}
			return employer;
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}
	}

	// Contact Person Functions

	public Set<PTSContactPerson> addEmployerContactPersons(List<PTSContactPerson> employerContactPersons)
			throws ValidationException, ServiceFailureException {

	     List<EmployerContactPerson> employerContactPersonList = new ArrayList<EmployerContactPerson>();

          for (PTSContactPerson ptsemployerContactPerson : employerContactPersons){
        	  EmployerContactPerson employerContactPerson = ptsemployerContactPerson.getEmployerContact();
        	  employerContactPersonList.add(employerContactPerson);
	      }

          employerComponent.addEmployerContactPersons(employerContactPersonList);	

	      PTSEmployer ptsEmployer = getPTSEmployer();
	      for (PTSContactPerson ptsemployerContactPerson : employerContactPersons) {
	          ptsEmployer.addPTSContactPerson(ptsemployerContactPerson);
	      }

	      ptsEmployer = savePTSEmployer(ptsEmployer);
	      handleEmployerEvent(WorkflowEventType.ADD_EMPLOYER_CONTACT , ptsEmployer.getEmployer());
	      return ptsEmployer.getPTSContactPersons();
	}

	public Set<PTSContactPerson> updateEmployerContactPersons(
			List<PTSContactPerson> updateEmployerContactPersons) throws ValidationException, ServiceFailureException {

		PTSEmployer ptsEmployer = getPTSEmployer();
        Set<PTSContactPerson> ptsContactPersons = ptsEmployer.getPTSContactPersons();
        for (PTSContactPerson contactPerson : updateEmployerContactPersons) {
        	for (PTSContactPerson ptsContactPerson : ptsContactPersons) {
				if (contactPerson.getId() == ptsContactPerson.getId()) {
					contactPerson = ptsContactPerson;
					break;
				}
			}
		}
        
        savePTSEmployer(ptsEmployer);
        handleEmployerEvent(WorkflowEventType.UPDATE_EMPLOYER_CONTACT , ptsEmployer.getEmployer());
        return ptsEmployer.getPTSContactPersons();
	}

	public void deleteEmployerContactPersons(List<PTSContactPerson> ptsContactPersons)
			throws ValidationException, ServiceFailureException {
		PTSEmployer ptsEmployer = getPTSEmployer();
		List<EmployerContactPerson> employerContactsToDelete = new ArrayList<EmployerContactPerson>();
		for (PTSContactPerson contactPerson : ptsContactPersons) {
			if(ptsEmployer.getPTSContactPersons().contains(contactPerson))
			{
				employerContactsToDelete.add(contactPerson.getEmployerContact());
				ptsEmployer.removePTSContactPerson(contactPerson);
			}
		}
		savePTSEmployer(ptsEmployer);
		employerComponent.deleteEmployerContactPersons(employerContactsToDelete);
	}

	public void deleteAllContactPersons() throws ValidationException, ServiceFailureException {

		PTSEmployer ptsemployer = getPTSEmployer();
		Employer employer = ptsemployer.getEmployer();
		Set<PTSContactPerson> ptsContactPersonSet = ptsemployer.getPTSContactPersons();
		
		for (PTSContactPerson contactPerson : ptsContactPersonSet) {
			EmployerContactPerson empContactPerson = contactPerson.getEmployerContact();
			contactPerson.setEmployerContact(null);
			if (employer.getEmployerContactPersons().contains(empContactPerson)){
				employer.removeEmployerContactPerson(empContactPerson);
			}
		}
		
		// Remove all PTS ContactPerson
		ptsemployer.removeAllPTSContactPersons();
		employerComponent.saveEmployer(employer);
        savePTSEmployer(ptsemployer);
	}
	
	public void updatePPS_PTS_InteractionState(Identifiable entity, InteractionEnum interactionType) throws ValidationException, ServiceFailureException{
		if (entity == null){
			throw new ValidationException("Entity interacted with PSP is null");
		}
		String entityname = StatusUpdateHandlerUtil.getCertifiedEntityName(entity);
		long entityId = entity.getId();
		sLogger.debug("PTS Interaction " + entityname + " "+ entityId + " "+ interactionType);
		// Get the existing one if available
		PTSEmployer ptsEmployer = getPTSEmployer();
		PPSPTSInteraction interaction = StatusUpdateHandlerUtil.getPPSPTSInteraction(entityname, entityId, interactionType);
		if (interaction == null){
			interaction = new PPSPTSInteraction();
			interaction.setEntity(entityname);
			interaction.setIdentifier(new Long(entityId));
			interaction.setOperation(interactionType);
			interaction.setInteractionDate(new Date());
			ptsEmployer.addPPSPTSInteraction(interaction);
		}else{
			interaction.setInteractionDate(new Date());
		}
		
		if (ppsPtsInteractionEAO.notPersistent(interaction)) {
			interaction = ppsPtsInteractionEAO.makePersistent(interaction);
		}
		ptsEmployer = savePTSEmployer(ptsEmployer);
	}
	
	public void updateEmployerTaxServiceState(TaxServiceStatusEnum taxServiceStatus) throws ServiceFailureException, ValidationException {
		if (taxServiceStatus == null){
			throw new ValidationException("Trying to update PTSEmployer will null status");
		}
		
		PTSEmployer ptsEmployer = getPTSEmployer();
		ptsEmployer.setCurrentTaxServiceStatus(taxServiceStatus);
		savePTSEmployer(ptsEmployer);
		
		if (taxServiceStatus.equals(TaxServiceStatusEnum.SIGNED_UP_FOR_ASSISTED)){
			messageService.dispatchAssistedSetupEvent(WorkflowEventType.ASSISTED_SETUP_COMPLETE, ptsEmployer);
		} else if (taxServiceStatus.equals(TaxServiceStatusEnum.READY_FOR_ADD_COMPANY)){
			sendCompanyInfoToPTS();
		} else if (taxServiceStatus.equals(TaxServiceStatusEnum.ACTIVATION_ACTIVE_ST_ACCEPTED)){
			EmployerPTSWorkflowTasks.createPtsActivationActiveAcceptedTask();
			// Send Accept Tax service
			EmployerPTSWorkflow.sendAcceptTaxService(ptsEmployer);
		} else if (taxServiceStatus.equals(TaxServiceStatusEnum.ACTIVATION_ACTIVE_ST_REJECTED)){
			EmployerPTSWorkflowTasks.closePtsActivationActiveTask();
		} else if (taxServiceStatus.equals(TaxServiceStatusEnum.PRE_ACTIVATE)) {
			messageService.dispatchPreAssistedActivationEvent(WorkflowEventType.EMPLOYER_IN_PRE_ACTIVATION, ptsEmployer);
		}
	}
	
	public PTSWorkflowState getEmployerAssistedState(Employer employer)
	  throws ValidationException {
		PTSWorkflowState ptsWorkflowState = null;
		List<PTSWorkflowState> ptsWorkflowStateList = entityService.findAll(PTSWorkflowState.class);
		if (ptsWorkflowStateList.size() > 0) {
			ptsWorkflowState = ptsWorkflowStateList.get(0);
		}
		return ptsWorkflowState;
		
		
	}
	
	public PTSContactPerson findEmployerContactPerson(Long empContactPersonuniqueId) throws ServiceFailureException{
		PTSEmployer employer = getPTSEmployer();

		Set<PTSContactPerson> ptsContactPersons = employer.getPTSContactPersons();
		for (PTSContactPerson contact : ptsContactPersons) {
			if (empContactPersonuniqueId.equals(new Long(contact.getId()))) {
				return contact;
			}
		}
		throw new ServiceFailureException(PayrollResourceManager.getInstance().getString(
				ResourceKeyConstants.ID_NOT_FOUND_EXCEPTION)
				+ empContactPersonuniqueId.toString());
		}
		
		
	public PTSEmployer getPTSEmployer(){
		List<PTSEmployer> ptsEmployerList = entityService.findAll(PTSEmployer.class);
		if (ptsEmployerList.size() > 0){
			return ptsEmployerList.get(0);
		}else{
			return null;
	    }
	}


    /**
     * Send Company information to PTS
     * -- Company information, Jurisdiction + Agency information, bank information
     */
	
	// TODO uphook this from directly being called from Service
	public void sendCompanyInfoToPTS() throws ServiceFailureException, ValidationException {
		PTSEmployer ptsEmployer = getPTSEmployer();
		boolean updateEmployer = handleUpdateEmployerCase(ptsEmployer);
		if (updateEmployer) {
			PTSRequestInvoker.updateEmployerInPTS(ptsEmployer);
		} else {	//	add employer
			PTSRequestInvoker.addEmployerInPTS(ptsEmployer);
			messageService.dispatchAssistedSetupEvent(WorkflowEventType.EMPLOYER_ADDED_IN_PTS, ptsEmployer);
		}
	}
	/**
	 * When Company data is being sent to PTS, check by looking at <br>  
	 * {@code EmployerPTSWorkflowTasks.REJECTED_EIN_MISMATCH_TASK} and {@code} 
	 * EmployerPTSWorkflowTasks.PRINT_AND_RESUBMIT_FORMS <br>
	 * If these tasks exist, then we update, else we add.
	 * @param ptsEmployer
	 */
	private boolean handleUpdateEmployerCase(PTSEmployer ptsEmployer) throws ServiceFailureException {
		boolean updateEmployer = EmployerPTSWorkflowTasks.updateEmployerTasksPresent(ptsEmployer);
		
		if (updateEmployer) {
			EmployerPTSWorkflowTasks.closeEmployerUpdateTasks(ptsEmployer);
		}

		return updateEmployer;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent#handleAssistedSetupEvent(com.intuit.sb.payroll.message.WorkflowEventType, com.intuit.sb.payroll.pts.entity.PTSEmployer)
	 */
	public void handleAssistedSetupEvent(WorkflowEventType workflowEventType, PTSEmployer employer) throws ServiceFailureException {
		if (WorkflowEventType.ASSISTED_SETUP_COMPLETE == workflowEventType) {
			EmployerPTSWorkflowTasks.handlePTSEmployerSetupComplete(taskManager, employer);
		}else if (WorkflowEventType.EMPLOYER_ADDED_IN_PTS == workflowEventType) {
			//	Send all the available Data to PTS
			handleEmployerCreationInPTSEvent();
		}
	}

	/**
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent#handleEmployerEvent(com.intuit.sb.payroll.common.WorkflowEventType, com.intuit.sb.payroll.entity.model.Employer)
	 */
	public void handleEmployerEvent(WorkflowEventType eventType, Employer employer) throws ServiceFailureException {
		
		if (WorkflowEventType.CREATE_EMPLOYER == eventType) {
			createPTSEmployer(employer);
		} else if (WorkflowEventType.UPDATE_EMPLOYER == eventType) {
			try {
				savePTSEmployer(getPTSEmployer());
			} catch (ValidationException e) {
				throw new ServiceFailureException(e);
			}
			// TODO refactor again after making UPDATE EMP more granular
			EmployerPTSWorkflowTasks.handleEmployerUpdatedEvent(getPTSEmployer(),eventType);
		} else if (WorkflowEventType.UPDATE_EMPLOYERID == eventType) { 
			//	create a task for "PRINT_AND_RESUBMIT" forms
			EmployerPTSWorkflowTasks.handleEmployerUpdatedEvent(getPTSEmployer(),eventType);
			
		}else if (WorkflowEventType.PREFETCH_EMPLOYERID.equals(eventType)){
			handleEmployerIdPreFetch();
		}else if (WorkflowEventType.ADD_EMPLOYER_CONTACT.equals(eventType) ||
				  WorkflowEventType.UPDATE_EMPLOYER_CONTACT.equals(eventType))
		{
			try {
				// Allow this call only if Employer has enough data to talk to PSP
				List<RequiredData> missingList = EmployerSetupRules.getMissingDataForWorkflow(EmployerWorkflowType.CreateAssistedCompany, 
						getPTSEmployer());
				if (missingList.size() == 0){
					// Send any updated or new information to PSP is company is registered
					if (StatusUpdateHandlerUtil.isEntityAddedToPTS(employer)){
						PTSRequestInvoker.updateEmployerInPTS(getPTSEmployer());
					}
				}
			} catch (ValidationException e) {
				throw new ServiceFailureException(e);
			}
		}
		
	}
	
	private void handleEmployerCreationInPTSEvent() {
		
		PTSEmployer ptsEmployer = getPTSEmployer();
		
		// Send Jurisdiction
		try {
			
			sLogger.debug("Sending all existing Jurisdiction to PTS");
			List<Jurisdiction> jurisdictionList = employerComponent.determineAllJurisdictions();
			for (Jurisdiction jurisdiction : jurisdictionList) {
				PTSRequestInvoker.addJurisdictionIdToPTS(ptsEmployer, jurisdiction);
			}
			
		} catch (ValidationException e) {
			e.printStackTrace();
		} catch (ServiceFailureException e) {
			e.printStackTrace();
		}
		// Send Agency
		try{
			// Sending AgencyId if available
			// TODO check if this method is already available
			EmployerId employerId = PTSRequestInvoker.getEmployerIDforAgency(ptsEmployer.getEmployer(), AgencyEnum.CA_EDD);
			if (employerId != null){
				PTSRequestInvoker.addCompanyTaxAgencyId(ptsEmployer, employerId);
			}
		} catch (ValidationException e) {
			e.printStackTrace();
		} catch (ServiceFailureException e) {
			e.printStackTrace();
		}

		// Send Bank Account to PSP
		Set<BankAccount> bankAccounts = ptsEmployer.getEmployer().getBankAccounts();

		/* Iterate through every account to be added and send it to PTS */
		for (BankAccount bankAccount : bankAccounts) {
			try {
				PTSRequestInvoker.addBankAccountInfotoPTS(ptsEmployer, bankAccount);
			} catch (ValidationException e) {
				e.printStackTrace();
			} catch (ServiceFailureException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * (non-Javadoc)
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 * @see com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent#handleEmployerBankAccountEvent(com.intuit.sb.payroll.message.WorkflowEventType, List<BankAccount>)
	 */
	public void handleEmployerBankAccountEvent(
			WorkflowEventType workflowEventType, List<BankAccount> bankAccounts) throws ValidationException, ServiceFailureException{
		if (WorkflowEventType.ADD_EMPLOYER_BANKACCOUNT.equals(workflowEventType)) {
			EmployerPTSWorkflow.addBankAccount(getPTSEmployer(), bankAccounts );
		}else if (WorkflowEventType.UPDATE_EMPLOYER_BANKACCOUNT.equals(workflowEventType)) {
			EmployerPTSWorkflowTasks.handleBankAccountUpdate(getPTSEmployer(),bankAccounts); 
			EmployerPTSWorkflow.updateBankAccount(getPTSEmployer(), bankAccounts );
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public Date getTaxPaymentsFormSubmissionDate() throws ServiceFailureException {
		PTSEmployer ptsEmployer = getPTSEmployer();
		
		Date addCompanyDate = null;
		Date updateCompanyDate = null;
		
		// if UPDATE interaction is there, it would be latest so send that date
		PPSPTSInteraction updateCompanyInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Employer.class.getName(), ptsEmployer.getEmployer().getId(), InteractionEnum.UPDATED);
		if (updateCompanyInteraction != null){
			updateCompanyDate = updateCompanyInteraction.getInteractionDate();
			return updateCompanyDate;
		}
		
		// there is no update interaction, so check for ADD
		PPSPTSInteraction addCompanyInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Employer.class.getName(), ptsEmployer.getEmployer().getId(), InteractionEnum.ADDED);
		if (addCompanyInteraction != null){
			addCompanyDate = addCompanyInteraction.getInteractionDate();
			return addCompanyDate;
		}
		
		// both ADD and UPDATE dates are null - throw exception
			throw new ServiceFailureException(ptsEmployer.getEmployer().getLegalName()
					+ " : Does not have an entry for Company being added");
		}
		
	/**
	 * {@inheritDoc}
	 */
	public boolean allowDataSubmissionToPTS() {
		sLogger.debug("Executing allowDataSubmissionToPTS()");
        PTSEmployer ptsEmployer = getPTSEmployer();
        return StatusUpdateHandlerUtil.allowDataSubmissionToPTS(ptsEmployer);

    }
		
    /**
     *  This method will return the BankAccountState of the BankAccount with the bankAccountID
     *  @param bankAccount
	 *  @return BankAccountState
     */
    public BankAccountState getMostRecentBankAccountState(BankAccount bankAccount) {
        BankAccountState bankAccountState = null;
        if(bankAccount != null){
        	List<BankAccountState> sortedBankAccountStates = bankAccountStateEAO.findAllInSortedOrder();
        	// iterate over the sorted list to get the most recent BankAccountState for this bankAccount
        	for(BankAccountState baState:sortedBankAccountStates){  
        		if(baState.getBankAcctId() == bankAccount.getId()){
        			bankAccountState = baState;
        			break;
        		}
        	} 
        	//Update the Core BankAccount entity with verfied date
        	if ((bankAccountState != null) &&
        		(bankAccountState.getStatus().equals(BankAccountStatusEnum.BANKACCOUNT_VERIFIED_RECEIVED))){
        		// check if this bankAccount verified date is already updated
        		Date verfiedDate = bankAccount.getDateLastVerified();
        		if ((verfiedDate == null) ||
        			(!verfiedDate.equals(bankAccountState.getStatusDate()))){
        			// update the verfied date
        			bankAccount.setDateLastVerified(bankAccountState.getStatusDate());
        			// Save the bankAccount
        			bankAccount = entityService.makeManaged(bankAccount);
        		}
        		
        	}
        	
        }
        return bankAccountState;
	}
    
    private StateIDModified getMostRecentActiveStateIDModifiedNotification(AgencyEnum agency){
    	List<StateIDModified> stateIdList = stateIDModifiedEAO.findAllInSortedOrder();
    	
    	for (StateIDModified stateIDModified : stateIdList) {
    		if ((stateIDModified.getAgency() != null) && 
    			(stateIDModified.getAgency().equals(agency)) &&
    			(Boolean.TRUE.equals(stateIDModified.isActive()))){
    			return stateIDModified;
    		}
		}
    	return null;
    }
    
    /**
     * This method should be hooked up in any biz logic to fetch EmployerId for an
     * Employer in PTS (PSP). PSP will send StateIdModified events to capability and
     * this method will lookup any active events for all agency and update the EmployerId
     * on presence of such event
     * 
     * 
     */
    private void handleEmployerIdPreFetch(){
    	Employer employer = getPTSEmployer().getEmployer();
    	Set<EmployerId> employerIds = employer.getEmployerIds();
    	for (EmployerId employerId : employerIds) {
			AgencyEnum agency = employerId.getAgencyIdType().getAgencyKey();
			// Get the active stateId Modified notification
			StateIDModified stateIdModified = getMostRecentActiveStateIDModifiedNotification(agency);
			
			if (stateIdModified != null){
				// check if this is the correct one
				if ((stateIdModified.getOldValue() != null) &&
					(!stateIdModified.getOldValue().equals(employerId.getValue()))){
					sLogger.warn("StateIdModified Old value does not matches the current value for employerId "+
							employerId.getId());
				}
				sLogger.debug(" employerId "+
						employerId.getId() + " being modified to new value "+ stateIdModified.getNewValue());
				// Set the new value
				employerId.setValue(stateIdModified.getNewValue());
				
				// In activate the activeStateIdModified notification
				StatusUpdateHandlerUtil.inActivateAllStateIDNotificationStates(agency);
			}
		}
    	if (employerEAO.notPersistent(employer)) {
			employer = employerEAO.makePersistent(employer);
		}
    }
}
