package com.intuit.sb.payroll.capability.employer.impl;

import java.util.ArrayList;
import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.impl.evaluator.EmployerEvaluator;
import com.intuit.sb.payroll.common.RequiredData;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;


public class EmployerSetupRules {
	
	private static final String EMPLOYER_EIN_KEY             = "employer.fein.key";
	
	private static final String EMPLOYER_PRIMARY_CONTACT_KEY = "employer.primaryContact.key";
	private static final String EMPLOYER_LEGALNAME_KEY       = "employer.legalName.key";
	private static final String EMPLOYER_LEGAL_ADD_KEY       = "employer.legalAddress.key";
	
	public static final RequiredData employerEIN = new RequiredData(EMPLOYER_EIN_KEY, "EIN", "Employer", "Employer Federal EIN");
	public static final RequiredData employerLegalName = new RequiredData(EMPLOYER_LEGALNAME_KEY, "legalName", 
																	"Employer", "Employer Legal Name ");
	
	public static final RequiredData employerLegalAddress = new RequiredData(EMPLOYER_LEGAL_ADD_KEY, "legalGlobalAddress", 
																	"Employer", "Employer Legal Address");
	
	public static final RequiredData employerPrimaryContact = new RequiredData(EMPLOYER_PRIMARY_CONTACT_KEY, "employerContactPersons", 
																	"Employer", "Employer Primary Contact");
	
	/**
	 * Holds the RequiredData for setting up an Assisted.
	 */
	private static List<RequiredData> assistedRequiredData = new ArrayList<RequiredData>();
	
	static {
		assistedRequiredData.add(employerEIN);
		assistedRequiredData.add(employerLegalName);
		assistedRequiredData.add(employerLegalAddress);
		assistedRequiredData.add(employerPrimaryContact);
		
	}
	
	/**
	 * Returns the full list of {@link RequiredData} for a given Workflow.
	 * @param workflow
	 * @return the list of {@link RequiredData}
	 */
	public static List<RequiredData> getRequiredDataForWorkflow(EmployerWorkflowType workflow) {
		switch (workflow) {
		case CreateAssistedCompany:
			return assistedRequiredData;
		default:
			return new ArrayList<RequiredData>();
		}
	}
	
	public boolean isReadyForWorkflow(EmployerWorkflowType workflow, PTSEmployer ptsEmployer){
		List<RequiredData> leftovers;
		try {
			leftovers = getMissingDataForWorkflow(workflow, ptsEmployer);
			if (leftovers == null) {
				throw new AssertionError("Don't return null");
			}
			if (leftovers.size() > 0) {
				return false;
			} else {
				return true;
			}
		} catch (ValidationException e) {
			return false;
		}
	}

	/**
	 * Given an Employer, this method will return the list of {@link RequiredData} that is missing
	 * for the {@link EmployerWorkflowType}.
	 * 
	 * @param workflow
	 * @param worker
	 * @return
	 * @throws ValidationException
	 */
	public static List<RequiredData> getMissingDataForWorkflow(EmployerWorkflowType workflow, PTSEmployer ptsEmployer)
			throws ValidationException {
		List<RequiredData> allData = getRequiredDataForWorkflow(workflow);
		List<RequiredData> completedData = EmployerEvaluator.getCompletedFields(ptsEmployer, allData);
		return diff(completedData, allData);
	}
	
	/**
	 * FIXME Duplicate from WorkerSetupRules.java
	 * Internal method to return the difference between two lists.
	 * @param completedData
	 * @param allData
	 * @return
	 */
	private static List<RequiredData> diff(List<RequiredData> completedData, List<RequiredData> allData) {
		List<RequiredData> results = new ArrayList<RequiredData>();
		for (RequiredData requiredData : allData) {
			if (!completedData.contains(requiredData)) {
				results.add(requiredData);
			}
		}
		return results;
	}


}
