package com.intuit.sb.payroll.capability.employer.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.EmployerTaxSettingComponent;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.message.PayrollMessageService;

/**
 * Implementation of the Employer Tax Related Management functions
 * 
 * $Author: amalik $ 
 * $Revision: #1 $ 
 * $DateTime: 2009/05/20 15:48:13 $
 */
@Stateless(name = "EmployerTaxSettingComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class EmployerTaxSettingComponentBean implements EmployerTaxSettingComponent{
	
	@EJB
	PayrollMessageService msgService;
	
	@EJB
	EmployerComponent employerComponent;

	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(EmployerTaxSettingComponentBean.class);
	
	public List<EmployerId> getEmployerIds() throws ServiceFailureException {
		Employer employer = employerComponent.getEmployer();
		
		// Dispatch pre-fetch event to update EmployerIds if required
		dispatchWorkflowEvent(WorkflowEventType.PREFETCH_EMPLOYERID, employer);
		
		List<EmployerId> employerIdList = new ArrayList<EmployerId>();
		Set<EmployerId> employerIdSet = employer.getEmployerIds();
		for (EmployerId employerId : employerIdSet) {
			employerIdList.add(employerId);
		}
		return employerIdList;
	}

	public void updateEmployerIds(List<EmployerId> employerIds) throws ValidationException,ServiceFailureException {
		try {
			Employer employer = employerComponent.getEmployer();
			employerComponent.saveEmployer(employer);
			dispatchWorkflowEvent(WorkflowEventType.UPDATE_EMPLOYERID, employer);
		} catch (Exception e) {
				throw new ServiceFailureException(e);
		}
	}
	
	
	
	
	/**
     * Sink for workflow events in this component.  Handles side-effects that are germain to this component, then
     * propagates the event to the Messaging infrastructure.
	 * 
	 * @param workflowEventType
	 * @param employer
	 * @throws ServiceFailureException
	 */
	private void dispatchWorkflowEvent(WorkflowEventType workflowEventType, Employer employer) throws ServiceFailureException {
		sLogger.debug("Event Dispatched : "+workflowEventType);
		//Add/Update/Close tasks we control
		try {
			switch (workflowEventType) {
				case PREFETCH_EMPLOYERID:
					break;
				case UPDATE_EMPLOYERID:

					break;
				default:
					break;
			}
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}
		//Publish event to interested parties 
		msgService.dispatchEmployerEvent(workflowEventType, employer);
	}


}
