package com.intuit.sb.payroll.capability.employer.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.JurisdictionType;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ResourceKeyConstants;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.taxauthority.JurisdictionEAO;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;

/**
 * FIXME [impl] Stub implementation of a real Jurisdiction resolution service.
 * This is not globalized nor resourced on purpose right now.
 * $Author$ $Revision$ $DateTime$
 */
@SuppressWarnings("nls")
public class JurisdictionResolver {
	private Map<String, Jurisdiction> jurisdictionAddress = new HashMap<String, Jurisdiction>();

	private static PayrollResourceManager resMan = PayrollResourceManager.getInstance();

	private static Logger logger = PayrollLoggerFactory.makeLogger(JurisdictionResolver.class);

	/**
	 * Loads our jurisdiction address mapper
	 */
	public JurisdictionResolver() {

		JurisdictionEAO jurisdictionDAO = ServiceLocator.getInstance().getService(JurisdictionEAO.JNDI_NAME);
		List<Jurisdiction> list = jurisdictionDAO.findAll();

		// Populate our map
		for (int i = 0; i < list.size(); i++) {
			Jurisdiction j = list.get(i);
			if (JurisdictionType.UNITED_STATES.getName().equals(j.getName())) {
				jurisdictionAddress.put(JurisdictionType.UNITED_STATES.getName(), j);

			} else if (JurisdictionType.CALIFORNIA.getName().equals(j.getName())) {
				jurisdictionAddress.put(JurisdictionType.CALIFORNIA.getName(), j);
			} else if (JurisdictionType.TEXAS.getName().equals(j.getName())) {
				jurisdictionAddress.put(JurisdictionType.TEXAS.getName(), j);

			} else if (JurisdictionType.SAN_FRANCISCO.getName().equals(j.getName())) {
				jurisdictionAddress.put(JurisdictionType.SAN_FRANCISCO.getName(), j);
			}
		}

		if (logger.isDebugEnabled()) {
			logger.debug("Jurisdiction Address has " + jurisdictionAddress.size() + " elements");
		}
	}

	/**
	 * Returns a list of jurisdictions that cover the address parameter.
	 * 
	 * @param address
	 * @return List The list of jurisdictions that cover the address parameter.
	 * @throws ValidationException
	 */
	public List<Jurisdiction> resolveJurisdictionsForAddress(USPostalAddress address) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(address, "address");

		List<Jurisdiction> jurisdictions = new ArrayList<Jurisdiction>();
		jurisdictions.add(jurisdictionAddress.get(JurisdictionType.UNITED_STATES.getName()));

		if (!("US".equals(address.getCountryISOCode()))) // any address outside US is not supported
		{
			throw new ValidationException(resMan
					.getString(ResourceKeyConstants.UNSUPPORTED_JURISDICTION_EXCEPTION)
					+ address.getCountryISOCode());
		}

		// now check the address falls within any of the supported states - CA or TX
		if ("CA".equals(address.getRegionOrState())) {
			jurisdictions.add(jurisdictionAddress.get(JurisdictionType.CALIFORNIA.getName()));
			if ("San Francisco".equals(address.getCity())) {
				jurisdictions.add(jurisdictionAddress.get(JurisdictionType.SAN_FRANCISCO.getName()));
			}
		} else if ("TX".equals(address.getRegionOrState())) {
			jurisdictions.add(jurisdictionAddress.get(JurisdictionType.TEXAS.getName()));
		} else // we do not support this address since it does not fall into any of these states
		{
			throw new ValidationException(resMan
					.getString(ResourceKeyConstants.UNSUPPORTED_JURISDICTION_EXCEPTION)
					+ address.getRegionOrState());
		}
		return jurisdictions;
	}
}
