package com.intuit.sb.payroll.capability.employer.impl;

import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentLineItem;
import com.intuit.sb.payroll.entity.model.PaymentLineItemSplit;
import com.intuit.sb.payroll.entity.model.PaymentType;

/**
 * Validates fields in the BankAccountValidator entity that have values. Fields with no
 * values are ignored.
 * 
 * This class does not ensure data integrity or completeness.
 * 
 * @see PaymentValidator
 * 
 */
public class PaymentValidator extends AbstractEntityDataValidator<Payment> {
	private static PaymentValidator sInstance = new PaymentValidator();

	private PaymentValidator() {
	}

	public static PaymentValidator getInstance() {
		return sInstance;
	}

	@Override
	public void validateEntityCollection(List<Payment> paymentCollection) throws ValidationException {
		for (Payment payment : paymentCollection) {
			validateEntityImpl(payment);
		}
	}

	@Override
	public void validateEntity(Payment payment) throws ValidationException {
		validateEntityImpl(payment);
	}
	
	/**
	 * @param payment - Payment entity object
	 * @throws ValidationException
	 */
	public void validateEntityImpl(Payment payment) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(payment, "Payment");
		PayrollParamValidator.checkIsNotNull(payment.getPaymentDate(), "Payment Date");
		
		//PayrollParamValidator.checkIsNotNull(payment.getPayrollPeriod(), "Payment Payroll Period");
		Set<PaymentLineItem> paymentLineItems = payment.getPaymentLineItems();
		if (paymentLineItems.size() == 0){
			throw new ValidationException("PaymentLineItem is not set (size 0)");
		}
		for (PaymentLineItem paymentLineItem:paymentLineItems){
			validatePaymentLineItem(paymentLineItem);
		}
	}

	private void validatePaymentLineItem(PaymentLineItem paymentLineItem) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(paymentLineItem, "PaymentLineItem ");
		PayrollParamValidator.checkIsNotNull(paymentLineItem.getAmount(), "PaymentLineItem amount");
		if (paymentLineItem.getPaymentType().equals(PaymentType.NOTSET)){
			throw new ValidationException("Payment type is not set");
		}
		
		Set<PaymentLineItemSplit> payLineItemSplits = paymentLineItem.getPaymentLineItemSplits();
		if (payLineItemSplits.size() == 0){
			throw new ValidationException("PaymentLineItemSplit is not set (size 0)");
		}
		for(PaymentLineItemSplit paylineItemSplit:payLineItemSplits){
			validatePaymentLineItemSplit(paylineItemSplit);
		}
		
	}

	private void validatePaymentLineItemSplit(
			PaymentLineItemSplit paylineItemSplit) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(paylineItemSplit, "PaymentLineItemSplit");
		PayrollParamValidator.checkIsNotNull(paylineItemSplit.getPayrollItem(),"PayrollItem");
		PayrollParamValidator.checkIsNotNull(paylineItemSplit.getSplitAmount(), "paymentLineItenSplit amount");
		
	}
	

}
