package com.intuit.sb.payroll.capability.employer.impl;

import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Validates {@link WorkLocation} entities.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
@SuppressWarnings("nls")
public class WorkLocationValidator extends AbstractEntityDataValidator<WorkLocation>{
	
	private static WorkLocationValidator sInstance = new WorkLocationValidator();
	
	private WorkLocationValidator() {}
	
	public static WorkLocationValidator getInstance() {
		return sInstance;
	}
	
	public void validateEntityCollection(List<WorkLocation> workLocations) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(workLocations, "Work Locations");
		for (WorkLocation workLocation : workLocations) {
			validateEntity(workLocation);
		}
	}

	public void validateEntity(WorkLocation workLocation) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(workLocation, "Work Location");
		PayrollParamValidator.checkIsNotNullOrEmptyString(workLocation.getName(), "Work Location Name");
		PayrollParamValidator.checkIsNotNull(workLocation.getTaxResidenceAddress(), "Work Location Tax Residence");
		USPostalAddress taxAddress = AddressConvertor.getUSFromGlobal(workLocation.getTaxResidenceAddress(), new USPostalAddressImpl());
		PayrollParamValidator.checkIsNotNullOrEmptyString(taxAddress.getRegionOrState(), "State Code");
	}
}
