package com.intuit.sb.payroll.capability.pts.adapter;

import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.jboss.ws.WSException;

import com.intuit.sb.payroll.capability.config.GeminiConfigurationUtility;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sbd.payroll.psp.adapters.gemini.AdapterResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.BankAccountAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyBankAccountAdapter;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyBankAccountAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyBankAccountAdapterService;

/**
 * PSP Request wrapper. This class will serve as wrapper over TPS webService for all
 * Company related services
 * 
 * @author amalik
 *
 */
public class CompanyBankAccountRequestWrapper {
	
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(CompanyBankAccountRequestWrapper.class);
	private static final String COMPANY_BANK_ADAPTER_WSDL = "/gemini-ws/services/CompanyBankAccountAdapter?wsdl";
	
	
	/**
	 * Add a new Company Bank Account with PSP
	 * @param sourceCompanyId
	 * @param companyBankAccountAdapterdto
	 * @return
	 */
	public AdapterResult addCompanyBankAccount(String sourceCompanyId, CompanyBankAccountAdapterDTO companyBankAccountAdapterdto) {
		if (companyBankAccountAdapterdto == null){
			return null; 
		}
		StringBuffer sblog = new StringBuffer();
		sblog.append("Adding Bank Information ").append(companyBankAccountAdapterdto.getSourceCompanyBankAccountId())
		   .append("for Company ").append("sourceCompanyId");
		sLogger.info(sblog.toString());

		logCompanyBankAccount(sourceCompanyId,  companyBankAccountAdapterdto);
		
		CompanyBankAccountAdapter bankAccountAdapter = getCompanyBankAccountAdapter();
		AdapterResult adapterResult = null;
        try {
        	adapterResult = bankAccountAdapter.addCompanyBankAccount(
        			sourceCompanyId, companyBankAccountAdapterdto);
        } catch (WSException e) {
        	StringBuffer sb = new StringBuffer().append("Error adding bank account to PSP ")
        	       .append(companyBankAccountAdapterdto.getSourceCompanyBankAccountId())
		           .append("for Company ").append("sourceCompanyId")
			       .append(e);
        	sLogger.error(sb.toString());
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
        }
        return adapterResult;
	}
	
	/**
	 * Add a new Company Bank Account with PSP
	 * @param sourceCompanyId
	 * @param companyBankAccountAdapterdto
	 * @return
	 */
	public AdapterResult updateCompanyBankAccount(String sourceCompanyId, CompanyBankAccountAdapterDTO companyBankAccountAdapterdto) {
		if (companyBankAccountAdapterdto == null){
			return null; 
		}

		StringBuffer sblog = new StringBuffer();
		sblog.append("Updating Bank Information ").append(companyBankAccountAdapterdto.getSourceCompanyBankAccountId())
		   .append("for Company ").append("sourceCompanyId");
		sLogger.info(sblog.toString());

		logCompanyBankAccount(sourceCompanyId,  companyBankAccountAdapterdto);
		
		CompanyBankAccountAdapter bankAccountAdapter = getCompanyBankAccountAdapter();
		AdapterResult adapterResult = null;
        try {
        	adapterResult = bankAccountAdapter.updateCompanyBankAccount(
        			sourceCompanyId, companyBankAccountAdapterdto);
        } catch (WSException e) {
			sLogger.error("Error adding bank account to PSP - local save successful", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
        }
        return adapterResult;
	}
	
	
	private void logCompanyBankAccount(String sourceCompanyId,
			CompanyBankAccountAdapterDTO companyBankAccountAdapterdto) {
		if (sLogger.isDebugEnabled()){
			StringBuffer sb = new StringBuffer()
			.append("\t Source Company BankAccount Id     | ").append(companyBankAccountAdapterdto.getSourceCompanyBankAccountId()).append("\n");
			
			BankAccountAdapterDTO bankaccountDTO = companyBankAccountAdapterdto.getBankAccount();
			if (bankaccountDTO == null){
			  sb.append("\t BankAccountDTO is <null>"); 
			}else{
			  sb.append("\t BankName                          | ").append(bankaccountDTO.getBankName()).append("\n")
			    .append("\t Bank Account Number               | ").append(bankaccountDTO.getAccountNumber()).append("\n")
		    	.append("\t Bank Account Type                 | ").append(bankaccountDTO.getAccountType()).append("\n")
		    	.append("\t Routing Number                    | ").append(bankaccountDTO.getRoutingNumber()).append("\n");	
			}
	    	sLogger.debug(sb.toString());
		}
	}

	/**
	 * Method to create CompanyBankAccountAdapter based on the TPS host provided in the Config bean
	 * @return
	 */
	private static CompanyBankAccountAdapter getCompanyBankAccountAdapter() {
		String tpsHost = GeminiConfigurationUtility.getTaxPaymentServiceHost();
		CompanyBankAccountAdapterService companyBankAccountService = null;
		
		if ((tpsHost == null) || (tpsHost.length() == 0)){
			// Use the default one used while building this service
			companyBankAccountService = new CompanyBankAccountAdapterService();
		}else{
			String wsdlUrlStr = new StringBuffer(tpsHost).append(COMPANY_BANK_ADAPTER_WSDL).toString();
			URL url = null;
	        try {
	            URL baseUrl = com.intuit.sbd.payroll.psp.adapters.gemini.CompanyAdapterService.class.getResource(".");
	            url = new URL(baseUrl, wsdlUrlStr);
	        } catch (MalformedURLException e) {
	        	sLogger.warn("Failed to create URL for the wsdl Location: "+ wsdlUrlStr);
	        	sLogger.warn(e.getMessage());
	        }
	        QName qname = new QName("http://gemini.adapters.psp.payroll.sbd.intuit.com/", "CompanyBankAccountAdapterService");
	        companyBankAccountService = new CompanyBankAccountAdapterService(url, qname);
		}
		return companyBankAccountService.getCompanyBankAccountAdapterPort();
	}
}
