package com.intuit.sb.payroll.capability.pts.adapter;


import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.jboss.ws.WSException;

import com.intuit.sb.payroll.capability.config.GeminiConfigurationUtility;
import com.intuit.sb.payroll.capability.pts.adapter.util.PTSLoggerUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sbd.payroll.psp.adapters.gemini.AdapterResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.DateAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.EmployeeAdapter;
import com.intuit.sbd.payroll.psp.adapters.gemini.EmployeeAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.EmployeeAdapterService;

public class EmployeeAdapterWrapper {
	
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(EmployeeAdapterWrapper.class);
	private static final String EMPLOYEE_ADAPTER_WSDL = "/gemini-ws/services/EmployeeAdapter?wsdl";

	public AdapterResult addEmployee(EmployeeAdapterDTO employeeDTO){
		sLogger.info("Adding Employee to PSP....");
		
		EmployeeAdapter employeeAdapter = getEmployeeAdapter();
        AdapterResult adapterResult = null;
        try {
        	logEmployeeAdapterDTO(employeeDTO);
        	List<EmployeeAdapterDTO> employeeDTOs = makeToList(employeeDTO);
        	adapterResult = employeeAdapter.addEmployees(employeeDTOs);
        } catch (WSException e) {
			sLogger.error("Error saving to PSP - local save successful", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
        }
        return adapterResult;
	}
	
	
	public AdapterResult updateEmployee(EmployeeAdapterDTO employeeDTO) {
		sLogger.info("Updating Employee to PSP....");
		
		EmployeeAdapter employeeAdapter = getEmployeeAdapter();
        AdapterResult adapterResult = null;
        try {
        	logEmployeeAdapterDTO(employeeDTO);
        	adapterResult = employeeAdapter.updateEmployee(employeeDTO);
        } catch (WSException e) {
			sLogger.error("Error updating to PSP ", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
        }
        return adapterResult;
	}
	
	public AdapterResult deleteEmployee(String sourceCompanyId, String sourceEmployeeId) {
		sLogger.info("Deleting Employee from PSP.... " + sourceEmployeeId);
		
		EmployeeAdapter employeeAdapter = getEmployeeAdapter();
        AdapterResult adapterResult = null;
        try {
        	adapterResult = employeeAdapter.deleteEmployee(sourceCompanyId,sourceEmployeeId);
        } catch (WSException e) {
			sLogger.error("Error deleting from PSP ", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
        }
        return adapterResult;
	}

	public AdapterResult rehireEmployee(String sourceCompanyId, String sourceEmployeeId, DateAdapterDTO rehireDate) {
		sLogger.info("Adding rehired Employee to PSP....");
		
		EmployeeAdapter employeeAdapter = getEmployeeAdapter();
        AdapterResult adapterResult = null;
        try {
        	logEmployeeInfo(sourceCompanyId,  sourceEmployeeId, rehireDate);
        	adapterResult = employeeAdapter.reactivateEmployee(sourceCompanyId,sourceEmployeeId, rehireDate);
        } catch (WSException e) {
			sLogger.error("Error adding to PSP ", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
        }
        return adapterResult;
	}
		
	private void logEmployeeInfo(String sourceCompanyId,
			String sourceEmployeeId, DateAdapterDTO rehireDate) {
		if (sLogger.isDebugEnabled()){
			StringBuffer empStrBuffer = new StringBuffer();
			empStrBuffer.append("\n\t SourceCompanyId     | ").append(sourceCompanyId).append("\n")
			    .append("\t sourceEmployeeId     | ").append(sourceEmployeeId).append("\n")
			    .append(PTSLoggerUtil.logDateInfo(rehireDate, "\t Date                 | ")).append("\n");
			
			sLogger.debug(empStrBuffer.toString());
		}
		
	}


	public AdapterResult terminateEmployee(String sourceCompanyId, String sourceEmployeeId, DateAdapterDTO terminationDate) {
		sLogger.info("Terminating Employee "+ sourceEmployeeId +" fo company "+ sourceCompanyId);
		
		EmployeeAdapter employeeAdapter = getEmployeeAdapter();
        AdapterResult adapterResult = null;
        try {
        	logEmployeeInfo(sourceCompanyId,  sourceEmployeeId, terminationDate);
        	adapterResult = employeeAdapter.deactivateEmployee(sourceCompanyId,sourceEmployeeId, terminationDate);
        } catch (WSException e) {
			sLogger.error("Error updating to PSP ", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
        }
        return adapterResult;
	}
	
	private List<EmployeeAdapterDTO> makeToList(EmployeeAdapterDTO employeeDTO) {
		List<EmployeeAdapterDTO> employeeDTOs = new ArrayList<EmployeeAdapterDTO>(1);
		employeeDTOs.add(employeeDTO);
		return employeeDTOs;
	}

	private void logEmployeeAdapterDTO(EmployeeAdapterDTO employeeDTO) {
		if (sLogger.isDebugEnabled()){
			if(employeeDTO == null){
				sLogger.debug("\t EmployeeAdapterDTO is <null>");
				return;
			}		
			
			StringBuffer logStrBuffer = new StringBuffer();
			logStrBuffer
		  .append("\n\t SourceCompanyId             | "+ employeeDTO.getSourceCompanyId()).append("\n")
			.append("\t SourceEmployeeId            | "+ employeeDTO.getSourceEmployeeId()).append("\n")
			.append("\t FirstName                   | "+ employeeDTO.getFirstName()).append("\n")
			.append("\t LastName                    | "+ employeeDTO.getLastName()).append("\n")
			.append("\t MiddleName                  | "+ employeeDTO.getMiddleName()).append("\n")
			.append("\t SocialSecurityNumber        | "+ employeeDTO.getSocialSecurityNumber()).append("\n")
			.append("\t Suffix                      | "+ employeeDTO.getSuffix()).append("\n").append("\n")
			.append("\t LiveAddress").append("\n")
			.append(PTSLoggerUtil.logAddressAdapterDTO(employeeDTO.getLiveAddress(), "LiveAddress"))
			.append("\t WorkState                   | "+ employeeDTO.getWorkState()).append("\n")
			.append(PTSLoggerUtil.logDateInfo(employeeDTO.getHireDate(), "\t HireDate                   | ")).append("\n")
			.append("\t FederalFilingStatus	        | "+ employeeDTO.getFedFilingStatus()).append("\n")
			.append("\t FederalAllowances           | "+ employeeDTO.getFedAllowances()).append("\n")
			.append("\t StateFilingStatus           | "+ employeeDTO.getStateFilingStatus()).append("\n")
			.append("\t StateAllowances             | "+ employeeDTO.getStateAllowances()).append("\n")
			.append("\t HasRetirementPlan           | "+ employeeDTO.isHasRetirementPlan()).append("\n")
			.append("\t HasThirdPartySickDay        | "+ employeeDTO.isHasThirdPartySickPay()).append("\n")
			.append("\t IsStatutory                 | "+ employeeDTO.isIsStatutory()).append("\n");
			
			sLogger.debug(logStrBuffer.toString());
		}
	}

	private static EmployeeAdapter getEmployeeAdapter(){
		String tpsHost = GeminiConfigurationUtility.getTaxPaymentServiceHost();
		EmployeeAdapterService employeeAdapterService = null;
		
		if ((tpsHost == null) || (tpsHost.length() == 0)){
			// Use the default one used while building this service
			employeeAdapterService = new EmployeeAdapterService();
		}else{
			String wsdlUrlStr = new StringBuffer(tpsHost).append(EMPLOYEE_ADAPTER_WSDL).toString();
			URL url = null;
	        try {
	            URL baseUrl = com.intuit.sbd.payroll.psp.adapters.gemini.EmployeeAdapterService.class.getResource(".");
	            url = new URL(baseUrl, wsdlUrlStr);
	        } catch (MalformedURLException e) {
	        	sLogger.warn("Failed to create URL for the wsdl Location: "+ wsdlUrlStr);
	        	sLogger.warn(e.getMessage());
	        }
	        QName qname = new QName("http://gemini.adapters.psp.payroll.sbd.intuit.com/", "EmployeeAdapterService");
	        employeeAdapterService = new EmployeeAdapterService(url, qname);
		}
		EmployeeAdapter employeeAdapter = employeeAdapterService.getEmployeeAdapterPort();
		return employeeAdapter;
	}

	

}
