package com.intuit.sb.payroll.capability.pts.adapter;




import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.jboss.ws.WSException;

import com.intuit.sb.payroll.capability.config.GeminiConfigurationUtility;
import com.intuit.sb.payroll.capability.pts.adapter.util.PTSLoggerUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sbd.payroll.psp.adapters.gemini.AcceptTaxServiceAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.AdapterMessage;
import com.intuit.sbd.payroll.psp.adapters.gemini.AdapterResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.CancelledPaycheckDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyBankInfoAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompensationTransactionAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.DeductionTransactionAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.LiabilityBalancesResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.LiabilityTransactionAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PaycheckAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PaycheckSplitAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollAdapter;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollAdapterService;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollRunAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollTaxAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollTaxCreditAmountDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.SubmitPayrollResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.TaxPaymentAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.TaxPaymentsAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.VoidPayrollResult;

public class PayrollRequestWrapper {

	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(PayrollRequestWrapper.class);
	private static final String PAYROLL_RUN_ADAPTER_WSDL = "/gemini-ws/services/PayrollAdapter?wsdl";

	public SubmitPayrollResult submitPayrollRun(PayrollRunAdapterDTO payrollRunDTO){
		sLogger.debug("Submitting Payroll Run to PSP "+ payrollRunDTO.getSourcePayrollRunId());
		
		PayrollAdapter payrollAdapter = getPayrollAdapter();
        SubmitPayrollResult adapterResult = null;
        try {
        	logPayrollAdapterDTO(payrollRunDTO);
        	adapterResult = payrollAdapter.submitPayroll(payrollRunDTO);
        	sLogger.debug(logSubmitPayrollAdapter(adapterResult).toString());;
        } catch (WSException e) {
			sLogger.error("Error saving to PSP - local save successful", e);
			adapterResult = new SubmitPayrollResult();
			adapterResult.setSuccess(false);
        }
        return adapterResult;
	}
	

	public AdapterResult acceptTaxService(AcceptTaxServiceAdapterDTO acceptTaxService){
		sLogger.debug("Sending Activation ack for first day of service ");
		AdapterResult adapterResult;
		
		try {
			PayrollAdapter payrollAdapter = getPayrollAdapter();
			adapterResult = payrollAdapter.acceptTaxService(acceptTaxService);
		} catch (WSException e) {
			sLogger.error("Error Accpeting firstDay of service", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
		}
		return adapterResult;
	}
	
	public AdapterResult submitHistoricalTaxPayments(TaxPaymentsAdapterDTO taxPaymentsAdapterDTO){
		sLogger.debug("Sending Historical Tax Payments "+taxPaymentsAdapterDTO.getSourceCompanyId());
		AdapterResult adapterResult;
		logTaxPaymentsAdapterDTO(taxPaymentsAdapterDTO);
		try {
			PayrollAdapter payrollAdapter = getPayrollAdapter();
			adapterResult = payrollAdapter.submitHistoricalTaxPayments(taxPaymentsAdapterDTO);
		} catch (WSException e) {
			sLogger.error("Error Sending Historical Tax Payments", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
		}
		return adapterResult;
	}
	
	
	public VoidPayrollResult voidPaychecks(String sourceCompanyId, String sourcePayrollRunId,
	        								List<String> sourcePaycheckIds, 
	        								List<PayrollTaxAdapterDTO> payrollTaxes){
		logVoidPaychecksRequestDetails(sourceCompanyId, sourcePayrollRunId, sourcePaycheckIds, payrollTaxes);
		VoidPayrollResult adapterResult;
		try {
			PayrollAdapter payrollAdapter = getPayrollAdapter();
			adapterResult = payrollAdapter.voidPaychecks(sourceCompanyId, sourcePayrollRunId, sourcePaycheckIds, payrollTaxes);
			sLogger.debug(logVoidPayrollResult(adapterResult).toString());
		} catch (WSException e) {
			sLogger.error("Error Voiding Paycheck", e);
			adapterResult = new VoidPayrollResult();
			adapterResult.setSuccess(false);
		}
		return adapterResult;
	}
	
	public LiabilityBalancesResult getHistoricalLiabilityBalances(String sourceCompanyId) {
		LiabilityBalancesResult adapterResult;
		try {
			PayrollAdapter payrollAdapter = getPayrollAdapter();
			adapterResult = payrollAdapter.getHistoricalLiabilityBalances(sourceCompanyId);
		} catch (WSException e) {
			sLogger.error("Error Getting Historical Balances", e);
			adapterResult = new LiabilityBalancesResult();
			adapterResult.setSuccess(false);
		}
		return adapterResult;
	}
	private void logTaxPaymentsAdapterDTO(TaxPaymentsAdapterDTO taxPaymentsAdapterDTO){
		if (sLogger.isDebugEnabled()){
			if (taxPaymentsAdapterDTO == null){
				sLogger.debug("taxPaymentsAdapterDTO is null");
				return;
			}
			
			StringBuffer logStrBuffer = new StringBuffer();
			logStrBuffer		
			.append("\t SourceCompanyId             | " + taxPaymentsAdapterDTO.getSourceCompanyId()).append("\n");
			
			List<TaxPaymentAdapterDTO> taxPayments = taxPaymentsAdapterDTO.getPayments();
			for(TaxPaymentAdapterDTO taxPaymentDTO: taxPayments){
				logStrBuffer
				.append("\t TaxPaymentAdapterDTO             | ").append(taxPaymentDTO.getLawId()).append(" ")
																 .append(PTSLoggerUtil.logDateInfo(taxPaymentDTO.getDate(), " Date "))
																 .append(taxPaymentDTO.getLawId()).append("\n");	
			}
			sLogger.debug(logStrBuffer.toString());
		}
	}


	private void logVoidPaychecksRequestDetails(String sourceCompanyId,
			String sourcePayrollRunId, List<String> sourcePaycheckIds,
			List<PayrollTaxAdapterDTO> payrollTaxes) {
		sLogger.debug("Sending Voided Paychecks " + sourceCompanyId);
		sLogger.debug("Payroll run Id for voiding " + sourcePayrollRunId);
		sLogger.debug("List of Paychecks");
		int i = 0;
		for (String paycheckId : sourcePaycheckIds) {
			sLogger.debug("PaycheckId #" + i + " = " + paycheckId);
		}
	}


	private void logPayrollAdapterDTO(PayrollRunAdapterDTO payrollRunDTO) {
		if (sLogger.isDebugEnabled()){
			if(payrollRunDTO == null){
				sLogger.debug("\t PayrollRunAdapterDTO is <null>");
				return;
			}		
		
		StringBuffer logStrBuffer = new StringBuffer();
		logStrBuffer		
		.append("\t SourceCompanyId             | " + payrollRunDTO.getSourceCompanyId()).append("\n")
		.append("\t SourcePayrollRunId          | " + payrollRunDTO.getSourcePayrollRunId()).append("\n")
		.append(logCompanyBankInfoAdapterDTO(payrollRunDTO.getCompanyBankInfo())).append("\n")
		.append("\t PayrollRunType              | " + payrollRunDTO.getPayrollRunType()).append("\n")
		.append(PTSLoggerUtil.logDateInfo(payrollRunDTO.getPaycheckDate(), "\t PayCheckDate                | ")).append("\n")
		.append(logPayChecksInfo(payrollRunDTO.getPaychecks())).append("\n")
		.append(logPayrollTaxes(payrollRunDTO.getPayrollTaxes()));
		sLogger.debug(logStrBuffer.toString());
		}
	}

	private StringBuffer logCompanyBankInfoAdapterDTO(List<CompanyBankInfoAdapterDTO> companyBankInfo){
		StringBuffer companyBankInfoStrBuffer = new StringBuffer();
		companyBankInfoStrBuffer.append("\t  CompanyBankInfoAdapterDTO ").append("\n");
		
		if( companyBankInfo == null || companyBankInfo.size() == 0){
			companyBankInfoStrBuffer.append(" is empty");	
			return companyBankInfoStrBuffer;
		}
		int i = 1;
		for (CompanyBankInfoAdapterDTO companyBankInfoAdapterDTO : companyBankInfo) {
			companyBankInfoStrBuffer
			.append("\t\t SourceCompanyBankAccountId  | ").append(i).append(" ")
						.append(companyBankInfoAdapterDTO.getSourceCompanyBankAccountId()).append("\n")
			.append("\t\t ServiceCode                 | ").append(i++).append(" ")
			.append(companyBankInfoAdapterDTO.getServiceCode());
		}
		return companyBankInfoStrBuffer;
	}
	
	private StringBuffer logPayChecksInfo(List<PaycheckAdapterDTO> paychecks){
		StringBuffer payCheckInfoStrBuffer = new StringBuffer();
		payCheckInfoStrBuffer.append("\t PaycheckAdapterDTO is ").append("\n");
		if(paychecks == null || paychecks.size() == 0){
			payCheckInfoStrBuffer.append(" empty");
			return payCheckInfoStrBuffer;
		}
		int i = 1;
		for (PaycheckAdapterDTO paycheckAdapterDTO : paychecks) {
			payCheckInfoStrBuffer
			.append("\t SourceEmployeeId            | ").append(i).append(" ").append(paycheckAdapterDTO.getSourceEmployeeId()).append("\n")
			.append("\t SourcePaycheckId            | ").append(i).append(" ").append(paycheckAdapterDTO.getSourcePaycheckId()).append("\n")
			.append("\t PayCheckNetAmount           | ").append(i).append(" ").append(paycheckAdapterDTO.getPaycheckNetAmount()).append("\n")
			.append(PTSLoggerUtil.logDateInfo(paycheckAdapterDTO.getPayPeriodBeginDate(), "\t PayPeriodBeginDate          | ")).append("\n")
			.append(PTSLoggerUtil.logDateInfo(paycheckAdapterDTO.getPayPeriodEndDate(), "\t PayPeriodEndDate            | ")).append("\n")
			.append(logLiabilityTransactions(paycheckAdapterDTO.getLiabilityTransactions())).append("\n")
			.append(logDeductionTransactions(paycheckAdapterDTO.getDeductionTransactions())).append("\n")
			.append(logCompensationTransactions(paycheckAdapterDTO.getCompensationTransactions())).append("\n")
			.append(logPayCheckSplits(paycheckAdapterDTO.getPaycheckSplits())).append("\n\n");
			i = i + 1;
		}
		return payCheckInfoStrBuffer;
	}
	
    private StringBuffer logLiabilityTransactions(List<LiabilityTransactionAdapterDTO> liabilityTransactions){
		StringBuffer liabilityTransStrBuffer = new StringBuffer();
		liabilityTransStrBuffer.append("\t LiabilityTransactionAdapterDTO is ").append("\n");
		if(liabilityTransactions == null || liabilityTransactions.size() == 0){
			liabilityTransStrBuffer.append(" empty");
			return liabilityTransStrBuffer;
		}
		int i = 1;
		for (LiabilityTransactionAdapterDTO liabilityTransactionAdapterDTO : liabilityTransactions) {
			liabilityTransStrBuffer
			.append("\t LiabilityLawID              | ").append(i).append(" ").append(liabilityTransactionAdapterDTO.getLawId()).append("\n")
			.append("\t LiabilityAmount             | ").append(i).append(" ").append(liabilityTransactionAdapterDTO.getLiabilityAmount()).append("\n")
			.append("\t LiabilityTaxableWagesAmount | ").append(i).append(" ").append(liabilityTransactionAdapterDTO.getLiabilityTaxableWagesAmount()).append("\n")
			.append("\t LiabilityTipsAmount         | ").append(i).append(" ").append(liabilityTransactionAdapterDTO.getLiabilityTipsAmount()).append("\n")
			.append("\t LiabilityTotalWagesAmount   | ").append(i++).append(" ").append(liabilityTransactionAdapterDTO.getLiabilityTotalWagesAmount()).append("\n");
		}
		return liabilityTransStrBuffer;
	}
	
	private StringBuffer logDeductionTransactions(List<DeductionTransactionAdapterDTO> deductionTransactions){
		StringBuffer deductionTransStrBuffer = new StringBuffer();
		if(deductionTransactions == null || deductionTransactions.size() == 0){
			sLogger.debug("\t DeductionTransactionAdapterDTO is empty");
			return deductionTransStrBuffer;
		}
		int i = 1;
		for (DeductionTransactionAdapterDTO deductionTransactionAdapterDTO : deductionTransactions) {
			deductionTransStrBuffer
			.append("\t SourceDeductionId           | ").append(i).append(" ").append(deductionTransactionAdapterDTO.getSourceDeductionId()).append("\n")
			.append("\t DeductionAmount             | ").append(i++).append(" ").append(deductionTransactionAdapterDTO.getDeductionAmount()).append("\n");
		}
		return deductionTransStrBuffer;
	}
	
	private StringBuffer logCompensationTransactions(List<CompensationTransactionAdapterDTO> compensationTransactions){
		StringBuffer compensationTransStrBuffer = new StringBuffer();
		if(compensationTransactions == null || compensationTransactions.size() == 0){
			sLogger.debug("\t CompensationTransactionAdapterDTO is empty");
			return compensationTransStrBuffer;
		}
		int i = 1;
		for (CompensationTransactionAdapterDTO compensationTransactionAdapterDTO : compensationTransactions) {
			compensationTransStrBuffer
			.append("\t SourceCompensationId        | ").append(i).append(" ").append(compensationTransactionAdapterDTO.getSourceCompensationId()).append("\n")
			.append("\t CompensationHoursWorked     | ").append(i).append(" ").append(compensationTransactionAdapterDTO.getCompensationHoursWorked()).append("\n")
			.append("\t CompensationAmount          | ").append(i++).append(" ").append(compensationTransactionAdapterDTO.getCompensationAmount()).append("\n");
		}
		return compensationTransStrBuffer;
	}
	
	private StringBuffer logPayCheckSplits(List<PaycheckSplitAdapterDTO> payCheckSplits){
		StringBuffer payCheckSplitStrBuffer = new StringBuffer();
		if(payCheckSplits == null || payCheckSplits.size() == 0){
			sLogger.debug("\t PaycheckSplitAdapterDTO is empty");
			return payCheckSplitStrBuffer;
		}
		for (PaycheckSplitAdapterDTO paycheckSplitAdapterDTO : payCheckSplits) {
			payCheckSplitStrBuffer
			.append("\t SourcePayCheckSplitId       | " + paycheckSplitAdapterDTO.getSourcePaycheckSplitId()).append("\n")
			.append("\t SourceEmployeeBankAccountId | " + paycheckSplitAdapterDTO.getSourceEmployeeBankAccountId()).append("\n")
			.append("\t SourcePayCheckSplitAmount   | " + paycheckSplitAdapterDTO.getPaycheckSplitAmount()).append("\n");
		}
		return payCheckSplitStrBuffer;
	}

	private StringBuffer logPayrollTaxes(List<PayrollTaxAdapterDTO> payrollTaxes){
		StringBuffer payrollTaxesStrBuffer = new StringBuffer();
		if(payrollTaxes == null || payrollTaxes.size() == 0){
			sLogger.debug("\t PayrollTaxAdapterDTO is empty");
			return payrollTaxesStrBuffer;
		}
		for (PayrollTaxAdapterDTO payrollTaxAdapterDTO : payrollTaxes) {
			payrollTaxesStrBuffer
			.append("\t PayrollTaxLawId             | " + payrollTaxAdapterDTO.getLawId()).append("\n")
			.append("\t TaxAmount                   | " + payrollTaxAdapterDTO.getAmount()).append("\n")
			.append(PTSLoggerUtil.logDateInfo(payrollTaxAdapterDTO.getEffectiveDate(), "\t TaxEffectiveDate            | ")).append("\n");
		}
		return payrollTaxesStrBuffer;
	}

	private StringBuffer logSubmitPayrollAdapter(SubmitPayrollResult adapterResult) {
		StringBuffer buffer = new StringBuffer();
		if(adapterResult == null) {
			return buffer;
		}
		buffer.append("\nEmployer DD Debit Amount is " + adapterResult.getEmployerDdDebitAmount())
			.append("\nEmployer Debit Amount is " + adapterResult.getEmployerDebitAmount())
			.append("\nEmployer Tax Debit Amount is " + adapterResult.getEmployerTaxDebitAmount())
			.append("\nEmployer Settlement Date is " + adapterResult.getSettlementDate());
			
		List<PayrollTaxCreditAmountDTO> taxCredits = adapterResult.getEmployerTaxCreditsByPayroll();
		
		if(taxCredits == null || taxCredits.size() == 0){
			buffer.append("\nThere are no Tax Credits \n");
		}
		else {
			buffer.append("\nList of Tax Credits \n");
			for (PayrollTaxCreditAmountDTO taxCredit : taxCredits) {
				buffer.append("\n\t taxCReditPayroll Run ID =  " + taxCredit.getSourcePayrollRunId())
				.append(" TaxCreditAmount = " + taxCredit.getTaxCreditAmount());
			}
		}
		appendAdapterMessages(adapterResult, buffer);
		return buffer;
	}

	
	
	private StringBuffer logVoidPayrollResult(VoidPayrollResult adapterResult) {
		StringBuffer buffer = new StringBuffer();
		if(adapterResult == null) {
			return buffer;
		}
		
		buffer.append("\n AppliedForwardRefundAmount " + adapterResult.getAppliedForwardRefundAmount())
			.append("\n CancelledAmount " + adapterResult.getCancelledAmount())
			.append("\n EmployerDdDebitAmount " + adapterResult.getEmployerDdDebitAmount())
			.append("\n EmployerDebitAmount " + adapterResult.getEmployerDebitAmount())
			.append("\n EmployerTaxDebitAmount " + adapterResult.getEmployerTaxDebitAmount())
			.append("\n ImmediateRefundAmount " + adapterResult.getImmediateRefundAmount())
			.append("\n SettlementDate " + adapterResult.getSettlementDate())
			.append("\n TakeOnReturnRefundAmount" + adapterResult.getTakeOnReturnRefundAmount());
		
		List<CancelledPaycheckDTO> paychecks = adapterResult.getCancelledPaychecks();
		if(paychecks == null || paychecks.size() == 0) {
			buffer.append("There are no Cancelled Paychecks ");
		}
		
		else {
			buffer.append("List of Paychecks \n");
			for (CancelledPaycheckDTO cancelledPaycheckDTO : paychecks) {
				buffer.append("\n\t PaycheckId = " + cancelledPaycheckDTO.getSourcePaycheckId())
					.append(" Amount = " + cancelledPaycheckDTO.getAmount());
			}
		}
		
		appendAdapterMessages(adapterResult, buffer);
		
		return buffer;
	}

	private void appendAdapterMessages(AdapterResult adapterResult, StringBuffer buffer) {
		if(buffer == null) {
			return;
		}
		
		if(adapterResult == null) {
			buffer.append("Adapter result is null");
			return;
		}
		
		List<AdapterMessage> messages = adapterResult.getMessages();
		
		if(messages == null || messages.size() == 0) {
			buffer.append("There are no messages");
		}
		else {
			buffer.append("\nList of messages");
			for (AdapterMessage message : messages) {
				buffer.append("\nMessage Code: " + message.getCode());
				buffer.append("\nMessage Description: " + message.getDesc());
				buffer.append("\nMessage Level: " + message.getLevel());
			}
		}
	}

	private static PayrollAdapter getPayrollAdapter(){
		String tpsHost = GeminiConfigurationUtility.getTaxPaymentServiceHost();
		PayrollAdapterService payrollAdapterService = null;
		
		if ((tpsHost == null) || (tpsHost.length() == 0)){
			// Use the default one used while building this service
			payrollAdapterService = new PayrollAdapterService();
		}else{
			String wsdlUrlStr = new StringBuffer(tpsHost).append(PAYROLL_RUN_ADAPTER_WSDL).toString();
			URL url = null;
	        try {
	            URL baseUrl = com.intuit.sbd.payroll.psp.adapters.gemini.PayrollAdapterService.class.getResource(".");
	            url = new URL(baseUrl, wsdlUrlStr);
	        } catch (MalformedURLException e) {
	        	sLogger.warn("Failed to create URL for the wsdl Location: "+ wsdlUrlStr);
	        	sLogger.warn(e.getMessage());
	        }
	        QName qname = new QName("http://gemini.adapters.psp.payroll.sbd.intuit.com/", "PayrollAdapterService");
	        payrollAdapterService = new PayrollAdapterService(url, qname);
		}
		PayrollAdapter payrollAdapter = payrollAdapterService.getPayrollAdapterPort();
		return payrollAdapter;
	}
}
