package com.intuit.sb.payroll.capability.pts.adapter.mapper;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sbd.payroll.psp.adapters.gemini.BankAccountAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.BankAccountTypeAdapterEnum;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyBankAccountAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyBankInfoAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.ServiceCodeAdapterEnum;

public class BankAccountMapper {
	
	public static CompanyBankInfoAdapterDTO mapToCompanyBankInfoAdapterDTO(BankAccount bankAccount) {
		CompanyBankInfoAdapterDTO companyBankInfoAdapterDTO = new CompanyBankInfoAdapterDTO();
		companyBankInfoAdapterDTO.setServiceCode(ServiceCodeAdapterEnum.TAX);
		companyBankInfoAdapterDTO.setSourceCompanyBankAccountId(Long.toString(bankAccount.getId()));
		return companyBankInfoAdapterDTO;
	}

	/**
	 * Map the Gemini BankAccount to the PSP DTO BankAccount
	 * @param employer - Company object
	 * @param bankAccount - BankAccount model object
	 * @return
	 */
	public static CompanyBankAccountAdapterDTO maptoBankAccountDTO(
			Employer employer, BankAccount bankAccount) 
	throws ServiceFailureException {
	
		BankAccountAdapterDTO bankAccountDTO = new BankAccountAdapterDTO();
		bankAccountDTO.setAccountNumber(bankAccount.getAccountNo());
		bankAccountDTO.setBankName(bankAccount.getBankName());
		
		String accountType = bankAccount.getAccountType().toString();
		// In Gemini, account type is provided as "CHECKING" or "SAVINGS" whereas
		// BankAccountTypeAdapterEnum expects the string as "Checking" or "Savings"
		// change the string accordingly.
		accountType = accountType.toLowerCase();	//	CHECKING/SAVINGS becomes checking/savings
		accountType = accountType.substring(0, 1).toUpperCase() + accountType.substring(1);
		bankAccountDTO.setAccountType(BankAccountTypeAdapterEnum.fromValue(accountType));
		bankAccountDTO.setRoutingNumber(bankAccount.getRoutingNo());
		
		CompanyBankAccountAdapterDTO companyBankAccountAdapterdto = new CompanyBankAccountAdapterDTO();
		companyBankAccountAdapterdto.setBankAccount(bankAccountDTO);
		companyBankAccountAdapterdto.setSourceCompanyBankAccountId(Long.toString(bankAccount.getId()));
		companyBankAccountAdapterdto.setSourceCompanyId(CompanyMapper.namespaceEmployerId(employer));
	
		return companyBankAccountAdapterdto;
	}
}
