package com.intuit.sb.payroll.capability.pts.adapter.mapper;

import java.util.Set;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSContactPersonTypeEnum;
import com.intuit.sbd.payroll.psp.adapters.gemini.ContactAdapterDTO;

// FIXME How do we determine what is PrimaryContact, SecondaryContact and PayrollAdmin

public class ContactsMapper {
	
	public static ContactAdapterDTO getPrimaryContact(Set<PTSContactPerson> ptsContactPersons)
	{
		EmployerContactPerson erContactPerson = getEmployerContactPerson(ptsContactPersons, PTSContactPersonTypeEnum.PRIMARY_CONTACT);
		ContactAdapterDTO contact = mapFromEmployerContactPerson(erContactPerson);
		return contact;
	}

	

	public static ContactAdapterDTO getSecondayContact(Set<PTSContactPerson> ptsContactPersons)
	{
		EmployerContactPerson erContactPerson = getEmployerContactPerson(ptsContactPersons, PTSContactPersonTypeEnum.SECONDARY_CONTACT);
		ContactAdapterDTO contact = mapFromEmployerContactPerson(erContactPerson);
		return contact;
	}
	
	public static ContactAdapterDTO getPayrollAdminContact(Set<PTSContactPerson> ptsContactPersons)
	{
		EmployerContactPerson erContactPerson =  getEmployerContactPerson(ptsContactPersons, PTSContactPersonTypeEnum.PAYROLLADMIN_CONTACT);
		/*if (erContactPerson == null) {
			// if payroll admin is not specified explicitly in PPS, 
			// send the primary contact as the payroll admin
			erContactPerson = getEmployerContactPerson(ptsContactPersons, PTSContactPersonTypeEnum.PRIMARY_CONTACT);
		} */
		ContactAdapterDTO contact = mapFromEmployerContactPerson(erContactPerson);
		return contact;
	}
	

	private static EmployerContactPerson getEmployerContactPerson(Set<PTSContactPerson> ptsContactPersons, PTSContactPersonTypeEnum type){
		for (PTSContactPerson employerContactPerson : ptsContactPersons) {
			if (employerContactPerson.getContactPersonType().equals(type)){
				return employerContactPerson.getEmployerContact();
			}
		}
		return null;
	}
	
	private static ContactAdapterDTO mapFromEmployerContactPerson(
			EmployerContactPerson erContactPerson) {
		if (erContactPerson == null){
			return null;
		}
		ContactAdapterDTO contact = new ContactAdapterDTO();

		// TODO use mobile phone incase home or work is null
		// erContactPerson.getMobilePhone()
		
		// TODO find out what does it mean 
		// contact.setAccountSignatory(value);
		
		contact.setSourceContactId(String.valueOf(erContactPerson.getId()));
		
		EmailAddress emailAdress = erContactPerson.getEmailAddress();
		contact.setEmail((emailAdress == null)? "":getValue(emailAdress.getAddress()));
		
		TelephoneNumber faxPhone = erContactPerson.getFaxPhone();
		contact.setFaxNumber((faxPhone == null)?"":getValue(faxPhone.getFreeFormNumber()));
		
		TelephoneNumber homePhone = erContactPerson.getHomePhone();
		contact.setSecondPhoneNumber((homePhone == null)?"":getValue(homePhone.getFreeFormNumber()));
		
		TelephoneNumber workPhone = erContactPerson.getWorkPhone();
		contact.setPhoneNumber((workPhone == null)?"":getValue(workPhone.getFreeFormNumber()));
		
		GlobalPostalAddress mailingAddress = erContactPerson.getMailingAddress();
		contact.setAddress(AddressMapper.mapAddress(mailingAddress));
		
		
		PersonName person = erContactPerson.getPersonName();
		contact.setFirstName((person == null)?"":getValue(person.getFirstName()));
		contact.setLastName((person == null)?"":getValue(person.getLastName()));
		contact.setMiddleName((person == null)?"":getValue(person.getMiddleName()));
		contact.setTitle((person == null)?"":getValue(person.getTitle()));
		contact.setTitleSuffix((person == null)?"":getValue(person.getSuffix()));
		
		contact.setJobTitle(getValue(erContactPerson.getRoleTitle()));
		
		return contact;
	}
	
	public static String getValue(String value){
		if (value == null){
			return "";
		}else{
			return value;
		}
	}
	
}
