/**
 * 
 */
package com.intuit.sb.payroll.capability.pts.adapter.mapper;

import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.taxes.withholding.TaxWithholdingsComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.CAWagePlan;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.PersonTaxId;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;
import com.intuit.sbd.payroll.psp.adapters.gemini.EmployeeAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.FilingStatusAdapterEnum;

/**
 * @author dkulkarni
 *
 */
public class EmployeeMapper {
	
	public static EmployeeAdapterDTO mapToEmployeeAdapterDTO(Worker worker)
		throws ServiceFailureException, ValidationException
	{
		if(worker == null){
			return null;
		}
		
 		EmployerPTSWorkflowComponent ptsEmployerComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
 		TaxWithholdingsComponent taxWithHoldingsComponent = ServiceLocator.getInstance().getService(TaxWithholdingsComponent.JNDI_NAME);
		
		EmployeeAdapterDTO employeeAdapterDTO = new EmployeeAdapterDTO();
		employeeAdapterDTO.setSourceCompanyId(CompanyMapper.namespaceEmployerId(ptsEmployerComponent.getPTSEmployer().getEmployer()));
		employeeAdapterDTO.setSourceEmployeeId(String.valueOf(worker.getId()));
		
		if (worker.getPersonName() != null){
			employeeAdapterDTO.setFirstName(worker.getPersonName().getFirstName());
			employeeAdapterDTO.setLastName(worker.getPersonName().getLastName());
			employeeAdapterDTO.setMiddleName(worker.getPersonName().getMiddleName());
			employeeAdapterDTO.setSuffix(worker.getPersonName().getSuffix());
		}
		
		PersonTaxId personTax = worker.getPersonTaxId();
		if (personTax != null){
		   employeeAdapterDTO.setSocialSecurityNumber(getValidSSN(personTax.getValue()));
		}
		
		employeeAdapterDTO.setLiveAddress(AddressMapper.mapAddress(worker.getTaxResidenceAddress()));
		
		WorkLocation worklocation = worker.getWorkLocation();
		if ( worklocation != null){
			GlobalPostalAddress address = worklocation.getTaxResidenceAddress();
			if (address != null){
				employeeAdapterDTO.setWorkState(address.getCountrySubDivisionCode());
			}
		}
		
		DateRange empPeriod = worker.getEmploymentPeriod();
		
		if (empPeriod != null){
			employeeAdapterDTO.setHireDate(DateMapper.mapDateToDTO(empPeriod.getStartDate()));
		}
		
		W4Detail w4Detail = null;
		DE4Detail de4Detail = null;
		List<WorkerComplianceDocDtl> complianceDtls = null;
		try {
			complianceDtls = taxWithHoldingsComponent.getCurrentAndDerivedWithholdings(worker);
		} catch (WorkflowException e) {
			e.printStackTrace();
			throw new ServiceFailureException("Failed while fetching the ComplianceDtls");
		}
		for (WorkerComplianceDocDtl compensationDtl : complianceDtls) {
			if (compensationDtl instanceof W4Detail){
				w4Detail = (W4Detail) compensationDtl;
			}else if (compensationDtl instanceof DE4Detail){
				de4Detail = (DE4Detail) compensationDtl;
			}
		}
		if (w4Detail != null){
			employeeAdapterDTO.setFedAllowances(w4Detail.getNoOfAllowances());
			employeeAdapterDTO.setFedFilingStatus(getFederalFilingStatus(w4Detail.getWithholdingStatus()));
		}
		
		if (de4Detail != null){
			employeeAdapterDTO.setStateAllowances(de4Detail.getNoOfAllowances());
			employeeAdapterDTO.setStateFilingStatus(getStateFilingStatus(de4Detail.getWithholdingStatus()));
		}
		
		Set<WorkerComplianceDocDtl> workerComplianceDtls = worker.getWorkerComplianceDocDtls();
		for (WorkerComplianceDocDtl workerComplianceDocDtl : workerComplianceDtls) {
			if (workerComplianceDocDtl instanceof CAWagePlan){
				String val = ((CAWagePlan) workerComplianceDocDtl).getValue();
				employeeAdapterDTO.setWagePlanCode(val);
			}
		}
		// TODO
		// hasRetirementPlan hasThirdPartySickPay isStatutory
		return employeeAdapterDTO;
	}
	
	private static FilingStatusAdapterEnum getFederalFilingStatus(W4WithholdingStatusEnum withholdingStatus){
		if (withholdingStatus == null){
			return null;
		}
		switch(withholdingStatus){
			case SINGLE:
				return FilingStatusAdapterEnum.SINGLE;
			case MARRIED:
				return FilingStatusAdapterEnum.MARRIED_FILING_JOINTLY;
			case MARRIEDSINGLERATE:
				return FilingStatusAdapterEnum.MARRIED_FILING_SEPARATELY;
		}
		return null;
	}
	
	private static FilingStatusAdapterEnum getStateFilingStatus(DE4WithholdingStatusEnum withholdingStatus){
		if (withholdingStatus == null){
			return null;
		}
		switch(withholdingStatus){
			case SINGLE:
				return FilingStatusAdapterEnum.SINGLE;
			case MARRIED:
				return FilingStatusAdapterEnum.MARRIED_FILING_JOINTLY;
			case MARRIEDONEINCOME:
				return FilingStatusAdapterEnum.MARRIED_FILING_SEPARATELY;
			case HEADOFHOUSEHOLD:
				return FilingStatusAdapterEnum.HEADOF_HOUSEHOLD;
		}
		return null;
	}
	
	private static String getValidSSN(String personId){
		if (personId.contains("-")){
			return personId.replace("-", "");
		}else{
			return personId;
		}
	}
}
