package com.intuit.sb.payroll.capability.pts.adapter.util;

import java.util.ArrayList;
import java.util.List;

import com.intuit.sbd.payroll.psp.adapters.gemini.AdapterMessage;
import com.intuit.sbd.payroll.psp.adapters.gemini.AdapterResult;


/**
 * When PTS responds to Gemini calls (Acknowledgement to webservice requests)
 * An "AdapterResult" object is returned.
 * This indicates success or failure as a boolean state.
 * In case of failure, the reasons are provided as a list of 
 * AdapterMessage objects.
 * AdapterMessage consists a CODE and a DESCRIPTION.
 * 
 * This utility will provide string indicators to the numeric codes of the AdapterMessage
 * and utility methods to check various states of the AdapterResult
 * 
 */
public class PTSAdapterResultUtil {

	/** Indicator that the Company sent to PTS was already added and is present in the PTS system */
	public static final String COMPANY_EXISTS_IN_PTS = "161";
	
	/** Check if the response indicates that the company already exists in PTS <br> 
	 *  
	 *  @see AdapterResult
	 *  @see AdapterMessage
	 * */
	public static boolean companyAlreadyExistsInPts(AdapterResult result) {
		List<AdapterMessage> messages = result.getMessages();
		
		for (AdapterMessage adapterMessage : messages) {
			if (adapterMessage.getCode().equals(COMPANY_EXISTS_IN_PTS)) {
				return true;
			}
		}
		return false;
	}
	
	/** Get a list of messages from AdapterResult as String */
	public static String getAdapterResultMessagesAsDelimitedString(AdapterResult result) {
		return getAdapterResultMessages(result).toString();
	}
		
	/** Get a list of messages from AdapterResult */
	public static List<String> getAdapterResultMessages(AdapterResult result) {
		List<String> list = new ArrayList<String>();
		List<AdapterMessage> messages = result.getMessages();
		
		for (AdapterMessage adapterMessage : messages) {
			if (adapterMessage.getDesc() != null && adapterMessage.getDesc().trim().length() > 0) {
				list.add(adapterMessage.getDesc());
			}
		}
		return list;
	}
}
