package com.intuit.sb.payroll.capability.pts.employer;

import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.pts.adapter.PTSRequestInvoker;
import com.intuit.sb.payroll.capability.pts.adapter.PayrollRequestWrapper;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.CompanyMapper;
import com.intuit.sb.payroll.capability.pts.adapter.util.PTSAdapterResultUtil;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.TaxServiceStatusEnum;
import com.intuit.sbd.payroll.psp.adapters.gemini.AcceptTaxServiceAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.AdapterResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.LiabilityBalanceAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.LiabilityBalancesResult;

public class EmployerPTSWorkflow {
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(EmployerPTSWorkflow.class);

	public static void addBankAccount(PTSEmployer ptsEmployer, List<BankAccount> bankAccounts) throws ValidationException, ServiceFailureException{
		// Check if Company is added in PTS
		if (StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			for (BankAccount bankAccount : bankAccounts) {
				if(StatusUpdateHandlerUtil.isEntityAddedToPTS(bankAccount)){
					throw new ValidationException("Bank Account is already exists in PTS");
				}else{
					PTSRequestInvoker.addBankAccountInfotoPTS(ptsEmployer, bankAccount);
				}	
			}
		}
	}
	
	public static void updateBankAccount(PTSEmployer ptsEmployer, List<BankAccount> bankAccounts) 
			throws ValidationException, ServiceFailureException{
		// Perform this only if Employer is Assisted
		if(StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			
			for(BankAccount bankAccount:bankAccounts){
				if(StatusUpdateHandlerUtil.isEntityAddedToPTS(bankAccount)){
					if(StatusUpdateHandlerUtil.isBankAccountRejected(bankAccount)){
						EmployerComponent sEmployerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
						
						// Create New Bank Account and copy data from the to be deleted one
						BankAccount newBankAccount = new BankAccount();
						newBankAccount.setAccountNo(bankAccount.getAccountNo());
						newBankAccount.setAccountType(bankAccount.getAccountType());
						newBankAccount.setBankName(bankAccount.getBankName());
						newBankAccount.setRoutingNo(bankAccount.getRoutingNo());
						newBankAccount.setBankAddress(bankAccount.getBankAddress());
						
						Employer employer = ptsEmployer.getEmployer();
						employer.removeBankAccount(bankAccount);
						
						// Delete the current one
						//FIXME EmployerComponent should have a removeBankAccount() API
						EntityService entityService = ServiceLocator.getInstance().getService(EntityService.JNDI_NAME);
						entityService.makeTransient(bankAccount);
						
						sEmployerComponent.addBankAccounts(new BankAccount[]{newBankAccount});
					}
				}
			}
		}
	}

	public static void sendAcceptTaxService(PTSEmployer ptsEmployer) throws ValidationException, 
					ServiceFailureException {
		// Ensure that Employer has accepted the Activation
		TaxServiceStatusEnum currentStatus = ptsEmployer.getCurrentTaxServiceStatus();
		if (!TaxServiceStatusEnum.ACTIVATION_ACTIVE_ST_ACCEPTED.equals(currentStatus)){
			throw new ValidationException("Employer has not yet accepted the Activation");
		}
		
		PayrollRequestWrapper requestWrapper = new PayrollRequestWrapper();
		
		AcceptTaxServiceAdapterDTO acceptTaxService = new AcceptTaxServiceAdapterDTO();
		Employer employer = ptsEmployer.getEmployer();

		Set<BankAccount> bankAccounts = employer.getBankAccounts();
		BankAccount companyBankAccount = null; // There will only be one Bank Account
		for (BankAccount bankAccount : bankAccounts) {
			companyBankAccount = bankAccount;
		}
		if (companyBankAccount == null){
			throw new ServiceFailureException("Bank Account not found for Employer");
		}
		
		String sourceCompanyId = CompanyMapper.namespaceEmployerId(employer);
		acceptTaxService.setSourceCompanyId(sourceCompanyId);
		acceptTaxService.setSourceCompanyBankAccountId(String.valueOf(companyBankAccount.getId()));
		// TODO find out what is the role of getBalances
		LiabilityBalancesResult liabilityBalancesResult = requestWrapper.getHistoricalLiabilityBalances(sourceCompanyId);

		if(!liabilityBalancesResult.isSuccess()) {
			StringBuffer sb = new StringBuffer().append("Unable to get Liability Balances for FirstDayOfService to PTS ")
			.append("PTS failed: \n REASONS: ") 
			.append(PTSAdapterResultUtil.getAdapterResultMessagesAsDelimitedString(liabilityBalancesResult));
			sLogger.error(sb.toString());
			throw new ServiceFailureException(sb.toString());
		}
		List<LiabilityBalanceAdapterDTO> balances = liabilityBalancesResult.getBalances();
		for (LiabilityBalanceAdapterDTO liabilityBalanceAdapterDTO : balances) {
			acceptTaxService.getBalances().add(liabilityBalanceAdapterDTO);
		}
		
		AdapterResult adapterResult = requestWrapper.acceptTaxService(acceptTaxService);
		if (adapterResult.isSuccess()){
			sLogger.debug("Employer accepted First day of service "+ employer.getId());
		}else{
			StringBuffer sb = new StringBuffer().append("Unable to send FirstDayOfService to PTS ")
			.append("PTS failed: \n REASONS: ") 
			.append(PTSAdapterResultUtil.getAdapterResultMessagesAsDelimitedString(adapterResult));
	
			sLogger.error(sb.toString());
			// PSP update failed - log the failure
			throw new ServiceFailureException(sb.toString());
		}
	}
}
