package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.Date;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSAgencyMapper;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.pts.entity.ACHEnrollmentState;
import com.intuit.sb.payroll.pts.entity.ACHEnrollmentStateEAO;
import com.intuit.sb.payroll.pts.entity.ACHEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSWorkflowState;
import com.intuit.sb.payroll.pts.entity.TaxServiceState;

public class ACHStatusUpdateHandler extends AbstractStatusUpdateHandler  {

	private static ACHStatusUpdateHandler sInstance = new ACHStatusUpdateHandler();

	private ACHStatusUpdateHandler() {
		registerStatusChangeWorkers();
	}

	
	public void validateNotificationRequest(PTSNotificationRequest ptsNotification)
			throws PTSNotificationException {
		
		// Check if this Handler is handling the right notification
		if (!ptsNotification.getNotificationType().equals(PTSNotificationTypeEnum.ACH_NOTIFICATION)){
			throw new PTSNotificationException("Wrong Notification being handler here");
		}
	}

	public static StatusUpdateHandler getInstance() {
		return sInstance;
	}
	
	// Inner classes for Individual state handlers

	static class GenericACHStatusChangeWorker extends StatusChangeWorker{

		@Override
		public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
			try {
				updateACHEnrollmentStatus(ptsEmployer, ptsNotification);
			} catch (ValidationException e) {
				throw new PTSNotificationException(e.getLocalizedMessage());
			} catch (ServiceFailureException e) {
				throw new PTSNotificationException(e.getLocalizedMessage());
			}

		}

	}
	
	
	/**
	 * Generic Post processing event after ACH Enrollment update has been received from PTS
	 */
	@SuppressWarnings("unused")
    private static class CommonStatusChangeWorker extends GenericACHStatusChangeWorker{
		public CommonStatusChangeWorker() {
			super();
			statusTypeChange.add(ACHEnrollmentStatusEnum.ACH_ENROLLMENT_INCOMPLETE);
			statusTypeChange.add(ACHEnrollmentStatusEnum.ACH_ENROLLMENT_IN_PROGRESS);
			statusTypeChange.add(ACHEnrollmentStatusEnum.ACH_ENROLLMENT_COMPLETE);
		}

    	@Override
		public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
    		super.updateStatus(ptsEmployer, ptsNotification);

		}
    }

	public synchronized static void updateACHEnrollmentStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws ValidationException, ServiceFailureException {
		ptsEmployer =  getPTSEmployer();
		ACHEnrollmentStateEAO achEnrollStateEAO = ServiceLocator.getInstance().getService(ACHEnrollmentStateEAO.JNDI_NAME);
		ACHEnrollmentState achEnrollmentState = new ACHEnrollmentState();
		achEnrollmentState.setAchEnrollmentStatus((ACHEnrollmentStatusEnum) ptsNotification.getNotificationStatus());
		if (ptsNotification.getTimestamp() != null){
			achEnrollmentState.setStatusDate(ptsNotification.getTimestamp());
		}else{
			achEnrollmentState.setStatusDate(new Date());
		}

		String agencyId = ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.AGENCY_ID);
		if (agencyId != null){
			achEnrollmentState.setAgency(PTSAgencyMapper.mapAgency(agencyId));
		}
		achEnrollmentState.setActive(Boolean.TRUE);
		// FIXME achEnrollmentState.setPTSMessages(ptsNotification.getPTSMessage());
		

		PTSWorkflowState ptsWorkflowState = ptsEmployer.getPTSWorkflowState();
		TaxServiceState taxServiceState = ptsWorkflowState.getTaxServiceState();
		achEnrollmentState = achEnrollStateEAO.makePersistent(achEnrollmentState);
		taxServiceState.addNotificationStatusEntity(achEnrollmentState);
	}
}
