package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;

/**
 * Abstract file for all PTS Status change handlers
 */

public abstract class AbstractStatusUpdateHandler implements StatusUpdateHandler{

	protected static final Logger sLogger = PayrollLoggerFactory.makeLogger(AbstractStatusUpdateHandler.class);

	protected Map<Object,StatusChangeWorker> statusChangeWorkersMap = new HashMap<Object, StatusChangeWorker>();

	public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification)
			throws PTSNotificationException {

		validateNotificationRequest(ptsNotification);
		// Get the Status Change Worker for the notification received
		Object notificationEnum = ptsNotification.getNotificationStatus();
		StatusChangeWorker statusChangeWorker = statusChangeWorkersMap.get(notificationEnum);
		sLogger.debug("StatusChangeWorker for "+ notificationEnum + " is "+ statusChangeWorker);
		if (statusChangeWorker != null){
			statusChangeWorker.updateStatus(ptsEmployer,ptsNotification);
		}else{
			throw new PTSNotificationException("No StateWorkerFound For "+notificationEnum);
		}
	}

	protected  abstract void validateNotificationRequest(PTSNotificationRequest ptsNotification) throws PTSNotificationException;

	@SuppressWarnings("unchecked")
	protected void registerStatusChangeWorkers(){
		Class<Object>[] stateChangeWorkers= this.getClass().getDeclaredClasses();
		StatusChangeWorker scw = null;
		for (Class<Object> stateChangeWorker : stateChangeWorkers) {
			try {
				if (stateChangeWorker.isMemberClass()){
					scw = (StatusChangeWorker) stateChangeWorker.newInstance();
					List<Object> statusTypes = scw.getStatusTypes();
					for (Object activationStateEnum : statusTypes) {
						if (!statusChangeWorkersMap.keySet().contains(activationStateEnum)) {
							statusChangeWorkersMap.put(activationStateEnum, scw);
						}
					}
				}
			} catch (Exception e) {
				// We just want to catch and log this exception. We don't want
				// the instantiation of this class to fail. Tests will determine the
				// logical failures for handlers
				sLogger.error("Error in instantiation", e);
			}
		}
	}
	
	protected static PTSEmployer getPTSEmployer() throws ServiceFailureException{
		EntityService entityService = ServiceLocator.getInstance().getService(EntityService.JNDI_NAME);
		List<PTSEmployer> ptsEmployeList = entityService.findAll(PTSEmployer.class);
		
		if (ptsEmployeList.size() == 1){
        	return ptsEmployeList.get(0);
        }else {
        	throw new ServiceFailureException("Not able to locate the Company");
        }
	}

}
