package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.Date;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSAgencyMapper;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.pts.entity.ActivationState;
import com.intuit.sb.payroll.pts.entity.ActivationStateEAO;
import com.intuit.sb.payroll.pts.entity.ActivationStatusEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSWorkflowState;
import com.intuit.sb.payroll.pts.entity.TaxServiceState;


public class ActivationStatusUpdateHandler extends AbstractStatusUpdateHandler {

    private static ActivationStatusUpdateHandler sInstance = new ActivationStatusUpdateHandler();

	private ActivationStatusUpdateHandler() {
		registerStatusChangeWorkers();
	}

	public static StatusUpdateHandler getInstance() {
		return sInstance;
	}

	public void validateNotificationRequest(PTSNotificationRequest ptsNotification) throws PTSNotificationException {

		// Check if this Handler is handling the right notification
		if (!ptsNotification.getNotificationType().equals(PTSNotificationTypeEnum.ACTIVATION_NOTIFICATION)){
			throw new PTSNotificationException("Wrong Notification being handler here");
		}
	}

	public synchronized  static void updatePTSAssistedActivationState(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification ) throws ValidationException, ServiceFailureException{
		ptsEmployer =  getPTSEmployer();
		ActivationStateEAO activationStateEAO = ServiceLocator.getInstance().getService(ActivationStateEAO.JNDI_NAME);
		ActivationState activationState = new ActivationState();
		activationState.setActivationStatus((ActivationStatusEnum) ptsNotification.getNotificationStatus());
		String ptsAgencyId = ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.AGENCY_ID);
		activationState.setAgency(PTSAgencyMapper.mapAgency(ptsAgencyId));
		if (ptsNotification.getTimestamp() != null){
			activationState.setStatusDate(ptsNotification.getTimestamp());
		}else{
			activationState.setStatusDate(new Date());
		}

		String agencyId = ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.AGENCY_ID);
		if (agencyId != null){
			activationState.setAgency(PTSAgencyMapper.mapAgency(agencyId));
		}
		activationState.setActive(Boolean.TRUE);
		// FIXME activationState.setPTSMessages(ptsNotification.getPTSMessage());

		PTSWorkflowState ptsWorkflowState = ptsEmployer.getPTSWorkflowState();
		TaxServiceState taxServiceState = ptsWorkflowState.getTaxServiceState();
		activationState = activationStateEAO.makePersistent(activationState);
		taxServiceState.addNotificationStatusEntity(activationState);
	}

	// Inner classes for Individual state handlers

	static class GenericStatusChangeWorker extends StatusChangeWorker{

		@Override
		public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
			try {
				updatePTSAssistedActivationState(ptsEmployer, ptsNotification);
			} catch (ValidationException e) {
				throw new PTSNotificationException(e);
			} catch (ServiceFailureException e) {
				throw new PTSNotificationException(e);
			}
		}
	}


	/** 
	 * StatusChangeWorker to invoke when "Activation Active" event is received from PTS 
	 * 
	 */
    @SuppressWarnings("unused")
	private static class ActivationActiveStatusChangeWorker extends GenericStatusChangeWorker {

		public ActivationActiveStatusChangeWorker() {
			super();
			statusTypeChange.add(ActivationStatusEnum.ACTIVATION_ACTIVE_ST_RECEIVED);
		}

		/**
		 * When updateStatus is called - we create a new Task for  ACTIVATION ACTIVE ST RECEIVED
		 */
		@Override
		public void updateStatus(PTSEmployer ptsEmployer,PTSNotificationRequest ptsNotification)
				throws PTSNotificationException {
			super.updateStatus(ptsEmployer,ptsNotification);
			try {
				EmployerPTSWorkflowTasks.createPtsActivationActiveTask();
			} catch (ServiceFailureException e) {
				throw new PTSNotificationException("Unable to create PTS ACTIVATION ACTIVE Task", e);
			}
		}		
	}

    
    @SuppressWarnings("unused")
    private static class CommonStatusChangeWorker extends GenericStatusChangeWorker{
		public CommonStatusChangeWorker() {
			super();
			statusTypeChange.add(ActivationStatusEnum.ACTIVATION_CANCELLED_ST_RECEIVED);
			statusTypeChange.add(ActivationStatusEnum.ACTIVATION_AGENT_REVIEW_APPROVED_ST_RECEIVED);
			statusTypeChange.add(ActivationStatusEnum.ACTIVATION_AWAITING_AGENT_REVIEW_ST_RECEIVED);
			statusTypeChange.add(ActivationStatusEnum.ACTIVATION_PENDING_RECEIVED);
		}

    	@Override
		public void updateStatus(PTSEmployer ptsEmployer,PTSNotificationRequest ptsNotification) throws PTSNotificationException {
    		super.updateStatus(ptsEmployer,ptsNotification);
		}


    }
}
