package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.Date;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSAgencyMapper;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSWorkflowState;
import com.intuit.sb.payroll.pts.entity.RAFEnrollmentState;
import com.intuit.sb.payroll.pts.entity.RAFEnrollmentStateEAO;
import com.intuit.sb.payroll.pts.entity.RAFEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.TaxServiceState;

public class RAFStatusUpdateHandler extends AbstractStatusUpdateHandler {
    
	private static RAFStatusUpdateHandler rafStatusUpdateHandler = new RAFStatusUpdateHandler();

	private RAFStatusUpdateHandler(){
		registerStatusChangeWorkers();
	}

	public void validateNotificationRequest(PTSNotificationRequest ptsNotification) throws PTSNotificationException {
		if (!ptsNotification.getNotificationType().equals(PTSNotificationTypeEnum.RAF_NOTIFICATION)){
			throw new PTSNotificationException("Wrong Notification being handler here");
		}
	}
	
	public static StatusUpdateHandler getInstance() {
		return rafStatusUpdateHandler;
	}
	
	// Inner classes for Individual state handlers

	static class GenericRAFStatusChangeWorker extends StatusChangeWorker{

		@Override
		public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
			try {
				updateRAFEnrollmentStatus(ptsEmployer,ptsNotification);
			} catch (ValidationException e) {
				throw new PTSNotificationException(e.getLocalizedMessage());
			} catch (ServiceFailureException e) {
				throw new PTSNotificationException(e.getLocalizedMessage());
			}

		}

	}
	
	/**
	 * Generic Post processing event after RAF Enrollment update has been received from PTS
	 */
	@SuppressWarnings("unused")
    private static class CommonStatusChangeWorker extends GenericRAFStatusChangeWorker{
		public CommonStatusChangeWorker() {
			super();
			statusTypeChange.add(RAFEnrollmentStatusEnum.RAF_ENROLLMENT_INCOMPLETE);
			statusTypeChange.add(RAFEnrollmentStatusEnum.RAF_ENROLLMENT_READY_FOR_RAF);
			statusTypeChange.add(RAFEnrollmentStatusEnum.RAF_ENROLLMENT_READY_FOR_TAPE);
			statusTypeChange.add(RAFEnrollmentStatusEnum.RAF_ENROLLMENT_AGENT_REJECTED_FORM);
			statusTypeChange.add(RAFEnrollmentStatusEnum.RAF_ENROLLMENT_IN_PROGRESS);
			statusTypeChange.add(RAFEnrollmentStatusEnum.RAF_ENROLLMENT_COMPLETE);
			statusTypeChange.add(RAFEnrollmentStatusEnum.RAF_ENROLLMENT_AGENCY_REJECTED);
			statusTypeChange.add(RAFEnrollmentStatusEnum.RAF_ENROLLMENT_AWAITING_SIGNED_FORMS);
		}

    	@Override
		public void updateStatus(PTSEmployer ptsEmployer,PTSNotificationRequest ptsNotification) throws PTSNotificationException {
    		super.updateStatus(ptsEmployer,ptsNotification);

		}
    }

	public synchronized static void updateRAFEnrollmentStatus(PTSEmployer ptsEmployer,PTSNotificationRequest ptsNotification) throws ValidationException, ServiceFailureException {
		ptsEmployer =  getPTSEmployer();
		RAFEnrollmentStateEAO rafEnrollStateEAO = ServiceLocator.getInstance().getService(RAFEnrollmentStateEAO.JNDI_NAME);
		RAFEnrollmentState rafEnrollmentState = new RAFEnrollmentState();
		rafEnrollmentState.setRafEnrollmentStatus((RAFEnrollmentStatusEnum) ptsNotification.getNotificationStatus());
		if (ptsNotification.getTimestamp() != null){
			rafEnrollmentState.setStatusDate(ptsNotification.getTimestamp());
		}else{
			rafEnrollmentState.setStatusDate(new Date());
		}

		String agencyId = ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.AGENCY_ID);
		if (agencyId != null){
			rafEnrollmentState.setAgency(PTSAgencyMapper.mapAgency(agencyId));
		}
		// FIXME achEnrollmentState.setPTSMessages(ptsNotification.getPTSMessage());
		rafEnrollmentState.setActive(Boolean.TRUE);
		PTSWorkflowState ptsWorkflowState = ptsEmployer.getPTSWorkflowState();
		TaxServiceState taxServiceState = ptsWorkflowState.getTaxServiceState();
		rafEnrollmentState = rafEnrollStateEAO.makePersistent(rafEnrollmentState);
		taxServiceState.addNotificationStatusEntity(rafEnrollmentState);
	}

	

}
