package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.Date;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSAgencyMapper;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.StateIDModified;
import com.intuit.sb.payroll.pts.entity.StateIDModifiedEAO;

public class StateIdModifiedNotificationHandler extends AbstractStatusUpdateHandler  {

	private static StateIdModifiedNotificationHandler sInstance = new StateIdModifiedNotificationHandler();

	private StateIdModifiedNotificationHandler() {
		registerStatusChangeWorkers();
	}

	
	public void validateNotificationRequest(PTSNotificationRequest ptsNotification)
			throws PTSNotificationException {
		
		// Check if this Handler is handling the right notification
		if (!ptsNotification.getNotificationType().equals(PTSNotificationTypeEnum.STATE_ID_MODIFIED)){
			throw new PTSNotificationException("Wrong Notification being handler here");
		}
		
		// Ensure AgencyId and NewValue is provided
		if (!ptsNotification.getNotificationAttributes().containsKey(PTSNotificationRequestMapper.NEW_VALUE)){
			throw new PTSNotificationException("StateIDModified Notification does not have New Value");
		}
		
		if (!ptsNotification.getNotificationAttributes().containsKey(PTSNotificationRequestMapper.AGENCY_ID)){
			throw new PTSNotificationException("StateIDModified Notification does not have AgencyId");
		}
	}

	public static StatusUpdateHandler getInstance() {
		return sInstance;
	}
	
	public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification)
	  throws PTSNotificationException {
		validateNotificationRequest(ptsNotification);
		try {
			updateStateIDModifiedNotification(ptsNotification);
		} catch (ValidationException e) {
			throw new PTSNotificationException(e.getLocalizedMessage());
		} catch (ServiceFailureException e) {
			throw new PTSNotificationException(e.getLocalizedMessage());
		}
	}
	
	// Inner classes for Individual state handlers

	public static void updateStateIDModifiedNotification(PTSNotificationRequest ptsNotification) throws ValidationException, ServiceFailureException {
		StateIDModifiedEAO stateIDModifiedEAO = ServiceLocator.getInstance().getService(StateIDModifiedEAO.JNDI_NAME);
		StateIDModified stateIDModified = new StateIDModified();
		String newValue = ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.NEW_VALUE);
		String oldValue = ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.OLD_VALUE);
		String agencyId = ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.AGENCY_ID);
		
		stateIDModified.setAgency(PTSAgencyMapper.mapAgency(agencyId));
		stateIDModified.setNewValue(newValue);
		stateIDModified.setOldValue(oldValue);
		
		if (ptsNotification.getTimestamp() != null){
			stateIDModified.setStatusDate(ptsNotification.getTimestamp());
		}else{
			stateIDModified.setStatusDate(new Date());
		}

		
		if ((oldValue == null) || (oldValue.length() == 0)){
			// PTS sends us the stateId Modified notification when a company is created, 
			// this stateId has to be ignored
			stateIDModified.setActive(Boolean.FALSE);
		}else{
			stateIDModified.setActive(Boolean.TRUE);
		}
		
		// FIXME achEnrollmentState.setPTSMessages(ptsNotification.getPTSMessage());

		stateIDModified = stateIDModifiedEAO.makePersistent(stateIDModified);
	}
}
