package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.ArrayList;
import java.util.List;

import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.pts.entity.BankAccountState;
import com.intuit.sb.payroll.pts.entity.BankAccountStateEAO;
import com.intuit.sb.payroll.pts.entity.BankAccountStatusEnum;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentState;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentStateEAO;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.InteractionEnum;
import com.intuit.sb.payroll.pts.entity.PPSPTSInteraction;
import com.intuit.sb.payroll.pts.entity.PPSPTSInteractionEAO;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.StateIDModified;
import com.intuit.sb.payroll.pts.entity.StateIDModifiedEAO;

public class StatusUpdateHandlerUtil {

    /**
     * This utility method will check if the entity has been added to PTS
     *
     * @return true if entity has been added, false otherwise
     */
    public static boolean isEntityAddedToPTS(Identifiable entity){
    	List<PPSPTSInteraction> interactionList = getPPSPTSInteractions(entity);

    	if (interactionList.size() == 0){
            return false;
        }else{
            return true;
        }
    }
    
 
    
    /**
     * This method will return a list of Interactions between the entity and the PTS service
     * @param ptsEmployer
     * @param entity
     * @return List<PPSPTSInteraction>
     */
    public static List<PPSPTSInteraction> getPPSPTSInteractions(Identifiable entity){

    	if(entity == null){
            return new ArrayList<PPSPTSInteraction>();
        }
        String entityname = getCertifiedEntityName(entity);
        long entityId = entity.getId();
        
        PPSPTSInteractionEAO ppsPTSInteractionEAO = ServiceLocator.getInstance().getService(PPSPTSInteractionEAO.JNDI_NAME);
        return ppsPTSInteractionEAO.findAll(entityname, new Long(entityId));
    }
    
    
    public static String getCertifiedEntityName(Identifiable entity) {
		if (entity instanceof Employee){
			return Worker.class.getName();
		}
		return entity.getClass().getName();
	}

    // TODO revisit and remove
	public static PPSPTSInteraction getPPSPTSInteraction(String entityname, long entityId, InteractionEnum interactionType){
    	PPSPTSInteractionEAO ppsPTSInteractionEAO = ServiceLocator.getInstance().getService(PPSPTSInteractionEAO.JNDI_NAME);
    	List<PPSPTSInteraction> intereactionSet =  ppsPTSInteractionEAO.findAll(entityname, new Long(entityId));
        
        for (PPSPTSInteraction interaction : intereactionSet) {
            if ((interaction.getEntity().equals(entityname)) &&
                    (interaction.getIdentifier().equals(new Long(entityId))) &&
                    interaction.getOperation().equals(interactionType)){
                return interaction;
            }
        }
        return null;
    }
    
    /**
     * Returns the Last PPSPTSInteraction by the entity with the PTS
     * @param ptsEmployer
     * @param entity
     * @return PPSPTSInteraction
     */
    private static PPSPTSInteraction getLastModifiedPPSPTSInteraction(Identifiable entity){
        List<PPSPTSInteraction> interactionList = getPPSPTSInteractions(entity);
        if (interactionList.size() > 0){
        	// This is a sorted list of descending interactionDate column
        	return interactionList.get(0);
        }
        return null;
    }

    /**
     * Returns the EFTPSEnrollment state for the recently received EFTPSRejection notification
     * @return EFTPSEnrollmentState
     */
    public static EFTPSEnrollmentState getRecentlyReceivedEFTPSRejection(){

    	EFTPSEnrollmentStateEAO eftpsEnrollState = ServiceLocator.getInstance().getService(EFTPSEnrollmentStateEAO.JNDI_NAME);
    	List<EFTPSEnrollmentState> eftpsStateList = eftpsEnrollState.findAllActiveInSortedOrder();
    	for (EFTPSEnrollmentState enrollmentState : eftpsStateList) {
			if ((enrollmentState.getEftpsEnrollmentStatus().equals(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH)) ||
			    (enrollmentState.getEftpsEnrollmentStatus().equals(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH)) ||
			    (enrollmentState.getEftpsEnrollmentStatus().equals(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_AGEDOUT))){
				return enrollmentState;
			}
		}
    	return null;
    }

    /**
     * Returns a boolean value indicating whether the EIN/Legal name is editable/updatable
     * @param ptsEmployer
     * @return isDataUpdateDisabled
     */
    public static boolean isEINOrLegalNameUpdateDisabled(PTSEmployer ptsEmployer){
        EFTPSEnrollmentState lastEFPTSRejectedRecd = getRecentlyReceivedEFTPSRejection();
        
        if (lastEFPTSRejectedRecd != null){ // error state
        	return false;
        }
        
        PPSPTSInteraction lastEmployerInteraction = getLastModifiedPPSPTSInteraction(ptsEmployer.getEmployer());
        if (lastEmployerInteraction != null){
        	// has been added/update but no active error yet
        	return true;
        }else {
        	return false;
        }
    }

    /**
     * Returns a boolean value indicating whether the bank account information is editable/updatable
     * @param ptsEmployer
     * @return isDataUpdateDisabled
     */
    public static boolean isBankAccountEditAllowed(BankAccount bankAccount){
       BankAccountState bankRejectedState = getRecentlyReceivedBankRejection(bankAccount);
       PPSPTSInteraction lastBankAccountInteraction = getLastModifiedPPSPTSInteraction(bankAccount);       
       
       // check if in error state
       if (bankRejectedState != null){
    	   return true;
       }else if(lastBankAccountInteraction != null){
    	   // has been added/update but no active error yet
    	   return false;   
       }
       return true;
    }
    
    /**
     * Returns a boolean value indicating whether the bank account information is editable/updatable
     * @param ptsEmployer
     * @return isDataUpdateDisabled
     */
    public static boolean isBankAccountRejected(BankAccount bankAccount){
       BankAccountState bankRejectedState = getRecentlyReceivedBankRejection(bankAccount);
       if (bankRejectedState != null){
    	   return true;
       }else{
           return false;
       }
    }

    
    
    /**
     * Returns the EFTPSEnrollment state for the recently received EFTPSRejection notification
     * @return EFTPSEnrollmentState
     */
    private static BankAccountState getRecentlyReceivedBankRejection(BankAccount bankAccount){

    	BankAccountStateEAO bankAccountStateEAO = ServiceLocator.getInstance().getService(BankAccountStateEAO.JNDI_NAME);
    	List<BankAccountState> bankAccountStates = bankAccountStateEAO.findAllActiveInSortedOrder();
    	
    	for (BankAccountState bankAccountState : bankAccountStates) {
			if (bankAccountState.getBankAcctId().equals(bankAccount.getId()) && 
					bankAccountState.getStatus().equals(BankAccountStatusEnum.BANKACCOUNT_VERIFICATION_FAILED_RECEIVED)){
				return bankAccountState;
			}
		}
    	return null;
    }


     /**
     * Check if user is allowed to send data to PTS - this is done by <br>
     * looking at the various states of information sent to PTS by Gemini and vice-versa.
     * @param ptsEmployer
     * @return
     */
    public static boolean allowDataSubmissionToPTS(PTSEmployer ptsEmployer) {
    	return !isEINOrLegalNameUpdateDisabled(ptsEmployer);
//    	
//        sLogger.debug("Executing allowDataSubmissionToPTS()");
//        // Gemini --> PTS
//        List<PPSPTSInteraction> employerInteractions = getPPSPTSInteractions(ptsEmployer, ptsEmployer.getEmployer());
//
//        // case where NO INFO has been sent to PTS yet - no further computation required
//        if (employerInteractions.size() == 0) {
//            return true;
//        }
//
//        // PTS --> Gemini
//    	EFTPSEnrollmentStateEAO eftpsEnrollState = ServiceLocator.getInstance().getService(EFTPSEnrollmentStateEAO.JNDI_NAME);
//        List<EFTPSEnrollmentState> eStates = eftpsEnrollState.findAllInSortedOrder();
//
//        //  TODO - check for BankAccount Verification Failure
//
//        // check for Employer add/updates
//        boolean employerAddedToPts = false;
//        Date employerAddedToPtsTime = null;
//        boolean employerUpdatedToPts = false;
//        Date employerUpdatedToPtsTime = null;
//
//        for (PPSPTSInteraction intrxn : employerInteractions) {
//            if(intrxn.getOperation().equals(InteractionEnum.ADDED)) {
//                employerAddedToPts = true;
//                employerAddedToPtsTime = intrxn.getInteractionDate();
//            }
//            if(intrxn.getOperation().equals(InteractionEnum.UPDATED)) {
//                employerUpdatedToPts = true;
//                employerUpdatedToPtsTime = intrxn.getInteractionDate();
//            }
//        }
//
//        if (employerAddedToPts) {
//            sLogger.debug("Employer " + ptsEmployer.getEmployer().getId() + " : " + ptsEmployer.getEmployer().getLegalName() +
//                    " added to PSP??? : " + employerAddedToPts + " : " + employerAddedToPtsTime);
//        }
//        if (employerUpdatedToPts) {
//            sLogger.debug("Employer " + ptsEmployer.getEmployer().getId() + " : " + ptsEmployer.getEmployer().getLegalName() +
//                    " updated to PSP??? : " + employerUpdatedToPts + " : " + employerUpdatedToPtsTime);
//        }
//
//        boolean einMM = false;
//        boolean lnMM = false;
//        Date einMMDate = null;
//        Date lnMMDate = null;
//        // loop through the sorted EFTPS Enrollment states - on first occurence of
//        //  EIN / Legal Name mismatch, pick up that info and exit
//        //  the data is sorted in descending order so the information picked up will be the latest
//        for (EFTPSEnrollmentState eState : eStates) {
//            if (eState.getEftpsEnrollmentStatus().equals(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH)) {
//                einMM = true;
//                einMMDate = eState.getStatusDate();
//            }
//            break;
//        }
//        for (EFTPSEnrollmentState eState : eStates) {
//            if (eState.getEftpsEnrollmentStatus().equals(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH)) {
//                lnMM = true;
//                lnMMDate = eState.getStatusDate();
//            }
//            break;
//        }
//
//        // employer added to PTS - no EIN or Legal Name mismatch received
//        if (employerAddedToPts && !einMM && !lnMM) {
//            sLogger.debug("Employer added to PTS and no EIN/LN mismatch after that");
//            return false;
//        }
//
//        if (einMM || lnMM) {    //   only one will be true - if any
//            Date d = (einMM?einMMDate:lnMMDate);    //  date-time of EIN/LN mismatch notification
//
//            if (!employerUpdatedToPts) {
//                sLogger.debug("TRUE: Employer UPDATE after Mismatch has not happened");
//                //  Employer has received Mismatch notification but not updated information
//                return true;
//            }
//
//            // there is an update from Employer to PTS - check that it is AFTER the last
//            // EIN/LN mismatch notification
//            // if Employer update happened BEFORE the Mismatch notification, return TRUE
//            // if Employer update happened AFTER the Mismatch notification, return FALSE
//            return employerUpdatedToPtsTime.before(d);
//        }
//
//        return true;    //  default
    }

    /**
     * Utility method to check if the Tax Id for a particular Agency can be updated
     * or still in locked state (under verfication)
     * @param name
     * @return
     */
	public static boolean isEmployerIdUpdateAllowed(String name) {
		// This function has to be implemented after more discussions
		return true;
	}
	
	public static void inActivateAllEFTPSNotificationStates(){
		EFTPSEnrollmentStateEAO eftpsEAO = ServiceLocator.getInstance().getService(EFTPSEnrollmentStateEAO.JNDI_NAME);
		List<EFTPSEnrollmentState> activeNotificationStates = eftpsEAO.findAllActiveInSortedOrder();
		for(EFTPSEnrollmentState activeState : activeNotificationStates){
			activeState.setActive(Boolean.FALSE);
			eftpsEAO.makeManaged(activeState);
		}
		
	}
	
	public static void inActivateAllBANKNotificationStates(Long bankId){
		BankAccountStateEAO bankStateEAO = ServiceLocator.getInstance().getService(BankAccountStateEAO.JNDI_NAME);
		List<BankAccountState> bankStates = bankStateEAO.findAllActiveInSortedOrder();
		for(BankAccountState activeState : bankStates){
			if (activeState.getBankAcctId().equals(bankId)){
				activeState.setActive(Boolean.FALSE);
				bankStateEAO.makeManaged(activeState);
			}
		}
	}
	
	public static void inActivateAllStateIDNotificationStates(AgencyEnum agency){
		StateIDModifiedEAO stateIDModEAO = ServiceLocator.getInstance().getService(StateIDModifiedEAO.JNDI_NAME);
		List<StateIDModified> stateIdModifiedStates = stateIDModEAO.findAllActiveInSortedOrder();
		for(StateIDModified activeState : stateIdModifiedStates){
			if (activeState.getAgency().equals(agency)){
				activeState.setActive(Boolean.FALSE);
				stateIDModEAO.makeManaged(activeState);
			}
		}
	}
	
}
