package com.intuit.sb.payroll.capability.taxes.withholding.impl;

import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * Validates fields in the WorkerComplianceDocDtl entity that have values. Fields with no values are ignored.
 * 
 * This class does not ensure data integrity or completeness.
 * 
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
public class WithholdingDataValidator extends AbstractEntityDataValidator<WorkerComplianceDocDtl> {
	private static WithholdingDataValidator sInstance = new WithholdingDataValidator();
	
	private WithholdingDataValidator() {

	}

	public static WithholdingDataValidator getInstance() {
		return sInstance;
	}

	@Override
	public void validateEntityCollection(List<WorkerComplianceDocDtl> employeeWithholdingDataCollection)
			throws ValidationException {
		for (WorkerComplianceDocDtl employeeWithholdingData : employeeWithholdingDataCollection) {
			validateEntityImpl(employeeWithholdingData);
		}
	}

	@Override
	public void validateEntity(WorkerComplianceDocDtl employeeWithholdingData) throws ValidationException {
		validateEntityImpl(employeeWithholdingData);
	}

	public void validateEntityImpl(WorkerComplianceDocDtl employeeWithholdingData) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(employeeWithholdingData, "employeeWithholdingData");
		WorkerComplianceDocValidator validator = WorkerComplianceDocValidatorFactory.getValidatorForDocType(employeeWithholdingData.getDocId());
		validator.validateWorkerComplianceDocDtl(employeeWithholdingData);
	}
}
