package com.intuit.sb.payroll.capability.taxes.withholding.impl.validators;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerComplianceDocValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * Validates DE4 docs.
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public class DE4WithholdingValidator implements WorkerComplianceDocValidator {
	
	public DE4WithholdingValidator() {
		
	}
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerComplianceDocValidator#validateWorkerComplianceDocDtl(com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl)
	 */
	public void validateWorkerComplianceDocDtl(WorkerComplianceDocDtl workerComplianceDoc) throws ValidationException {	
		DE4Detail de4Detail = (DE4Detail) workerComplianceDoc;
		PayrollParamValidator.checkIsNotNull(de4Detail.getWithholdingStatus(), "withholdingStatus");
		if (de4Detail.getWithholdingStatus() == DE4WithholdingStatusEnum.NOTSET) {
			throw new ValidationException("Withholding Status is not set.");
		}
		PayrollParamValidator.checkIsNonNegative(de4Detail.getNoOfAllowances(), "Allowance");
		Money dE4AdditionalAmount = de4Detail.getAdditionalAmount();
		if (dE4AdditionalAmount != null) {
			PayrollParamValidator.checkIsNonNegative(dE4AdditionalAmount.getAmount(), "Extra Withholding");
		}
	}

}
