package com.intuit.sb.payroll.capability.taxes.withholding.impl.validators;

import java.util.HashSet;
import java.util.Set;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerComplianceDocValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.CAWagePlan;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * Validates DE545 Docs.
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public class DE545WagePlanValidator implements WorkerComplianceDocValidator {

	/**
	 * List of supported Wage Plan Codes.  FIXME: Get this from compliance.
	 */
	private static Set<String> wagePlanCodes = new HashSet<String>();
	
	static {
		wagePlanCodes.add("S");
		wagePlanCodes.add("A");
		wagePlanCodes.add("L");
		wagePlanCodes.add("J");
		wagePlanCodes.add("P");
		wagePlanCodes.add("R");
		wagePlanCodes.add("U");
	}
	
	public DE545WagePlanValidator() {
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerComplianceDocValidator#validateWorkerComplianceDocDtl(com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl)
	 */
	public void validateWorkerComplianceDocDtl(WorkerComplianceDocDtl workerComplianceDoc) throws ValidationException {
		CAWagePlan caWagePlan = (CAWagePlan) workerComplianceDoc;
		String wagePlanCode = caWagePlan.getValue();
		PayrollParamValidator.checkIsNotNullOrEmptyString(wagePlanCode, "wagePlanCode");
		if (!wagePlanCodes.contains(wagePlanCode)) {
			throw new ValidationException("Invalid Wage Plan Code: " + wagePlanCode);
		}
	}

}
