package com.intuit.sb.payroll.capability.util;

import java.util.Date;

import org.apache.commons.lang.time.DateUtils;

import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.utils.PtdUtils;

/**
 * Utility class with {@link DateRange DateRange} related methods
 */
public class DateRangeUtil {

	// utility to check if given date falls within the date range - inclusive of
	// date range start/end dates
	public static boolean isDateWithinDateRange(Date date, DateRange dateRange) {
		Date start = dateRange.getStartDate();
		Date end   = dateRange.getEndDate();

//		check if "date" is between the start/end dates of the daterange
//		or equal to the start date of the daterange
		return ((date.after(start) && date.before(end)) || dateEquals(date, start));
	}

	/** Compare dates using only the month, day and year - ignore hour/min/sec */
	public static boolean dateEquals(Date d1, Date d2) {
		return DateUtils.isSameDay(d1, d2);
	}

	/**
	 * Given a {@link DateRange}, check that it maps to a quarter.
	 * Create a quarter from the start date of the {@link DateRange}.
	 * Then check the the start and end dates of the input and created DateRange
	 * @param dateRange
	 * @return
	 */
	public static boolean doesDateRangeMapToQuarter(DateRange dateRange) {
		Date start = dateRange.getStartDate();
		Date end = dateRange.getEndDate();

		DateRange calculatedRange = PtdUtils.getQtrRangeForDate(start);
		return
			DateUtils.isSameDay(start, calculatedRange.getStartDate()) &&
			DateUtils.isSameDay(end, calculatedRange.getEndDate());
	}
}
