package com.intuit.sb.payroll.capability.worker.impl.evaluator;

import com.intuit.sb.payroll.common.AbstractPayrollEntityDataEvaluator;
import com.intuit.sb.payroll.common.PayrollEntityDataState;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Evaluates the WorkLocation to assess it's completeness.
 * 
 * $Author: AStec $
 * $Revision: #1 $
 * $DateTime: 2008/05/19 13:00:31 $
 *
 */
public class WorkLocationEvaluator extends AbstractPayrollEntityDataEvaluator<Worker>
{
	private static WorkLocationEvaluator sInstance = new WorkLocationEvaluator();

	private WorkLocationEvaluator() {}
	
	public static WorkLocationEvaluator getInstance()
	{
		return sInstance ;
	}
	
	/**
	 * Workers are considered complete when there is a liveLocation, workLocation and a hire date.
     * @param worker the Worker for evaluating the work location.
	 */
	public PayrollEntityDataState evaluateState(Worker worker)
	{
	    WorkLocation workLocation = worker.getWorkLocation();
      
		if (workLocation != null && workLocation.getTaxResidenceAddress() != null && workLocation.getTaxResidenceAddress().getCountrySubDivisionCode() != null)
		{
			return PayrollEntityDataState.Complete;
			
		} else
		{
			return PayrollEntityDataState.Incomplete;
		}
	}
}
