package com.intuit.sb.payroll.capability.worker.impl.evaluator;

import java.util.Set;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.AbstractPayrollEntityDataEvaluator;
import com.intuit.sb.payroll.common.PayrollEntityDataEvaluator;
import com.intuit.sb.payroll.common.PayrollEntityDataState;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;


public class WorkerStateWithholdingEvaluator extends AbstractPayrollEntityDataEvaluator<Worker>
{
    private static WorkerStateWithholdingEvaluator sInstance = new WorkerStateWithholdingEvaluator();
    
    private WorkerStateWithholdingEvaluator(){}
    
    public static PayrollEntityDataEvaluator<Worker> getInstance()
    {
        return sInstance ;
    }
    
    @Override
    public PayrollEntityDataState evaluateState(Worker entity) throws ValidationException
    {
        PayrollParamValidator.checkIsNotNull(entity, "Worker entity");
        Set<WorkerComplianceDocDtl> compDataColl = entity.getWorkerComplianceDocDtls();
        if (compDataColl.size() == 0)
        {
            return PayrollEntityDataState.Incomplete; 
        }
        
        for (WorkerComplianceDocDtl employeeWithholdingData : compDataColl) {
        	if (employeeWithholdingData instanceof DE4Detail) {
    			DE4Detail dE4Detail = (DE4Detail) employeeWithholdingData;
    			if ((null != dE4Detail.getNoOfAllowances()) && (dE4Detail.getNoOfAllowances() >= 0)) {
    				return PayrollEntityDataState.Complete;
    			}
    		}
        }
        return PayrollEntityDataState.Incomplete;
        
    }

}
