package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.capability.taxes.withholding.TaxWithholdingsComponent;
import com.intuit.sb.payroll.capability.worker.FindWorkersFilter;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerWorkflowType;
import com.intuit.sb.payroll.capability.worker.impl.WorkerComponentTasks;
import com.intuit.sb.payroll.capability.worker.impl.WorkerSetupRules;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculationRequest;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckInputs;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckUpdateRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunCommitRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunCreateRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInputs;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunUpdateRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ResourceKeyConstants;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.common.management.PayrollService;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.employer.EmployerEAO;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerAdjLineItem;
import com.intuit.sb.payroll.entity.model.EmployerAdjustment;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PaycheckWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;
import com.intuit.sb.payroll.entity.utils.PersonNameUtils;
import com.intuit.sb.payroll.entity.worker.WorkerEAO;
import com.intuit.sb.payroll.entity.workerpayments.PaycheckEAO;
import com.intuit.sb.payroll.entity.workerpayments.PayrollRunEAO;
import com.intuit.sb.payroll.message.PayrollMessageService;

/**
 * Provides Paycheck calculation and Payroll Run facilities.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
@Stateless(name = "WorkerPaymentsComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class WorkerPaymentsComponentBean implements WorkerPaymentsComponent {

	@EJB
	private WorkerComponent workerComponent;
	
	@EJB
	private TaxWithholdingsComponent taxWithholdingsComponent;
	
	@EJB
	private EmployerComponent employerComponent;
	
	@EJB
	private PayrollRunEAO prEAO;
	
	@EJB
	private PaycheckCalculator paycheckCalculator;
	
	@EJB
	private PaycheckEAO paycheckEAO;
	
	@EJB
	private WorkerEAO workerEAO;
	
	@EJB
	private EmployerEAO employerEAO;
	
	@EJB
	private PayrollScheduleComponent payrollScheduleComponent;
	
	@EJB
	private TaskManager taskManager;
	
	//@EJB
	//private WorkerPaymentsDataService workerPaymentsDataService;
	
	@EJB
	private PayrollMessageService messageService;

	private static Logger sLogger = PayrollLoggerFactory.makeLogger(WorkerPaymentsComponentBean.class);
	
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#createNextPayrollRun()
	 */
	public PayrollRun getNextDefaultScheduledPayrollRun() throws ServiceFailureException, ValidationException {
		PayrollSchedule payrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
		if (payrollSchedule != null) {
			PayrollRunInfo nextPayRunInfo = payrollScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
			if (sLogger.isDebugEnabled()) sLogger.debug("Next PayrollRun has been calculated as: " + nextPayRunInfo.toString());
			PayrollRun payrollRun = PayrollRunFactory.createPayrollRunFromPayrollRunInfo(nextPayRunInfo);
			payrollRun.setTypeOf(PayrollRunTypeEnum.SCHEDULED);
	        Employer employer = employerComponent.getEmployer();
	        //FIXME Hack for lazy load.
	        employer.getEmployerComplianceDocDtls();
			try {
				// Get the workers and do some check action.
				List<Worker> workers = workerComponent.findAllWorkers();
				Set<Paycheck> paychecks = new HashSet<Paycheck>();
				for (Worker worker : workers) {
					final boolean workerReadyForPay = WorkerPaymentUtilities.isWorkerReadyForPayment(worker, payrollRun);
					if (workerReadyForPay && worker.isActive()) {
						PaycheckCalculationRequest calcRequest = new PaycheckCalculationRequest();
						calcRequest.setDistributionMethod(worker.getPayrollDistributionMethod());
						calcRequest.setFrequency(payrollSchedule.getFrequency());
						calcRequest.setPaycheckDate(payrollRun.getPayDate());
						calcRequest.setPayPeriod(payrollRun.getPayPeriod());
						calcRequest.setWorkerId(worker.getId());
						//No inputs since we are using the existing pay items on the worker.
						Paycheck paycheck = calculatePaycheckImpl(calcRequest, worker, employer);
						paychecks.add(paycheck);
					}
				}
				payrollRun.setPaychecks(paychecks);
				return payrollRun;
			} catch (Exception e) {
				sLogger.error("Error pre-caculating paycheck", e);
			}		
		}
		return null;
	}
	

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#findPayrollRun(java.lang.String)
	 */
	public PayrollRun findPayrollRun(Long payrollRunId) throws ValidationException, ServiceFailureException {
		PayrollRun payrollRun = retrievePayrollRun(payrollRunId);
        //FIXME Hack for lazy load.
		payrollRun.getPaychecks().size();
		return payrollRun;
	}
	
    /* (non-Javadoc)
     * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#findRecentPayrollRuns(com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum, int)
     */
    public List<PayrollRun> findRecentPayrollRuns(PayrollRunTypeEnum payrollRunType, int numberOfRuns) throws ServiceFailureException, ValidationException
    {
		PayrollParamValidator.checkIsNotNull(numberOfRuns, "numberOfRuns");
		PayrollParamValidator.checkIsPositive(numberOfRuns, "numberOfRuns");
		PayrollParamValidator.checkIsTrue(payrollRunType != PayrollRunTypeEnum.NOTSET, "payrollRunType", "Must not be PayrollRunTypeEnum.NOTSET");
		
		return prEAO.findRecentPayrollRuns(payrollRunType, numberOfRuns);
    }
    
    /* (non-Javadoc)
     * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#findRecentPayrollRuns(com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum, int)
     */
    public List<PayrollRun> findRecentPayrollRunsForYear(PayrollRunTypeEnum payrollRunType, int year) throws ServiceFailureException, ValidationException
    {
		PayrollParamValidator.checkIsNotNull(year, "year");
		PayrollParamValidator.checkIsPositive(year, "year");
		PayrollParamValidator.checkIsTrue(payrollRunType != PayrollRunTypeEnum.NOTSET, "payrollRunType", "Must not be PayrollRunTypeEnum.NOTSET");
		
		return prEAO.findRecentPayrollRunsForYear(payrollRunType, year);
    }
    
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#findAllPayrollRuns()
	 */
	public List<PayrollRun> findAllPayrollRuns() {
		return prEAO.findAllDescending();
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#deletePayrollRun(com.intuit.sb.payroll.entity.model.PayrollRun)
	 * FIXME this implementation is not complete.  Should not be able to delete a committed pay run, for example
	 */
	public void deletePayrollRun(Long payrollRunId) throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(payrollRunId, "payrollRunId");
		PayrollRun payrollRun = retrievePayrollRun(payrollRunId);
		//FIXME [model] the employer should be seen from adjustment?
        Employer employer = employerComponent.getEmployer();

		for (Paycheck paycheck : payrollRun.getPaychecks()) {
			// Remove associations from workers to their paychecks in the run
			Worker worker = paycheck.getWorker();
			worker.removePaycheck(paycheck);
			for (PaycheckLineItem paycheckLineItem : paycheck.getPaycheckLineItems()) {
				paycheckLineItem.setSourcePayrollItemDtl(null);
			}

			// Remove Line items from the paycheck
			paycheck.removeAllPaycheckLineItems();
		}
		
		//Remove assocations from the adjustment.
		for (EmployerAdjustment employerAdjustment : payrollRun.getEmployerAdjustments()) {
			employer.removeEmployerAdjustment(employerAdjustment);
			for (EmployerAdjLineItem employerAdjLineItem : employerAdjustment.getEmployerAdjLineItems()) {
				employerAdjLineItem.setPayrollItem(null);
			}
			employerAdjustment.removeAllEmployerAdjLineItems();
		}
		
		prEAO.makeTransient(payrollRun);
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#cancelPayrollRunById(java.lang.Long)
	 * FIXME this implementation is not complete either.  Does cancel mean delete or just mark as cancelled?  What about
	 * the paychecks? They should be marked as cancelled or deleted. Not sure the cascade here is for free.
	 */
	public void cancelPayrollRunById(Long payrollRunId) throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(payrollRunId, "payrollRunId");
		PayrollRun payrollRun = retrievePayrollRun(payrollRunId);

		if (PayrollRunWorkflowStatusEnum.COMMITTED.equals(payrollRun.getWorkflowStatus())) {
			throw new ValidationException("PayrollRunId:" + payrollRunId + "is already committed");
		}
		deletePayrollRun(payrollRunId);
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#createPayrollRun(com.intuit.sb.payroll.capability.workerpayments.PayrollRunCreateRequest)
	 */
	public PayrollRun createPayrollRun(PayrollRunCreateRequest request) throws ServiceFailureException, ValidationException {
		PayrollRunCreateRequestValidator.getInstance().validateEntity(request);
		PayrollRun payrollRun = PayrollRunFactory.createPayrollRunFromPayrollRunInfo(request.getPayrollRunInfo());
        Employer employer = employerComponent.getEmployer();
        //FIXME Hack for lazy load.
        employer.getEmployerComplianceDocDtls();

		Set<Paycheck> paychecks = new HashSet<Paycheck>();
		for (Long workerId : request.getWorkerIdList()) {
			Worker worker = workerComponent.findWorker(workerId);
			final boolean workerReadyForPay = WorkerPaymentUtilities.isWorkerReadyForPayment(worker, payrollRun);
			if (workerReadyForPay) {
				PaycheckCalculationRequest calcRequest = new PaycheckCalculationRequest();
				calcRequest.setDistributionMethod(worker.getPayrollDistributionMethod());
				calcRequest.setFrequency(request.getPayrollRunInfo().getFrequency());
				calcRequest.setPartialPayPeriod(request.getPayrollRunInfo().isPartialPayPeriod());
				calcRequest.setPaycheckDate(payrollRun.getPayDate());
				calcRequest.setPayPeriod(payrollRun.getPayPeriod());
				calcRequest.setWorkerId(worker.getId());

				Paycheck paycheck = calculatePaycheckImpl(calcRequest, worker, employer);
				paychecks.add(paycheck);
			} else {
				String mesg = MessageFormat.format("Unable to calculate paycheck for {0}.  They are not Payroll Ready", 
						PersonNameUtils.getFullName(worker.getPersonName()));
				throw new ValidationException(mesg);
			}
		}
		payrollRun.setPaychecks(paychecks);
		//Totals
		WorkerPaymentUtilities.updatePayrollRunTotals(payrollRun);
		return payrollRun;
	}
	
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#savePayrollRun(com.intuit.sb.payroll.entity.model.PayrollRun)
	 */
	public PayrollRun commitPayrollRun(PayrollRunCommitRequest request) throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollRunRequestValidator.getInstance().validateEntity(request);

		List<PaycheckCalculationRequest> calculationRequests = request.getCalculationRequests();
		PayrollParamValidator.checkIsNotNull(calculationRequests, "calculationRequests");
        Employer employer = employerComponent.getEmployer();
        //FIXME Hack for lazy load.
        employer.getEmployerComplianceDocDtls();

        final PayrollRunInputs payRunInputs = new PayrollRunInputs(employer, request.getPayrollRunInfo());

        final Set<PaycheckInputs> paycheckInputSet = new HashSet<PaycheckInputs>();
		
		for (PaycheckCalculationRequest paycheckCalculationRequest : calculationRequests) {
		    PaycheckCalculationRequestValidator.validatePaycheckCalculationRequest(paycheckCalculationRequest);
	        Worker worker = workerComponent.findWorker(paycheckCalculationRequest.getWorkerId());
	        this.checkState(worker, paycheckCalculationRequest);
            final PaycheckInputs paycheckInputs = createPaycheckInputs(paycheckCalculationRequest, worker);
            paycheckInputSet.add(paycheckInputs);
		}

		PayrollRun payrollRun = paycheckCalculator.calculatePayrollRun(payRunInputs, paycheckInputSet);
		payrollRun = prEAO.makePersistent(payrollRun);

		final Set<Paycheck> paychecks = payrollRun.getPaychecks();
		fixupPaycheckAssociationAndStatus(paychecks, PaycheckWorkflowStatusEnum.COMMITTED);
		
		final List<Worker> workers = workerComponent.findAllWorkers();
		//We do this now but it may/should be policy-based.
		PayrollAdjustmentsController adjustmentController = new PayrollAdjustmentsController(employer, workers, payrollRun, this.paycheckCalculator);
		adjustmentController.adjustForRounding();
		
		//Totals
		WorkerPaymentUtilities.updatePayrollRunTotals(payrollRun);

		payrollRun.setWorkflowStatus(PayrollRunWorkflowStatusEnum.COMMITTED);
		payrollRun.setDescription(request.getPayrollRunInfo().getDescription());

		if (request.getPayrollRunInfo().getPayrollRunType() == PayrollRunTypeEnum.SCHEDULED) {
			dispatchWorkflowEvent(WorkflowEventType.COMMIT_SCHEDULED_PAYROLL_RUN, payrollRun);
		} else if (request.getPayrollRunInfo().getPayrollRunType() == PayrollRunTypeEnum.TERMINATION) {
			dispatchWorkflowEvent(WorkflowEventType.COMMIT_TERMINATION_PAYROLL_RUN, payrollRun);
		} else if (request.getPayrollRunInfo().getPayrollRunType() == PayrollRunTypeEnum.ADHOC) {
			dispatchWorkflowEvent(WorkflowEventType.COMMIT_ADHOC_PAYROLL_RUN, payrollRun);
		}
		//etc... add any other types of workflows if components required specific types (eg, Termination, AdHoc)
		return payrollRun;
	}


	private void fixupPaycheckAssociationAndStatus(Set<Paycheck> paychecks, PaycheckWorkflowStatusEnum status) {
		for (Paycheck paycheck : paychecks) {
			paycheck.setWorkflowStatus(status);
			//Associations
			paycheck.getWorker().addPaycheck(paycheck); // sets both ends of the bi-di relationship
	
			//Status
			paycheck.setWorkflowStatus(status);
		}
	}

	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#updatePayrollRun(com.intuit.sb.payroll.capability.workerpayments.PayrollRunUpdateRequest)
	 */
	public PayrollRun updatePayrollRun(PayrollRunUpdateRequest updateRequest) throws ValidationException, ServiceFailureException {
		PayrollRunUpdateRequestValidator.validatePayrollRunUpdateRequest(updateRequest);
		PayrollRun payrollRun = retrievePayrollRun(updateRequest.getPayrollRunId());
		Set<Paycheck> paychecks = payrollRun.getPaychecks();
		Map<Long, Paycheck> paycheckMap = new HashMap<Long, Paycheck>();
		for (Paycheck paycheck : paychecks) {
			paycheckMap.put(paycheck.getId(), paycheck);
		}
		// Update the three fields in the paychecks we are allowed to.
		List<PaycheckUpdateRequest> paycheckUpdateRequests = updateRequest.getPaycheckUpdateRequests();
		for (PaycheckUpdateRequest paycheckUpdateRequest : paycheckUpdateRequests) {
			Paycheck paycheckToUpdate = paycheckMap.get(paycheckUpdateRequest.getPaycheckId());
			if (paycheckToUpdate != null) {
				paycheckToUpdate.setTxnNo(paycheckUpdateRequest.getCheckNumber());
				paycheckToUpdate.setMemo(paycheckUpdateRequest.getMemo());
				paycheckToUpdate.setWorkflowStatus(paycheckUpdateRequest.getPaycheckStatus());
			} else {
				throw new ValidationException("Can't update a paycheck that isn't committed");
			}
		}
		
		dispatchWorkflowEvent(WorkflowEventType.PAYROLL_RUN_UPDATED, payrollRun);
		return payrollRun;
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#voidPaychecks(com.intuit.sb.payroll.capability.workerpayments.VoidPaychecksRequest)
	 */
	public PayrollRun voidPaychecks(List<Long> paycheckIds) throws ValidationException, ServiceFailureException {
		VoidPaychecksRequestValidator.validateVoidPaychecksRequest(paycheckIds);
		long payrollRunIdNotSet = -1;
		long payrollRunId = payrollRunIdNotSet;
		// Void the paychecks
		Paycheck paycheckToVoid = null;
		for (Long paycheckId : paycheckIds) {
			paycheckToVoid = getPaycheckById(paycheckId);
			if (paycheckToVoid.getWorkflowStatus() != PaycheckWorkflowStatusEnum.COMMITTED &&
				paycheckToVoid.getWorkflowStatus() != PaycheckWorkflowStatusEnum.DISTRIBUTED) {
				String mesg = MessageFormat.format("Paycheck status ''{0}'' on paycheck with id ''{1}'' is inappropriate for voiding.",
						paycheckToVoid.getWorkflowStatus().toString(), paycheckToVoid.getId());
				throw new ValidationException(mesg);
			}

			long thisPayrollRunId = paycheckToVoid.getPayrollRun().getId();
			if (thisPayrollRunId != payrollRunId && payrollRunId != payrollRunIdNotSet) {
				throw new ValidationException("paycheckIds cannot belong to multiple payroll runs: [" +
						thisPayrollRunId + "," + payrollRunId + "]");
			}
			payrollRunId = thisPayrollRunId;
			paycheckToVoid.setWorkflowStatus(PaycheckWorkflowStatusEnum.VOIDED);
			paycheckToVoid.setBeingVoided(true);
		}
		
		//FIXME we could just get this from the Paycheck?
		PayrollRun payrollRun = retrievePayrollRun(payrollRunId);
		
		//Totals
		WorkerPaymentUtilities.updatePayrollRunTotals(payrollRun);

		dispatchWorkflowEvent(WorkflowEventType.PAYROLL_RUN_PAYCHECKS_VOIDED, payrollRun);
		return payrollRun;
	}

	/* ===================== Paycheck Calculation  ===========================*/
	
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#calculatePaycheck(com.intuit.sb.payroll.entity.model.Worker, com.intuit.sb.payroll.entity.model.DateRange, java.util.Date, java.util.List)
	 */
	public Paycheck calculatePaycheck(PaycheckCalculationRequest request) throws WorkflowException, ValidationException, ServiceFailureException {
	    PaycheckCalculationRequestValidator.validatePaycheckCalculationRequest(request);
        Worker worker = workerComponent.findWorker(request.getWorkerId());
        this.checkState(worker, request);
        Employer employer = employerComponent.getEmployer();
        //FIXME Hack for lazy load.
        employer.getEmployerComplianceDocDtls();
		Paycheck paycheck = calculatePaycheckImpl(request, worker, employer);
		return paycheck;
	}

	/**
	 * For internal use only, this can be called after we know that the worker is properly validated and in the right
	 * Workflow state.  
	 * 
	 * @param request
	 * @param worker
	 * @return
	 * @throws ServiceFailureException
	 */
	private Paycheck calculatePaycheckImpl(PaycheckCalculationRequest request, Worker worker, Employer employer)
			throws ServiceFailureException {
		try {			
            final PaycheckInputs paycheckInputs = createPaycheckInputs(request, worker);
			final PayrollRunInputs payRunInputs = createPayrollRunInputs(request, employer);
			final Paycheck paycheck = paycheckCalculator.calculatePaycheck(payRunInputs, 	paycheckInputs);
			return paycheck;
		} catch (Exception e) {
				String mesg = MessageFormat.format("Unable to calculate paycheck for {0}.  {1}", 
												PersonNameUtils.getFullName(worker.getPersonName()),
												e.getMessage());
				throw new ServiceFailureException(mesg, e);
		}
	}

	private PayrollRunInputs createPayrollRunInputs(PaycheckCalculationRequest request, Employer employer) {
		final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(request.getPayPeriod(), 
																request.getPaycheckDate(), 
																PayrollService.getInstance().getSystemDate().getTime(), 
																request.getFrequency(), 
																null,
																request.isPartialPayPeriod());
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(employer, payrollRunInfo);
		return payRunInputs;
	}

	private PaycheckInputs createPaycheckInputs(PaycheckCalculationRequest request, Worker worker)
			throws ServiceFailureException, WorkflowException, ValidationException {
		// Make sure the withholdings are set - if not this method will get defaults - this is US-specific code.
		List<WorkerComplianceDocDtl> withholdings = taxWithholdingsComponent.getCurrentAndDerivedWithholdings(worker);
		worker.setWithholdingDocsForCalc(withholdings);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(worker, 
																request.getCalculationInputs(), 
																request.getDistributionMethod(), 
																//FIXME this is not a paycheck attribute but rather a payroll run.  Change later.
																request.isPartialPayPeriod());
		return paycheckInputs;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#getPaycheckById(java.lang.Long)
	 */
	public Paycheck getPaycheckById(Long paycheckId) throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(paycheckId, "payrollRunId");
		Paycheck paycheck = null;
		try {
			paycheck = paycheckEAO.findById(paycheckId);
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}
		if (null == paycheck) {
			throw new ServiceFailureException(PayrollResourceManager.getInstance()
					.getString(ResourceKeyConstants.ID_NOT_FOUND_EXCEPTION)
					+ paycheckId.toString());
		}
		return paycheck;
	}
	


	/**
	 * Ensures we don't go calculating on a dude who's not ready to pay.
	 * 
	 * @param worker
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 * @throws WorkflowValidationException
	 */
    private void checkState(Worker worker, PaycheckCalculationRequest request) throws WorkflowException, ValidationException, ServiceFailureException {
        if (!worker.isReadyForPay()) {
        	ExceptionUtils.generateWorkflowExceptionFromRequiredData(WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.Payroll, worker));
        }
        final boolean workerHireDatePriorToPeriodEnd = 
			DateUtil.isBeforeOrOnSameDay(worker.getEmploymentPeriod().getStartDate(), 
														request.getPayPeriod().getEndDate());
        if (!workerHireDatePriorToPeriodEnd) {
        	throw new ValidationException("Worker Hire date is after the end of the payperiod, and cannot be included in this payroll.");
        }
    }
    
    private PayrollRun retrievePayrollRun(Long payrollRunId) throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(payrollRunId, "payrollRunId");
		PayrollRun payrollRun = null;
		try {
			payrollRun = prEAO.findById(payrollRunId);
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}
		if (null == payrollRun) {
			throw new ServiceFailureException(PayrollResourceManager.getInstance()
					.getString(ResourceKeyConstants.ID_NOT_FOUND_EXCEPTION)
					+ payrollRunId.toString());
		}
		return payrollRun;
    }
    
    /**
     * Sink for workflow events in this component.  Handles side-effects that are germain to this component, then
     * propagates the event to the Messaging infrastructure.
     * 
     * @param workflowEventType
     * @param payrollRun
     * @throws ServiceFailureException
     * @throws ValidationException 
     */
	private void dispatchWorkflowEvent(WorkflowEventType workflowEventType, PayrollRun payrollRun) throws ServiceFailureException, ValidationException {
		switch (workflowEventType) {
			case COMMIT_SCHEDULED_PAYROLL_RUN:
				PayrollSchedule payrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
				//Close the task associated with this run
				WorkerPaymentsComponentTasks.closeScheduledPayrollRunTask(taskManager, payrollRun, payrollSchedule);
				//When there are checks that are not in DISTRIBUTED status we create a reminder to finish distribution.
				if (!WorkerPaymentUtilities.distributionComplete(payrollRun)) {
					WorkerPaymentsComponentTasks.createFinishPaycheckDistributionTask(taskManager, payrollRun);
				}
				//Create the task for the next scheduled one.
				PayrollRunInfo payrollRunInfo = payrollScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
				WorkerPaymentsComponentTasks.createNextScheduledPayrollRunTask(taskManager, payrollRunInfo, payrollSchedule);
				break;
			case PAYROLL_RUN_UPDATED:
				// When all the checks are in DISTRIBUTED status we can close any tasks that might be open to do so.
				if (WorkerPaymentUtilities.distributionComplete(payrollRun)) {
					WorkerPaymentsComponentTasks.closeFinishPaycheckDistributionTask(taskManager, payrollRun);
				}
				break;
			case COMMIT_TERMINATION_PAYROLL_RUN:
				//When there are checks that are not in DISTRIBUTED status we create a reminder to finish distribution.
				if (!WorkerPaymentUtilities.distributionComplete(payrollRun)) {
					WorkerPaymentsComponentTasks.createFinishPaycheckDistributionTask(taskManager, payrollRun);
				}
				break;
			case COMMIT_ADHOC_PAYROLL_RUN:
				if (!WorkerPaymentUtilities.distributionComplete(payrollRun)) {
					WorkerPaymentsComponentTasks.createFinishPaycheckDistributionTask(taskManager, payrollRun);
				}
				break;
			case PAYROLL_RUN_PAYCHECKS_VOIDED:
				//Deal with adjustments and stale cached totals.
				handleVoid(payrollRun);
				break;
			default:
				break;
		}
		//Publish event to interested parties 
		messageService.dispatchPayrollRunEvent(workflowEventType, payrollRun);
	}
    

	/**
	 * Handles side effects from a Void workflow.  Includes updating the totals on the affected worker, adjustments
	 * at the worker and company level.
	 * 
	 * @param payrollRun
	 */
	private void handleVoid(PayrollRun payrollRun) throws ServiceFailureException {
//		List<Worker> affectedWorkers = new ArrayList<Worker>();
		for(Paycheck paycheck : payrollRun.getPaychecks()) {
			if (paycheck.isBeingVoided()) {
				Worker worker = paycheck.getWorker();
				//Stale caches get updated.
				WorkerPaymentUtilities.updatePtdsForWorkerAfterVoid(worker, paycheck);
//				affectedWorkers.add(worker);
			}
		}
		
//        final Employer employer = employerComponent.getEmployer();
//		final List<Worker> workers = workerComponent.findAllWorkers();
//        
//       //Get the Adjustments figured out
//		PayrollAdjustmentsController controller = new PayrollAdjustmentsController(employer, workers, payrollRun, paycheckCalculator);
//		controller.adjustForVoid(affectedWorkers);

		// Close any no-longer-needed "Finish distribution" task
		if (WorkerPaymentUtilities.distributionComplete(payrollRun)) {
			WorkerPaymentsComponentTasks.closeFinishPaycheckDistributionTask(taskManager, payrollRun);
		}
	}


	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#findMostRecentPaycheck(com.intuit.sb.payroll.entity.model.Worker)
	 */
	public Paycheck findMostRecentPaycheck(Worker worker) throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		PayrollParamValidator.checkIsNotNull(worker.getId(), "worker.id");
		Paycheck paycheck = null;
		try {
			paycheck = paycheckEAO.findMostRecentPaycheckForWorker(worker);
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}
		return paycheck;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#handlePayrollScheduleEvent(com.intuit.sb.payroll.message.WorkflowEventType, com.intuit.sb.payroll.entity.model.PayrollSchedule)
	 */
	public void handlePayrollScheduleEvent(WorkflowEventType workflowEventType, PayrollSchedule payrollSchedule) throws ServiceFailureException {
		try {
			List<Worker> workers = workerComponent.findWorkersByFilter(FindWorkersFilter.READY_FOR_PAYROLL);
			if (WorkflowEventType.ADD_PAYROLL_SCHEDULE == workflowEventType) {				
				if (workers.size() > 0) {
					PayrollRunInfo payrollRunInfo = payrollScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
					WorkerPaymentsComponentTasks.createNextScheduledPayrollRunTask(taskManager, payrollRunInfo, payrollSchedule);
				}
			} else if (WorkflowEventType.UPDATE_PAYROLL_SCHEDULE == workflowEventType) {
				WorkerPaymentsComponentTasks.updatePayrollRunTasksForSchedule(taskManager, payrollSchedule);
				if (workers.size() > 0) {
					PayrollRunInfo payrollRunInfo = payrollScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
					WorkerPaymentsComponentTasks.createNextScheduledPayrollRunTask(taskManager, payrollRunInfo, payrollSchedule);
				}
			} else if (WorkflowEventType.DELETE_PAYROLL_SCHEDULE == workflowEventType) {
				WorkerPaymentsComponentTasks.removePayrollRunTasksForSchedule(taskManager, payrollSchedule);
			}
		} catch (ValidationException e) {
			//This comes from getting the default schedule.  If there's a problem, we have nothing to do here.
			sLogger.error(e.getLocalizedMessage(), e);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent#handleWorkerEvent(com.intuit.sb.payroll.message.WorkflowEventType, com.intuit.sb.payroll.entity.model.Worker)
	 */
	public void handleWorkerEvent(WorkflowEventType workflowEventType, Worker worker) throws ServiceFailureException {
		try {
			if (WorkflowEventType.ADD_WORKER == workflowEventType || WorkflowEventType.UPDATE_WORKER == workflowEventType) {
				if (worker.isReadyForPay()) {
					// If the schedule is there we are ready for payroll (unless we already are)
					PayrollSchedule payrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
					if (payrollSchedule != null) {
						// If there is no existing payroll run task, add it.
						if (!WorkerPaymentsComponentTasks.payrollRunTaskExists(taskManager, payrollSchedule)) {
							PayrollRunInfo payrollRunInfo = payrollScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
							WorkerPaymentsComponentTasks.createNextScheduledPayrollRunTask(taskManager, payrollRunInfo, payrollSchedule);
						}
					}
				}
			}
			else if (WorkflowEventType.DELETE_WORKER == workflowEventType || WorkflowEventType.TERMINATE_WORKER == workflowEventType) {
				if(workerEAO.getWorkerCount() == 0) {
					Employer employer = employerEAO.findCurrentEmployer();
					WorkerComponentTasks.createAddEmployeeTask(taskManager, employer);					
				}				
				if(workerEAO.findWorkersReadyForPayroll().size() == 0) {
					PayrollSchedule payrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
					if(payrollSchedule != null) {
						WorkerPaymentsComponentTasks.removePayrollRunTasksForSchedule(taskManager, payrollSchedule);
					}
				}
			}
		} catch (ValidationException e) {
			//This comes from getting the default schedule.  If there's a problem, we have nothing to do here.
			sLogger.error(e.getLocalizedMessage(), e);
		}
	}
}