package com.intuit.sb.payroll.intl.capability.address;


import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;

/**
 * Utility Class to Convert To And From GlobalPostalAddress to Locale Specific PostalAddress
 * @author rmahadevan
 *
 */
public class AddressConvertor 
{
	private AddressConvertor()
	{
		//No instances
	}
	
	
	/**
	 * Converts a GlobalPostalAddress to USPostalAddress. The destination object is mutated and also returned in case
	 * chaining is desired. Null input => Null output
	 * @param src The Address To Convert From
	 * @param dest The Address To Convert To
	 * @return the Converted Address
	 */
	public static USPostalAddress getUSFromGlobal(GlobalPostalAddress src, USPostalAddress dest)
	{
		if(src != null)
		{
			if(dest != null)
			{
				dest.setAddressLine1(src.getAddressLineOne());
				dest.setAddressLine2(src.getAddressLineTwo());
				dest.setAddressLine3(src.getAddressLineThree());
				dest.setCity(src.getPostTown());
				dest.setCounty(src.getCounty());
				dest.setCountyCode(src.getCountyCode());
				dest.setRegionOrState(src.getCountrySubDivisionCode());
				dest.setZipOrPostalCode(src.getPostalCode());					
				dest.setCountry(src.getCountryName());
				dest.setCountryISOCode(src.getCountryCode());
				dest.setGeoCode(src.getGeoCode());
				dest.setIsNotToBeStandardized(src.getIsNotToBeStandardized());
				dest.setIsStandardized(src.getIsStandardized());
				dest.setIsStreetAddress(src.getIsStreetAddress());	
				return dest;	
				
			}
		}
		return null;

			
	}
	
	/**
	 * Converts a USPostalAddress to GlobalPostalAddress. The destination object is mutated and also returned in case
	 * chaining is desired. Null input => Null output
	 * @param src The Address To Convert From
	 * @param dest The Address To Convert To
	 * @return the Converted Address
	 */
	
	public static GlobalPostalAddress getGlobalFromUS(USPostalAddress src, GlobalPostalAddress dest )
	{
		if(src != null)
		{
			if(dest != null)
			{
				dest.setAddressLineOne(src.getAddressLine1());
				dest.setAddressLineTwo(src.getAddressLine2());
				dest.setAddressLineThree(src.getAddressLine3());
				dest.setPostTown(src.getCity());		
				dest.setCounty(src.getCounty());
				dest.setCountyCode(src.getCountyCode());
				dest.setCountrySubDivisionCode(src.getRegionOrState());
				dest.setPostalCode(src.getZipOrPostalCode());
				dest.setCountryName(src.getCountry());
				dest.setCountryCode(src.getCountryISOCode());
				dest.setGeoCode(src.getGeoCode());
				dest.setIsNotToBeStandardized(src.getIsNotToBeStandardized());
				dest.setIsStandardized(src.getIsStandardized());
				dest.setIsStreetAddress(src.getIsStreetAddress());	
				return dest;				
			}
		}
		return null;				
	}

}
