package com.intuit.sb.payroll.calcengine;

import org.testng.annotations.BeforeMethod;

import com.intuit.sb.payroll.entity.model.DisabilityInsDocDtl;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.UnemploymentInsDocDtl;

/**
 * Base test class for testing calc engine integration.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class CalcEngineTestBase extends PaycheckCalcTestBase {

    private static final Double sCASUIRate = 3.4;  // This is the common new employer rates for sTaxYear in percentage
    private static final Double sCASDIRate = 0.8;  // This is the rate for sTaxYear in percentage
    private static final Double sTXSUIRate = 2.7;  // This is the common new employer rates for sTaxYear in percentage
    
    private boolean isSetUp = false;
   
    @BeforeMethod
    public void setUp() throws Exception {
        super.setUp();
        if ( ! isSetUp ) {
        	isSetUp = true;
            mEmployer = sEmployerEAO.findCurrentEmployer();
            mEmployer.setFirstPayrollDate(sCompanyStartDate.getTime());
            setupEmployerComplianceInfo(mEmployer);
        }
    }
    
    private void setupEmployerComplianceInfo(Employer employer) {
        // Add CA unemployment Insurance rate info
        UnemploymentInsDocDtl caUnemploymentInsDtl = new UnemploymentInsDocDtl();
        caUnemploymentInsDtl.setPercentageRate(sCASUIRate);
        caUnemploymentInsDtl.getLawId().setValue(PayrollItemType.CASUI.getLawId());
        employer.addEmployerComplianceDocDtl(caUnemploymentInsDtl);
        // Add CA Disability Insurance rate info
        DisabilityInsDocDtl caDisabilityInsuranceDtl = new DisabilityInsDocDtl();
        caDisabilityInsuranceDtl.setPercentageRate(sCASDIRate);
        caDisabilityInsuranceDtl.getLawId().setValue(PayrollItemType.CASDI.getLawId());
        employer.addEmployerComplianceDocDtl(caDisabilityInsuranceDtl);
        // Add TX unemployment Insurance rate info
        UnemploymentInsDocDtl txUnemploymentInsDtl = new UnemploymentInsDocDtl();
        txUnemploymentInsDtl.setPercentageRate(sTXSUIRate);
        txUnemploymentInsDtl.getLawId().setValue(PayrollItemType.TXSUI.getLawId());
        employer.addEmployerComplianceDocDtl(txUnemploymentInsDtl);
    }
   
}
