package com.intuit.sb.payroll.calcengine;

import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;



//TODO Should we have a PaidBy type in this Enum
//New Typesafe Enum, the ids are the Law Ids followed by compliance as well as QBOE.
//The other ids like Salary are from QBOE codes only

enum PayrollItemType
{
    SALARIED(QboeLawIdConstants.sSalaryLawId, "Salary"),
    HOURLY(QboeLawIdConstants.sHourlyLawId, "Hourly"),
    BONUS(QboeLawIdConstants.sBonusLawId, "Bonus"),
    TIPS_PAYCHECK(QboeLawIdConstants.sPaycheckTipsLawId, "Paycheck Tips"),
    TIPS_CASH(QboeLawIdConstants.sCashTipsLawId, "Cash Tips"),
    TIPS_CASHOFFSET(QboeLawIdConstants.sCashTipsOutLawId, "Cash Tips Offset"),
    TIPS_ALLOCATED(QboeLawIdConstants.sAllocatedTipsLawId, "Allocated Tips"),
    OVERTIME(QboeLawIdConstants.sOvertimeLawId, "Overtime"),
    DOUBLETIME(QboeLawIdConstants.sDoubleTimeLawId, "Doubletime"),
    SICK_ACCRUE(QboeLawIdConstants.sSickTimeAccrualLawId, "Sick Accrue"),
    SICK_DISBURSE(QboeLawIdConstants.sSickTimeDisbursementLawId, "Sick Disburse"),
    SICK_PAYOUT(QboeLawIdConstants.sSickTimePayoutLawId, "Sick Payout"),
    COMMISSION(QboeLawIdConstants.sCommissionLawId, "Commission"),
    VACATION_ACCRUE (QboeLawIdConstants.sVacationAccrualLawId, "Vacation Accrue"),
    VACATION_DISBURSE (QboeLawIdConstants.sVacationDisbursementLawId, "Vacation Disburse"),
    VACATION_PAYOUT (QboeLawIdConstants.sVacationPayoutLawId, "Vacation Payout"),
    FOUR01K (QboeLawIdConstants.s401KId, "401K"),
    FOUR01KMATCH (QboeLawIdConstants.s401KMatchId, "401K Co. Match"),
    SEC125HEALTH (QboeLawIdConstants.sSec125HealthId, "Pre-Tax Employee-Paid Health"),
    MILEAGEREIMBURSEMENT(QboeLawIdConstants.sMileageReimbursementId, "Mileage Reimbursement"),
    PIECEWORK(QboeLawIdConstants.sPieceworkLawId, "Piecework"),
    USFUI(QboeLawIdConstants.sFUTALawId, "Federal Unemployment"),
    USFIT(QboeLawIdConstants.sFICALawId, "Federal Income Tax"),
    USSSTE(QboeLawIdConstants.sSST_EELawId, "Social Security Employee"),
    USSSTER(QboeLawIdConstants.sSST_ERLawId, "Social Security Company"),
    USMEDTE(QboeLawIdConstants.sMED_EELawId,"Medicare Employee"),
    USMEDTER(QboeLawIdConstants.sMED_ERLawId,"Medicare Company"),
    USAEIC(QboeLawIdConstants.sAeicId,"Advance Earned Income Credit"),
    CAETT(QboeLawIdConstants.sCAETTLawId, "CA ETT"),
    CASUI(QboeLawIdConstants.sCASUILawId, "CA - Unemployment"),
    CAPIT(QboeLawIdConstants.sCAPITLawId, "CA - Income Tax"),
    CASDI(QboeLawIdConstants.sCASDILawId, "CA - Disability"),
    TXSUI(QboeLawIdConstants.sTXSUILawId, "TX - Unemployment");
    
    private String lawId;
    private String lawLabel;
    
    private PayrollItemType(String lawId, String lawLabel)
    {
        this.lawId = lawId;
        this.lawLabel = lawLabel;       
    }
   
    public String getLawId()
    {
        return lawId;
    }
    
    public String getLawLabel()
    {
        return lawLabel;
    }
    
    /**
     * Will return a null if no matching id is found
     * @param lawId
     * @return
     */
    public static PayrollItemType parseLawId(String lawId)
    {
        for (PayrollItemType p : PayrollItemType.values())
        {
            if(p.lawId.equals(lawId))
            {
                return p;
            }
        }
        return null;
    }
}
