package com.intuit.sb.payroll.capability.benefits;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.entity.domain.EntityId;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Tests the HealthBenefits operations of the BenefitsComponentBean.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class BenefitsComponentTest extends BasicTestBase {
	    
	private List<HealthBenefitDtl> healthBenefitsDtlsList;
    	
	@BeforeMethod
	public void initializeBenefitComponentTest()
	{
		healthBenefitsDtlsList = new ArrayList<HealthBenefitDtl>();
	}
	
	@Test
	public void testCreateHealthBenefitDtl() throws Exception {
    	HealthBenefitTypeEnum healthBenefitTypeEnum = HealthBenefitTypeEnum.SEC125HEALTH;
    	HealthBenefitDtl healthBenefitDtl = benefitsComponent.createHealthBenefitDtl(healthBenefitTypeEnum);
    	AssertJUnit.assertNotNull(healthBenefitDtl);
	}
    
    @Test (expectedExceptions=ValidationException.class)
    public void testCreateHealthBenefitDtlWithNull() throws Exception {
        @SuppressWarnings("unused")
        HealthBenefitDtl healthBenefitDtl = benefitsComponent.createHealthBenefitDtl(null);
    }
    
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testCreateHealthBenefitDtlWithUnsupportedType() throws Exception {
            @SuppressWarnings("unused")
            HealthBenefitDtl healthBenefitDtl = benefitsComponent.createHealthBenefitDtl(HealthBenefitTypeEnum.SEC125OTHER);
    }
    
    @Test
    public void testAddHealthBenefitDtls() throws Exception {
    	
    	createHealthBenefitDtls();
    	Worker eiWorker = benefitsComponent.addHealthBenefitDtls(sWorker, healthBenefitsDtlsList);
    	Set<HealthBenefitDtl> returnedHealthBenefitDtlSet = eiWorker.getHealthBenefitDtls();
    	List<HealthBenefitDtl> returnedList = new ArrayList<HealthBenefitDtl>(returnedHealthBenefitDtlSet);
    	AssertJUnit.assertEquals(returnedList, healthBenefitsDtlsList);
    	
    }
    
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testFindHealthBenefitDtlWithNonExistentId() throws Exception {
        EntityId entityId = new EntityId(793493943L, 0L);
        @SuppressWarnings("unused")
        HealthBenefitDtl healthBenefitDtl = benefitsComponent.findHealthBenefitDtl(entityId);
    }
    
    @Test (expectedExceptions=ValidationException.class)
    public void testDeleteHealthBenefitDtlWithNullWorker() throws Exception {
        @SuppressWarnings("unused")
        Worker eiWorker = benefitsComponent.deleteHealthBenefitDtl(null, new HealthBenefitDtl());
    }
    
    @Test (expectedExceptions=ValidationException.class)
    public void testDeleteHealthBenefitDtlWithNullHealthBenefitDtl() throws Exception {
        @SuppressWarnings("unused")
        Worker eiWorker = benefitsComponent.deleteHealthBenefitDtl(sWorker, null);
    }
    
    @Test (expectedExceptions=ValidationException.class)
    public void testAddHealthBenefitDtlsWithNullHealthBenefitDtlSet() throws Exception {
        @SuppressWarnings("unused")
        Worker eiWorker = benefitsComponent.addHealthBenefitDtls(sWorker, null);
    }
    
    @Test
    public void testDeleteHealthBenefitDtls() throws Exception {
    	
    	createHealthBenefitDtls();
    	Worker eiWorker = benefitsComponent.addHealthBenefitDtls(sWorker, healthBenefitsDtlsList);
		Set<HealthBenefitDtl> returnedHealthBenefitDtlSet = eiWorker.getHealthBenefitDtls();
		List<HealthBenefitDtl> returnedList = new ArrayList<HealthBenefitDtl>(returnedHealthBenefitDtlSet);
		AssertJUnit.assertEquals(returnedList, healthBenefitsDtlsList);
    	List<HealthBenefitDtl> benefitsToDelete = new ArrayList<HealthBenefitDtl>(returnedHealthBenefitDtlSet);
    	Worker eiWorker2 = benefitsComponent.deleteHealthBenefitDtls(sWorkerId, benefitsToDelete);
    	Set<HealthBenefitDtl> retAfterDeleteHealthBenefitDtlSet = eiWorker2.getHealthBenefitDtls();
    	AssertJUnit.assertEquals(retAfterDeleteHealthBenefitDtlSet.size(), 0);
    	
    }
    
    @Test
    public void testUpdateHealthBenefitDtls() throws Exception {
    	
    	createHealthBenefitDtls();
    	Worker eiWorker = benefitsComponent.addHealthBenefitDtls(sWorker, healthBenefitsDtlsList);
		Set<HealthBenefitDtl> returnedHealthBenefitDtlSet = eiWorker.getHealthBenefitDtls();
		List<HealthBenefitDtl> returnedList = new ArrayList<HealthBenefitDtl>(returnedHealthBenefitDtlSet);
		AssertJUnit.assertEquals(returnedList, healthBenefitsDtlsList);
    	
    	// Now update some data in one of the healthBenefitDtl
    	// Currently the HealthBenefitDtlMap contains only one HealthBenefitDtl
    	for (HealthBenefitDtl existingItem : returnedHealthBenefitDtlSet)
    	{
    	    Datum datum = existingItem.getRate();
    	    datum.setValue1AsUSDMoneyAmount(new BigDecimal(123.0));
    		break;
    	}
    	
    	List<HealthBenefitDtl> benefitsToUpdate = new ArrayList<HealthBenefitDtl>(returnedHealthBenefitDtlSet);
    	Worker eiWorker2 = benefitsComponent.updateHealthBenefitDtls(sWorkerId, benefitsToUpdate);
    	Set<HealthBenefitDtl> retAfterUpdateHealthBenefitDtlSet = eiWorker2.getHealthBenefitDtls();

    	// Check to see if the change in HealthBenefitDtl sticks
    	// Currently the HealthBenefitDtlMap contains only one HealthBenefitDtl
    	HealthBenefitDtl expectedDtl = returnedHealthBenefitDtlSet.iterator().next();
    	for (HealthBenefitDtl existingItem : retAfterUpdateHealthBenefitDtlSet)
    	{
    		AssertJUnit.assertEquals(expectedDtl.getRate(), existingItem.getRate());
    		break;
    	}	
    	
    }
    
    /*
     * Creates a Map of HealthBenefitDtl objects.
     * Currently only Health Benefit type is supported.
     */
    private void createHealthBenefitDtls() throws Exception {
    	
    	if (healthBenefitsDtlsList.size() == 0) {
	    	HealthBenefitDtl healthBenefitDtl1 = benefitsComponent.createHealthBenefitDtl(HealthBenefitTypeEnum.SEC125HEALTH);
	    	Datum rate = new Datum();
	    	rate.setValue1AsUSDMoneyAmount(new BigDecimal(50.00));
	    	healthBenefitDtl1.setRate(rate);

	    	healthBenefitsDtlsList.add(healthBenefitDtl1);
    	}
    	
    }
    

}
