package com.intuit.sb.payroll.capability.common;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.ejb.EJBException;
import javax.transaction.Status;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.benefits.BenefitsComponent;
import com.intuit.sb.payroll.capability.compensation.CompensationComponent;
import com.intuit.sb.payroll.capability.config.GeminiConfigurationService;
import com.intuit.sb.payroll.capability.config.GeminiConfigurationUtility;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent;
import com.intuit.sb.payroll.capability.qa.QaBootstrap;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.management.PayrollService;
import com.intuit.sb.payroll.entity.benefits.HealthBenefitEAO;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.deductions.DeductionEAO;
import com.intuit.sb.payroll.entity.employer.EmployerEAO;
import com.intuit.sb.payroll.entity.employer.WorkLocationEAO;
import com.intuit.sb.payroll.entity.model.AgencyIdType;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.ComplianceDoc;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.EmployerIdTypeEnum;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.task.TaskEAO;
import com.intuit.sb.payroll.entity.taxauthority.JurisdictionEAO;
import com.intuit.sb.payroll.entity.taxauthority.LawEAO;
import com.intuit.sb.payroll.entity.taxes.AgencyEAO;
import com.intuit.sb.payroll.entity.taxes.AgencyIdTypeEAO;
import com.intuit.sb.payroll.entity.taxes.TaxEAO;
import com.intuit.sb.payroll.entity.taxes.withholding.ComplianceDocEAO;
import com.intuit.sb.payroll.entity.wages.CompensationEAO;
import com.intuit.sb.payroll.entity.worker.WorkerEAO;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;


/**
 *  <P>The <CODE>BasicTestBase</CODE> class contains helper methods to set up test environment</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class BasicTestBase extends PayrollTestBase
{
	/**
	 * <P>The DocId for the Federal W4 ComplianceDoc</P>
	 */
	protected static final String sW4DocId = "US-FED-W4";
	/**
	 * <P>The DocId for the California DE4 ComplianceDoc</P>
	 */
	protected static final String sDE4DocId = "US-CA-DE-4";
	/**
	 * <P>The name of the Mountain View work location</P>
	 */
	protected static final String sMountainViewLocation = "Mountain View, CA";
	/**
	 * <P>The name of the San Francisco work location</P>
	 */
	protected static final String sSanFranciscoLocation = "San Francisco, CA";
	/**
	 * <P>The name of the Mesquite work location</P>
	 */
	protected static final String sMesquiteLocation = "Mesquite, TX";
	/**
	 * <P>The UpdateWorkerWithholdings task name</P>
	 */
	protected static final String sUpdateWorkerWithholdings = "UpdateWorkerWithholdings";
	/**
	 * <P>The WorkerHireDateSaveOperation task name</P>
	 */
	protected static final String sWorkerHireDateSaveOperation = "WorkerHireDateSaveOperation";
	/**
	 * <P>A boolean value to represent that a worker is exempt from a tax</P>
	 */
	protected static final boolean sExemptFromTax = true;
	/**
	 * <P>A boolean value to represent that a worker is not exempt from a tax</P>
	 */
	protected static final boolean sNotExemptFromTax = false;
	/**
	 * <P>Holds an instance of PayrollResourceManager</P>
	 */
	protected static final PayrollResourceManager sResourceManager = PayrollResourceManager.getInstance();
	/**
	 * <P>Holds an instance of Logger</P>
	 */
	protected static Logger sLogger = LogManager.getLogger(BasicTestBase.class);
	/**
	 * <P>Holds an instance of a Company object, this company is used for most tests</P>
	 */
	protected static Employer sEmployer = null;
	/**
	 * <P>Holds an instance of a Worker object, this worker is used for most tests</P>
	 */
	protected static Worker sWorker = null;    // Use the same Worker object for most tests
	/**
	 * <P>Holds an instance of a Worker's UniqueId, this worker is used for most tests</P>
	 */
	protected static Long sWorkerId = null;
	/**
	 * <P>Holds an instance of a PhysicalAddress object, this PhysicalAddress is used for most tests</P>
	 */
	protected static USPostalAddress sPhysicalAddress = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 40</P>
	 */
	protected static Calendar sAge40BirthDate = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 22</P>
	 */
	protected static Calendar sAge22BirthDate = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 21</P>
	 */
	protected static Calendar sAge21BirthDate = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 20</P>
	 */
	protected static Calendar sAge20BirthDate = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 18</P>
	 */
	protected static Calendar sAge18BirthDate = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 17</P>
	 */
	protected static Calendar sAge17BirthDate = null;

	/**
	 * <P>Holds an instance of a WorkLocationAndWorkerDataPopulator object, for setting up work locations and workers data</P>
	 */
	protected static WorkLocationAndWorkerDataPopulator workLocationAndWorkerDataPopulator = null;
	private static boolean bInitialized = false;
    private static Employer mOriginalEmployerInfo = null;
    private static USPostalAddress mOriginalPostalAddress = null;

	protected static WorkerEAO sWorkerEAO;
	protected static LawEAO sLawEAO;
	protected static EmployerEAO sEmployerEAO;
	protected static WorkLocationEAO locationEAO;
	protected static AgencyIdTypeEAO agencyIdTypeEAO;
	protected static WorkerComponent sWorkerComponent;
	protected static EmployerComponent sEmployerComponent;
	protected static PayrollScheduleComponent sPayrollScheduleComponent;
	protected static EmployerPTSWorkflowComponent sEmpPTSWFComponent;
	protected static WorkerPaymentsComponent sWorkerPaymentsComponent;
	
	protected static AgencyEAO agencyEAO;
	protected static JurisdictionEAO jurisdictionEAO ;
	protected static TaskEAO taskEAO;
	
    protected static PaycheckCalculator mCalcEngine;
    protected static HealthBenefitEAO healthBenefitEAO;
    protected static TaxComponent taxComponent;
    protected static TaxEAO taxEAO;
	protected static CompensationEAO compensationEAO;
	protected static DeductionEAO deductionEAO;
	protected static PaidTimeOffComponent paidTimeOffComponent;
	protected static ComplianceDocEAO complianceDocEAO;
	protected static BenefitsComponent benefitsComponent;
	protected static CompensationComponent compensationComponent;
	protected static EntityService entityService;
	protected static GeminiConfigurationService geminiConfigurationService ;
	/**
	 * <P>Calls initializeTestData() to initialize the test data if the test data hasn't been seeded yet.</P>
	 */
	@BeforeClass
	public static void setUpBasicTestBase()
	{
		sLogger = LogManager.getLogger(BasicTestBase.class);
		//Initialize the data just once
		setPTSHosttoQA5();
		if (false == bInitialized)
		{
	        initializeEAOs();
			try {
	            getUserTransaction().begin();
	            initializeTestData();
	            getUserTransaction().commit();
	        }
	        catch (Throwable t) {
	            try
	            {
	                // Roll back the transaction
	                getUserTransaction().rollback();
	            }
	            catch (Exception ex) {
	            }
	            t.printStackTrace();
	            sLogger.error("Error in setUpClass(): ", t);
	            AssertJUnit.fail("Error in setUpClass(): " + t.getMessage());
	        }
		}
	}

	public static void initializeEAOs() {
		if ( qaBootstrap == null )
		{
			qaBootstrap = ServiceLocator.getInstance().getService(QaBootstrap.JNDI_NAME);
		}
		entityService = qaBootstrap.getEjbFromClass(EntityService.class);
		sWorkerEAO = qaBootstrap.getEjbFromClass(WorkerEAO.class);
		sLawEAO = qaBootstrap.getEjbFromClass(LawEAO.class);
		sEmployerEAO = qaBootstrap.getEjbFromClass(EmployerEAO.class);
		locationEAO = qaBootstrap.getEjbFromClass(WorkLocationEAO.class);
		agencyIdTypeEAO = qaBootstrap.getEjbFromClass(AgencyIdTypeEAO.class);
	    sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        sEmployerComponent = qaBootstrap.getEjbFromClass(EmployerComponent.class);
        sEmpPTSWFComponent = qaBootstrap.getEjbFromClass(EmployerPTSWorkflowComponent.class);
        sPayrollScheduleComponent = qaBootstrap.getEjbFromClass(PayrollScheduleComponent.class);
        agencyEAO = qaBootstrap.getEjbFromClass(AgencyEAO.class);
    	jurisdictionEAO = qaBootstrap.getEjbFromClass(JurisdictionEAO.class);
    	taskEAO = qaBootstrap.getEjbFromClass(TaskEAO.class);
    	mCalcEngine = qaBootstrap.getEjbFromClass(PaycheckCalculator.class);
    	healthBenefitEAO = qaBootstrap.getEjbFromClass(HealthBenefitEAO.class);
    	taxComponent = qaBootstrap.getEjbFromClass(TaxComponent.class);
    	compensationEAO = qaBootstrap.getEjbFromClass(CompensationEAO.class);
    	deductionEAO = qaBootstrap.getEjbFromClass(DeductionEAO.class);
    	paidTimeOffComponent = qaBootstrap.getEjbFromClass(PaidTimeOffComponent.class);
    	taxEAO = qaBootstrap.getEjbFromClass(TaxEAO.class);
    	complianceDocEAO = qaBootstrap.getEjbFromClass(ComplianceDocEAO.class);
    	benefitsComponent = qaBootstrap.getEjbFromClass(BenefitsComponent.class);
    	sWorkerPaymentsComponent = qaBootstrap.getEjbFromClass(WorkerPaymentsComponent.class);
    	compensationComponent = qaBootstrap.getEjbFromClass(CompensationComponent.class);
	}
	
	/**
	 * <P>Method for cleaning up resources when this class is no longer needed.  Currently there is nothing to clean up.</P>
	 *
	 */
	@AfterClass
	public static void tearDownBasicTestBase()
	{
		// Nothing to tear down
	}

	/**
	 * <P>Method that gets called before each test is executed.  Calls initializeTestData() to initialize the test data
	 * if the test data hasn't been seeded yet.  Also, begins a Transaction for any test which will finally be
	 * rolled back in tearDown().  In other words, this effectively wraps an entire test inside a transaction.  It also
	 * gets a fresh copy of sWorker from the database and sets up the workflow for this worker.</P>
	 */
	@BeforeMethod
	public void setUp() throws Exception
	{
		try {
			// Begin a Txn that is used for the test step setup and
			// is then left active for the test itself. i.e. every test
			// can assume it started with an active txn. This is the same
			// model as the engine runs in where all service requests to
			// it come via a SLSB which will begin a txn so the request
			// implementation doesn't need to worry about txn handling.
			getUserTransaction().begin();
		}
		catch (Throwable t) {
			sLogger.error("Unable to begin transaction", t);
			AssertJUnit.fail("Unable to begin transaction: " + t.getMessage());
		}
		//Initialize the data just once
		if (false == bInitialized)
		{
			initializeTestData();
		}
	}

	/**
	 * <P>If the transaction is still active, then tear down the workflow for sWorker and roll back the transaction.</P>
	 */
	@AfterMethod
	public void tearDown() throws Exception
	{
       try
        {
    	   int status = getUserTransaction().getStatus();
    	   if ((Status.STATUS_ACTIVE == status) || (Status.STATUS_MARKED_ROLLBACK == status)) {
               // Default test step conclusion is to rollback and hence
               // restore the DB to the same state as at the test start.
               // If the test needs to commit it must do so in the @Test method
               // and it then becomes responsible for unwinding DB changes.
               getUserTransaction().rollback();
    	   }
        }
        catch (Exception ex) {
        	ex.printStackTrace();
        }
	}

    /**
     * This utility routine will set the Payroll system date to the specified date
     * 
     * @param systemDate The date to set the Payroll system date to
     */
    public static void setPayrollSystemDate(Date systemDate) {
    	PayrollService.getInstance().setSystemDate(DateUtil.dateToCalendar(systemDate));
    }
    
    /**
     * This utility routine will reset the Payroll system date to the current date & time
     */
    public static void resetPayrollSystemDate() {
    	PayrollService.getInstance().clearSystemDate();
    }
    
	/**
	 * <P>Refreshes sWorker from the database using the Id specified in the sWorkerId variable.</P>
	 *
	 */
	public static void refreshWorker()
	{
		sWorker = entityService.findById(Worker.class, sWorkerId);
	}

	/**
	 * <P>Sets up the test environment by performing the following:</P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Check to see if the global payroll data are set up in the database, call the seed data populator to
	 *      populate the data if necessary</LI>
	 * <LI>Check to see if any workers are created in the database, call the worker data populator to populate some
	 *      workers if necessary</LI>
	 * <LI>Sets sWorkerId with the id of the first employee from the database</LI>
	 * <LI>If the payroll data is being seeded in the current invocation of this method, then seed the employer
	 *      compliance data as well.</LI>
	 * <LI>Initializes the following static variables with some valid values:
	 *      sEmployer, sPhysicalAddress, sAge40BirthDate, sAge22BirthDate, sAge21BirthDate, sAge20BirthDate,
	 *      sAge18BirthDate, sAge17BirthDate</LI>
	 * </OL>
	 */
	public static void initializeTestData() throws Exception
	{
    	
		//check whether an Employer already exists in the system for this user credentials
		List<Employer>  employerList = entityService.findAll(Employer.class);
		
		//If there are no employers in the DB, create one.
		if(employerList == null || employerList.size() == 0)
		{
			sEmployer = new Employer();
			sEmployer.setCompanyName("D' Apes");
			sEmployer.setDbaName("Apes Incorporated");
			sEmployer = sEmployerComponent.createEmployer(sEmployer);
		}
		else if(sEmployer == null)
		{
			sEmployer = sEmployerComponent.getEmployer();
		}
		
		// Seed Default Payroll Schedule (if required) after the employer has been created.
		List<PayrollSchedule>  payrollSchedules = entityService.findAll(PayrollSchedule.class);
		if (payrollSchedules.size() == 0){
			PayrollScheduleTestUtils.createDefaultSchedule();
		}
		
		if(sEmployer != null && !bInitialized)
		{
			setUpEmployer();
			
			if (null == workLocationAndWorkerDataPopulator)
			{
				workLocationAndWorkerDataPopulator = new WorkLocationAndWorkerDataPopulator(3);
			}
	
			List<Worker>  workerList = entityService.findAll(Worker.class);
	
			if (workerList == null || workerList.size() == 0)
			{
				workLocationAndWorkerDataPopulator.populateWorkLocAndWorkers();
				workerList = entityService.findAll(Worker.class);
			}

			// Bcos now worklocation workflow adds SUI and Disability taxes whenever
			// we add a new work location there will be no need to seed from here
			// Only seed the Employer compliance data when the other seed data was just added
//			if ( hadJurisdication == false ) // null != dataPopulator)
//			{
//				dataPopulator.populateEmployerComplianceData();
//			}
	
			
			setUpPostalAddress();
			setUpBirthDates();
	
			bInitialized = true;
		}
		if(sWorkerId == null)
		{
			System.out.println(">>>>>> get first worker");
			// use the first employee
			List<Employee>  employeeList = entityService.findAll(Employee.class);
			AssertJUnit.assertNotNull("No employees found in database", employeeList);
			AssertJUnit.assertTrue("No employees found in database", employeeList.size() > 0);
			Employee employee = employeeList.get(0);
			// Remember the unique id associated with this Worker
			sWorkerId = employee.getId();
			sWorker = entityService.findById(Worker.class, sWorkerId);
		}
	}


	/**
	 * <P>Create and initializes the sEmployer object with some valid values</P>
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 *
	 *
	 */
	public static void setUpEmployer() throws ValidationException, ServiceFailureException
	{
		sEmployer = sEmployerEAO.findCurrentEmployer();
		
		sEmployer.setCompanyName("Bogus");
		sEmployer.setLegalName("Bogus Inc");
		sEmployer.setDbaName("Bogus Trading");
		sEmployer.setNonProfit(false);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setFirstPayrollDate(new Date(0));
		
		Set<EmployerId> empIdSet = sEmployer.getEmployerIds();
		boolean found = false;
		for (EmployerId employerId : empIdSet) {
			if (employerId.getAgencyIdType().getIdTypeKey() == EmployerIdTypeEnum.EIN)
				found = true;
		}

		if (!found){
			// Employer Id
			EmployerId employerIdElement = new EmployerId();
			List<AgencyIdType>  agencyIdtypes = entityService.findAll(AgencyIdType.class);
			for (AgencyIdType agencyIdType : agencyIdtypes)
			{
				if(agencyIdType.getIdTypeKey() == EmployerIdTypeEnum.EIN)
				{
					employerIdElement.setAgencyIdType(agencyIdType);
					break;
				}
			}
			employerIdElement.setValue(EmployerComponentTestUtils.generateRandomEIN());
			sEmployer.addEmployerId(employerIdElement);
		}
		sEmployer = sEmployerComponent.saveEmployer(sEmployer);
	}

    protected static void saveWorkLocationAddress()
    {
        mOriginalPostalAddress = getWorkerWorkLocationAddress();
    }
    
    protected static void restoreWorkLocationAddress() throws Exception
    {
        // Restore original workLocation address
        USPostalAddress address = getWorkerWorkLocationAddress();
        if (address != null)
        {
            updateAddress(address, mOriginalPostalAddress);
        }
        sWorker = sWorkerComponent.updateWorker(sWorker);
    }
    
    private static void updateAddress(USPostalAddress toAddress, USPostalAddress fromAddress)
    {
        toAddress.setAddressLine1(fromAddress.getAddressLine1());
        toAddress.setAddressLine2(fromAddress.getAddressLine2());
        toAddress.setAddressLine3(fromAddress.getAddressLine3());
        toAddress.setCity(fromAddress.getCity());
        toAddress.setCountry(fromAddress.getCountry());
        toAddress.setCountryISOCode(fromAddress.getCountryISOCode());
        toAddress.setCounty(fromAddress.getCounty());
        toAddress.setRegionOrState(fromAddress.getRegionOrState());
        toAddress.setZipOrPostalCode(fromAddress.getZipOrPostalCode());
    }
    
    private static USPostalAddress getWorkerWorkLocationAddress()
    {
        WorkLocation workLocation = getWorkerWorkLocation();
        return AddressConvertor.getUSFromGlobal(workLocation.getTaxResidenceAddress(), new USPostalAddressImpl());       
    }
    
	/**
	 * <P>Create the sPhysicalAddress object and initializes the CountryName and CountryISOCode fields</P>
	 *
	 */
	private static void setUpPostalAddress()
	{
		sPhysicalAddress = new USPostalAddressImpl();
		// Set up fields that don't change between tests
		sPhysicalAddress.setCountry("United States");
		sPhysicalAddress.setCountryISOCode("US");
	}

	/**
	 * <P>Initializes sAge40BirthDate, sAge22BirthDate, sAge21BirthDate, sAge20BirthDate, sAge18BirthDate and
	 * sAge17BirthDate with dates that represents their age values</P>
	 *
	 */
	private static void setUpBirthDates() {
		Calendar currentDate = DateUtil.getNowCalendar();
		// Go back one day (to avoid issues with timezone)
		currentDate.add(Calendar.DAY_OF_MONTH, -1);
		int day = currentDate.get(Calendar.DAY_OF_MONTH);
		int month = currentDate.get(Calendar.MONTH);
		// Avoid issue with leap year
		if ((month == 2) && (day == 29))
		{
			day -= 1;
		}
		// set up birth date for age 40
		sAge40BirthDate = DateUtil.createCalendar(currentDate.get(Calendar.YEAR)-40, month, day);
		// set up birth date for age 22
		sAge22BirthDate = DateUtil.createCalendar(currentDate.get(Calendar.YEAR)-22, month, day);
		// set up birth date for age 21
		sAge21BirthDate = DateUtil.createCalendar(currentDate.get(Calendar.YEAR)-21, month, day);
		// set up birth date for age 20
		sAge20BirthDate = DateUtil.createCalendar(currentDate.get(Calendar.YEAR)-20, month, day);
		// set up birth date for age 18
		sAge18BirthDate = DateUtil.createCalendar(currentDate.get(Calendar.YEAR)-18, month, day);
		// set up birth date for age 17
		sAge17BirthDate = DateUtil.createCalendar(currentDate.get(Calendar.YEAR)-17, month, day);
	}

	/**
	 * <p>Initialize sPhysicalAddress with an address located in San Francisco, CA</P>
	 *
	 */
	protected static void setUpSanFranciscoAddress()
	{
		sPhysicalAddress.setAddressLine1("2701 Balboa St");
		sPhysicalAddress.setCity("San Francisco");
		sPhysicalAddress.setRegionOrState("CA");
		sPhysicalAddress.setZipOrPostalCode("94121");
	}

	/**
	 * <P>Initialize sPhysicalAddress with an address located in San Mateo, CA</P>
	 *
	 */
	protected static void setUpSanMateoAddress()
	{
		sPhysicalAddress.setAddressLine1("1880 South Grant St");
		sPhysicalAddress.setCity("San Mateo");
		sPhysicalAddress.setRegionOrState("CA");
		sPhysicalAddress.setZipOrPostalCode("94402");
	}

	/**
	 * <P>Initialize sPhysicalAddress with an address located in Mesquite, TX</P>
	 *
	 */
	protected static void setUpTexasAddress()
	{
		sPhysicalAddress.setAddressLine1("3733 West Emporium Circle");
		sPhysicalAddress.setCity("Mesquite");
		sPhysicalAddress.setRegionOrState("TX");
		sPhysicalAddress.setZipOrPostalCode("75150");
	}

    protected static void saveEmployerInfo()
    {
    	//[rj]no more accessing the Employer thro EAO
        Employer employer = sEmployerEAO.findCurrentEmployer();
//    	Employer employer = sEmployerComponent.findCurrentEmployer();
        // Remember the original Employer information
        mOriginalEmployerInfo = new Employer();
        mOriginalEmployerInfo.setIndustryType(employer.getIndustryType());
        mOriginalEmployerInfo.setLegalName(employer.getLegalName());
//            mOriginalEmployerInfo.setOrganizationTaxId(employer.getOrganizationTaxId());
        mOriginalEmployerInfo.setCompanyName(employer.getCompanyName());
        mOriginalEmployerInfo.setDbaName(employer.getDbaName());
        mOriginalEmployerInfo.setLegalStructure(employer.getLegalStructure());
        mOriginalEmployerInfo.setNonProfit(employer.isNonProfit());
    }
    
    protected static void restoreEmployerInfo()
    {
    	//[rj]no more accessing the Employer thro EAO
        Employer employer = sEmployerEAO.findCurrentEmployer();
//    	Employer employer = sEmployerComponent.findCurrentEmployer();
        employer.setIndustryType(mOriginalEmployerInfo.getIndustryType());
        employer.setLegalName(mOriginalEmployerInfo.getLegalName());
//            employer.setOrganizationTaxId(mOriginalEmployerInfo.getOrganizationTaxId());
        employer.setCompanyName(mOriginalEmployerInfo.getCompanyName());
        employer.setDbaName(mOriginalEmployerInfo.getDbaName());
        employer.setLegalStructure(mOriginalEmployerInfo.getLegalStructure());
        employer.setNonProfit(mOriginalEmployerInfo.isNonProfit());
    }
    
    /**
     * <P>Sets up the Employer and Worker info using the specified data</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the company legal structure info</LI>
     * <LI>Set up the work location info</LI>
     * <LI>Set up relationship to employer</LI>
     * <LI>Set up the worker birthdate</LI>
     * </OL>
     *
     */
    public static void setUpTestData(Employer org, USPostalAddress address, Calendar dateOfBirth,
                                     RelationshipToOwnerEnum relationship) throws Exception
    {       
    	//[rj]no more accessing the Employer thro EAO
        Employer employer = sEmployerEAO.findCurrentEmployer();
//    	Employer employer = sEmployerComponent.findCurrentEmployer();
        employer.setIndustryType(org.getIndustryType());
        employer.setLegalName(org.getLegalName());
//            employer.setOrganizationTaxId(org.getOrganizationTaxId());
        employer.setCompanyName(org.getCompanyName());
        employer.setDbaName(org.getDbaName());
        employer.setLegalStructure(org.getLegalStructure());
        employer.setNonProfit(org.isNonProfit());
        
        WorkLocation workLocation = getWorkerWorkLocation();
        
        updateAddressInfo(workLocation, address);

        // Update the employee relationship to the employer
        sWorker.setRelationshipToOwner(relationship);

        // Update the date of birth
        sWorker.setDateOfBirth(dateOfBirth.getTime());
        

        //Save the sWorker - this sets the appropriate state on the Workflow automatically.
        sWorker = sWorkerComponent.updateWorker(sWorker);
    }

    private static void updateAddressInfo(WorkLocation workLocation, USPostalAddress address)
    {
    	GlobalPostalAddress globalAddress = AddressConvertor.getGlobalFromUS(address, new GlobalPostalAddress());
        workLocation.setTaxResidenceAddress(globalAddress);
    }
    
    protected static WorkLocation getWorkerWorkLocation()
    {
        refreshWorker();
        return sWorker.getWorkLocation();
    }
    
	/**
	 * <P>Removes all payroll item details associated with the given worker and also deletes them from the database</P>
	 *
	 * @param workerId The worker's uniqueId
	 * @return The updated worker
	 */
	protected static Worker removeAllWorkerPayrollItems(Long workerId)
	{
		Worker worker = entityService.findById(Worker.class, workerId);
		worker = WorkerTestUtils.removeAllWorkerPayrollItems(worker);
		return worker;
	}

	/**
	 * <P>Returns true if the given payrollItem is of type Compensation</P>
	 *
	 * @param payrollItem The PayrollItem to check whether or not it is a wage type
	 * @return true if the payrollItem is a wage type, false otherwise
	 */
	protected boolean isWageType(PayrollItem payrollItem)
	{
		return (payrollItem != null && payrollItem instanceof Compensation);
	}

	/**
	 * <P>Returns a ComplianceDoc that's specified by the given docId</P>
	 *
	 * @param docId The docId for retrieving the ComplianceDoc
	 * @return a Compliancedoc object
	 */
	protected static ComplianceDoc findComplianceDoc(Long docId)
	{
		ComplianceDoc complianceDoc = entityService.findById(ComplianceDoc.class, docId);
		return complianceDoc;
	}

	/**
	 * <P>Returns a Law object that's specified by the given lawId</P>
	 *
	 * @param lawId The lawId for retrieving the Law
	 * @return a Law object
	 */
	protected static Law findLaw(String lawId) throws ServiceFailureException
	{
		return sLawEAO.findByLawId(lawId);
	}

	protected void assertNoActiveTransaction() throws Exception
	{
		AssertJUnit.assertEquals("", Status.STATUS_NO_TRANSACTION, getUserTransaction().getStatus());
	}

	/**
	 * <P>Sets up the W4 withholding information for the specified Worker using the given information.</P>
	 *
	 * @param workerId The worker's UniqueId
	 * @param docId The docId for the withholding info
	 * @param docType The ComplianceDocType in which the withholding information is for
	 * @param isTaxExempt Whether or not the worker is exempt from income tax
	 * @param withholdingStatus The worker's withholding status
	 * @param allowance The worker's withholding allowance
	 * @param additonalAmount The worker's additional withholding
	 */
	protected static void setUpW4WithholdingInfo(Long workerId, String docId, ComplianceDocIdEnum docType,
			boolean isTaxExempt, W4WithholdingStatusEnum withholdingStatus, Integer allowance,
			BigDecimal additonalAmount) throws ServiceFailureException
	{
		Worker worker = entityService.findById(Worker.class, workerId);
		WorkerTestUtils.setUpW4WithholdingInfo(worker, docId, docType, isTaxExempt, withholdingStatus, 
						allowance, additonalAmount);
	}

	/**
	 * <P>Sets up a non-derived DE4 withholding information for the specified Worker using the given information.</P>
	 *
	 * @param workerId The worker's Id
	 * @param docId The docId for the withholding info
	 * @param docType The ComplianceDocType in which the withholding information is for
	 * @param isTaxExempt Whether or not the worker is exempt from income tax
	 * @param withholdingStatus The worker's withholding status
	 * @param allowance The worker's withholding allowance
	 * @param deductions The worker's withholding deductions
	 * @param additonalAmount The worker's additional withholding
	 */
	protected static void setUpDe4WithholdingInfo(Long workerId, String docId, ComplianceDocIdEnum docType,
			boolean isTaxExempt, DE4WithholdingStatusEnum withholdingStatus, Integer allowance,
			Integer deductions, BigDecimal additonalAmount) throws Exception
	{
		Worker worker = entityService.findById(Worker.class, workerId);
		WorkerTestUtils.setUpDe4WithholdingInfo(worker, docId, docType, isTaxExempt, withholdingStatus, allowance,
						deductions, additonalAmount);
	}

	/**
	 * <P>Removes the worker's withholding info that's specified by the given lawId.  Also delete the withholding
	 *    related data from the database.</P>
	 *
	 * @param workerId The worker's UniqueId
	 * @param lawId The lawId specifying which withholdingInfo to remove
	 * @return The updated worker
	 */
	protected static Worker removeWorkerWithholdingInfoByType(Long workerId, ComplianceDocIdEnum docType) throws Exception
	{
		Worker worker = entityService.findById(Worker.class, workerId);
		worker = WorkerTestUtils.removeWorkerWithholdingInfoByType(worker, docType);
		return worker;
	}


	protected Exception getEJBInnerException(EJBException e)
	{
	    return e.getCausedByException();
	}

	public static void deleteAllPayrollSchedule() {
		List<com.intuit.sb.payroll.entity.model.PayrollSchedule>  pScheduleList = entityService.findAll(com.intuit.sb.payroll.entity.model.PayrollSchedule.class);
		List<Worker>  workList = entityService.findAll(Worker.class);
	
		for (Worker worker : workList) {
			if (worker.getPayrollSchedule() != null) {
				worker.setPayrollSchedule(null);
			}
		}
		sEmployer = sEmployerEAO.findCurrentEmployer();
		Set<PayrollSchedule> employerPS = sEmployer.getPayrollSchedules();
		for (PayrollSchedule payrollSchedule : employerPS) {
			sEmployer.removePayrollSchedule(payrollSchedule);
		}
		
		for (com.intuit.sb.payroll.entity.model.PayrollSchedule pSchedule : pScheduleList) {
			entityService.makeTransient(pSchedule);
		}
	}
	
	// QA5 is high availability machine.Even if the url is down this method will not throw any exception
	private static void setPTSHosttoQA5(){
		geminiConfigurationService = GeminiConfigurationUtility.getGeminiConfigurationService();
		geminiConfigurationService.setTaxPaymentServiceHost("http://sdgapppspqa11.corp.intuit.net");
	}

	public static String getSW4DocId() {
		return sW4DocId;
	}

	public static String getSDE4DocId() {
		return sDE4DocId;
	}

	public static String getSMountainViewLocation() {
		return sMountainViewLocation;
	}

	public static String getSSanFranciscoLocation() {
		return sSanFranciscoLocation;
	}

	public static String getSMesquiteLocation() {
		return sMesquiteLocation;
	}

	public static String getSUpdateWorkerWithholdings() {
		return sUpdateWorkerWithholdings;
	}

	public static String getSWorkerHireDateSaveOperation() {
		return sWorkerHireDateSaveOperation;
	}

	public static boolean isSExemptFromTax() {
		return sExemptFromTax;
	}

	public static boolean isSNotExemptFromTax() {
		return sNotExemptFromTax;
	}

	public static PayrollResourceManager getSResourceManager() {
		return sResourceManager;
	}

	public static Logger getSLogger() {
		return sLogger;
	}

	public static Employer getSEmployer() {
		return sEmployer;
	}

	public static Worker getSWorker() {
		return sWorker;
	}

	public static Long getSWorkerId() {
		return sWorkerId;
	}

	public static USPostalAddress getSPhysicalAddress() {
		return sPhysicalAddress;
	}

	public static Calendar getSAge40BirthDate() {
		return sAge40BirthDate;
	}

	public static Calendar getSAge22BirthDate() {
		return sAge22BirthDate;
	}

	public static Calendar getSAge21BirthDate() {
		return sAge21BirthDate;
	}

	public static Calendar getSAge20BirthDate() {
		return sAge20BirthDate;
	}

	public static Calendar getSAge18BirthDate() {
		return sAge18BirthDate;
	}

	public static Calendar getSAge17BirthDate() {
		return sAge17BirthDate;
	}

	public static WorkLocationAndWorkerDataPopulator getWorkLocationAndWorkerDataPopulator() {
		return workLocationAndWorkerDataPopulator;
	}

	public static boolean isBInitialized() {
		return bInitialized;
	}

	public static Employer getMOriginalEmployerInfo() {
		return mOriginalEmployerInfo;
	}

	public static USPostalAddress getMOriginalPostalAddress() {
		return mOriginalPostalAddress;
	}

	public static WorkerEAO getSWorkerEAO() {
		return sWorkerEAO;
	}

	public static LawEAO getLawEAO() {
		return sLawEAO;
	}

	public static EmployerEAO getSEmployerEAO() {
		return sEmployerEAO;
	}

	public static WorkLocationEAO getLocationEAO() {
		return locationEAO;
	}

	public static AgencyIdTypeEAO getAgencyIdTypeEAO() {
		return agencyIdTypeEAO;
	}

	public static WorkerComponent getSWorkerComponent() {
		return sWorkerComponent;
	}

	public static EmployerComponent getSEmployerComponent() {
		return sEmployerComponent;
	}

	public static PayrollScheduleComponent getSPayrollScheduleComponent() {
		return sPayrollScheduleComponent;
	}

	public static EmployerPTSWorkflowComponent getSEmpPTSWFComponent() {
		return sEmpPTSWFComponent;
	}

	public static AgencyEAO getAgencyEAO() {
		return agencyEAO;
	}

	public static JurisdictionEAO getJurisdictionEAO() {
		return jurisdictionEAO;
	}

	public static TaskEAO getTaskEAO() {
		return taskEAO;
	}

	public static PaycheckCalculator getMCalcEngine() {
		return mCalcEngine;
	}

	public static HealthBenefitEAO getHealthBenefitEAO() {
		return healthBenefitEAO;
	}

	public static TaxComponent getTaxComponent() {
		return taxComponent;
	}

	public static CompensationEAO getCompensationEAO() {
		return compensationEAO;
	}

	public static DeductionEAO getDeductionEAO() {
		return deductionEAO;
	}

	public static PaidTimeOffComponent getPaidTimeOffComponent() {
		return paidTimeOffComponent;
	}

	public static TaxEAO getTaxEAO() {
		return taxEAO;
	}

	public static ComplianceDocEAO getComplianceDocEAO() {
		return complianceDocEAO;
	}

	public static CompensationComponent getCompensationComponent() {
		return compensationComponent;
	}

	public static WorkerPaymentsComponent getSWorkerPaymentsComponent() {
		return sWorkerPaymentsComponent;
	}

	public static BenefitsComponent getBenefitsComponent() {
		return benefitsComponent;
	}

	public static EntityService getEntityService() {
		return entityService;
	}
}
