package com.intuit.sb.payroll.capability.common;

import org.apache.log4j.Logger;
import org.jboss.embedded.Bootstrap;
import org.jboss.virtual.plugins.context.vfs.AssembledContextFactory;
import org.jboss.virtual.plugins.context.vfs.AssembledDirectory;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.rules.RulesBridge;

/**
 * This base TestCase class implements @BeforeClass to deploy payroll 
 * EJB jar file into the container. The @BeforeMethod adds sb-payroll-component.jar via {@code addDeployJarFromClasspath} 
 * and then call @{code deploy()}.
 * 
 * Undeployment is automatically handled by this class before the container is shutdown.
 * 
 */
public class PayrollEmbeddedContainer extends EmbeddedContainerTestBase {
	
	private static final Logger sLogger = Logger.getLogger(PayrollEmbeddedContainer.class);
	
	private static boolean sDeployed = false;
	
	private static boolean seedInRealm = false;
	
	@BeforeClass
    synchronized public static void startContainer() throws Exception
    {
		bootstrapEmbeddedContainer();

		String seedinrealm = System.getProperty("seedinrealm");
		if (seedinrealm != null) {
			seedInRealm = true;
		}
		
		AssembledDirectory commonServiceJar =  AssembledContextFactory.getInstance().create("sb-payroll-common-employee.jar");
		AssembledDirectory capabilityUsJar = AssembledContextFactory.getInstance().create("sb-payroll-capability-us.jar");
		AssembledDirectory capabilityIntlJar = AssembledContextFactory.getInstance().create("sb-payroll-capability-intl.jar");
		AssembledDirectory compositeUsJar = AssembledContextFactory.getInstance().create("sb-payroll-composed-us.jar");
	    
		String [] beanIncludes = {"**/*Bean.class"};
				
		capabilityUsJar.addResources(EmployerComponent.class, beanIncludes, null);
		capabilityIntlJar.addResources(PaycheckCalculator.class, beanIncludes, null);
		commonServiceJar.addResources(RulesBridge.class, beanIncludes, null);
		//if run from composite, this class will exist
		Class<?> companyClz = null;
		try
		{
		  companyClz = Class.forName("com.intuit.sb.payroll.services.common.BankAccount");
		  compositeUsJar.addResources(companyClz, beanIncludes, null);
		}
		catch (ClassNotFoundException ex)
		{
			//do nothing, if capability project, CompanyInfo can not be found.
		}
		
	    if (!sDeployed)
	    {	    	
	    	//deploy entity jar
    		/* Bootstrap.getInstance().deployResourceBases("META-INF/persistence.xml") does 
    		 * not work in embedded container when entity jar is broken into two jars 
    		 * and one is refered in persistence.xml by <jar-file>
    		*/
    	    //Bootstrap.getInstance().deployResourceBases("META-INF/persistence.xml");
			if (seedInRealm) {
				addDeployJarFromClasspath("lib/sb-payroll-entity-us.jar");
			} else {				
	    		String filename = "../PayrollEntityService/target/sb-payroll-entity-us-" +
									getVersionString("pom.xml") + ".jar";
	    		addDeployJarFromFileSystem(filename);
			}
	    		deploy();
	    		Bootstrap bootStrap = Bootstrap.getInstance();
	    		bootStrap.deploy(capabilityIntlJar);
	    		bootStrap.deploy(capabilityUsJar);
	    		bootStrap.deploy(compositeUsJar);
	    		bootStrap.deploy(commonServiceJar);
            
            sDeployed = true;

            sLogger.debug(JNDITestUtil.listJNDITree(null)); 	
	    }
    }
	
}
