package com.intuit.sb.payroll.capability.common;

import java.lang.reflect.Method;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.transaction.UserTransaction;

import org.apache.log4j.Logger;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.capability.qa.QaBootstrap;
import com.intuit.sb.payroll.common.ServiceLocator;

/**
 * This base TestCase class implements @BeforeClass to deploy payroll 
 * EJB jar file into the container. The @BeforeMethod adds sb-payroll-component.jar via {@code addDeployJarFromClasspath} 
 * and then call @{code deploy()}.
 * 
 * Undeployment is automatically handled by this class before the container is shutdown.
 * 
 */
public class PayrollTestBase {
	
	static private InitialContext jndi;
	
	private static final Logger sLogger = Logger.getLogger(PayrollTestBase.class);
	private static String defaultContainer = "EmbeddedContainer";
	private static String container;
    protected static QaBootstrap qaBootstrap;
		
	/**
	 * For embedded container, bootstrap, for whp, do nothing
	 * @throws Exception
	 */
	@BeforeClass
    synchronized public static void startContainer() throws Exception
    {
		container = System.getProperty("container");
		if ( container == null )
		{
			//set default container to embedded container
			container = defaultContainer;
		}
		else
		{
			  Properties env = new Properties();
			  env.setProperty(Context.SECURITY_PRINCIPAL, "paul");
			  env.setProperty(Context.SECURITY_CREDENTIALS, "paul");
			  env.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.security.jndi.JndiLoginInitialContextFactory");
			  new InitialContext(env);
		}
		sLogger.debug("Container is " + container);
		//when tests against BHP is stable, remove embedded container totally. Keep it for now
		if ( "EmbeddedContainer".equals(container))
		{
			//when -Dcontainer=bhp is specified, embedded container dependencies are not there
			//use reflection to make compile happy when run mvn with -Dcontainer=bhp 
			Class<?> cl = Class.forName("com.intuit.sb.payroll.capability.common.PayrollEmbeddedContainer");
			Method mth = cl.getMethod("startContainer");
			mth.invoke(null);
			//PayrollEmbeddedContainer.startContainer();
		}
		qaBootstrap = ServiceLocator.getInstance().getService(QaBootstrap.JNDI_NAME);
    }
	
	@AfterClass
	public static void classTearDown() throws Exception
	{
		if ( "EmbeddedContainer".equals(container))
		{
			//when -Dcontainer=bhp is specified, embedded container dependencies are not there
			//use reflection to make compile happy when run mvn with -Dcontainer=bhp 
			Class<?> cl = Class.forName("com.intuit.sb.payroll.capability.common.PayrollEmbeddedContainer");
			Method mth = cl.getMethod("classTearDown");
			mth.invoke(null);
			//PayrollEmbeddedContainer.classTearDown();		
		}
	}
	
	public static UserTransaction getUserTransaction()
	{
		UserTransaction tx = null;
		try {
			//this line is temporarily needed for Seed Data project. seed data does  not call startContainter for now, so 
			//jndi needs to be initialized here
			if ( jndi == null)
			{
				jndi = new InitialContext();
			}
			tx = (UserTransaction)jndi.lookup("UserTransaction");
		}
		catch (Exception ex)
		{
			throw new RuntimeException("Error in look up transaction in jndi", ex);
		}
		return tx;
		
	}

	public static InitialContext getJndi() {
		return jndi;
	}

	public static Logger getSLogger() {
		return sLogger;
	}

	public static String getDefaultContainer() {
		return defaultContainer;
	}

	public static String getContainer() {
		return container;
	}

	public static QaBootstrap getQaBootstrap() {
		return qaBootstrap;
	}
 	
}
