package com.intuit.sb.payroll.capability.employer;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.cdm.basetypes.WebSiteAddress;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * EmployeeCrud tests.
 *
 * $Author: dyee $
 * $Revision: #2 $
 * $DateTime: 2008/09/12 08:19:42 $
 *
 */
public class EmployerCRUDTest extends EmployerComponentTestBase
{
	private final String companyName = "Intuit";
	private final String dbaName = "Quickbooks";
	private final String legalName = "Intuit Corp.";
	private static final LegalStructureEnum companyLegalType = LegalStructureEnum.CCORP;
	private static final IndustryTypeEnum industryType = IndustryTypeEnum.OTHER;
	private static final String legalPhoneNr = "650-944-6000";
	private static final String legalExt = "7777";
	private static final String addressLine1 = "2550 Garcia Ave.";
	private static final String addressLine2 = "Bldg 8";
	private static final String addressLine3 = "Floor 1";
	private static final String city = "Mtn View";
	private static final String country = "United States of America";
	private static final String countryISOCode = "USA";
	private static final String regionOrState = "CA";
	private static final String zipOrPostalCode = "94043";
	private static final Boolean legalAddressSameAsBusinessAddress = true;
	private static final Integer firstPayrollDateYear = 2008;
	private static final Integer firstPayrollDateMonth = 12;
	private static final Integer firstPayrollDateDay = 25;
	private static final Boolean isNonProfit = false;
	private static final String webAddressValue = "abc@abc.com";

	private static java.util.Date firstPayrollDate = new java.util.Date();
	private static USPostalAddress legalAddress = new USPostalAddressImpl();
	private static USPostalAddress businessAddress = new USPostalAddressImpl();
	private static WebSiteAddress webAddress = new WebSiteAddress();
	private static TelephoneNumber legalPhone = new TelephoneNumber();
	
	private void setDefaultAddress(USPostalAddress a)
	{
		a.setAddressLine1(addressLine1);
		a.setAddressLine2(addressLine2);
		a.setAddressLine3(addressLine3);
		a.setCity(city);
		a.setCountry(country);
		a.setCountryISOCode(countryISOCode);
		a.setRegionOrState(regionOrState);
		a.setZipOrPostalCode(zipOrPostalCode);
	}

	private void updateEmployer(Employer e)
	{
		e.setCompanyName(companyName);
		e.setDbaName(dbaName);
		e.setFirstPayrollDate(firstPayrollDate);
		e.setBusinessAddress(
				AddressConvertor.getGlobalFromUS(businessAddress, new GlobalPostalAddress()));
		e.setIndustryType(industryType);
		e.setLegalAndBusAddressSame(legalAddressSameAsBusinessAddress);
		e.setNonProfit(isNonProfit);
		e.setLegalAddress(
				AddressConvertor.getGlobalFromUS(legalAddress, new GlobalPostalAddress()));
		e.setWebAddress(webAddress);
		e.setLegalName(legalName);
		e.setLegalPhone(legalPhone);
		e.setLegalStructure(companyLegalType);
	}

	private Employer copyEmployerObject(Employer in)
	{
		Employer e = new Employer();
		
		e.setCompanyName(in.getCompanyName());
		e.setDbaName(in.getDbaName());
		e.setFirstPayrollDate(in.getFirstPayrollDate());
		e.setBusinessAddress(in.getBusinessAddress());
		e.setIndustryType(in.getIndustryType());
		e.setLegalAndBusAddressSame(in.isLegalAndBusAddressSame());
		e.setNonProfit(in.isNonProfit());
		e.setLegalAddress(in.getLegalAddress());
		e.setWebAddress(in.getWebAddress());
		e.setLegalName(in.getLegalName());
		e.setLegalPhone(in.getLegalPhone());
		e.setLegalStructure(in.getLegalStructure());
		
		return e;
	}

	@SuppressWarnings("deprecation")
	public void setValues() 
	{
		firstPayrollDate.setYear(firstPayrollDateYear);
		firstPayrollDate.setMonth(firstPayrollDateMonth);
		firstPayrollDate.setDate(firstPayrollDateDay);
		
		webAddress.setURI(webAddressValue);
		
		legalPhone.setFreeFormNumber(legalPhoneNr);
		legalPhone.setExtension(legalExt);
		
		setDefaultAddress(legalAddress);	
	}
	
	private void compareEmployerObjects(Employer e1, Employer e2) 
	{
		AssertJUnit.assertEquals("employer1 and employer2 companyName are different", e1.getCompanyName(), e2.getCompanyName());
		AssertJUnit.assertEquals("employer1 and employer2 DbaName are different", e1.getDbaName(), e2.getDbaName());
		AssertJUnit.assertEquals("employer1 and employer2 FirstPayrollDate are different", e1.getFirstPayrollDate(), e2.getFirstPayrollDate());
		AssertJUnit.assertEquals("employer1 and employer2 BusinessAddress are different", e1.getBusinessAddress(), e2.getBusinessAddress());
		AssertJUnit.assertEquals("employer1 and employer2 IndustryType are different", e1.getIndustryType(), e2.getIndustryType());
		AssertJUnit.assertEquals("employer1 and employer2 LegalAndBusAddressSame are different", e1.isLegalAndBusAddressSame(), e2.isLegalAndBusAddressSame());
		AssertJUnit.assertEquals("employer1 and employer2 isNonProfit are different", e1.isNonProfit(), e2.isNonProfit());
		AssertJUnit.assertEquals("employer1 and employer2 LegalAddres are different", e1.getLegalAddress(), e2.getLegalAddress());
		AssertJUnit.assertEquals("employer1 and employer2 WebAddress are different", e1.getWebAddress(), e2.getWebAddress());
		AssertJUnit.assertEquals("employer1 and employer2 LegalName are different", e1.getLegalName(), e2.getLegalName());
		AssertJUnit.assertEquals("employer1 and employer2 LgalPhone are different", e1.getLegalPhone(), e2.getLegalPhone());
		AssertJUnit.assertEquals("employer1 and employer2 are LegalStructure different", e1.getLegalStructure(), e2.getLegalStructure());
	}
	
    /**
     * <P>Tests Employer Validation through CRUD</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a new Employee</LI>
     * <LI>Set the DBA Name to be empty (a violation of the validation rules)</LI>
     * <LI>Set the Company Name to be empty (not a violation of the validation rules)</LI>
     * <LI>Save the updated employer</LI>
     * </OL>
     * <UL>
     * <LI>Verify that the employer does not save, and that the proper exception comes back.</LI>
     * </UL>
     */
	@Test(enabled=false, expectedExceptions=ValidationException.class)
	public void testEmployerValidation() throws Exception
	{
    	Employer employer = EmployerComponentTestUtils.createNewEmployerInMemory("testCoName", "testDBAName", "testDBAName",  "123456789", IndustryTypeEnum.RESTAURANT,
				LegalStructureEnum.PARTNERSHIP);
    	employer.setCompanyName("");
    	employer.setDbaName("");
    	
    	//not valid, DBA name is nullable
    	sEmployerComponent.saveEmployer(employer);
        AssertJUnit.fail("Should not have worked");
	}

	@Test
	public void testEmployerValidateAndUpdate() throws Exception
	{
		setValues();
		
		// employer already exists
		Employer e1 = sEmployerComponent.getEmployer();
		
		// update employer first
		updateEmployer(e1);
		Employer e1Copy = copyEmployerObject(e1);
   	
    	// persist modified employer
		sEmployerComponent.saveEmployer(e1);
		
		// reread employer to check if properly persisted
		Employer e2 = sEmployerComponent.getEmployer();

		// compare employer1 and employer2
    	compareEmployerObjects(e1Copy, e2);

    	return;
	}

}
