package com.intuit.sb.payroll.capability.employer;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;


public class EmployerComponentTestBase extends BasicTestBase {

	/**
	 * Deletes the employer from the database
	 * 
	 * @param 
	 */
	protected void cleanUpEmployer()
	{
		Employer employer = sEmployerEAO.findCurrentEmployer();
		entityService.makeTransient(employer);
	}
	
	/**
	 * Creates a basic EmployerContactPerson that should pass muster with the validators.
	 * @param firstName the firstName 
	 * @param lastName the lastName 
	 * @param title the contact title 
	 * @param phoneNumber the contact phoneNumber 
	 * @return the populated EmployerContactPerson.
	 */
	protected EmployerContactPerson createEmployerContactPerson(String firstName, String lastName, String title, String phoneNumber) {
		EmployerContactPerson employerContactPerson = new EmployerContactPerson();
		
		// Set passed-in name as contact name
		PersonName personName = new PersonName();
		personName.setFirstName(firstName);
		personName.setLastName(lastName);
		personName.setTitle(title);
		employerContactPerson.setPersonName(personName);

		// Use work phone as contact phone
		TelephoneNumber phone = new TelephoneNumber();
		phone.setFreeFormNumber(phoneNumber);
		employerContactPerson.setWorkPhone(phone);
		
		// Use canned address
		USPostalAddress taxAddress = WorkerTestUtils.createUSPhysicalAddress();
		
		employerContactPerson.setMailingAddress(
				AddressConvertor.getGlobalFromUS(taxAddress, new GlobalPostalAddress()));
		
		// Set email address
		EmailAddress emailAddress = new EmailAddress();
		emailAddress.setAddress("test@test.com");
		employerContactPerson.setEmailAddress(emailAddress);
		
		return employerContactPerson;
	}

}
