package com.intuit.sb.payroll.capability.employer;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.cdm.basetypes.WebSiteAddress;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.taxes.impl.LawUtils;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.model.AgencyIdType;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.EmployerIdTypeEnum;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentLineItem;
import com.intuit.sb.payroll.entity.model.PaymentLineItemSplit;
import com.intuit.sb.payroll.entity.model.PaymentType;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;



public class EmployerComponentTestUtils extends BasicTestBase{
	
	public static WorkLocation workLocation = null;
	
	/**
	 * Creates and returns an in-memory instance of Employee which has not been saved into the database
	 * @return Employee
	 */
	
	/**
	 * Creates and returns an in-memory instance of Employee which has not been saved into the database
	 * @param industryType 
	 * @param legalStructure 
	 * @return Employee
	 */
	public static Employer createNewEmployerInMemory(String companyName, String dbaName, String legalName,  String fein,
				IndustryTypeEnum industryType, LegalStructureEnum legalStructure)
	{
		// Generate random Federal EIN when fein is null
		if (fein == null){
			fein = generateRandomEIN();
		}
		
		Employer employer = new Employer();
		employer.setCompanyName(companyName);
		// DBA Name
		employer.setDbaName(dbaName);
		employer.setIndustryType(industryType); //IndustryTypeEnum.RESTAURANT
		employer.setLegalStructure(legalStructure); //LegalStructureEnum.PARTNERSHIP
		
		// Legal Name
		employer.setLegalName(legalName);
		
		// Employer Id (EIN)
		EmployerId employerIdElement = createEmployerId(fein);
		employer.addEmployerId(employerIdElement);

		employer.setFirstPayrollDate(new Date());
		employer.setNonProfit(false);
		return employer;
	}
	
	public static Employer createNewEmployer()
	{
		Employer  employer = createNewEmployerInMemory("myCoName", "myDbaName", "myLegalName", null,
				IndustryTypeEnum.RESTAURANT,LegalStructureEnum.PARTNERSHIP);
		
		employer.setBusinessAddress( createGlobalPostalAddress("1234 El Camino Real","San Carlos","CA","93122"));

		employer.setLegalAndBusAddressSame(false);
		USPostalAddress legalAddress = createLegalAddress();
		employer.setLegalAddress(
				AddressConvertor.getGlobalFromUS(legalAddress, new GlobalPostalAddress()));
		employer.setFirstPayrollDate(new Date());
		employer.setNonProfit(false);

		TelephoneNumber legalPhone = new TelephoneNumber();
		legalPhone.setFreeFormNumber("650 593-1212");
		legalPhone.setExtension("9999");
		employer.setLegalPhone(legalPhone);

		WebSiteAddress webAddress = new WebSiteAddress();
		webAddress.setURI("www.myCompanyWebAddress.com");
		employer.setWebAddress(webAddress);

		return employer;
	}
	
	/**
	 * <P>
	 * Utility method to create a new Employer with out worklocation
	 * </P>
	 */
    public static void createAndSaveNewEmployer() throws Exception {
    	sEmployer = createNewEmployer();
    	sEmployer.setNonProfit(true);
    	sEmployer.setFirstPayrollDate(new Date());
    	sEmployer = sEmployerComponent.createEmployer(sEmployer);
    }
    
    /**
     * Create an EmployerId entity for Federal Tax Id
     * @param ein
     * @return
     */
    private static  EmployerId createEmployerId(String ein){
		EmployerId employerIdElement = new EmployerId();
		List<AgencyIdType> agencyIdtypes = entityService.findAll(AgencyIdType.class);
		for (AgencyIdType agencyIdType : agencyIdtypes)
		{
			if(EmployerIdTypeEnum.EIN.equals(agencyIdType.getIdTypeKey()))
			{
				employerIdElement.setAgencyIdType(agencyIdType);
				break;
			}
		}
		employerIdElement.setValue(ein);
		return employerIdElement;
	}
    
    public static GlobalPostalAddress createGlobalPostalAddress(String addressLine1,String city,String state,String zip){

		USPostalAddress businessAddress = new USPostalAddressImpl();
		businessAddress.setAddressLine1(addressLine1);
		businessAddress.setCity(city);
		businessAddress.setRegionOrState(state);
		businessAddress.setZipOrPostalCode(zip);
		return 	AddressConvertor.getGlobalFromUS(businessAddress, new GlobalPostalAddress());
	}
    
    /**
	 * Creates a basic EmployerContactPerson that should pass with the validators.
	 * @param firstName the firstName 
	 * @param lastName the lastName 
	 * @param title the contact title 
	 * @param phoneNumber the contact phoneNumber 
	 * @return the populated EmployerContactPerson.
	 */
	public static EmployerContactPerson createEmployerContactPerson(String firstName, String lastName, String title, String phoneNumber) {
		EmployerContactPerson employerContactPerson = new EmployerContactPerson();
		
		// Set passed-in name as contact name
		PersonName personName = new PersonName();
		personName.setFirstName(firstName);
		personName.setLastName(lastName);
		personName.setTitle(title);
		employerContactPerson.setPersonName(personName);

		// Use work phone as contact phone
		TelephoneNumber phone = new TelephoneNumber();
		phone.setFreeFormNumber(phoneNumber);
		employerContactPerson.setWorkPhone(phone);
		
		// Use canned address
		USPostalAddress taxAddress = WorkerTestUtils.createUSPhysicalAddress();
		
		employerContactPerson.setMailingAddress(
				AddressConvertor.getGlobalFromUS(taxAddress, new GlobalPostalAddress()));
		
		// Set email address
		EmailAddress emailAddress = new EmailAddress();
		emailAddress.setAddress("test@test.com");
		employerContactPerson.setEmailAddress(emailAddress);
		
		return employerContactPerson;
	}
    
	public static USPostalAddress createLegalAddress()
	{
		USPostalAddress legalAddress = new USPostalAddressImpl();
		legalAddress.setAddressLine1("44 Elm Street");
		legalAddress.setCity("San Carlos");
		legalAddress.setRegionOrState("CA");
		legalAddress.setCountry("USA");
		legalAddress.setZipOrPostalCode("93122");
		return legalAddress;
	}
	
	/**
	 * <P>
	 * Utility method to create a new Employer with a worklocation
	 * </P>
	 */
    public static void createWorkLocation() throws Exception {
    	
    	workLocation = new WorkLocation();
		workLocation.setName(sSanFranciscoLocation); 
		TelephoneNumber mainPhone = new TelephoneNumber();
        mainPhone.setFreeFormNumber("650-944-500");
        workLocation.setMainPhone(mainPhone);
        // Make the first WorkLocation default
		workLocation.setDefaultt(true);
		USPostalAddress address = new USPostalAddressImpl();
		address.setAddressLine1("1231 22nd Ave");
        address.setCity("San Francisco");
        address.setCountry("United States");
        address.setIsNotToBeStandardized(true);
        address.setIsStandardized(true);
        address.setIsStreetAddress(true);
        address.setRegionOrState("CA");
        address.setZipOrPostalCode("94122");
        address.setCountryISOCode("US");
		USPostalAddress mailingAddress = address;
        workLocation.setMailingAddress(
        		AddressConvertor.getGlobalFromUS(mailingAddress, new GlobalPostalAddress()));

        //set tax address to be the same as mailing address
        workLocation.setTaxResidenceAddress(
        		AddressConvertor.getGlobalFromUS(mailingAddress, new GlobalPostalAddress()));

        WebSiteAddress webAddress = new WebSiteAddress();
        webAddress.setURI("http://www.intuit.com");
        workLocation.setWebAddress(webAddress);
        
        sEmployer = sEmployerComponent.addWorkLocation(workLocation);
    	//sEmployerComponent.saveEmployer(sEmployer);    	
    }    
    
    /**
	 * <P>
	 * Utility method to create a new Employer with out worklocation, Name
	 * </P>
	 */
    public static void createNewEmployerWithOutName() throws Exception {
    	sEmployer = new Employer();
    	sEmployer.setDbaName("Apes Incorporated");
    	sEmployerComponent.saveEmployer(sEmployer);    	    	  	
    }	
	
    /**
	 * <P>
	 * Utility method to delete all Employers
	 * </P>
     * @throws ValidationException 
	 */
    @SuppressWarnings("null")
	public static void deleteCurrentEmployer() throws ValidationException {
    	
    	
        //check whether an Employer already exists in the system for this user credentials
        List<Employer>  employerList = entityService.findAll(Employer.class);
        
        EntityService ptsEmployerEAO = qaBootstrap.getEjbFromClass(EntityService.class);
        List<PTSEmployer> ptsempList = ptsEmployerEAO.findAll(PTSEmployer.class);
        
        		
        //If there are employers in the DB, delete them.
        if (employerList != null || employerList.size() != 0) {
           	sEmployer = sEmployerEAO.findCurrentEmployer();
           	PTSEmployer ptsEmployer = ptsempList.get(0);
        	List<Worker> workerList= entityService.findAll(Worker.class);
        	
        	for (Worker worker : workerList) {
        		entityService.makeTransient(worker);
        	}
        	
        	sEmployer.removeAllEmployerContactPersons();        	
        	ptsEmployer.removeAllPTSContactPersons();
        	
        	List<WorkLocation> worklocationList = sEmployerComponent.findAllWorkLocations();
        	
        	for (WorkLocation workLocation : worklocationList) {
    			// Cannot delete a WL if there are workers associated with it.
    			int workersCount = locationEAO.getWorkerCountForWorkLocation(workLocation);
    			if (workersCount > 0) {
    				//Long worklocationId; 
    				//sWorkerEAO.findWorkersByWorkLocation();
    				throw new ValidationException("Cannot delete a Work Location with Workers associated with it.");
    			}
    			sEmployer.removeWorkLocation(workLocation);	    		
    			entityService.makeTransient(workLocation);
    		}	        	
        	
        	// Removing all the SeedData that is populated while Employer is created
        	//PayrollItem includes Compensation, Tax, Benefits, Deductions, TimeOff
        	List<PayrollItem>  payrollItems = entityService.findAll(PayrollItem.class);
        	for(PayrollItem pItem : payrollItems)
        		entityService.makeTransient(pItem);
        	
        	List<Jurisdiction>  jurisdictionList = entityService.findAll(Jurisdiction.class);
        	for (Jurisdiction jurisdiction: jurisdictionList)
        		entityService.makeTransient(jurisdiction);
        	
        	List<PayrollSchedule>  scheduleList = entityService.findAll(PayrollSchedule.class);
        	for (PayrollSchedule payrollSchedule: scheduleList)
        		entityService.makeTransient(payrollSchedule);
        	
        	//Removing the task list
        	List<com.intuit.sb.payroll.entity.model.Task>  tasks = entityService.findAll(com.intuit.sb.payroll.entity.model.Task.class);
        	for (com.intuit.sb.payroll.entity.model.Task task : tasks) {
        		entityService.makeTransient(task);
        	}        	
        	entityService.makeTransient(sEmployer);
        	entityService.makeTransient(ptsEmployer);
        }
        
        
    }    
            
    /**
	 * <P>
	 * Utility method to create Company With NonPayrollReadyWorker but no PayrollSchedule
	 * </P>
	 */
    public static void createNonPayrollReadyWorker() throws Exception {    	//Save NonPayroll-ready workers
     	String firstName = "Jim";
    	sWorker = WorkerTestUtils.createEmployee(firstName);    	
    	//Save Worker
    	sWorker = sWorkerComponent.addWorker(sWorker);    	
    }    
    
    /**
	 * <P>
	 * Utility method to create Company With PayrollSchedule and PayrollReadyWorker
	 * </P>
	 */
    public static void createPayrollReadyWorker() throws Exception {
    	//Save payroll-ready workers
     	String firstName = "Jim";
    	String workLocation = sSanFranciscoLocation;    	
    	sWorker = WorkerTestUtils.createAPayrollReadyWorker(firstName, workLocation);    	
    }
            
    /**
	 * <P>
	 * Utility method to create Company With PayrollSchedule and PayrollReadyWorker
	 * </P>
	 */
    public static void createPayrollReadyWorkerWithNoWithHolding() throws Exception {
    	//Save payroll-ready workers
     	String firstName = "Jim";
    	String workLocation = sSanFranciscoLocation;    	
    	sWorker = WorkerTestUtils.createAPayrollReadyWorkerWithNoWithHolding(firstName, workLocation);    	
    }   
    
    /**
	 * <P>
	 * Utility method to remove PayrollItems from Worker
	 * </P>
	 */
    public static void deleteWageItems() throws Exception{
    	sWorker = WorkerTestUtils.removeAllWorkerPayrollItems(sWorker);
    	//Save the worker
        sWorker = sWorkerComponent.updateWorker(sWorker);	
    }
    
    public static String generateRandomEIN() {

    	long randomEIN;
    	long min = 1000000;
    	long max = 9999999;
    	String [] validPrefixes = null;
    	try {
			 try {
				validPrefixes = sEmployerComponent.getAllValidEinPrefixes();
			} catch (IOException e) {
				e.printStackTrace();
			}
		} catch (ServiceFailureException e) {
			e.printStackTrace();
		}
        randomEIN = (long)Math.floor((Math.random() * (max - min + 1) )) + min;
        int index = (int)(Math.random()*validPrefixes.length);
        String prefix = validPrefixes[index];
        StringBuilder str = new StringBuilder();
        str.append(prefix).append(randomEIN);        
        return str.toString();          
    }
    
    public static String generateBankAccountNumber(){
    	
    	long randomEIN;
    	long min = 10000000;
    	long max = 99999999;        
        randomEIN = (long)Math.floor((Math.random() * (max - min + 1) )) + min;
        StringBuilder str = new StringBuilder();
        str.append(randomEIN);        
        return str.toString();  
    	
    }
    
    public static Payment createPaymentInMemory(PaymentType paymentType, String lawid, BigDecimal decimalAmount){
		Payment payment = new Payment();
		Calendar cal = DateUtil.createCalendar(2009, 1, 15);
		payment.setPaymentDate(cal.getTime());
		
		
		PaymentLineItem payLineItem = new PaymentLineItem();
		Money money = new Money();
		money.setAmount(decimalAmount);
		payLineItem.setAmount(money);
		payLineItem.setPaymentType(paymentType);
		
		PaymentLineItemSplit payItemSplit = new PaymentLineItemSplit();
		payItemSplit.setSplitAmount(money);
		
		List<PayrollItem>  payrollItems = entityService.findAll(PayrollItem.class);
		PayrollItem caSUIPayrollItem = LawUtils.getPayrollItem(payrollItems, lawid);
		payItemSplit.setPayrollItem(caSUIPayrollItem);
		payItemSplit.setDerived(Boolean.FALSE);
		payLineItem.addPaymentLineItemSplit(payItemSplit);
		payment.addPaymentLineItem(payLineItem);
		
		return payment;
	}
}
