package com.intuit.sb.payroll.capability.employer;

import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSContactPersonTypeEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;

public class PTSEmployerContactCapabilityTest extends EmployerComponentTestBase {

	private static EmployerPTSWorkflowComponent mEmployerPTSWorkflowComponentService;

	@BeforeClass
	public static void initializePTSEmployerContactCapabilityTestClass()
	{
		mEmployerPTSWorkflowComponentService = qaBootstrap.getEjbFromClass(EmployerPTSWorkflowComponent.class);
	}
	
	/**
	 * This test will ensure ManytoOne mapping between PTSEmployerContact and
	 * EmployerContactPerson
	 * 
	 * Add new PTSEmployerContact with different EmployerContact and verify the
	 * count for both PTSEmployerContact and EmployerContactPerson
	 * 
	 * @throws Exception
	 */
	@Test
	public void testPTSEmployerContactfunctionality() throws Exception {

		int existingEmpContactsCount = sEmployerComponent.getEmployerContactPersons().size();
		PTSEmployer ptsEmployer = mEmployerPTSWorkflowComponentService.getPTSEmployer();
		int existingPTSEmpContactsCount = ptsEmployer.getPTSContactPersons().size();

		EmployerContactPerson person1 = createEmployerContactPerson(
				"DeleteLastContactPerson", "Test",
				"DeleteLastContactPersonsTest", "650-555-1212");
		EmployerContactPerson person2 = createEmployerContactPerson("John",
				"Smith", "someTitle", "123-456-7890");
		
		EmployerContactPerson person3 = createEmployerContactPerson(
				"DeleteLastContactPerson", "Test",
				"DeleteLastContactPersonsTest", "650-555-1212");

		// Add 2 PTSContactPersons with their EmployerContact referencing to any
		// one of the EmployerContacts added
		// Both the PTSContactPersons have their EmployerContact referencing to
		// the different EmployerContact

		List<PTSContactPerson> ptsContactPerson = new ArrayList<PTSContactPerson>();
		PTSContactPerson contactPerson = new PTSContactPerson();
		contactPerson.setContactPersonType(PTSContactPersonTypeEnum.SECONDARY_CONTACT);
		contactPerson.setEmployerContact(person1);
		ptsContactPerson.add(contactPerson);

		PTSContactPerson contactPerson2 = new PTSContactPerson();
		contactPerson2.setContactPersonType(PTSContactPersonTypeEnum.SECONDARY_CONTACT);
		contactPerson2.setEmployerContact(person2);
		ptsContactPerson.add(contactPerson2);
		mEmployerPTSWorkflowComponentService.addEmployerContactPersons(ptsContactPerson);

		// verify the count of EmployerContacts
		AssertJUnit.assertEquals("EmployerContact count mis-match",	existingEmpContactsCount + 2,
					sEmployerComponent.getEmployerContactPersons().size());

		// verify the count of PTSEmployerContacts
		AssertJUnit.assertEquals("PTSEmployerContact count mis-match", existingPTSEmpContactsCount + 2, 
					ptsEmployer.getPTSContactPersons().size());

		List<PTSContactPerson> ptsContactPerson2 = new ArrayList<PTSContactPerson>();
		PTSContactPerson contactPerson3 = new PTSContactPerson();
		contactPerson3.setContactPersonType(PTSContactPersonTypeEnum.PRIMARY_CONTACT);
		// Already added EmployerContactPerson
		contactPerson3.setEmployerContact(person3);
		ptsContactPerson2.add(contactPerson3);
		mEmployerPTSWorkflowComponentService.addEmployerContactPersons(ptsContactPerson2);

		// verify the count of EmployerContacts
		AssertJUnit.assertEquals("EmployerContact count mis-match", existingEmpContactsCount + 3, 
					sEmployerComponent.getEmployerContactPersons().size());

		// verify the count of PTSEmployerContacts
		AssertJUnit.assertEquals("PTSEmployerContact count mis-match", existingPTSEmpContactsCount + 3, 
					ptsEmployer.getPTSContactPersons().size());

	}

	/**
	 * Test first add 3 same EmployerContactPerson as PTSContactPerson and then delete them one by one
	 * until all of them gets deleted.
	 * 
	 * @throws Exception
	 */
	@Test
	public void testDeletePTSEmployerContact()
			throws Exception {

		int existingEmpContactsCount = sEmployerComponent.getEmployerContactPersons().size();
		PTSEmployer ptsEmployer = mEmployerPTSWorkflowComponentService.getPTSEmployer();
		int existingPTSEmpContactsCount = ptsEmployer.getPTSContactPersons().size();
		

		EmployerContactPerson person1 = createEmployerContactPerson(
				"DeleteLastContactPerson1", "Test1",
				"DeleteLastContactPersonsTest1", "650-555-1212");
		
		EmployerContactPerson person2 = createEmployerContactPerson(
				"DeleteLastContactPerson2", "Test2",
				"DeleteLastContactPersonsTest2", "650-555-1212");
		
		EmployerContactPerson person3 = createEmployerContactPerson(
				"DeleteLastContactPerson3", "Test3",
				"DeleteLastContactPersonsTest3", "650-555-1212");

		// Add 3 PTSContactPersons with their EmployerContact referencing to
		// different EmployerContactPersons

		List<PTSContactPerson> ptsContactPerson = new ArrayList<PTSContactPerson>();
		PTSContactPerson contactPerson1 = new PTSContactPerson();
		contactPerson1.setContactPersonType(PTSContactPersonTypeEnum.SECONDARY_CONTACT);
		contactPerson1.setEmployerContact(person1);
		ptsContactPerson.add(contactPerson1);
		
		// This will persist the EmployerContactPerson
		mEmployerPTSWorkflowComponentService.addEmployerContactPersons(ptsContactPerson);
		
		// verify the count of EmployerContacts
		AssertJUnit.assertEquals("EmployerContact count mis-match",existingEmpContactsCount + 1,
				 				sEmployerComponent.getEmployerContactPersons().size());

		// verify the count of PTSEmployerContacts
		AssertJUnit.assertEquals("PTSEmployerContact count mis-match",existingPTSEmpContactsCount + 1,
				 				ptsEmployer.getPTSContactPersons().size());
		
		// Persist remaining 2 with same EmployerContactPerson (assumption made here is that EmployerContactPerson person1
		// will now point to the persisted entity
		ptsContactPerson = new ArrayList<PTSContactPerson>();
		
		PTSContactPerson contactPerson2 = new PTSContactPerson();
		contactPerson2.setContactPersonType(PTSContactPersonTypeEnum.SECONDARY_CONTACT);
		contactPerson2.setEmployerContact(person2);
		ptsContactPerson.add(contactPerson2);
		
		PTSContactPerson contactPerson3 = new PTSContactPerson();
		contactPerson3.setContactPersonType(PTSContactPersonTypeEnum.PRIMARY_CONTACT);
		// Already added EmployerContactPerson
		contactPerson3.setEmployerContact(person3);
		ptsContactPerson.add(contactPerson3);
		mEmployerPTSWorkflowComponentService.addEmployerContactPersons(ptsContactPerson);

		// verify the count of EmployerContacts
		AssertJUnit.assertEquals("EmployerContact count mis-match", existingEmpContactsCount + 3, 
					sEmployerComponent.getEmployerContactPersons().size());

		// verify the count of PTSEmployerContacts
		AssertJUnit.assertEquals("PTSEmployerContact count mis-match", existingPTSEmpContactsCount + 3, 
				ptsEmployer.getPTSContactPersons().size());

		// delete one PTSContactPerson which has reference to
		// EmployerContactPerson (say A)
		List<PTSContactPerson> ptsContactPersonToDelete = new ArrayList<PTSContactPerson>();
		ptsContactPersonToDelete.add(contactPerson1);
		mEmployerPTSWorkflowComponentService.deleteEmployerContactPersons(ptsContactPersonToDelete);

		// verify that the EmployerContact Person is not deleted, since another
		// PTSContactPerson also has its EmployerContact referencing to A
		AssertJUnit.assertEquals("EmployerContact count mis-match", existingEmpContactsCount + 2,
				sEmployerComponent.getEmployerContactPersons().size());

		// verify that the PTSEmployerContact Person is not deleted, since
		// another PTSContactPerson also has its EmployerContact referencing to
		// A
		AssertJUnit.assertEquals("EmployerContact count mis-match",
				existingPTSEmpContactsCount + 2,
				mEmployerPTSWorkflowComponentService.getPTSEmployer()
						.getPTSContactPersons().size());

		// delete the other PTSContactPerson which has reference to
		// EmployerContactPerson (say A)
		ptsContactPersonToDelete = new ArrayList<PTSContactPerson>();
		ptsContactPersonToDelete.add(contactPerson2);
		mEmployerPTSWorkflowComponentService
				.deleteEmployerContactPersons(ptsContactPersonToDelete);
		// verify that the EmployerContact Person (say A) got deleted, because
		// it has no PTSContactPerson`s EmployerContact referencing it
		AssertJUnit.assertEquals("EmployerContact count mis-match",existingEmpContactsCount + 1, 
					sEmployerComponent.getEmployerContactPersons().size());

		// verify that the PTSEmployerContact Person is not deleted, since
		// another PTSContactPerson also has its EmployerContact referencing to
		// A
		AssertJUnit.assertEquals("EmployerContact count mis-match", existingPTSEmpContactsCount + 1,
				mEmployerPTSWorkflowComponentService.getPTSEmployer()
						.getPTSContactPersons().size());

		try {
			ptsContactPersonToDelete = new ArrayList<PTSContactPerson>();
			ptsContactPersonToDelete.add(contactPerson3);
			mEmployerPTSWorkflowComponentService.deleteEmployerContactPersons(ptsContactPersonToDelete);
		}catch (ValidationException valExp){
			// This is expected
			// Don't do anything since we tried to delete the last contact person
		}

	}
	
	/**
	 * Delete all PTSContactPerson and ensure that associated EmployerContactPerson also
	 * gets deleted.
	 * @throws Exception
	 */
	
	@Test
	public void deleteAllPTSContactPerson() throws Exception {
		
		PTSEmployer ptsEmployer = mEmployerPTSWorkflowComponentService.getPTSEmployer();
        int existingPTSEmpContactsCount = ptsEmployer.getPTSContactPersons().size();
        int existEmployerContactPersonCount = ptsEmployer.getEmployer().getEmployerContactPersons().size();
        int standaloneEmpContactPersonCount = (existEmployerContactPersonCount - existingPTSEmpContactsCount);
        
		// Add 2 PTSContactPersons with their EmployerContact referencing to
		// ONLY one of the EmployerContacts added
		
        EmployerContactPerson person1 = createEmployerContactPerson(
				"DeleteLastContactPerson1", "Test1",
				"DeleteLastContactPersonsTest1", "650-555-1212");
		
		EmployerContactPerson person2 = createEmployerContactPerson(
				"DeleteLastContactPerson2", "Test2",
				"DeleteLastContactPersonsTest2", "650-555-1212");
		

		List<PTSContactPerson> ptsContactPerson = new ArrayList<PTSContactPerson>();
		PTSContactPerson contactPerson = new PTSContactPerson();
		contactPerson.setContactPersonType(PTSContactPersonTypeEnum.SECONDARY_CONTACT);
		contactPerson.setEmployerContact(person1);
		ptsContactPerson.add(contactPerson);

		PTSContactPerson contactPerson2 = new PTSContactPerson();
		contactPerson2.setContactPersonType(PTSContactPersonTypeEnum.SECONDARY_CONTACT);
		contactPerson2.setEmployerContact(person2);
		ptsContactPerson.add(contactPerson2);
		mEmployerPTSWorkflowComponentService.addEmployerContactPersons(ptsContactPerson);
		
		AssertJUnit.assertEquals("PTSEmployerContact count mis-match",
				existingPTSEmpContactsCount + 2, ptsEmployer.getPTSContactPersons().size());
		
		// Exercise
		mEmployerPTSWorkflowComponentService.deleteAllContactPersons();
		
		// Called a named query to trigger some internal hibernate session work
		ptsEmployer = mEmployerPTSWorkflowComponentService.getPTSEmployer();
		AssertJUnit.assertEquals("PTSEmployerContact count mis-match", 0, ptsEmployer.getPTSContactPersons().size());
		AssertJUnit.assertEquals("EmployerContact count mis-match", standaloneEmpContactPersonCount+ 0, ptsEmployer.getEmployer().getEmployerContactPersons().size());
		
	}
	
	// TODO add Update tests especially PC becoming PP
}
