package com.intuit.sb.payroll.capability.employer;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Map;

import org.testng.Assert;
import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculationRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunCommitRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponentTestBase;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.utils.PtdUtils;

public class TaxPaymentsLiabilitiesTest extends WorkerPaymentsComponentTestBase {

	private BigDecimal mHourlyRate = new BigDecimal("10.0");
	private BigDecimal mBonusAmount = new BigDecimal("200.0");

	@SuppressWarnings("unused")
	@Test
	public void testLiabilities() throws Exception {
    	PayrollRun payrollRun = null;
    	try {
        	payrollRun = getCommitedPayrollRun();
        	
        	DateRange dateRange = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER1, 2008);
        	Map<String, Money> map = sEmployerComponent.getLiabilitiesFromExistingPayrollRuns(dateRange);
        	Assert.assertNotNull(map, "Map should not be null");
        	Assert.assertEquals(10, map.size(), "There should be TEN (10) liability items");
        	Assert.assertEquals(new Money("22.55"), map.get(QboeLawIdConstants.sCAPITLawId), 
        		"California Personal Income Tax should be $122");	// California Personal Income Tax
        	Assert.assertEquals(new Money("71.30"), map.get(QboeLawIdConstants.sSST_EELawId), 
				"EE Social Security Tax should be $122");	// EE Social Security Tax
        	Assert.assertEquals(new Money("71.30"), map.get(QboeLawIdConstants.sSST_ERLawId), 
				"ER Social Security Tax should be $122");	// ER Social Security Tax
        	Assert.assertEquals(new Money("16.68"), map.get(QboeLawIdConstants.sMED_EELawId), 
				"EE Medicare Tax should be $122");	// EE Medicare Tax
        	Assert.assertEquals(new Money("16.68"), map.get(QboeLawIdConstants.sMED_ERLawId), 
				"ER Medicare Tax should be $122");	// ER Medicare Tax
        	Assert.assertEquals(new Money("9.20"), map.get(QboeLawIdConstants.sFUTALawId), 
				"Federal Unemployment Insurance should be $122");	// Federal Unemployment Insurance
        	Assert.assertEquals(new Money("9.20"), map.get(QboeLawIdConstants.sCASDILawId), 
				"California State Disability Insurance should be $122");	// California State Disability Insurance
        	Assert.assertEquals(new Money("39.10"), map.get(QboeLawIdConstants.sCASUILawId), 
				"California State Unemployment Insurance should be $122");	// California State Unemployment Insurance
        	Assert.assertEquals(new Money("1.15"), map.get(QboeLawIdConstants.sCAETTLawId), 
				"California Employment Training Tax should be $122");	// California Employment Training Tax
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
    	} catch (Exception e) {
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	}
    }
	
	protected PayrollRun getCommitedPayrollRun() throws Exception {
		PayrollRun payrollRun = null;
    	PayrollSchedule pSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.JANUARY, 14),
                sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(pSchedule);
		List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
		Long sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
		PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
		payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
		PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
		paycheckCalculationRequests.add(calcRequest);
		PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
		payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
		payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
		
		// Set the Payroll system date to the payroll run date
		setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
		//Exercise
		payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
		
		return payrollRun;
	}
}
