package com.intuit.sb.payroll.capability.payrollRun;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.PaycheckMapper;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.PayrollRunMapper;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculationRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunCommitRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunCreateRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponentTestBase;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.EmployerAdjLineItem;
import com.intuit.sb.payroll.entity.model.EmployerAdjustment;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.LawId;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PaycheckLineItemSortSequenceEnum;
import com.intuit.sb.payroll.entity.model.PaycheckWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.TaxServiceStatusEnum;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyBankInfoAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompensationTransactionAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.DateAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.DeductionTransactionAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.LiabilityTransactionAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PaycheckAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollRunAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollRunTypeAdapterEnum;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollTaxAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.ServiceCodeAdapterEnum;

public class PayrollRunMapperTest extends WorkerPaymentsComponentTestBase {

	private static Long sWorkerIds[] = {0L, 0L, 0L, 0L, 0L};
	private BankAccount mBankAccount = null;

	@BeforeMethod
	public void initialize() throws Exception {
		sWorkerIds[0] = WorkerTestUtils.createSalaryPayrollReadyWorker("Abdul", sMountainViewLocation);
		sWorkerIds[1] = WorkerTestUtils.createSalaryPayrollReadyWorker("Lawrence", sMountainViewLocation);
		sWorkerIds[2] = WorkerTestUtils.createSalaryPayrollReadyWorker("Levi", sMountainViewLocation);
		sWorkerIds[3] = WorkerTestUtils.createSalaryPayrollReadyWorker("Renaldo", sMountainViewLocation);
		sWorkerIds[4] = WorkerTestUtils.createSalaryPayrollReadyWorker("Blooper", sMountainViewLocation);
		mBankAccount = createBankAccount();
	}

	protected PayrollRun createAdjustmentPayrollRun() throws Exception {
    	//Setup
    	PayrollSchedule pSchedule = createBiweeklySchedule(DateUtil.createDate(sTaxYear, Calendar.JANUARY, 14),
    	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(pSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	for (int i = 0; i < sWorkerIds.length; i++) {
	    	PaycheckCalculationRequest calcRequest = createSalaryPaycheckCalculationRequest(sWorkerIds[i],
	    					payrollRunInfo, PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
	    	paycheckCalculationRequests.add(calcRequest);
    	}
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
    	//Exercise
    	PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

     	return payrollRun;
	}

	@Test
	public void testPayrollRunMapper() throws Exception {
		try {
			createPayrollRun();
			
			PayrollRun payrollRun = createAdjustmentPayrollRun();
	        BankAccount[] savedBA = sEmployerComponent.addBankAccounts(new BankAccount[]{mBankAccount});
	        mBankAccount = savedBA[0];

	        EmployerPTSWorkflowComponent empPTSComp = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
	        // create the required DTO
	        PTSEmployer ptsEmployer = empPTSComp.getPTSEmployer();
	        PayrollRunAdapterDTO prDto = PayrollRunMapper.mapToPayrollRunDTO(ptsEmployer, payrollRun);

//	        Now validate the payroll run id
	        Assert.assertEquals(prDto.getSourcePayrollRunId(), Long.toString(payrollRun.getId()), "PayrollRun Id does not match");

//	        Validate the payroll run paycheck date against the DTO date
	        validateDtoDate(prDto.getPaycheckDate(), payrollRun.getPayDate());
//	        Validate the payroll run type mapping
	        PayrollRunTypeAdapterEnum payrollType;
	        if (ptsEmployer.getCurrentTaxServiceStatus() == TaxServiceStatusEnum.ACTIVATION_ACTIVE_ST_ACCEPTED){
	        	payrollType = PayrollRunTypeAdapterEnum.REGULAR;
	        }else{
	        	payrollType = PayrollRunTypeAdapterEnum.HISTORICAL;
	        }
	        Assert.assertEquals(prDto.getPayrollRunType(), payrollType, "PayrollRunTypeAdapterEnum does not match");
	        
	        validateBankInfo(prDto);
	        validatePayrollRunDtoTaxes(prDto, payrollRun);
		} finally {
//	        Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
		}
	}
	
	
    protected PayrollRun createPayrollRun() throws Exception {
        //Setup
        PayrollSchedule pSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
                                           sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(pSchedule);
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());

        List<Long> workerIdList = new ArrayList<Long>();
        workerIdList.add(sWorkerIds[4]);

        //Create a PayrollRunCreateRequest
        PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);

        //Execute
        PayrollRun payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);

        return payrollRun;
    }

    private void validateBankInfo(PayrollRunAdapterDTO prDto) {
		List<CompanyBankInfoAdapterDTO> banks = prDto.getCompanyBankInfo();
		Assert.assertEquals(banks.size(), 1, "There should be only ONE bank account in the PayrollRunAdapterDTO");
		CompanyBankInfoAdapterDTO bankDto = banks.get(0);
		Assert.assertNotNull(bankDto, "CompanyBankInfoAdapterDTO has NULL bank account info");
		Assert.assertEquals(bankDto.getSourceCompanyBankAccountId(), Long.toString(mBankAccount.getId()), 
				"Bank Account numbers are not matching");
		Assert.assertEquals(bankDto.getServiceCode(), ServiceCodeAdapterEnum.TAX, 
				"Bank Account Service codes do not match");
	}
	
	/**
	 * Create a random BankAccount object with valid values.
	 * @return BankAccount object
	 */
	private BankAccount createBankAccount() {
		BankAccount acc = new BankAccount();
		acc.setAccountNo("123456");
		acc.setAccountType(BankAccountTypeEnum.SAVINGS);
		acc.setBankName("Bank of Jhumri Thalayya");
		acc.setName("JT Bank");
		acc.setRoutingNo("110000000");

		return acc;
	}

	/**
	 * Validate the PayrollRunAdapterDTO taxes against the PayrollRun EmployerAdjustment items.
	 * For every PayrollTaxAdapterDTO object, validate the fields against the original EmployerAdjustment
	 * and EmployerAdjustment line item
	 * Validate all the fields
	 * @param prDto
	 * @param payrollRun
	 * @return
	 */
	private void validatePayrollRunDtoTaxes(PayrollRunAdapterDTO prDto, PayrollRun payrollRun) {
		List<PayrollTaxAdapterDTO> dtoTaxes = prDto.getPayrollTaxes();
		for (PayrollTaxAdapterDTO dtoTax : dtoTaxes) {
			EmployerAdjLineItem lineitem = null;
			EmployerAdjustment adj = null;
			Set<EmployerAdjustment> adjustments = payrollRun.getEmployerAdjustments();
			for (EmployerAdjustment adjustment : adjustments) {
				DateAdapterDTO dateDto = dtoTax.getEffectiveDate();
				if (dateCompare(getJavaDateFromDateAdapterDTO(dateDto), adjustment.getEffectiveDate())) {
					Set<EmployerAdjLineItem> lineitems = adjustment.getEmployerAdjLineItems();					
					for (EmployerAdjLineItem item : lineitems) {
						if (dtoTax.getLawId().equals(item.getLawId().getValue())) {
							lineitem = item;
							adj = adjustment;
							break;
						}
					}
				}
			}

			Assert.assertNotNull(lineitem, "Could not find PayrollRun EmployerAdjustment lineitem for DTO element");
			Assert.assertNotNull(adj, "Could not find PayrollRun EmployerAdjustment for DTO element");
			Assert.assertEquals(dtoTax.getAmount(), lineitem.getAdjustmentAmount().getAmount(), "Tax Amount mismatch");
			validateDtoDate(dtoTax.getEffectiveDate(), adj.getEffectiveDate());
		}
	}

	private Date getJavaDateFromDateAdapterDTO(DateAdapterDTO dateFromDto) {
        int day  = dateFromDto.getDay();
        int mon  = dateFromDto.getMonth()-1;
        int year = dateFromDto.getYear();
		Calendar calendar = LocaleGuru.getInstance().getCalendar();
		calendar.set(year, mon, day);
		Date dateFromMapper = calendar.getTime();

		return dateFromMapper;
	}

	private void validateDtoDate(DateAdapterDTO dateFromDto, Date pDate) {
//      Validate the date from the Date DTO
		Date dateFromMapper = getJavaDateFromDateAdapterDTO(dateFromDto);
		Assert.assertEquals(dateCompare(dateFromMapper, pDate), true, "Pay Date not matching");
	}

	/** Compare dates using only the month, day and year - ignore hour/min/sec */
	private boolean dateCompare(Date d1, Date d2) {
		Calendar c1 = new GregorianCalendar();
		c1.setTime(d1);
		Calendar c2 = new GregorianCalendar();
		c2.setTime(d2);

		boolean monthCompare = c1.get(Calendar.MONTH) == c2.get(Calendar.MONTH);
		boolean dayCompare   = c1.get(Calendar.DAY_OF_MONTH) == c2.get(Calendar.DAY_OF_MONTH);
		boolean yearCompare  = c1.get(Calendar.YEAR) == c2.get(Calendar.YEAR);

		return monthCompare && dayCompare && yearCompare;
	}

    @Test
    public void testPaycheckMapper() throws Exception{
        //create a Paycheck
        Paycheck testPaycheck = createPaycheck();

        //map the paycheck created to PaycheckAdapterDTO
        PaycheckAdapterDTO payCheckAdapterDTO = PaycheckMapper.mapToPaycheck(testPaycheck);

        //verify the mapped paycheckAdapterDTO
        verifyPayCheckAdapterDTO(payCheckAdapterDTO, testPaycheck);
    }

    protected Paycheck createPaycheck(){
        Paycheck paycheck = new Paycheck();

        paycheck.setGrossAmount(getMoney());
        paycheck.setNetAmount(getMoney());
        paycheck.setDeductionAmount(getMoney());
        paycheck.setWorkerPdTaxes(getMoney());
        paycheck.setEmployerPdTaxes(getMoney());
        paycheck.setWorkerPdBenefits(getMoney());
        paycheck.setEmployerPdBenefits(getMoney());

        // add a few paycheck line items
        paycheck.addPaycheckLineItem(createPayCheckLineItem(PaycheckLineItemSortSequenceEnum.EMPLOYEEPAIDTAX));
        paycheck.addPaycheckLineItem(createPayCheckLineItem(PaycheckLineItemSortSequenceEnum.EMPLOYERPAIDTAX));
        paycheck.addPaycheckLineItem(createPayCheckLineItem(PaycheckLineItemSortSequenceEnum.WAGES));
        paycheck.addPaycheckLineItem(createPayCheckLineItem(PaycheckLineItemSortSequenceEnum.PAIDTIMEOFF));
        paycheck.addPaycheckLineItem(createPayCheckLineItem(PaycheckLineItemSortSequenceEnum.POSTTAXDEDUCTION));
        paycheck.addPaycheckLineItem(createPayCheckLineItem(PaycheckLineItemSortSequenceEnum.PRETAXDEDUCTION));

        // TODO - these paycheck line items are currently not mapped to the PaycheckAdapterDTO

//      paycheck.addPaycheckLineItem(createPayCheckLineItem(PaycheckLineItemSortSequenceEnum.POSTTAXADJUSTMENT));
//      paycheck.addPaycheckLineItem(createPayCheckLineItem(PaycheckLineItemSortSequenceEnum.PRETAXADJUSTMENT));
//      paycheck.addPaycheckLineItem(createPayCheckLineItem(PaycheckLineItemSortSequenceEnum.NOTSET));
//
        //set up the payperiod
        DateRange payPeriod = new DateRange();
        payPeriod.setStartDate(DateUtil.createDate(2009, 06, 01));
        payPeriod.setEndDate(DateUtil.createDate(2009, 06, 11));

        paycheck.setPayPeriod(payPeriod);
        paycheck.setDistributionMethod(PayrollDistributionMethodEnum.CASH);
        paycheck.setWorkflowStatus(PaycheckWorkflowStatusEnum.COMMITTED);
        paycheck.setWorker(createWorker());

        return paycheck;
    }

    protected void verifyPayCheckAdapterDTO(PaycheckAdapterDTO payCheckAdapterDTO, Paycheck testPaycheck){
        Assert.assertEquals(payCheckAdapterDTO.getPaycheckNetAmount(), testPaycheck.getNetAmount().getAmount(), "Pay Check Net amount is not mapped correctly");
        Assert.assertEquals(payCheckAdapterDTO.getSourcePaycheckId(), Long.toString(testPaycheck.getId()), "Source Pay Check ID is not mapped correctly");
        Assert.assertEquals(payCheckAdapterDTO.getSourceEmployeeId(), Long.toString(testPaycheck.getWorker().getId()), "Source Employee ID is not mapped correctly");
        verifyDate(payCheckAdapterDTO.getPayPeriodEndDate(), testPaycheck.getPayPeriod().getEndDate());
        verifyDate(payCheckAdapterDTO.getPayPeriodBeginDate(), testPaycheck.getPayPeriod().getStartDate());


        // put all the paycheckLine items in a hashmap so that it will be easy to verify
        HashMap<String , PaycheckLineItem> paycheckLineItems = new HashMap<String, PaycheckLineItem>();
        for (PaycheckLineItem payCheckLineItem : testPaycheck.getPaycheckLineItems()) {
            String id = ((Long)payCheckLineItem.getId()).toString();
            paycheckLineItems.put(id, payCheckLineItem);
        }

        // create hashMap for each of the Transactions
        HashMap<String, Object> transactions = new HashMap<String, Object>();

        //verify the compensationItems
        for (CompensationTransactionAdapterDTO compensationTransactionDTO: payCheckAdapterDTO.getCompensationTransactions()) {
            PaycheckLineItem paycheckLineItem = paycheckLineItems.get(compensationTransactionDTO.getSourceCompensationId());
            transactions.put(compensationTransactionDTO.getSourceCompensationId(), compensationTransactionDTO);
            verifyCompensationTransactionAdapterDTO(compensationTransactionDTO, paycheckLineItem);
        }

        //verify the Deduction items
        for(DeductionTransactionAdapterDTO deductionTransactionDTO: payCheckAdapterDTO.getDeductionTransactions()){
            verifyDeductionTransactionAdapterDTO(deductionTransactionDTO, paycheckLineItems.get(deductionTransactionDTO.getSourceDeductionId()));
            transactions.put(deductionTransactionDTO.getSourceDeductionId(),deductionTransactionDTO);
        }

        //verify the Deduction items
        for(LiabilityTransactionAdapterDTO liabilityTransactionDTO: payCheckAdapterDTO.getLiabilityTransactions()){
            for (PaycheckLineItem payCheckLineItem : testPaycheck.getPaycheckLineItems()) {
                if(payCheckLineItem.getLawId().getValue().equals(liabilityTransactionDTO.getLawId())){
                    verifyLiabililtyTransactionAdapterDTO(liabilityTransactionDTO, payCheckLineItem);
                    transactions.put(liabilityTransactionDTO.getLawId(), liabilityTransactionDTO);
                }
            }
        }

        // verify whether all the paycheck line items are mapped
        int numberOfTransactionsInPaycheckAdapterDTO = payCheckAdapterDTO.getCompensationTransactions().size()
                                                    +  payCheckAdapterDTO.getDeductionTransactions().size()
                                                    +  payCheckAdapterDTO.getLiabilityTransactions().size();
        Assert.assertEquals(numberOfTransactionsInPaycheckAdapterDTO, testPaycheck.getPaycheckLineItems().size(), "All the paycheck line items in the paycheck are not mapped to the paycheckAdapterDTO");

        //verify that the paycheck line items are mapped to the corresponding transaction
        for (PaycheckLineItem payCheckLineItem : testPaycheck.getPaycheckLineItems()) {
            Object object=null;
            switch (payCheckLineItem.getSortSequence()) {
            case WAGES:
            case PAIDTIMEOFF:
                object = transactions.get(String.valueOf(payCheckLineItem.getId()));
                Assert.assertEquals(object instanceof CompensationTransactionAdapterDTO, true, "PaycheckLineItem is not mapped correctly");
                break;
            case PRETAXDEDUCTION:
            case POSTTAXDEDUCTION:
                object = transactions.get(String.valueOf(payCheckLineItem.getId()));
                Assert.assertEquals(object instanceof DeductionTransactionAdapterDTO, true, "PaycheckLineItem is not mapped correctly");
                break;
            case EMPLOYERPAIDTAX:
            case EMPLOYEEPAIDTAX:
                object = transactions.get(payCheckLineItem.getLawId().getValue());
                Assert.assertEquals(object instanceof LiabilityTransactionAdapterDTO, true, "PaycheckLineItem is not mapped correctly");
                break;
            }
        }
    }

    // verifies the mapping of values for the paychecklineItem and the compensationTransactionDTO
    protected void verifyCompensationTransactionAdapterDTO(CompensationTransactionAdapterDTO compensationTransactionDTO, PaycheckLineItem payCheckLineItem){
        Assert.assertEquals(compensationTransactionDTO.getSourceCompensationId(), ((Long)(payCheckLineItem.getId())).toString(), "Compensation Id is not mapped correctly");
        Assert.assertEquals(compensationTransactionDTO.getCompensationAmount(), payCheckLineItem.getValue().getAmount(), "Compensation Amount is not mapped correctly");
        Assert.assertEquals(compensationTransactionDTO.getCompensationHoursWorked().doubleValue(), payCheckLineItem.getQuantity().getValue1AsQuantity(), "Compensation Hours worked is not mapped correctly");
    }

    // verifies the mapping of values for the paychecklineItem and the deductionTransactionDTO
    protected void verifyDeductionTransactionAdapterDTO(DeductionTransactionAdapterDTO deductionTransactionDTO, PaycheckLineItem payCheckLineItem){
        Assert.assertEquals(deductionTransactionDTO.getSourceDeductionId(), ((Long)(payCheckLineItem.getId())).toString(), "Deduction Id is not mapped correctly");
        Assert.assertEquals(deductionTransactionDTO.getDeductionAmount(), payCheckLineItem.getValue().getAmount(), "Deduction Amount is not mapped correctly");
    }

    // verifies the mapping of values for the paychecklineItem and the liabilityTransactionDTO
    protected void verifyLiabililtyTransactionAdapterDTO(LiabilityTransactionAdapterDTO liabilityTransactionDTO, PaycheckLineItem payCheckLineItem){
        Assert.assertEquals(liabilityTransactionDTO.getLawId(), payCheckLineItem.getLawId().getValue(), "Liability Id is not mapped correctly");
        Assert.assertEquals(liabilityTransactionDTO.getLiabilityAmount(), payCheckLineItem.getValue().getAmount(), "Deduction Amount is not mapped correctly");
        Assert.assertEquals(liabilityTransactionDTO.getLiabilityTaxableWagesAmount(), payCheckLineItem.getIncomeSubjectToTax().getAmount());
        Assert.assertEquals(liabilityTransactionDTO.getLiabilityTotalWagesAmount(), payCheckLineItem.getWageBase().getAmount());

            //what has to be mapped to tips
        //Assert.assertEquals(liabilityTransactionDTO.getLiabilityTipsAmount(),,"Tips is not mapped correctly");
    }


    // verifies the mapping between the Date object and the dateAdapterDTO
    protected void verifyDate(DateAdapterDTO dateAdapterDTO, Date date){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        Assert.assertEquals(dateAdapterDTO.getDay(), calendar.get(Calendar.DAY_OF_MONTH), "Day is not mapped correctly");
        Assert.assertEquals(dateAdapterDTO.getMonth(), calendar.get(Calendar.MONTH)+ 1, "Month is not mapped correctly");
        Assert.assertEquals(dateAdapterDTO.getYear(), calendar.get(Calendar.YEAR), "Year is not mapped correctly");
    }

    // creates a worker
    private Worker createWorker(){
        Worker worker = new Worker();
        worker.setEmploymentStatus(EmploymentStatusEnum.EMPLOYED);
        worker.setRelationshipToOwner(RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER);
        worker.setWorkerId(new Long("1234567").toString());
        worker.setDateOfBirth(sAge40BirthDate.getTime());

        PersonName personName = new PersonName();
        personName.setFirstName("Real");
        personName.setLastName("Employee");
        worker.setPersonName(personName);

        DateRange employmentPeriod = new DateRange();
        employmentPeriod.setStartDate(DateUtil.createDate(2009, 06, 1));
        employmentPeriod.setEndDate(null);
        worker.setEmploymentPeriod(employmentPeriod);
        GlobalPostalAddress taxResidenceGlobalAddress = new GlobalPostalAddress();
        taxResidenceGlobalAddress.setCountrySubDivisionCode("CA");
        worker.setTaxResidenceAddress(taxResidenceGlobalAddress);
        return worker;
    }

    private Money getMoney(){
        Money value = new Money();
        value.setAmount(new BigDecimal(12345));
        return value;
    }

    //creates a paycheckLineItem
    private PaycheckLineItem createPayCheckLineItem(PaycheckLineItemSortSequenceEnum sortSequence){
        PaycheckLineItem payCheckLineItem = new PaycheckLineItem();
        Datum quantity = new Datum();
        quantity.setValue1AsQuantity(1.1);
        payCheckLineItem.setQuantity(quantity);

        payCheckLineItem.setValue(getMoney());
        payCheckLineItem.setSortSequence(sortSequence);

        Datum rate = new Datum();
        rate.setValue1AsPercentage(new Double(1.2));
        payCheckLineItem.setRate(rate);

        Long id = generateID();
        payCheckLineItem.setId(id);

        LawId lawId = new LawId();
        lawId.setValue(id.toString());
        payCheckLineItem.setLawId(lawId);

        return payCheckLineItem;
    }

    private Long generateID(){
        Double id = Math.random() * 10000;
        return id.longValue();
    }
}
