package com.intuit.sb.payroll.capability.pto;

import java.math.BigDecimal;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.calcengine.PaycheckCalcTestBase;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.compensation.CompensationComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.PaidTimeOff;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualStartTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffDtl;
import com.intuit.sb.payroll.entity.model.PaidTimeOffTypeEnum;
import com.intuit.sb.payroll.entity.model.PtoBaseRateTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.pto.PtoDatumHelper;

public class PaidTimeOffComponentTest extends BasicTestBase {

	protected static PaidTimeOffComponent sPaidTimeOffComponent;
	protected static CompensationComponent sCompensationComponent;
	
	@BeforeClass
	public static void initializePaidTimeOff()
	{
		sPaidTimeOffComponent = qaBootstrap.getEjbFromClass(PaidTimeOffComponent.class);
		sCompensationComponent = qaBootstrap.getEjbFromClass(CompensationComponent.class);
	}
	
	@BeforeMethod
	public void setUp()	throws Exception {
		super.setUp();
		refreshWorker();
	}

	/** This basically tests the seed data and access to it. */
	@Test
	public void testFindPaidTimeOff() throws Exception {
		PaidTimeOff paidTimeOff = sPaidTimeOffComponent.findPaidTimeOff(PaidTimeOffTypeEnum.SICK);
		AssertJUnit.assertNotNull(paidTimeOff);
		AssertJUnit.assertEquals(PaidTimeOffTypeEnum.SICK, paidTimeOff.getPaidTimeOffType());
		AssertJUnit.assertEquals("SICKACCRUE", paidTimeOff.getName());
		AssertJUnit.assertEquals("1000020", paidTimeOff.getLawId().getValue());
		
		paidTimeOff = sPaidTimeOffComponent.findPaidTimeOff(PaidTimeOffTypeEnum.VACATION);
		AssertJUnit.assertNotNull(paidTimeOff);
		AssertJUnit.assertEquals(PaidTimeOffTypeEnum.VACATION, paidTimeOff.getPaidTimeOffType());
		AssertJUnit.assertEquals("VACATIONACCRUE", paidTimeOff.getName());
		AssertJUnit.assertEquals("1000018", paidTimeOff.getLawId().getValue());
	}
	
	@Test
	public void testCreatePaidTimeOffDtl() throws Exception {
		PaidTimeOffDtl paidTimeOffDtl = sPaidTimeOffComponent.createPaidTimeOffDtl(PaidTimeOffTypeEnum.SICK);
		AssertJUnit.assertNotNull(paidTimeOffDtl);
		AssertJUnit.assertEquals(PaidTimeOffTypeEnum.SICK, paidTimeOffDtl.getPaidTimeOffType());
		AssertJUnit.assertEquals("SICKACCRUE", paidTimeOffDtl.getName());
		AssertJUnit.assertEquals("1000020", paidTimeOffDtl.getPayrollItem().getLawId().getValue());
	}
	
	@Test
	public void testAddPaidTimeOffDtl() throws Exception {
		//Setup a sick PTO and an hourly wage set it to handle sick PTO.
		PaidTimeOffDtl paidTimeOffDtl = createPto(PaidTimeOffTypeEnum.SICK);
		CompensationDtl hourlyWage = PaycheckCalcTestBase.createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
																						new BigDecimal(10.00), 0d);
		hourlyWage.setPtoBaseRateType(PtoBaseRateTypeEnum.SICK);
		sWorker = sCompensationComponent.addCompensationToWorker(hourlyWage, sWorker);
		
		//Exercise
		sWorker = sPaidTimeOffComponent.addPaidTimeOffDtl(sWorker, paidTimeOffDtl);
		//Find the PTO
		boolean foundPto = false;
		Set<PaidTimeOffDtl> paidTimeOffDtls = sWorker.getPaidTimeOffDtls();
		for (PaidTimeOffDtl paidTimeOffDtl2 : paidTimeOffDtls) {
			if (paidTimeOffDtl2.getPaidTimeOffType() == PaidTimeOffTypeEnum.SICK) {
				foundPto = true;
				break;
			}
		}
		AssertJUnit.assertTrue("The Sick PTO did not appear to be set on the worker.", foundPto);
		
		//Find the disbursement
		boolean found = false;
		Set<CompensationDtl> compensationDtls = sWorker.getCompensationDtls();
		for (CompensationDtl compensationDtl : compensationDtls) {
			//Find the dude with the sick pto base wage.
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.SICKTAKEN) {
				found = true;
				break;
			}
		}
		AssertJUnit.assertTrue("Disbursement Compensation for Sick PTO not found on worker", found);
		
	}
	
	@Test(expectedExceptions = ValidationException.class)
	public void testAddPaidTimeOffDtlWithoutCompensationDtl() throws Exception {
		//Setup a sick PTO
		PaidTimeOffDtl paidTimeOffDtl = createPto(PaidTimeOffTypeEnum.SICK);
		
		//Exercise
		sWorker = sPaidTimeOffComponent.addPaidTimeOffDtl(sWorker, paidTimeOffDtl);
		
	}
	
	@Test(expectedExceptions = ValidationException.class)
	public void testAddSickPaidTimeOffDtlToCompensationDtlForVacation() throws Exception {
		//Setup a sick PTO and an hourly wage set it to handle sick PTO.
		PaidTimeOffDtl paidTimeOffDtl = createPto(PaidTimeOffTypeEnum.SICK);
		CompensationDtl hourlyWage = PaycheckCalcTestBase.createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
																						new BigDecimal(10.00), 0d);
		hourlyWage.setPtoBaseRateType(PtoBaseRateTypeEnum.VACATION);
		sWorker = sCompensationComponent.addCompensationToWorker(hourlyWage, sWorker);
		
		//Exercise
		sWorker = sPaidTimeOffComponent.addPaidTimeOffDtl(sWorker, paidTimeOffDtl);
	}
	
	@Test(expectedExceptions = ValidationException.class)
	public void testDeleteCompensationDtlForVacation() throws Exception {
		//Setup a vacation PTO and an hourly wage set it to handle vacation PTO.
		PaidTimeOffDtl paidTimeOffDtl = createPto(PaidTimeOffTypeEnum.VACATION);
		CompensationDtl hourlyWage = PaycheckCalcTestBase.createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
																						new BigDecimal(10.00), 0d);
		hourlyWage.setPtoBaseRateType(PtoBaseRateTypeEnum.VACATION);
		sWorker = sCompensationComponent.addCompensationToWorker(hourlyWage, sWorker);
		
		//Exercise
		sWorker = sPaidTimeOffComponent.addPaidTimeOffDtl(sWorker, paidTimeOffDtl);
		//Find the PTO
		boolean foundPto = false;
		Set<PaidTimeOffDtl> paidTimeOffDtls = sWorker.getPaidTimeOffDtls();
		for (PaidTimeOffDtl paidTimeOffDtl2 : paidTimeOffDtls) {
			if (paidTimeOffDtl2.getPaidTimeOffType() == PaidTimeOffTypeEnum.VACATION) {
				foundPto = true;
				break;
			}
		}
		AssertJUnit.assertTrue("The Vacation PTO did not appear to be set on the worker.", foundPto);
		
		//Find the disbursement
		boolean found = false;
		CompensationDtl savedHourlyWage = null;
		Set<CompensationDtl> compensationDtls = sWorker.getCompensationDtls();
		for (CompensationDtl compensationDtl : compensationDtls) {
			//Find the dude with the sick pto base wage.
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.VACTAKEN) {
				found = true;
				continue;
			}
			if (compensationDtl.getPtoBaseRateType() == PtoBaseRateTypeEnum.VACATION) {
				savedHourlyWage = compensationDtl;
				continue;
			}
		}
		AssertJUnit.assertTrue("Disbursement Compensation for Vacation PTO not found on worker", found);
		
		refreshWorker();

		// Now try to delete the compensationDtl on which the vacation pto rate is based
		sWorker = sCompensationComponent.removeCompensationDtlFromWorker(sWorker, savedHourlyWage);
	}

	@Test (expectedExceptions = ValidationException.class)
	public void testAddPaidTimeOffDtlToInvalidCompensationDtl() throws Exception {
		//Setup a sick PTO and an hourly wage set it to handle sick PTO.
		PaidTimeOffDtl paidTimeOffDtl = createPto(PaidTimeOffTypeEnum.SICK);
		CompensationDtl pieceworkWage = PaycheckCalcTestBase.createPieceworkWageItemDtl(new BigDecimal("1.0"), 1.0, 300.0);
		pieceworkWage.setPtoBaseRateType(PtoBaseRateTypeEnum.SICK);
		sWorker = sCompensationComponent.addCompensationToWorker(pieceworkWage, sWorker);
		
		//Exercise
		sWorker = sPaidTimeOffComponent.addPaidTimeOffDtl(sWorker, paidTimeOffDtl);
		
	}
	
	@Test
	public void testUpdatePaidTimeOffDtlOnExistingWorker() throws Exception {
		//Setup a vacation PTO and an hourly wage set it to handle vacation PTO.
		PaidTimeOffDtl paidTimeOffDtl = createPto(PaidTimeOffTypeEnum.VACATION);
		CompensationDtl hourlyWage = PaycheckCalcTestBase.createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
																						new BigDecimal(10.00), 0d);
		hourlyWage.setPtoBaseRateType(PtoBaseRateTypeEnum.VACATION);
		sWorker = sCompensationComponent.addCompensationToWorker(hourlyWage, sWorker);
		
		//Exercise
		sWorker = sPaidTimeOffComponent.addPaidTimeOffDtl(sWorker, paidTimeOffDtl);
		//Find the PTO
		boolean foundPto = false;
		PaidTimeOffDtl savedPaidTimeOffDtl = null;
		Set<PaidTimeOffDtl> paidTimeOffDtls = sWorker.getPaidTimeOffDtls();
		for (PaidTimeOffDtl paidTimeOffDtl2 : paidTimeOffDtls) {
			if (paidTimeOffDtl2.getPaidTimeOffType() == PaidTimeOffTypeEnum.VACATION) {
				foundPto = true;
				savedPaidTimeOffDtl = paidTimeOffDtl2;
				break;
			}
		}
		AssertJUnit.assertTrue("The Vacation PTO did not appear to be set on the worker.", foundPto);
		
		//Find the disbursement
		boolean found = false;
		Set<CompensationDtl> compensationDtls = sWorker.getCompensationDtls();
		for (CompensationDtl compensationDtl : compensationDtls) {
			//Find the dude with the sick pto base wage.
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.VACTAKEN) {
				found = true;
				break;
			}
		}
		AssertJUnit.assertTrue("Disbursement Compensation for Vacation PTO not found on worker", found);
		
		// Update the previously saved PTO
		savedPaidTimeOffDtl.setAccruedHours(new Double(26));
		savedPaidTimeOffDtl.setAccrualType(PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
		
		refreshWorker();
		
		//Exercise
		sWorker = sPaidTimeOffComponent.updatePaidTimeOffDtl(sWorker, savedPaidTimeOffDtl);
		//Find the PTO
		foundPto = false;
		Set<PaidTimeOffDtl> updatedPaidTimeOffDtls = sWorker.getPaidTimeOffDtls();
		for (PaidTimeOffDtl paidTimeOffDtl2 : updatedPaidTimeOffDtls) {
			if (paidTimeOffDtl2.getPaidTimeOffType() == PaidTimeOffTypeEnum.VACATION) {
				foundPto = true;
				AssertJUnit.assertEquals(savedPaidTimeOffDtl.getAccruedHours(), paidTimeOffDtl2.getAccruedHours());
				AssertJUnit.assertEquals(savedPaidTimeOffDtl.getAccrualType(), paidTimeOffDtl2.getAccrualType());
				break;
			}
		}
		AssertJUnit.assertTrue("The Vacation PTO did not appear to be set on the worker.", foundPto);
		
	}
	
	
	@Test
	public void testUpdatePaidTimeOffDtlOnNewWorker() throws Exception {
		Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Jane", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = PaycheckCalcTestBase.createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 80.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.VACATION);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.07692 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.07692,
    	                                        DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
		
		//Find the PTO
		boolean foundPto = false;
		PaidTimeOffDtl savedPaidTimeOffDtl = null;
		Set<PaidTimeOffDtl> paidTimeOffDtls = worker.getPaidTimeOffDtls();
		for (PaidTimeOffDtl paidTimeOffDtl2 : paidTimeOffDtls) {
			if (paidTimeOffDtl2.getPaidTimeOffType() == PaidTimeOffTypeEnum.VACATION) {
				foundPto = true;
				savedPaidTimeOffDtl = paidTimeOffDtl2;
				break;
			}
		}
		AssertJUnit.assertTrue("The Vacation PTO did not appear to be set on the worker.", foundPto);
		
		//Find the disbursement
		boolean found = false;
		Set<CompensationDtl> compensationDtls = worker.getCompensationDtls();
		for (CompensationDtl compensationDtl : compensationDtls) {
			//Find the dude with the sick pto base wage.
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.VACTAKEN) {
				found = true;
				break;
			}
		}
		AssertJUnit.assertTrue("Disbursement Compensation for Vacation PTO not found on worker", found);
		
		// Update the previously saved PTO
		savedPaidTimeOffDtl.setAccruedHours(new Double(26));
		savedPaidTimeOffDtl.setAccrualType(PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
		
		//Exercise
		worker = sPaidTimeOffComponent.updatePaidTimeOffDtl(worker, savedPaidTimeOffDtl);
		//Find the PTO
		foundPto = false;
		Set<PaidTimeOffDtl> updatedPaidTimeOffDtls = worker.getPaidTimeOffDtls();
		for (PaidTimeOffDtl paidTimeOffDtl2 : updatedPaidTimeOffDtls) {
			if (paidTimeOffDtl2.getPaidTimeOffType() == PaidTimeOffTypeEnum.VACATION) {
				foundPto = true;
				AssertJUnit.assertEquals(savedPaidTimeOffDtl.getAccruedHours(), paidTimeOffDtl2.getAccruedHours());
				AssertJUnit.assertEquals(savedPaidTimeOffDtl.getAccrualType(), paidTimeOffDtl2.getAccrualType());
				break;
			}
		}
		AssertJUnit.assertTrue("The Vacation PTO did not appear to be set on the worker.", foundPto);
		
	}
	
	/**
	 * Should not allow the accrual start date to be before the worker hire date (DateUtil.createDate(2006, 0, 1)
	 * @throws Exception
	 */
	@Test(expectedExceptions=ValidationException.class)
	public void testAddPaidTimeOffDtlOnExistingWorker_BadStartDate() throws Exception {
		//Setup a vacation PTO and an hourly wage set it to handle vacation PTO.
		CompensationDtl hourlyWage = PaycheckCalcTestBase.createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
																						new BigDecimal(10.00), 0d);
		hourlyWage.setPtoBaseRateType(PtoBaseRateTypeEnum.VACATION);
		sWorker = sCompensationComponent.addCompensationToWorker(hourlyWage, sWorker);

		PaidTimeOffDtl paidTimeOffDtl = createPto(PaidTimeOffTypeEnum.VACATION);
		paidTimeOffDtl.setAccrualStartDate(DateUtil.createDate(2005, 0, 1));
		
		//Exercise
		sWorker = sPaidTimeOffComponent.addPaidTimeOffDtl(sWorker, paidTimeOffDtl);
	}
	
	private PaidTimeOffDtl createPto(PaidTimeOffTypeEnum ptoType) throws Exception {
		PaidTimeOffDtl paidTimeOffDtl = sPaidTimeOffComponent.createPaidTimeOffDtl(ptoType);
		paidTimeOffDtl.setAccruedHours(new Double(13));
		paidTimeOffDtl.setAccrualStartDate(DateUtil.getNowDate());
		paidTimeOffDtl.setAccrualStartType(PaidTimeOffAccrualStartTypeEnum.ON_START_DATE);
		paidTimeOffDtl.setAccrualType(PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
		paidTimeOffDtl.setAllowCarryOver(true);
		if (ptoType.equals(PaidTimeOffTypeEnum.SICK))
		{
			paidTimeOffDtl.setDescription("Sick Time Off");
			paidTimeOffDtl.setName("Sick");
		} else if (ptoType.equals(PaidTimeOffTypeEnum.VACATION))
		{
			paidTimeOffDtl.setDescription("Vacation Time Off");
			paidTimeOffDtl.setName("Vacation");
		}
		paidTimeOffDtl.setLimitAccrual(true);
		paidTimeOffDtl.setMaximumAccrual(new Double(160));
		
		paidTimeOffDtl.setRate(PtoDatumHelper.createRateDatum(new Double(1), PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK));
		return paidTimeOffDtl;
	}
	
}
