package com.intuit.sb.payroll.capability.pts.adapter.mapper;



import java.util.HashSet;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.employer.EmployerComponentTestBase;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.CompanyMapper;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.model.AgencyIdType;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSContactPersonTypeEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sbd.payroll.psp.adapters.gemini.AddressAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.ContactAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollFrequencyAdapterEnum;

/**
 *  The <CODE>CompanyMapperTest</CODE> class contains test methods for the CompanyMapper class</P>
 * 
 * $Author: vbudania $
 * $Revision: #1 $
 * $DateTime: february 25,2009 $
 *
 */

public class CompanyMapperTest extends EmployerComponentTestBase {
	
	/**
	 * Ensure that we have mapped fields from Employer entity correctly to EmployerDTO
	 */
	@Test
	public void testMapCompanyRequiredFields() throws Exception
	{
		String companyName = "testCo";
		String dbaName = "testDBAName";
		String fein = "123456789";
		PTSEmployer ptsEmployer = new PTSEmployer();
		Employer employer =	EmployerComponentTestUtils.createNewEmployerInMemory(companyName, dbaName, dbaName, fein, IndustryTypeEnum.RESTAURANT,
				LegalStructureEnum.PARTNERSHIP);
		
		// Add Business Address
		employer.setBusinessAddress(EmployerComponentTestUtils.createGlobalPostalAddress("442-A Gem","Sen Carlo","CA","12213"));
		
		// Add Legal Address
		GlobalPostalAddress globalAddress = EmployerComponentTestUtils.createGlobalPostalAddress("1221 Avenue","LA","CA","12345");
		employer.setLegalAddress(globalAddress);
		
		//Set EmployerContactPersons
		
		EmployerContactPerson empContactPerson = createEmployerContactPerson("Vikram", "Budania", "Owner", "1231231231");
		empContactPerson.setOwner(Boolean.TRUE);
		PTSContactPerson ptsContactPerson = new PTSContactPerson();
		ptsContactPerson.setContactPersonType(PTSContactPersonTypeEnum.PRIMARY_CONTACT);
		ptsContactPerson.setEmployerContact(empContactPerson);
		
		ptsEmployer.addPTSContactPerson(ptsContactPerson);
		
		//Set IsLegalAndBusAddressSame
		employer.setLegalAndBusAddressSame(false);
		
		//Set LegalName
		String legalName = "Red Chilli Pvt. Ltd.";
		employer.setLegalName(legalName);
		
		//Set EmployerIds
		// Just for the fun added EmployerId for CA to ensure that
		EmployerId employerId = new EmployerId();
		employerId.setValue("9901111111");
		AgencyIdType agencyIdType1 = new AgencyIdType();
		AgencyEnum agencyKey1 = AgencyEnum.CA_EDD;
		agencyIdType1.setAgencyKey(agencyKey1);
		employerId.setAgencyIdType(agencyIdType1);
		employer.addEmployerId(employerId);
		
		//Set PayrollFrequency
		PayrollSchedule payrollSchedule = new PayrollSchedule();
		PayrollFrequencyEnum frequencyEnum = PayrollFrequencyEnum.MONTHLY;
		payrollSchedule.setFrequency(frequencyEnum);
		Set<PayrollSchedule> payrollSchedules = new HashSet<PayrollSchedule>();
		payrollSchedules.add(payrollSchedule);
		employer.setPayrollSchedules(payrollSchedules);
		ptsEmployer.setEmployer(employer);
		//Get AdaptorDTO
		CompanyAdapterDTO adapterDTO =  CompanyMapper.mapToCompanyAdapterDTO(ptsEmployer);
		
		//Verification
		verifyDbaName(dbaName,adapterDTO);
		verifyLegalName(legalName,adapterDTO);
		verifyAddress(globalAddress,adapterDTO.getLegalAddress());
		verifyEmployerId(fein,adapterDTO.getFein());
		verifyParollFrequency(PayrollFrequencyAdapterEnum.MONTHLY,adapterDTO.getPayrollFrequencyCd());
		verifyPrimaryContact(empContactPerson,adapterDTO.getPrimaryContact());
		verifySecondaryContact(null,adapterDTO.getSecondaryContact());
		verifyAdminContact(null,adapterDTO.getPayrollAdminContact());
	}
	
	@Test
	public void testMapCompanyMissingFields() throws Exception
	{
		Employer employer =	new Employer();
		PTSEmployer ptsEmployer = new PTSEmployer();
		ptsEmployer.setEmployer(employer);
		//Get AdaptorDTO
		CompanyAdapterDTO adapterDTO =  CompanyMapper.mapToCompanyAdapterDTO(ptsEmployer);
		
		//Verification
		verifyDbaName(employer.getDbaName(),adapterDTO);
		verifyLegalName(employer.getLegalName(),adapterDTO);
		verifyAddress(new GlobalPostalAddress(),adapterDTO.getLegalAddress());
		verifyEmployerId("",adapterDTO.getFein());
		// Using default
		verifyParollFrequency(PayrollFrequencyAdapterEnum.BI_WEEKLY,adapterDTO.getPayrollFrequencyCd());
		
		verifyPrimaryContact(null,adapterDTO.getPrimaryContact());
		verifySecondaryContact(null,adapterDTO.getSecondaryContact());
		verifyAdminContact(null,adapterDTO.getPayrollAdminContact());
	}
	

	

	protected void verifyDbaName(String dbaName,CompanyAdapterDTO adapterDTO)
	{
		AssertJUnit.assertEquals("DbaName not matching", dbaName == null?"":dbaName, adapterDTO.getDoingBusinessAsName());
	}
	
	protected void verifyLegalName(String legalName,CompanyAdapterDTO adapterDTO)
	{
		AssertJUnit.assertEquals("LegalName not matching", legalName==null?"":legalName, adapterDTO.getLegalName());
	}
	
	protected void verifyAddress(GlobalPostalAddress emploAddress,AddressAdapterDTO addressAdapterDTO)
	{
		AssertJUnit.assertEquals("AddressLineOne not matching", emploAddress.getAddressLineOne()== null? "":emploAddress.getAddressLineOne() 
								,addressAdapterDTO.getAddressLine1());

		AssertJUnit.assertEquals("AddressLineTwo not matching", emploAddress.getAddressLineTwo()== null? "": emploAddress.getAddressLineTwo(),
								addressAdapterDTO.getAddressLine2());
		
		AssertJUnit.assertEquals("AddressLineThree not matching", emploAddress.getAddressLineThree()== null? "": emploAddress.getAddressLineThree(),
								addressAdapterDTO.getAddressLine3());	
		
		AssertJUnit.assertEquals("CountryName not matching", emploAddress.getCountryName()== null? "": emploAddress.getCountryName(),
								addressAdapterDTO.getCountry());
		
		AssertJUnit.assertEquals("CityName not matching", emploAddress.getPostTown()==null?"": emploAddress.getPostTown() ,
								addressAdapterDTO.getCity());	
		
		AssertJUnit.assertEquals("ZipCode not matching", emploAddress.getPostalCode()== null? "":emploAddress.getPostalCode() ,
								addressAdapterDTO.getZipCode());	
	    AssertJUnit.assertEquals("State not matching", emploAddress.getCountrySubDivisionCode()== null?"": emploAddress.getCountrySubDivisionCode(),
	    						addressAdapterDTO.getState());
	    AssertJUnit.assertEquals("State not matching", emploAddress.getPostalCodeSuffix()== null?"": emploAddress.getPostalCodeSuffix(),
				addressAdapterDTO.getZipCodeExtension());
	}
	
	protected void verifyEmployerId(String fein,String feString)
	{
		AssertJUnit.assertEquals("EIN not matching", fein==null?"":fein ,feString);
	}
	
	protected void verifyParollFrequency(PayrollFrequencyAdapterEnum freqEnum,PayrollFrequencyAdapterEnum paEnum)
	{
		AssertJUnit.assertEquals("PayrollFrequency not matching", freqEnum ,paEnum);
	}
	
	//FIXME:handle primaryContact
	protected void verifyPrimaryContact(EmployerContactPerson employerContactPerson,ContactAdapterDTO prContactAdapterDTO)
	{
		verifyContact(employerContactPerson, prContactAdapterDTO);
	}
	
	//FIXME:handle secondaryContact
	protected void verifySecondaryContact(EmployerContactPerson employerContactPerson,ContactAdapterDTO prContactAdapterDTO)
	{
		verifyContact(employerContactPerson, prContactAdapterDTO);	
	}
	
	private void verifyAdminContact(
			EmployerContactPerson employerContactPerson,
			ContactAdapterDTO payrollAdminContact) {
		verifyContact(employerContactPerson, payrollAdminContact);
	}
	
	protected void verifyContact(EmployerContactPerson employerContactPerson,ContactAdapterDTO prContactAdapterDTO)
	{
			if (employerContactPerson == null){
				AssertJUnit.assertTrue("ContactDTO not null for an empty EmployerContact" , prContactAdapterDTO == null);
				return;
			}
		    String emailValue = "";
			if (employerContactPerson.getEmailAddress() != null){
				emailValue = employerContactPerson.getEmailAddress().getAddress();
			}
			AssertJUnit.assertEquals("EmailAddress not matching", emailValue ,prContactAdapterDTO.getEmail());
			
			String expectedFirstName = "";
			String expectedLastName = "";
			String exptectedTitle = "";
			if (employerContactPerson.getPersonName() != null){
				expectedFirstName = employerContactPerson.getPersonName().getFirstName();
				expectedLastName =  employerContactPerson.getPersonName().getLastName();
				exptectedTitle  = employerContactPerson.getPersonName().getTitle();
			}
		
			AssertJUnit.assertEquals("FirstName not matching", expectedFirstName ,prContactAdapterDTO.getFirstName());
			AssertJUnit.assertEquals("LastName not matching", expectedLastName ,prContactAdapterDTO.getLastName());
			AssertJUnit.assertEquals("Title not matching",exptectedTitle,prContactAdapterDTO.getTitle());
			
			String expectedPhoneNumber = "";
			if (employerContactPerson.getWorkPhone() != null){
				expectedPhoneNumber = employerContactPerson.getWorkPhone().getFreeFormNumber();
			}
			AssertJUnit.assertEquals("PhoneNumber not matching",  expectedPhoneNumber,prContactAdapterDTO.getPhoneNumber());
			
			verifyAddress(employerContactPerson.getMailingAddress()==null? new GlobalPostalAddress():
				    employerContactPerson.getMailingAddress(),prContactAdapterDTO.getAddress());
	}
	
	
	
	
}
