package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.List;
import java.util.Set;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerId;

public class StateIdModifierNotificationHandlerTest extends StatusUpdateHandlerTestBase {

	
	@Test(enabled=true)
	public void testBasicStateIdModifiedEvent() throws PTSNotificationException{
		// Create a PTSnotification for StateIdModified
		
		PTSNotificationRequest stateIdModifiedRequest = new PTSNotificationRequest();
		stateIdModifiedRequest.setNotificationType(PTSNotificationTypeEnum.STATE_ID_MODIFIED);
		stateIdModifiedRequest.addNotificationAttributes(PTSNotificationRequestMapper.OLD_VALUE, "111");
		stateIdModifiedRequest.addNotificationAttributes(PTSNotificationRequestMapper.NEW_VALUE, "222");
		stateIdModifiedRequest.addNotificationAttributes(PTSNotificationRequestMapper.AGENCY_ID, "CAEDD");
		
		updateNotificationRequest(stateIdModifiedRequest);
		
		// Verify
	}
	
	@Test(enabled=true, expectedExceptions=PTSNotificationException.class)
	public void testStateIdModifiedEventWithoutNewValue() throws PTSNotificationException{
		// Create a PTSnotification for StateIdModified
		
		PTSNotificationRequest stateIdModifiedRequest = new PTSNotificationRequest();
		stateIdModifiedRequest.setNotificationType(PTSNotificationTypeEnum.STATE_ID_MODIFIED);
		stateIdModifiedRequest.addNotificationAttributes(PTSNotificationRequestMapper.OLD_VALUE, "111");
		stateIdModifiedRequest.addNotificationAttributes(PTSNotificationRequestMapper.AGENCY_ID, "CAEDD");
		
		updateNotificationRequest(stateIdModifiedRequest);
	}

	@Test(enabled=true, expectedExceptions=PTSNotificationException.class)
	public void testStateIdModifiedEventWithoutAgency() throws PTSNotificationException{
		// Create a PTSnotification for StateIdModified
		
		PTSNotificationRequest stateIdModifiedRequest = new PTSNotificationRequest();
		stateIdModifiedRequest.setNotificationType(PTSNotificationTypeEnum.STATE_ID_MODIFIED);
		stateIdModifiedRequest.addNotificationAttributes(PTSNotificationRequestMapper.OLD_VALUE, "111");
		stateIdModifiedRequest.addNotificationAttributes(PTSNotificationRequestMapper.NEW_VALUE, "222");
		
		updateNotificationRequest(stateIdModifiedRequest);
	}
	
	@Test(enabled=true)
	public void testEmployerIdChangedforNotification() throws Exception
	{
		List<EmployerId> employerIds = empTaxSettingComponent.getEmployerIds();
		
		// Check if CA is there, else add one
		EmployerId caEmployerId = getEmployerId(employerIds, AgencyEnum.CA_EDD);
		if (caEmployerId == null){
			EmployerComponentTestUtils.createWorkLocation();
			employerIds = empTaxSettingComponent.getEmployerIds();
			caEmployerId = getEmployerId(employerIds, AgencyEnum.CA_EDD);
		}
		
		if (caEmployerId.getValue() == null){
			changeEmployerIdValue(caEmployerId, "12345678");
		}
		String newTaxId = "90909090";
		// Send PTSNotification for StateIdModified
		PTSNotificationRequest ptsNotification = createPTSNotificationRequest(PTSNotificationTypeEnum.STATE_ID_MODIFIED, null);
		ptsNotification.addNotificationAttributes(PTSNotificationRequestMapper.AGENCY_ID, 
				com.intuit.sb.payroll.capability.pts.adapter.dtos.AgencyEnum.CAEDD.toString());
		ptsNotification.addNotificationAttributes(PTSNotificationRequestMapper.OLD_VALUE, caEmployerId.getValue());
		ptsNotification.addNotificationAttributes(PTSNotificationRequestMapper.NEW_VALUE, newTaxId);
		updateNotificationRequest(ptsNotification);
		
		employerIds = empTaxSettingComponent.getEmployerIds();
		EmployerId caEmployerIdMod = getEmployerId(employerIds, AgencyEnum.CA_EDD);
		Assert.assertEquals(newTaxId, caEmployerIdMod.getValue(), "StateModifiedId was not consumed");
		
		// Check if the activation was set to inactive
		String changedValue = "99999999";		
		changeEmployerIdValue(caEmployerIdMod, changedValue);
		
		employerIds = empTaxSettingComponent.getEmployerIds();
		caEmployerIdMod = getEmployerId(employerIds, AgencyEnum.CA_EDD);
		Assert.assertEquals(caEmployerIdMod.getValue(), changedValue,  "In active StateModifiedId was consumed");
	}
	
	private void changeEmployerIdValue(EmployerId caEmployerIdMod, String changedValue) throws ValidationException, ServiceFailureException{
		Employer employer = sEmployerComponent.getEmployer();
		Set<EmployerId> empIdSet = employer.getEmployerIds();
		

		for (EmployerId employerId : empIdSet) {
			if (employerId.getId() == caEmployerIdMod.getId()){
				employerId.setValue(changedValue);;
			}
		}
		employer = sEmployerComponent.saveEmployer(employer);
	}
	
	
	@Test(enabled=true)
	public void testEmployerIdChangedforInActiveFirstNotification() throws Exception
	{
		List<EmployerId> employerIds = empTaxSettingComponent.getEmployerIds();
		
		// Check if CA is there, else add one
		EmployerId caEmployerId = getEmployerId(employerIds, AgencyEnum.CA_EDD);
		if (caEmployerId == null){
			EmployerComponentTestUtils.createWorkLocation();
			employerIds = empTaxSettingComponent.getEmployerIds();
			caEmployerId = getEmployerId(employerIds, AgencyEnum.CA_EDD);
		}
		String orginalId = caEmployerId.getValue();
		String newTaxId = "90909090";
		// Send PTSNotification for StateIdModified
		PTSNotificationRequest ptsNotification = createPTSNotificationRequest(PTSNotificationTypeEnum.STATE_ID_MODIFIED, null);
		ptsNotification.addNotificationAttributes(PTSNotificationRequestMapper.AGENCY_ID, 
				com.intuit.sb.payroll.capability.pts.adapter.dtos.AgencyEnum.CAEDD.toString());
		ptsNotification.addNotificationAttributes(PTSNotificationRequestMapper.NEW_VALUE, newTaxId);
		updateNotificationRequest(ptsNotification);
		
		employerIds = empTaxSettingComponent.getEmployerIds();
		EmployerId caEmployerIdMod = getEmployerId(employerIds, AgencyEnum.CA_EDD);
		Assert.assertEquals(orginalId, caEmployerIdMod.getValue(), "StateModifiedId was not consumed");
	}

	private EmployerId getEmployerId(List<EmployerId> employerIds, AgencyEnum ca_edd) {
		for (EmployerId employerId : employerIds) {
			if (employerId.getAgencyIdType().getAgencyKey().equals(ca_edd)){
				return employerId;
			}
		}
		return null;
	}
	
	
	
}
