package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.Date;

import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.employer.EmployerTaxSettingComponent;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;
import com.intuit.sb.payroll.pts.entity.ActivationStateEAO;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;


public class StatusUpdateHandlerTestBase extends BasicTestBase {
	
	protected EmployerPTSWorkflowComponent employerPTSComponent = null;
	protected ActivationStateEAO activationStateEAO = null;
	protected EmployerTaxSettingComponent empTaxSettingComponent = null;
	
	@BeforeClass
	public void setUpStatusUpdateHandlerTestBase(){
		employerPTSComponent = 
			qaBootstrap.getEjbFromClass(EmployerPTSWorkflowComponent.class);
		activationStateEAO =
			qaBootstrap.getEjbFromClass(ActivationStateEAO.class);
		empTaxSettingComponent = 
			qaBootstrap.getEjbFromClass(EmployerTaxSettingComponent.class);
	}
	
	protected PTSNotificationRequest createPTSNotificationRequest(
			PTSNotificationTypeEnum ptsNotificationType,
			Object activationStatusEnum) {
		PTSNotificationRequest ptsNotificationRequest = new PTSNotificationRequest();
		ptsNotificationRequest.setNotificationType(ptsNotificationType);
		ptsNotificationRequest.setNotificationStatus(activationStatusEnum);
		ptsNotificationRequest.setTimestamp(new Date());
		return ptsNotificationRequest;
	}
	
	protected void updateNotificationRequest(
			PTSNotificationRequest ptsNotificationRequest) throws PTSNotificationException {
		PTSEmployer ptsEmployer = employerPTSComponent.getPTSEmployer();
		
		PTSNotificationTypeEnum ptsNotificationType = ptsNotificationRequest.getNotificationType();
		StatusUpdateHandler statusUpdateHandler = StatusUpdateHandlerFactory.getStatusUpdateHandler(ptsNotificationType);
		statusUpdateHandler.updateStatus(ptsEmployer, ptsNotificationRequest);
	}

	protected String generateRandomAccountNumber() {

    	long randomEIN;
    	long min = 10000000;
    	long max = 99999999;        
        randomEIN = (long)Math.floor((Math.random() * (max - min + 1) )) + min;
        StringBuilder str = new StringBuilder();
        str.append(randomEIN);        
        return str.toString();          
    }
	
	/**
	 * Utility method to create a Bank Account
	 */
	protected BankAccount createBankAccount(String bankName,BankAccountTypeEnum accountType, String accountNo,String routingNo) {
		BankAccount bankAccount = new BankAccount();		
		bankAccount.setBankName(bankName);
		bankAccount.setAccountType(accountType);
		bankAccount.setAccountNo(accountNo);
		bankAccount.setRoutingNo(routingNo);
		return bankAccount;
	}
}
