package com.intuit.sb.payroll.capability.taxes.withholding;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>DeriveStateWithholdingFromFederalWithholdingExceptionsTest</CODE> class contains tests for testing the
 * Exceptions that can be thrown by deriveStateWithholdingFromFederalWithholding() in TaxWithholdingsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class DeriveStateWithholdingFromFederalWithholdingExceptionsTest extends TaxWithholdingsComponentTestBase {
	private static Worker sEmployee;

    @BeforeMethod
    public static void initializeBeforeTest() {
    	sEmployee = WorkerTestUtils.createEmployee();
    }
    
    @AfterMethod
    public static void cleanUpAfterTest() {
    	sEmployee = null;
    }

	/**
	 * <P>Test calling deriveStateWithholdingFromFederalWithholding() with a null Worker</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.deriveStateWithholdingFromFederalWithholding() with a null Worker</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWorker() throws Exception {
    	// Exercise
		sTaxWithholdingsComponent.getCurrentAndDerivedWithholdings(null);
	}

	/**
	 * <P>Test calling deriveStateWithholdingFromFederalWithholding() with no taxes applied</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Call TaxWithholdingsComponent.deriveStateWithholdingFromFederalWithholding() with no taxes applied</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that WorkflowException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = WorkflowException.class)
	public void testTaxesNotApplied() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Exercise
		sTaxWithholdingsComponent.getCurrentAndDerivedWithholdings(sEmployee);
	}

	/**
	 * <P>Test calling deriveStateWithholdingFromFederalWithholding() with no WorkLocation set</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.deriveStateWithholdingFromFederalWithholding() with no WorkLocation set</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that WorkflowException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = WorkflowException.class)
	public void testNoWorkLocationSet() throws Exception {
		// Setup
    	// Exercise
		sTaxWithholdingsComponent.getCurrentAndDerivedWithholdings(sEmployee);
	}

}
