package com.intuit.sb.payroll.capability.taxes.withholding;

import java.util.Iterator;

import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * <P>
 * The <CODE>TaxWithholdingsComponentTestBase</CODE> class contains utility methods for testing TaxWithholdingsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class TaxWithholdingsComponentTestBase extends BasicTestBase {
	protected static TaxWithholdingsComponent sTaxWithholdingsComponent;
	protected static TaxComponent sTaxComponent;

    @BeforeClass
    public static void initializeTaxWithholdingsComponent()
    {
    	sTaxWithholdingsComponent = qaBootstrap.getEjbFromClass(TaxWithholdingsComponent.class);
    	sTaxComponent = qaBootstrap.getEjbFromClass(TaxComponent.class);
    }
    
    /**
     * Utility method to verify that relevant fields in the W4Detail objects are equal
     *  
     * @param expected The expected W4Detail values
     * @param actual The actual W4Detail values to verify
     * @return true if the relevant fields are equal, false otherwise
     */
    protected boolean verifyW4Detail(W4Detail expected, W4Detail actual) {
    	boolean result = true;
    	result = result && (expected.getDocId().equals(actual.getDocId()));
    	result = result && (expected.isDerived().equals(actual.isDerived()));
    	result = result && (expected.getLawId().equals(actual.getLawId()));
    	result = result && (expected.getAdditionalAmount().equals(actual.getAdditionalAmount()));
    	result = result && (expected.isTaxExempt().equals(actual.isTaxExempt()));
    	result = result && (expected.getNoOfAllowances() == actual.getNoOfAllowances());
    	if (null == expected.getWithholdingStatus()) {
    		result = result && (null == actual.getWithholdingStatus());
    	} else {
    		result = result && (expected.getWithholdingStatus().equals(actual.getWithholdingStatus()));
    	}
    	if (false == result) {
    		sLogger.error("Expected:");
    		sLogger.error(expected.toString());
    		sLogger.error("Actual:");
    		sLogger.error(actual.toString());
    	}
    	return result;
    }
    
    /**
     * Utility method to verify that relevant fields in the DE4Detail objects are equal
     *  
     * @param expected The expected W4Detail values
     * @param actual The actual W4Detail values to verify
     * @return true if the relevant fields are equal, false otherwise
     */
    protected boolean verifyDE4Detail(DE4Detail expected, DE4Detail actual) {
    	boolean result = true;
    	result = result && (expected.getDocId().equals(actual.getDocId()));
    	result = result && (expected.isDerived().equals(actual.isDerived()));
    	result = result && (expected.getLawId().equals(actual.getLawId()));
    	result = result && (expected.getAdditionalAmount().equals(actual.getAdditionalAmount()));
    	result = result && (expected.isTaxExempt().equals(actual.isTaxExempt()));
    	result = result && (expected.getNoOfAllowances() == actual.getNoOfAllowances());
    	result = result && (expected.getWithholdingStatus().equals(actual.getWithholdingStatus()));
    	if (false == result) {
    		sLogger.error("Expected:");
    		sLogger.error(expected.toString());
    		sLogger.error("Actual:");
    		sLogger.error(actual.toString());
    	}
    	return result;
    }
    
    /**
     * Utility method to retrieve the W4Detail from the specified worker
     * 
     * @param worker The worker to retrieve the W4Detail from
     * @return The W4Detail if it exists, null if not
     */
    protected W4Detail getW4Detail(Worker worker) {
		// Find the W4Detail
		Iterator<WorkerComplianceDocDtl> iterator = worker.getWorkerComplianceDocDtls().iterator();
		while (iterator.hasNext()) {
			WorkerComplianceDocDtl compDtl = iterator.next();
			if (compDtl instanceof W4Detail) {
				return (W4Detail)compDtl;
			}
		}
		return null;
    }
    
    /**
     * Utility method to retrieve the DE4Detail from the specified worker
     * 
     * @param worker The worker to retrieve the DE4Detail from
     * @return The DE4Detail if it exists, null if not
     */
    protected DE4Detail getDe4Detail(Worker worker) {
		// Find the DE4Detail
		Iterator<WorkerComplianceDocDtl> iterator = worker.getWorkerComplianceDocDtls().iterator();
		while (iterator.hasNext()) {
			WorkerComplianceDocDtl compDtl = iterator.next();
			if (compDtl instanceof DE4Detail) {
				return (DE4Detail)compDtl;
			}
		}
		return null;
    }
    
}
