package com.intuit.sb.payroll.capability.worker;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.entity.model.NonEmployee;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * NonEmployeeCRUD tests.
 * 
 * $Author $ $Revision $ $DateTime $
 * 
 */
public class NonEmployeeCRUDTest extends WorkerComponentTestBase {
	
	private static WorkerComponent workerComponent;

	private static NonEmployee nonEmployee = null;
	
	@BeforeClass
	public static void initializeNonEmployeeCrud()
	{
		workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
	}
	
    @BeforeMethod
    public void testInit()
    {
        BasicTestBase.deleteAllPayrollSchedule();
    }
    
    @AfterMethod
    public void testTeardown() {
		try {
            if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE)
            {
        		BasicTestBase.deleteAllPayrollSchedule();
        		cleanUpNonEmployee(nonEmployee.getId());
            }
		}
		catch (Exception e) {
			String mesg = "Error during teardown after test step: " + ExceptionUtils.throwableMessage(e);
			sLogger.error(mesg, e);
			System.err.println(mesg);
			AssertJUnit.fail(mesg);
		}
		finally
		{
           try
            {
               // Default test step conclusion is to rollback and hence
               // restore the DB to the same state as at the test start.
               // If the test needs to commit it must do so in the @Test method
               // and it then becomes responsible for unwinding DB changes.
               getUserTransaction().setRollbackOnly();
            }
            catch (Exception ex) {
            }
		}
    }

	@Test
	/**
	 * <P>
	 * Tests whether an non-employee can be created without any Payroll Schedule
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete All payroll Schedules</LI>
	 * <LI>Create an non-employee</LI>
	 * <LI>Save the non-employee</LI>
	 * <LI>Load the payroll worker associated with non-employee</LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that worker payroll schedule associated with the worker is
	 * null</LI>
	 * </UL>
	 */
	public void testCreateNonEmployeeWithNoPayrollSchedule() throws Exception {

		nonEmployee = createNewNonEmployee();
		nonEmployee = (NonEmployee) workerComponent.addWorker(nonEmployee);
		Worker worker = entityService.findById(Worker.class, nonEmployee.getId());

		AssertJUnit.assertNotNull(worker);
	}

	/**
	 * <P>
	 * Tests whether an non-employee can be created without any Payroll Schedule
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete All payroll Schedules</LI>
	 * <LI>Retireve the default Payroll Schedule</LI>
	 * <LI>Create an non-employee</LI>
	 * <LI>Save the non-employee</LI>
	 * <LI>Load the payroll worker associated with non-employee</LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that worker payroll schedule associated with the worker is is
	 * the default payroll Schedule</LI>
	 * </UL>
	 */
	@Test(enabled=false, groups={"broken"})
	public void testCreateNonEmployeeWithPayrollSchedule() throws Exception {
		PayrollScheduleTestUtils.createDefaultSchedule();
		PayrollScheduleComponent cService = qaBootstrap.getEjbFromClass(PayrollScheduleComponent.class);
		PayrollSchedule paySchedule = cService.getDefaultPayrollSchedule();

		nonEmployee = createNewNonEmployee();
		workerComponent.addWorker(nonEmployee);
		Worker worker = entityService.findById(Worker.class, nonEmployee.getId());

		AssertJUnit.assertEquals(paySchedule.getId(), worker
					.getPayrollSchedule().getId());
	}
}
