package com.intuit.sb.payroll.capability.worker;

import java.util.Date;

import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.NonEmployee;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;

public class WorkerComponentTestBase extends BasicTestBase {
	
	protected static final Date sHireDate = DateUtil.createDate(2008, 1, 1);
	protected static final Date sInvalidHireDate = DateUtil.createDate(1000, 1, 1);
	protected static WorkerComponent sWorkerComponent;
	
	@BeforeClass
	public static void initializeWorkerComponent()
	{
		sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
	}
	/**
	 * Creates and returns an in-memory instance of Employee which has not been saved into the database
	 * @return Employee
	 */
	protected Employee createNewEmployee()
	{
		Employee employee = new Employee();
		employee.setRelationshipToOwner(RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER);
		employee.setWorkerId(new Long("1234567").toString());
		employee.setDateOfBirth(sAge40BirthDate.getTime());
		PersonName personName = new PersonName();
		personName.setFirstName("Real");
		personName.setLastName("Employee");
		employee.setPersonName(personName);
		DateRange employmentPeriod = new DateRange();
		employmentPeriod.setStartDate(sHireDate);
		employmentPeriod.setEndDate(null);
		employee.setEmploymentPeriod(employmentPeriod);
		GlobalPostalAddress taxResidenceGlobalAddress = new GlobalPostalAddress();
		taxResidenceGlobalAddress.setCountrySubDivisionCode("CA");
		employee.setTaxResidenceAddress(taxResidenceGlobalAddress);
		return employee;
	}

	/**
	 * Deletes the specified employee from the database
	 * 
	 * @param employeeId The id of the employee to delete
	 */
	protected void cleanUpEmployee(Long employeeId)
	{
		Employee worker = entityService.findById(Employee.class, employeeId);
		entityService.makeTransient(worker);
	}
    
	/**
	 * Creates and returns an in-memory instance of NonEmployee which has not been saved into the database
	 * @return NonEmployee
	 */
	protected NonEmployee createNewNonEmployee()
	{
		NonEmployee nonEmployee = new NonEmployee();
		nonEmployee.setWorkerId("1234567");
		PersonName personName = new PersonName();
		personName.setFirstName("Non");
		personName.setLastName("Employee");
		nonEmployee.setPersonName(personName);
		DateRange employmentPeriod = new DateRange();
		employmentPeriod.setStartDate(new Date());
		employmentPeriod.setEndDate(null);
		nonEmployee.setEmploymentPeriod(employmentPeriod);
		return nonEmployee;
	}

	/**
	 * Deletes the specified nonEmployee from the database
	 * 
	 * @param nonEmployeeId The id of the nonEmployee to delete
	 */
	protected void cleanUpNonEmployee(Long nonEmployeeId)
	{
		NonEmployee worker = entityService.findById(NonEmployee.class, nonEmployeeId);
		entityService.makeTransient(worker);
	}

}
