package com.intuit.sb.payroll.capability.worker;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Currency;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.Gender;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent;
import com.intuit.sb.payroll.capability.task.impl.TaskUtils;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.benefits.HealthBenefitEAO;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.deductions.DeductionEAO;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.Deduction;
import com.intuit.sb.payroll.entity.model.DeductionDtl;
import com.intuit.sb.payroll.entity.model.DeductionTypeEnum;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.HealthBenefit;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualStartTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffDtl;
import com.intuit.sb.payroll.entity.model.PaidTimeOffTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.PersonTaxId;
import com.intuit.sb.payroll.entity.model.PersonTaxIdEnum;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;
import com.intuit.sb.payroll.entity.wages.CompensationEAO;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 *  <P>The <CODE>WorkerTestUtils</CODE> class contains Worker helper methods</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class WorkerTestUtils  {
	private static final int YEAR = 2008;
	public static final Date ACCRUAL_START_DATE = DateUtil.createDate(YEAR, Calendar.JANUARY, 1);
	public static final Date HIRE_DATE = DateUtil.createDate(YEAR, Calendar.JANUARY, 1);
	public static final boolean LIMIT_ACCRUAL = true;
	public static final boolean DONT_LIMIT_ACCRUAL = false;
	public static final boolean ALLOW_CARRY_OVER = true;
	public static final boolean DONT_ALLOW_CARRY_OVER = false;
	
	public static final BigDecimal sSalaryAmount = new BigDecimal("66000.00");

	private static EmployerComponent employerComponent = BasicTestBase.getSEmployerComponent();
	private static TaxComponent taxComponent = BasicTestBase.getTaxComponent();
    private static WorkerComponent workerComponent = BasicTestBase.getSWorkerComponent();
    private static CompensationEAO compensationEAO = BasicTestBase.getCompensationEAO();
    private static HealthBenefitEAO healthBenefitEAO = BasicTestBase.getHealthBenefitEAO();
    private static DeductionEAO deductionEAO = BasicTestBase.getDeductionEAO();
    private static EntityService entityService = BasicTestBase.getEntityService();
    private static PaidTimeOffComponent paidTimeOffComponent = BasicTestBase.getPaidTimeOffComponent();

	//Define all the static variables for data population
	private static final String EMPLOYEE_FIRST_NAME = "Joe";
	private static final String EMPLOYEE_LAST_NAME = "Smith";
	private static final String EMPLOYEE_TITLE = "Editor";
	private static final String EMPLOYEE_SUFFIX = "II";
	private static final String EMPLOYEE_ADDRESSLINE1 = "2650 Casey Ave";
	private static final String EMPLOYEE_CITY = "Mountain View";
	private static final String EMPLOYEE_STATE = "CA";
	private static final String EMPLOYEE_COUNTRY = "USA";
    private static final String EMPLOYEE_COUNTRYISOCODE = "US";
	private static final String EMPLOYEE_ZIPCODE = "94043";
	private static final String EMPLOYEE_EMAIL = "Smith@madison.com";
	private static final String EMPLOYEE_WPHONE = "111-111-1111";
	private static final String EMPLOYEE_MPHONE = "222-222-2222";
	private static final String EMPLOYEE_HPHONE = "333-333-3333";
	private static final String EMPLOYEE_DOB = "11-June-1965";
	private static final String EMPLOYEE_SSN = "123-45-6789";
	private static final String sW4DocId = "US-FED-W4";
	private static final String sDE4DocId = "US-CA-DE-4";
//	private static final boolean EXEMPT_FROM_TAX = true;
	private static final boolean NOT_EXEMPT_FROM_TAX = false;

	
	/**
	 * Create a PostalAddress Object with specified values
	 * @param addressLine1 The address line of an address
	 * @param city The city of an address
	 * @param state The state of an address
	 * @param zipCode The zip code of an address
	 * @param country The country of an address
	 * @param countryISOCode The country ISO code of an address
	 * @return A PostalAddress object with initialized values
	 */
	public static USPostalAddress createUSPhysicalAddress(String addressLine1, String city, String state, String zipCode,
					String country, String countryISOCode) {
		USPostalAddress pAddress = new USPostalAddressImpl();
		pAddress.setAddressLine1(addressLine1);
		pAddress.setCity(city);
		pAddress.setRegionOrState(state);
		pAddress.setZipOrPostalCode(zipCode);
		pAddress.setCountry(country);
		pAddress.setCountryISOCode(countryISOCode);
		return pAddress;
	}

	/**
	 * create a USPPhysicalAddress with default values
	 * @return A PostalAddress object with initialized values
	 */
	public static USPostalAddress createUSPhysicalAddress() {
		return createUSPhysicalAddress(EMPLOYEE_ADDRESSLINE1, EMPLOYEE_CITY, EMPLOYEE_STATE, EMPLOYEE_ZIPCODE,
						EMPLOYEE_COUNTRY, EMPLOYEE_COUNTRYISOCODE);
	}

	/**
	 * Create a PersonName Object with the specified values
	 * @param firstName The person's first name
	 * @param lastName The person's last name
	 * @param title The person's title
	 * @param suffix The person's suffix
	 * @return A PersonName object with initialized values
	 */
	public static PersonName createPersonName(String firstName, String lastName, String title, String suffix) {
		PersonName pname = new PersonName();
		pname.setFirstName(firstName);
		pname.setLastName(lastName);
		pname.setTitle(title);
		pname.setSuffix(suffix);
		return pname;
	}

	/**
	 * Create a PersonName Object with default values
	 * @return A PersonName object with initialized values
	 */
	public static PersonName createPersonName() {
		return createPersonName(EMPLOYEE_FIRST_NAME, EMPLOYEE_LAST_NAME, EMPLOYEE_TITLE, EMPLOYEE_SUFFIX);
	}

	/**
	 * Create a WorkLocation object with the specified name using some default values
	 *  
	 * @param name The name of the work location
	 * @return A WorkLocation object with initialized values
	 */
	public static WorkLocation createWorkLocation(String name) {
		WorkLocation workLocation = new WorkLocation();
		workLocation.setName(name);
		USPostalAddress taxAddress = new USPostalAddressImpl();
		taxAddress.setAddressLine1("1880 South Grant St");
		taxAddress.setCity("San Mateo");
		taxAddress.setRegionOrState("CA");
		taxAddress.setZipOrPostalCode("94402");
		taxAddress.setCountryISOCode("US");
		workLocation.setTaxResidenceAddress(
				AddressConvertor.getGlobalFromUS(taxAddress, new GlobalPostalAddress()));
		return workLocation;
	}

	/**
	 * Create an EMAIL ADDRESS with the specified value
	 * @return An EmailAddress with initialized values
	 */
	public static EmailAddress createEmailAddress(String email) {
		EmailAddress eAddress = new EmailAddress();
		eAddress.setAddress(email);
		return eAddress;
	}

	/**
	 * Create an EMAIL ADDRESS with default values
	 * @return An EmailAddress with initialized values
	 */
	public static EmailAddress createEmailAddress() {
		return createEmailAddress(EMPLOYEE_EMAIL);
	}

	/**
	 * Create a PhoneNumber object with the specified value
	 * @param PhoneNo
	 * @return A TelephoneNumber object with initialized values
	 */
	public static TelephoneNumber CreatePhone(String PhoneNo) {
		TelephoneNumber PhoneNumber = new TelephoneNumber();
		PhoneNumber.setFreeFormNumber(PhoneNo);
		return PhoneNumber;
	}

	/**
	 * Create a PhoneNumber object with default value
	 * @return A TelephoneNumber object with initialized values
	 */
	public static TelephoneNumber CreatePhone() {
		return CreatePhone(EMPLOYEE_HPHONE);
	}

	/**
	 * This utility method will create an Employee object using the specified values
	 * 
	 * @param firstName The first name of the employee
	 * @param lastName The last name of the employee
	 * @param title The employee's title
	 * @param suffix The suffix name of the employee 
	 * @param dateofbirth The employee's date of birth
	 * @param emailAddress The employee's email address
	 * @param wPhoneNo The employee's work phone number
	 * @param mPhoneNo The employee's mobile phone number
	 * @param hPhoneNo The employee's home phone number
	 * @param addressLine1 The address line of the employee's tax residence address
	 * @param city The city of the employee's tax residence address
	 * @param state The state of the employee's tax residence address
	 * @param zipCode The zip code of the employee's tax residence address
	 * @param country The country of the employee's tax residence address
	 * @param countryISOCode The country ISO code of the employee's tax residence address
	 * @param SSN The employee's Social Security number
	 * @return An employee object initialized with the specified values
	 */
	public static Worker createEmployee(String firstName, String lastName, String title, String suffix,
					String dateofbirth, String emailAddress, String wPhoneNo, String mPhoneNo, String hPhoneNo,
					String addressLine1, String city, String state, String zipCode, String country,
					String countryISOCode, String SSN) {
		Worker worker = new Employee();

		//Set names,title,suffix of the worker
		PersonName pname = createPersonName(firstName, lastName, title, suffix);
		worker.setPersonName(pname);

		//set gender
		worker.setGender(Gender.Male);

		//set DOB
		try {
			DateFormat formatter = new SimpleDateFormat("dd-MMM-yy");
			Date dateOfBirth = (Date) formatter.parse(dateofbirth);
			worker.setDateOfBirth(dateOfBirth);
		} catch (ParseException pe) {
		}

		// Set the hire date
		DateRange dateRange = new DateRange();
		dateRange.setStartDate(HIRE_DATE);
        worker.setEmploymentPeriod(dateRange);
		
		//set email address
		EmailAddress eAddress = createEmailAddress(emailAddress);
		worker.setEmailAddress(eAddress);

		//set TelephoneNumber nos
		TelephoneNumber wPhoneNumber = CreatePhone(wPhoneNo);
		TelephoneNumber mPhoneNumber = CreatePhone(mPhoneNo);
		TelephoneNumber hPhoneNumber = CreatePhone(hPhoneNo);
		worker.setHomePhone(hPhoneNumber);
		worker.setWorkPhone(wPhoneNumber);
		worker.setMobilePhone(mPhoneNumber);

		//set Address
		USPostalAddress pAddress = createUSPhysicalAddress(addressLine1, city, state, zipCode, country, countryISOCode);
		worker.setTaxResidenceAddress(
				AddressConvertor.getGlobalFromUS(pAddress, new GlobalPostalAddress()));

		//Setting Ids
		PersonTaxId pTaxId = new PersonTaxId();
		pTaxId.setType(PersonTaxIdEnum.SSN);
		pTaxId.setValue(SSN);

		// Set the worker to active
		worker.setActive(true);
		
		return worker;
	}
	
	public static PersonTaxId generateRandomPersonTaxId(){
		PersonTaxId personTaxId = new PersonTaxId();
		personTaxId.setType(PersonTaxIdEnum.SSN);
		personTaxId.setValue(generateRandomSSN());
		personTaxId.setLastVerifiedDate(new Date(0));
		return personTaxId;
	}
	
	private static String generateRandomSSN() {
		String randomEIN = EmployerComponentTestUtils.generateRandomEIN();
		return randomEIN;
//		StringBuffer sb = new StringBuffer(randomEIN.substring(0, 3)).append("-")
//			.append(randomEIN.substring(3, 5)).append("-")
//			.append(randomEIN.substring(5));
//		return sb.toString();
	}

	/**
	 * Create employee with default values using the specified first name
	 * 
	 * @param firstName The first name of the employee
	 * @return a Worker object with default values
	 */
	public static Worker createEmployee(String firstName) {
		return createEmployee(firstName, EMPLOYEE_LAST_NAME, EMPLOYEE_TITLE, EMPLOYEE_SUFFIX, EMPLOYEE_DOB,
						EMPLOYEE_EMAIL, EMPLOYEE_WPHONE, EMPLOYEE_MPHONE, EMPLOYEE_HPHONE, EMPLOYEE_ADDRESSLINE1,
						EMPLOYEE_CITY, EMPLOYEE_STATE, EMPLOYEE_ZIPCODE, EMPLOYEE_COUNTRY, EMPLOYEE_COUNTRYISOCODE,
						EMPLOYEE_SSN);
	}

	/**
	 * Create employee with default values
	 */
	public static Worker createEmployee() {
		return createEmployee(EMPLOYEE_FIRST_NAME);
	}

	/**
	 * This utility method will set up the worker's employment period using the specified start and end dates,
	 * it returns the original start and end dates so that the caller can restore the original values if needed.
	 * 
	 * @param worker The worker to set up the employment period
	 * @param startDate The employment start date
	 * @param endDate The employment end date
	 * @return The original employment period values
	 * @throws Exception
	 */
	public static DateRange setUpEmploymentPeriod(Worker worker, Date startDate, Date endDate) throws Exception {
		DateRange origDateRange = new DateRange();
		// Remember the original values
		DateRange origEmploymentPeriod = worker.getEmploymentPeriod();
		if (null != origEmploymentPeriod.getStartDate()) {
			origDateRange.setStartDate((Date)origEmploymentPeriod.getStartDate().clone());
		}
		if (null != origEmploymentPeriod.getEndDate()) {
			origDateRange.setEndDate((Date)origEmploymentPeriod.getEndDate().clone());
		}
		// Now set up the new values
		origEmploymentPeriod.setStartDate(startDate);
		origEmploymentPeriod.setEndDate(endDate);
		// Return the original values so the caller can restore the original values if necessary
		return origDateRange;
	}
	
	/**
	 * This utility method will return a WorkLocation specified by the work location name
	 * 
	 * @param workLocationName The name of the work location to retrieve
	 * @return A workLocation object specified by the work location name if found, null otherwise
	 */
	public static WorkLocation getNamedWorkLocation(String workLocationName)
	{
		WorkLocation workLocation = null;
		List<WorkLocation> workLocations = employerComponent.findAllWorkLocations();
		for (WorkLocation aWorkLocation : workLocations) {
			if (workLocationName.equalsIgnoreCase(aWorkLocation.getName())) {
				workLocation = aWorkLocation;
				break;
			}
		}
		return workLocation;
	}
	
	/**
	 * <P>Sets up the W4 withholding information for the specified Worker using the given information.</P>
	 *
	 * @param worker The worker to set up withholding information
	 * @param complianceDocId The docId for the withholding info
	 * @param docType The ComplianceDocType in which the withholding information is for
	 * @param isTaxExempt Whether or not the worker is exempt from income tax
	 * @param withholdingStatus The worker's withholding status
	 * @param allowance The worker's withholding allowance
	 * @param additonalAmount The worker's additional withholding
	 * @return the updated Worker object
	 */
	public static Worker setUpW4WithholdingInfo(Worker worker, String complianceDocId, ComplianceDocIdEnum docType,
			boolean isTaxExempt, W4WithholdingStatusEnum withholdingStatus, Integer allowance,
			BigDecimal additonalAmount) throws ServiceFailureException {
		W4Detail eWithholdingData = null;
		// Update existing withholdingData if it exists for this docId
		for (WorkerComplianceDocDtl workerComplianceDocDtl : worker.getWorkerComplianceDocDtls()) {
			ComplianceDocIdEnum compDocType = workerComplianceDocDtl.getDocId();
			if (compDocType == docType) {
				eWithholdingData = (W4Detail)workerComplianceDocDtl;
				break;
			}
		}
		// Doesn't exist, create a new one
		if (eWithholdingData == null) {
			eWithholdingData = new W4Detail();
			// set associations
			eWithholdingData.setDocId(ComplianceDocIdEnum.W4IRS);
			worker.addWorkerComplianceDocDtl(eWithholdingData);
		}
		// set fields in EmployeeWithholdingData
		eWithholdingData.setTaxExempt(isTaxExempt);
		eWithholdingData.setWithholdingStatus(withholdingStatus);
		eWithholdingData.setNoOfAllowances(allowance);
		Money amount = new Money();
		amount.setCurrencyCode(Currency.getInstance("USD"));
		amount.setAmount(additonalAmount);
		eWithholdingData.setAdditionalAmount(amount);
		// set fields in WithholdingData
		eWithholdingData.setDerived(false);
		return worker;
	}

	/**
	 * <P>Sets up a non-derived DE4 withholding information for the specified Worker using the given information.</P>
	 *
	 * @param worker The worker to set up withholding information
	 * @param docId The docId for the withholding info
	 * @param docType The ComplianceDocType in which the withholding information is for
	 * @param isTaxExempt Whether or not the worker is exempt from income tax
	 * @param withholdingStatus The worker's withholding status
	 * @param allowance The worker's withholding allowance
	 * @param deductions The worker's withholding deductions
	 * @param additonalAmount The worker's additional withholding
	 * @param isDerived Whether or not this withholding derives from the W4 withholding
	 * @return the updated Worker object
	 * @throws ServiceFailureException
	 */
	public static Worker setUpDe4WithholdingInfo(Worker worker, String docId, ComplianceDocIdEnum docType,
			boolean isTaxExempt, DE4WithholdingStatusEnum withholdingStatus, Integer allowance, Integer deductions,
			BigDecimal additonalAmount) throws Exception {
		return setUpDe4WithholdingInfo(worker, docId, docType, isTaxExempt, withholdingStatus, allowance,
					deductions, additonalAmount, false);
	}
	
	/*
	 * <P>Sets up the DE4 withholding information for the specified Worker using the given information.</P>
	 *
	 * @param worker The worker to set up withholding information
	 * @param docId The docId for the withholding info
	 * @param docType The ComplianceDocType in which the withholding information is for
	 * @param isTaxExempt Whether or not the worker is exempt from income tax
	 * @param withholdingStatus The worker's withholding status
	 * @param allowance The worker's withholding allowance
	 * @param deductions The worker's withholding deductions
	 * @param additonalAmount The worker's additional withholding
	 * @param derived Whether or not this withholding derives from the W4 withholding
	 * @return the updated Worker object
	 */
	private static Worker setUpDe4WithholdingInfo(Worker worker, String docId, ComplianceDocIdEnum docType,
			boolean isTaxExempt, DE4WithholdingStatusEnum withholdingStatus, Integer allowance, Integer deductions,
			BigDecimal additonalAmount, boolean derived) throws Exception {
		Set<WorkerComplianceDocDtl> compDataForWorkers = worker.getWorkerComplianceDocDtls();
		DE4Detail eWithholdingData = null;
		// Update existing withholdingData if it exists for this docId
		for (WorkerComplianceDocDtl complianceData : compDataForWorkers) {
			ComplianceDocIdEnum compDocType = complianceData.getDocId();
			if (compDocType == docType) {
				eWithholdingData = (DE4Detail)complianceData;
			}
		}
		// Doesn't exist, create a new one
		if (eWithholdingData == null)
		{
			eWithholdingData = new DE4Detail();
			// set associations
			eWithholdingData.setDocId(ComplianceDocIdEnum.DE4);
			worker.addWorkerComplianceDocDtl(eWithholdingData);
		}
		// set fields in EmployeeWithholdingData
		eWithholdingData.setTaxExempt(isTaxExempt);
		eWithholdingData.setWithholdingStatus(withholdingStatus);
		eWithholdingData.setNoOfAllowances(allowance);
		eWithholdingData.setNoOfDeductions(deductions);
		Money amount = new Money();
		amount.setCurrencyCode(Currency.getInstance("USD"));
		amount.setAmount(additonalAmount);
		eWithholdingData.setAdditionalAmount(amount);
		// set fields in WithholdingData
		eWithholdingData.setDerived(derived);
		worker = workerComponent.updateWorker(worker);
	
		return worker;
	}

	/**
	 * This utility method will initialize some default values for compensation item specified by the given lawId
	 *   The default values are as follows:
	 *     Hourly - $10.00/Hour,
	 *     SickHourly - $10.00/Hour,
	 *     VacationHourly - $10.00/Hour,
	 *     Overtime - 1.5 X Hourly,
	 *     Doubletime - 2.0 X Hourly,
	 *     PaycheckTips - $100.00,
	 *     CashTips - $100.00,
	 *     AllocatedTips - $100.00,
	 *     Piecework - $1.00/piece,
	 *     Bonus - $1,000.00,
	 *     Salary - $66,000.00/Year,
	 *     SickSalary - $66,000.00/Year,
	 *     VacationSalary - $66,000.00/Year,
	 *     Commission - $500.00
	 * 
	 * @param lawId The law id of the compensation item
	 * @param compensationTypeEnum The CompensationTypeEnum value
	 * @return A Datum object with default values
	 */
	public static Datum initDefaultCompensationDatumValue(String lawId, CompensationTypeEnum compensationTypeEnum) {
        Datum payrollDatum = new Datum();
        if (lawId.equals(QboeLawIdConstants.sHourlyLawId)) {
        	payrollDatum.setValue1AsUSDMoneyAmount(new BigDecimal("10.0"));
        	payrollDatum.setType2(DatumType2Enum.HOUR) ;
        	payrollDatum.setValue2(1.0);
        } else if (lawId.equals(QboeLawIdConstants.sOvertimeLawId)) {
            payrollDatum.setValue1AsQuantity(1.5);
            payrollDatum.setType2(DatumType2Enum.NOTAPPLICABLE);
        } else if (lawId.equals(QboeLawIdConstants.sDoubleTimeLawId)) {
            payrollDatum.setValue1AsQuantity(2.0);
            payrollDatum.setType2(DatumType2Enum.NOTAPPLICABLE);
        } else if (lawId.equals(QboeLawIdConstants.sPaycheckTipsLawId) ||
        		   lawId.equals(QboeLawIdConstants.sCashTipsLawId) ||
        		   lawId.equals(QboeLawIdConstants.sAllocatedTipsLawId)) {
            payrollDatum.setValue1AsUSDMoneyAmount(new BigDecimal("100.0"));
            payrollDatum.setType2(DatumType2Enum.NOTAPPLICABLE) ;
        } else if (lawId.equals(QboeLawIdConstants.sPieceworkLawId)) {
        	payrollDatum.setValue1AsUSDMoneyAmount(new BigDecimal("1.0"));
        	payrollDatum.setType2(DatumType2Enum.PERUNIT);
        	payrollDatum.setValue2(1.0);
        } else if (lawId.equals(QboeLawIdConstants.sBonusLawId)) {
            payrollDatum.setValue1AsUSDMoneyAmount(new BigDecimal("1000.0"));
            payrollDatum.setType2(DatumType2Enum.NOTAPPLICABLE) ;
        } else if (lawId.equals(QboeLawIdConstants.sSalaryLawId)) {
            payrollDatum.setValue1AsUSDMoneyAmount(sSalaryAmount);
        	payrollDatum.setType2(DatumType2Enum.YEAR) ;
        	payrollDatum.setValue2(1.0);
        } else if (lawId.equals(QboeLawIdConstants.sCommissionLawId)) {
        	payrollDatum.setValue1AsUSDMoneyAmount(new BigDecimal("500.0"));
        	payrollDatum.setType2(DatumType2Enum.PERPAYCHECK) ;
        	payrollDatum.setValue2(1.0);
        } else {
            // TODO  Other types of wage items
        }
        return payrollDatum;
    }
    
	/**
	 * This utility method will initialize some default values for health benefit item specified by the given lawId
	 *   The default values are as follows:
	 *     Section 125 health - $65.00
	 * 
	 * @param lawId The law id of the health benefit item
	 * @return A Datum object with default values
	 */
	public static Datum initDefaultHealthBenefitDatumValue(String lawId) {
        Datum payrollDatum = new Datum();
        if (lawId.equals(QboeLawIdConstants.sSec125HealthId)) {
            payrollDatum.setValue1AsUSDMoneyAmount(new BigDecimal("65.0"));
        	payrollDatum.setType2(DatumType2Enum.PERPAYCHECK) ;
        	payrollDatum.setValue2(1.0);
        } else {
            // TODO  Other types of wage items
        }
        return payrollDatum;
    }
    
	/**
	 * This utility method will create a CompensationDtl object with the specified CompensationTypeEnum and some
	 *   default values
	 * 
	 * @param compensationTypeEnum The compensationType to create
	 * @return A CompensationDtl object
	 */
    public static CompensationDtl createCompensationDtl(CompensationTypeEnum compensationTypeEnum, boolean fixed) {
		Compensation compensation = compensationEAO.findCompensation(compensationTypeEnum);

    	CompensationDtl compensationDtl = new CompensationDtl();

    	compensationDtl.setPayrollItem(compensation);
    	compensationDtl.setCompensationType(compensation.getCompensationType());
    	Datum datum = initDefaultCompensationDatumValue(compensation.getLawId().getValue(), compensationTypeEnum);
    	compensationDtl.setRate(datum);
		Datum quantityDatum = new Datum();
		quantityDatum.setValue1AsQuantity(0.0);
    	compensationDtl.setQuantity(quantityDatum);
    	compensationDtl.setDescription(compensation.getDescription());
    	compensationDtl.setName(compensation.getName());
    	compensationDtl.setFixedRate(fixed);
    	return compensationDtl;
    }
    
    /**
     * <P>Sets up an Hourly, Overtime and Bonus wage item for the specified worker.</P>
     * 
     * @param worker The worker to set up the wages
     */
    public static Worker setupWages(Worker worker) throws Exception {
        // Set up compensation for a worker
        CompensationDtl hourlyDtl = createCompensationDtl(CompensationTypeEnum.HOURLYREG, true);
    	worker.addCompensationDtl(hourlyDtl);
    	CompensationDtl overtimeDtl = createCompensationDtl(CompensationTypeEnum.HOURLYOVT, true);
    	// Link the overtime to the hourly item
    	overtimeDtl.setLinkedPayrollItemDtl(hourlyDtl);
    	worker.addCompensationDtl(overtimeDtl);
    	worker.addCompensationDtl(createCompensationDtl(CompensationTypeEnum.BONUS, false));
        return worker;
    }

    /**
     * <P>Adds the double-time item to the specified worker.</P>
     * 
     * @param worker The worker to add the double-time item
     * @param fixed is Fixed rate or not
     */
    public static Worker addDoubleTime(Worker worker, boolean fixed) throws Exception {
    	// Setup double-time item
    	CompensationDtl doubleTimeDtl = createCompensationDtl(CompensationTypeEnum.HOURLYDBL, fixed);
    	CompensationDtl hourly = findCompensationDtl(worker, CompensationTypeEnum.HOURLYREG);
    	doubleTimeDtl.setLinkedPayrollItemDtl(hourly);
    	worker = BasicTestBase.getCompensationComponent().addCompensationToWorker(doubleTimeDtl, worker);
        return worker;
    }

    /**
     * <P>Adds the bonus item to the specified worker.</P>
     * 
     * @param worker The worker to add the bonus item
     * @param fixed is Fixed rate or not
     */
    public static Worker addBonus(Worker worker, boolean fixed) throws Exception {
    	// Setup bonus item
    	CompensationDtl bonusDtl = createCompensationDtl(CompensationTypeEnum.BONUS, fixed);
    	worker = BasicTestBase.getCompensationComponent().addCompensationToWorker(bonusDtl, worker);
        return worker;
    }

    /**
     * <P>Adds the piecework item to the specified worker.</P>
     * 
     * @param worker The worker to add the piecework item
     * @param fixed is Fixed rate or not
     */
    public static Worker addPiecework(Worker worker, boolean fixed) throws Exception {
    	// Setup piecework item
    	CompensationDtl pieceworkDtl = createCompensationDtl(CompensationTypeEnum.PIECEWORK, fixed);
    	worker = BasicTestBase.getCompensationComponent().addCompensationToWorker(pieceworkDtl, worker);
        return worker;
    }

    /**
     * <P>Adds the commission item to the specified worker.</P>
     * 
     * @param worker The worker to add the commission items
     * @param fixed is Fixed rate or not
     */
    public static Worker addCommission(Worker worker, boolean fixed) throws Exception {
    	// Setup Commission item
    	CompensationDtl commissionDtl = createCompensationDtl(CompensationTypeEnum.COMMISSION, fixed);
    	worker = BasicTestBase.getCompensationComponent().addCompensationToWorker(commissionDtl, worker);
        return worker;
    }

    /**
     * <P>Adds the PaycheckTips item to the specified worker.</P>
     * 
     * @param worker The worker to add the PaycheckTips item
     * @param fixed is Fixed rate or not
     */
    public static Worker addPaycheckTips(Worker worker, boolean fixed) throws Exception {
    	// Setup PaycheckTips item
    	CompensationDtl paycheckTipsDtl = createCompensationDtl(CompensationTypeEnum.TIPSPAYCHECK, fixed);
    	worker = BasicTestBase.getCompensationComponent().addCompensationToWorker(paycheckTipsDtl, worker);
        return worker;
    }

    /**
     * <P>Adds the CashTips item to the specified worker.  An offset deduction item is automatically added.</P>
     * 
     * @param worker The worker to add the CashTips item
     * @param fixed is Fixed rate or not
     */
    public static Worker addCashTips(Worker worker, boolean fixed) throws Exception {
    	// Setup CashTips item
    	CompensationDtl cashTipsDtl = createCompensationDtl(CompensationTypeEnum.TIPSCASH, fixed);
    	worker = BasicTestBase.getCompensationComponent().addCompensationToWorker(cashTipsDtl, worker);
        return worker;
    }

    /**
     * <P>Adds the AllocatedTips item to the specified worker.</P>
     * 
     * @param worker The worker to add the AllocatedTips item
     * @param fixed is Fixed rate or not
     */
    public static Worker addAllocatedTips(Worker worker, boolean fixed) throws Exception {
    	// Setup AllocatedTips item
    	CompensationDtl allocatedTipsDtl = createCompensationDtl(CompensationTypeEnum.TIPSALLOCATED, fixed);
    	worker = BasicTestBase.getCompensationComponent().addCompensationToWorker(allocatedTipsDtl, worker);
        return worker;
    }

	/**
	 * This utility method will create a PaidTimeOffDtl object with the specified input values
	 * 
	 * @param paidTimeOffType The PaidTimeOffType to create
	 * @param limitAccrual Whether or not to limit accrual
	 * @param allowCarryOver Whether or not to allow carry over to the next year
	 * @param maximumAccrual The maximum accrual amount
	 * @param accruedHours The accrued hours
	 * @param accrualStartType The accrual start type
	 * @param accrualStartDate The date that the accrual commences
	 * @param accrualType The accrual type
	 * @return A PaidTimeOffDtl object
	 */
    public static PaidTimeOffDtl createPaidTimeOffDtl(PaidTimeOffTypeEnum paidTimeOffType, boolean limitAccrual,
    				boolean allowCarryOver, Double maximumAccrual, Double accruedHours,
    				PaidTimeOffAccrualStartTypeEnum accrualStartType, Date accrualStartDate,
    				PaidTimeOffAccrualTypeEnum accrualType) throws Exception {
    	PaidTimeOffDtl paidTimeOffDtl = paidTimeOffComponent.createPaidTimeOffDtl(paidTimeOffType);
    	paidTimeOffDtl.setAccrualType(accrualType);
    	paidTimeOffDtl.setLimitAccrual(limitAccrual);
    	paidTimeOffDtl.setAllowCarryOver(allowCarryOver);
    	paidTimeOffDtl.setMaximumAccrual(maximumAccrual);
    	paidTimeOffDtl.setAccruedHours(accruedHours);
    	paidTimeOffDtl.setAccrualStartType(accrualStartType);
    	paidTimeOffDtl.setAccrualStartDate(accrualStartDate);
    	return paidTimeOffDtl;
    }
    
    private static Worker addAPto(Worker worker, PaidTimeOffTypeEnum type, Date accrualStart, Double maxAccrual,
    				 Double accrualRate, DatumType2Enum accrualType, PaidTimeOffAccrualTypeEnum ptoAcrrualType)
    				throws Exception {
        // Setup sick PTO item
        PaidTimeOffDtl ptoDtl = createPaidTimeOffDtl(type, LIMIT_ACCRUAL, ALLOW_CARRY_OVER, maxAccrual, 0.0,
                                                     PaidTimeOffAccrualStartTypeEnum.IMMEDIATELY,
                                                     accrualStart, ptoAcrrualType);
        Datum datum = ptoDtl.getRate();
        if (null == datum) {
            datum = new Datum();
            ptoDtl.setRate(datum);
        }
        datum.setValue1AsQuantity(accrualRate);
        datum.setType2Value2(accrualType, 1.0);
        worker = paidTimeOffComponent.addPaidTimeOffDtl(worker, ptoDtl);
        return worker;
    }

    /**
     * <P>Adds the sick PTO item to the specified worker.  The sick PTO item is set to a maximum accrual of 80 hours,
     *    0 hours accrued, accrual starts immediately, an accrual start date of 01-01-2008 and accrual type of
     *    per paycheck.  </P>
     * 
     * @param worker The worker to add the sick PTO item
     * @param accrualStart The date that the accrual starts
     * @param accrualRate The accrual rate
     * @param accrualType The accrual type of type DatumType2Enum
     * @param ptoAcrrualType The accrual type of type PaidTimeOffAccrualTypeEnum 
     */
    public static Worker addSickPTO(Worker worker, Date accrualStart, Double accrualRate, DatumType2Enum accrualType,
    				PaidTimeOffAccrualTypeEnum ptoAcrrualType) throws Exception {
    	// Setup sick PTO item
        return addAPto(worker, PaidTimeOffTypeEnum.SICK, accrualStart, 80.0, accrualRate, accrualType, ptoAcrrualType);
    }

    /**
     * <P>Adds the vacation PTO item to the specified worker.  The vacation PTO item is set to a maximum
     *    accrual of 160 hours, 0 hours accrued, accrual starts immediately, an accrual start date of 01-01-2008 and
     *    accrual type of per paycheck.</P>
     * 
     * @param worker The worker to add the vacationPTO items
     * @param accrualStart The date that the accrual starts
     * @param accrualRate The accrual rate
     * @param accrualType The accrual type of type DatumType2Enum
     * @param ptoAcrrualType The accrual type of type PaidTimeOffAccrualTypeEnum 
     */
    public static Worker addVacationPTO(Worker worker, Date accrualStart, Double accrualRate, DatumType2Enum accrualType,
    				PaidTimeOffAccrualTypeEnum ptoAcrrualType) throws Exception {
    	// Setup vacation PTO item
        return addAPto(worker, PaidTimeOffTypeEnum.VACATION, accrualStart, 80.0, accrualRate, accrualType, ptoAcrrualType);
    }

    /**
     * This utility method will return a worker's CompensationDtl specified by the given CompensationTypeEnum.
     * If the specified CompensationTypeEnum does not exists in the worker's compensationList, an assertion will be
     * thrown.
     * 
     * @param worker The worker
     * @param compensationTypeEnum The compensationType to look for
     * @return The worker's CompensationDtl that was specified by the CompensationTypeEnum if it exists
     */
    public static CompensationDtl findCompensationDtl(Worker worker, CompensationTypeEnum compensationTypeEnum) {
    	CompensationDtl hourlyDtl = null;
    	for (CompensationDtl compensationDtl : worker.getCompensationDtls()) {
    		if (compensationTypeEnum == compensationDtl.getCompensationType()) {
    			hourlyDtl = compensationDtl;
    			break;
    		}
    	}
    	AssertJUnit.assertNotNull("Worker does not have a regular hourly item set up!", hourlyDtl);
    	return hourlyDtl;
    }
    
    private static Worker setPtoDisburseHours(Worker worker, CompensationTypeEnum type, Double hours) throws Exception {
    	boolean found = false;
    	for (CompensationDtl compensationDtl : worker.getCompensationDtls()) {
    		if (type == compensationDtl.getCompensationType()) {
    			found = true;
    			Datum quantityDatum = compensationDtl.getQuantity();
    			if (null == quantityDatum) {
    		        quantityDatum = new Datum();
    		        compensationDtl.setQuantity(quantityDatum);
    			}
		        quantityDatum.setValue1AsQuantity(hours);
		        worker = workerComponent.updateWorker(worker);
		        break;
    		}
    	}
    	AssertJUnit.assertTrue("Cannot find compensation " + type.toString() + " on the worker!", found);
        return worker;
    }
    
    /**
     * This utility routine will set the number of hours taken for the Sick compensation item for the specified worker
     * 
     * @param worker The worker to set the sick hours taken
     * @param hours The number of sick hours taken
     * @return The updated worker
     * @throws Exception
     */
    public static Worker setSickDisburseHours(Worker worker, Double hours) throws Exception {
        return setPtoDisburseHours(worker, CompensationTypeEnum.SICKTAKEN, hours);
    }
    
    /**
     * This utility routine will set the number of hours taken for the Vacation compensation item for the specified worker
     * 
     * @param worker The worker to set the vacation hours taken
     * @param hours The number of vacation hours taken
     * @return The updated worker
     * @throws Exception
     */
    public static Worker setVacationDisburseHours(Worker worker, Double hours) throws Exception {
        return setPtoDisburseHours(worker, CompensationTypeEnum.VACTAKEN, hours);
    }
    
    /**
     * <P>Adds an hourly wage item with the specified name for the specified worker.</P>
     * 
     * @param worker The worker to add the hourly wage
     * @param customName The custom name of the hourly wage.  If customName is null, the default name is used.
     * @param addOvertime An overtime item will be added if addOvertime is true.  The customName for the overtime item
     *                      is the name of the hourly item with " overtime" appended to it.
     */
    public static Worker addHourlyWage(Worker worker, String customName, boolean addOvertime) throws Exception {
        // Set up compensation for a worker
        CompensationDtl hourlyDtl = createCompensationDtl(CompensationTypeEnum.HOURLYREG, true);
        if (null != customName) {
        	hourlyDtl.setDescription(customName);
        }
    	worker.addCompensationDtl(hourlyDtl);
    	if (addOvertime) {
        	CompensationDtl overtimeDtl = createCompensationDtl(CompensationTypeEnum.HOURLYOVT, true);
        	// Link the overtime to the hourly item
        	overtimeDtl.setLinkedPayrollItemDtl(hourlyDtl);
        	// Set the custom name
        	overtimeDtl.setDescription(hourlyDtl.getDescription() + " overtime");
        	worker.addCompensationDtl(overtimeDtl);
    	}
        worker = workerComponent.updateWorker(worker);
        return worker;
    }

    /**
     * <P>Sets up a salary wage item for the specified worker.</P>
     * 
     * @param worker The worker to set up the wages
     */
    public static Worker setupSalaryWage(Worker worker) throws Exception {
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
    	worker.addCompensationDtl(salarlyDtl);
        worker = workerComponent.updateWorker(worker);
        return worker;
    }

	/**
	 * This utility method will create a HealthBenefitDtl object with the specified HealthBenefitTypeEnum and some
	 *   default values
	 * 
	 * @param healthBenefitTypeEnum The healthBenefitType to create
	 * @return A HealthBenefitDtl object
	 */
    public static HealthBenefitDtl createHealthBenefitDtl(HealthBenefitTypeEnum healthBenefitTypeEnum) {
    	HealthBenefit healthBenefit = healthBenefitEAO.findHealthBenefit(healthBenefitTypeEnum);

		HealthBenefitDtl healthBenefitDtl = new HealthBenefitDtl();

		healthBenefitDtl.setPayrollItem(healthBenefit);
		healthBenefitDtl.setBenefitType(healthBenefitTypeEnum);
    	Datum datum = initDefaultHealthBenefitDatumValue(healthBenefit.getLawId().getValue());
    	healthBenefitDtl.setRate(datum);
    	healthBenefitDtl.setDescription(healthBenefit.getDescription());
    	healthBenefitDtl.setName(healthBenefit.getName());
    	return healthBenefitDtl;
    }
    
    /**
     * <P>Adds the Section 125 health benefit item to the specified worker.</P>
     * 
     * @param worker The worker to add the Section 125 health benefit item
     */
    public static Worker addHealthBenefit(Worker worker) throws Exception {
    	// Setup Section 125 health benefit item
    	HealthBenefitDtl sec125HealthDtl = createHealthBenefitDtl(HealthBenefitTypeEnum.SEC125HEALTH);
    	worker.addHealthBenefitDtl(sec125HealthDtl);
        worker = workerComponent.updateWorker(worker);
        return worker;
    }

	/**
	 * This utility method will create a DeductionDtl object with the specified DeductionTypeEnum and some
	 *   default values
	 * 
	 * @param deductionTypeEnum The DeductionType to create
	 * @return A DeductionDtl object
	 */
    public static DeductionDtl createDeductionDtl(DeductionTypeEnum deductionTypeEnum) {
    	Deduction deduction = deductionEAO.findDeduction(deductionTypeEnum);

    	DeductionDtl deductionDtl = new DeductionDtl();

    	deductionDtl.setPayrollItem(deduction);
    	deductionDtl.setDeductionType(deductionTypeEnum);
    	// create an empty container, the actual value depends on the value from the CashTips CompensationDtl item
    	Datum datum = new Datum();
    	deductionDtl.setRate(datum);
    	deductionDtl.setDescription(deduction.getDescription());
    	deductionDtl.setName(deduction.getName());
    	return deductionDtl;
    }
    
    /**
     * This utility method will create a payroll ready worker with the given first name and work location,
     *   it will use default values for the rest of the fields
     *   NOTE: The workLocationName value must be one of the seeded workLocation name, currently the values are:
     *         "San Francisco, CA", "Mountain View, CA", "Mesquite, TX"
     *   
     * @param firstName the first name of the worker
     * @param workLocationName the name of the work location
     * @param createWages Create default wage items if true
     * @return The newly created worker
     * @throws Exception
     */
    private static Worker createAPayrollReadyWorker(String firstName, String workLocationName, boolean createWages)
    		throws Exception {
    	// Create an employee with some default values
    	Worker worker = createEmployee(firstName);
    	// Now set the workLocation
    	WorkLocation workLocation = getNamedWorkLocation(workLocationName);
    	worker.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	worker = taxComponent.applyApplicableTaxes(worker);
    	// Setup withholdings
        worker = setUpW4WithholdingInfo(worker, sW4DocId, ComplianceDocIdEnum.W4IRS, NOT_EXEMPT_FROM_TAX, 
                		W4WithholdingStatusEnum.SINGLE, 1, new BigDecimal("0.00"));
        worker = setUpDe4WithholdingInfo(worker, sDE4DocId, ComplianceDocIdEnum.DE4, NOT_EXEMPT_FROM_TAX, 
        		DE4WithholdingStatusEnum.SINGLE, 1, 0, new BigDecimal("0.00"));
        // Set the payrollDistributionMethod
        worker.setPayrollDistributionMethod(PayrollDistributionMethodEnum.HANDWRITTENPAYCHECK);
        if (createWages) {
        	// Setup wages
            worker = setupWages(worker);
        }
    	// Save the worker
        worker = workerComponent.addWorker(worker);
    	return worker;
    }
    
    
    /**
     * This utility method will create a payroll ready worker(no withholdings) with the given first name and work location,
     *   it will use default values for the rest of the fields
     *   NOTE: The workLocationName value must be one of the seeded workLocation name, currently the values are:
     *         "San Francisco, CA", "Mountain View, CA", "Mesquite, TX"
     *   
     * @param firstName the first name of the worker
     * @param workLocationName the name of the work location
     * @param createWages Create default wage items if true
     * @return The newly created worker
     * @throws Exception
     */
    private static Worker createAPayrollReadyWorkerWithNoWithHolding(String firstName, String workLocationName, boolean createWages)
    		throws Exception {
    	// Create an employee with some default values
    	Worker worker = createEmployee(firstName);
    	// Now set the workLocation
    	WorkLocation workLocation = getNamedWorkLocation(workLocationName);
    	worker.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	worker = taxComponent.applyApplicableTaxes(worker);
    	
        // Set the payrollDistributionMethod
        worker.setPayrollDistributionMethod(PayrollDistributionMethodEnum.HANDWRITTENPAYCHECK);
        if (createWages) {
        	// Setup wages
            worker = setupWages(worker);
        }
    	// Save the worker
        worker = workerComponent.addWorker(worker);
    	return worker;
    }

    /**
     * This utility method will create a payroll ready worker with the given first name and work location,
     *   it will use default values for the rest of the fields
     *   NOTE: The workLocationName value must be one of the seeded workLocation name, currently the values are:
     *         "San Francisco, CA", "Mountain View, CA", "Mesquite, TX"
     *   
     * @param firstName the first name of the worker
     * @param workLocationName the name of the work location
     * @param createWages Create default wage items if true
     * @return the id of the newly created worker
     * @throws Exception
     */
    private static Long createPayrollReadyWorker(String firstName, String workLocationName, boolean createWages)
    		throws Exception {
        Worker worker = createAPayrollReadyWorker(firstName, workLocationName, createWages);
    	return worker.getId();
    }

    /**
     * This utility method will create a payroll ready worker with the given first name, work location and some default
     *   wage items; it will use default values for the rest of the fields
     *   NOTE: The workLocationName value must be one of the seeded workLocation name, currently the values are:
     *         "San Francisco, CA", "Mountain View, CA", "Mesquite, TX"
     *   
     * @param firstName the first name of the worker
     * @param workLocationName the name of the work location
     * @return the id of the newly created worker
     * @throws Exception
     */
    public static Long createPayrollReadyWorker(String firstName, String workLocationName) throws Exception {
    	return createPayrollReadyWorker(firstName, workLocationName, true);
    }
    
    /**
     * This utility method will create a payroll ready worker with the given first name, work location and some default
     *   wage items; it will use default values for the rest of the fields
     *   NOTE: The workLocationName value must be one of the seeded workLocation name, currently the values are:
     *         "San Francisco, CA", "Mountain View, CA", "Mesquite, TX"
     *   
     * @param firstName the first name of the worker
     * @param workLocationName the name of the work location
     * @return The newly created worker
     * @throws Exception
     */
    public static Worker createAPayrollReadyWorker(String firstName, String workLocationName) throws Exception {
    	return createAPayrollReadyWorker(firstName, workLocationName, true);
    }
    
    /**
     * This utility method will create a payroll ready(no withholding) worker with the given first name, work location and some default
     *   wage items; it will use default values for the rest of the fields
     *   NOTE: The workLocationName value must be one of the seeded workLocation name, currently the values are:
     *         "San Francisco, CA", "Mountain View, CA", "Mesquite, TX"
     *   
     * @param firstName the first name of the worker
     * @param workLocationName the name of the work location
     * @return The newly created worker
     * @throws Exception
     */
    public static Worker createAPayrollReadyWorkerWithNoWithHolding(String firstName, String workLocationName) throws Exception {
    	return createAPayrollReadyWorkerWithNoWithHolding(firstName, workLocationName, true);
    }
    
    /**
     * This utility method will create a payroll ready worker with the given first name, work location and no wages;
     *   it will use default values for the rest of the fields
     *   NOTE: The workLocationName value must be one of the seeded workLocation name, currently the values are:
     *         "San Francisco, CA", "Mountain View, CA", "Mesquite, TX"
     *   
     * @param firstName the first name of the worker
     * @param workLocationName the name of the work location
     * @return the id of the newly created worker
     * @throws Exception
     */
    public static Long createNoWagePayrollReadyWorker(String firstName, String workLocationName) throws Exception {
    	return createPayrollReadyWorker(firstName, workLocationName, false);
    }
    
    /**
     * This utility method will create a payroll ready worker with the given first name, work location and no wages;
     *   it will use default values for the rest of the fields
     *   NOTE: The workLocationName value must be one of the seeded workLocation name, currently the values are:
     *         "San Francisco, CA", "Mountain View, CA", "Mesquite, TX"
     *   
     * @param firstName the first name of the worker
     * @param workLocationName the name of the work location
     * @return The newly created worker
     * @throws Exception
     */
    public static Worker createANoWagePayrollReadyWorker(String firstName, String workLocationName) throws Exception {
    	return createAPayrollReadyWorker(firstName, workLocationName, false);
    }
    
    /**
     * This utility method will create a payroll ready worker with the given first name, work location and a salary wage;
     *   it will use default values for the rest of the fields
     *   NOTE: The workLocationName value must be one of the seeded workLocation name, currently the values are:
     *         "San Francisco, CA", "Mountain View, CA", "Mesquite, TX"
     *   
     * @param firstName the first name of the worker
     * @param workLocationName the name of the work location
     * @return the id of the newly created worker
     * @throws Exception
     */
    public static Long createSalaryPayrollReadyWorker(String firstName, String workLocationName) throws Exception {
    	Worker worker = createASalaryPayrollReadyWorker(firstName, workLocationName);
    	return worker.getId();
    }
    
    /**
     * This utility method will create a payroll ready worker with the given first name, work location and a salary wage;
     *   it will use default values for the rest of the fields
     *   NOTE: The workLocationName value must be one of the seeded workLocation name, currently the values are:
     *         "San Francisco, CA", "Mountain View, CA", "Mesquite, TX"
     *   
     * @param firstName the first name of the worker
     * @param workLocationName the name of the work location
     * @return The newly created worker
     * @throws Exception
     */
    public static Worker createASalaryPayrollReadyWorker(String firstName, String workLocationName) throws Exception {
    	Worker worker = createANoWagePayrollReadyWorker(firstName, workLocationName);
    	worker = setupSalaryWage(worker);
    	return worker;
    }
    
	/**
	 * <P>Removes all payroll item details associated with the given worker</P>
	 *
	 * @param worker The worker to remove all payroll items
	 * @return The updated worker
	 */
	public static Worker removeAllWorkerPayrollItems(Worker worker) {
		// Remove all compensation items associated with the worker
		Iterator<CompensationDtl> compensationDtlIterator = worker.getCompensationDtls().iterator();
		List<CompensationDtl> listToDelete = new ArrayList<CompensationDtl>();
		while (compensationDtlIterator.hasNext()) {
		    CompensationDtl item = compensationDtlIterator.next();
			listToDelete.add(item);
		}
		for (CompensationDtl item : listToDelete) {
			// Remove the CompensationDtl association with the worker
			worker.removeCompensationDtl(item);
			Datum payrollDatum = item.getRate();
			if (null != payrollDatum) {
				// Remove the payrollDatum association with the CompensationDtl
				item.setRate(null);
			}
		}

		// Remove all PTO items associated with the worker
		Iterator<PaidTimeOffDtl> paidTimeOffDtlIterator = worker.getPaidTimeOffDtls().iterator();
		List<PaidTimeOffDtl> ptoListToDelete = new ArrayList<PaidTimeOffDtl>();
		while (paidTimeOffDtlIterator.hasNext()) {
			PaidTimeOffDtl item = paidTimeOffDtlIterator.next();
			ptoListToDelete.add(item);
		}
		for (PaidTimeOffDtl item : ptoListToDelete) {
			// Remove the PaidTimeOffDtl association with the worker
			worker.removePaidTimeOffDtl(item);
		}

		// Remove all health benefit items associated with the worker
		Iterator<HealthBenefitDtl> healthBenefitDtlIterator = worker.getHealthBenefitDtls().iterator();
		List<HealthBenefitDtl> healthBenefitListToDelete = new ArrayList<HealthBenefitDtl>();
		while (healthBenefitDtlIterator.hasNext()) {
			HealthBenefitDtl item = healthBenefitDtlIterator.next();
			healthBenefitListToDelete.add(item);
		}
		for (HealthBenefitDtl item : healthBenefitListToDelete) {
			// Remove the HealthBenefitDtl association with the worker
			worker.removeHealthBenefitDtl(item);
			Datum payrollDatum = item.getRate();
			if (null != payrollDatum) {
				// Remove the payrollDatum association with the HealthBenefitDtl
				item.setRate(null);
			}
		}

		// Remove all deduction items associated with the worker
		Iterator<DeductionDtl> deductionDtlIterator = worker.getDeductionDtls().iterator();
		List<DeductionDtl> deductionListToDelete = new ArrayList<DeductionDtl>();
		while (deductionDtlIterator.hasNext()) {
			DeductionDtl item = deductionDtlIterator.next();
			deductionListToDelete.add(item);
		}
		for (DeductionDtl item : deductionListToDelete) {
			// Remove the DeductionDtl association with the worker
			worker.removeDeductionDtl(item);
			Datum payrollDatum = item.getRate();
			if (null != payrollDatum) {
				// Remove the payrollDatum association with the DeductionDtl
				item.setRate(null);
			}
		}

		// Now remove the tax items
		Iterator<TaxDtl> taxDtlIterator = worker.getTaxDtls().iterator();
		List<TaxDtl> listOfTaxToDelete = new ArrayList<TaxDtl>();
		while (taxDtlIterator.hasNext()) {
			TaxDtl item = taxDtlIterator.next();
			listOfTaxToDelete.add(item);
		}
		for (TaxDtl item : listOfTaxToDelete) {
			// Remove the TaxDtl association with the worker
			worker.removeTaxDtl(item);

			Datum payrollDatum = item.getRate();
			if (null != payrollDatum) {
				// Remove the payrollDatum association with the PayrollItemDetail
				item.setRate(null);
			}
		}
		return worker;
	}

	/**
	 * <P>Removes the worker's withholding info that's specified by the given lawId.</P>
	 *
	 * @param worker The worker to remove the withholding information
	 * @param lawId The lawId specifying which withholdingInfo to remove
	 * @return The updated worker
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 * @throws WorkflowException 
	 */
	public static Worker removeWorkerWithholdingInfoByType(Worker worker, ComplianceDocIdEnum docType) throws Exception {
		Set<WorkerComplianceDocDtl> compDataForWorkers = worker.getWorkerComplianceDocDtls();
		WorkerComplianceDocDtl eComplianceData = null;
		// Locate the correct one
		for (WorkerComplianceDocDtl complianceData : compDataForWorkers) {
			if (complianceData.getDocId() == docType) {
				eComplianceData = complianceData;
				break;
			}
		}
		// Remove the associations if found
		if (null != eComplianceData) {
			worker.removeWorkerComplianceDocDtl(eComplianceData);
		}
		worker = workerComponent.updateWorker(worker);
		return worker;
	}

	/**
	 * This utility method will cleanup/delete the specified worker
	 * 
	 * @param worker The worker to delete
	 */
    public static void cleanUpWorker(Worker worker) {
    	if (null != worker) {
    		try {
    			worker = removeWorkerWithholdingInfoByType(worker, ComplianceDocIdEnum.W4IRS);
    			worker = removeWorkerWithholdingInfoByType(worker, ComplianceDocIdEnum.DE4);
    			// Remove the work location association
    			worker.setWorkLocation(null);
    			// Remove the worker's paycheck(s) if they exists
				worker.removeAllPaychecks();
    			worker = removeAllWorkerPayrollItems(worker);
    			//Before making the worker transient, remove tasks of this worker that are indirectly created while creation of this worker.
    			TaskUtils.removeAllWorkerTasks(worker.getId());
    			entityService.makeTransient(worker);
    		} catch (Exception e) {
	            e.printStackTrace();
	            AssertJUnit.fail("Error in cleanUpWorker: " + e.getMessage());
    		}
    	}
    }
    
}
