package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.Worker;



/**
 *  <P>The <CODE>CalculatePaycheckTest</CODE> class contains tests for testing the calculatePaycheck() in WorkerPaymentsComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class CalculatePaycheckExceptionsTest extends WorkerPaymentsComponentTestBase {
    private static WorkerPaymentsComponent sWorkerPaymentsComponent; 

    @BeforeClass
    public static void setUpCalculatePaycheckTest() {
    	sWorkerPaymentsComponent = 
            qaBootstrap.getEjbFromClass(WorkerPaymentsComponent.class);
        try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpCalculatePaycheckTest(): " + t.getMessage());
        }
    }
    
    @AfterClass
    public static void tearDownCalculatePaycheckTest() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownCalculatePaycheckTest(): " + t.getMessage());
        }
    }
    
    @BeforeMethod
    public void setUp() throws Exception {
        super.setUp();
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
        sEmployer.setNonProfit(false);
        setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
    }
    
    @AfterMethod
    public void tearDown() throws Exception
    {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
            // Remove withholdingInfo associated with the employee
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
        }
        super.tearDown();
    }
    
    /**
     * <P>Test calling calculatePaycheck() with a null PaycheckCalculationRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() with a null PaycheckCalculationRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testCalculatePaycheckWithNullRequest() throws Exception {
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(null);
    }

    /**
     * <P>Test calling calculatePaycheck() with a null PaycheckDate in PaycheckCalculationRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() with a null PaycheckDate in PaycheckCalculationRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testCalculatePaycheckWithNullPaycheckDate() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());

		PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
        paycheckCalculationRequest.setPaycheckDate(null);
        paycheckCalculationRequest.setPayPeriod(payPeriod);
        paycheckCalculationRequest.setCalculationInputs(calculationInputs);
        paycheckCalculationRequest.setWorkerId(worker.getId());
        paycheckCalculationRequest.setFrequency(frequency);

        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

    /**
     * <P>Test calling calculatePaycheck() with a null PayPeriod in PaycheckCalculationRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() with a null PayPeriod in PaycheckCalculationRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testCalculatePaycheckWithNullPayPeriod() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
        paycheckCalculationRequest.setPaycheckDate(mPaycheckDate.getTime());
        paycheckCalculationRequest.setPayPeriod(null);
        paycheckCalculationRequest.setCalculationInputs(calculationInputs);
        paycheckCalculationRequest.setWorkerId(worker.getId());
        paycheckCalculationRequest.setFrequency(frequency);
        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

    /**
     * <P>Test calling calculatePaycheck() with a null PayPeriod StartDate in PaycheckCalculationRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() with a null PayPeriod StartDate in PaycheckCalculationRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testCalculatePaycheckWithNullPayPeriodStartDate() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(null, mPeriodEndDate.getTime());
        Worker worker = entityService.findById(Worker.class, sWorkerId);

        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
        paycheckCalculationRequest.setPaycheckDate(mPaycheckDate.getTime());
        paycheckCalculationRequest.setPayPeriod(payPeriod);
        paycheckCalculationRequest.setCalculationInputs(calculationInputs);
        paycheckCalculationRequest.setWorkerId(worker.getId());
        paycheckCalculationRequest.setFrequency(frequency);
        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

    /**
     * <P>Test calling calculatePaycheck() with a null PayPeriod EndDate in PaycheckCalculationRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() with a null PayPeriod EndDate in PaycheckCalculationRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testCalculatePaycheckWithNullPayPeriodEndDate() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), null);
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
        paycheckCalculationRequest.setPaycheckDate(mPaycheckDate.getTime());
        paycheckCalculationRequest.setPayPeriod(payPeriod);
        paycheckCalculationRequest.setCalculationInputs(calculationInputs);
        paycheckCalculationRequest.setWorkerId(worker.getId());
        paycheckCalculationRequest.setFrequency(frequency);
        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

    /**
     * <P>Test calling calculatePaycheck() with a null PayrollFrequencyEnum in PaycheckCalculationRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() with a null PayrollFrequencyEnum in PaycheckCalculationRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testCalculatePaycheckWithNullPayrollFrequencyEnum() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
        paycheckCalculationRequest.setPaycheckDate(mPaycheckDate.getTime());
        paycheckCalculationRequest.setPayPeriod(payPeriod);
        paycheckCalculationRequest.setCalculationInputs(calculationInputs);
        paycheckCalculationRequest.setWorkerId(worker.getId());
        paycheckCalculationRequest.setFrequency(null);
        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

    /**
     * <P>Test calling calculatePaycheck() with PayrollFrequencyEnum.NOTSET in PaycheckCalculationRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() with PayrollFrequencyEnum.NOTSET in PaycheckCalculationRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testCalculatePaycheckWithPayrollFrequencyEnumNotSet() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Worker worker = entityService.findById(Worker.class, sWorkerId);

        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
        paycheckCalculationRequest.setPaycheckDate(mPaycheckDate.getTime());
        paycheckCalculationRequest.setPayPeriod(payPeriod);
        paycheckCalculationRequest.setCalculationInputs(calculationInputs);
        paycheckCalculationRequest.setWorkerId(worker.getId());
        paycheckCalculationRequest.setFrequency(PayrollFrequencyEnum.NOTSET);
        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

    /**
     * <P>Test calling calculatePaycheck() with a null WorkerId in PaycheckCalculationRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() with a null WorkerId in PaycheckCalculationRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testCalculatePaycheckWithNullWorkerId() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());

        List<CalculationInputs> calculationInputs = new ArrayList<CalculationInputs>();       
        
        PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
        paycheckCalculationRequest.setPaycheckDate(mPaycheckDate.getTime());
        paycheckCalculationRequest.setPayPeriod(payPeriod);
        paycheckCalculationRequest.setCalculationInputs(calculationInputs);
        paycheckCalculationRequest.setWorkerId(null);
        paycheckCalculationRequest.setFrequency(frequency);
        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

    /**
     * <P>Test calling calculatePaycheck() with a zero WorkerId in PaycheckCalculationRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() with a zero WorkerId in PaycheckCalculationRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testCalculatePaycheckWithZeroWorkerId() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = new ArrayList<CalculationInputs>();
        
        PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
        paycheckCalculationRequest.setPaycheckDate(mPaycheckDate.getTime());
        paycheckCalculationRequest.setPayPeriod(payPeriod);
        paycheckCalculationRequest.setCalculationInputs(calculationInputs);
        paycheckCalculationRequest.setWorkerId(0L);
        paycheckCalculationRequest.setFrequency(frequency);
        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

    /**
     * <P>Test calling calculatePaycheck() with a not ready for pay worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() to calculate the paycheck</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that WorkflowValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=WorkflowException.class)
        public void testCalculatePaycheckWithNotReadyForPayWorker() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Worker worker = entityService.findById(Worker.class, sWorkerId);

        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());        
        // Don't apply applicable taxes and don't set withholding information
        
        PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
        paycheckCalculationRequest.setPaycheckDate(mPaycheckDate.getTime());
        paycheckCalculationRequest.setPayPeriod(payPeriod);
        paycheckCalculationRequest.setCalculationInputs(calculationInputs);
        paycheckCalculationRequest.setWorkerId(worker.getId());
        paycheckCalculationRequest.setFrequency(frequency);
        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

    /**
     * <P>Test calling calculatePaycheck() with a non-existent workerId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Apply all applicable California taxes to the employee</LI>
     * <LI>Setup the withholding information for the employee</LI>
     * <LI>Call the WorkerPaymentsComponent.calculatePaycheck() with a non-existent workerId in PaycheckCalculationRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testCalculatePaycheckWithNonExistentWorkerId() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
        paycheckCalculationRequest.setPaycheckDate(mPaycheckDate.getTime());
        paycheckCalculationRequest.setPayPeriod(payPeriod);
        paycheckCalculationRequest.setCalculationInputs(calculationInputs);
        paycheckCalculationRequest.setWorkerId(784737438L);
        paycheckCalculationRequest.setFrequency(frequency);
        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

    /**
     * <P>Test pay period begin date after pay period end date</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Create a PayrollRun with the dates for a weekly payroll</LI>
     * <LI>Set the pay period begin date to be after the pay period end date</LI>
     * <LI>Call WorkerPaymentsComponent.calculatePaycheck() to calculate the paycheck</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void test_PayPeriod_BeginDate_After_EndDate() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 40.0);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(mPeriodEndDate.getTime(), sPeriodStartDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        @SuppressWarnings("unused")
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    }

}
