package com.intuit.sb.payroll.capability.workerpayments;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>CreatePayrollRunTest</CODE> class contains tests for testing the
 * createPayrollRun() in WorkerPaymentsComponent
 * </P>
 * 
 * @author smadduri
 *
 */
public class CreatePayrollRunTest extends WorkerPaymentsComponentTestBase
{	
	//Payroll ready workers with Hourly wages with variable Bonus
	protected static Long sReadyForPayWorkerId = 0L;
	protected static Long sReadyForPayWorkerId2 = 0L;
	//A non payroll ready worker (No wages)
	protected static Long sNotReadyForPayWorkerId = 0L;
	//Payreoll ready worker with salary variable Bonus
	protected static Long sSalaryWorkerId = 0L;	
	//Payroll ready worker with hourly wages and fixed commission
	protected static Long sFixedCommissionWorkerId = 0L;

	@BeforeClass
	public static void setUpCreatePayrollRunTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			sReadyForPayWorkerId2 = WorkerTestUtils.createPayrollReadyWorker("Paul", sMountainViewLocation);
			sFixedCommissionWorkerId = WorkerTestUtils.createPayrollReadyWorker("Trump", sMountainViewLocation);
			sNotReadyForPayWorkerId = WorkerTestUtils.createNoWagePayrollReadyWorker("Brad", sMountainViewLocation);
			sSalaryWorkerId = WorkerTestUtils.createSalaryPayrollReadyWorker("Madoff", sMountainViewLocation);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpFindPaycheckTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownCreatePayrollRunTest() {
		try {
			getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			if (0 != sReadyForPayWorkerId2) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId2);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId2 = 0L;
			}
			if (0 != sFixedCommissionWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sFixedCommissionWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId2 = 0L;
			}

			if(0 != sNotReadyForPayWorkerId)
			{
				Worker worker = entityService.findById(Worker.class, sNotReadyForPayWorkerId);
				WorkerTestUtils.cleanUpWorker(worker);
				sNotReadyForPayWorkerId = 0L;
			}
			if(0 != sSalaryWorkerId)
			{
				Worker worker = entityService.findById(Worker.class, sSalaryWorkerId);
				WorkerTestUtils.cleanUpWorker(worker);
				sSalaryWorkerId = 0L;
			}

			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
		}
	}
	
	//Utility method to validate payroll run
	private boolean validatePayrollRun(PayrollRunInfo payrollRunInfo, PayrollRun payrollRun)
	{
		if(payrollRunInfo.getPayDate().equals(payrollRun.getPayDate()) && 
				payrollRunInfo.getPayPeriod().equals(payrollRun.getPayPeriod()) &&
				payrollRunInfo.getPayrollRunDate().equals(payrollRun.getPayrollRunDate()) &&
				payrollRunInfo.getPayrollRunType().equals(payrollRun.getTypeOf()))
		{
			return true;
		}
		return false;
	}
	
	//Utility method to validate paycheck of an hourly worker
	private void validateHourlyWorkerPaycheck(Paycheck workerPaycheck)
	{
        //Since this worker doesn't have fixed wage, paycheck is not calculated during payroll run creation
    	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "0.00"), workerPaycheck.getNetAmount());
    	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "0.00"),
    			workerPaycheck.getWorkerPdTaxes());
    	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "0.00"),
    			workerPaycheck.getEmployerPdTaxes());
    	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
    			workerPaycheck.getWorkerPdBenefits());
    	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
    			workerPaycheck.getEmployerPdBenefits());

	}
	
	//Utility method to validate paycheck of a salaried worker
	private void validateSalaryWorkerPaycheck(Paycheck workerPaycheck)
	{
    	AssertJUnit.assertEquals("totalNetPay mismatch!",  new Money("USD", "1748.48"), workerPaycheck.getNetAmount());
    	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!",  new Money("USD", "789.98"),
    			workerPaycheck.getWorkerPdTaxes());
    	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!",new Money("USD", "303.35"),
    			workerPaycheck.getEmployerPdTaxes());
    	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
    			workerPaycheck.getWorkerPdBenefits());
    	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
    			workerPaycheck.getEmployerPdBenefits());

	}
	
	/**
	 * <P>This test is to create a scheduled payroll run with one payroll ready worker with hourly wages</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add worker ID to it</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the workers paycheck is not calculated since we passed in a worker who 
	 * doesn't have any fixed wages. Also verify the payroll run created is in DRAFT state</LI>
	 * </UL>
	 * 	
	 */
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"})
	public void testCreatePayrollRunScheduledWithOneHourlyPayrollReadyWorker() throws Exception
	{
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);
            
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
            
         	//get paychecks from payroll run for verification
         	Paycheck workerPaycheck = null;
            for (Paycheck paycheck : payrollRun.getPaychecks()) {
            	if (paycheck.getWorker().getId() == sReadyForPayWorkerId) {
            		workerPaycheck = paycheck;
            		break;
            	}
            }
            
            //Verify
            AssertJUnit.assertTrue(validatePayrollRun(payrollRunInfo, payrollRun));
            AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, payrollRun.getPaychecks().size());
            AssertJUnit.assertEquals("Workflow status mismatch!", PayrollRunWorkflowStatusEnum.DRAFT, 
            		payrollRun.getWorkflowStatus());
            
            validateHourlyWorkerPaycheck(workerPaycheck);
		}
		catch(Exception e)
		{
			AssertJUnit.fail("Unexpected exception occured: "+e.getMessage());
		}
		finally
		{
			//Clean up
			getUserTransaction().begin();
			if(payrollRun != null && payrollRun.getId() != 0)
			{
				cleanUpPayrollRun(payrollRun.getId());
			}
			getUserTransaction().commit();
		}
	}
	
	/**
	 * <P>This test is to create a AdHoc payroll run with one payroll ready worker with hourly wages</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add worker ID to it</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the workers paycheck is not calculated since we passed in a worker who 
	 * doesn't have any fixed wages. Also verify the payroll run created is in DRAFT state</LI>
	 * </UL>
	 * 	
	 */

	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"})
	public void testCreatePayrollRunAdHocWithOneHourlyPayrollReadyWorker() throws Exception
	{
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule, PayrollRunTypeEnum.ADHOC);
        	
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);
            
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
         	
         	//get paychecks from payroll run for verification
         	Paycheck workerPaycheck = null;
            for (Paycheck paycheck : payrollRun.getPaychecks()) {
            	if (paycheck.getWorker().getId() == sReadyForPayWorkerId) {
            		workerPaycheck = paycheck;
            		break;
            	}
            }
            
            //Verify
            AssertJUnit.assertTrue(validatePayrollRun(payrollRunInfo, payrollRun));
            AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, payrollRun.getPaychecks().size());
            AssertJUnit.assertEquals("Workflow status mismatch!", PayrollRunWorkflowStatusEnum.DRAFT, 
            		payrollRun.getWorkflowStatus());
            
            validateHourlyWorkerPaycheck(workerPaycheck);
		}
		catch(Exception e)
		{
			AssertJUnit.fail("Unexpected exception occured: "+e.getMessage());
		}
		finally
		{
			//Clean up
			getUserTransaction().begin();
			if(payrollRun != null && payrollRun.getId() != 0)
			{
				cleanUpPayrollRun(payrollRun.getId());
			}
			getUserTransaction().commit();
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run with one payroll ready worker with salary wages</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add worker ID to it</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the workers paycheck is calculated correctly.</LI> 
	 * <LI>Also verify the payroll run created is in DRAFT state.</LI>
	 * </UL>
	 * 	
	 */

	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"})
	public void testCreatePayrollRunWithOneSalariedWorker() throws Exception
	{
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sSalaryWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);
            
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();            
            
            //Verify
            AssertJUnit.assertTrue(validatePayrollRun(payrollRunInfo, payrollRun));
            AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, payrollRun.getPaychecks().size());
            AssertJUnit.assertEquals("Workflow status mismatch!", PayrollRunWorkflowStatusEnum.DRAFT, 
            		payrollRun.getWorkflowStatus());
            
         	//get paychecks from payroll run for verification
         	for (Paycheck paycheck : payrollRun.getPaychecks())
         	{
         		validateSalaryWorkerPaycheck(paycheck);
         	}
        }
		finally
		{
			//Clean up
			getUserTransaction().begin();
			if(payrollRun != null && payrollRun.getId() != 0)
			{
				cleanUpPayrollRun(payrollRun.getId());
			}
			getUserTransaction().commit();

		}

	}
	
	/**
	 * <P>This test is to create a scheduled payroll run with two payroll ready workers with 
	 * hourly wages</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add worker IDs to it</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the workers paycheck is not calculated since we passed in workers who 
	 * doesn't have any fixed wages. Also verify the payroll run created is in DRAFT state</LI>
	 * </UL>
	 * 	
	 */
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"})
	public void testCreatePayrollRunWithTwoHourlyPayrollReadyWorkers() throws Exception
	{
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            workerIdList.add(sReadyForPayWorkerId2);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);
            
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

            
            
            //Verify
            AssertJUnit.assertTrue(validatePayrollRun(payrollRunInfo, payrollRun));
            AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 2, payrollRun.getPaychecks().size());
            AssertJUnit.assertEquals("Workflow status mismatch!", PayrollRunWorkflowStatusEnum.DRAFT, 
            		payrollRun.getWorkflowStatus());
            
         	//get paychecks from payroll run for verification
         	for (Paycheck paycheck : payrollRun.getPaychecks())
         	{
         		//No need to validate for the worker because both the workers are hourly and
         		//paycheck should not be calculated. So, it doesn't matter in what order you check but
         		//all amounts in the paycheck should be USD0.00
         		validateHourlyWorkerPaycheck(paycheck);
         	}
        }
		finally
		{
			//Clean up
			getUserTransaction().begin();
			if(payrollRun != null && payrollRun.getId() != 0)
			{
				cleanUpPayrollRun(payrollRun.getId());
			}
			getUserTransaction().commit();

		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run with two payroll ready workers. One worker
	 *  with hourly wages and one with salary wages</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add worker IDs to it</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the workers paycheck calculated only for the salaried worker.</LI>
	 * <LI>Also verify the payroll run created is in DRAFT state.</LI>
	 * </UL>
	 * 	
	 */
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"})
	public void testCreatePayrollRunWithOneHourlyAndOneSalariedWorker() throws Exception
	{
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            workerIdList.add(sSalaryWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);
            
            
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
            
            //Verify
            AssertJUnit.assertTrue(validatePayrollRun(payrollRunInfo, payrollRun));
            AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 2, payrollRun.getPaychecks().size());
            AssertJUnit.assertEquals("Workflow status mismatch!", PayrollRunWorkflowStatusEnum.DRAFT, 
            		payrollRun.getWorkflowStatus());
            
         	//get paychecks from payroll run for verification
         	for (Paycheck paycheck : payrollRun.getPaychecks())
         	{
         		if(paycheck.getWorker().getId() == sReadyForPayWorkerId)
         		{
         			validateHourlyWorkerPaycheck(paycheck);
         		}
         		else if(paycheck.getWorker().getId() == sSalaryWorkerId)
         		{
         			validateSalaryWorkerPaycheck(paycheck);
         		}
         	}
		}
		finally
		{
			//Clean up
			getUserTransaction().begin();
			if(payrollRun != null && payrollRun.getId() != 0)
			{
				cleanUpPayrollRun(payrollRun.getId());
			}
			getUserTransaction().commit();
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run with one payroll ready worker with 
	 * hourly wages and a fixed wage(In this case commission is used)</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add worker ID to it</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the workers paycheck is calculated only for the commission amount.</LI> 
	 * <LI>Also verify the payroll run created is in DRAFT state</LI>
	 * </UL>
	 * 	
	 */
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"})
	public void testCreatepayrollRunForWorkerWithHourlyWageAndFixedCommission() throws Exception
	{
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
			Worker worker = entityService.findById(Worker.class, sFixedCommissionWorkerId);
			worker = WorkerTestUtils.addCommission(worker, true);
			
            getUserTransaction().commit();		
            
            sFixedCommissionWorkerId = worker.getId();
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sFixedCommissionWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);
            
            
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
            
         	//get paychecks from payroll run for verification
         	Paycheck workerPaycheck = null;
            for (Paycheck paycheck : payrollRun.getPaychecks()) {
            	if (paycheck.getWorker().getId() == sFixedCommissionWorkerId) {
            		workerPaycheck = paycheck;
            		break;
            	}
            }

            //Verify
            AssertJUnit.assertTrue(validatePayrollRun(payrollRunInfo, payrollRun));
            AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, payrollRun.getPaychecks().size());
            AssertJUnit.assertEquals("Workflow status mismatch!", PayrollRunWorkflowStatusEnum.DRAFT, 
            		payrollRun.getWorkflowStatus());
            //Since this worker doesn't have fixed wage, paycheck is not calculated during payroll run creation
        	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "430.70"), workerPaycheck.getNetAmount());
        	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "69.30"),
        			workerPaycheck.getWorkerPdTaxes());
        	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "59.75"),
        			workerPaycheck.getEmployerPdTaxes());
        	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
        			workerPaycheck.getWorkerPdBenefits());
        	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
        			workerPaycheck.getEmployerPdBenefits());

            
		}
		finally
		{
			//Clean up
			getUserTransaction().begin();
			if(payrollRun != null && payrollRun.getId() != 0)
			{
				cleanUpPayrollRun(payrollRun.getId());
			}
			getUserTransaction().commit();
		}
	}
}
