package com.intuit.sb.payroll.capability.workerpayments;

import java.util.Calendar;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;


/**
 * The <CODE>GetNextScheduledPayrollRunInfoSemiMonthlyTest</CODE> class contains tests for testing the
 * getNextScheduledPayrollRunInfo() in PayrollScheduleComponent using a semimonthly schedule with no existing PayrollRuns
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetNextScheduledPayrollRunInfoSemiMonthlyTest extends WorkerPaymentsComponentTestBase {
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 1st and the 16th of the month,
     * next period ends on 1/01/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_1stAnd16thDayOfMonth_01012009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 1);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 1, 16,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 17),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 1)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 6),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 1st and the 16th of the month,
     * next period ends on 1/16/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_1stAnd16thDayOfMonth_01162009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 16);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 1, 16,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 2),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 16)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 22),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 20),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 5th and the 20th of the month,
     * next period ends on 1/05/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_5thAnd20thDayOfMonth_01052009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 5);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 5, 20,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 21),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 5)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 8),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 6),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 5th and the 20th of the month,
     * next period ends on 1/20/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_5thAnd20thDayOfMonth_01202009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 20);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 5, 20,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 6),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 20)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 23),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 21),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 12th and the 27th of the month,
     * next period ends on 1/12/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_12thAnd27thDayOfMonth_01122009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 12);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 12, 27,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 28),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 12)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 15),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 13),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 12th and the 27th of the month,
     * next period ends on 1/27/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_12thAnd27thDayOfMonth_01272009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 27);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 12, 27,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 13),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 27)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 30),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 28),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 1/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_01152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 15)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 21),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 1/28/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_01282009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 28);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 1/29/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_01292009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 29);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 1/30/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_01302009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 30);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 1/31/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_01312009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 31);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 2/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_02152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.FEBRUARY, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 15)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 17),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 2/28/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_02282009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.FEBRUARY, 28);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 28)),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 4),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 3/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_03152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.MARCH, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 1),
		                                                           DateUtil.createDate(2009, Calendar.MARCH, 15)),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 18),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 16),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 3/31/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_03312009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.MARCH, 31);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 16),
		                                                           DateUtil.createDate(2009, Calendar.MARCH, 31)),
		                                             DateUtil.createDate(2009, Calendar.APRIL, 3),
		                                             DateUtil.createDate(2009, Calendar.APRIL, 1),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 4/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_04152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.APRIL, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 1),
		                                                           DateUtil.createDate(2009, Calendar.APRIL, 15)),
		                                             DateUtil.createDate(2009, Calendar.APRIL, 20),
		                                             DateUtil.createDate(2009, Calendar.APRIL, 16),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 4/30/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_04302009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.APRIL, 30);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 16),
		                                                           DateUtil.createDate(2009, Calendar.APRIL, 30)),
		                                             DateUtil.createDate(2009, Calendar.MAY, 5),
		                                             DateUtil.createDate(2009, Calendar.MAY, 1),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 5/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_05152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.MAY, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 1),
		                                                           DateUtil.createDate(2009, Calendar.MAY, 15)),
		                                             DateUtil.createDate(2009, Calendar.MAY, 20),
		                                             DateUtil.createDate(2009, Calendar.MAY, 18),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 5/31/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_05312009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.MAY, 31);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 16),
		                                                           DateUtil.createDate(2009, Calendar.MAY, 31)),
		                                             DateUtil.createDate(2009, Calendar.JUNE, 3),
		                                             DateUtil.createDate(2009, Calendar.JUNE, 1),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 6/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_06152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JUNE, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 1),
		                                                           DateUtil.createDate(2009, Calendar.JUNE, 15)),
		                                             DateUtil.createDate(2009, Calendar.JUNE, 18),
		                                             DateUtil.createDate(2009, Calendar.JUNE, 16),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 6/30/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_06302009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JUNE, 30);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 16),
		                                                           DateUtil.createDate(2009, Calendar.JUNE, 30)),
		                                             DateUtil.createDate(2009, Calendar.JULY, 3),
		                                             DateUtil.createDate(2009, Calendar.JULY, 1),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 7/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_07152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JULY, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 1),
		                                                           DateUtil.createDate(2009, Calendar.JULY, 15)),
		                                             DateUtil.createDate(2009, Calendar.JULY, 20),
		                                             DateUtil.createDate(2009, Calendar.JULY, 16),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 7/31/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_07312009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JULY, 31);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JULY, 31)),
		                                             DateUtil.createDate(2009, Calendar.AUGUST, 5),
		                                             DateUtil.createDate(2009, Calendar.AUGUST, 3),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 8/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_08152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.AUGUST, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 1),
		                                                           DateUtil.createDate(2009, Calendar.AUGUST, 15)),
		                                             DateUtil.createDate(2009, Calendar.AUGUST, 19),
		                                             DateUtil.createDate(2009, Calendar.AUGUST, 17),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 8/31/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_08312009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.AUGUST, 31);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 16),
		                                                           DateUtil.createDate(2009, Calendar.AUGUST, 31)),
		                                             DateUtil.createDate(2009, Calendar.SEPTEMBER, 3),
		                                             DateUtil.createDate(2009, Calendar.SEPTEMBER, 1),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 9/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_09152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.SEPTEMBER, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 1),
		                                                           DateUtil.createDate(2009, Calendar.SEPTEMBER, 15)),
		                                             DateUtil.createDate(2009, Calendar.SEPTEMBER, 18),
		                                             DateUtil.createDate(2009, Calendar.SEPTEMBER, 16),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 9/30/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_09302009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.SEPTEMBER, 30);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 16),
		                                                           DateUtil.createDate(2009, Calendar.SEPTEMBER, 30)),
		                                             DateUtil.createDate(2009, Calendar.OCTOBER, 5),
		                                             DateUtil.createDate(2009, Calendar.OCTOBER, 1),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 10/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_10152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.OCTOBER, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 1),
		                                                           DateUtil.createDate(2009, Calendar.OCTOBER, 15)),
		                                             DateUtil.createDate(2009, Calendar.OCTOBER, 20),
		                                             DateUtil.createDate(2009, Calendar.OCTOBER, 16),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 10/31/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_10312009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.OCTOBER, 31);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 16),
		                                                           DateUtil.createDate(2009, Calendar.OCTOBER, 31)),
		                                             DateUtil.createDate(2009, Calendar.NOVEMBER, 4),
		                                             DateUtil.createDate(2009, Calendar.NOVEMBER, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 11/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_11152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.NOVEMBER, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 1),
		                                                           DateUtil.createDate(2009, Calendar.NOVEMBER, 15)),
		                                             DateUtil.createDate(2009, Calendar.NOVEMBER, 18),
		                                             DateUtil.createDate(2009, Calendar.NOVEMBER, 16),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 11/30/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_11302009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.NOVEMBER, 30);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 16),
		                                                           DateUtil.createDate(2009, Calendar.NOVEMBER, 30)),
		                                             DateUtil.createDate(2009, Calendar.DECEMBER, 3),
		                                             DateUtil.createDate(2009, Calendar.DECEMBER, 1),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 12/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_12152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.DECEMBER, 15);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.DECEMBER, 1),
		                                                           DateUtil.createDate(2009, Calendar.DECEMBER, 15)),
		                                             DateUtil.createDate(2009, Calendar.DECEMBER, 18),
		                                             DateUtil.createDate(2009, Calendar.DECEMBER, 16),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 12/31/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_12312009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.DECEMBER, 31);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.DECEMBER, 16),
		                                                           DateUtil.createDate(2009, Calendar.DECEMBER, 31)),
		                                             DateUtil.createDate(2010, Calendar.JANUARY, 6),
		                                             DateUtil.createDate(2010, Calendar.JANUARY, 4),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 2/28/2012 (which is a leap year)</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_02282012_LeapYear() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2012, Calendar.FEBRUARY, 28);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2012, Calendar.FEBRUARY, 16),
		                                                           DateUtil.createDate(2012, Calendar.FEBRUARY, 29)),
		                                             DateUtil.createDate(2012, Calendar.MARCH, 5),
		                                             DateUtil.createDate(2012, Calendar.MARCH, 1),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period ends on the 15th and last day of the month,
     * next period ends on 2/29/2012 (which is a leap year)</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_15thAndLastDayOfMonth_02292012_LeapYear() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2012, Calendar.FEBRUARY, 29);
    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2012, Calendar.FEBRUARY, 16),
		                                                           DateUtil.createDate(2012, Calendar.FEBRUARY, 29)),
		                                             DateUtil.createDate(2012, Calendar.MARCH, 5),
		                                             DateUtil.createDate(2012, Calendar.MARCH, 1),
		                                             PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
}