package com.intuit.sb.payroll.capability.workerpayments;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;


/**
 * <P>The <CODE>GetUpcomingScheduledPayrollRunInfoBiweeklyTest</CODE> class contains tests for testing
 * getUpcomingScheduledPayrollRunInfo() in PayrollScheduleComponent using a biweekly schedule with no existing
 * PayrollRuns.</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetUpcomingScheduledPayrollRunInfoBiweeklyTest extends WorkerPaymentsComponentTestBase {
	
    /**
     * <P>Test getting up-coming PayrollRunInfo for the entire 2009 year, Pay period is from Monday to Sunday</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a biweekly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getUpcomingScheduledPayrollRunInfos() to get the up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcoming2009PayrollRunInfo_MondayToSunday() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 4);
    	aSchedule = createBiweeklySchedule(periodEndDate.getTime(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		List<PayrollRunInfo> computedList = pScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(aSchedule,
				DateUtil.createDate(2009, Calendar.JANUARY, 1));
		// Verify
		List<PayrollRunInfo> expectedList = new ArrayList<PayrollRunInfo>();
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 22),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 4)),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 7),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 5),
		                                    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 5),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 18)),
										                  // 1/19/2009 is a holiday
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 22),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 20),
		                                    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 19),
		                                                  DateUtil.createDate(2009, Calendar.FEBRUARY, 1)),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
                										  DateUtil.createDate(2009, Calendar.FEBRUARY, 15)),
										                  // 2/16/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 17),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 16),
														  DateUtil.createDate(2009, Calendar.MARCH, 1)),
										    DateUtil.createDate(2009, Calendar.MARCH, 4),
										    DateUtil.createDate(2009, Calendar.MARCH, 2),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 2),
														  DateUtil.createDate(2009, Calendar.MARCH, 15)),
										    DateUtil.createDate(2009, Calendar.MARCH, 18),
										    DateUtil.createDate(2009, Calendar.MARCH, 16),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 16),
														  DateUtil.createDate(2009, Calendar.MARCH, 29)),
										    DateUtil.createDate(2009, Calendar.APRIL, 1),
										    DateUtil.createDate(2009, Calendar.MARCH, 30),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 30),
														  DateUtil.createDate(2009, Calendar.APRIL, 12)),
										    DateUtil.createDate(2009, Calendar.APRIL, 15),
										    DateUtil.createDate(2009, Calendar.APRIL, 13),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 13),
														  DateUtil.createDate(2009, Calendar.APRIL, 26)),
										    DateUtil.createDate(2009, Calendar.APRIL, 29),
										    DateUtil.createDate(2009, Calendar.APRIL, 27),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 27),
														  DateUtil.createDate(2009, Calendar.MAY, 10)),
										    DateUtil.createDate(2009, Calendar.MAY, 13),
										    DateUtil.createDate(2009, Calendar.MAY, 11),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 11),
														  DateUtil.createDate(2009, Calendar.MAY, 24)),
										                  // 5/25/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.MAY, 28),
										    DateUtil.createDate(2009, Calendar.MAY, 26),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 25),
														  DateUtil.createDate(2009, Calendar.JUNE, 7)),
										    DateUtil.createDate(2009, Calendar.JUNE, 10),
										    DateUtil.createDate(2009, Calendar.JUNE, 8),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 8),
														  DateUtil.createDate(2009, Calendar.JUNE, 21)),
										    DateUtil.createDate(2009, Calendar.JUNE, 24),
										    DateUtil.createDate(2009, Calendar.JUNE, 22),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 22),
														  DateUtil.createDate(2009, Calendar.JULY, 5)),
										    DateUtil.createDate(2009, Calendar.JULY, 8),
										    DateUtil.createDate(2009, Calendar.JULY, 6),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 6),
														  DateUtil.createDate(2009, Calendar.JULY, 19)),
										    DateUtil.createDate(2009, Calendar.JULY, 22),
										    DateUtil.createDate(2009, Calendar.JULY, 20),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 20),
														  DateUtil.createDate(2009, Calendar.AUGUST, 2)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 5),
										    DateUtil.createDate(2009, Calendar.AUGUST, 3),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 3),
														  DateUtil.createDate(2009, Calendar.AUGUST, 16)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 19),
										    DateUtil.createDate(2009, Calendar.AUGUST, 17),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 17),
														  DateUtil.createDate(2009, Calendar.AUGUST, 30)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 2),
										    DateUtil.createDate(2009, Calendar.AUGUST, 31),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 31),
														  DateUtil.createDate(2009, Calendar.SEPTEMBER, 13)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 16),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 14),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 14),
														  DateUtil.createDate(2009, Calendar.SEPTEMBER, 27)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 30),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 28),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 28),
														  DateUtil.createDate(2009, Calendar.OCTOBER, 11)),
										                  // 10/12/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.OCTOBER, 15),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 13),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 12),
														  DateUtil.createDate(2009, Calendar.OCTOBER, 25)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 28),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 26),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 26),
														  DateUtil.createDate(2009, Calendar.NOVEMBER, 8)),
										                  // 11/11/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 12),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 9),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 9),
														  DateUtil.createDate(2009, Calendar.NOVEMBER, 22)),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 25),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 23),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 23),
														  DateUtil.createDate(2009, Calendar.DECEMBER, 6)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 9),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 7),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.DECEMBER, 7),
														  DateUtil.createDate(2009, Calendar.DECEMBER, 20)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 23),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 21),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfoList(expectedList, computedList));
	}
	
    /**
     * <P>Test getting the up-coming PayrollRunInfo for the entire 2009 year, Pay period is from Wednesday to Tuesday</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a biweekly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getUpcomingScheduledPayrollRunInfos() to get the up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcoming2009PayrollRunInfo_WednesdayToTuesday() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 6);
    	aSchedule = createBiweeklySchedule(periodEndDate.getTime(), sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		List<PayrollRunInfo> computedList = pScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(aSchedule,
				DateUtil.createDate(2009, Calendar.JANUARY, 1));
		// Verify
		List<PayrollRunInfo> expectedList = new ArrayList<PayrollRunInfo>();
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 24),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 6)),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 9),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 7),
		                                    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 7),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 20)),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 23),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 21),
		                                    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 21),
		                                                  DateUtil.createDate(2009, Calendar.FEBRUARY, 3)),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 6),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
										                  DateUtil.createDate(2009, Calendar.FEBRUARY, 17)),
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 20),
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 18),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 18),
										                  DateUtil.createDate(2009, Calendar.MARCH, 3)),
										    DateUtil.createDate(2009, Calendar.MARCH, 6),
										    DateUtil.createDate(2009, Calendar.MARCH, 4),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 4),
										                  DateUtil.createDate(2009, Calendar.MARCH, 17)),
										    DateUtil.createDate(2009, Calendar.MARCH, 20),
										    DateUtil.createDate(2009, Calendar.MARCH, 18),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 18),
										                  DateUtil.createDate(2009, Calendar.MARCH, 31)),
										    DateUtil.createDate(2009, Calendar.APRIL, 3),
										    DateUtil.createDate(2009, Calendar.APRIL, 1),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 1),
										                  DateUtil.createDate(2009, Calendar.APRIL, 14)),
										    DateUtil.createDate(2009, Calendar.APRIL, 17),
										    DateUtil.createDate(2009, Calendar.APRIL, 15),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 15),
										                  DateUtil.createDate(2009, Calendar.APRIL, 28)),
										    DateUtil.createDate(2009, Calendar.MAY, 1),
										    DateUtil.createDate(2009, Calendar.APRIL, 29),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 29),
										                  DateUtil.createDate(2009, Calendar.MAY, 12)),
										    DateUtil.createDate(2009, Calendar.MAY, 15),
										    DateUtil.createDate(2009, Calendar.MAY, 13),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 13),
										                  DateUtil.createDate(2009, Calendar.MAY, 26)),
										    DateUtil.createDate(2009, Calendar.MAY, 29),
										    DateUtil.createDate(2009, Calendar.MAY, 27),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 27),
										                  DateUtil.createDate(2009, Calendar.JUNE, 9)),
										    DateUtil.createDate(2009, Calendar.JUNE, 12),
										    DateUtil.createDate(2009, Calendar.JUNE, 10),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 10),
										                  DateUtil.createDate(2009, Calendar.JUNE, 23)),
										    DateUtil.createDate(2009, Calendar.JUNE, 26),
										    DateUtil.createDate(2009, Calendar.JUNE, 24),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 24),
										                  DateUtil.createDate(2009, Calendar.JULY, 7)),
										    DateUtil.createDate(2009, Calendar.JULY, 10),
										    DateUtil.createDate(2009, Calendar.JULY, 8),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 8),
										                  DateUtil.createDate(2009, Calendar.JULY, 21)),
										    DateUtil.createDate(2009, Calendar.JULY, 24),
										    DateUtil.createDate(2009, Calendar.JULY, 22),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 22),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 4)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 7),
										    DateUtil.createDate(2009, Calendar.AUGUST, 5),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 5),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 18)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 21),
										    DateUtil.createDate(2009, Calendar.AUGUST, 19),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 19),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 1)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 4),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 2),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 2),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 15)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 18),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 16),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 16),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 29)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 2),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 30),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 30),
										                  DateUtil.createDate(2009, Calendar.OCTOBER, 13)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 16),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 14),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 14),
										                  DateUtil.createDate(2009, Calendar.OCTOBER, 27)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 30),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 28),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 28),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 10)),
										                  // 11/11/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 16),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 12),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 11),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 24)),
										                  // 11/26/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 30),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 25),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 25),
										                  DateUtil.createDate(2009, Calendar.DECEMBER, 8)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 11),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 9),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.DECEMBER, 9),
										                  DateUtil.createDate(2009, Calendar.DECEMBER, 22)),
										                  // 12/25/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.DECEMBER, 28),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 23),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfoList(expectedList, computedList));
	}
	
    /**
     * <P>Test getting the up-coming PayrollRunInfo in a leap year</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a biweekly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getUpcomingScheduledPayrollRunInfos() to get the up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcoming2008PayrollRunInfo_LeapYear() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2008, Calendar.MARCH, 4);
    	aSchedule = createBiweeklySchedule(periodEndDate.getTime(), sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		List<PayrollRunInfo> computedList = pScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(aSchedule,
				DateUtil.createDate(2008, Calendar.JANUARY, 1));
		// Verify
		List<PayrollRunInfo> expectedList = new ArrayList<PayrollRunInfo>();
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.FEBRUARY, 20),
		                                                  DateUtil.createDate(2008, Calendar.MARCH, 4)),
		                                    DateUtil.createDate(2008, Calendar.MARCH, 7),
		                                    DateUtil.createDate(2008, Calendar.MARCH, 5),
		                                    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.MARCH, 5),
		                                                  DateUtil.createDate(2008, Calendar.MARCH, 18)),
		                                    DateUtil.createDate(2008, Calendar.MARCH, 21),
		                                    DateUtil.createDate(2008, Calendar.MARCH, 19),
		                                    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.MARCH, 19),
		                                                  DateUtil.createDate(2008, Calendar.APRIL, 1)),
		                                    DateUtil.createDate(2008, Calendar.APRIL, 4),
		                                    DateUtil.createDate(2008, Calendar.APRIL, 2),
		                                    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.APRIL, 2),
										                  DateUtil.createDate(2008, Calendar.APRIL, 15)),
										    DateUtil.createDate(2008, Calendar.APRIL, 18),
										    DateUtil.createDate(2008, Calendar.APRIL, 16),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.APRIL, 16),
										                  DateUtil.createDate(2008, Calendar.APRIL, 29)),
										    DateUtil.createDate(2008, Calendar.MAY, 2),
										    DateUtil.createDate(2008, Calendar.APRIL, 30),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.APRIL, 30),
										                  DateUtil.createDate(2008, Calendar.MAY, 13)),
										    DateUtil.createDate(2008, Calendar.MAY, 16),
										    DateUtil.createDate(2008, Calendar.MAY, 14),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.MAY, 14),
										                  DateUtil.createDate(2008, Calendar.MAY, 27)),
										    DateUtil.createDate(2008, Calendar.MAY, 30),
										    DateUtil.createDate(2008, Calendar.MAY, 28),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.MAY, 28),
										                  DateUtil.createDate(2008, Calendar.JUNE, 10)),
										    DateUtil.createDate(2008, Calendar.JUNE, 13),
										    DateUtil.createDate(2008, Calendar.JUNE, 11),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.JUNE, 11),
										                  DateUtil.createDate(2008, Calendar.JUNE, 24)),
										    DateUtil.createDate(2008, Calendar.JUNE, 27),
										    DateUtil.createDate(2008, Calendar.JUNE, 25),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.JUNE, 25),
										                  DateUtil.createDate(2008, Calendar.JULY, 8)),
										    DateUtil.createDate(2008, Calendar.JULY, 11),
										    DateUtil.createDate(2008, Calendar.JULY, 9),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.JULY, 9),
										                  DateUtil.createDate(2008, Calendar.JULY, 22)),
										    DateUtil.createDate(2008, Calendar.JULY, 25),
										    DateUtil.createDate(2008, Calendar.JULY, 23),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.JULY, 23),
										                  DateUtil.createDate(2008, Calendar.AUGUST, 5)),
										    DateUtil.createDate(2008, Calendar.AUGUST, 8),
										    DateUtil.createDate(2008, Calendar.AUGUST, 6),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.AUGUST, 6),
										                  DateUtil.createDate(2008, Calendar.AUGUST, 19)),
										    DateUtil.createDate(2008, Calendar.AUGUST, 22),
										    DateUtil.createDate(2008, Calendar.AUGUST, 20),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.AUGUST, 20),
										                  DateUtil.createDate(2008, Calendar.SEPTEMBER, 2)),
										    DateUtil.createDate(2008, Calendar.SEPTEMBER, 5),
										    DateUtil.createDate(2008, Calendar.SEPTEMBER, 3),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.SEPTEMBER, 3),
										                  DateUtil.createDate(2008, Calendar.SEPTEMBER, 16)),
										    DateUtil.createDate(2008, Calendar.SEPTEMBER, 19),
										    DateUtil.createDate(2008, Calendar.SEPTEMBER, 17),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.SEPTEMBER, 17),
										                  DateUtil.createDate(2008, Calendar.SEPTEMBER, 30)),
										    DateUtil.createDate(2008, Calendar.OCTOBER, 3),
										    DateUtil.createDate(2008, Calendar.OCTOBER, 1),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.OCTOBER, 1),
										                  DateUtil.createDate(2008, Calendar.OCTOBER, 14)),
										    DateUtil.createDate(2008, Calendar.OCTOBER, 17),
										    DateUtil.createDate(2008, Calendar.OCTOBER, 15),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.OCTOBER, 15),
										                  DateUtil.createDate(2008, Calendar.OCTOBER, 28)),
										    DateUtil.createDate(2008, Calendar.OCTOBER, 31),
										    DateUtil.createDate(2008, Calendar.OCTOBER, 29),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.OCTOBER, 29),
										                  DateUtil.createDate(2008, Calendar.NOVEMBER, 11)),
										    DateUtil.createDate(2008, Calendar.NOVEMBER, 14),
										    DateUtil.createDate(2008, Calendar.NOVEMBER, 12),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.NOVEMBER, 12),
										                  DateUtil.createDate(2008, Calendar.NOVEMBER, 25)),
										                  // 11/27/2009 is a holiday
										    DateUtil.createDate(2008, Calendar.DECEMBER, 1),
										    DateUtil.createDate(2008, Calendar.NOVEMBER, 26),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.NOVEMBER, 26),
										                  DateUtil.createDate(2008, Calendar.DECEMBER, 9)),
										    DateUtil.createDate(2008, Calendar.DECEMBER, 12),
										    DateUtil.createDate(2008, Calendar.DECEMBER, 10),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 10),
										                  DateUtil.createDate(2008, Calendar.DECEMBER, 23)),
										                  // 12/25/2009 is a holiday
										    DateUtil.createDate(2008, Calendar.DECEMBER, 29),
										    DateUtil.createDate(2008, Calendar.DECEMBER, 24),
										    PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfoList(expectedList, computedList));
	}
	
}
