package com.intuit.sb.payroll.capability.workerpayments;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;


/**
 * <P>The <CODE>GetUpcomingScheduledPayrollRunInfoSemiMonthlyTest</CODE> class contains tests for testing
 * getUpcomingScheduledPayrollRunInfo() in PayrollScheduleComponent using a semimonthly schedule with no existing
 * PayrollRuns.</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetUpcomingScheduledPayrollRunInfoSemiMonthlyTest extends WorkerPaymentsComponentTestBase {
	
    /**
     * <P>Test getting the up-coming PayrollRunInfo, Pay period end date is on the 15th and last day of the
     *  month, next period ends on 1/15/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getUpcomingScheduledPayrollRunInfos() to get the up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_15thAndLastDayOfMonth_01152009() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 15);
		PayrollSchedule aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15,
		                                                      PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		
		// Exercise
		List<PayrollRunInfo> computedList = pScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(aSchedule,
				DateUtil.createDate(2009, Calendar.JANUARY, 1));
		
		// Verify
		List<PayrollRunInfo> expectedList = new ArrayList<PayrollRunInfo>();
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 1),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 15)),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 21),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                  DateUtil.createDate(2009, Calendar.FEBRUARY, 15)),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 17),
		                                    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 16),
										                  DateUtil.createDate(2009, Calendar.FEBRUARY, 28)),
										    DateUtil.createDate(2009, Calendar.MARCH, 4),
										    DateUtil.createDate(2009, Calendar.MARCH, 2),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 1),
										                  DateUtil.createDate(2009, Calendar.MARCH, 15)),
										    DateUtil.createDate(2009, Calendar.MARCH, 18),
										    DateUtil.createDate(2009, Calendar.MARCH, 16),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 16),
										                  DateUtil.createDate(2009, Calendar.MARCH, 31)),
										    DateUtil.createDate(2009, Calendar.APRIL, 3),
										    DateUtil.createDate(2009, Calendar.APRIL, 1),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 1),
										                  DateUtil.createDate(2009, Calendar.APRIL, 15)),
										    DateUtil.createDate(2009, Calendar.APRIL, 20),
										    DateUtil.createDate(2009, Calendar.APRIL, 16),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 16),
										                  DateUtil.createDate(2009, Calendar.APRIL, 30)),
										    DateUtil.createDate(2009, Calendar.MAY, 5),
										    DateUtil.createDate(2009, Calendar.MAY, 1),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 1),
										                  DateUtil.createDate(2009, Calendar.MAY, 15)),
										    DateUtil.createDate(2009, Calendar.MAY, 20),
										    DateUtil.createDate(2009, Calendar.MAY, 18),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 16),
										                  DateUtil.createDate(2009, Calendar.MAY, 31)),
										    DateUtil.createDate(2009, Calendar.JUNE, 3),
										    DateUtil.createDate(2009, Calendar.JUNE, 1),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 1),
										                  DateUtil.createDate(2009, Calendar.JUNE, 15)),
										    DateUtil.createDate(2009, Calendar.JUNE, 18),
										    DateUtil.createDate(2009, Calendar.JUNE, 16),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 16),
										                  DateUtil.createDate(2009, Calendar.JUNE, 30)),
										    DateUtil.createDate(2009, Calendar.JULY, 3),
										    DateUtil.createDate(2009, Calendar.JULY, 1),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 1),
										                  DateUtil.createDate(2009, Calendar.JULY, 15)),
										    DateUtil.createDate(2009, Calendar.JULY, 20),
										    DateUtil.createDate(2009, Calendar.JULY, 16),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 16),
										                  DateUtil.createDate(2009, Calendar.JULY, 31)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 5),
										    DateUtil.createDate(2009, Calendar.AUGUST, 3),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 1),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 15)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 19),
										    DateUtil.createDate(2009, Calendar.AUGUST, 17),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 16),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 31)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 3),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 1),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 1),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 15)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 18),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 16),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 16),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 30)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 5),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 1),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 1),
										                  DateUtil.createDate(2009, Calendar.OCTOBER, 15)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 20),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 16),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 16),
										                  DateUtil.createDate(2009, Calendar.OCTOBER, 31)),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 4),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 2),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 1),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 15)),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 18),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 16),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 16),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 30)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 3),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 1),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.DECEMBER, 1),
										                  DateUtil.createDate(2009, Calendar.DECEMBER, 15)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 18),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 16),
										    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfoList(expectedList, computedList));
	}
	
    /**
     * <P>Test getting the up-coming PayrollRunInfo, Pay period end date is on the 5th and the 20th of the
     *  month, next period ends on 12/20/2008</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a semimonthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getUpcomingScheduledPayrollRunInfos() to get the up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_5thAnd20thOfMonth_12202008() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2008, Calendar.DECEMBER, 20);
		PayrollSchedule aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 5, 20,
		                                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		
		// Exercise
		List<PayrollRunInfo> computedList = pScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(aSchedule,
				DateUtil.createDate(2008, Calendar.JANUARY, 1));
		
		// Verify
		List<PayrollRunInfo> expectedList = new ArrayList<PayrollRunInfo>();
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 6),
		                                        		  DateUtil.createDate(2008, Calendar.DECEMBER, 20)),
		                                    DateUtil.createDate(2008, Calendar.DECEMBER, 24),
		                                    DateUtil.createDate(2008, Calendar.DECEMBER, 22),
		                                    PayrollFrequencyEnum.SEMIMONTHLY, PayrollRunTypeEnum.SCHEDULED));
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfoList(expectedList, computedList));
	}
	
}