package com.intuit.sb.payroll.capability.workerpayments;

import java.util.Calendar;
import java.util.List;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.common.PayrollTestBase;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.workerpayments.impl.PayrollScheduleDateUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;

/**
 * This java file contains all the utility methods use for tests PayrollSchedule implementation
 * 
 */

public class PayrollScheduleTestUtils {

    
	/**
	 * <P>
	 * Utility method to create the PayrollSchedule
	 * </P>
	 */
	public static PayrollSchedule createPayrollSchedule(Calendar startDate, int endDate,
			PayrollFrequencyEnum frequency, int payrollRunDateOffset, int payrollPayDateOffset) throws Exception {

		PayrollSchedule payrollSchedule = new PayrollSchedule();
		payrollSchedule.setFrequency(frequency);
		payrollSchedule.setPayDayOffset(payrollPayDateOffset);
		payrollSchedule.setPayrollRunDayOffset(payrollRunDateOffset);
		payrollSchedule.setPeriodStartDate(startDate.getTime());
		
		if (frequency == PayrollFrequencyEnum.MONTHLY) {
			payrollSchedule.setPayPeriodEndDayofMonth1(28);
		} else if (frequency == PayrollFrequencyEnum.SEMIMONTHLY) {
			payrollSchedule.setPayPeriodEndDayofMonth1(15);

			payrollSchedule.setPayPeriodEndDayofMonth2(PayrollSchedule.LAST_DAY_OF_MONTH);
		} else {
			Calendar periodEndCal = PayrollScheduleDateUtils.getPayPeriodEndDate(startDate, frequency);
			payrollSchedule.setNextPayPeriodEndDate(periodEndCal.getTime());
		}

		return payrollSchedule;

	}
	
	/**
	 * Creates a default schedule and assigns is to the current employer.
	 * 
	 * @return the default PayrollSchedule.
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 */
	public static PayrollSchedule createDefaultSchedule()  {
		final int sDefaultPayrollDayOffset = 3;
		final int sDefaultPayrollRunDayOffset = 1;
		final int sDefaultPayPeriodEndDayOfMonth1 = 0;
		final int sDefaultPayPeriodEndDayOfMonth2 = 0;

		PayrollSchedule payrollSchedule = new PayrollSchedule();
		payrollSchedule.setDefaultt(true);
		payrollSchedule.setFrequency(PayrollFrequencyEnum.BIWEEKLY);
		payrollSchedule.setPayDayOffset(sDefaultPayrollDayOffset);
		payrollSchedule.setPayrollRunDayOffset(sDefaultPayrollRunDayOffset);
		payrollSchedule.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
		payrollSchedule.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
		Calendar pDate = LocaleGuru.getInstance().getCalendar();
		// To avoid Thursday and Friday
		pDate.add(Calendar.DAY_OF_YEAR, sDefaultPayrollDayOffset - sDefaultPayrollRunDayOffset);
		pDate = DateUtil.getNextWeekDayOfWeek(pDate, Calendar.FRIDAY);
		pDate.add(Calendar.DAY_OF_YEAR, -sDefaultPayrollDayOffset);
		payrollSchedule.setNextPayPeriodEndDate(pDate.getTime());
		
	    PayrollScheduleComponent psComponent  = PayrollTestBase.getQaBootstrap().getEjbFromClass(PayrollScheduleComponent.class);
	    EmployerComponent employerComponent = PayrollTestBase.getQaBootstrap().getEjbFromClass(EmployerComponent.class);

		//Remove any existing ones
		List<PayrollSchedule>  allSchedules = BasicTestBase.getEntityService().findAll(PayrollSchedule.class);
		if (allSchedules.size() > 0) {
			// TODO Remove the Worker associations too
			
			try {
				Employer employer = employerComponent.getEmployer();
				for (PayrollSchedule payrollSchedule2 : allSchedules) {
					if (employer.getPayrollSchedules().contains(payrollSchedule2)){
						psComponent.removePayrollScheduleFromEmployer(payrollSchedule2);
					}
				}
			} catch (ServiceFailureException e) {
				e.printStackTrace();
			} catch (ValidationException e) {
				e.printStackTrace();
			}
			
			for (PayrollSchedule payrollSchedule2 : allSchedules) {
				BasicTestBase.getEntityService().makeTransient(payrollSchedule2);
			}
		}
		// saving using PayrollSchedule component
		try {
			payrollSchedule = psComponent.createPayrollSchedule(payrollSchedule);
			// Assign to the employer
			psComponent.assignPayrollScheduleToEmployer(payrollSchedule);
		} catch (ValidationException e) {
			// TODO handle exceptions in proper way
			e.printStackTrace();
		} catch (ServiceFailureException e) {
			e.printStackTrace();
		}
		
		return payrollSchedule;
	}
}
