package com.intuit.sb.payroll.capability.workerpayments;

import java.util.ArrayList;
import java.util.List;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.PayrollRun;

/**
 * <P>
 * The <CODE>VoidPaychecksExceptionsTest</CODE> class contains tests for testing the Exceptions that can be thrown by
 * voidPaychecks() method in WorkerPaymentsComponent
 * </P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class VoidPaychecksExceptionsTest extends WorkerPaymentsComponentTestBase {

	/**
	 * <P>Test calling voidPaychecks() with a null paycheckIds list</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call WorkerPaymentsComponent.voidPaychecks() with a null paycheckIds list</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPaycheckIdsList() throws Exception {
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.voidPaychecks(null);
	}

	/**
	 * <P>Test calling voidPaychecks() with an empty paycheckIds list</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call WorkerPaymentsComponent.voidPaychecks() with an empty paycheckIds list</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testEmptyPaycheckIdsList() throws Exception {
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.voidPaychecks(new ArrayList<Long>());
	}

	/**
	 * <P>Test calling voidPaychecks() with a null paycheckIds list element</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.voidPaychecks() with a null paycheckIds list element</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPaycheckIdsElement() throws Exception {
		// Setup
		List<Long> paycheckIds = new ArrayList<Long>();
		paycheckIds.add(null);
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	}

	/**
	 * <P>Test calling voidPaychecks() with a non-existent paycheckIds list element</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.voidPaychecks() with a non-existent paycheckIds list element</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testNonExistentPaycheckIdsElement() throws Exception {
		// Setup
		List<Long> paycheckIds = new ArrayList<Long>();
		paycheckIds.add(793849349L);
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	}
}
