package com.intuit.sb.payroll.reports.impl;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.persistence.ColumnResult;
import javax.persistence.EntityManager;
import javax.persistence.SqlResultSetMapping;

import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.reports.ReportDetail;
import com.intuit.sb.payroll.reports.ReportRequest;
import com.intuit.sb.payroll.reports.ReportResponse;
import com.intuit.sb.payroll.reports.ReportService;
import com.intuit.sb.payroll.reports.ReportSubtotals;
import com.intuit.sb.payroll.reports.ReportType;
import com.intuit.sb.payroll.takehome.entity.TakehomeReportsEAO;

/**
 * Implementation of ReportService interface.
 * 
 */
@Stateless(name = "ReportService")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })

@SqlResultSetMapping (
		name = "XmlResult",
		columns = {
		@ColumnResult( name = "XMLRES")})

public class ReportServiceBean implements ReportService {
	
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(ReportServiceBean.class);
	
	private static final String sCaTaxLiabilitiesSampleResult = new String (
		""
		+ "<ROWSET>"
		+ 	 "<ROW>"
        +           "<COL>December 2008</COL>"
        +           "<COL>CA Employment Training Tax (ETT)</COL>"
        +           "<COL>CA State Disability Insurance (SDI)</COL>"
        +           "<COL>CA Unemployment Insurance (UI)</COL>"
        +           "<COL>CA Personal Income Tax Withholding (PIT)</COL>"
        +           "<COL>TOTAL DUE</COL>"
        +     "</ROW>"
        +   "<ROW>"
        +           "<COL>Paycheck 28/05/09</COL>"
        +           "<COL>3.08</COL><COL>33.84</COL><COL>104.62</COL><COL>184.32</COL>"
        +           "<COL>$400.00</COL>"
        +     "</ROW>"
        +   "<ROW>"
        +           "<COL>Paycheck 04/06/09</COL>"
        +           "<COL>3.96</COL><COL>43.67</COL><COL>134.94</COL><COL>267.3</COL>"
        +           "<COL>$400.00</COL>"
        +     "</ROW>"
        +   "<ROW>"
        +           "<COL>Paycheck 11/06/09</COL>"
        +           "<COL>3.97</COL><COL>43.66</COL><COL>134.96</COL><COL>267.3</COL>"
        +           "<COL>$400.00</COL>"
        +     "</ROW>"
        +   "<ROW>"
        +           "<COL>Total</COL>"
        +           "<COL>300.00</COL><COL>300.00</COL><COL>300.00</COL><COL>300.00</COL>"
        +           "<COL>$1,200.00</COL>"
        +     "</ROW>"
		+ "</ROWSET>"
	);

	private static final String sCaTaxLiabilitiesReportQueryPerPaycheckPart1 = new String (
			  "	'select DISTINCT to_date(''01-JAN-1700'') as IGNORE_COL,''"
	);
	private static final String sCaTaxLiabilitiesReportQueryPerPaycheckPart2 = new String (		
			  "'' as COL  " 
			// The four lines below show the recommended way of retrieving payroll item description
			+ "		, (select DISTINCT description from payrollitem plPIT where lawid_value = ''6'') as COL "
			+ "		, (select DISTINCT description from payrollitem plSDI where lawid_value = ''67'') as COL "
			+ "		, (select DISTINCT description from payrollitem plSUI where lawid_value = ''87'') as COL "
			+ "		, (select DISTINCT description from payrollitem plETT where lawid_value = ''142'') as COL "
            + "     , ''TOTAL DUE'' as COL " 
            + "from dual "
            + ""
            + "UNION ALL "
            + ""
            + "	select "
            + "       p.paydate as IGNORE_COL, "
            + "       to_char(p.paydate, ''mm/dd/yyyy'') as COL "
            + "     , to_char(sum(plPIT.value_amt), ''FM$99,999,990.90'') as COL " 
            + "     , to_char(sum(plSDI.value_amt), ''FM$99,999,990.90'') as COL " 
            + "     , to_char(sum(plSUI.value_amt), ''FM$99,999,990.90'') as COL " 
            + "     , to_char(sum(plETT.value_amt), ''FM$99,999,990.90'') as COL"
            + "     , to_char(	sum(plPIT.value_amt) + sum(plSDI.value_amt) "
            + "                 + sum(plSUI.value_amt) + sum(plETT.value_amt), ''FM$99,999,990.90'') as COL "
            + "	from "
            + "     paychecklineitem plPIT, paychecklineitem plSDI, paychecklineitem plSUI, "
            + "     paychecklineitem plETT, paycheck p " 
            + " where "
            + "         plPIT.paycheck = p.id and plPIT.lawid_value = ''6'' "
            + "     and plSDI.paycheck = p.id and plSDI.lawid_value = ''67'' " 
            + "     and plSUI.paycheck = p.id and plSUI.lawid_value = ''87''"  
            + "     and plETT.paycheck = p.id and plETT.lawid_value = ''142'' "   
            + "     and p.paydate >= ''"  
);
	private static final String sCaTaxLiabilitiesReportQueryPerPaycheckPart3and5 = new String (
			" '' and p.paydate <= ''"
);
	
	private static final String sCaTaxLiabilitiesReportQueryPerPaycheckPart4 = new String (
			" '' "
            + " group by p.paydate "
            + ""
            + "UNION ALL " 
            + ""
            + " select 	"
            + "       to_date(''31-DEC-5000'') as IGNORE_COL, "
            + "       ''TOTAL''  as COL "
            + "     , to_char(nvl(sum(plPIT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL " 
            + "     , to_char(nvl(sum(plSDI.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL "
            + "     , to_char(nvl(sum(plSUI.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL "
            + "     , to_char(nvl(sum(plETT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL "
            + "     , to_char(nvl(	sum(plPIT.value_amt) + sum(plSDI.value_amt) + sum(plSUI.value_amt) "
            + "                     + sum(plETT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL "
            + " from "
            + "     paychecklineitem plPIT, paychecklineitem plSDI, paychecklineitem plSUI, "
            + "     paychecklineitem plETT, paycheck p "
            + " where "
            + "         plPIT.paycheck = p.id and plPIT.lawid_value = ''6'' " 
            + "     and plSDI.paycheck = p.id and plSDI.lawid_value = ''67'' "  
            + "     and plSUI.paycheck = p.id and plSUI.lawid_value = ''87'' "  
            + "     and plETT.paycheck = p.id and plETT.lawid_value = ''142'' "
            + "     and p.paydate >= ''"
);
	
	private static final String sCaTaxLiabilitiesReportQueryPerPaycheckPart6 = new String (
			" '' order by 1'"     
	);
	
	private static final String sCaTaxLiabilitiesReportQueryNoSubTotalsPart1 = new String (
			  "	'select DISTINCT to_date(''01-JAN-1700'') as IGNORE_COL,''"
	);
	private static final String sCaTaxLiabilitiesReportQueryNoSubTotalsPart2 = new String (		
			  "'' as COL, plPIT.description as COL , plSDI.description as COL, "
			+ "plSUI.description as COL, plETT.description as COL, "
			+ "''TOTAL DUE'' as COL from payrollitem plPIT, payrollitem plSDI, payrollitem plSUI, payrollitem plETT "
			+ "where plPIT.lawid_value = ''6'' and plSDI.lawid_value = ''67'' "
			+ "and plSUI.lawid_value = ''87'' and plETT.lawid_value = ''142'' "
			+ "UNION ALL "
			+ "select to_date(''31-DEC-5000'') as IGNORE_COL, "
			+ "''TOTAL''  as COL, to_char(nvl(sum(plPIT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(nvl(sum(plSDI.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(nvl(sum(plSUI.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(nvl(sum(plETT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(nvl(sum(plPIT.value_amt) + " 
			+ "sum(plSDI.value_amt) + "
			+ "sum(plSUI.value_amt)+ "
			+ "sum(plETT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL  "
			+ "from paychecklineitem plPIT, paychecklineitem plSDI, "
			+ "paychecklineitem plSUI, paychecklineitem plETT, "
			+ "paycheck p  where plPIT.paycheck = p.id and plPIT.lawid_value = ''6'' "
			+ "and plSDI.paycheck = p.id and plSDI.lawid_value = ''67'' "
			+ "and plSUI.paycheck = p.id and plSUI.lawid_value = ''87'' "
			+ "and plETT.paycheck = p.id and plETT.lawid_value = ''142'' "
			+ "and p.paydate >= ''"
	);
	
	private static final String sCaTaxLiabilitiesReportQueryNoSubTotalsPart3 = new String (
			"'' "
			+ "and p.paydate <= ''"
	);
	
	private static final String sCaTaxLiabilitiesReportQueryNoSubTotalsPart4 = new String (
			"'' order by 1' "
	);

	
	private static final String sCaTaxLiabilitiesReportQueryPerMonthPart1 = new String (
			"'select DISTINCT to_date(''01-JAN-1700'') as IGNORE_COL, ''"
			
	);
	
	private static final String sCaTaxLiabilitiesReportQueryPerMonthPart2 = new String (
			"'' as COL ,"
			+ "plPIT.description as COL, "
			+ "plSDI.description as COL, "
			+ "plSUI.description as COL, "
			+ "plETT.description as COL, "
			+ "''TOTAL DUE'' as COL "
			+ "from payrollitem plPIT, payrollitem plSDI, payrollitem plSUI, payrollitem plETT "
			+ "where plPIT.lawid_value = ''6'' "
			+ "and plSDI.lawid_value = ''67'' "
			+ "and plSUI.lawid_value = ''87'' "
			+ "and plETT.lawid_value = ''142'' "
			+ "union all "
			+ "select to_date(''01''||to_char(p.paydate, ''Mon yyyy'')) as IGNORE_COL, "
			+ "to_char(p.paydate, ''Mon yyyy'') as COL, "
			+ "to_char(SUM(plPIT.value_amt), ''FM$99,999,990.90'')as COL, "
			+ "to_char(SUM(plSDI.value_amt), ''FM$99,999,990.90'') as COL, "
			+ "TO_CHAR(SUM(plSUI.value_amt), ''FM$99,999,990.90'') as COL, "
			+ "TO_CHAR(SUM(plETT.value_amt), ''FM$99,999,990.90'') as COL, "
			+ "TO_CHAR( SUM(plPIT.value_amt) + SUM(plSDI.value_amt) + "
			+ "SUM(plSUI.value_amt) + SUM(plETT.value_amt), ''FM$99,999,990.90'') AS COL "
			+ "FROM paychecklineitem plPIT, paychecklineitem plSDI, paychecklineitem plSUI, paychecklineitem plETT, "
			+ "paycheck p "
			+ "WHERE plPIT.paycheck = p.id AND plPIT.lawid_value  = ''6'' "
			+ "AND plSDI.paycheck = p.id AND plSDI.lawid_value  = ''67'' "
			+ "AND plSUI.paycheck = p.id AND plSUI.lawid_value  = ''87'' "
			+ "AND plETT.paycheck = p.id AND plETT.lawid_value  = ''142'' "
			+ "AND p.paydate >= ''"
			);
	
	private static final String sCaTaxLiabilitiesReportQueryPerMonthPart3 = new String (
			"'' AND p.paydate <= ''"
			);
	
	private static final String sCaTaxLiabilitiesReportQueryPerMonthPart4 = new String (
			"'' group by to_char(p.paydate, ''Mon yyyy'') "
			+ "UNION ALL "
			+ "select to_date(''31-DEC-5000'') as IGNORE_COL, "
			+ "''TOTAL'' as COL, "
			+ "TO_CHAR(NVL(SUM(plPIT.value_amt), ''0.00''), ''FM$99,999,990.90'') AS COL, "
			+ "TO_CHAR(NVL(SUM(plSDI.value_amt), ''0.00''), ''FM$99,999,990.90'') AS COL, "
			+ "TO_CHAR(NVL(SUM(plSUI.value_amt), ''0.00''), ''FM$99,999,990.90'') AS COL, "
			+ "TO_CHAR(NVL(SUM(plETT.value_amt), ''0.00''), ''FM$99,999,990.90'') AS COL, "
			+ "TO_CHAR(NVL( SUM(plPIT.value_amt) + SUM(plSDI.value_amt) + "
			+ "SUM(plSUI.value_amt) + SUM(plETT.value_amt), ''0.00''), ''FM$99,999,990.90'') AS COL "
			+ "FROM paychecklineitem plPIT, paychecklineitem plSDI, paychecklineitem plSUI,  " 
			+ "paychecklineitem plETT, "
			+ "paycheck p "
			+ "WHERE plPIT.paycheck = p.id "
			+ "AND plPIT.lawid_value  = ''6'' "
			+ "AND plSDI.paycheck     = p.id "
			+ "AND plSDI.lawid_value  = ''67'' "
			+ "AND plSUI.paycheck     = p.id "
			+ "AND plSUI.lawid_value  = ''87'' "
			+ "AND plETT.paycheck     = p.id "
			+ "AND plETT.lawid_value  = ''142'' "
			+ "AND p.paydate >= ''"
	);
	
	private static final String sCaTaxLiabilitiesReportQueryPerMonthPart5 = new String (
			"'' AND p.paydate <= ''"
	);
	
	private static final String sCaTaxLiabilitiesReportQueryPerMonthPart6 = new String (
			"'' order by 1' "	
	);
	
	private static final String sCaTaxLiabilitiesReportQueryPerQuarterPart1 = new String (
			"'select DISTINCT to_char(to_number(''0000'')) as IGNORE_COL, ''"
	);
	
	private static final String sCaTaxLiabilitiesReportQueryPerQuarterPart2 = new String (
			"'' as COL ,"
			+ "plPIT.description AS COL , "
			+ "plSDI.description AS COL , "
			+ "plSUI.description AS COL , "
			+ "plETT.description AS COL , "
			+ "''TOTAL DUE'' AS COL "
			+ "FROM payrollitem plPIT, "
			+ "payrollitem plSDI, "
			+ "payrollitem plSUI, "
			+ "payrollitem plETT "
			+ "WHERE plPIT.lawid_value = ''6'' "
			+ "AND plSDI.lawid_value     = ''67'' "
			+ "AND plSUI.lawid_value     = ''87'' "
			+ "AND plETT.lawid_value     = ''142'' "
			+ "UNION ALL "
			+ "SELECT TO_CHAR(p.paydate, ''yyyy'') || TO_CHAR(p.paydate, ''Q'') AS IGNORE_COL ,"			
			+ "''Q''|| TO_CHAR(p.paydate, ''Q'')  || '' '' || TO_CHAR(p.paydate, ''yyyy'') AS COL ,"
			+ "TO_CHAR(SUM(plPIT.value_amt), ''FM$99,999,990.90'') AS COL, "
			+ "TO_CHAR(SUM(plSDI.value_amt), ''FM$99,999,990.90'') AS COL, "
			+ "TO_CHAR(SUM(plSUI.value_amt), ''FM$99,999,990.90'') AS COL, "
			+ "TO_CHAR(SUM(plETT.value_amt), ''FM$99,999,990.90'') AS COL , "
			+ "TO_CHAR( SUM(plPIT.value_amt) + SUM(plSDI.value_amt) + SUM(plSUI.value_amt) " 
			+ "+ SUM(plETT.value_amt), ''FM$99,999,990.90'') AS COL "
			+ "FROM "
			+ "paychecklineitem plPIT, "
			+ "paychecklineitem plSDI, "
			+ "paychecklineitem plSUI, "
			+ "paychecklineitem plETT, "
			+ "paycheck p "
			+ "WHERE "
			+ "plPIT.paycheck = p.id "
			+ "AND plPIT.lawid_value  = ''6'' "
			+ "AND plSDI.paycheck     = p.id "
			+ "AND plSDI.lawid_value  = ''67'' "
			+ "AND plSUI.paycheck     = p.id "
			+ "AND plSUI.lawid_value  = ''87'' "
			+ "AND plETT.paycheck     = p.id "
			+ "AND plETT.lawid_value  = ''142'' "
			+ "AND p.paydate >= ''"
	);
	
	private static final String sCaTaxLiabilitiesReportQueryPerQuarterPart3 = new String (
			"'' and p.paydate <= ''"
	);
	
	private static final String sCaTaxLiabilitiesReportQueryPerQuarterPart4 = new String (
			"'' group by TO_CHAR(p.paydate, ''yyyy''), TO_CHAR(p.paydate, ''Q'') "
			+ ""
			+ "union all "
			+ ""
			+ "select to_char(to_number(''55555'')) AS IGNORE_COL, "
			+ "''TOTAL'' as COL , "
			+ "to_char(NVL(sum(plPIT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(NVL(sum(plSDI.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(NVL(sum(plSUI.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(NVL(sum(plETT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(NVL(sum(plPIT.value_amt) + sum(plSDI.value_amt) + sum(plSUI.value_amt) " 
			+ "+ sum(plETT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL "
			+ "from paychecklineitem plPIT, "
			+ "paychecklineitem plSDI, "
			+ "paychecklineitem plSUI, "
			+ "paychecklineitem plETT, "
			+ "paycheck p "
			+ "where plPIT.paycheck = p.id "
			+ "and plPIT.lawid_value  = ''6'' "
			+ "and plSDI.paycheck     = p.id "
			+ "and plSDI.lawid_value  = ''67'' "
			+ "and plSUI.paycheck     = p.id "
			+ "and plSUI.lawid_value  = ''87'' "
			+ "and plETT.paycheck     = p.id "
			+ "and plETT.lawid_value  = ''142'' "
			+ "and p.paydate  >= ''"
	);
	
	private static final String sCaTaxLiabilitiesReportQueryPerQuarterPart5 = new String (
			"'' and p.paydate  <= ''"
	);
	
	private static final String sCaTaxLiabilitiesReportQueryPerQuarterPart6 = new String (
			"''order by 1' "
	);

//	select DISTINCT 
//	to_date('01-JAN-1700') as IGNORE_COL,
//	'REPORT_TITLE' AS COL, 
//	(select DISTINCT description from payrollitem Hourly where lawid_value = '1000012') as COL, 
//	(select DISTINCT description from payrollitem Salary where lawid_value = '1000013') as COL, 
//	(select DISTINCT description from payrollitem Piecework where lawid_value = '1000015') as COL, 
//	(select DISTINCT description from payrollitem Bonus where lawid_value = '1000005') as COL, 
//	'Total gross pay' as COL,
//	'Federal Wages, Tips, Other Compensation' as COL,
//	'Social Security Wages' as COL,
//	'Social Security Tips' as COL,
//	'Medicare Wages and Tips' as COL,
//	'CA Total Subject Wages' as COL,
//	'CA Personal Income Tax (PIT) Wages' as COL,
//	'Regular' as COL,
//	'Overtime' as COL,
//	'Piecework' as COL,
//	'Total hours' as COL,
//	'Health Insurance' as COL,
//	'Total Employee-Paid Benefits' as COL,
//	(select DISTINCT description from payrollitem FederalIncomeTax where lawid_value = '1') as COL, 
//	(select DISTINCT description from payrollitem EeSocialSecurity where lawid_value = '61') as COL, 
//	(select DISTINCT description from payrollitem EeMedicare where lawid_value = '63') as COL, 
//	(select DISTINCT description from payrollitem CaPIT where lawid_value = '6') as COL, 
//	(select DISTINCT description from payrollitem CaSDI where lawid_value = '67') as COL, 
//	'Total Employee-Paid Taxes' as COL,
//	'TOTAL DUE' as COL 
//	
//	from
//		dual

	private static final String sUsPayrollReportQueryNoSubTotalsPart1 = new String (
			  "	'select DISTINCT to_date(''01-JAN-1700'') as IGNORE_COL,''"
	);
	private static final String sUsPayrollReportQueryNoSubTotalsPart2 = new String (		
			  "'' as COL, plPIT.description as COL , plSDI.description as COL, "
			+ "plSUI.description as COL, plETT.description as COL, "
			+ "''TOTAL DUE'' as COL from payrollitem plPIT, payrollitem plSDI, payrollitem plSUI, payrollitem plETT "
			+ "where plPIT.lawid_value = ''6'' and plSDI.lawid_value = ''67'' "
			+ "and plSUI.lawid_value = ''87'' and plETT.lawid_value = ''142'' "
			+ "UNION ALL "
			+ "select to_date(''31-DEC-5000'') as IGNORE_COL, "
			+ "''TOTAL''  as COL, to_char(nvl(sum(plPIT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(nvl(sum(plSDI.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(nvl(sum(plSUI.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(nvl(sum(plETT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL, "
			+ "to_char(nvl(sum(plPIT.value_amt) + " 
			+ "sum(plSDI.value_amt) + "
			+ "sum(plSUI.value_amt)+ "
			+ "sum(plETT.value_amt), ''0.00''), ''FM$99,999,990.90'') as COL  "
			+ "from paychecklineitem plPIT, paychecklineitem plSDI, "
			+ "paychecklineitem plSUI, paychecklineitem plETT, "
			+ "paycheck p  where plPIT.paycheck = p.id and plPIT.lawid_value = ''6'' "
			+ "and plSDI.paycheck = p.id and plSDI.lawid_value = ''67'' "
			+ "and plSUI.paycheck = p.id and plSUI.lawid_value = ''87'' "
			+ "and plETT.paycheck = p.id and plETT.lawid_value = ''142'' "
			+ "and p.paydate >= ''"
	);
	
	private static final String sUsPayrollReportQueryNoSubTotalsPart3 = new String (
			"'' "
			+ "and p.paydate <= ''"
	);
	
	private static final String sUsPayrollReportQueryNoSubTotalsPart4 = new String (
			"'' order by 1' "
	);

	private static final String sSQL92Test = new String (	
			"select 1 " 
			+	    "from dual d1 " 
			+	    "cross join dual d2 " 
			+	    "inner join dual d3 ON 1=1 "
			+	    "left outer join dual d4 on 2=2 "
			+	    "right outer join dual d5 on 0=0"
	);
	
	@SuppressWarnings("deprecation")
	public ReportResponse getReport(ReportRequest reportRequest)
		throws ServiceFailureException, ValidationException {
		
		if (reportRequest == null) {
			throw new ValidationException("Report request has not been set.");
			
		}
		
		if (reportRequest.getFromDate() == null || reportRequest.getToDate() == null) {
			throw new ValidationException("The start date or end date have not been set.");
			
		}
		if (reportRequest.getFromDate().after(reportRequest.getToDate())) {
			String mesg = MessageFormat.format("Report start date ({0}) cannot be after report end date ({1}).",
					new Object[]{reportRequest.getFromDate(),reportRequest.getToDate()});
			throw new ValidationException(mesg);
		}
		
		ReportResponse response = new ReportResponse();
		if (reportRequest.getReportDetail().ordinal() == ReportDetail.NOT_SET.ordinal()) {
			response.setXmlData(sCaTaxLiabilitiesSampleResult);
			return response;
		}
		
		CallableStatement cstmt = null;  
	    Exception exception = null;
		try {
			TakehomeReportsEAO reportsEAO = ServiceLocator.getInstance().getService(TakehomeReportsEAO.JNDI_NAME);
			EntityManager em = reportsEAO.getEntityManager();
			Session session = (Session)em.getDelegate();
			Connection conn = (Connection)session.connection();
		    	    	
	    	java.sql.Clob xmlClob;
	    	
	    	String queryString = getReportQuery(reportRequest);
	    	
	    	if (queryString == null) {
	    		ServiceFailureException sfException = new ServiceFailureException("Sub total not implemented as yet!");
	    		sLogger.error(sfException.getMessage());
	    		throw new ServiceFailureException(sfException);
	    	}
	    	
			cstmt  = conn.prepareCall(queryString);
	    	                       
			cstmt.registerOutParameter(1, java.sql.Types.CLOB); 
			@SuppressWarnings("unused")
	        boolean ret = cstmt.execute();
	            
	        xmlClob = cstmt.getClob(1);
            if (null != xmlClob) {
	            Integer len = (int)xmlClob.length();
	            String xmlString = xmlClob.getSubString(1, len);
	            response.setXmlData(xmlString);
            }
	    }
	    catch (SQLException e) {
			sLogger.error(MessageFormat.format("SQL error occured while retrieving Report: {0}",
												new Object[]{ExceptionUtils.throwableMessageChain(e)}));
			exception = e;
	    }
	    catch (Exception e) {
			sLogger.error(MessageFormat.format("Error occured while retrieving Report: {0}",
												new Object[]{ExceptionUtils.throwableMessageChain(e)}));
			exception = e;
	    }
	    finally {
	        if (cstmt != null) {
	            try {
	            	cstmt.close();
	            }
	    	    catch (SQLException e) {
	    			sLogger.error(MessageFormat.format("SQL error occured while retrieving Report {0}",
	    												new Object[]{ExceptionUtils.throwableMessageChain(e)}));
	    			if (exception == null) exception = e; // don't overwrite a first failure
	    	    }
	        }
    	    if (exception != null) {
    			throw new ServiceFailureException(exception);
    	    }
	    }
								
		return response;
	}
	
	private String getReportQuery(ReportRequest reportRequest)
	{
		String queryString = null;
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd-MMM-yyyy");
    	
		
		if (reportRequest.getReportType().equals(ReportType.CA_TAX_LIABILITIES)) {
			if (reportRequest.getReportSubtotals().equals(ReportSubtotals.NOT_SET)) {
				
				queryString = "{ ? = call dbms_xmlgen.getxml("
					+ sCaTaxLiabilitiesReportQueryNoSubTotalsPart1
					+ reportRequest.getReportTitle()
					+ sCaTaxLiabilitiesReportQueryNoSubTotalsPart2
					+ simpleDateFormat.format(reportRequest.getFromDate())
					+ sCaTaxLiabilitiesReportQueryNoSubTotalsPart3
					+ simpleDateFormat.format(reportRequest.getToDate())
					+ sCaTaxLiabilitiesReportQueryNoSubTotalsPart4
					+ ")}";
				
			}
			else if (reportRequest.getReportSubtotals().equals(ReportSubtotals.PER_PAYCHECK)) {
				
		    	queryString = "{ ? = call dbms_xmlgen.getxml("
		    	+ sCaTaxLiabilitiesReportQueryPerPaycheckPart1
		    	+ reportRequest.getReportTitle()
		    	+ sCaTaxLiabilitiesReportQueryPerPaycheckPart2
		    	+ simpleDateFormat.format(reportRequest.getFromDate())
		    	+ sCaTaxLiabilitiesReportQueryPerPaycheckPart3and5
		    	+ simpleDateFormat.format(reportRequest.getToDate())
		    	+ sCaTaxLiabilitiesReportQueryPerPaycheckPart4
		    	+ simpleDateFormat.format(reportRequest.getFromDate())
		    	+ sCaTaxLiabilitiesReportQueryPerPaycheckPart3and5
		    	+ simpleDateFormat.format(reportRequest.getToDate())
		    	+ sCaTaxLiabilitiesReportQueryPerPaycheckPart6
		    	+ ")}";
		    	
			}
			
			else if (reportRequest.getReportSubtotals().equals(ReportSubtotals.PER_MONTH)) {
				queryString = "{ ? = call dbms_xmlgen.getxml("
			    	+ sCaTaxLiabilitiesReportQueryPerMonthPart1
			    	+ reportRequest.getReportTitle()
			    	+ sCaTaxLiabilitiesReportQueryPerMonthPart2
			    	+ simpleDateFormat.format(reportRequest.getFromDate())
			    	+ sCaTaxLiabilitiesReportQueryPerMonthPart3
			    	+ simpleDateFormat.format(reportRequest.getToDate())
			    	+ sCaTaxLiabilitiesReportQueryPerMonthPart4
			    	+ simpleDateFormat.format(reportRequest.getFromDate())
			    	+ sCaTaxLiabilitiesReportQueryPerMonthPart5
			    	+ simpleDateFormat.format(reportRequest.getToDate())
			    	+ sCaTaxLiabilitiesReportQueryPerMonthPart6
			    	+ ")}";
			    	
			}
			
			else if (reportRequest.getReportSubtotals().equals(ReportSubtotals.PER_QUARTER)) {
				queryString = "{ ? = call dbms_xmlgen.getxml("
			    	+ sCaTaxLiabilitiesReportQueryPerQuarterPart1
			    	+ reportRequest.getReportTitle()
			    	+ sCaTaxLiabilitiesReportQueryPerQuarterPart2
			    	+ simpleDateFormat.format(reportRequest.getFromDate())
			    	+ sCaTaxLiabilitiesReportQueryPerQuarterPart3
			    	+ simpleDateFormat.format(reportRequest.getToDate())
			    	+ sCaTaxLiabilitiesReportQueryPerQuarterPart4
			    	+ simpleDateFormat.format(reportRequest.getFromDate())
			    	+ sCaTaxLiabilitiesReportQueryPerQuarterPart5
			    	+ simpleDateFormat.format(reportRequest.getToDate())
			    	+ sCaTaxLiabilitiesReportQueryPerQuarterPart6
			    	+ ")}";
			}
		} else if (reportRequest.getReportType().equals(ReportType.US_PAYROLLREPORT)) {
			if (reportRequest.getReportSubtotals().equals(ReportSubtotals.NOT_SET)) {
				
				queryString = "{ ? = call dbms_xmlgen.getxml("
					+ sCaTaxLiabilitiesReportQueryNoSubTotalsPart1
					+ reportRequest.getReportTitle()
					+ sCaTaxLiabilitiesReportQueryNoSubTotalsPart2
					+ simpleDateFormat.format(reportRequest.getFromDate())
					+ sCaTaxLiabilitiesReportQueryNoSubTotalsPart3
					+ simpleDateFormat.format(reportRequest.getToDate())
					+ sCaTaxLiabilitiesReportQueryNoSubTotalsPart4
					+ ")}";
				
			}

		} else if (reportRequest.getReportType().equals(ReportType.SQL92TEST)) {

			queryString = "{ ? = call dbms_xmlgen.getxml("
				+ "'"
				+ sSQL92Test
				+ "'"
				+ ")}";

		} else if (reportRequest.getReportType().equals(ReportType.NOTSET)) {
			// exception?
		} else {
			// exception?
		}
		
		if (queryString != null) sLogger.info(queryString);
		return queryString;
	}

}
