package com.intuit.sb.payroll.services.common.impl;

import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Mapper class to handle WorkLocation Mappings between the DTO/Service layer and the Entity Model.
 *
 * $Author: $ 
 * $Revision: $
 * $DateTime: $
 */
public class WorkLocationMapper {
	
	/**
	 * Maps from an Entity WorkLocation to a DTO WorkLocation.  Null input yields null output.
	 * @param workLocation
	 * @return
	 */
	public static WorkLocation mapWorkLocation(com.intuit.sb.payroll.entity.model.WorkLocation workLocation) {
		if (workLocation != null) {
			WorkLocation result = new WorkLocation();
			result.setName(workLocation.getName());
			result.setIsDefault(workLocation.isDefaultt());
			result.setObjectHandle(new ObjectHandle(workLocation.getId(), workLocation.getVersion()));
			
			GlobalPostalAddress taxAddressEntity = workLocation.getTaxResidenceAddress();
			USPostalAddress entityAddress = null;
			if(taxAddressEntity != null)
			{
				entityAddress = AddressConvertor.getUSFromGlobal(workLocation.getTaxResidenceAddress(), 
						new USPostalAddressImpl());				
			}
			PhysicalAddress address = PhysicalAddressMapper.mapAddress(entityAddress);
			result.setAddress(address);
			TelephoneNumber mainPhone = workLocation.getMainPhone();
			if (mainPhone != null) {
				result.setPhoneNumber(mainPhone.getFreeFormNumber());
				result.setExtn(workLocation.getMainPhone().getExtension());
			}
			result.setWorkerCount(workLocation.getWorkerCount());
			return result;
		} else {
			return null;
		}
	}

	/**
	 * Maps from a DTO WorkLocation to an Entity WorkLocation.  Null input = null output.
	 * 
	 * WorkerCount is NOT mapped - it's read only from the DTOs POV.
	 * 
	 * @param dto
	 * @return
	 * @throws ServiceFailureException 
	 */
	public static com.intuit.sb.payroll.entity.model.WorkLocation mapWorkLocation(WorkLocation dto) throws ServiceFailureException {
		if (dto != null) {
			EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
			ObjectHandle workLocationId = dto.getObjectHandle();
			com.intuit.sb.payroll.entity.model.WorkLocation entity = null;
			if (null == workLocationId) {
				entity = new com.intuit.sb.payroll.entity.model.WorkLocation();
			} else {
				entity = employerComponent.findWorkLocation(workLocationId.getId());
			}
			
			entity.setName(dto.getName());
			entity.setDefaultt(dto.getIsDefault());
			
			if (null == entity.getMainPhone()) {
				entity.setMainPhone(new TelephoneNumber());
			}
			entity.getMainPhone().setFreeFormNumber(dto.getPhoneNumber());
			entity.getMainPhone().setExtension(dto.getExtn());
			
			//Nothing is initialized, so checking for nulls
			if (dto.getAddress() != null) {
				//We have an Address from DTO
				if (null == entity.getTaxResidenceAddress()) {
					entity.setTaxResidenceAddress(new GlobalPostalAddress());
				}
				GlobalPostalAddress entityAddress = entity.getTaxResidenceAddress();
				USPostalAddress usAddress = AddressConvertor.getUSFromGlobal(
						entityAddress, new USPostalAddressImpl());
				//Send the US Address for Mapping
				PhysicalAddressMapper.mapAddress(dto.getAddress(), usAddress);
				//Now we have the mapped US Address so put it back into the entity
				entityAddress = AddressConvertor.getGlobalFromUS(usAddress, entityAddress);
			} else {
				entity.setTaxResidenceAddress(null);
			}
			
			// do NOT map workerCount - it's managed internally and the API user cannot set it
			
			return entity;
		} else {
			return null;
		}
	}
}
