package com.intuit.sb.payroll.services.employer.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.services.employer.CompanyLegalTaxFilingInfo;


/**
 * Class implementing the CompanyLegalTaxFilingInfo DTO interface. 
 */
public class CompanyLegalTaxFilingInfoImpl {
	/**
	 * Retrieves the getCompanyInfo from the current employer entity. 
	 * If the employer entity cannot be found, a ServiceFailureException is thrown 
	 * and the CompanyLegalTaxFilingInfo is not retrieved.
	 * 
	 * @param 
	 * @return CompanyLegalTaxFilingInfo
	 * @throws ServiceFailureException
	 */
	public static CompanyLegalTaxFilingInfo getCompanyInfo() throws ServiceFailureException
	{
			EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
			com.intuit.sb.payroll.entity.model.Employer employer = employerComponent.getEmployer();
	
			//Map from the Model info to the DTO
			CompanyLegalTaxFilingInfo cInfo = CompanyLegalTaxFilingInfoMapper.mapCompanyInfo(employer);
				
			//return to client.
			return cInfo;
	}
	
	/**
	 * Saves the CompanyLegalTaxFilingInfo passed via the DTO. If the employer entity cannot be saved, 
	 * a ValidationException exception is thrown and the CompanyLegalTaxFilingInfo is not saved.
	 * 
	 * @param CompanyLegalTaxFilingInfo
	 * @return
	 * @throws ServiceFailureException, ValidationException
	 */
	public static void saveCompanyInfo(CompanyLegalTaxFilingInfo cInfo) throws ServiceFailureException, ValidationException
	{
		validateCompanyInfo(cInfo);
		Employer employer = CompanyLegalTaxFilingInfoMapper.mapCompanyInfo(cInfo);
					
		EmployerComponent employerPTSComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
		employer = employerPTSComponent.saveEmployer(employer);
	}
	
	/**
	 * Validates the CompanyLegalTaxFilingInfo passed via the DTO. If validation fails,
	 * PayrollValidationException is thrown.
	 * 
	 * @param CompanyLegalTaxFilingInfo
	 * @return
	 * @throws PayrollValidationException
	 */
	private static void validateCompanyInfo(CompanyLegalTaxFilingInfo cInfo) throws ValidationException {
		if (null == cInfo) {
			throw new ValidationException("CompanyLegalTaxFilingInfo cannot be null.");
		}
		
		if (null == cInfo.getLegalAddress()) {
			throw new ValidationException("CompanyLegalTaxFilingInfo's Legal Address cannot be null.");
		}
		
		if (LegalStructureEnum.NOTSET == cInfo.getCompanyLegalType()) {
			throw new ValidationException("CompanyLegalTaxFilingInfo's Legal Type not set.");
		}
		
		if (IndustryTypeEnum.NOTSET == cInfo.getIndustryType()) {
			throw new ValidationException("CompanyLegalTaxFilingInfo's Industry Type not set.");
		}
		
		// add validations here		
	}
	
}
