package com.intuit.sb.payroll.services.employer.impl;

import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.services.employer.CompanyType;

/**
 * Mapper class to handle CompanyType Mappings between the DTO/Service layer and
 * the Entity Model.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
public class CompanyTypeMapper {

	/**
	 * Maps from a DTO CompanyType to an Entity LegalStructureEnum. Null input = null output.
	 * 
	 * @param CompanyType
	 * @return
	 */
	public static com.intuit.sb.payroll.entity.model.LegalStructureEnum mapCompanyType(CompanyType type) {

		LegalStructureEnum legalType;

		if (type == null) {
			return null;
		}

		if (type.equals(CompanyType.CORPORATION)) {
			legalType = LegalStructureEnum.CCORP;
		} else if (type.equals(CompanyType.LLC)) {
			legalType = LegalStructureEnum.LLC;
		} else if (type.equals(CompanyType.NON_PROFIT)) {
			legalType = LegalStructureEnum.STATUS501C3;
		} else if (type.equals(CompanyType.PARTNERSHIP)) {
			legalType = LegalStructureEnum.PARTNERSHIP;
		} else if (type.equals(CompanyType.SOLE_PROPRIETORSHIP)) {
			legalType = LegalStructureEnum.SOLEPROPRIETOR;
		} else {
			legalType = LegalStructureEnum.NOTSET;
		}

		return legalType;
	}

	/**
	 * Maps from an Entity LegalStructureEnum to a DTO CompanyType. Null input = null output.
	 * 
	 * @param LegalStructureEnum
	 * @return
	 */
	public static CompanyType mapCompanyType(com.intuit.sb.payroll.entity.model.LegalStructureEnum type) {

		CompanyType companyType;

		if (type == null) {
			return null;
		}

		if (type.equals(LegalStructureEnum.CCORP)) {
			companyType = CompanyType.CORPORATION;
		} else if (type.equals(LegalStructureEnum.LLC)) {
			companyType = CompanyType.LLC;
		} else if (type.equals(LegalStructureEnum.STATUS501C3)) {
			companyType = CompanyType.NON_PROFIT;
		} else if (type.equals(LegalStructureEnum.PARTNERSHIP)) {
			companyType = CompanyType.PARTNERSHIP;
		} else if (type.equals(LegalStructureEnum.SOLEPROPRIETOR)) {
			companyType = CompanyType.SOLE_PROPRIETORSHIP;
		} else {
			companyType = null;
		}

		return companyType;
	}
}
