package com.intuit.sb.payroll.services.employer.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.config.GeminiConfigurationUtility;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.TaxServiceStatusEnum;
import com.intuit.sb.payroll.services.employer.AssistedStateType;

public class EmployerAssistedImpl {

	public static void updateAssistedState(AssistedStateType assistedStateType) throws ValidationException, ServiceFailureException {
		if (assistedStateType == null){
			throw new ValidationException("Null AssistedStateType provided");
		}
		EmployerPTSWorkflowComponent employerPTSComponent = 
								ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
		
		TaxServiceStatusEnum taxServiceStatus = 
				AssistedStateTypeMapper.mapAssistedStateType(assistedStateType);
		employerPTSComponent.updateEmployerTaxServiceState(taxServiceStatus);
	}

	public static AssistedStateType getAssistedState() throws ServiceFailureException {
		if (!GeminiConfigurationUtility.isTaxPaymentsEnabled()){
			return AssistedStateType.ASSISTED_NOT_SUPPORTED;
		}
		EmployerPTSWorkflowComponent employerPTSComponent = 
			ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
		PTSEmployer ptsEmployer = employerPTSComponent.getPTSEmployer();
		// This method can be called from UI before an employer gets created. 
		if (ptsEmployer == null){
			return AssistedStateType.NOT_ASSISTED;
		}
		TaxServiceStatusEnum taxServiceStatus = ptsEmployer.getCurrentTaxServiceStatus();
		return AssistedStateTypeMapper.mapEmployerAssistedStateEnum(taxServiceStatus);
	}

	public static void sendCompanyInfoToPTS() throws ServiceFailureException, ValidationException {
		EmployerPTSWorkflowComponent employerPTSComponent = 
			ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
		employerPTSComponent.updateEmployerTaxServiceState(TaxServiceStatusEnum.READY_FOR_ADD_COMPANY);
	}
	
	/**
	 * check if user is allowed to submit data to PTS
	 * @return
	 */
	public static boolean allowDataSubmissionToPTS() {
		EmployerPTSWorkflowComponent employerPTSComponent = 
			ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
		return employerPTSComponent.allowDataSubmissionToPTS();
	}
}
