package com.intuit.sb.payroll.services.employer.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.services.common.BankAccount;

public class EmployerBankImpl {

	/**
	 * Persist the bank accounts - do necessary DTO <--> Model mapping before and after.
	 * @param bankAccounts: new DTO BankAccount objects to persist
	 * @return DTO BankAccount objects persisted
	 * @throws ServiceFailureException
	 * @throws ValidationException
	 */
	public static BankAccount[] addBankAccounts(BankAccount[] bankAccounts) 
			throws ServiceFailureException, ValidationException {

		com.intuit.sb.payroll.entity.model.BankAccount[] accounts = 
			new com.intuit.sb.payroll.entity.model.BankAccount[bankAccounts.length];
		BankAccount[] newAccounts = new BankAccount[bankAccounts.length];
		
		for (int i = 0; i < bankAccounts.length; i++) {
			accounts[i] = BankAccountMapper.mapDtoBankAccountToModel(bankAccounts[i]);
		}

		EmployerComponent employerComponent = 
			ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
		
		accounts = employerComponent.addBankAccounts(accounts);

		for (int i = 0; i < accounts.length; i++) {
			newAccounts[i] = BankAccountMapper.mapModelBankAccountToDto(accounts[i]);
		}
		
		return newAccounts;
	}

	/**
	 * Get BankAccounts for this Organization
	 * @return DTO BankAccount objects 
	 * @throws ServiceFailureException
	 * @throws ValidationException
	 */
	public static BankAccount[] getBankAccounts() 
			throws ServiceFailureException, ValidationException {

		EmployerComponent employerComponent = 
			ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
		
		com.intuit.sb.payroll.entity.model.BankAccount[] accounts = employerComponent.getBankAccounts();
		BankAccount[] existingAccounts = new BankAccount[accounts.length];
		
		for (int i = 0; i < accounts.length; i++) {
			existingAccounts[i] = BankAccountMapper.mapModelBankAccountToDto(accounts[i]);
		}
		
		return existingAccounts;
	}

	/**
	 * Persist the bank accounts - do necessary DTO <--> Model mapping before and after.
	 * @param bankAccounts: DTO BankAccount objects to persist
	 * @return DTO BankAccount objects persisted
	 * @throws ServiceFailureException
	 * @throws ValidationException
	 */
	public static BankAccount[] updateBankAccounts(BankAccount[] bankAccounts) 
			throws ServiceFailureException, ValidationException {

		com.intuit.sb.payroll.entity.model.BankAccount[] accounts = 
			new com.intuit.sb.payroll.entity.model.BankAccount[bankAccounts.length];
		BankAccount[] newAccounts = new BankAccount[bankAccounts.length];
		
		for (int i = 0; i < bankAccounts.length; i++) {
			accounts[i] = BankAccountMapper.mapDtoBankAccountToModel(bankAccounts[i]);
		}

		EmployerComponent employerComponent = 
			ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
		
		accounts = employerComponent.updateBankAccounts(accounts);

		for (int i = 0; i < accounts.length; i++) {
			newAccounts[i] = BankAccountMapper.mapModelBankAccountToDto(accounts[i]);
		}
		
		return newAccounts;
	}
}
