package com.intuit.sb.payroll.services.pto;

import javax.ejb.Remote;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;

/**
 * TODO: This should move up to service adapter layer - above composite.
 */
@Remote
public interface PTOService {
	
	public static final String JNDI_NAME = "PTOService/remote";
	
	/**
	 * Gets all PTO for the worker
	 * @param workerId The ID of the worker.
	 * @return a list of all PTOInfo
	 * @throws ServiceFailureException
	 * @throws ValidationException
	 */
	public PTOInfo[] getAllPTOInfo(Long workerId) throws ServiceFailureException, ValidationException;
	
	/**
	 * Gets a single named and typed PTO for a worker.
	 * 
	 * @param workerId The ID of the worker.
	 * @param ptoName The string name of the PTO e.g. "floating holiday".
	 * @param ptoType The type of the PTO.
	 * @return The requested PTO or null if it doesn't existing on the specified worker.
	 * @throws ServiceFailureException Thrown if a request is made for a PTO on a nonexistent worker.
	 * @throws ValidationException If workerId or ptoName are null or empty.
	 */
	public PTOInfo getPTOByNameAndType(Long workerId, String ptoName, PTOType ptoType) throws ServiceFailureException, ValidationException;

	/**
	 * Saves a PTO on an existing worker. If the PTO does not exist it is created, otherwise it is updated.
	 * 
	 * @param savePTORequest The PTO to save.
	 * @throws ServiceFailureException Thrown if a request is made to save a PTO on a nonexistent worker.
	 * @throws ValidationException If workerId or ptoName are null or empty.
	 * @return SavePTOResponse containing updated worker and all PTOInfos for the worker
	 */
	public SavePTOResponse savePTO(PTOInfo ptoInfo) throws WorkflowException, ServiceFailureException, ValidationException;
}
