package com.intuit.sb.payroll.services.pto.impl;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType1Enum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualStartTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffDtl;
import com.intuit.sb.payroll.entity.model.PaidTimeOffTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollItemDtlPtdDatum;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.model.PtoBaseRateTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.common.impl.ObjectHandleMapper;
import com.intuit.sb.payroll.services.pto.AccrualStartType;
import com.intuit.sb.payroll.services.pto.PTOAccrualType;
import com.intuit.sb.payroll.services.pto.PTOInfo;
import com.intuit.sb.payroll.services.pto.PTOType;

public class PTOMapper {

	private static final Logger logger = PayrollLoggerFactory.makeLogger(PTOMapper.class);
	private static PaidTimeOffComponent ptoComponent = ServiceLocator.getInstance().getService(
			PaidTimeOffComponent.JNDI_NAME);
	private static WorkerComponent workerComponent = ServiceLocator.getInstance().getService(WorkerComponent.JNDI_NAME);

	public static PaidTimeOffDtl mapToPaidTimeOffDtl(PTOInfo ptoInfo) throws ValidationException,
		ServiceFailureException {

		Worker worker = workerComponent.findWorker(ptoInfo.getWorkerId());
		return mapToPaidTimeOffDtl(worker, ptoInfo);
	}
		
	public static PaidTimeOffDtl mapToPaidTimeOffDtl(Worker worker, PTOInfo ptoInfo) throws ValidationException,
			ServiceFailureException {

		PaidTimeOffTypeEnum paidTimeOffType = PTOMapper.mapToPaidTimeOffTypeEnum(worker, ptoInfo.getPtoType());
		PaidTimeOffDtl paidTimeOffDtl;
		if (ptoInfo.getObjectHandle() == null) {
			paidTimeOffDtl = ptoComponent.createPaidTimeOffDtl(paidTimeOffType);
		} else {
			Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(ptoInfo.getObjectHandle());
			paidTimeOffDtl = ptoComponent.findPaidTimeOffDtl(identifiable);
		}

		paidTimeOffDtl.setAccrualStartDate(ptoInfo.getAccrualStartDate());
		paidTimeOffDtl.setAccrualStartType(mapToAccrualStartTypeEnum(ptoInfo.getAccrualStartType()));
		PaidTimeOffAccrualTypeEnum paidTimeOffAccrualTypeEnum = mapToAccrualTypeEnum(ptoInfo.getPtoAccrualType());
		paidTimeOffDtl.setAccrualType(paidTimeOffAccrualTypeEnum);
		paidTimeOffDtl.setAccruedHours(ptoInfo.getAccruedHours());
		paidTimeOffDtl.setAllowCarryOver(ptoInfo.isAllowCarryOver());
		paidTimeOffDtl.setDescription(ptoInfo.getName());
		paidTimeOffDtl.setLimitAccrual(ptoInfo.isLimitAccrual());
		paidTimeOffDtl.setMaximumAccrual(ptoInfo.getMaximumAccrual());
		paidTimeOffDtl.setPaidTimeOffType(paidTimeOffType);
		// Datum for accrual rate
		Datum rate = new Datum();
		rate.setValue1AsQuantity(ptoInfo.getAccrualRate());
		rate.setType2Value2(mapToDatumType2Enum(paidTimeOffAccrualTypeEnum), new Double(1));
		paidTimeOffDtl.setRate(rate);
		// PtdDatum for disbursed hours year-to-date
		addOrUpdatePtdDatum(paidTimeOffDtl, PtdTypeEnum.YEAR, ptoInfo.getDisbursedHoursYTD());
		return paidTimeOffDtl;
	}

	public static PTOInfo mapToPTOInfo(Long workerId, PaidTimeOffDtl paidTimeOffDtl) throws ServiceFailureException {

		PTOInfo ptoInfo = new PTOInfo();

		ptoInfo.setObjectHandle(ObjectHandleMapper.mapToObjectHandle(paidTimeOffDtl));
		ptoInfo.setWorkerId(workerId);
		Datum rate = paidTimeOffDtl.getRate();
		ptoInfo.setAccrualRate(rate.getValue1AsQuantity());
		ptoInfo.setAccrualStartDate(paidTimeOffDtl.getAccrualStartDate());
		ptoInfo.setAccrualStartType(mapToAccrualStartType(paidTimeOffDtl.getAccrualStartType()));
		ptoInfo.setAccruedHours(paidTimeOffDtl.getAccruedHours());
		ptoInfo.setAllowCarryOver(paidTimeOffDtl.isAllowCarryOver());
		ptoInfo.setLimitAccrual(paidTimeOffDtl.isLimitAccrual());
		ptoInfo.setMaximumAccrual(paidTimeOffDtl.getMaximumAccrual());
		ptoInfo.setName(paidTimeOffDtl.getDescription());
		PTOAccrualType ptoAccrualType = mapToAccrualType(paidTimeOffDtl.getAccrualType());
		if (!mapToDatumType2Enum(paidTimeOffDtl.getAccrualType()).equals(rate.getType2())) {
			String msg = MessageFormat.format(
					"Data integrity problem: PaidTimeOffDetail has accrual type ''{0}'' but Datum type 2 ''{1}''",
					mapToDatumType2Enum(paidTimeOffDtl.getAccrualType()), rate.getType2());
			logger.error(msg);
			throw new ServiceFailureException(msg);
		}
		ptoInfo.setPtoAccrualType(ptoAccrualType);
		ptoInfo.setPtoType(mapToPTOType(paidTimeOffDtl.getPaidTimeOffType()));
		// disbursed hours year-to-date
		Set<PayrollItemDtlPtdDatum> ptdDatumSet = paidTimeOffDtl.getPtdDatums();
		for (PayrollItemDtlPtdDatum ptdDatum : ptdDatumSet) {
			PtdDatumType datum = ptdDatum.getDatum();
			// PTO payrollItems can contain both MONEY and QUANTITY ptdDatums, we want the QUANTITY type
			if (datum.getPtdType().equals(PtdTypeEnum.YEAR) && DatumType1Enum.QUANTITY == datum.getDatumType()) {
				// Assume only one of type YEAR
				ptoInfo.setDisbursedHoursYTD(datum.getQuantityValue());
				break;
			}
		}

		return ptoInfo;
	}

	private static void addOrUpdatePtdDatum(PaidTimeOffDtl paidTimeOffDtl, PtdTypeEnum type, Double hoursYtd) {
		PtdDatumType datum = null;
		Set<PayrollItemDtlPtdDatum> ptdDatumSet = paidTimeOffDtl.getPtdDatums();
		// Check to see if there is an existing PtdDatum of the correct type
		for (PayrollItemDtlPtdDatum ptdDatum : ptdDatumSet) {
			PtdDatumType ptdDatumType = ptdDatum.getDatum();
			if ((type == ptdDatumType.getPtdType()) && (DatumType1Enum.QUANTITY == ptdDatumType.getDatumType())) {
				datum = ptdDatumType;
				break;
			}
		}
		if (null == datum) {
			// No existing PtdDatum, so create one
			datum = new PtdDatumType();
			PayrollItemDtlPtdDatum ptdDatum = new PayrollItemDtlPtdDatum();
			ptdDatum.setDatum(datum);
			ptdDatumSet.add(ptdDatum);
		}
		// Set/update the values
		datum.setQuantityValue(hoursYtd);
		datum.setPtdType(type);
	}
	
	// Maybe we don't need this...
	// private static EnumMap<DatumType2Enum, PaidTimeOffAccrualTypeEnum> datumType2Map =
	// new EnumMap<DatumType2Enum, PaidTimeOffAccrualTypeEnum>(DatumType2Enum.class);
	// static {
	// datumType2Map.put(DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
	// datumType2Map.put(DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
	// datumType2Map.put(DatumType2Enum.MONTH, PaidTimeOffAccrualTypeEnum.MONTHLY);
	// datumType2Map.put(DatumType2Enum.WEEK, PaidTimeOffAccrualTypeEnum.WEEKLY);
	// datumType2Map.put(DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.START_OF_YEAR);
	// }
	// private static PaidTimeOffAccrualTypeEnum mapToPaidTimeOffAccrualTypeEnum(DatumType2Enum datumType2Enum) {
	// PaidTimeOffAccrualTypeEnum paidTimeOffAccrualTypeEnum = datumType2Map.get(datumType2Enum);
	// return paidTimeOffAccrualTypeEnum == null ? PaidTimeOffAccrualTypeEnum.NOTSET : paidTimeOffAccrualTypeEnum;
	// }

	private static EnumMap<PaidTimeOffAccrualTypeEnum, DatumType2Enum> paidTimeOffAccrualTypeMap = new EnumMap<PaidTimeOffAccrualTypeEnum, DatumType2Enum>(
			PaidTimeOffAccrualTypeEnum.class);
	static {
		paidTimeOffAccrualTypeMap.put(PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK, DatumType2Enum.PERPAYCHECK);
		paidTimeOffAccrualTypeMap.put(PaidTimeOffAccrualTypeEnum.HOURLY, DatumType2Enum.HOUR);
		paidTimeOffAccrualTypeMap.put(PaidTimeOffAccrualTypeEnum.PER_YEAR_START_OF_YEAR, DatumType2Enum.YEAR);
		paidTimeOffAccrualTypeMap.put(PaidTimeOffAccrualTypeEnum.PER_YEAR_FROM_HIRE_DATE, DatumType2Enum.YEAR);
		paidTimeOffAccrualTypeMap.put(PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED, DatumType2Enum.PERPAYCHECK);
	}

	private static DatumType2Enum mapToDatumType2Enum(PaidTimeOffAccrualTypeEnum paidTimeOffAccrualType) {
		DatumType2Enum datumType2Enum = paidTimeOffAccrualTypeMap.get(paidTimeOffAccrualType);
		return datumType2Enum == null ? DatumType2Enum.NOTSET : datumType2Enum;
	}

	private static EnumMap<PaidTimeOffAccrualTypeEnum, PTOAccrualType> ptoAccrualTypeEnumMap = new EnumMap<PaidTimeOffAccrualTypeEnum, PTOAccrualType>(
			PaidTimeOffAccrualTypeEnum.class);
	static {
		ptoAccrualTypeEnumMap.put(PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK, PTOAccrualType.EACH_PAYCHECK);
		ptoAccrualTypeEnumMap.put(PaidTimeOffAccrualTypeEnum.HOURLY, PTOAccrualType.HOURLY);
		ptoAccrualTypeEnumMap.put(PaidTimeOffAccrualTypeEnum.PER_YEAR_START_OF_YEAR, PTOAccrualType.PER_YEAR_START_OF_YEAR);
		ptoAccrualTypeEnumMap.put(PaidTimeOffAccrualTypeEnum.PER_YEAR_FROM_HIRE_DATE, PTOAccrualType.PER_YEAR_FROM_HIRE_DATE);
		ptoAccrualTypeEnumMap.put(PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED, PTOAccrualType.PER_YEAR_PRORATED);
	}

	private static PTOAccrualType mapToAccrualType(PaidTimeOffAccrualTypeEnum accrualTypeEnum) {
		return ptoAccrualTypeEnumMap.get(accrualTypeEnum);
	}

	private static Map<PTOAccrualType, PaidTimeOffAccrualTypeEnum> accrualTypeMap = new HashMap<PTOAccrualType, PaidTimeOffAccrualTypeEnum>();
	static {
		accrualTypeMap.put(PTOAccrualType.EACH_PAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
		accrualTypeMap.put(PTOAccrualType.HOURLY, PaidTimeOffAccrualTypeEnum.HOURLY);
		accrualTypeMap.put(PTOAccrualType.PER_YEAR_START_OF_YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_START_OF_YEAR);
		accrualTypeMap.put(PTOAccrualType.PER_YEAR_FROM_HIRE_DATE, PaidTimeOffAccrualTypeEnum.PER_YEAR_FROM_HIRE_DATE);
		accrualTypeMap.put(PTOAccrualType.PER_YEAR_PRORATED, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
	}

	public static PaidTimeOffAccrualTypeEnum mapToAccrualTypeEnum(PTOAccrualType accrualType) {
		PaidTimeOffAccrualTypeEnum accrualTypeEnum = accrualTypeMap.get(accrualType);
		return (accrualTypeEnum == null) ? PaidTimeOffAccrualTypeEnum.NOTSET : accrualTypeEnum;
	}

	private static EnumMap<PaidTimeOffAccrualStartTypeEnum, AccrualStartType> accrualStartTypeEnumMap = new EnumMap<PaidTimeOffAccrualStartTypeEnum, AccrualStartType>(
			PaidTimeOffAccrualStartTypeEnum.class);
	static {
		accrualStartTypeEnumMap.put(PaidTimeOffAccrualStartTypeEnum.IMMEDIATELY, AccrualStartType.IMMEDIATELY);
		accrualStartTypeEnumMap.put(PaidTimeOffAccrualStartTypeEnum.ON_START_DATE, AccrualStartType.ON_START_DATE);
	}

	private static AccrualStartType mapToAccrualStartType(PaidTimeOffAccrualStartTypeEnum accrualStartTypeEnum) {
		return accrualStartTypeEnumMap.get(accrualStartTypeEnum);
	}

	private static Map<AccrualStartType, PaidTimeOffAccrualStartTypeEnum> accrualStartTypeMap = new HashMap<AccrualStartType, PaidTimeOffAccrualStartTypeEnum>();
	static {
		accrualStartTypeMap.put(AccrualStartType.IMMEDIATELY, PaidTimeOffAccrualStartTypeEnum.IMMEDIATELY);
		accrualStartTypeMap.put(AccrualStartType.ON_START_DATE, PaidTimeOffAccrualStartTypeEnum.ON_START_DATE);
	}

	public static PaidTimeOffAccrualStartTypeEnum mapToAccrualStartTypeEnum(AccrualStartType accrualStartType) {
		PaidTimeOffAccrualStartTypeEnum accrualStartTypeEnum = accrualStartTypeMap.get(accrualStartType);
		return (accrualStartTypeEnum == null) ? PaidTimeOffAccrualStartTypeEnum.NOTSET : accrualStartTypeEnum;
	}

	private static EnumMap<PaidTimeOffTypeEnum, PTOType> paidTimeOffTypeEnumMap = new EnumMap<PaidTimeOffTypeEnum, PTOType>(
			PaidTimeOffTypeEnum.class);
	static {
		paidTimeOffTypeEnumMap.put(PaidTimeOffTypeEnum.SICK, PTOType.SICK_TIME);
		paidTimeOffTypeEnumMap.put(PaidTimeOffTypeEnum.VACATION, PTOType.VACATION);
	}

	public static PTOType mapToPTOType(PaidTimeOffTypeEnum PTOType) {
		return paidTimeOffTypeEnumMap.get(PTOType);
	}

	private static Map<PTOType, PaidTimeOffTypeEnum> hourlyPTOTypeMap = new HashMap<PTOType, PaidTimeOffTypeEnum>();
	static {
		hourlyPTOTypeMap.put(PTOType.SICK_TIME, PaidTimeOffTypeEnum.SICK);
		hourlyPTOTypeMap.put(PTOType.VACATION, PaidTimeOffTypeEnum.VACATION);
	}

	private static Map<PTOType, PaidTimeOffTypeEnum> salaryPTOTypeMap = new HashMap<PTOType, PaidTimeOffTypeEnum>();
	static {
		salaryPTOTypeMap.put(PTOType.SICK_TIME, PaidTimeOffTypeEnum.SICK);
		salaryPTOTypeMap.put(PTOType.VACATION, PaidTimeOffTypeEnum.VACATION);
	}

	/**
	 * The UI now specifically sets the ptoBaseRateType on a salary or hourly wage. It follows this logic:
	 * compensation is the base rate for Vacation pto - PtoBaseRateType.VACATION
	 * compensation is the base rate for Sick pto - PtoBaseRateType.SICK
	 * compensation is the base rate for Vacation & Sick pto's - PtoBaseRateType.PTO
	 * 
	 * @param worker
	 * @param ptoType
	 * @return the appropriate PaidTimeOffTypeEnum
	 * @throws ServiceFailureException
	 * @throws ValidationException
	 */
	private static PaidTimeOffTypeEnum mapToPaidTimeOffTypeEnum(Worker worker, PTOType ptoType)
		throws ServiceFailureException, ValidationException {

		PaidTimeOffTypeEnum paidTimeOffTypeEnum = null;
		if (PTOType.SICK_TIME.equals(ptoType)) {
			paidTimeOffTypeEnum = PaidTimeOffTypeEnum.SICK;
		} else if (PTOType.VACATION.equals(ptoType)) {
			paidTimeOffTypeEnum = PaidTimeOffTypeEnum.VACATION;
		} 
		
		return (paidTimeOffTypeEnum == null) ? PaidTimeOffTypeEnum.NOTSET : paidTimeOffTypeEnum;
	}

	public static PTOInfo[] mapToPTOInfos(Long workerId, Set<PaidTimeOffDtl> paidTimeOffDtls) throws ServiceFailureException {
		List<PTOInfo> results = new ArrayList<PTOInfo>();
		if (paidTimeOffDtls == null || paidTimeOffDtls.size() == 0) {
			//
		} else {
			for (PaidTimeOffDtl paidTimeOffDtl : paidTimeOffDtls) {
				PTOInfo info = mapToPTOInfo(workerId, paidTimeOffDtl);
				results.add(info);
			}
		}
		PTOInfo[] result = results.toArray(new PTOInfo[0]);

		return result;
	}
}
