package com.intuit.sb.payroll.services.taxes;

import java.util.Date;

import javax.ejb.Remote;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;

/**
 * Service interface definition for EmployerTaxSettings. This include setting up teh TxIDs, SUI rates, retrieving the tax rates from the system etc.
 * Accessible through the PayrollServiceFactory
 *
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
@Remote
public interface EmployerTaxSettingsService
{
	public static final String JNDI_NAME = "EmployerTaxSettingsService/remote";

	/**
	 * retrieves and returns the collection of SuiSetting objects for the Employer
	 * If no SuiSettings are found for the employer, an empty collection is returned
	 * @return
	 */
	public SuiSetting[] getEmployerSUIsettings();

	/**
	 * Updates the Employer SuiSetting collection passed in
	 * The SuiSetting Array parameter can contain the subset of the SUI Array for the company
	 * in that case, only the received parameter will be updated
	 * @param suiSettings
	 */
	public SuiSetting[] updateEmployerSUIsettings(SuiSetting[] settings) throws ServiceFailureException , ValidationException;
	
	/**
	 * gets a list all EmployerId's
	 * @return
	 * @throws ValidationException 
	 * @throws ServiceFailureException 
	 */
	public EmployerTaxIdInfo[] getEmployerTaxIDs() throws ValidationException, ServiceFailureException;

	/**
	 * updates a EmployerId's in the list provided
	 * @param taxIDs
	 * @throws ServiceFailureException 
	 */
	public void updateEmployerTaxIDs(EmployerTaxIdInfo[] taxIDinfos) throws ValidationException, ServiceFailureException;

	/**
	 * get the Tax Details applicable for the company
	 * If there is no TaxDetails present, return an empty array
	 * @return TaxDetails array
	 * @throws ValidationException 
	 * @throws ServiceFailureException 
	 */
	public TaxDetails[] getTaxDetails() throws ValidationException, ServiceFailureException;

	/**
	 * Get the Jurisdictions on which this employer will operate one
	 * @return Array of Jurisdiction
	 * @throws ServiceFailureException 
	 */
	public Jurisdiction[] getJurisdictions() throws ServiceFailureException;

	/**
	 * Get the date when the user sent his/her forms to PTS - basically, <br>
	 * this would be the last date when the user has sent (add or update) company information to PTS
	 * for Printing forms
	 * @return Date of user sending company info to PTS
	 * @throws ServiceFailureException if there is any problem retrieving this information
	 */
	public Date getTaxPaymentsFormSubmissionDate() throws ServiceFailureException;
	/**
	 * Get all the valid EIN prefixes 
	 * @return array of string that contain all valid EIN prefixes
	 * @throws ServiceFailureException
	 */
	public String[] getAllValidEinPrefixes() throws ServiceFailureException;
	
	/**
	 * retrieves and returns the collection of EmployerUnemploymentIns objects for the Employer
	 * If no EmployerUnemploymentIns are found for the employer, an empty collection is returned
	 * @return
	 */
	public EmployerUnemploymentIns[] getEmployerUnemploymentIns();
	
	/**
	 * Updates the Employer EmployerUnemploymentIns collection passed in
	 * The EmployerUnemploymentIns Array parameter can contain the subset of the SUI Array for the company
	 * in that case, only the received parameter will be updated
	 * @param EmployerUnemploymentIns
	 */
	public EmployerUnemploymentIns[] updateEmployerUnemploymentIns(EmployerUnemploymentIns[] employerUnemploymentIns) throws ServiceFailureException , ValidationException;


}

