package com.intuit.sb.payroll.services.taxes;

import javax.ejb.Remote;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;

/**
 * Service interface definition for TaxWithholdings.  Handles retrieval of Withholding data as
 * well as updates (including free Fed to State mapping!).
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
@Remote
public interface TaxWithholdingsService
{
	public static final String JNDI_NAME = "TaxWithholdingsService/remote";

	/**
	 * Returns the list of employee WithholdingInformation instances related to the worker.
	 * 
	 * @param workerId
	 * @return the list.
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 */
	public WithholdingInformation[] getEmployeeWithholdings(Long workerId) throws ServiceFailureException, ValidationException;
    
    /**
     * Does the CRUD operations as requested in the packet, as well it sets up any defaults/derivations that need to be
     * done.
     * 
     * @param request
     * @return
     * @throws ValidationException
     * @throws WorkflowException
     * @throws ServiceFailureException
     */
    public SaveWithholdingResponse saveWithholdingInformation(SaveWithholdingInfoRequest request) 
    	throws ValidationException, WorkflowException, ServiceFailureException;
}
