package com.intuit.sb.payroll.services.taxes.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.employer.EmployerTaxSettingComponent;
import com.intuit.sb.payroll.capability.employer.impl.EmployerTaxIdValidator;
import com.intuit.sb.payroll.capability.ppe.AgencyView;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.UnemploymentInsDocDtl;
import com.intuit.sb.payroll.entity.taxauthority.JurisdictionEAO;
import com.intuit.sb.payroll.services.taxes.EmployerTaxIdInfo;
import com.intuit.sb.payroll.services.taxes.EmployerTaxSettingsService;
import com.intuit.sb.payroll.services.taxes.EmployerUnemploymentIns;
import com.intuit.sb.payroll.services.taxes.Jurisdiction;
import com.intuit.sb.payroll.services.taxes.SuiSetting;
import com.intuit.sb.payroll.services.taxes.TaxDetails;

@Stateless(name = "EmployerTaxSettingsService")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class EmployerTaxSettingsServiceBean implements EmployerTaxSettingsService {
	@EJB
	private JurisdictionEAO jurisdictionEAO;
	@EJB
	private TaxComponent taxComponent;
	@EJB
	private EmployerComponent employerComponent;
	@EJB
	private EmployerPTSWorkflowComponent employerPtsWkflowComponent;
	@EJB
	private EmployerTaxSettingComponent  employerTaxSettingComponent;

	/**
	 * {@inheritDoc}
	 */
	public SuiSetting[] getEmployerSUIsettings() {
		List<UnemploymentInsDocDtl> suiTaxes = taxComponent.getEmployerUnemploymentInsurances();

		if (suiTaxes == null) {
			return new SuiSetting[0];
		}

		SuiSetting[] suiSettings = new SuiSetting[suiTaxes.size()];
		int i = 0;
		for (UnemploymentInsDocDtl suiTax : suiTaxes) {
			com.intuit.sb.payroll.entity.model.Jurisdiction jurisdiction = jurisdictionEAO
					.findJurisdictionForLawId(suiTax.getLawId().getValue());
			suiSettings[i++] = SuiTaxMapper.mapToSuiSetting(suiTax, jurisdiction);
		}
		return suiSettings;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public EmployerUnemploymentIns[] getEmployerUnemploymentIns() {
		List<UnemploymentInsDocDtl> suiTaxes = taxComponent.getEmployerUnemploymentInsurances();

		if (suiTaxes == null) {
			return new EmployerUnemploymentIns[0];
		}

		EmployerUnemploymentIns[] employerUnemploymentIns = new EmployerUnemploymentIns[suiTaxes.size()];
		int i = 0;
		for (UnemploymentInsDocDtl suiTax : suiTaxes) {
			com.intuit.sb.payroll.entity.model.Jurisdiction jurisdiction = jurisdictionEAO
					.findJurisdictionForLawId(suiTax.getLawId().getValue());
			employerUnemploymentIns[i++] = SuiTaxMapper.mapToEmployerUnemploymentIns(suiTax, jurisdiction);
		}
		return employerUnemploymentIns;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws ValidationException
	 */
	public SuiSetting[] updateEmployerSUIsettings(SuiSetting[] settings) throws ServiceFailureException, ValidationException {
		SuiSetting[] savedSuiSettings = null;
		int i = 0;
		if (settings != null) {
			savedSuiSettings = new SuiSetting[settings.length];
		}
		validateSuiSettings(settings);
		for (SuiSetting suiSetting : settings) {
			if (suiSetting == null) {
				continue;
			}
			// Locate the UnemploymentInsDocDtl in the db
			UnemploymentInsDocDtl unemploymentInsDocDtl = taxComponent.getEmployerUnemploymentInsurance(suiSetting
					.getUniqueId());

			// Fill in the data
			Double currentRate = Double.valueOf(suiSetting.getCurrentRate());
			Double futureRate = null;
			Date futureRateStartDate = suiSetting.getFutureRateStartDate();
			if (suiSetting.getFutureRate() != null && !suiSetting.getFutureRate().trim().equals("")) {
				validateRate(suiSetting.getFutureRate());
				futureRate = Double.valueOf(suiSetting.getFutureRate());
			}

			unemploymentInsDocDtl.setPercentageRate(currentRate);
			unemploymentInsDocDtl.setPercentageFutureRate(futureRate);
			unemploymentInsDocDtl.setFutureRateStartDate(futureRateStartDate);
			unemploymentInsDocDtl.setRateChangedByIntuit(suiSetting.isRateChangedByIntuit());
			unemploymentInsDocDtl.setDefaultRate(false);

			// Update it
			UnemploymentInsDocDtl savedUnemploymentInsDocDtl =
				taxComponent.updateEmployerUnemploymentInsurance(unemploymentInsDocDtl);

			//  create a dummy Jurisdiction object with just a name
			//	this is used simply to provide the state name to the mapper which will use that
			//	to create a SuiSetting object that will be returned
			com.intuit.sb.payroll.entity.model.Jurisdiction dummyJurisdiction = 
				new com.intuit.sb.payroll.entity.model.Jurisdiction();
			dummyJurisdiction.setName(suiSetting.getState());
			if (savedSuiSettings != null) {
				savedSuiSettings[i++] = SuiTaxMapper.mapToSuiSetting(savedUnemploymentInsDocDtl, dummyJurisdiction);
			}
		}
		
		return savedSuiSettings;
	}

	
/**
 * {@inheritDoc}
 * 
 * @throws ValidationException
 */
public EmployerUnemploymentIns[] updateEmployerUnemploymentIns(EmployerUnemploymentIns[] employerUnemploymentIns ) throws ServiceFailureException, ValidationException {
	EmployerUnemploymentIns[] savedEmployerUnemploymentIns = null;

	validateEmployerUnemploymentIns(employerUnemploymentIns);
	List<UnemploymentInsDocDtl> unList = new ArrayList<UnemploymentInsDocDtl>();
	if (employerUnemploymentIns != null) {
		savedEmployerUnemploymentIns = new EmployerUnemploymentIns[employerUnemploymentIns.length];
	}
	int i = 0;
	int index = 0;
	//validateSuiSettings(employerUnemploymentIns);
	for (EmployerUnemploymentIns eIns : employerUnemploymentIns) {
		if (eIns == null) {
			continue;
		}
		unList.add(SuiTaxMapper.mapToUnemploymentInsDocDtl(eIns));
	}
	
	// Update it
	List<UnemploymentInsDocDtl> savedUnemploymentInsDocDtl = taxComponent.updateUnemploymentInsurances(unList);

	for (EmployerUnemploymentIns eIns:employerUnemploymentIns){
		if (eIns == null) {
			continue;
		}
		//create a dummy Jurisdiction object with just a name
		//	this is used simply to provide the state name to the mapper which will use that
		//	to create a SuiSetting object that will be returned
		com.intuit.sb.payroll.entity.model.Jurisdiction dummyJurisdiction = 
			new com.intuit.sb.payroll.entity.model.Jurisdiction();
		dummyJurisdiction.setName(eIns.getState());

		if (savedEmployerUnemploymentIns != null) {
			savedEmployerUnemploymentIns[i++] = SuiTaxMapper.mapToEmployerUnemploymentIns(savedUnemploymentInsDocDtl.get(index++), dummyJurisdiction);

		}
	}
	
	return savedEmployerUnemploymentIns;
}


	/**
	 * {@inheritDoc}
	 * 
	 * @throws ValidationException
	 * @throws ServiceFailureException 
	 */
	public EmployerTaxIdInfo[] getEmployerTaxIDs() throws ValidationException, ServiceFailureException {
		
		List<EmployerId> employerIds = employerTaxSettingComponent.getEmployerIds();
		
		EmployerTaxIdInfo[] employerTaxIdInfos = new EmployerTaxIdInfo[employerIds.size()];
		int i = 0;
		for (EmployerId employerId : employerIds) {
			employerTaxIdInfos[i] = EmployerTaxIdMapper.mapToEmployerTaxIdInfo(employerId);
			i++;
		}
		return employerTaxIdInfos;
	}

	/**
	 * {@inheritDoc}
	 * @throws ServiceFailureException 
	 */
	public void updateEmployerTaxIDs(EmployerTaxIdInfo[] taxIDinfos) throws ValidationException, ServiceFailureException {
		validateEmployerTaxIdInfo(taxIDinfos);
		checkIfUpdateisAllowed(taxIDinfos);
		Employer currentEmployer = getCurrentEmployer();
		List<EmployerId> employerIds = new ArrayList<EmployerId>();
		for (EmployerTaxIdInfo idInfo : taxIDinfos) {
			EmployerId employerId = getTaxIdFromDb(idInfo.getUniqueId(), currentEmployer);
			validateTaxIdFormat(idInfo.getTaxId(), employerId.getAgencyIdType().getAgencyIdValidationPattern());
			String strippedId = stripSpecialChars(idInfo.getTaxId());
			employerId.setValue(strippedId);
			EmployerTaxIdValidator.getInstance().validateEntity(employerId);
			employerIds.add(employerId);
		}
		employerTaxSettingComponent.updateEmployerIds(employerIds); 
	}


	/**
	 * {@inheritDoc}
	 * 
	 * @throws ValidationException
	 * @throws ServiceFailureException 
	 */
	public TaxDetails[] getTaxDetails() throws ValidationException, ServiceFailureException {
		Employer employer = getCurrentEmployer();
		List<com.intuit.sb.payroll.entity.model.Jurisdiction> jurisdictions = employerComponent
				.determineAllJurisdictions();

		List<AgencyView> agancyViews = taxComponent.getEmployerAgencyDetails(employer, jurisdictions);
		// now get the TaxDetails for each of the AgencyView
		TaxDetails[] taxDetailsArray = new TaxDetails[agancyViews.size()];
		int i = 0;
		for (AgencyView agencyView : agancyViews) {
			taxDetailsArray[i++] = TaxDetailsMapper.mapToTaxDetails(agencyView);
		}
		return taxDetailsArray;
	}

	public Jurisdiction[] getJurisdictions() throws ServiceFailureException {
		List<com.intuit.sb.payroll.entity.model.Jurisdiction> entityJurisdictionList = employerComponent
				.determineAllJurisdictions();
		if (entityJurisdictionList == null)
			return new Jurisdiction[0];

		Jurisdiction[] serviceJurisdiction = new Jurisdiction[entityJurisdictionList.size()];
		int i = 0;
		for (com.intuit.sb.payroll.entity.model.Jurisdiction jurisdiction : entityJurisdictionList) {
			serviceJurisdiction[i++] = JurisdictionMapper.mapToService(jurisdiction);
		}
		return serviceJurisdiction;
	}

	/********************* private methods 
	 * @throws ServiceFailureException *****************************************/

	private Employer getCurrentEmployer() throws ValidationException, ServiceFailureException {
		Employer employer = employerComponent.getEmployer();
		if (employer == null) {
			throw new IllegalStateException("Employer has not been configured.");
		}
		return employer;
	}

	private EmployerId getTaxIdFromDb(Long uniqueid, Employer currentEmployer) throws ValidationException {
		if (uniqueid == null) {
			throw new ValidationException("Empty uniqueId to find Id from list of employerid's");
		}

		Set<EmployerId> empIdSet = currentEmployer.getEmployerIds();

		if (empIdSet == null) {
			return null;
		}

		for (EmployerId employerId : empIdSet) {
			if (uniqueid.equals(employerId.getId())) {
				return employerId;
			}
		}

		throw new ValidationException("tax IdInfo Not Found");
	}

	private String stripSpecialChars(String taxId) {
		return (taxId == null) ? null : taxId.replace("-", "");
	}

	private void validateTaxIdFormat(String taxId, String agencyIdValidationPattern) throws ValidationException {
		Pattern validationPattern = Pattern.compile(agencyIdValidationPattern);
		if ((taxId == null) || ("".equals(taxId.trim()))) {
			// Some Id's can be empty till they are added
			return;
		}
		if (!validationPattern.matcher(taxId).matches()) {
			throw new ValidationException("TaxId: " + taxId + " not in proper format");
		}
	}

	private void validateSuiSettings(SuiSetting[] settings) throws ValidationException {
		if (settings == null) {
			throw new ValidationException("SUISettings can not be null");
		}

		if (settings.length == 0) {
			throw new ValidationException("SUISettings can not be empty");
		}
		Date today = LocaleGuru.getInstance().getCalendar().getTime();
		Date futureRateStartDate;
		String futureRate;
		SuiSetting[] actualSettings = getEmployerSUIsettings();
		for (SuiSetting suiSetting : settings) {
			
			for(SuiSetting aSetting:actualSettings){//IP000106
				
				if(suiSetting.getUniqueId().longValue()== aSetting.getUniqueId().longValue()){
					if(!suiSetting.getState().equals(aSetting.getState())){
						throw new ValidationException("SuiSetting state cannot be updated");
					}
				}
					
			}
			
			String currentRate = suiSetting.getCurrentRate();
			validateRate(currentRate);
			futureRate = suiSetting.getFutureRate();
			futureRateStartDate = suiSetting.getFutureRateStartDate();
			if (!((futureRate == null) || (futureRate.length() == 0))) {
				if (futureRateStartDate != null && futureRateStartDate.before(today)) {
					throw new ValidationException("Future Start Date cannot be previous to today's date");
				}
			}

		}
		
	}
	
	private void validateEmployerUnemploymentIns(EmployerUnemploymentIns[] settings) throws ValidationException {
		if (settings == null) {
			throw new ValidationException("EmployerUnemploymentIns can not be null");
		}

		if (settings.length == 0) {
			throw new ValidationException("EmployerUnemploymentIns can not be empty");
		}
		EmployerUnemploymentIns[] actualSettings = getEmployerUnemploymentIns();
		for (EmployerUnemploymentIns suiSetting : settings) {
			
			for(EmployerUnemploymentIns aSetting:actualSettings){//IP000106
				
				if(suiSetting.getObjectHandle().getId() == aSetting.getObjectHandle().getId()){
					if(!suiSetting.getState().equals(aSetting.getState())){
						throw new ValidationException("EmployerUnemploymentIns state cannot be updated");
					}
				}
					
			}
			
			String currentRate = suiSetting.getEffectiveRate();
			validateRate(currentRate);
		}
		
	}

	private void validateRate(String currentRate) throws ValidationException {
		if (currentRate == null || currentRate.length() == 0) {
			throw new ValidationException("Current Rate cannot be empty");
		}
		try {
			Double.valueOf(currentRate);
		} catch (NumberFormatException nfe) {
			throw new ValidationException(nfe);
		}
	}

	private void validateEmployerTaxIdInfo(EmployerTaxIdInfo[] taxIDinfos) throws ValidationException {
		if (taxIDinfos == null) {
			throw new ValidationException("EmployerTaxIdInfo can not be null");
		}

		if (taxIDinfos.length == 0) {
			throw new ValidationException("EmployerTaxIdInfo can not be empty");
		}
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIDinfos) {
			if (employerTaxIdInfo.getTaxAgency() == null) {
				throw new ValidationException("EmployerTaxIdInfo Tax Agency can not be null");
			}
		}
	}
	
	private void checkIfUpdateisAllowed(EmployerTaxIdInfo[] taxIDinfos) throws ValidationException {
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIDinfos) {
			if (!StatusUpdateHandlerUtil.
			    isEmployerIdUpdateAllowed(employerTaxIdInfo.getTaxAgency().name)){
				throw new ValidationException("Tax Id update not allowed for "+employerTaxIdInfo.getTaxAgency().name);
			}
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Date getTaxPaymentsFormSubmissionDate() throws ServiceFailureException {
		Date date = employerPtsWkflowComponent.getTaxPaymentsFormSubmissionDate();
		return date;
	}
	
	/**
	 * Get all the valid EIN prefixes 
	 * @return array of string that contain all valid EIN prefixes
	 * @throws ServiceFailureException
	 */
	public String[] getAllValidEinPrefixes() throws ServiceFailureException {
		String[] validEinPrefixes = null;
		try {
			validEinPrefixes = employerComponent.getAllValidEinPrefixes();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return validEinPrefixes;
	}
}
