package com.intuit.sb.payroll.services.taxes.impl;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingInfoRequest;
import com.intuit.sb.payroll.services.taxes.WithholdingInfoOperation;

/**
 * Utility class for performing SaveWithholdingInfoRequest validation.  Validates the SaveWithholdingInfoRequest
 * passed in by the client.  If any of the validations fails throws ValidationException.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 */
public class SaveWithholdingInfoRequestValidator {

	/**
	 * Private constructor to prevent instantiation.
	 */
    private SaveWithholdingInfoRequestValidator() {}
    
    
    /**
     * <p>
     * Checks if {@link SaveWithholdingInfoRequest} is valid.
     * </p>
     *
     * @param saveRequest The value validation is being performed on.
     * @throws ValidationException if the validation fails.
     */
    public static void validate(SaveWithholdingInfoRequest saveRequest) throws ValidationException {
    	PayrollParamValidator.checkIsNotNull(saveRequest, "saveRequest");
		PayrollParamValidator.checkIsNotNull(saveRequest.getWorkerId(), "workerId");
		PayrollParamValidator.checkIsNotNull(saveRequest.getWithholdingInfoOperation(), "withholdingInfoOperation");
		// If the operation is anything other than NOT_SPECIFIED, then WithholdingInformation must be set
		if (WithholdingInfoOperation.NOT_SPECIFIED != saveRequest.getWithholdingInfoOperation()) {
			PayrollParamValidator.checkIsNotNull(saveRequest.getWithholdingInfo(), "withholdingInfo");
		}
    }

}
