package com.intuit.sb.payroll.services.taxes.impl;

import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.services.taxes.WithholdingStatus;

/**
 * Maps from the EI to the Entity Type.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
class WithholdingStatusMapper {

	/**
	 * Map a WithholdingStatusType enum to the equivalent WorkerComplianceDocDtl enum.
	 * @param status the enum to translate.
	 * @return the translated enum.
	 */
	static DE4WithholdingStatusEnum mapWithholdingStatusToDE4Detail(WithholdingStatus status) {
		// These "fake" enums can be null.
		if (null == status) {
			return DE4WithholdingStatusEnum.NOTSET;
		}
		if (status.equals(WithholdingStatus.MARRIED)) {
			return DE4WithholdingStatusEnum.MARRIED;
		} else if (status.equals(WithholdingStatus.MARRIED_HEAD_OF_HOUSEHOLD)) {
			return DE4WithholdingStatusEnum.HEADOFHOUSEHOLD;
		} else if (status.equals(WithholdingStatus.MARRIED_SINGLE_RATE)) {
			return DE4WithholdingStatusEnum.MARRIEDONEINCOME;
		} else if (status.equals(WithholdingStatus.SINGLE)) {
			return DE4WithholdingStatusEnum.SINGLE;
		}
		return DE4WithholdingStatusEnum.NOTSET;
	}

	static W4WithholdingStatusEnum mapWithholdingStatusToW4Detail(WithholdingStatus status) {
		// These "fake" enums can be null.
		if (null == status) {
			return W4WithholdingStatusEnum.NOTSET;
		}
		if (status.equals(WithholdingStatus.MARRIED)) {
			return W4WithholdingStatusEnum.MARRIED;
		} else if (status.equals(WithholdingStatus.MARRIED_HEAD_OF_HOUSEHOLD)) {
			//FIXME W4 DOES NOT SUPPORT HEADOFHOUSEHOLD.. HENCE SETTING IT TO NOTSET
			return W4WithholdingStatusEnum.NOTSET;
		} else if (status.equals(WithholdingStatus.MARRIED_SINGLE_RATE)) {
			return W4WithholdingStatusEnum.MARRIEDSINGLERATE;
		} else if (status.equals(WithholdingStatus.SINGLE)) {
			return W4WithholdingStatusEnum.SINGLE;
		}
		return W4WithholdingStatusEnum.NOTSET;
	}
		
	static WithholdingStatus mapDE4WithholdingStatusEnumToWithholdingStatus(DE4WithholdingStatusEnum status) {
		if (status == null) {
			return WithholdingStatus.NOT_SET;
		}
		switch (status) {
		case MARRIED:
			return WithholdingStatus.MARRIED;
		case MARRIEDONEINCOME:
			//FIXME  Hack: Since there isn't a corresponding enum value in WithholdingStatus, so setting it to
			//             MARRIED_SINGLE_RATE for now so the Gemini app can use this value to map to the correct value
			return WithholdingStatus.MARRIED_SINGLE_RATE;
		case  SINGLE:
			return WithholdingStatus.SINGLE;
		case HEADOFHOUSEHOLD:
			return WithholdingStatus.MARRIED_HEAD_OF_HOUSEHOLD;
		case NOTSET:
		   return WithholdingStatus.NOT_SET;
		default:
			return WithholdingStatus.NOT_SET;
		}
	}
	
	static WithholdingStatus mapW4WithholdingStatusEnumToWithholdingStatus(W4WithholdingStatusEnum status) {
		if (status == null) {
			return WithholdingStatus.NOT_SET;
		}
		switch (status) {
		case MARRIED:
			return WithholdingStatus.MARRIED;
		case MARRIEDSINGLERATE:
			return WithholdingStatus.MARRIED_SINGLE_RATE;
		case  SINGLE:
			return WithholdingStatus.SINGLE;
		case NOTSET:
		   return WithholdingStatus.NOT_SET;
		default:
			return WithholdingStatus.NOT_SET;
		}
	}
	
	
	
	
	
}
