package com.intuit.sb.payroll.services.taxpayments;

import java.io.Serializable;
import java.math.BigDecimal;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.services.common.ObjectHandle;

/**
 * This class represents a single YTD TaxPayment line item.<br>
 * Each YTD TaxPayment consists of a Law item and the payment amount.<br>
 * The Law item indicates the detail of tax payment - whether Federal or State,<br>
 * If state, then the name of the state, and also the type of tax payment (FUTA, SS, etc).
 *
 */
public class TaxPaymentLineItem implements Serializable {

	private static final long serialVersionUID = -705548867723641489L;

	private ObjectHandle objectHandle;

	/** numeric law id stored internally - Refer to {@code QboeLawIdConstants} */
	private String lawId; 
	
	private String paymentType;	//	// FORM940FUTA, COBRA,  CAUI,  CAETT, etc
	
	/** The actual tax payment amount. */
	private BigDecimal amount;
	
	public String getLawId() {
		return lawId;
	}

	public void setLawId(String lawId) {
		this.lawId = lawId;
	}

	public String getPaymentType() {
		return paymentType;
	}

	public void setPaymentType(String paymentType) {
		this.paymentType = paymentType;
	}

	public BigDecimal getAmount() {
		return amount;
	}

	public void setAmount(BigDecimal amount) {
		this.amount = amount;
	}

	public ObjectHandle getObjectHandle() {
		return objectHandle;
	}

	public void setObjectHandle(ObjectHandle objectHandle) {
		this.objectHandle = objectHandle;
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}	
	
}
