package com.intuit.sb.payroll.services.wages;

import javax.ejb.Remote;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;

/**
 * Interface to managing Wage related concepts supported by the Payroll
 * Composite Wage Services.
 */
@Remote
public interface WageService
{
	public static final String JNDI_NAME = "WageService/remote";

	/**
	 * Gets the CompensationItems for a Worker.
	 * 
	 * @param workerId the Worker's unique identifier.
	 * @return A collection of CompensationItems for the worker. Can be empty.
	 * @throws ValidationException if there is a constraint that is violated.
	 * @throws ServiceFailureException workerId is a well formed value but the worker does not exist
	 * (perhaps it has been deleted) or the indicated method argument is an invalid value (eg. null).
	 * The provided value can never, in any context, be valid and result in a successful call. 
	 * This is an application coding error and not something the client user can correct.
	 */
	public CompensationItem[] getCompensationItems(Long workerId) throws ServiceFailureException, ValidationException;
	
	/**
	 * Adds, updates or deletes an array of CompensationItems for an individual worker.
	 * The operation (Add, update or delete) is specific to each array element; it is not required 
	 * that all operations be the same for a single invocation of this API.  
	 * The objectHandle of a CompensationItem is ignored for add operations.
	 * @param workerId the Worker's unique identifier.
	 * @param compensationItemRequests specifies an array of SaveCompensationItemRequests for the worker. Each
	 * array member specifies the operation and the CompensationItem it applies to.
	 * @throws ValidationException if there is a constraint that is violated, example WorkerId is not well formed or
	 * the parameters does not have sufficient values 
	 * @throws ServiceFailureException the indicated method argument is an invalid value (eg. null).
	 * The provided value can never, in any context, be valid and result in a successful call. 
	 * This is an application coding error and not something the client user can correct.
	 * @return SaveCompensationItemResponse containing the updated worker and all the compensation items.
	 */
	public SaveCompensationItemResponse saveCompensationItems(Long workerId, SaveCompensationItemRequest[] compensationItemRequests)
		throws WorkflowException, ServiceFailureException, ValidationException;

}
