package com.intuit.sb.payroll.services.worker.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;
import com.intuit.sb.payroll.services.worker.Worker;

/**
 * Implements the GetWorkerById function exposed by the WorkerService.
 * 
 * @see WorkerServiceBean
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
class GetWorkerByIdImpl {
	/**
	 * Gets the Worker DTO for the specified worker id. The DTO is comprised of:
	 * <ul>
	 * <li>Properties from the Payroll worker</li>
	 * <li>If there is a Takehome application worker, it's properties
	 * are merged into the DTO too. If the application worker is not
	 * found the respective DTO properties are null</li>
	 * </ul>
	 * @param uniqueId The payroll worker unique ID.
	 * @return The Worker DTO.
	 * @throws ServiceFailureException
	 * @throws ValidationException 
	 */
	static Worker getWorkerById(Long uniqueId) throws ServiceFailureException, ValidationException {
		CommonValidators.validateId(uniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = getWorkerEntity(uniqueId);
		Worker worker = WorkerMapper.mapFromEntityToPojo(workerEntity);
		return worker;
	}

	private static com.intuit.sb.payroll.entity.model.Worker getWorkerEntity(Long uniqueId) 
			throws ServiceFailureException, ValidationException {
		com.intuit.sb.payroll.entity.model.Worker prWorker = null;
		WorkerComponent workerComponent = ServiceLocator.getInstance().getService(WorkerComponent.JNDI_NAME);
		prWorker = workerComponent.findWorker(uniqueId);
		
		return prWorker;
	}
}
