/**
 *
 */
package com.intuit.sb.payroll.services.workerpayments.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PaycheckUpdateRequest;

/**
 * @author rjohn2
 *
 * Utility class for performing PayrollRunSaveRequest validation
 *
 * Validates the PayrollRunSaveRequest passed in by the client
 * If any of the validations fails throws ValidationException */

class PaycheckRequestValidator
{
    /**
     * <p>Checks if {@link PaycheckRequest} is valid.</p>
     *
     * @param runRequest The value validation is being performed on.  A <code>null</code>
     * value is considered invalid.
     * @throws  ValidationException if the validation fails.
     */
    static void validate(PaycheckRequest request) throws ValidationException, ServiceFailureException
    {
    	if (request == null) {
    		throw new ServiceFailureException("PaycheckRequest can not be null ..");
    	}

    	//now validate the workerId is a valid UniqueId
    	Long workerId = request.getWorkerId();
    	
		CommonValidators.validateId(workerId);

    	//now validate the payrollrun period
    	PayrollRunPeriodValidator.validate(request.getPayrollRunPeriod());

    	//FIXME validate the CalcInputs
    	CalcInputs[] inputs = request.getCalcInputs();
    	CalcInputsParamValidator.validateCalcInputs(inputs);
    }

	public static void validate(PaycheckUpdateRequest request) throws ValidationException, ServiceFailureException {
    	if (request == null) {
    		throw new ServiceFailureException("PaycheckUpdateRequest can not be null ..");
    	}

    	Long paycheckId = request.getPaycheckId();
    	
		CommonValidators.validateId(paycheckId);
		
		// Now validate the PaycheckStatus
		if (null == request.getPaycheckStatus()) {
    		throw new ServiceFailureException("PaycheckStatus can not be null ..");
		}
	}
}
