package com.intuit.sb.payroll.services.benefits;

import java.math.BigDecimal;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.services.common.ObjectHandle;

public class BenefitsServiceTest extends BenefitsServiceTestBase {

    protected static BenefitsService sBenefitsService;
	
    @BeforeClass
    public static void initializeBenefitsServiceTestClass()
    {
    	sBenefitsService = qaBootstrap.getEjbFromClass(BenefitsService.class);
    }
    
	@Test
	public void testFindEmployerBenefits() throws Exception {
		// Get all health benefits on the employer
		HealthBenefitTemplate[] benefitTemplates = sBenefitsService.findEmployerHealthBenefits();
		
		AssertJUnit.assertNotNull(benefitTemplates);
		
		// At this time, there is only one being created in seed data
		AssertJUnit.assertEquals(sDefaultHealthBenefitType, benefitTemplates[0].getType());
		AssertJUnit.assertEquals(sDefaultHealthBenefitAmount, benefitTemplates[0].getRate());
	}
	
	@Test
	public void testAddHealthBenefit() throws Exception {
		// Get all health benefits on the employer
		HealthBenefitTemplate[] benefitTemplates = sBenefitsService.findEmployerHealthBenefits();
		
		AssertJUnit.assertNotNull(benefitTemplates);
		
		// At this time, there is only one being created in seed data
		AssertJUnit.assertEquals(sDefaultHealthBenefitType, benefitTemplates[0].getType());
		AssertJUnit.assertEquals(sDefaultHealthBenefitAmount, benefitTemplates[0].getRate());

		// Create a test worker
		ObjectHandle workerHandle = getCreatedPayrollReadyWorker();
		
		// Create default benefit from template
		HealthBenefit benefit = new HealthBenefit();
		benefit.setType(benefitTemplates[0].getType());
		benefit.setRate(benefitTemplates[0].getRate());
		
		HealthBenefit[] benefits = new HealthBenefit[1];
		benefits[0] = benefit;
		// Add benefit
		HealthBenefitResponse addResponseBenefits = sBenefitsService.addHealthBenefits(workerHandle, benefits);
		AssertJUnit.assertNotNull(addResponseBenefits);
		
		// Get benefits and compare 
		HealthBenefit[] workerBenefits = sBenefitsService.findAllHealthBenefits(workerHandle);
		
		AssertJUnit.assertNotNull(workerBenefits);
		
		// Compare benefits
		AssertJUnit.assertEquals(addResponseBenefits.getBenefits()[0].getType(), workerBenefits[0].getType());
		AssertJUnit.assertEquals(addResponseBenefits.getBenefits()[0].getRate(), workerBenefits[0].getRate());
		
		// Update benefit
		BigDecimal newRate = new BigDecimal("123.45");
		workerBenefits[0].setRate(newRate);
		HealthBenefitResponse updateResponseBenefits = sBenefitsService.updateHealthBenefits(workerHandle, workerBenefits);
		
		AssertJUnit.assertNotNull(updateResponseBenefits);
		AssertJUnit.assertEquals(newRate, updateResponseBenefits.getBenefits()[0].getRate());
		
		// Delete the benefits from the worker
		HealthBenefitResponse deleteResponseBenefits = sBenefitsService.deleteHealthBenefits(workerHandle, updateResponseBenefits.getBenefits());
		
		AssertJUnit.assertNotNull(deleteResponseBenefits);
		AssertJUnit.assertEquals(0, deleteResponseBenefits.getBenefits().length);
		
		//cleanup the database from the objects created for this test
		cleanUpEIWorker(workerHandle.getId());

	}
}
