package com.intuit.sb.payroll.services.common;


import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.common.impl.PhysicalAddressMapper;
import com.intuit.sb.payroll.services.employer.CompanyLegalTaxFilingInfo;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.services.employer.impl.CompanyLegalTaxFilingInfoMapper;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Tests the CompanyLegalTaxFilingInfoMapper.
 *
 * $Author: astec $ 
 * $Revision: $
 * $DateTime: $
 */
public class CompanyLegalTaxFilingInfoMapperTest extends EmployerServiceTestBase 
{
	private static final String legalName = "INTUIT";
	private static final LegalStructureEnum companyLegalType = LegalStructureEnum.SCORP;
	private static final IndustryTypeEnum industryType = IndustryTypeEnum.CHURCH;
	private static final String legalPhoneNr = "650-944-6000";
	private static final String legalExt = "7777";
	private static final String addressLine1 = "A1";
	private static final String addressLine2 = "A2";
	private static final String city = "City";
	private static final String country = "United States of America";
	private static final String regionOrState = "CA";
	private static final String zipOrPostalCode = "94043";
	private static final Boolean legalAddressSameAsBusinessAddress = false;
	 
	private PhysicalAddress CreatePhysicalAddress() 
	{
		PhysicalAddress physAddress = new PhysicalAddress();
		physAddress.setAddressLine1(addressLine1);
		physAddress.setAddressLine2(addressLine2);
		physAddress.setCity(city);
		physAddress.setCountry(country);
		physAddress.setStateOrRegion(regionOrState);
		physAddress.setZipOrPostalCode(zipOrPostalCode);
		
		return physAddress;
	}
	
	private USPostalAddress CreatePostalAddress() 
	{
		USPostalAddress postAddress = new USPostalAddressImpl();
		postAddress.setAddressLine1(addressLine1);
		postAddress.setAddressLine2(addressLine2);
		postAddress.setCity(city);
		postAddress.setCountry(country);
		postAddress.setRegionOrState(regionOrState);
		postAddress.setZipOrPostalCode(zipOrPostalCode);
		
		return postAddress;
	}
	
	private void CheckPhysicalAddress(PhysicalAddress physAddr) 
	{
		AssertJUnit.assertEquals("addressLine1 does not match", addressLine1, physAddr.getAddressLine1());
		AssertJUnit.assertEquals("addressLine2 does not match", addressLine2, physAddr.getAddressLine2());
		AssertJUnit.assertEquals("city does not match", city, physAddr.getCity());
		AssertJUnit.assertEquals("country does not match", country, physAddr.getCountry());
		AssertJUnit.assertEquals("regionOrState does not match", regionOrState, physAddr.getStateOrRegion());
		AssertJUnit.assertEquals("zipOrPostalCode does not match", zipOrPostalCode, physAddr.getZipOrPostalCode());

		return;
	}

	@Test
	public void OneGoodBogusTest() throws Exception 
	{
		return;
	}
	
	@Test
	public void testDtoToEntityMapping() throws Exception 
	{	
		CompanyLegalTaxFilingInfo dto = new CompanyLegalTaxFilingInfo();
		
		dto.setLegalName(legalName);
		dto.setCompanyLegalType(companyLegalType);
		dto.setIndustryType(industryType);
		dto.setLegalPhone(legalPhoneNr);
		dto.setExtn(legalExt);

		PhysicalAddress dtoAddress = CreatePhysicalAddress();
		dto.setLegalAddress(dtoAddress);
		
		dto.setLegalAddressSameAsBusinessAddress(legalAddressSameAsBusinessAddress);
				
		Employer employer = CompanyLegalTaxFilingInfoMapper.mapCompanyInfo(dto);
		
		AssertJUnit.assertEquals("legalName does not match", legalName, employer.getLegalName());
		AssertJUnit.assertEquals("legalPhoneNr does not match", legalPhoneNr, employer.getLegalPhone().getFreeFormNumber());
		AssertJUnit.assertEquals("legalExt does not match", legalExt, employer.getLegalPhone().getExtension());
		AssertJUnit.assertEquals("legalStructure does not match", companyLegalType, employer.getLegalStructure());
		AssertJUnit.assertEquals("IndustryType does not match", industryType, employer.getIndustryType());
		AssertJUnit.assertEquals("legalAddressSameAsBusinessAddress does not match", legalAddressSameAsBusinessAddress.booleanValue(), employer.isLegalAndBusAddressSame());

		USPostalAddress entityLegalAddress = AddressConvertor.getUSFromGlobal(employer.getLegalAddress(), new USPostalAddressImpl());
		PhysicalAddress physicalLegalAddress = PhysicalAddressMapper.mapAddress(entityLegalAddress);

		CheckPhysicalAddress(physicalLegalAddress);

		return;
	}

	
	@Test
	public void testEntityToDtoMapping() throws Exception 
	{	
		Employer employer = new Employer();
		
		USPostalAddress entityAddress = CreatePostalAddress();

		TelephoneNumber legalPhone = new TelephoneNumber();
		legalPhone.setFreeFormNumber(legalPhoneNr);
		legalPhone.setExtension(legalExt);

		employer.setLegalName(legalName);	
		employer.setLegalAddress(
				AddressConvertor.getGlobalFromUS(entityAddress, new GlobalPostalAddress()));
		employer.setIndustryType(industryType);
		employer.setLegalStructure(companyLegalType);
		employer.setLegalPhone(legalPhone);
		employer.setLegalAndBusAddressSame(legalAddressSameAsBusinessAddress);
		
		CompanyLegalTaxFilingInfo dto = CompanyLegalTaxFilingInfoMapper.mapCompanyInfo(employer);
		
		AssertJUnit.assertEquals("legalName does not match", legalName, dto.getLegalName());
		AssertJUnit.assertEquals("legalPhoneNr does not match", legalPhoneNr, dto.getLegalPhone());
		AssertJUnit.assertEquals("legalExt does not match", legalExt, dto.getExtn());
		AssertJUnit.assertEquals("legalStructure does not match", companyLegalType, dto.getCompanyLegalType());
		AssertJUnit.assertEquals("IndustryType does not match", industryType, dto.getIndustryType());
		AssertJUnit.assertEquals("legalAddressSameAsBusinessAddress does not match", legalAddressSameAsBusinessAddress.booleanValue(), dto.isLegalAddressSameAsBusinessAddress());

		PhysicalAddress physicalLegalAddress = dto.getLegalAddress();
		CheckPhysicalAddress(physicalLegalAddress);
		
		return;
	}	
}
