package com.intuit.sb.payroll.services.common;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSContactPersonTypeEnum;
import com.intuit.sb.payroll.services.common.impl.EmployerContactMapper;
import com.intuit.sb.payroll.services.employer.EmployerContact;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

public class EmployerContactMapperTest extends EmployerServiceTestBase {


	Boolean equal(EmployerContactPerson entityContact, EmployerContact dtoContact)
	{
		AssertJUnit.assertEquals("Email does not match", entityContact.getEmailAddress().getAddress(), dtoContact.getEmail());
		AssertJUnit.assertEquals("Contact fax nr does not match", entityContact.getFaxPhone().getFreeFormNumber(), dtoContact.getFax());
		AssertJUnit.assertEquals("Contact phone nr does not match", entityContact.getWorkPhone().getFreeFormNumber(), dtoContact.getPhone());
		AssertJUnit.assertEquals("Contact phone ext does not match", entityContact.getWorkPhone().getExtension(), dtoContact.getExtn());
		AssertJUnit.assertEquals("First name does not match", entityContact.getPersonName().getFirstName(), dtoContact.getFirstName());
		AssertJUnit.assertEquals("Last name does not match", entityContact.getPersonName().getLastName(), dtoContact.getLastName());
		AssertJUnit.assertEquals("IsOwner flag does not match", entityContact.isOwner().booleanValue(), dtoContact.isOwner());
		AssertJUnit.assertEquals("Role title does not match", entityContact.getRoleTitle(), dtoContact.getTitle());
		// address structures are not the same thus fields need to be tested individually 
		GlobalPostalAddress global = entityContact.getMailingAddress();
		USPostalAddress mailingAddress = AddressConvertor.getUSFromGlobal(global, new USPostalAddressImpl());
		AssertJUnit.assertEquals("Mailing address line1 does not match", mailingAddress.getAddressLine1(), dtoContact.getAddress().getAddressLine1());
		AssertJUnit.assertEquals("Mailing address line2 does not match", mailingAddress.getAddressLine2(), dtoContact.getAddress().getAddressLine2());
		AssertJUnit.assertEquals("Mailing address city does not match", mailingAddress.getCity(), dtoContact.getAddress().getCity());
		AssertJUnit.assertEquals("Mailing address country does not match", mailingAddress.getCountry(), dtoContact.getAddress().getCountry());
		AssertJUnit.assertEquals("Mailing address state does not match", mailingAddress.getRegionOrState(), dtoContact.getAddress().getStateOrRegion());
		AssertJUnit.assertEquals("Mailing address zip does not match", mailingAddress.getZipOrPostalCode(), dtoContact.getAddress().getZipOrPostalCode());
		
		return true;
	}
	
	@Test
	public void testBasicDtoToEntityMapping() throws Exception {
		
		EmployerContact dtoContact = new EmployerContact();
		
		PhysicalAddress dtoAddress = PojoUtils.createUSCAPhysicalAddress();
		dtoContact.setAddress(dtoAddress);

		dtoContact.setEmail(PojoUtils.email);
		
		dtoContact.setFirstName(PojoUtils.firstName);
		dtoContact.setLastName(PojoUtils.lastName);
		dtoContact.setOwner(PojoUtils.owner);
//		if(PojoUtils.owner)
//		{
//			dtoContact.setEmployerContactType(EmployerContactType.EmployerContact);
//		}	
		dtoContact.setPhone(PojoUtils.contactPhoneNr);
		dtoContact.setExtn(PojoUtils.contactExt);
		dtoContact.setFax(PojoUtils.contactFaxNr);

		dtoContact.setSameAsCompanyAddress(PojoUtils.sameAsCompanyAddress);
		dtoContact.setTitle(PojoUtils.title);
			
/*		dtoContact.setUniqueId(PojoUtils.uniqueId);
		dtoContact.setVersion(PojoUtils.version);
*/		
		// test mappings
		EmployerContactPerson entityContact = EmployerContactMapper.mapEmployerContact(dtoContact).getEmployerContact();
		
		// check correctness of mapping
		equal(entityContact, dtoContact);
	}

	
	@Test
	public void testBasicEntityToDtoMapping() throws Exception {
		
		EmployerContactPerson entityContact = new EmployerContactPerson();
		
		EmailAddress emailAddress = new EmailAddress();
		emailAddress.setAddress(PojoUtils.email);
		entityContact.setEmailAddress(emailAddress);
		
		TelephoneNumber contactPhone = new TelephoneNumber();
		contactPhone.setFreeFormNumber(PojoUtils.contactPhoneNr);
		contactPhone.setExtension(PojoUtils.contactExt);
		entityContact.setWorkPhone(contactPhone);
		
		TelephoneNumber faxPhone = new TelephoneNumber();
		faxPhone.setFreeFormNumber(PojoUtils.contactFaxNr);
		// no ext on fax phones
		entityContact.setFaxPhone(faxPhone);
		
		entityContact.setOwner(PojoUtils.owner);
		
		PersonName personName = new PersonName();
		personName.setFirstName(PojoUtils.firstName);
		personName.setLastName(PojoUtils.lastName);
		entityContact.setPersonName(personName);
		
		entityContact.setRoleTitle(PojoUtils.title);
		
		USPostalAddress entityAddress = PojoUtils.createUSCAPostalAddress();
		entityContact.setMailingAddress(
				AddressConvertor.getGlobalFromUS(entityAddress, new GlobalPostalAddress()));
		
		PTSContactPerson ptsContactPerson = new PTSContactPerson();
		ptsContactPerson.setContactPersonType(PTSContactPersonTypeEnum.PRIMARY_CONTACT);
		ptsContactPerson.setEmployerContact(entityContact);
		// test mappings
		EmployerContact dtoContact = EmployerContactMapper.mapEmployerContact(ptsContactPerson);

		// check correctness of mapping
		equal(entityContact, dtoContact);
	}
	
}
