package com.intuit.sb.payroll.services.common;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertFalse;

import org.testng.annotations.Test;


/**
 * Test typesafe enum implementation using a inner enum classes.
 */
public class EnumTest 
{

	@SuppressWarnings("deprecation")
	@Test
	public void testEnum()
	{
		// Test equality
		assertEquals(BikeEnum.Track, BikeEnum.Track);
		assertEquals(BikeEnum.Road, BikeEnum.Road);
		assertEquals(BikeEnum.Mountain, BikeEnum.Mountain);
		
		// Test inequality
		assertFalse(BikeEnum.Track.equals(BikeEnum.Road));
		assertFalse(BikeEnum.Track.equals(BikeEnum.Mountain));
		
		assertFalse(BikeEnum.Road.equals(BikeEnum.Track));
		assertFalse(BikeEnum.Road.equals(BikeEnum.Mountain));

		assertFalse(BikeEnum.Mountain.equals(BikeEnum.Track));
		assertFalse(BikeEnum.Mountain.equals(BikeEnum.Road));
		
		assertFalse(BikeEnum.Track.equals(null));
		assertFalse(BikeEnum.Mountain.equals(ColorEnum.Blue));		
		
		// Test equality using no-arg constructor (trying to simulate 
		// what LCDS does.  Since these use deprecated setters (which
		// regular code should not call, I added a SuppressWarnings
		// annotation above.
		BikeEnum trackBike = new BikeEnum();
		trackBike.setName("Track");
		trackBike.setOrdinal(0);
		assertEquals(trackBike, BikeEnum.Track);
		
		BikeEnum roadBike = new BikeEnum();
		roadBike.setName("Road");
		roadBike.setOrdinal(1);
		assertEquals(roadBike, BikeEnum.Road);
		
		BikeEnum mountainBike = new BikeEnum();
		mountainBike.setName("Mountain");
		mountainBike.setOrdinal(2);
		assertEquals(mountainBike, BikeEnum.Mountain);
		
		// Test inequality using no-arg constructor
		trackBike.setOrdinal(1);
		assertFalse(trackBike.equals(BikeEnum.Track));
		
		trackBike.setOrdinal(2);
		assertFalse(trackBike.equals(BikeEnum.Track));
		
		// Test Comparable
		assertEquals(0, BikeEnum.Track.compareTo(BikeEnum.Track));
		assertEquals(0, BikeEnum.Road.compareTo(BikeEnum.Road));
		assertEquals(0, BikeEnum.Mountain.compareTo(BikeEnum.Mountain));
		
		assertEquals(1, BikeEnum.Road.compareTo(BikeEnum.Track));
		assertEquals(1, BikeEnum.Mountain.compareTo(BikeEnum.Road));
		assertEquals(2, BikeEnum.Mountain.compareTo(BikeEnum.Track));
		
		assertEquals(-1, BikeEnum.Track.compareTo(BikeEnum.Road));
		assertEquals(-2, BikeEnum.Track.compareTo(BikeEnum.Mountain));
		assertEquals(-1, BikeEnum.Road.compareTo(BikeEnum.Mountain));
	}
	
	/**
	 * Test comparing an enum to an object of a different class.
	 */
	@Test (expectedExceptions=ClassCastException.class)
	public void testCompareToDifferentClass()
	{
		BikeEnum.Road.compareTo(new Object());
	}
	
	
	/**
	 * Test comparing to a different enum.  This does not test the case
	 * where an enum is extended by another enum.
	 */
	@Test (expectedExceptions=ClassCastException.class)
	public void testCompareToDifferentEnum()
	{
		BikeEnum.Track.compareTo(ColorEnum.Blue);
	}
	
	/** Enum for types of bikes */
	public static class BikeEnum extends EnumBase 
	{
		private static final long serialVersionUID = -1656412028762076875L;
		
		public static final BikeEnum Track = new BikeEnum("Track", 0);
		public static final BikeEnum Road = new BikeEnum("Road", 1);
		public static final BikeEnum Mountain = new BikeEnum("Mountain", 2);
		
		/** Constructor should not be called other than by this class itself. */
		private BikeEnum(String name, int ordinal)
		{
			super(name, ordinal);
		}

		/** 
		 * Public no-arg constructor is required by LCDS.  Without it, you get an exception
		 * with the following message: "Types must have a public, no arguments constructor".
		 * This should never be used by hand-written code.
		 * @deprecated
		 */
		public BikeEnum()
		{
			super();
		}
	}

	/** Enum for colors */
	public static class ColorEnum extends EnumBase
	{
		private static final long serialVersionUID = -4003082127241607377L;
		
		public static final ColorEnum Red = new ColorEnum("Red", 0);
		public static final ColorEnum Blue = new ColorEnum("Blue", 1);
		public static final ColorEnum Green = new ColorEnum("Green", 2);
		
		/** Constructor should not be called other than by this class itself. */
		private ColorEnum(String name, int ordinal)
		{
			super(name, ordinal);			
		}
		
		/** 
		 * Public no-arg constructor is required by LCDS.  Without it, you get an exception
		 * with the following message: "Types must have a public, no arguments constructor".
		 * This should never be used by hand-written code.
		 * @deprecated
		 */
		public ColorEnum()
		{
			super();
		}

	}
}
