package com.intuit.sb.payroll.services.common;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.services.common.impl.WorkLocationMapper;

public class WorkLocationMapperTest extends BasicTestBase {
	
	private static final String WORK_LOCATION_NAME = "WLNAME";
	private static final String PHONE_EXT = "123";
	private static final String PHONE_NUMBER = "650-111-2233";

	/**
	 * Basic test to ensure field mapping is correct.  Does not cover the PhysicalAddress
	 * mapping since that's covered in another test suite.
	 * 
	 * @throws Exception
	 */
	@Test
	public void mapFromDtoToEntityTest() throws Exception {
		com.intuit.sb.payroll.services.common.WorkLocation dto = new com.intuit.sb.payroll.services.common.WorkLocation();
		dto.setName(WORK_LOCATION_NAME);
		dto.setAddress(PojoUtils.createUSCAPhysicalAddress());
		dto.setExtn(PHONE_EXT);
		dto.setPhoneNumber(PHONE_NUMBER);
		
		com.intuit.sb.payroll.entity.model.WorkLocation entity = WorkLocationMapper.mapWorkLocation(dto);
		
		AssertJUnit.assertEquals("WorkLocation Name does not match", WORK_LOCATION_NAME, entity.getName());
		AssertJUnit.assertEquals("TelephoneNumber Extension does not match", PHONE_EXT, entity.getMainPhone().getExtension());
		AssertJUnit.assertEquals("TelephoneNumber Number does not match", PHONE_NUMBER, entity.getMainPhone().getFreeFormNumber());
	}
	
	/**
	 * Basic test to ensure field mapping is correct.  Does not cover the PhysicalAddress
	 * mapping since that's covered in another test suite.
	 * 
	 * @throws Exception
	 */
	@Test
	public void mapFromEntityToDtoTest() throws Exception {
		com.intuit.sb.payroll.entity.model.WorkLocation entity = WorkerTestUtils.createWorkLocation(WORK_LOCATION_NAME);
		TelephoneNumber mainPhone = new TelephoneNumber();
		mainPhone.setFreeFormNumber(PHONE_NUMBER);
		mainPhone.setExtension(PHONE_EXT);
		entity.setMainPhone(mainPhone);
		entity.setName(WORK_LOCATION_NAME);
		entity.setTaxResidenceAddress(
				AddressConvertor.getGlobalFromUS(
						WorkerTestUtils.createUSPhysicalAddress(), new GlobalPostalAddress()));
		
		com.intuit.sb.payroll.services.common.WorkLocation dto = WorkLocationMapper.mapWorkLocation(entity);
		AssertJUnit.assertEquals("WorkLocation Name does not match", WORK_LOCATION_NAME, dto.getName());
		AssertJUnit.assertEquals("TelephoneNumber Extension does not match", PHONE_EXT, dto.getExtn());
		AssertJUnit.assertEquals("TelephoneNumber Number does not match", PHONE_NUMBER, dto.getPhoneNumber());
	}
}
