package com.intuit.sb.payroll.services.employer;

import java.util.Date;

import org.testng.Assert;
import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandler;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerFactory;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;
import com.intuit.sb.payroll.services.common.BankAccount;
import com.intuit.sb.payroll.services.common.BankAccountStatusEnum;

public class EmployerBankAccountTest extends EmployerServiceTestBase {
	

	
	 /**
	  * Test Verifying the BankAccount Status after it is added and when it receives any notification from PTS
	  * @throws PTSNotificationException 
	  */ 
	@Test
	public void testBankAccountStatus() throws ServiceFailureException, ValidationException, PTSNotificationException{
		EmployerPTSWorkflowComponent sEmployerPTSWorkFlowComponent = qaBootstrap.getEjbFromClass(EmployerPTSWorkflowComponent.class);
		
		//create a bank Account
		BankAccount[] bankAccounts=new BankAccount[1];		
		BankAccount bankAccount = createBankAccount(BankAccountTypeEnum.SAVINGS);
		bankAccounts[0] = bankAccount;
		
		//add bankAccount to the employer
		BankAccount[] savedBankAccounts = sEmployerService.addBankAccounts(bankAccounts);
		
		//get the bankaccount saved
		BankAccount bankAccountRetrieved = getBankAccountBasedOnAccountNo(savedBankAccounts, bankAccount.getAccountNo());
		Long id = bankAccountRetrieved.getObjectHandle().getId();
		
		//verify the bankAccount Status
		AssertJUnit.assertEquals("BankAccount Status is not Mapped correctly", 
							BankAccountStatusEnum.BANKACCOUNT_UNDER_VERIFICATION, bankAccountRetrieved.getBankAccountStatus());
		
		//create a PTSNotificationRequest 
		PTSNotificationRequest ptsNotificationRequest = createPTSNotificationRequest(PTSNotificationTypeEnum.BANK_ACCOUNT_NOTIFICATION,
				com.intuit.sb.payroll.pts.entity.BankAccountStatusEnum.BANKACCOUNT_VERIFICATION_FAILED_RECEIVED);
		ptsNotificationRequest.addNotificationAttributes("SourceBankAccountId", id.toString());
		
		// send a PTSNotification indicating that the bankAccount verification is rejected
		PTSNotificationTypeEnum ptsNotificationType = ptsNotificationRequest.getNotificationType();
		StatusUpdateHandler statusUpdateHandler = StatusUpdateHandlerFactory.getStatusUpdateHandler(ptsNotificationType);
		statusUpdateHandler.updateStatus(sEmployerPTSWorkFlowComponent.getPTSEmployer(), ptsNotificationRequest);
			
		//retrieve all the bankAccounts of the current employer
		bankAccounts = sEmployerService.getBankAccounts();
		
		//iterate over the bankAccounts retrieved to get the bankAccount whose id is bankAccountId
		bankAccountRetrieved = null;
		for (BankAccount ba:bankAccounts){
			if(ba.getObjectHandle().getId() == id)
			{
				bankAccountRetrieved = ba;
				break;
			}
		}
	
		// verify the retrieved bankAccountStatus
		AssertJUnit.assertEquals("BankAccount Status is not Mapped correctly",BankAccountStatusEnum.BANKACCOUNT_VERIFICATION_FAILED_RECEIVED, bankAccountRetrieved.getBankAccountStatus());
		
		// send a PTSNotification indicating that the bankAccount is Verified
		ptsNotificationRequest.setNotificationStatus(com.intuit.sb.payroll.pts.entity.BankAccountStatusEnum.BANKACCOUNT_VERIFIED_RECEIVED);
		statusUpdateHandler = StatusUpdateHandlerFactory.getStatusUpdateHandler(ptsNotificationType);
		Date statusUpdateDate = new Date();
		ptsNotificationRequest.setTimestamp(statusUpdateDate);
		statusUpdateHandler.updateStatus(sEmployerPTSWorkFlowComponent.getPTSEmployer(), ptsNotificationRequest);
		
		//retrieve all the bankAccounts of the current employer
		bankAccounts = sEmployerService.getBankAccounts();
		
		//iterate over the bankAccounts retrieved to get the bankAccount whose id is bankAccountId
		bankAccountRetrieved = null;
		for (BankAccount ba:bankAccounts){
			if(ba.getObjectHandle().getId() == id)
			{
				bankAccountRetrieved = ba;
				break;
			}
		}
		// verify the retrieved bankAccountStatus
		AssertJUnit.assertEquals("BankAccount Status is not Mapped correctly",bankAccountRetrieved.getBankAccountStatus().name(),
				BankAccountStatusEnum.BANKACCOUNT_VERIFIED_RECEIVED.name());
		
		// Verify that the BankAccount Entity date has now been updated
		com.intuit.sb.payroll.entity.model.BankAccount bankEntity = entityService.findById(com.intuit.sb.payroll.entity.model.BankAccount.class, id.longValue());
		Assert.assertEquals(bankEntity.getDateLastVerified(), statusUpdateDate, "Last Verified Date for a verfifed Bank Account does not match");
	}
	
	// creates a PTSNotificationRequest
	private PTSNotificationRequest createPTSNotificationRequest(PTSNotificationTypeEnum ptsNotificationType, 
									com.intuit.sb.payroll.pts.entity.BankAccountStatusEnum notificationStatus){
		PTSNotificationRequest ptsNotificationRequest = new PTSNotificationRequest();
		ptsNotificationRequest.setNotificationType(ptsNotificationType);
		ptsNotificationRequest.setNotificationStatus(notificationStatus);
		return ptsNotificationRequest;
	}

}
