package com.intuit.sb.payroll.services.employer;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

public class EmployerContactsTest extends EmployerServiceTestBase {
    
    
    /**
     * Adds a new EmployerContact and retrieves it back to see it is created properly
     * @throws Exception 
     */
    @Test
    public void addEmployerContactTest() throws Exception
    {
    	String primaryContactFirstName = "John";
    	
		EmployerContact newPC = createEmployerContact(primaryContactFirstName, "Adams", "415-321-1212", EmployerContactType.PrimaryContact);
		EmployerContact[] contacts = new EmployerContact[1];
    	contacts[0] = newPC;
    	
		//Check that we have the same number of primaryContacts.
    	EmployerContact[] initcontactsFromDb = sEmployerService.getEmployerContacts();
		
    	
    	EmployerContact[] expectedcontacts = new EmployerContact[initcontactsFromDb.length + 1];
    	
    	for (int i = 0; i < initcontactsFromDb.length; i++) {
    		expectedcontacts[i] = initcontactsFromDb[i];
		}
    	expectedcontacts[initcontactsFromDb.length] = newPC;
    	
    	EmployerContact[] contactsFromAdd = sEmployerService.addEmployerContacts(contacts);
    	
    	AssertJUnit.assertTrue("Different Number of Contacts came back", contactsFromAdd.length == initcontactsFromDb.length + 1);
    	//Find the new guy
    	boolean found = false;
    	for (EmployerContact primaryContact : contactsFromAdd) {
			if (primaryContact.getFirstName().equals(primaryContactFirstName)) {
				found = true;
				newPC = primaryContact;
				break;
			}
		}
    	AssertJUnit.assertTrue("Did not find the EmployerContact named addEmployerContactTest", found);
    	
    	verifyEmployerContacts(expectedcontacts, contactsFromAdd,true);
    	// Clean up not required as this txn will be rolled back
    }
    
    
    /**
     * Deletes a EmployerContact and checks it is deleted
     */
    @Test
    public void deleteEmployerContactTest() throws Exception
    {
    	// Same setup as the add, but now we assert the dude is not in the DB
    	String primaryContactFirstName = "Robert";
		EmployerContact newPC = createEmployerContact(primaryContactFirstName, "Mugabe", "650-555-6666", 
								EmployerContactType.SecondaryContact);
		EmployerContact newSC = createEmployerContact("john2", "Mugabe1", "650-555-6666", 
				EmployerContactType.PrimaryContact);
    	EmployerContact[] contacts = new EmployerContact[2];
    	contacts[0] = newPC;
    	contacts[1] = newSC;
    	EmployerContact[] contactsFromAdd = sEmployerService.addEmployerContacts(contacts);

    	//Find the new guy
    	boolean found = false;
    	for (EmployerContact primaryContact : contactsFromAdd) {
			if (primaryContact.getFirstName().equals(primaryContactFirstName)) {
				found = true;
				newPC = primaryContact;
				break;
			}
		}
    	AssertJUnit.assertTrue("Did not find the EmployerContact named deleteEmployerContactTest", found);
    	
    	// Clean up
    	EmployerContact[] contactsToDelete = new EmployerContact[1];
    	contactsToDelete[0] = newPC;
    	sEmployerService.deleteEmployerContacts(contactsToDelete);
    	
    	//Check that we have the same number of contacts.
    	EmployerContact[] prunedContactsFromDb = sEmployerService.getEmployerContacts();
    	
    	//We should not find our friend in here.
    	boolean deleted = true;
    	for (EmployerContact primaryContact : prunedContactsFromDb) {
			if (primaryContact.getFirstName().equals(primaryContactFirstName)) {
				deleted = false;
				break;
			}
		}
    	AssertJUnit.assertTrue("The EmployerContact named deleteEmployerContactTest was not deleted", deleted);
    	
    }
    
    /**
     * Updates a EmployerContact and verifies the update succeeded.  
     */
    @Test
    public void updateEmployerContactTest() throws Exception
    {
    	String primaryContactName = "updateEmployerContactTest";
		EmployerContact newPC = createEmployerContact(primaryContactName, "lastName", "650-555-9999", 
									EmployerContactType.SecondaryContact);
    	EmployerContact[] contacts = new EmployerContact[1];
    	contacts[0] = newPC;
    	EmployerContact[] contactsFromUpdate = sEmployerService.addEmployerContacts(contacts);

    	//Find the new guy
    	boolean found = false;
    	for (EmployerContact primaryContact : contactsFromUpdate) {
			if (primaryContact.getFirstName().equals(primaryContactName)) {
				found = true;
				newPC = primaryContact;
				break;
			}
		}
    	AssertJUnit.assertTrue("Did not find the EmployerContact named updateEmployerContactTest", found);
    	
    	//Update the TelephoneNumber Number
    	String updatedPhoneNumber = "415-555-0000";
		newPC.setPhone(updatedPhoneNumber);

    	EmployerContact[] contactsToUpdate = new EmployerContact[1];
    	contactsToUpdate[0] = newPC;
    	EmployerContact[] updatedContacts = sEmployerService.updateEmployerContacts(contactsToUpdate);
    	
    	//Find the updated guy
    	boolean updated = false;
    	for (EmployerContact primaryContact : updatedContacts) {
			if (primaryContact.getFirstName().equals(primaryContactName)) {
				newPC = primaryContact;
				AssertJUnit.assertEquals("The TelephoneNumber Number was not updated", updatedPhoneNumber, newPC.getPhone());
				updated = true;
				break;
			}
		}
    	AssertJUnit.assertTrue("The EmployerContact named updateEmployerContactTest was not updated", updated);
    	
    	// Clean up not required as this txn will be rolled back
    }
}
