package com.intuit.sb.payroll.services.employer;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.WorkLocation;

public class EmployerWorkLocationsTest extends EmployerServiceTestBase {
   
    
    /**
     * Adds a new WorkLocation and retrieves it back to see it is created properly
     * @throws Exception 
     */
    @Test
    public void addWorkLocationTest() throws Exception
    {

    	String workLocationName = "addWorkLocationTest";
		WorkLocation newWL = createWorkLocation(workLocationName, "650-555-1212");
    	WorkLocation[] locations = new WorkLocation[1];
    	locations[0] = newWL;
    	WorkLocation[] locationsFromAdd = sEmployerService.addWorkLocations(locations).getWorkLocations();

    	//Check that we have the same number of worklocations.
    	WorkLocation[] locationsFromDb = sEmployerService.getWorkLocations();
    	
    	AssertJUnit.assertTrue("Different Number of Locations came back", locationsFromAdd.length == locationsFromDb.length);
    	//Find the new guy
    	boolean found = false;
    	for (WorkLocation workLocation : locationsFromDb) {
			if (workLocation.getName().equals(workLocationName)) {
				found = true;
				newWL = workLocation;
				break;
			}
		}
    	AssertJUnit.assertTrue("Did not find the WorkLocation named addWorkLocatonTest", found);
    	
    	// Clean up
    	WorkLocation[] locationsToDelete = new WorkLocation[1];
    	locationsToDelete[0] = newWL;
    	sEmployerService.deleteWorkLocations(locationsToDelete);
    }
    
    
    /**
     * Deletes a WorkLocation and checks it is deleted
     */
    @Test
    public void deleteWorkLocationTest() throws Exception
    {
    	// Same setup as the add, but now we assert the dude is not in the DB
    	String workLocationName = "deleteWorkLocationTest";
		WorkLocation newWL = createWorkLocation(workLocationName, "650-555-6666");
    	WorkLocation[] locations = new WorkLocation[1];
    	locations[0] = newWL;
    	WorkLocation[] locationsFromAdd = sEmployerService.addWorkLocations(locations).getWorkLocations();

    	//Check that we have the same number of worklocations.
    	WorkLocation[] locationsFromDb = sEmployerService.getWorkLocations();

    	AssertJUnit.assertTrue("Different Number of Locations came back", locationsFromAdd.length == locationsFromDb.length);
    	
    	//Find the new guy
    	boolean found = false;
    	for (WorkLocation workLocation : locationsFromDb) {
			if (workLocation.getName().equals(workLocationName)) {
				found = true;
				newWL = workLocation;
				break;
			}
		}
    	AssertJUnit.assertTrue("Did not find the WorkLocation named deleteWorkLocatonTest", found);
    	
    	// Clean up
    	WorkLocation[] locationsToDelete = new WorkLocation[1];
    	locationsToDelete[0] = newWL;
    	sEmployerService.deleteWorkLocations(locationsToDelete);
    	
    	//Check that we have the same number of worklocations.
    	WorkLocation[] prunedLocationsFromDb = sEmployerService.getWorkLocations();
    	
    	//We should not find our friend in here.
    	boolean deleted = true;
    	for (WorkLocation workLocation : prunedLocationsFromDb) {
			if (workLocation.getName().equals(workLocationName)) {
				deleted = false;
				break;
			}
		}
    	AssertJUnit.assertTrue("The WorkLocation named deleteWorkLocatonTest was not deleted", deleted);
    	
    }
    
    /**
     * Updates a WorkLocation and verifies the update succeeded.  
     */
    @Test
    public void updateWorkLocationTest() throws Exception
    {
    	String workLocationName = "updateWorkLocationTest";
		WorkLocation newWL = createWorkLocation(workLocationName, "650-555-9999");
    	WorkLocation[] locations = new WorkLocation[1];
    	locations[0] = newWL;
    	WorkLocation[] locationsFromUpdate = sEmployerService.addWorkLocations(locations).getWorkLocations();

    	//Check that we have the same number of worklocations.
    	WorkLocation[] locationsFromDb = sEmployerService.getWorkLocations();

    	AssertJUnit.assertTrue("Different Number of Locations came back", locationsFromUpdate.length == locationsFromDb.length);

    	//Find the new guy
    	boolean found = false;
    	for (WorkLocation workLocation : locationsFromDb) {
			if (workLocation.getName().equals(workLocationName)) {
				found = true;
				newWL = workLocation;
				break;
			}
		}
    	AssertJUnit.assertTrue("Did not find the WorkLocation named updateWorkLocatonTest", found);
    	
    	//Update the TelephoneNumber Number
    	String updatedPhoneNumber = "415-555-0000";
		newWL.setPhoneNumber(updatedPhoneNumber);

    	WorkLocation[] locationsToUpdate = new WorkLocation[1];
    	locationsToUpdate[0] = newWL;
    	WorkLocation[] updatedLocations = sEmployerService.updateWorkLocations(locationsToUpdate).getWorkLocations();
    	
    	//Find the updated guy
    	boolean updated = false;
    	for (WorkLocation workLocation : updatedLocations) {
			if (workLocation.getName().equals(workLocationName)) {
				newWL = workLocation;
				AssertJUnit.assertEquals("The TelephoneNumber Number was not updated", updatedPhoneNumber, newWL.getPhoneNumber());
				updated = true;
				break;
			}
		}
    	AssertJUnit.assertTrue("The WorkLocation named updateWorkLocatonTest was not updated", updated);
    	
    	// Clean up
    	WorkLocation[] locationsToDelete = new WorkLocation[1];
    	locationsToDelete[0] = newWL;
    	sEmployerService.deleteWorkLocations(locationsToDelete);
    }
    
    //**********************************************************
    // Validation Tests - these only validate the basic validations at the composite layer.
    // Business logic validations and their tests are at the Capability layer.
    //***********************************************************
    
    
    @Test(expectedExceptions = ValidationException.class)
    public void nullWorkLocationArrayAddTest() throws Exception {
    	sEmployerService.addWorkLocations(null);
    }

    @Test(expectedExceptions = ValidationException.class)
    public void nullWorkLocationArrayUpdateTest() throws Exception {
    	sEmployerService.updateWorkLocations(null);
    }

    @Test(expectedExceptions = ValidationException.class)
    public void nullWorkLocationArrayDeleteTest() throws Exception {
    	sEmployerService.deleteWorkLocations(null);
    }

    @Test(expectedExceptions = ValidationException.class)
    public void emptyWorkLocationArrayAddTest() throws Exception {
    	sEmployerService.addWorkLocations(new WorkLocation[0]);
    }

    @Test(expectedExceptions = ValidationException.class)
    public void emptyWorkLocationArrayUpdateTest() throws Exception {
    	sEmployerService.updateWorkLocations(new WorkLocation[0]);
    }

    @Test(expectedExceptions = ValidationException.class)
    public void emptyWorkLocationArrayDeleteTest() throws Exception {
    	sEmployerService.deleteWorkLocations(new WorkLocation[0]);
    }

}
