package com.intuit.sb.payroll.services.pto;

import java.math.BigDecimal;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.PtoBaseRateType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;

public class PTOServiceTest extends WorkerServiceTestBase {

    protected static PTOService sPTOService;
	
    @BeforeClass
    public static void initializePTOServiceTestClass()
    {
    	sPTOService = qaBootstrap.getEjbFromClass(PTOService.class);
    }
    
//ASTEC PTO FIXME
	@SuppressWarnings("deprecation")
	@Test(enabled=false, groups={"broken"})
	public void testPTOSalary() throws Exception {
		// create a new worker
		Worker worker = createWorker(); // create a worker with minimum data

		// Test SaveWorker with no compensations
		Worker savedWorker = sWorkerService.saveWorker(worker);
		
		Long id = savedWorker.getObjectHandle().getId();

		// save salary compensation item
		CompensationItem[] inputCompensations = new CompensationItem[1];
		inputCompensations[0] = new CompensationItem(null, new BigDecimal(10000), 
				0, CompensationType.Salary, "salary",true);
		inputCompensations[0].setPtoBaseRateType(PtoBaseRateType.PTO);
        // Test inserting compensations
		sWageService.saveCompensationItems(id, 
			mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, inputCompensations));
		
		// create sick pto
		PTOInfo ptoInfo = createSickTimePTO(id);
		
		// Save pto
		sPTOService.savePTO(ptoInfo);
		
		// Read back and compare pto
		PTOInfo readPTOInfo = sPTOService.getPTOByNameAndType(
			id, "sick time", PTOType.SICK_TIME);

		checkPTOInfo(ptoInfo, readPTOInfo);
		
		// Change and update pto
		ptoInfo = readPTOInfo;
		ptoInfo.setName("vacation");
		ptoInfo.setPtoType(PTOType.VACATION);
		ptoInfo.setAccrualRate(new Double("1.1"));
		ptoInfo.setAccruedHours(new Double(90));
		ptoInfo.setDisbursedHoursYTD(new Double(10));		
		sPTOService.savePTO(ptoInfo);
		
		// Read back and compare pto
		readPTOInfo = sPTOService.getPTOByNameAndType(id, "vacation", PTOType.VACATION);

		// Rev expected version
		long version = ptoInfo.getObjectHandle().getVersion();
		ptoInfo.getObjectHandle().setVersion(version + 1);
		checkPTOInfo(ptoInfo, readPTOInfo);
	}
	
	@Test
	public void testPTOHourly() throws Exception {
		// create a new worker
		Worker worker = createWorker(); // create a worker with minimum data

		// Test SaveWorker with no compensations
		Worker savedWorker = sWorkerService.saveWorker(worker);
		
		Long id = savedWorker.getObjectHandle().getId();
		
		// create wages including an hourly wage - no salary
        CompensationItem[] inputCompensations = new CompensationItem[1];
        inputCompensations[0] = new CompensationItem(null, new BigDecimal(HOURLY_RATE_40), 
        		80.00, CompensationType.Hourly, "Hourly",true);
		inputCompensations[0].setPtoBaseRateType(PtoBaseRateType.VACATION);
        // Test inserting compensations
		sWageService.saveCompensationItems(id, 
			mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, inputCompensations));
		
		PTOInfo ptoInfo = createVacationPTO(id);
		
		// Save pto
		sPTOService.savePTO(ptoInfo);
		
		// Read back and compare pto
		PTOInfo readPTOInfo = sPTOService.getPTOByNameAndType(id, "vacation", PTOType.VACATION);

		checkPTOInfo(ptoInfo, readPTOInfo);
	}

	private void checkPTOInfo(PTOInfo expectedPTOInfo, PTOInfo actualPTOInfo) {

		AssertJUnit.assertEquals("accrual rate doesn't match", 
				expectedPTOInfo.getAccrualRate(), actualPTOInfo.getAccrualRate());
		AssertJUnit.assertEquals("pto name doesn't match", 
			expectedPTOInfo.getName(), actualPTOInfo.getName());
		AssertJUnit.assertEquals("pto type doesn't match", 
			expectedPTOInfo.getPtoType(), actualPTOInfo.getPtoType());
		AssertJUnit.assertEquals("accrued hours doesn't match", 
			expectedPTOInfo.getAccruedHours(), actualPTOInfo.getAccruedHours());
		AssertJUnit.assertEquals("disbursed hours YTD doesn't match",
			expectedPTOInfo.getDisbursedHoursYTD(), actualPTOInfo.getDisbursedHoursYTD());
	}
	

	@Test 
	public void testGetAllPTOInfo() throws Exception{
		// create a new worker
		Worker worker = createWorker(); // create a worker with minimum data

		// Test SaveWorker with no compensations
		Worker savedWorker = sWorkerService.saveWorker(worker);
		
		// create wages including an hourly wage - no salary
        CompensationItem[] inputCompensations = new CompensationItem[1];
        inputCompensations[0] = new CompensationItem(null, new BigDecimal(HOURLY_RATE_40), 
        		80.00, CompensationType.Hourly, "Hourly",true);
		inputCompensations[0].setPtoBaseRateType(PtoBaseRateType.PTO);
        // Test inserting compensations
        	Long id = savedWorker.getObjectHandle().getId();
		sWageService.saveCompensationItems(id, 
			mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, inputCompensations));
		
		// create vacation Card and sick time card
		PTOInfo vacationPTO = createVacationPTO(id);
		PTOInfo sickTimePTO = createSickTimePTO(id);
		PTOInfo[] savedPTO = {vacationPTO, sickTimePTO};
		
		sPTOService.savePTO(vacationPTO);
		sPTOService.savePTO(sickTimePTO);
				
		// Read back and compare pto
		PTOInfo[] readPTOInfo = sPTOService.getAllPTOInfo(id);
		
		checkAllPTOInfo(savedPTO, readPTOInfo);
		
		//cleanup the database from the objects created for this test
		cleanUpEIWorker(id);
		
	}
	
	private void checkAllPTOInfo(PTOInfo[] expectedPTOInfo, PTOInfo[] actualPTOInfo) {
		 for (PTOInfo ptoInfoExpected : expectedPTOInfo){
			 for(PTOInfo ptoInfoRead : actualPTOInfo){
				 if(ptoInfoExpected.getPtoType()== ptoInfoRead.getPtoType()){
					 checkPTOInfo(ptoInfoExpected, ptoInfoRead);
				 }
			 }
		 }      
	}
	
	private PTOInfo createVacationPTO(Long workerId) {
		// create vacation pto
		PTOInfo ptoInfo = new PTOInfo();
		ptoInfo.setWorkerId(workerId);
		ptoInfo.setName("vacation");
		ptoInfo.setPtoType(PTOType.VACATION);
		ptoInfo.setAccrualRate(new Double("3.1416"));
		ptoInfo.setPtoAccrualType(PTOAccrualType.EACH_PAYCHECK);
		ptoInfo.setAllowCarryOver(false);
		ptoInfo.setLimitAccrual(false);
		ptoInfo.setAccrualStartType(AccrualStartType.IMMEDIATELY);
		ptoInfo.setAccrualStartDate(DateUtil.getNowDate());
		ptoInfo.setAccruedHours(new Double(100));
		ptoInfo.setDisbursedHoursYTD(new Double(20));
		return ptoInfo;
	}
	
	private PTOInfo createSickTimePTO(Long workerId) {
		//create sicktime pto
		PTOInfo ptoInfo = new PTOInfo();
		ptoInfo.setWorkerId(workerId);
		ptoInfo.setName("sick");
		ptoInfo.setPtoType(PTOType.SICK_TIME);
		ptoInfo.setAccrualRate(new Double(1));
		ptoInfo.setPtoAccrualType(PTOAccrualType.HOURLY);
		ptoInfo.setAllowCarryOver(false);
		ptoInfo.setLimitAccrual(false);
		ptoInfo.setAccrualStartType(AccrualStartType.ON_START_DATE);
		ptoInfo.setAccrualStartDate(DateUtil.getNowDate());
		ptoInfo.setAccruedHours(new Double(100));
		ptoInfo.setDisbursedHoursYTD(new Double(20));	
		return ptoInfo;
	}
}
