package com.intuit.sb.payroll.services.workerpayments;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

public class VoidPaychecksTest extends WorkerPaymentServiceTestBase {
	
    private static Long sReadyForPayWorkerId = null;
    private static PayrollRunSummary sPayrollSummary = null;

    @BeforeClass
    public static void setUpVoidPaychecksTest() {
        try {
            getUserTransaction().begin();
            sReadyForPayWorkerId = createMVPayrollReadyWorker("Paul");
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpCommitPayrollRunTest(): " + t.getMessage());
            t.printStackTrace();
        }
    }

    @AfterClass
    public static void tearDownVoidPaychecksTest() {
        try {
            getUserTransaction().begin();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate(); 
         	if (null != sReadyForPayWorkerId) {
                cleanUpEIWorker(sReadyForPayWorkerId);
                sReadyForPayWorkerId = null;
            }
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownCommitPayrollRunTest(): " + t.getMessage());
        }
    }
    
    private PayrollRunSummary createAPayrollRun() throws Exception {
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sReadyForPayWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
    	return payrollSummary;
    }
    
	/**
	 * <P>This test is to void a paycheck.</P>
	 */
    @Test
    public void testVoidPaycheck() throws Exception
    {
    	try {
        	//Setup
            sPayrollSummary = createAPayrollRun();

    		Long payrollRunId = sPayrollSummary.getPayrollRunId();
    		PaycheckSummary[] paycheckSummaries = sPayrollSummary.getPaycheckSummaries();
    		Long[] paycheckIdArray = new Long[paycheckSummaries.length];
    		for (int i=0; i<paycheckSummaries.length; i++) {
    			paycheckIdArray[i] = paycheckSummaries[i].getPaycheckId();
    		}
    		PayrollRunSummary payrollRunSummary = sWorkerPayService.voidPaychecks(paycheckIdArray);  		
    		PaycheckSummary[] paycheckSummaryArray = payrollRunSummary.getPaycheckSummaries();
    		
    		//Verify
    		AssertJUnit.assertEquals("payroll run id mismatch!", payrollRunId, payrollRunSummary.getPayrollRunId());
    		for (PaycheckSummary paycheckSummary : paycheckSummaryArray) {
    			AssertJUnit.assertEquals("paycheck status mismatch", PaycheckStatus.VOIDED, paycheckSummary.getPaycheckStatus());  			
    		}
    	}
    	finally
    	{
    		//Nothing to do
    	}
    }
}
