package com.intuit.sb.payroll.entity.benefits.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.EntityResources;
import com.intuit.sb.payroll.entity.NamedQueryCatalog;
import com.intuit.sb.payroll.entity.benefits.HealthBenefitEAO;
import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.entity.model.HealthBenefit;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;

@Stateless(name="HealthBenefitEAO")
@TransactionManagement(TransactionManagementType.CONTAINER) // Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY) 
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
@SuppressWarnings("nls")
public final class HealthBenefitEjbEAO extends GenericEjbEAO<HealthBenefit>	implements HealthBenefitEAO {

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.wages.HealthBenefitEAO#findHealthBenefit(com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum)
	 */
	public HealthBenefit findHealthBenefit(HealthBenefitTypeEnum healthBenefitType) {
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put(NamedQueryCatalog.BENEFIT_TYPE_QUERY_PARAM, healthBenefitType);
        
        List<HealthBenefit> results = null;
		try	{
			results = findByNamedQuery(NamedQueryCatalog.FIND_HEALTH_BENEFIT_BY_TYPE, queryParameters);
		}
		catch(RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		switch (results.size()) {
		case 0:
			String mesg1 = MessageFormat.format(EntityResources.getString("healthbenefit.does.not.exist"), healthBenefitType.toString());
			logger.error(mesg1);
			throw new IllegalStateException(mesg1);
		case 1:
			return results.get(0);
		default:
			String mesg2 = MessageFormat.format(EntityResources.getString("healthbenefit.multiple.for.type"), healthBenefitType.toString());
			logger.error(mesg2);
			throw new IllegalStateException(mesg2);
		}
	}
}
