package com.intuit.sb.payroll.entity.pto.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.EntityResources;
import com.intuit.sb.payroll.entity.NamedQueryCatalog;
import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.entity.model.PaidTimeOff;
import com.intuit.sb.payroll.entity.model.PaidTimeOffTypeEnum;
import com.intuit.sb.payroll.entity.pto.PaidTimeOffEAO;

@Stateless(name = "PaidTimeOffEAO")
@TransactionManagement(TransactionManagementType.CONTAINER)
// Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY)
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
@SuppressWarnings("nls")
public class PaidTimeOffEjbEAO extends GenericEjbEAO<PaidTimeOff> implements PaidTimeOffEAO {

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.entity.pto.PaidTimeOffEAO#findPaidTimeOff(com.intuit.sb.payroll.entity.model.
	 * PaidTimeOffTypeEnum)
	 */
	public PaidTimeOff findPaidTimeOff(PaidTimeOffTypeEnum paidTimeOffType) {
		Map<String, Object> queryParameters = new HashMap<String, Object>();
		queryParameters.put(NamedQueryCatalog.PAID_TIME_OFF_TYPE_QUERY_PARAM, paidTimeOffType);

		List<PaidTimeOff> results = null;
		try {
			results = findByNamedQuery(NamedQueryCatalog.FIND_PAID_TIME_OFF_BY_TYPE, queryParameters);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		switch (results.size()) {
			case 0:
				String mesg1 = MessageFormat.format(EntityResources.getString("paidtimeoff.does.not.exist"), paidTimeOffType
						.toString());
				logger.error(mesg1);
				throw new IllegalStateException(mesg1);
			case 1:
				return results.get(0);
			default:
				String mesg2 = MessageFormat.format(EntityResources.getString("paidtimeoff.multiple.for.type"), paidTimeOffType
						.toString());
				logger.error(mesg2);
				throw new IllegalStateException(mesg2);
		}
	}
}
