package com.intuit.sb.payroll.entity.taxauthority.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.entity.EntityResources;
import com.intuit.sb.payroll.entity.NamedQueryCatalog;
import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.entity.taxauthority.LawEAO;

@Stateless(name = "LawEAO")
@TransactionManagement(TransactionManagementType.CONTAINER)
// Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY)
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
@SuppressWarnings("nls")
public final class LawEjbEAO extends GenericEjbEAO<Law> implements LawEAO {


	public Law findByLawId(String lawId) throws ServiceFailureException {
		Map<String, Object> queryParameters = new HashMap<String, Object>();
		queryParameters.put(NamedQueryCatalog.LAW_ID_QUERY_PARAM, lawId);

		List<Law> results = null;
		try {
			results = findByNamedQuery(NamedQueryCatalog.FIND_LAW_BY_LAW_ID, queryParameters);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		if (results == null) {
			return null;
		}
		switch (results.size()) {
			case 0:
				return null;
			case 1:
				return results.get(0);
			default:
				String mesg = MessageFormat.format(EntityResources.getString("law.multiple.for.id"),
						lawId, results.get(0).getDescription());
				throw new ServiceFailureException(mesg);
		}
	}

}
