package com.intuit.sb.payroll.entity.worker.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.persistence.Query;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.NamedQueryCatalog;
import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO; // import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.worker.WorkerEAO;

@Stateless(name = "WorkerEAO")
@TransactionManagement(TransactionManagementType.CONTAINER)
// Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY)
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public final class WorkerEjbEAO extends GenericEjbEAO<Worker> implements WorkerEAO {

	public List<Worker> findWorkersReadyForPayroll() {
		List<Worker> results = null;
		try {
			results = this.findByNamedQuery(NamedQueryCatalog.FIND_WORKERS_READY_FOR_PAYROLL);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return results;
	}

	public List<Worker> findWorkersNotReadyForPayroll() {
		List<Worker> results = null;
		try {
			results = this.findByNamedQuery(NamedQueryCatalog.FIND_WORKERS_NOT_READY_FOR_PAYROLL);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return results;
	}

	@SuppressWarnings("unchecked")
	public int getWorkerCount() {
		// TODO [rj] - this needs to be active workers, not all workers
		String queryString = "SELECT COUNT(*) " + "FROM com.intuit.sb.payroll.entity.model.Worker wr where wr.active = 1";

		Query query = this.getEntityManager().createQuery(queryString);

		List resultList;
		try {
			resultList = query.getResultList();
			for (Object object : resultList) {
				Long count = (Long) object;
				return count.intValue();
			}
		} catch (RuntimeException e) {
			handleExceptionMightBeOLE(e);
		}

		return 0;
	}

	public List<Worker> findWorkersByWorkLocation(Long uniqueId) {
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put(NamedQueryCatalog.WORK_LOCATION_ID_QUERY_PARAM, uniqueId);
        
		List<Worker> results = null;
		try {
			results = this.findByNamedQuery(NamedQueryCatalog.FIND_WORKERS_BY_WORK_LOCATION, queryParameters);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return results;
	}
	
    public List<Worker> findActiveWorkers() {
		List<Worker> results = null;
		try {
			results = this.findByNamedQuery(NamedQueryCatalog.FIND_ACTIVE_WORKERS);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return results;
    }

    public List<Worker> findInactiveWorkers() {
		List<Worker> results = null;
		try {
			results = this.findByNamedQuery(NamedQueryCatalog.FIND_INACTIVE_WORKERS);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return results;
    }

}