package com.intuit.sb.payroll.entity.workerpayments;

import java.util.Date;
import java.util.List;

import javax.ejb.Local;

import com.intuit.sb.payroll.entity.eao.GenericEAO;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;

@Local
public interface PayrollRunEAO extends GenericEAO<PayrollRun> {

	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "PayrollRunEAO/local";

	/**
	 * Returns a List of {@link PayrollRun}s that occur on or after the date parameter.
	 * 
	 * @param fromDate
	 * @return
	 */
	public List<PayrollRun> findPayrollRunsFromDate(Date fromDate);

    /**
     * Retrieves recent, committed, payroll runs. The runs are selected (and ordered) based on paycheck date, newest
     * to oldest. The result set is limited to the most recent N runs as specified by numberOfRuns. If N is greater
     * than the total number of payrollRunType payrolls for the company the results are limited to all the existing
     * runs of payrollRunType. 
     *
     * @param payrollRunType the type of payroll runs to retrieve.
     * @param numberOfRuns the maximum number of payroll runs to retrieve.
     * @return an List of PayrollRun, one for each of the most recent payroll runs. PayrollRun.get(0) is the
     * most recent run and PayrollRun.get(numberOfRuns - 1) is the oldest selected run. The list will be empty of
     * no committed payroll runs exist for the company.
     */
	public List<PayrollRun> findRecentPayrollRuns(PayrollRunTypeEnum payrollRunType, int numberOfRuns);
	
	/**
     * Retrieves recent, committed, payroll runs. The runs are selected (and ordered) based on paycheck date, newest
     * to oldest. The result set is limited to the runs in the year passed.
     *
     * @param payrollRunType the type of payroll runs to retrieve.
     * @param year the year for which payroll runs to retrieve.
     * @return an List of PayrollRun, one for each of the most recent payroll runs. PayrollRun.get(0) is the
     * most recent run. The list will be empty of no committed payroll runs exist for the company.
     */
	public List<PayrollRun> findRecentPayrollRunsForYear(PayrollRunTypeEnum payrollRunType, int year);

	/**
	 * Returns the last PayrollRun with a status of Committed, if any.
	 * @return the latest {@link PayrollRun} with a status of {@link PayrollRunWorkflowStatusEnum#COMMITTED}.
	 */
	public PayrollRun findLastCompletedPayrollRun();

	/**
	 * Returns the list of all {@link PayrollRun}s, sorted Descending.
	 * @return
	 */
	public List<PayrollRun> findAllDescending();

}
