package com.intuit.sb.payroll.takehome.entity;

import javax.ejb.Local;

import com.intuit.sb.payroll.entity.eao.GenericEAO;

/**
 * Interface defines the TakehomeWorkerEAO. 
 * TakehomeWorkerEAO can be implemented as POJO or EJB, the interface provides this abstraction. 
 * 
 * @param TakehomeWorker is a concrete extension of {@code BaseEntity} that implements the entity
 * managed by this EAO. 
 * 
 * @see GenericEAO
 * 
 */
@Local
public interface TakehomeCompanyEAO extends GenericEAO<TakehomeCompany> {

	public static final String JNDI_NAME =  "TakehomeCompanyEAO/local";

	/**
	 * Find a TakehomeCompany for the given payroll domain employer ID.
	 * This is <b>not</b> the same as the TakehomeCompany Id. It is (logically)
	 * a foreign key to the PR Employer.
	 * @param uniqueId the PK of the associated Payroll Employer entity.
	 * @return the TakehomeCompany entity, or null if it isn't found.
	 * @throws IllegalStateException if there are multiple matches.
	 */
	public TakehomeCompany findByPayrollEmployerId(Long uniqueId);
}
