package com.intuit.sb.payroll.takehome.entity;

import javax.ejb.Local;

import com.intuit.sb.payroll.entity.eao.GenericEAO;

/**
 * Interface defines the TakehomeWorkerEAO. 
 * TakehomeWorkerEAO can be implemented as POJO or EJB, the interface provides this abstraction. 
 * 
 * @param TakehomeWorker is a concrete extension of {@code BaseEntity} that implements the entity
 * managed by this EAO. 
 * 
 * @see GenericEAO
 * 
 */
@Local
public interface TakehomeWorkerEAO extends GenericEAO<TakehomeWorker> {

	public static final String JNDI_NAME =  "TakehomeWorkerEAO/local";

	/**
	 * Find a Takehome Worker for the given payroll domain worker's ID.
	 * This is <b>not</b> the same as the Takehome worker's Id. It is (logically)
	 * a foreign key to the PR Worker.
	 * @param uniqueId the PK of the associated Payroll Worker entity.
	 * @return the Takehome Worker entity, or null if it isn't found.
	 * @throws IllegalStateException if there are multiple matches.
	 */
	public TakehomeWorker findByPayrollWorkerId(Long uniqueId);
}
