package com.intuit.sb.payroll.takehome.entity.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.takehome.entity.TakehomeWorker;
import com.intuit.sb.payroll.takehome.entity.TakehomeWorkerEAO;

/**
 * Implements the generic CRUD data access operations 

 * @param TakehomeWorker the concrete extension of {@code BaseEntity} that implements the entity
 * managed by this EAO. 
 * 
 * @see GenericEjbEAO
 */

@Stateless(name="TakehomeWorkerEAO")
@TransactionManagement(TransactionManagementType.CONTAINER) // Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY) 
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public final class TakehomeWorkerEjbEAO extends GenericEjbEAO<TakehomeWorker>
	implements TakehomeWorkerEAO {

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.takehome.entity.TakehomeWorkerEAO#findByPayrollWorkerId(java.lang.Long)
	 */
	public TakehomeWorker findByPayrollWorkerId(Long uniqueId) {
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("payrollWorkerId", uniqueId);
        
        List<TakehomeWorker> results = null;
		try	{
			// named query is in orm.xml
			results = findByNamedQuery("findTakehomeWorkerByWorkerId", queryParameters);
		}
		catch(RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		switch (results.size()) {
		case 0:
			return null;
		case 1:
			return results.get(0);
		default:
			String mesg2 = MessageFormat.format("Found multiple Takehome Workers with Payroll Worker ID ''{0}''.", uniqueId);
			logger.error(mesg2);
			throw new IllegalStateException(mesg2);
		}
	}
}
