package com.intuit.sb.payroll.capability.common;

import java.util.ArrayList;
import java.util.List;

import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;

import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeSuite;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestBase;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.CompanyMapper;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.pts.entity.InteractionEnum;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSContactPersonTypeEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;

public class BasicPTSTestBase extends EmployerComponentTestBase{

	protected static EmployerPTSWorkflowComponent sEmployerPTSWorkflowComponent = null;
	protected static String sSourceCompanyId = null;

	@BeforeSuite
    synchronized public static void initializePTSBasicTestBaseSuite() throws Exception
    {
//		Properties props = new Properties();
//		props.setProperty("container", "BHP");
//		System.setProperties(props);
		
//		try
//		{
//			// Get the default JNDI context as configured in jndi.properties.
//			// If a remote context is required it should be configured in jndi.properties.
//			  Properties env = new Properties();
//			  env.setProperty(Context.SECURITY_PRINCIPAL, "paul");
//			  env.setProperty(Context.SECURITY_CREDENTIALS, "paul");
//			  env.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.security.jndi.JndiLoginInitialContextFactory");
//			  InitialContext initialContext = new InitialContext(env);
//			  ServiceLocator.getInstance().setInitialContext(initialContext);
//		}
//		catch (NamingException ne)
//		{
//            String mesg = MessageFormat.format("Unable to access JNDI: {0}", new Object[] {ExceptionUtils.throwableMessageChain(ne)});
//            throw new RuntimeException(mesg);
//		}
		
    }

	
	
	
	/**
	 * initialize 
	 * @throws ValidationException 
	 * @throws ServiceFailureException 
	 * @throws SystemException 
	 * @throws NotSupportedException 
	 * @throws HeuristicRollbackException 
	 * @throws HeuristicMixedException 
	 * @throws RollbackException 
	 * @throws IllegalStateException 
	 * @throws SecurityException 
	 */
	@BeforeClass
	public void initializePTSBasicTestBaseMethod() throws ServiceFailureException, ValidationException, NotSupportedException, SystemException, SecurityException, IllegalStateException, RollbackException, HeuristicMixedException, HeuristicRollbackException{
		sEmployerPTSWorkflowComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
		// Add a company into PSP for other tests and set a static source companyId
		if (sSourceCompanyId == null){
			getUserTransaction().begin();
			PTSEmployer ptsEmployer = sEmployerPTSWorkflowComponent.getPTSEmployer();
			if (StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
				sSourceCompanyId = CompanyMapper.namespaceEmployerId(ptsEmployer.getEmployer());
			}else{
				addCurrentCompanytoPSP(ptsEmployer);
			}
			getUserTransaction().commit();
		}
	}
	
	
	
	
	protected PTSContactPerson createPTSEmployerContactPerson(EmployerContactPerson employerContactPerson, PTSContactPersonTypeEnum contactPersonType) {
		
		PTSContactPerson ptsContactPerson = new PTSContactPerson();
		ptsContactPerson.setEmployerContact(employerContactPerson);
		ptsContactPerson.setContactPersonType(contactPersonType);
		
		return ptsContactPerson;
	}
	
	private void addCurrentCompanytoPSP(PTSEmployer ptsEmployer) throws ValidationException, ServiceFailureException{
		// Get current Employer
		
		ptsEmployer = makeEmployerPTSReady(ptsEmployer);
		
		try {
			sEmployerPTSWorkflowComponent.sendCompanyInfoToPTS();
		} catch (ServiceFailureException e) {
			// [amalik] does not work properly, seems exception thrown does not let the txn to go through 
			String message = e.getMessage();
			if (message.contains("already exists")){
				sSourceCompanyId = CompanyMapper.namespaceEmployerId(ptsEmployer.getEmployer());
				sEmployerPTSWorkflowComponent.updatePPS_PTS_InteractionState(ptsEmployer, InteractionEnum.ADDED);
			}else{
				throw e;
			}
				
		}
		
		// If company gets added to PSP
		if (StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			sSourceCompanyId = CompanyMapper.namespaceEmployerId(ptsEmployer.getEmployer());
		}
	}
	
	


	private PTSEmployer makeEmployerPTSReady(PTSEmployer ptsEmployer) throws ValidationException, ServiceFailureException {
		// Add Primary Contact only
		String firstName = "CPerson";
		String lastName = "Test";
		String title = "Mr";
		String phone = "408-555-1234";
		EmployerContactPerson person = createEmployerContactPerson(
				firstName, lastName, title, phone);
		person.setOwner(Boolean.TRUE);

		PTSContactPerson ptsContactPerson = createPTSEmployerContactPerson(person, PTSContactPersonTypeEnum.PRIMARY_CONTACT);
		List<PTSContactPerson> employerContactPersons = new ArrayList<PTSContactPerson>();
		employerContactPersons.add(ptsContactPerson);
		sEmployerPTSWorkflowComponent.addEmployerContactPersons(employerContactPersons);
		
		Employer employer = ptsEmployer.getEmployer();
		
		// Legal Address
		
		GlobalPostalAddress legalAddress = EmployerComponentTestUtils
				.createGlobalPostalAddress("44 Elm Street", "San Carlos",
						"CA", "93122");
		employer.setLegalAddress(legalAddress);

		// Mailing address
		GlobalPostalAddress mailingAddress = EmployerComponentTestUtils
				.createGlobalPostalAddress("45 Elm Street", "San Carlos",
						"CA", "93122");
		employer.setBusinessAddress(mailingAddress);
		sEmployerComponent.saveEmployer(employer);
		return sEmployerPTSWorkflowComponent.savePTSEmployer(ptsEmployer);
	}
	
	
}

