package com.intuit.sb.payroll.capability.common;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.testng.Assert;
import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.CompanyActivationStatusEnum;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.NotificationAttribute;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.NotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.PTSNotificationDTO;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandler;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerFactory;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.services.common.BankAccount;
import com.intuit.sb.payroll.services.common.BankAccountStatusEnum;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.CompanyType;
import com.intuit.sb.payroll.services.employer.EmployerContact;
import com.intuit.sb.payroll.services.employer.EmployerContactType;
import com.intuit.sb.payroll.services.employer.EmployerInfo;
import com.intuit.sb.payroll.services.employer.EmployerService;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.services.employer.SaveEmployerInfoRequest;
import com.intuit.sb.payroll.services.taxes.EmployerTaxIdInfo;
import com.intuit.sb.payroll.services.taxes.EmployerTaxSettingsService;
import com.intuit.sb.payroll.task.Task;
import com.intuit.sb.payroll.task.TaskService;

public class PTSEmployerServiceTestBase extends EmployerServiceTestBase{
	
	
	protected static EmployerTaxSettingsService sEmployerTaxSettingsService;
	protected static TaskService sTaskService;
	
	@BeforeClass
	public static void initialize()
	{
	
		sTaskService = ServiceLocator.getInstance().getService(TaskService.JNDI_NAME);
		sEmployerTaxSettingsService = ServiceLocator.getInstance().getService(EmployerTaxSettingsService.JNDI_NAME);
		
	}
	

	
	public static void mockUpdateCompanyActivationStatus(PTSNotificationDTO ptsNotificationDTO) throws PTSNotificationException{
	        
		PTSNotificationRequest ptRequest = null;
		ptRequest = PTSNotificationRequestMapper.mapPTSNotificationRequest(ptsNotificationDTO);
		
		StatusUpdateHandler statusUpdateHandler = StatusUpdateHandlerFactory.getStatusUpdateHandler(ptRequest.getNotificationType());
		statusUpdateHandler.updateStatus(sEmpPTSWFComponent.getPTSEmployer(), ptRequest);
	}
	    
	public static void mockBankAccountInboundCall(NotificationTypeEnum notificationTypeEnum,CompanyActivationStatusEnum cStatusEnum , String bankId) throws ServiceFailureException, PTSNotificationException{
	  	  
	  	  PTSNotificationDTO ptsNotificationDTO = createPTSNotiticationDTO(notificationTypeEnum, cStatusEnum);
	  	  NotificationAttribute notificationAttribute = new NotificationAttribute();
	  	  notificationAttribute.setName("SourceBankAccountId");
	  	  notificationAttribute.setValue(bankId);
	  	  ptsNotificationDTO.getAttributes().add(notificationAttribute);			
	  	  mockUpdateCompanyActivationStatus(ptsNotificationDTO);				
	  	  
	}
		
	protected void verifyBankVerifiedStatusInDB(BankAccountStatusEnum bAccountStatusEnum,String accNo) throws ServiceFailureException, ValidationException{
			
		BankAccount[] savedBankAccounts = sEmployerService.getBankAccounts();
		BankAccount bankAccountAfterSuccess = getBankAccountBasedOnAccountNo(savedBankAccounts, accNo);
			
		// verify the retrieved bankAccountStatus
		AssertJUnit.assertEquals("BankAccount Status is not Mapped correctly",bAccountStatusEnum,
		bankAccountAfterSuccess.getBankAccountStatus());

	}
		
		
	  
	    
	    // TODO : look at Verify in GetCurrentTaskTest and refactor both
	    // verify the tasks status
		protected void verifyTaskName(Task[] tasks , String[] expectedTaskNames){
			
			List<String> expectedTaskNamesList = Arrays.asList(expectedTaskNames);
			List <String> actualTaskNameList = new ArrayList<String>();
	 
		
			for(int i = 0 ; i < tasks.length ; i++){
				
				actualTaskNameList.add(tasks[i].getName());
			}
			
			Assert.assertTrue(actualTaskNameList.containsAll(expectedTaskNamesList),"Expected List of Task not found");
						
		}
		
		public static  void prepareCurrentEmployerForAssisted(EmployerService employerService, EmployerTaxSettingsService employerTaxSettingService) throws Exception{    	
			// create a DIY Employer
			createDIYEmployer();
			
	    	if(employerService!=null){
	    		sEmployerService = employerService;
	    	}
			//update the assisted state
			sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
			
			//add primary contact
			EmployerContact[] primaryContacts = new EmployerContact[1];
			primaryContacts[0]= EmployerServiceTestBase.createEmployerContact("Brad", "Pitt", "6505551212", EmployerContactType.PrimaryContact);		
			sEmployerService.addEmployerContacts(primaryContacts);
			
			// add legal Address			
			EmployerInfo employerInfo = sEmployerService.getEmployerInfo();
			PhysicalAddress legalAddress = EmployerServiceTestBase.createPhysicalAddress();
			employerInfo.setLegalAddress(legalAddress);
			
			//add legal Name
			employerInfo.setLegalName("Red Chilli Corp");
			employerInfo.setCompanyType(CompanyType.CORPORATION);
			
			if(employerTaxSettingService!=null){
				sEmployerTaxSettingsService = employerTaxSettingService;
	    	}
			
			//add CA state Tax ID
			EmployerTaxIdInfo[] taxIdInfos = null;
			taxIdInfos = sEmployerTaxSettingsService.getEmployerTaxIDs();
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos)
			{
				if (employerTaxIdInfo.getTaxAgency().name.contains("CA_EDD"))
				{
					employerTaxIdInfo.setTaxId("44000044");
				}
			}
								
			sEmployerTaxSettingsService.updateEmployerTaxIDs(taxIdInfos);			
			
			SaveEmployerInfoRequest saveEmployerInfoRequest = new SaveEmployerInfoRequest();
			saveEmployerInfoRequest.setEmployerInfo(employerInfo);		
			sEmployerService.saveEmployerInfo(saveEmployerInfoRequest);
			
		}
		
		public static void createDIYEmployer() throws Exception{
			// delete the current Employer
	    	EmployerComponentTestUtils.deleteCurrentEmployer();
	    	
	    	// create a new employer with random EIN
	    	EmployerComponentTestUtils.createAndSaveNewEmployer();
	    	
	    	// create WorkLocation Sanfrancisco
	    	EmployerComponentTestUtils.createWorkLocation();
	    	
	    	PayrollScheduleTestUtils.createDefaultSchedule();
		}


	  

		


	
	

}
