package com.intuit.sb.payroll.capability.common;

import java.util.Date;


import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.UpdateWorkLocationsResponse;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.CompanyType;
import com.intuit.sb.payroll.services.employer.EmployerContact;
import com.intuit.sb.payroll.services.employer.EmployerContactType;
import com.intuit.sb.payroll.services.employer.EmployerInfo;

import com.intuit.sb.payroll.services.employer.SaveEmployerInfoRequest;
import com.intuit.sb.payroll.services.employer.WorkerLocation;
import com.intuit.sb.payroll.services.wages.CompensationItemTestBase;

import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;
import com.intuit.sb.payroll.services.worker.WorkerType;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;

public class PTSWorkerTestBase extends WorkerServiceTestBase{
	
	
	   protected Worker createBasicEmployee(String firstname , String lastname) throws ValidationException, WorkflowException, ServiceFailureException{

		   Worker worker = new Worker();
			
			// Worker Type
			worker.setWorkerType(WorkerType.EMPLOYEE);
			
			//First Name
			worker.setFirstName(firstname);
			
			//Last Name
			worker.setLastName(lastname);
			
			// Live Location
			PhysicalAddress taxResidenceAddress = new PhysicalAddress();
			taxResidenceAddress.setStateOrRegion("CA");
			
			worker.setTaxResidenceAddress(taxResidenceAddress);
			
			// Hire Date
			worker.setHireDate(new Date());
			
			worker.setBirthDate(new Date());
			
			// Payment Method
			worker.setPaycheckDistributionMethod(PaycheckDistributionMethod.CASH);
			
			//SSN
			worker.setWorkerTaxIdNumber(EmployerComponentTestUtils.generateRandomEIN()); // TODO : Rename generateRandomEIN to generic name since 
																						 // both SSN and EIN has same digits we are calling this
			worker = sWorkerService.saveWorker(worker);
			return worker;
	   }
	   
	   protected Worker addPayItemToWorker(Worker worker) throws Exception{
		   
		   CompensationItemTestBase.setupSalaryAndBonusWages(worker.getObjectHandle().getId());
		   worker = sWorkerService.getWorkerById(worker.getObjectHandle().getId());
		   return worker;
	   }
	   
	   protected Worker addWorkLocationToWorker(Worker worker , String AddressLine1 , String city) throws Exception{
		   	int existingWorkLocationscount = sEmployerService.getWorkLocations().length;
			
		   	WorkLocation[] worklocations = new WorkLocation[1];
			WorkLocation wrkl= new WorkLocation();
			
			PhysicalAddress address = new PhysicalAddress();
			address.setStateOrRegion("CA");
			address.setAddressLine1(AddressLine1);
			address.setCity(city);
			address.setCountry("US");
			address.setStateOrRegion("CA");
			address.setZipOrPostalCode("12345");
			
			wrkl.setAddress(address);
			wrkl.setName("Main");			
			worklocations[0]=wrkl;
			
			UpdateWorkLocationsResponse updateWorkLocationResponse = sEmployerService.addWorkLocations(worklocations);
			worklocations = updateWorkLocationResponse.getWorkLocations();
			
			WorkerLocation[] workLocations = new WorkerLocation[1];
			WorkerLocation workL = new WorkerLocation(); 
			
			workLocations[0] = workL;
			
			workL.setWorkerId(worker.getObjectHandle().getId());
			workL.setWorkLocationId(worklocations[existingWorkLocationscount].getObjectHandle().getId());
			
			sEmployerService.assignWorkLocationsToWorkers(workLocations);
			
			worker = sWorkerService.getWorkerById(worker.getObjectHandle().getId());
			
			
			return worker;
	   }
	   
		
	private static PhysicalAddress createPhysicalAddress() {
			PhysicalAddress physicalAddress = new PhysicalAddress();
			physicalAddress.setAddressLine1("123 Main St.");
			physicalAddress.setCity("Mountain View");
			physicalAddress.setStateOrRegion("CA");
			physicalAddress.setZipOrPostalCode("94043");
			physicalAddress.setCountry("USA");
			
			return physicalAddress;
	}
	
	private static EmployerContact createEmployerContact(String firstName, String lastName, String phone,
    		EmployerContactType empContactType){
		
 	    	EmployerContact employerContact = new EmployerContact();
 			employerContact.setFirstName(firstName);
 			employerContact.setLastName(lastName);
 			employerContact.setEmail(firstName+ "." + lastName +"@MyCompany.com");
 			employerContact.setPhone(phone);
 			employerContact.setExtn("4444");
 			employerContact.setOwner(true);
 			employerContact.setEmployerContactType(empContactType);
 			employerContact.setTitle("Payroll Manager");
 			employerContact.setFax("111-222-5555");
 			employerContact.setSameAsCompanyAddress(false);
 			
 			PhysicalAddress address = createPhysicalAddress();
 			
 			employerContact.setAddress(address);
 			
 			return employerContact;
		
	}
	
	public  void createAndSaveAssistedReadyEmployer() throws Exception{
		
		EmployerComponentTestUtils.createAndSaveNewEmployer();
		
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		//add primary contact
		EmployerContact[] primaryContacts = new EmployerContact[1];
		primaryContacts[0]= createEmployerContact("Brad", "Pitt", "6505551212", EmployerContactType.PrimaryContact);		
		sEmployerService.addEmployerContacts(primaryContacts);
		
		// add legal Address
		
		EmployerInfo employerInfo = sEmployerService.getEmployerInfo();
		PhysicalAddress legalAddress = createPhysicalAddress();
		employerInfo.setLegalAddress(legalAddress);
		
		//add legal Name
		employerInfo.setLegalName("Red Chilli Corp");
		employerInfo.setCompanyType(CompanyType.CORPORATION);
		
		SaveEmployerInfoRequest saveEmployerInfoRequest = new SaveEmployerInfoRequest();
		saveEmployerInfoRequest.setEmployerInfo(employerInfo);		
		sEmployerService.saveEmployerInfo(saveEmployerInfoRequest);
		
		// Add worklocation
		WorkLocation[] wrkLocations= new WorkLocation[1];
		wrkLocations[0]=createWorkLocation("MV", "2234567656");
		
		sEmployerService.addWorkLocations(wrkLocations);
		
	}
	

	/**
     * Utility method to create a work Location from the name and worker count passed in
     * @param name
     * @param workerCount
     * @return
     */
	private static WorkLocation createWorkLocation(String name, String phoneNumber) {
    	WorkLocation workLocation = new WorkLocation();
    	
    	workLocation.setName(name);
    	workLocation.setPhoneNumber(phoneNumber);
    	workLocation.setExtn("5440");
    	//set the Physical Address
    	workLocation.setAddress(createPhysicalAddress());
    	
    	return workLocation;
    }
	
	public static void sendCompanyInfoToPTS() throws ServiceFailureException, ValidationException{
		
		sEmployerService.sendCompanyInfoToPTS();
	}

}
