package com.intuit.sb.payroll.itest;

import java.util.List;

import javax.management.MalformedObjectNameException;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.itest.common.EmployerServiceITestBase;
import com.intuit.sb.payroll.itest.util.PayrollTestUtil;
import com.intuit.sb.payroll.psp.testws.CompanyTestWSRequestWrapper;
import com.intuit.sb.payroll.services.common.BankAccount;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.EmployerContact;
import com.intuit.sb.payroll.services.employer.EmployerInfo;
import com.intuit.sb.payroll.services.employer.SaveEmployerInfoRequest;
import com.intuit.sbd.payroll.psp.webservices.BankAccountWSDTO;
import com.intuit.sbd.payroll.psp.webservices.CompanyBankAccountWSDTO;
import com.intuit.sbd.payroll.psp.webservices.CompanyWSDTO;
import com.intuit.sbd.payroll.psp.webservices.Exception_Exception;


@Test(groups = "serviceittests")
//@Test(groups = "broken")
public class CreateCompanyTest extends EmployerServiceITestBase {

	/*@Test
	public void testJMXConfiguration() throws MalformedObjectNameException, InterruptedException{
		setTaxServiceHost("http://localhost:8080");
		Thread.sleep(60000);
	}*/
	
	@Test
	public void testCreateCompanyInPTS() throws ServiceFailureException,
			ValidationException, Exception_Exception, MalformedObjectNameException, InterruptedException {
		
		PayrollTestUtil.initialize(PSP_HOST);
		setTaxServiceHost(PSP_HOST);

		employerService.loadApplication();

		employerService
				.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);

		// Add WorkLocation to Employer
		WorkLocation[] workLocations = new WorkLocation[] { createWorkerLocation() };
		employerService.addWorkLocations(workLocations);

		// Add Primary Contact to Employer
		EmployerContact[] employerContacts = new EmployerContact[] { createPrimaryContact() };
		employerService.addEmployerContacts(employerContacts);

		// Add Legal Info to Employer
		String legalName = "SOAMatrix Technologies";
		EmployerInfo employerInfo = createEmployerInfo(legalName);
		SaveEmployerInfoRequest saveEmployerInfoRequest = new SaveEmployerInfoRequest();
		saveEmployerInfoRequest.setEmployerInfo(employerInfo);
		employerService.saveEmployerInfo(saveEmployerInfoRequest);

		// Set EIN to Employer
		setEmployerTaxId("US_IRS",PayrollTestUtil.generateRandomEIN()); 

		// Set CA EAN to Employer
		setEmployerTaxId("CA_EDD","898-7611-2");
		
		String bankName = "ICICI BANK";
		String accountNumber = "898786";
		String routingNumber = "110000000";
			
		BankAccount[] bankAccounts = new BankAccount[] { createBankAccount(bankName,accountNumber,routingNumber) };
		employerService.addBankAccounts(bankAccounts);

		// send Employer Info to PSP
		employerService.sendCompanyInfoToPTS();
		
		// ----------------- PSP SIDE VALIDATION --------------------------
		
		CompanyTestWSRequestWrapper coWrapper = new CompanyTestWSRequestWrapper(PSP_HOST);
		List<CompanyWSDTO> companyWSDTOs = coWrapper.queryCompanyByLegalName(legalName);
		
		CompanyWSDTO companyWSDTO = companyWSDTOs.get(0);
		Assert.assertEquals(companyWSDTO.getLegalName(), legalName);
		
		
		String pspCompanyID = companyWSDTO.getPspCompanyID();
		String srcCompanyID = companyWSDTO.getSourceCompanyID();
		
		List<CompanyBankAccountWSDTO> banList = coWrapper.queryCompanyBankAccounts(pspCompanyID);
		CompanyBankAccountWSDTO coAccountWSDTO = banList.get(0);
		
		BankAccountWSDTO pspbankAccount = coAccountWSDTO.getBankAccount();
		String sourceCompanyBankID = coAccountWSDTO.getSourceBankAccountID();
		
		Assert.assertEquals(pspbankAccount.getBankName(), bankName);
		Assert.assertEquals(pspbankAccount.getAccountNumber(), accountNumber);
		Assert.assertEquals(pspbankAccount.getRoutingNumber(), routingNumber);
		
		
		PayrollTestUtil.activateTaxPaymentService(srcCompanyID, sourceCompanyBankID);
		

	}


}