package com.intuit.sb.payroll.itest.common;

import org.testng.AssertJUnit;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;
import com.intuit.sb.payroll.services.common.BankAccount;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.employer.CompanyType;
import com.intuit.sb.payroll.services.employer.EmployerContact;
import com.intuit.sb.payroll.services.employer.EmployerContactType;
import com.intuit.sb.payroll.services.employer.EmployerInfo;
import com.intuit.sb.payroll.services.taxes.EmployerTaxIdInfo;


public class EmployerServiceITestBase extends IntegrationTestBase{
	
	
	
	protected WorkLocation createWorkerLocation() {

		WorkLocation workLocation = new WorkLocation();

		workLocation.setName("Main");
		workLocation.setPhoneNumber("787-997-7888");
		workLocation.setExtn("5440");
		// set the Physical Address
		workLocation.setAddress(createPhysicalAddress());

		return workLocation;
	}

	protected BankAccount createBankAccount(String bankName,String accNo,String rtNo) {

		BankAccount bankAccount = new BankAccount();
		bankAccount.setBankName(bankName);
		bankAccount.setAccountNo(accNo);
		bankAccount.setAccountType(BankAccountTypeEnum.SAVINGS);
		bankAccount.setRoutingNo(rtNo);
		return bankAccount;
	}

	protected EmployerContact createPrimaryContact() {
		EmployerContact employerContact = new EmployerContact();
		employerContact.setFirstName("Kumar");
		employerContact.setLastName("Venkatraman");
		employerContact.setPhone("231-967-9855");
		employerContact.setEmail("kumarve@gmail.com");
		employerContact.setTitle("Mr");
		employerContact.setAddress(createPhysicalAddress());
		employerContact
				.setEmployerContactType(EmployerContactType.PrimaryContact);
		return employerContact;
	}

	protected PhysicalAddress createPhysicalAddress() {

		PhysicalAddress physicalAddress = new PhysicalAddress();
		physicalAddress.setAddressLine1("142 , Green View Park");
		physicalAddress.setCity("San Jose");
		physicalAddress.setCountry("US");
		physicalAddress.setStateOrRegion("CA");
		physicalAddress.setZipOrPostalCode("94056");
		return physicalAddress;
	}

	protected EmployerInfo createEmployerInfo(String legalName) {

		EmployerInfo employerInfo = new EmployerInfo();

		employerInfo.setCompanyAddress(createPhysicalAddress());
		employerInfo.setLegalName(legalName);
		employerInfo.setDBAName("Reform Systems");
		employerInfo.setCompanyType(CompanyType.CORPORATION);
		employerInfo.setLegalAddress(createPhysicalAddress());

		return employerInfo;
	}

	/**
	 * Helper Method to set EIN to current employer
	 * */
	protected void setEmployerTaxId(String agencyId ,String taxid)
			throws ServiceFailureException {

		try {
			EmployerTaxIdInfo[] taxIdInfos = employerTaxSettingsService
					.getEmployerTaxIDs();
			// assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(agencyId)) { //CA_EDD | "US_IRS"
					employerTaxIdInfo.setTaxId(taxid);
					break;
				}
			}
			employerTaxSettingsService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			AssertJUnit.fail("Validation Exception thrown " + ex.getMessage());
		}
	}


}
