package com.intuit.sb.payroll.itest.common;

import java.util.Calendar;

import javax.management.MalformedObjectNameException;
import javax.transaction.NotSupportedException;
import javax.transaction.SystemException;

import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;

import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.remoteejb.GeminiConfigService;
import com.intuit.sb.payroll.services.employer.EmployerService;
import com.intuit.sb.payroll.services.taxes.EmployerTaxSettingsService;
import com.intuit.sb.payroll.services.worker.WorkerService;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService;
import com.intuit.sb.payroll.util.RemoteServiceLocator;


/**
 *  <P>The <CODE>BasicTestBase</CODE> class contains helper methods to set up test environment</P>
 *  This class also contains tests, which test that EAOs can be found, and test data can be initialized.
 *
 * $Author: jzhong $
 * $Revision: #14 $
 * $DateTime: 2008/10/07 14:32:08 $
 *
 */
public class IntegrationTestBase
{
	/**
	/**
	 * <P>The UpdateWorkerWithholdings task name</P>
	 */
	protected static final String sUpdateWorkerWithholdings = "UpdateWorkerWithholdings";
	/**
	 * <P>The WorkerHireDateSaveOperation task name</P>
	 */
	protected static final String sWorkerHireDateSaveOperation = "WorkerHireDateSaveOperation";
	/**
	 * <P>A boolean value to represent that a worker is exempt from a tax</P>
	 */
	protected static final boolean sExemptFromTax = true;
	/**
	 * <P>A boolean value to represent that a worker is not exempt from a tax</P>
	 */
	protected static final boolean sNotExemptFromTax = false;
	/**
	 * <P>Holds an instance of PayrollResourceManager</P>
	 */
	protected static final PayrollResourceManager sResourceManager = PayrollResourceManager.getInstance();
	/**
	 * <P>Holds an instance of Logger</P>
	 */
	/**
	 * <P>Holds an instance of a Company object, this company is used for most tests</P>
	 */
	protected static Employer employer = null;
	/**
	 * <P>Holds an instance of a Worker object, this worker is used for most tests</P>
	 */
	protected static Worker sWorker = null;    // Use the same Worker object for most tests
	/**
	 * <P>Holds an instance of a Worker's SpcfUniqueId, this worker is used for most tests</P>
	 */
	protected static Long sWorkerId = null;

	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 40</P>
	 */
	protected static Calendar sAge40BirthDate = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 22</P>
	 */
	protected static Calendar sAge22BirthDate = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 21</P>
	 */
	protected static Calendar sAge21BirthDate = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 20</P>
	 */
	protected static Calendar sAge20BirthDate = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 18</P>
	 */
	protected static Calendar sAge18BirthDate = null;
	/**
	 * <P>Holds an instance of a Calendar object which contains a date that represents age 17</P>
	 */
	protected static Calendar sAge17BirthDate = null;
	
	protected static EmployerService employerService;
	protected static WorkerService workerService;
	protected static WorkerPaymentService workerPaymentService;
	protected static EmployerTaxSettingsService employerTaxSettingsService;
	
	public static String PSP_HOST = "http://sdgapppspqa11.corp.intuit.net";
		
	private static void initializeEAOs() {
		ServiceLocator serviceLocator = ServiceLocator.getInstance();
	    workerService = serviceLocator.getService(WorkerService.JNDI_NAME);
	    workerPaymentService = serviceLocator.getService(WorkerPaymentService.JNDI_NAME);
		employerService = serviceLocator.getService(EmployerService.JNDI_NAME);
		employerTaxSettingsService = serviceLocator.getService(EmployerTaxSettingsService.JNDI_NAME);

	  //Initialize properties for InitialContext, user login is set in the properties
	    RemoteServiceLocator.getInstance();
	}

	/**
	 * <P>Calls initializeTestData() to initialize the test data if the test data hasn't been seeded yet.</P>
	 * use a group in @BeforeClass, otherwise, @BeforeClass won't run, see http://jira.codehaus.org/browse/SUREFIRE-479
	 */
	@BeforeClass(groups={"serviceittests"})
	public static void setUpBasicTestBase()
	{		
		
		initializeEAOs();
		
	}

	/**
	 * <P>Method for cleaning up resources after all tests in the test suite have run.  Currently there is nothing to clean up.</P>
	 *
	 */
	@AfterClass
	public static void tearDownBasicTestBase()
	{
		// Nothing to tear down
	}
	
	@BeforeMethod
	public void setUp() throws NotSupportedException, SystemException{
		
		
	}
	
	@AfterMethod
	public void tearDown() throws IllegalStateException, SecurityException, SystemException{
		
		
		
	}
	
	public void setTaxServiceHost(String url) throws MalformedObjectNameException{
		
		GeminiConfigService rs = RemoteServiceLocator.getServiceFromJndi("GeminiConfigService");
		rs.setTaxServiceHost(url);
		
	}
	
	
	
	

}
