package com.intuit.sb.payroll.qa.services.employer;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.PTSEmployerServiceTestBase;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.NotificationAttribute;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.NotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.PTSNotificationDTO;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.services.taxes.EmployerTaxIdInfo;


/**
 * <P>The <CODE>PTSACHNotificationTests</CODE> class contains test methods for verifying the Server side behavior of Legal Info on ACH Notification : 
 * Failure and subsequent change/update of CAEAN in Gemini
 * $Author: kbommisetty $
 */
public class PTSStateIDModificationNotificationTest extends PTSEmployerServiceTestBase{
	
		
	/**
	 * <P>
	 * Test Gemini internal update of StateID , after Gemini receives State Id modification notification
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer 
	 * <LI>Create new Employer with random EIN</LI>
	 * <LI>Add current Employer to PSP
	 * <LI>Construct PTSNotificationDTO based on add Employer detail</LI>
	 * <LI>Mock the PTS inbound with chanedd CA Tax Id
	 * </OL>
	 * <OL>
	 * <LI>Verify that the CAEAN field in not same as what was sent earlier to PSP
	 * </LI>
	 * </OL>
	 * @throws Exception 
	 */
	@Test(enabled =false)
	public void testCAEANUpdateFromStateIdNotification() throws Exception{
		
		EmployerTaxIdInfo[] taxIdInfos = null;
		String ca_taxid= null;
	
		
		prepareCurrentEmployerForAssisted(null,null);
		//sEmployerService.sendCompanyInfoToPTS();
		
		taxIdInfos = sEmployerTaxSettingsService.getEmployerTaxIDs();
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos){
			
			if (employerTaxIdInfo.getTaxAgency().name.equals(AgencyEnum.CA_EDD.toString())){
				ca_taxid=employerTaxIdInfo.getTaxId();
			}
		}

		
		mockCATaxIdChangeNotification(ca_taxid,"22200222");
		
		taxIdInfos = sEmployerTaxSettingsService.getEmployerTaxIDs();
		
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos)
		{
			if (employerTaxIdInfo.getTaxAgency().name.contains("CA_EDD"))
			{
				Assert.assertTrue(employerTaxIdInfo.getTaxId().equals("22200222"), "The CA Taxid did not get updated CA TaxId");
			}
		}
		
		// update the tax id
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos)
		{
			if (employerTaxIdInfo.getTaxAgency().name.contains("CA_EDD"))
			{
				employerTaxIdInfo.setTaxId("44400444");
			}
		}
		
		try{
		
			sEmployerTaxSettingsService.updateEmployerTaxIDs(taxIdInfos);
			Assert.fail("UEXPECTED: update is possible after internal CA state id change by Engine");
		}
		catch(ValidationException ve){
			
		}

		
	}

	private void mockCATaxIdChangeNotification(String oldVal , String newVal) throws ServiceFailureException, PTSNotificationException{
		
		PTSNotificationDTO pNotificationDTO = createPTSNotiticationDTO(NotificationTypeEnum.STATE_ID_MODIFIED, null);
		
		NotificationAttribute nameAttribute = new NotificationAttribute();
		nameAttribute.setName(PTSNotificationRequestMapper.AGENCY_ID);
		nameAttribute.setValue(com.intuit.sb.payroll.capability.pts.adapter.dtos.AgencyEnum.CAEDD.toString());
		
		NotificationAttribute oldValue = new NotificationAttribute();
		oldValue.setName(PTSNotificationRequestMapper.OLD_VALUE);
		oldValue.setValue(oldVal);
		
		NotificationAttribute newValue = new NotificationAttribute();
		newValue.setName(PTSNotificationRequestMapper.NEW_VALUE);
		newValue.setValue(newVal);
		
		pNotificationDTO.getAttributes().add(nameAttribute);
		pNotificationDTO.getAttributes().add(oldValue);
		pNotificationDTO.getAttributes().add(newValue);
		
		mockUpdateCompanyActivationStatus(pNotificationDTO);

	}

}
