package com.intuit.sb.payroll.qa.services.employer;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.PTSEmployerServiceTestBase;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.taxpayments.FutaAmountType;
import com.intuit.sb.payroll.services.taxpayments.FutaPaymentLiability;
import com.intuit.sb.payroll.services.taxpayments.FutaPaymentsAndLiabilities;
import com.intuit.sb.payroll.services.taxpayments.PaymentPeriod;
import com.intuit.sb.payroll.services.taxpayments.TaxPaymentLineItem;
import com.intuit.sb.payroll.services.taxpayments.TaxPaymentService;
import com.intuit.sb.payroll.services.taxpayments.TaxPayments;
import com.intuit.sb.payroll.services.taxpayments.TaxPaymentsTestUtil;
import com.intuit.sb.payroll.services.taxpayments.ToDateTaxPayments;
import com.intuit.sb.payroll.services.taxpayments.impl.DateRangeMapper;
import com.intuit.sb.payroll.services.taxpayments.impl.TaxPaymentsMapper;

public class SubmitYTDPaymentsTest extends PTSEmployerServiceTestBase{
	private static TaxPaymentService sTaxPaymentService = null;
	
	@BeforeClass
	public void initializeQTDTaxPaymentsTestClass() throws Exception{
		
		sTaxPaymentService = qaBootstrap.getEjbFromClass(TaxPaymentService.class);		
			
	}
	
	
	@Test(enabled=false)// to be fixed by Dev--
	public void testOnlyCurrentQTRPaymentsSentOnAddCompany() throws Exception{		
		
		prepareCurrentEmployerForAssisted(null,null);		
		Calendar cal = Calendar.getInstance();		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1  -- current QTR
		Date currentQTRDate = cal.getTime();
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#2		-- current QTR
		
		cal.add(Calendar.MONTH, 4);
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("1000"));
		paymentsQTD.add(payment);						//Add Payment#3 	-- Future QTR
		Date futureQTRDate = cal.getTime();
		
		cal.add(Calendar.MONTH,-8);
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("1000"));
		paymentsQTD.add(payment);						//Add Payment#3 	-- Future QTR
		Date pastQTRDate = cal.getTime();		
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1,2 and 3	
		
		//Add company to PSP
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		// get ToDate TaxPayements				
		//Assert that payments not belonging to this QTR are not sent to PSP
		PaymentPeriod paymentPeriod = new PaymentPeriod();		
		paymentPeriod.setPeriodStartDate(futureQTRDate);
		paymentPeriod.setPeriodEndDate(DateUtil.getNextDate(futureQTRDate, 1));
		ToDateTaxPayments[] toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		Payment[] payments = TaxPaymentsMapper.mapTaxPaymentsDtoToPaymentEntity(toDateTaxPaymentsSaved[0]);
		Assert.assertFalse(StatusUpdateHandlerUtil.isEntityAddedToPTS(payments[0]),"Submit QTD Payments call not made");
		//Assert that the current QTR payments are sent to PSP
		paymentPeriod.setPeriodStartDate(currentQTRDate);
		paymentPeriod.setPeriodEndDate(DateUtil.getNextDate(currentQTRDate, 1));		
		toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		payments = TaxPaymentsMapper.mapTaxPaymentsDtoToPaymentEntity(toDateTaxPaymentsSaved[0]);		
		Assert.assertTrue(StatusUpdateHandlerUtil.isEntityAddedToPTS(payments[0]),"Submit QTD Payments call not made");
		Assert.assertTrue(StatusUpdateHandlerUtil.isEntityAddedToPTS(payments[1]),"Submit QTD Payments call not made");
		//Assert that the past QTR payments are also not sent to PSP
		paymentPeriod.setPeriodStartDate(pastQTRDate);
		paymentPeriod.setPeriodEndDate(DateUtil.getNextDate(pastQTRDate, 1));		
		toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		payments = TaxPaymentsMapper.mapTaxPaymentsDtoToPaymentEntity(toDateTaxPaymentsSaved[0]);		
		Assert.assertFalse(StatusUpdateHandlerUtil.isEntityAddedToPTS(payments[0]),"Submit QTD Payments call not made");
		
	}
	
	@Test(expectedExceptions=ValidationException.class,enabled=false)// expected ValidationException
	public void testSubmitFutureDatePaymentsToPSP() throws Exception{
		prepareCurrentEmployerForAssisted(null,null);
		
		Calendar cal = Calendar.getInstance();
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1
		
		cal.add(Calendar.WEEK_OF_YEAR, 4);
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("1000"));
		paymentsQTD.add(payment);						//Add Payment#2 	-- Future date payment but of current QTR
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1 and 2	
		
		//Add company to PSP-- submit ToDate taxPayments
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
	}
	
	@Test(enabled=false)
	public void testSubmitQTDPaymentsNotCalledIfNotExist() throws Exception{
		prepareCurrentEmployerForAssisted(null,null);
		
		Date periodStartDate = DateUtil.createDate(2009, Calendar.MARCH, 6);
		Calendar cal = Calendar.getInstance();
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1		
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1 		
		
		//Add company to PSPs
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);
		
		// get ToDate TaxPayements
		PaymentPeriod paymentPeriod = new PaymentPeriod();		
		Date periodEndDate = DateUtil.getNextDate(cal.getTime(), 1);		
		paymentPeriod.setPeriodStartDate(cal.getTime());
		paymentPeriod.setPeriodEndDate(periodEndDate);
		//ToDateTaxPayments[] toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		//Payment[] payments = TaxPaymentsMapper.mapTaxPaymentsDtoToPaymentEntity(toDateTaxPaymentsSaved[0]);
		
		//Assert no SubmitQTDPayments call is made????
		
	}
		
	@Test(expectedExceptions=ValidationException.class,enabled=false) // update is locked after first day of service
	public void testUpdateFailsAfterQTDPaymentsSubmitted() throws Exception{
		
		prepareCurrentEmployerForAssisted(null,null);
		
		Calendar cal = Calendar.getInstance();
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1
		Date payment1Date = cal.getTime();
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1 and 2	
		
		//Add company to PSP
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		// get ToDate TaxPayements
		PaymentPeriod paymentPeriod = new PaymentPeriod();		
		paymentPeriod.setPeriodStartDate(payment1Date);
		paymentPeriod.setPeriodEndDate(DateUtil.getNextDate(payment1Date, 1));
		ToDateTaxPayments[] toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		
		// TODO Logic for simulating first day of Service		
		
		//Update the TaxPayment
		// Edit taxPayment column added
		TaxPayments[] taxPayments = toDateTaxPaymentsSaved[0].getTaxPayments();
		TaxPaymentLineItem[] lineItems = taxPayments[0].getTaxPaymentLineItems();
		lineItems[0].setAmount(new BigDecimal("0"));		
		taxPayments[0].setTaxPaymentLineItems(lineItems);
		toDateTaxPaymentsSaved[0].setTaxPayments(taxPayments);		
		sTaxPaymentService.updateTaxPayments(toDateTaxPaymentsSaved);
		
	}
	
	@Test(enabled=false)// update is locked after first day of service and FUTA is not complete
	public void testUpdateFUTAInfoAfterQTDPaymentsSubmitted() throws Exception{
		prepareCurrentEmployerForAssisted(null,null);
		
		Date periodStartDate = DateUtil.createDate(2009, Calendar.MARCH, 5);
		Calendar cal = Calendar.getInstance();
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1		
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1 
		
		//Add Futa Payments
		FutaPaymentLiability futaPayment = new FutaPaymentLiability();
		futaPayment.setFutaAmountType(FutaAmountType.FUTAPAYMENT);
		futaPayment.setAmount(new BigDecimal("100"));
		FutaPaymentsAndLiabilities futaPayments = new FutaPaymentsAndLiabilities();
		futaPayments.setFutaPayment(futaPayment);
		com.intuit.sb.payroll.entity.model.DateRange computedRangeForQ2 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER2, 2009);
		PaymentPeriod quarterPaymentPeriod = DateRangeMapper. mapDateRangeToPaymentPeriod(computedRangeForQ2);
		futaPayments.setQuarterPaymentPeriod(quarterPaymentPeriod);
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();		
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);
		
		//Add company to PSP
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);
		
		// get ToDate TaxPayements
		//List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
		
		//Update FUTA payments
		
		
	}
	
	@Test(enabled=false)//FUTA is not sent, yet to be implemented by Dev
	public void testFUTAPaymentsSentIfExistOnSubmitQTDPaymentsCall() throws Exception{
		
		prepareCurrentEmployerForAssisted(null,null);
		
		Date paymentDate = DateUtil.createDate(2009, Calendar.MARCH, 5);
		
		// Add FUTA payments
		FutaPaymentLiability futaPayment = new FutaPaymentLiability();
		futaPayment.setFutaAmountType(FutaAmountType.FUTAPAYMENT);
		futaPayment.setAmount(new BigDecimal("100"));
		FutaPaymentsAndLiabilities futaPayments = new FutaPaymentsAndLiabilities();
		futaPayments.setFutaPayment(futaPayment);
		com.intuit.sb.payroll.entity.model.DateRange computedRangeForQ2 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER2, 2009);
		PaymentPeriod quarterPaymentPeriod = DateRangeMapper. mapDateRangeToPaymentPeriod(computedRangeForQ2);
		futaPayments.setQuarterPaymentPeriod(quarterPaymentPeriod);
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();		
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);
		
		//Add company to PSP
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		// get ToDate TaxPayements
		PaymentPeriod paymentPeriod = new PaymentPeriod();		
		Date periodEndDate = DateUtil.getNextDate(paymentDate, 1);		
		paymentPeriod.setPeriodStartDate(paymentDate);
		paymentPeriod.setPeriodEndDate(periodEndDate);
//		ToDateTaxPayments[] toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
//		Payment[] payments = TaxPaymentsMapper.mapTaxPaymentsDtoToPaymentEntity(toDateTaxPaymentsSaved[0]);
//		
		//Assert no SubmitQTDPayments call is made????
		
		
	}
	
	
	
	
	
	
	
}
