package com.intuit.sb.payroll.qa.reports;

import java.util.Calendar;
import java.util.Date;

import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponentTestBase;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.reports.ReportDetail;
import com.intuit.sb.payroll.reports.ReportRequest;
import com.intuit.sb.payroll.reports.ReportService;
import com.intuit.sb.payroll.reports.ReportSubtotals;
import com.intuit.sb.payroll.reports.ReportType;

/**
 * <P>
 * The <CODE>GetReportExceptionsTest</CODE> class contains tests for testing exceptions that can be thrown by
 *  getReport() in ReportService
 * </P>
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 * 
 */
public class GetReportExceptionsTest extends WorkerPaymentsComponentTestBase {
	private static ReportService reportService = null;
	
	@BeforeClass
	public static void setUpPayrollRunAdjustmentsTest() throws Exception {
    	reportService = qaBootstrap.getEjbFromClass(ReportService.class);
	}
	
    /**
     * <P>Test calling ReportService.getReport() with a null ReportRequest.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() with a null ReportRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullReportRequest() throws Exception {
		// Exercise
		reportService.getReport(null);
	}

    /**
     * <P>Test calling ReportService.getReport() with a null "From" date.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() with a null "From" date</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullFromDate() throws Exception {
		// Setup
		String reportTitle = "2008";
		Date toDate = DateUtil.createDate(2008, Calendar.JANUARY, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, null, toDate, 
				ReportSubtotals.NOT_SET, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		reportService.getReport(reportRequest);
	}

    /**
     * <P>Test calling ReportService.getReport() with a null "To" date.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() with a null "To" date</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullToDate() throws Exception {
		// Setup
		String reportTitle = "2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, null, 
				ReportSubtotals.NOT_SET, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		reportService.getReport(reportRequest);
	}

    /**
     * <P>Test calling ReportService.getReport() with a "From" date that is after the "To" date.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() with a "From" date after the "To" date</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions = ValidationException.class)
	public void testFromDateAfterToDate() throws Exception {
		// Setup
		String reportTitle = "2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.DECEMBER, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.JANUARY, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.NOT_SET, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		reportService.getReport(reportRequest);
	}

}
