package com.intuit.sb.payroll.qa.services.employer;

import java.util.HashSet;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Tests the getWorkLocation operation of the EmployerService.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetWorkLocationTest extends EmployerServiceTestBase
{
    private static String newWorkLocationName = "Temp Location";
    
    private boolean verifyWorkLocations(List<com.intuit.sb.payroll.entity.model.WorkLocation> expectedLocation, WorkLocation[] computedLocations,
                                        String lineItemTypeStr)
    {
        boolean compareResult = expectedLocation.size() == computedLocations.length;
        if (true == compareResult)
        {
            HashSet<WorkLocation> computedLocationSet = new HashSet<WorkLocation>();
            for(int i=0; i < computedLocations.length; i++)
            {
                computedLocationSet.add(computedLocations[i]);
            } 
            HashSet<WorkLocation> expectedLocationSet = new HashSet<WorkLocation>();
            for(int i=0; i < expectedLocation.size(); i++)
            {
            	com.intuit.sb.payroll.entity.model.WorkLocation location = expectedLocation.get(i);
                WorkLocation workLocation = new WorkLocation(new ObjectHandle(location.getId(), location.getVersion()), location.getName(),location.isDefaultt());
                //Do it Slowly
                GlobalPostalAddress addrEntity = location.getTaxResidenceAddress();
                USPostalAddress usAddr  = AddressConvertor.getUSFromGlobal(addrEntity, 
                		new USPostalAddressImpl());               	           	
                PhysicalAddress mappedAddress = mapEntityAddressToPhysicalAddress(usAddr);
                workLocation.setAddress(mappedAddress);
                TelephoneNumber mainPhone = location.getMainPhone();
                if (mainPhone != null) {
                    workLocation.setPhoneNumber(mainPhone.getFreeFormNumber());
                    workLocation.setExtn(mainPhone.getExtension());
                }
                // fix up the workerCount value.  Seeded workLocation has 1 worker each, new workLocation has 0 worker
                if (newWorkLocationName.equals(location.getName()))
                {
                    workLocation.setWorkerCount(0);
                }
                else
                {
                    workLocation.setWorkerCount(1);
                }
                expectedLocationSet.add(workLocation);
            }
            compareResult = computedLocationSet.equals(expectedLocationSet);
        }
        if (false == compareResult)
        {
            sLogger.error("Computed " + lineItemTypeStr);            
            for (int i=0; i < computedLocations.length; i++)
            {
                sLogger.error("\tName=" + computedLocations[i].getName() + ",  uniqueId=" + computedLocations[i].getObjectHandle().getId());            
            }            
            sLogger.error("Expected " + lineItemTypeStr);            
            for (int i=0; i < expectedLocation.size(); i++)
            {
                sLogger.error("\tName=" + expectedLocation.get(i).getName() + ",  Id=" + expectedLocation.get(i).getId());            
            }            
        }
        return compareResult;
    }

    private com.intuit.sb.payroll.entity.model.WorkLocation createWorkLocationInDatabase()
    {
        com.intuit.sb.payroll.entity.model.WorkLocation workLocation = new com.intuit.sb.payroll.entity.model.WorkLocation();
        workLocation.setName(newWorkLocationName);
        TelephoneNumber mainPhone = new TelephoneNumber();
        mainPhone.setFreeFormNumber("650-944-9999");
        mainPhone.setExtension("1234");
        workLocation.setMainPhone(mainPhone);
        return workLocation;
    }
    
    private PhysicalAddress mapEntityAddressToPhysicalAddress(USPostalAddress entityAddress) {
        if (entityAddress != null) {
            PhysicalAddress physicalAddress = new PhysicalAddress();
            physicalAddress.setAddressLine1(entityAddress.getAddressLine1());
            physicalAddress.setAddressLine2(entityAddress.getAddressLine2());
            physicalAddress.setCity(entityAddress.getCity());
            physicalAddress.setStateOrRegion(entityAddress.getRegionOrState());
            physicalAddress.setZipOrPostalCode(entityAddress.getZipOrPostalCode());
            physicalAddress.setCountry(entityAddress.getCountry());
            return physicalAddress;
        } else {
            return null;
        }
    }
    
    /**
     * <P>Test getting existing WorkLocations</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call Repository command to get the list of workLocations in the database</LI>
     * <LI>Call WorkerService's getWorkLocations command to get the list of workLocations</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that list returned matches expected result
     * </UL>
     */
    @Test
    public void testGetExistingWorkLocations() throws Exception
    {
        List<com.intuit.sb.payroll.entity.model.WorkLocation>  existingLocationList = entityService.findAll(com.intuit.sb.payroll.entity.model.WorkLocation.class);
        WorkLocation[] workLocations = sEmployerService.getWorkLocations();
        AssertJUnit.assertTrue("WorkLocations mismatch!", verifyWorkLocations(existingLocationList, workLocations, "WorkLocations"));
     }

    /**
     * <P>Test getting WorkLocations after adding additional workLocation</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Add a new workLocation into the database</LI>
     * <LI>Call Repository command to get the list of workLocations in the database</LI>
     * <LI>Call WorkerService's getWorkLocations command to get the list of workLocations</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that list returned matches expected result
     * </UL>
     */
    @Test
    public void testGetWorkLocationsAfterAdding() throws Exception
    {
        com.intuit.sb.payroll.entity.model.WorkLocation workLocation = null;
        try
        {
            workLocation = createWorkLocationInDatabase();
            List<com.intuit.sb.payroll.entity.model.WorkLocation>  existingLocationList = entityService.findAll(com.intuit.sb.payroll.entity.model.WorkLocation.class);
            WorkLocation[] workLocations = sEmployerService.getWorkLocations();
            AssertJUnit.assertTrue("WorkLocations mismatch!", verifyWorkLocations(existingLocationList, workLocations, "WorkLocations"));
        }
        finally
        {
            if (null != workLocation)
            {
                entityService.makeTransient(workLocation);
            }
        }
    }

}
