package com.intuit.sb.payroll.qa.services.employer;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.naming.CannotProceedException;

import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentLineItem;
import com.intuit.sb.payroll.entity.model.PaymentLineItemSplit;
import com.intuit.sb.payroll.entity.model.PaymentType;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.taxpayments.PaymentPeriod;
import com.intuit.sb.payroll.services.taxpayments.PaymentTypeLiability;
import com.intuit.sb.payroll.services.taxpayments.TaxPaymentLineItem;
import com.intuit.sb.payroll.services.taxpayments.TaxPaymentService;
import com.intuit.sb.payroll.services.taxpayments.TaxPayments;
import com.intuit.sb.payroll.services.taxpayments.TaxPaymentsTestUtil;
import com.intuit.sb.payroll.services.taxpayments.ToDateTaxPayments;
import com.intuit.sb.payroll.services.taxpayments.impl.PaymentTypeLawMapper;
import com.intuit.sb.payroll.services.taxpayments.impl.TaxPaymentsMapper;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

public class QTDTaxPaymentsTest extends WorkerPaymentServiceTestBase{
	private static TaxPaymentService sTaxPaymentService = null;
	HashMap<String,Money> taxLineItemsMap = null; //Expected Liabilities of the Employer	
	
	@BeforeClass
	public void initializeQTDTaxPaymentsTestClass() throws Exception{
		
		sTaxPaymentService = qaBootstrap.getEjbFromClass(TaxPaymentService.class);		
				
	}
	
	
		
	
	/**
     * <P>Test retrieving liabilities by TaxPaymentService for a period where there are no Payrolls</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>set the Payment period where there are no payrolls</LI>
     * <LI>call retrieveLiabilitiesForGivenPayment() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that List returned is not null and the line items returned are all having a value of '0'</LI>
     * </UL>
     */
	@Test
	public void testRetrieveLiabilitiesForNoPayrollsInPeriod() throws Exception{
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		paymentPeriod.setPeriodStartDate( DateUtil.createDate(2009, Calendar.JUNE, 5));
		paymentPeriod.setPeriodEndDate(DateUtil.createDate(2009, Calendar.JULY, 6));
		
		//getTotalLaiability for the payment period		
		List<PaymentTypeLiability> qtdLiabilityList = sTaxPaymentService.getLiabilitiesFromExistingPayrollRuns(paymentPeriod);
		Assert.assertNotNull(qtdLiabilityList,"null QTD Liabilities found");
		
		//Check each line item is of value zero		
	}	
	
	/**
     * <P>Test retrieving liabilities by TaxPaymentService</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create two payroll ready employees</LI>
     * <LI>Run two payroll including these 2 employees</LI>
     * <LI>Calculate the total Liability items for employer</LI>
     * <LI>call retrieveLiabilitiesForGivenPayment() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that the line items returned are same as calculated from employee Paychecks</LI>
     * </UL>
     */
	@Test
	public void testRetrieveLiabilities()throws Exception{
		taxLineItemsMap = new HashMap<String,Money>();
		initliazeTaxLineItemsMap();
		Worker worker1 = WorkerTestUtils.createAPayrollReadyWorker("Abhijit", sSanFranciscoLocation);
		Worker worker2 = WorkerTestUtils.createAPayrollReadyWorker("Sawanth", sSanFranciscoLocation);
		
		Long[] workerIds = new Long[2];
		workerIds[0] = worker1.getId();
		workerIds[1] = worker2.getId();
		Date payperiodEndDate = DateUtil.createDate(2009, Calendar.JUNE, 5);
		Date periodStartDate = payperiodEndDate;
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, payperiodEndDate);
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
		
		Calendar cal = Calendar.getInstance();
		cal.setTime(payperiodEndDate);
		cal.add(Calendar.WEEK_OF_YEAR, 2);
		payperiodEndDate = cal.getTime();
		payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, payperiodEndDate);
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
		
		getLiabitiesForEmployee(worker1);
		getLiabitiesForEmployee(worker2);
		Map<String, BigDecimal> qtdLiabilityFromPaychecks = mapToTaxPaymentsLineItems(taxLineItemsMap);
		
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		cal.add(Calendar.WEEK_OF_YEAR, 2);
		Date d = cal.getTime();
		d = DateUtil.getNextDate(d, 1);	//	add one day
		paymentPeriod.setPeriodEndDate(d);
		cal.setTime(periodStartDate);
		cal.add(Calendar.WEEK_OF_YEAR, -1);		
		paymentPeriod.setPeriodStartDate(cal.getTime());
		
		
		//getTotalLaiability for the payment period		
		List<PaymentTypeLiability> qtdLiabilityList = sTaxPaymentService.getLiabilitiesFromExistingPayrollRuns(paymentPeriod);
		
		//Verify each line item of Employer Liabilities in QTD with the one calculated from all Paychecks
		for(PaymentTypeLiability paymentTypeLiability:qtdLiabilityList){
			String paymentType = paymentTypeLiability.getPaymentType();
			if(!paymentType.equals("NOTSET")){
				Assert.assertTrue(qtdLiabilityFromPaychecks.get(paymentType).equals(paymentTypeLiability.getLiability()),
						"Liability amount not matched for type"+" "+paymentType);
			}
		}		
	}
	
	
	/**
     * <P>Test retrieving liabilities by TaxPaymentService for a null payment period</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>set the Payment period to null</LI>
     * <LI>call retrieveLiabilitiesForGivenPayment() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Validation exception is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testRetrieveLiabilitiesForNullPeriod() throws Exception{
		
		//getTotalLaiability for the payment period		
		List<PaymentTypeLiability> qtdLiabilityList = sTaxPaymentService.getLiabilitiesFromExistingPayrollRuns(null);
		Assert.assertNull(qtdLiabilityList,"QTD liabilities are not null");
			
	}
	
	/**
     * <P>Test retrieving liabilities by TaxPaymentService for a payment period where start date is null</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>set the Payment period with start date as null</LI>
     * <LI>call retrieveLiabilitiesForGivenPayment() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Validation exception is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testRetrieveLiabilitiesForNullPeriodStartDate() throws Exception{
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		paymentPeriod.setPeriodStartDate( null);
		paymentPeriod.setPeriodEndDate(DateUtil.createDate(2009, Calendar.JULY, 5));
		
		//getTotalLaiability for the payment period		
		List<PaymentTypeLiability> qtdLiabilityList = sTaxPaymentService.getLiabilitiesFromExistingPayrollRuns(paymentPeriod);
		Assert.assertNull(qtdLiabilityList,"QTD liabilities are not null");			
	}
	
	/**
     * <P>Test retrieving liabilities by TaxPaymentService for a payment period where end date is null</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>set the Payment period with end date as null</LI>
     * <LI>call retrieveLiabilitiesForGivenPayment() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Validation exception is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testRetrieveLiabilitiesForNullPeriodEndDate() throws Exception{
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		paymentPeriod.setPeriodStartDate(DateUtil.createDate(2009, Calendar.JULY, 5));
		paymentPeriod.setPeriodEndDate(null);
		
		//getTotalLaiability for the payment period		
		List<PaymentTypeLiability> qtdLiabilityList = sTaxPaymentService.getLiabilitiesFromExistingPayrollRuns(paymentPeriod);
		Assert.assertNull(qtdLiabilityList,"QTD liabilities are not null");			
	}
	
	
	/**
     * <P>Test Adding tax payments by TaxPaymentService</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a tax payment with Payment date as June5th 2009</LI>
     * <LI>Create another tax payment with Payment date as June19th 2009</LI>
     * <LI>add these tax payments by calling UpdateTaxPayments()</LI>
     * </OL>
     * <UL>
     * <LI>call getToDateTaxPayments()</LI>
     * <LI>Verify that the tax payments retrieved are same as tax payments added</LI>
     * </UL>
     */
	@Test		 
	public void testAddTaxPayments() throws Exception{
		taxLineItemsMap = new HashMap<String,Money>();
		initliazeTaxLineItemsMap();		
		
		Date periodStartDate = DateUtil.createDate(2009, Calendar.JUNE, 5);
		Calendar cal = Calendar.getInstance();
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1		
		cal.add(Calendar.WEEK_OF_YEAR, 2);
		Date payperiodEndDate = cal.getTime();
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("1000"));
		paymentsQTD.add(payment);						//Add Payment#2
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1 and 2
		
		// get ToDate TaxPayements
		PaymentPeriod paymentPeriod = new PaymentPeriod();				
		Date periodEndDate = DateUtil.getNextDate(payperiodEndDate, 1);		
		paymentPeriod.setPeriodStartDate(periodStartDate);
		paymentPeriod.setPeriodEndDate(periodEndDate);
		ToDateTaxPayments[] toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		// compare both all the values
		Assert.assertEquals(toDateTaxPaymentsSaved.length, 1); // Since the paymentPeriod spans only one quarter
		Boolean same = TaxPaymentsTestUtil.compareToDateTaxPayments(toDateTaxPayments[0],toDateTaxPaymentsSaved[0] );
		Assert.assertTrue(same, "Expected and Saved ToDateTaxPayments are not same");
		//Verify the split amounts of 94x
		Payment[] payments = TaxPaymentsMapper.mapTaxPaymentsDtoToPaymentEntity(toDateTaxPayments[0]);
		Assert.assertTrue(verify94xSplit(payments, taxLineItemsMap),"Error in split logic");
		
	}
	
	/**
     * <P>Test getToDateTaxPayments for payment period where start date and end date are same as tax payment date</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a tax payment with Payment date as June5th 2009</LI>
     * <LI>Create another tax payment with Payment date as June19th 2009</LI>
     * <LI>add these tax payments by calling UpdateTaxPayments()</LI>
     * </OL>
     * <UL>
     * <LI>call getToDateTaxPayments() with payment period where start date and end date are same as tax payment date</LI>
     * <LI>Verify that at least one tax payment is retrieved</LI>
     * </UL>
     */
	@Test
	public void testGetToDateTaxPaymentsForPaymentPeriodIsSingleDay() throws Exception{
		Date periodStartDate = DateUtil.createDate(2009, Calendar.JUNE, 5);
		Calendar cal = Calendar.getInstance();
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1 for June 5th
		cal.add(Calendar.DAY_OF_YEAR, 1);
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("1000"));
		paymentsQTD.add(payment);						//Add Payment#2 for June 6th
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1&2 
		
		PaymentPeriod paymentPeriodNew = new PaymentPeriod();
		paymentPeriodNew.setPeriodStartDate(periodStartDate);
		paymentPeriodNew.setPeriodEndDate(cal.getTime());
		
		ToDateTaxPayments[] toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriodNew);
		Assert.assertEquals(toDateTaxPaymentsSaved.length, 1);
		//Assert only one payment is retrieved 
		Assert.assertEquals(toDateTaxPaymentsSaved[0].getTaxPayments().length,1);
		// compare the dates
		TaxPayments[] taxPayments = toDateTaxPaymentsSaved[0].getTaxPayments();
		Assert.assertTrue(taxPayments[0].getTaxPaymentDate().equals(periodStartDate),"Expected Payment not retrieved");
					
	}
	
	/**
     * <P>Test  UpdateTaxPayments of TaxPaymentService</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a tax payment with Payment date as June5th 2009</LI>
     * <LI>Create another tax payment with Payment date as June19th 2009</LI>
     * <LI>add these tax payments by calling UpdateTaxPayments()</LI>
     * <LI>Update one of the line item of a TaxPayment by calling UpdateTaxPayments()</LI>
     * </OL>
     * <UL>
     * <LI>call getToDateTaxPayments()</LI>
     * <LI>Verify that retrieved tax payment is same as updated</LI>
     * </UL>
     */
	@Test
	public void testUpdateTaxPayments() throws Exception{
		Date periodStartDate = DateUtil.createDate(2009, Calendar.JUNE, 5);
		Calendar cal = Calendar.getInstance();
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1		
		cal.add(Calendar.WEEK_OF_YEAR, 2);
		Date payperiodEndDate = cal.getTime();
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("1000"));
		paymentsQTD.add(payment);						//Add Payment#2
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1 and 2
		
		// get ToDate TaxPayements
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		Date periodEndDate = DateUtil.getNextDate(payperiodEndDate, 1);		
		paymentPeriod.setPeriodStartDate(periodStartDate);
		paymentPeriod.setPeriodEndDate(periodEndDate);
		ToDateTaxPayments[] toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);		
		
		// Edit taxPayment column added in above test
		TaxPayments[] taxPayments = toDateTaxPaymentsSaved[0].getTaxPayments();
		TaxPaymentLineItem[] lineItems = taxPayments[0].getTaxPaymentLineItems();
		lineItems[0].setAmount(new BigDecimal("0"));		
		toDateTaxPaymentsSaved[0].setTaxPayments(taxPayments);		
		sTaxPaymentService.updateTaxPayments(toDateTaxPaymentsSaved);
		
		// get ToDate TaxPayements
		ToDateTaxPayments[] toDateTaxPaymentsUpdated = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		
		// compare both all the values	
		Assert.assertEquals(toDateTaxPaymentsUpdated.length, 1); // Since the paymentPeriod spans only one quarter
		Boolean same = TaxPaymentsTestUtil.compareToDateTaxPayments(toDateTaxPaymentsSaved[0],toDateTaxPaymentsUpdated[0] );
		Assert.assertTrue(same, "Expected and Saved ToDateTaxPayments are not same");
		
	}
	
	/**
     * <P>Test  UpdateTaxPayments with negative values of TaxPaymentService</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a tax payment with Payment date as June5th 2009</LI>
     * <LI>Create another tax payment with Payment date as June19th 2009</LI>
     * <LI>add these tax payments by calling UpdateTaxPayments()</LI>
     * <LI>Update one of the line item to negative value of a TaxPayment by calling UpdateTaxPayments()</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Validation exception is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions = ValidationException.class)
	public void testUpdateTaxPaymentsWithNegativeValues() throws Exception{
		
		Date periodStartDate = DateUtil.createDate(2009, Calendar.JUNE, 5);
		Calendar cal = Calendar.getInstance();
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1		
		cal.add(Calendar.WEEK_OF_YEAR, 2);
		Date payperiodEndDate = cal.getTime();
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("1000"));
		paymentsQTD.add(payment);						//Add Payment#2
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1 and 2
		
		// get ToDate TaxPayements
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		Date periodEndDate = DateUtil.getNextDate(payperiodEndDate, 1);		
		paymentPeriod.setPeriodStartDate(periodStartDate);
		paymentPeriod.setPeriodEndDate(periodEndDate);
		ToDateTaxPayments[] toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);		
		
		// Edit taxPayment column added in above test
		TaxPayments[] taxPayments = toDateTaxPaymentsSaved[0].getTaxPayments();
		TaxPaymentLineItem[] lineItems = taxPayments[0].getTaxPaymentLineItems();
		lineItems[0].setAmount(new BigDecimal("-15.8"));		
		taxPayments[0].setTaxPaymentLineItems(lineItems);
		toDateTaxPaymentsSaved[0].setTaxPayments(taxPayments);		
		sTaxPaymentService.updateTaxPayments(toDateTaxPaymentsSaved);
		
		// get ToDate TaxPayements
		sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
				
	}
	
	/**
     * <P>Test  removeTaxPayments of TaxPaymentService</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a tax payment with Payment date as June5th 2009</LI>
     * <LI>Create another tax payment with Payment date as June19th 2009</LI>
     * <LI>add these tax payments by calling UpdateTaxPayments()</LI>
     * <LI>call removeTaxPayments()</LI>
     * </OL>
     * <UL>
     * <LI>call getToDateTaxPayments()</LI>
     * <LI>Verify that retrieved ToDateTaxPayment don't have tax payment</LI>
     * </UL>
     */
	@Test
	public void testRemoveTaxPaymentsForPeriod() throws Exception{
		Date periodStartDate = DateUtil.createDate(2009, Calendar.JUNE, 5);
		Calendar cal = Calendar.getInstance();
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1		
		cal.add(Calendar.WEEK_OF_YEAR, 2);
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("1000"));
		paymentsQTD.add(payment);						//Add Payment#2
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1 and 2
		
		// get ToDate TaxPayements
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		Date periodEndDate = DateUtil.getNextDate(cal.getTime(), 1);		
		paymentPeriod.setPeriodStartDate(periodStartDate);
		paymentPeriod.setPeriodEndDate(periodEndDate);
		
		//Remove the TaxPayment added in the above test
		sTaxPaymentService.removeTaxPayments(paymentPeriod);
		
		//get Todate TaxPayments
		ToDateTaxPayments[] toDateTaxPaymentsDeleted = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		
		//Check if removed
		Assert.assertEquals(toDateTaxPaymentsDeleted.length,0,"Tax Payments not removed");
		// check size is zero		
	}	
	
	/**
     * <P>Test  removeTaxPayments of TaxPaymentService for a period which is null</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>call removeTaxPayments() for payment period which is null</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Validation exception is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testRemoveTaxPaymentsForNullPeriod() throws Exception{
		
		sTaxPaymentService.removeTaxPayments(null);				
	}
	
	
	
	/**
     * <P>Test  getToDateTaxPayments for payment period where start date is null</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a tax payment with Payment date as June5th 2009</LI>
     * <LI>add the tax payments by calling UpdateTaxPayments()</LI>
     * <LI>call getToDateTaxPayments() for payment period where start date is null and end date is June6th</LI>
     * </OL>
     * <UL>
     * <LI>Verify that validation exception is thrown</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"}) // to be fixed by dev
	public void testGetToDateTaxPaymentsForNullStartDate() throws Exception{
		Date periodStartDate = DateUtil.createDate(2009, Calendar.JUNE, 5);
		Calendar cal = Calendar.getInstance();
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1		
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1
		
		// get ToDate TaxPayements
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		Date periodEndDate = DateUtil.getNextDate(cal.getTime(), 1);		
		paymentPeriod.setPeriodStartDate(null);
		paymentPeriod.setPeriodEndDate(periodEndDate);
		sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		
	}
	
	/**
     * <P>Test  getToDateTaxPayments for payment period where start date is null</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a tax payment with Payment date as June5th 2009</LI>
     * <LI>add the tax payments by calling UpdateTaxPayments()</LI>
     * <LI>call getToDateTaxPayments() for payment period where end date is null and start date is June5th</LI>
     * </OL>
     * <UL>
     * <LI>Verify that validation exception is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions = ValidationException.class)
	public void testGetToDateTaxPaymentsForNullEndDate() throws Exception{
		Date periodStartDate = DateUtil.createDate(2009, Calendar.JUNE, 5);
		Calendar cal = Calendar.getInstance();
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1		
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1		
		
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		paymentPeriod.setPeriodStartDate(periodStartDate);
		paymentPeriod.setPeriodEndDate(null);
		// get ToDate TaxPayements
		sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		
	}
	
	@Test
	public void test94xSplitsFor94xLiabilitiesAllPostive() throws Exception{
		taxLineItemsMap = new HashMap<String,Money>();
		initliazeTaxLineItemsMap();
		
		Worker worker1 = WorkerTestUtils.createAPayrollReadyWorker("Abhijit", sSanFranciscoLocation);
		Worker worker2 = WorkerTestUtils.createAPayrollReadyWorker("Sawanth", sSanFranciscoLocation);
		
		Long[] workerIds = new Long[2];
		workerIds[0] = worker1.getId();
		workerIds[1] = worker2.getId();
		Date payperiodEndDate = DateUtil.createDate(2009, Calendar.JUNE, 5);
		Date periodStartDate = payperiodEndDate;
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, payperiodEndDate);
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
		
		Calendar cal = Calendar.getInstance();
		cal.setTime(payperiodEndDate);
		cal.add(Calendar.WEEK_OF_YEAR, 2);
		payperiodEndDate = cal.getTime();
		payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, payperiodEndDate);
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
		
		getLiabitiesForEmployee(worker1);
		getLiabitiesForEmployee(worker2);
		
		//add Payments
		Date payment1Date = DateUtil.createDate(2009, Calendar.JUNE, 5);
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1		
		cal.add(Calendar.WEEK_OF_YEAR, 2);
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("1000"));
		paymentsQTD.add(payment);						//Add Payment#2
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1 and 2
		
		// get ToDate TaxPayements
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		Date periodEndDate = DateUtil.getNextDate(cal.getTime(), 1);		
		paymentPeriod.setPeriodStartDate(payment1Date);
		paymentPeriod.setPeriodEndDate(periodEndDate);
		ToDateTaxPayments[] toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		
		//Verify the split amounts of 94x
		Payment[] payments = TaxPaymentsMapper.mapTaxPaymentsDtoToPaymentEntity(toDateTaxPaymentsSaved[0]);
		Assert.assertTrue(verify94xSplit(payments, taxLineItemsMap),"Error in split logic");
		
	}
	
	@Test
	public void test94xSplitsFor94xLiabilityItemsWithZeroValues() throws Exception{
		taxLineItemsMap = new HashMap<String,Money>();
		initliazeTaxLineItemsMap();
		
		Worker worker1 = WorkerTestUtils.createAPayrollReadyWorker("Abhijit", sSanFranciscoLocation);
		Worker worker2 = WorkerTestUtils.createAPayrollReadyWorker("Sawanth", sSanFranciscoLocation);
		Set<CompensationDtl> compDtl = worker1.getCompensationDtls();		
		Iterator<CompensationDtl> compItr = compDtl.iterator();
		while(compItr.hasNext()){
			CompensationDtl compensationDtl = compItr.next();
			if(compensationDtl.getCompensationType().equals(CompensationTypeEnum.BONUS)){
				CompensationDtl compensationDtlUpdated = compensationDtl;
				worker2.getCompensationDtls().remove(compensationDtl);
				compensationDtlUpdated.getRate().setValue1AsUSDMoneyAmount(new BigDecimal("100"));
				worker2.getCompensationDtls().add(compensationDtlUpdated);
			}
		}
		worker1 = sWorkerComponent.updateWorker(worker1);
		
		compDtl = worker2.getCompensationDtls();		
		compItr = compDtl.iterator();
		while(compItr.hasNext()){
			CompensationDtl compensationDtl = compItr.next();
			if(compensationDtl.getCompensationType().equals(CompensationTypeEnum.BONUS)){
				CompensationDtl compensationDtlUpdated = compensationDtl;
				worker2.getCompensationDtls().remove(compensationDtl);
				compensationDtlUpdated.getRate().setValue1AsUSDMoneyAmount(new BigDecimal("100"));
				worker2.getCompensationDtls().add(compensationDtlUpdated);
			}
		}
		worker2 = sWorkerComponent.updateWorker(worker2);
		
		Long[] workerIds = new Long[2];
		workerIds[0] = worker1.getId();
		workerIds[1] = worker2.getId();
		Date payperiodEndDate = DateUtil.createDate(2009, Calendar.JUNE, 5);
		Date periodStartDate = payperiodEndDate;
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, payperiodEndDate);
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
		
		Calendar cal = Calendar.getInstance();
		cal.setTime(payperiodEndDate);
		cal.add(Calendar.WEEK_OF_YEAR, 2);
		payperiodEndDate = cal.getTime();
		payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, payperiodEndDate);
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
		
		getLiabitiesForEmployee(worker1);
		getLiabitiesForEmployee(worker2);
		
		//add Payments
		Date payment1Date = DateUtil.createDate(2009, Calendar.JUNE, 5);
		cal.setTime(periodStartDate);		
		
		//Exercise  // add taxPayment column
		List<Payment> paymentsQTD = new ArrayList<Payment>();		
		Payment payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("500"));
		paymentsQTD.add(payment);						//Add Payment#1		
		cal.add(Calendar.WEEK_OF_YEAR, 2);
		payment = TaxPaymentsTestUtil.addPaymentWithDateAnd94xItem(cal, new Money("1000"));
		paymentsQTD.add(payment);						//Add Payment#2
		
		ToDateTaxPayments[] toDateTaxPayments = new ToDateTaxPayments[1];
		toDateTaxPayments[0] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(paymentsQTD);
		sTaxPaymentService.updateTaxPayments(toDateTaxPayments);	//Save Payments 1 and 2
		
		// get ToDate TaxPayements
		PaymentPeriod paymentPeriod = new PaymentPeriod();
		Date periodEndDate = DateUtil.getNextDate(cal.getTime(), 1);		
		paymentPeriod.setPeriodStartDate(payment1Date);
		paymentPeriod.setPeriodEndDate(periodEndDate);
		ToDateTaxPayments[] toDateTaxPaymentsSaved = sTaxPaymentService.getToDateTaxPayments(paymentPeriod);
		
		//Verify the split amounts of 94x
		Payment[] payments = TaxPaymentsMapper.mapTaxPaymentsDtoToPaymentEntity(toDateTaxPaymentsSaved[0]);
		Assert.assertTrue(verify94xSplit(payments, taxLineItemsMap),"Error in split logic");
		
	}
		
		
	/**
	 * gets all the Liabilities for this employee
	 * 
	 * @param Worker
	 */	
	public void getLiabitiesForEmployee(Worker worker) throws Exception{
		
		Set<Paycheck> paychecks = worker.getPaychecks();
		Iterator<Paycheck> paycheckIterator = paychecks.iterator();
		Money money = null;
		
		while(paycheckIterator.hasNext()){
			
			Paycheck paycheck = paycheckIterator.next();			
			Set<PaycheckLineItem> lineItems = paycheck.getPaycheckLineItems();
			
			Iterator<PaycheckLineItem> lineitemIterator = lineItems.iterator();
			while(lineitemIterator.hasNext()){
				PaycheckLineItem lineItem = lineitemIterator.next();
				String lawIdValue = lineItem.getLawId().getValue();
				if(TaxPaymentsTestUtil.getAllLawIdsList2().contains(lawIdValue)){
					if(lineItem.getValue()!=null){
					money = lineItem.getValue();					
					Money money1 = taxLineItemsMap.get(lawIdValue);
					money = money.add(money1);
					taxLineItemsMap.put(lawIdValue,money);
					}
				}
					
			}			
		}
	}
	
	/**
	 * Maps taxLineItemsMap from Paycheck to QTD TaxPayment Line Items
	 * 
	 * @param HashMap<String,Money>
	 * @return Map<PaymentType, Double>
	 */
	public Map<String, BigDecimal> mapToTaxPaymentsLineItems(HashMap<String,Money> taxLineItemsMap){
		Set<String> lawIds = taxLineItemsMap.keySet();
		PaymentType type = null;
		Map<String, BigDecimal> outputMap = new HashMap<String, BigDecimal>();
		// loop through the law ids of the original map and build the new one
		for (String lawId : lawIds) {
			Money liability = taxLineItemsMap.get(lawId);
			type = PaymentTypeLawMapper.mapLawIdToPaymentType(lawId);
			if (outputMap.keySet().contains(type.value())) {
				// aggregate the liabilities for that PaymentType
				outputMap.put(type.value(),outputMap.get(type.value()).add(liability.getAmount()) );
			} else {
				// add the liability for that PaymentType
				outputMap.put(type.value(), liability.getAmount());
			}
		}
		return outputMap;		
	}
	
	/**
	 * Initializes Tax Line Items Map
	 *
	 * @throws Exception
	 */
	public void initliazeTaxLineItemsMap() throws Exception{
		List<String> lawIds = TaxPaymentsTestUtil.getAllLawIdsList2();
		for(String lawId:lawIds){
			taxLineItemsMap.put(lawId, new Money("0"));			
		}
	}
	
	/**
	 * Verifies if all 94x split of all the payments is following the split logic with respect to the 94x liabilities
	 * @param Payment[],HashMap<String,Money>
	 *
	 * @throws Exception
	 */
	public boolean verify94xSplit(Payment[] payments, HashMap<String,Money> liabilityLineItemsMap) throws Exception{
		
		boolean splitmatch = true;		
				
		// Extract 94x LineItems from payments and their sum
		PaymentLineItem[] payments94xLineItems = new PaymentLineItem[payments.length];
		for(int i=0;i<payments.length;i++){
			Set<PaymentLineItem> paymentLineItems = payments[i].getPaymentLineItems();
			Iterator<PaymentLineItem> paymentLineItemIterator = paymentLineItems.iterator();	
			
			while(paymentLineItemIterator.hasNext()){
				PaymentLineItem lineItem = paymentLineItemIterator.next();
				if(lineItem.getPaymentType() == PaymentType.FORM94x){
					payments94xLineItems[i]= lineItem;				
				}
			}			
		}		
		Money total94xPayments = new Money("0");
		for(int i=0; i < payments94xLineItems.length; i++){
			total94xPayments = total94xPayments.add(payments94xLineItems[i].getAmount());
		}
		
		// Extract 94x LineItems from Liabilities and their sum
		HashMap<String,Money> balanceLiabilities94x = new HashMap<String, Money>();  // helper to check split
		
		Money balanceLiabilities94xTotal = new Money("0");
		List<String> lawIds = TaxPaymentsTestUtil.getAll94xLawIds();
		
		for(String lawId:lawIds){
			balanceLiabilities94x.put(lawId, liabilityLineItemsMap.get(lawId));
			balanceLiabilities94xTotal = balanceLiabilities94xTotal.add(liabilityLineItemsMap.get(lawId));
		}
		
		try{
			for(int i=0; i< payments94xLineItems.length;i++){
				PaymentLineItem paymentLineItem = payments94xLineItems[i];
				Set<PaymentLineItemSplit> payLineItemsplit = paymentLineItem.getPaymentLineItemSplits();
				List<PaymentLineItemSplit> payLineItemsplitList = new ArrayList<PaymentLineItemSplit>();
				
				for(String lawId:lawIds){			// Arrange the split in the order of Law Id
				Iterator<PaymentLineItemSplit> splitIterator = payLineItemsplit.iterator();
					while(splitIterator.hasNext()){
						PaymentLineItemSplit paymentLineItemSplit = splitIterator.next();
						if(paymentLineItemSplit.getPayrollItem().getLawId().getValue().equals(lawId)){
							payLineItemsplitList.add(paymentLineItemSplit);
						}
					}
				}
				BigDecimal splitBalance = paymentLineItem.getAmount().getAmount();
				// If 94x component of a payment is greater than or equal to total 94x Liability
				try{
					if(paymentLineItem.getAmount().getAmount().compareTo(balanceLiabilities94xTotal.getAmount()) > 0){
								
						Double excessAmount = paymentLineItem.getAmount().getAmount().doubleValue() - balanceLiabilities94xTotal.getAmount().doubleValue();
						BigDecimal total94xpositiveLiabilities = new BigDecimal("0");
						
						for(String lawId:lawIds){
							if(balanceLiabilities94x.get(lawId).getAmount().compareTo(new BigDecimal("0"))>=0){
								total94xpositiveLiabilities = total94xpositiveLiabilities.add(balanceLiabilities94x.get(lawId).getAmount());
							}
						}				
						if(splitBalance.compareTo(total94xpositiveLiabilities)>=0){
							Double excessAmountSplit = excessAmount/payLineItemsplit.size();
				
							for(PaymentLineItemSplit paylineItemSplitItem:payLineItemsplitList){
								Money payLineItemAmount = paylineItemSplitItem.getSplitAmount();
								Double paylineItemSplitDoubleValue = payLineItemAmount.getAmount().doubleValue();
								Money liabiltyLineItemAmount = balanceLiabilities94x.get(paylineItemSplitItem.getPayrollItem().getLawId().getValue());
								Double liabiltyLineItemAmountdoubleValue = liabiltyLineItemAmount.getAmount().doubleValue();
								if(liabiltyLineItemAmount.getAmount().compareTo(new BigDecimal("0"))<=0){ // if 94x liability balance line item is zero or negative
									if(!paylineItemSplitDoubleValue.equals(new Double("0")+ excessAmountSplit)){ // check the split item of this payment is zero+ excess amount split
										throw new Exception("splits not matched");							
									}
								}	
								else if(!paylineItemSplitDoubleValue.equals(liabiltyLineItemAmountdoubleValue + excessAmountSplit)){
									throw new Exception("splits not matched");
								}
								Money balanceLiability = balanceLiabilities94x.get(paylineItemSplitItem.getPayrollItem().getLawId().getValue());
					
								balanceLiability = balanceLiability.subtract(payLineItemAmount);
								balanceLiabilities94x.put(paylineItemSplitItem.getPayrollItem().getLawId().getValue(), balanceLiability);						
							}
							balanceLiabilities94xTotal = new Money("0");
							for(String lawId:lawIds){
								if(balanceLiabilities94x.get(lawId).getAmount().compareTo(new BigDecimal("0"))>0){
									balanceLiabilities94xTotal = balanceLiabilities94xTotal.add(balanceLiabilities94x.get(lawId));
								}
							}
						}
						else{
							throw new CannotProceedException("dummy exception");
						}					
					}
					else{// If 94x component of a payment is less than total 94x Liability
						throw new CannotProceedException("dummy exception");
					}
				}catch(CannotProceedException cpe){
					for(PaymentLineItemSplit paylineItemSplitItem:payLineItemsplitList){
						Money paylineItemSplitItemAmount = paylineItemSplitItem.getSplitAmount();					
						BigDecimal liabiltyLineItemAmount = balanceLiabilities94x.get(paylineItemSplitItem.getPayrollItem().getLawId().getValue()).getAmount();
						if(splitBalance.compareTo(liabiltyLineItemAmount)>=0){ // if split balance of this payment is greater than or equal to balance liability line item
							if(liabiltyLineItemAmount.compareTo(new BigDecimal("0"))<=0){ // if 94x liability balance line item is zero or negative
								if(!paylineItemSplitItemAmount.equals(new Money("0"))){ // check the split item of this payment is zero
									throw new Exception("splits not matched");
								}
							}
							else if(!paylineItemSplitItemAmount.getAmount().equals(liabiltyLineItemAmount)){//if 94x liability balance line item is natural number
								throw new Exception("splits not matched");						// check if split item of the payment is same as balance liability lie item
							}
						
							Money balanceLiability = balanceLiabilities94x.get(paylineItemSplitItem.getPayrollItem().getLawId().getValue());
							
							balanceLiability = balanceLiability.subtract(paylineItemSplitItemAmount);
							balanceLiabilities94x.put(paylineItemSplitItem.getPayrollItem().getLawId().getValue(), balanceLiability);
						
						}
						else{
							if(splitBalance.compareTo(paylineItemSplitItemAmount.getAmount())!=0){ // if split balance is not greater than balance liability line item
								throw new Exception("splits not matched");						//check if split item is same as split balance
							}
							Money balanceLiability = balanceLiabilities94x.get(paylineItemSplitItem.getPayrollItem().getLawId().getValue());
							
							balanceLiability = balanceLiability.subtract(paylineItemSplitItemAmount);
							balanceLiabilities94x.put(paylineItemSplitItem.getPayrollItem().getLawId().getValue(), balanceLiability);
						
						}
						splitBalance = splitBalance.subtract(paylineItemSplitItemAmount.getAmount());						
						}											
					balanceLiabilities94xTotal = new Money("0");
					for(String lawId:lawIds){
						if(balanceLiabilities94x.get(lawId).getAmount().compareTo(new BigDecimal("0"))>0){
							balanceLiabilities94xTotal = balanceLiabilities94xTotal.add(balanceLiabilities94x.get(lawId));
						}
					}					
				}
			}
		}catch(Exception e){
			splitmatch = false;
		}
		
		return splitmatch;		
		
	}	

	
}
