package com.intuit.sb.payroll.qa.services.taxes;

import java.util.HashMap;
import java.util.Iterator;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.services.taxes.EmployerTaxIdInfo;
import com.intuit.sb.payroll.services.taxes.EmployerTaxSettingsServiceTestBase;
import com.intuit.sb.payroll.services.taxes.Jurisdiction;

/**
 * <P>
 * The <CODE>TINSettingTest</CODE> class contains test methods for Testing the Employer Tax Settings service get/Update
 * EmployerTaxId
 * </P>
 * 
 * $Author: rjohn2 $ $Revision: #1 $ $DateTime: 2008/08/21 14:10:25 $
 * 
 */

public class TINSettingTest extends EmployerTaxSettingsServiceTestBase {

	// The number of Jurisdictions defined at the start.
	private static int numberOfJurisdiction = 0;

	// The map of agency to the expected value for the agency once the update of tax
	// info is done.
	private HashMap<String, String> expectedAgencyToValueMap;

	// Reference to the current employer in context.
	private Employer empl;

	// Last EIN Number for EIN fields having 9 digits.
	private int lastEINNumberForNineDigits = 555555555;

	// Last EIN number for EIN fields having 8 digits.
	private int lastEINNumberForEightDigits = 66666666;

	// Last TIN number for the Federal field having 9 digits.
	private int lastTINNumberForNineDigits = 777777777;

	public static String US_IRS_NAME = "US_IRS";

	public static String CA_EDD_NAME = "CA_EDD";

	public static String TX_TWC_NAME = "TX_TWC";

	@BeforeMethod
	public void setup() throws Exception {
		expectedAgencyToValueMap = new HashMap<String, String>();
		empl = null;
		numberOfJurisdiction = mEmployerComponent.determineAllJurisdictions().size() - 1;
		// Setup the base.
		// super.setUp();
		// Set the initial set of values for each of the taxes applicable for the
		// employer. This set would be updated everytime an successful add/update/delete
		// is made to the tax records.
		empl = sEmployerEAO.findCurrentEmployer();
		initialSetupData();
	}

	/**
	 * Setup of Initial data for validation.
	 */
	private void initialSetupData() throws Exception {

		// Validate the associated employer tax agency data.
		java.util.Set<EmployerId> emplIdSet = empl.getEmployerIds();

		for (EmployerId emplIds : emplIdSet) {
			if (emplIds.getValue() != null) {
				AssertJUnit.assertTrue("Value submitted for EmployerId value cannot be an empty string", emplIds.getValue()
						.trim().length() > 0);
			}
			this.expectedAgencyToValueMap.put(emplIds.getAgencyIdType().getAgencyKey().name(),
					emplIds.getValue() == null ? "NULL" : emplIds.getValue());
		}
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() with all valid values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get the EmployerTaxIdInfo from the database</LI>
	 * <LI>Loop through the taxid info collection and check for the Label with String Federal , California and Texas and
	 * assign taxids</LI>
	 * <LI>Call updateEmployerTaxIDs() with updated collection as parameter</LI>
	 * <LI>Get the EmployerTaxIdInfo from the database again</LI>
	 * <LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the EmployerTaxIdInfo collection is same as that of updated one</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateEmployerTaxIDInfoWithHyphen() throws Exception {
		int updateCount = 0;
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				AssertJUnit.assertNotNull(employerTaxIdInfo.getTaxAgency());
				if (employerTaxIdInfo.getTaxAgency().name.contains(US_IRS_NAME)) {
					employerTaxIdInfo.setTaxId("22-7777777");
				} else if (employerTaxIdInfo.getTaxAgency().name.contains(CA_EDD_NAME)) {
					employerTaxIdInfo.setTaxId("333-4444-1");
				} else if (employerTaxIdInfo.getTaxAgency().name.contains(TX_TWC_NAME)) {
					employerTaxIdInfo.setTaxId("22-666666-1");
				}
			}

			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
			EmployerTaxIdInfo[] updatedTaxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			for (EmployerTaxIdInfo employerTaxIdInfo : updatedTaxIdInfos) {
				AssertJUnit.assertNotNull(employerTaxIdInfo.getTaxAgency());
				if (employerTaxIdInfo.getTaxAgency().name.contains(US_IRS_NAME)) {
					AssertJUnit.assertEquals("227777777", employerTaxIdInfo.getTaxId());
					this.expectedAgencyToValueMap.put(US_IRS_NAME, "227777777");
					updateCount++;
				} else if (employerTaxIdInfo.getTaxAgency().name.contains(CA_EDD_NAME)) {
					AssertJUnit.assertEquals("33344441", employerTaxIdInfo.getTaxId());
					this.expectedAgencyToValueMap.put(CA_EDD_NAME, "33344441");
					updateCount++;
				} else if (employerTaxIdInfo.getTaxAgency().name.contains(TX_TWC_NAME)) {
					AssertJUnit.assertEquals("226666661", employerTaxIdInfo.getTaxId());
					this.expectedAgencyToValueMap.put("TX_TWS", "226666661");
					updateCount++;
				}
			}
			AssertJUnit.assertEquals(3, updateCount);
		} catch (ValidationException ex) {
			AssertJUnit.fail("Exception raised in update of TIN/EIN with Hyphens.");
		}
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() passing null parameter
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>call updateEmployerTaxIDs() with null parameter
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateEmployerTaxIDsWithNullEmployerTaxIdInfo() throws Exception {
		boolean validationExceptionThrown = false;
		try {

			mEmployerTaxSettingService.updateEmployerTaxIDs(null);

		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
		// Make sure that the model remained intact when the update failed.
		validateDataModel();
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() passing empty array of employertaxinfo as parameter
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>call updateEmployerTaxIDs() with empty array of employertaxinfo as parameter
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateEmployerTaxIdWithEmptyEmplyerTaxIdInfo() throws Exception {
		boolean validationExceptionThrown = false;
		try {

			mEmployerTaxSettingService.updateEmployerTaxIDs(new EmployerTaxIdInfo[] {});

		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
		validateDataModel();
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() passing array of employertaxinfo with empty fields as parameter
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>call updateEmployerTaxIDs() with empty array of employertaxinfo with empty fields as parameter
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateEmployerTaxIdWithEmptyTaxAgencyInEmployerTaxIdInfo() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				employerTaxIdInfo.setTaxAgency(null);
			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
		validateDataModel();
	}

	/**
	 * Validate that with Valid EIN and invalid null tax agency the
	 * 
	 * @throws ValidationException
	 */
	@Test
	public void testUpdateEmployerTaxIdWithValidEINAndInvalidAgency() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			assertEmployerTaxIdInfo(taxIdInfos);

			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency() != null
						&& employerTaxIdInfo.getTaxAgency().name.contains(TX_TWC_NAME)) {
					employerTaxIdInfo.setTaxId("999999999");
					employerTaxIdInfo.setTaxAgency(null);
				}

				if (employerTaxIdInfo.getTaxAgency() != null
						&& employerTaxIdInfo.getTaxAgency().name.contains(CA_EDD_NAME)) {
					employerTaxIdInfo.setTaxId("99999999");
					employerTaxIdInfo.setTaxAgency(null);
				}

			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
		validateDataModel();
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() passing array of employertaxinfo where the taxid format is invalid for federal
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get taxIdInfos from the db by calling getEmployerTaxIDs()
	 * <LI>Loop through the taxid info array and check for label Federal
	 * <LI>set the taxId with invalid format
	 * <LI>Set different Id for Federal
	 * <LI>update the system by calling updateEmployerTaxIDs and passing the modified array
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateEmployerTaxIdWithInvalidFederalTaxIdFormat() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs(); // revisit
			assertEmployerTaxIdInfo(taxIdInfos);

			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(US_IRS_NAME)) {
					employerTaxIdInfo.setTaxId("2227-77777");
				}
			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
		validateDataModel();
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() passing array of employertaxinfo where the taxid format is invalid for California
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get taxIdInfos from the db by calling getEmployerTaxIDs()
	 * <LI>Loop through the taxid info array and check for label California
	 * <LI>set the taxId with invalid format for California
	 * <LI>update the system by calling updateEmployerTaxIDs and passing the modified array
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateEmployerTaxIdWithInvalidCATaxIdFormat() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs(); // revisit
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(CA_EDD_NAME)) {
					employerTaxIdInfo.setTaxId("4444-1-3333");

				}
			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
		validateDataModel();
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() passing array of employertaxinfo where the taxid format is invalid for Texas
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get taxIdInfos from the db by calling getEmployerTaxIDs()
	 * <LI>Loop through the taxid info array and check for label Texas
	 * <LI>set the taxId with invalid format for Texas
	 * <LI>update the system by calling updateEmployerTaxIDs and passing the modified array
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateEmployerTaxIdWithInvalidTXTaxIdFormat() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs(); // revisit
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(TX_TWC_NAME)) {
					employerTaxIdInfo.setTaxId("1-22-666666");
				}
			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
		validateDataModel();
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() passing array of employertaxinfo where the taxids for federal is empty
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get taxIdInfos from the db by calling getEmployerTaxIDs()
	 * <LI>Loop through the taxid info array and check for taxlabel Federal
	 * <LI>set the taxId empty for Federal
	 * <LI>update the system by calling updateEmployerTaxIDs and passing the modified array
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateEmployerTaxIDWithNoFederalEIN() throws Exception {
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs(); // revisit
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(US_IRS_NAME)) {
					employerTaxIdInfo.setTaxId("");
					this.expectedAgencyToValueMap.put(US_IRS_NAME, "");
					break;
				}
			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			AssertJUnit.fail("Validation Exception thrown on empty TIN" + ex.getMessage());
		}
		validateDataModel();
	}

	/**
	 * Test to validate if a NULL is allowed value for the Federal TIN.
	 * 
	 * @throws ValidationException
	 */
	@Test
	public void testUpdateEmployerTaxIDWithNULLFederalTIN() throws Exception {
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs(); // revisit
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(US_IRS_NAME)) {
					employerTaxIdInfo.setTaxId(null);
					this.expectedAgencyToValueMap.put(US_IRS_NAME, "");
				}
			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			AssertJUnit.fail("Validation exception throws on update on NULL TIN : " + ex.getMessage());
		}
		updateTaxIDs();
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() passing array of employertaxinfo where the taxids for federal is Negative
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get taxIdInfos from the db by calling getEmployerTaxIDs()
	 * <LI>Loop through the taxid info array and check for taxlabel Federal
	 * <LI>set the taxId with negative number for Federal
	 * <LI>update the system by calling updateEmployerTaxIDs and passing the modified array
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the ValidationException is thrown</LI>
	 * </UL>
	 */

	@Test
	public void testUpdateEmployerTaxIDWithNegativeTaxIdForFederal() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(US_IRS_NAME)) {
					employerTaxIdInfo.setTaxId("-22-7777777");
				}
			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
		validateDataModel();
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() passing array of employertaxinfo where the taxids for California
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get taxIdInfos from the db by calling getEmployerTaxIDs()
	 * <LI>Loop through the taxid info array and check for taxlabel California
	 * <LI>set the taxId with negative number for California
	 * <LI>update the system by calling updateEmployerTaxIDs and passing the modified array
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateEmployerTaxIDWithNegativeTaxIdCA() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(CA_EDD_NAME)) {
					employerTaxIdInfo.setTaxId("-33344441");
				}
			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
		validateDataModel();
	}

	/**
	 * <P>
	 * Test updateEmployerTaxIDs() passing array of employertaxinfo where the taxids for Texas
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get taxIdInfos from the db by calling getEmployerTaxIDs()
	 * <LI>Loop through the taxid info array and check for taxlabel Texas
	 * <LI>set the taxId with negative number for Texas
	 * <LI>update the system by calling updateEmployerTaxIDs and passing the modified array
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateEmployerTaxIDWithNegativeTaxIdTX() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(TX_TWC_NAME)) {
					employerTaxIdInfo.setTaxId("-26666661");
				}
			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
		validateDataModel();
	}

	/**
	 * Validate that the same tax ids can be assigned for more than agency.
	 * 
	 * @throws ValidationException
	 */
	@Test
	public void testUpdateEmployerTaxIDWithSameEmployerTaxIds() throws Exception {
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(US_IRS_NAME)) {
					employerTaxIdInfo.setTaxId("999999999");
					this.expectedAgencyToValueMap.put(US_IRS_NAME, "999999999");
				}
				if (employerTaxIdInfo.getTaxAgency().name.contains(TX_TWC_NAME)) {
					employerTaxIdInfo.setTaxId("999999999");
					this.expectedAgencyToValueMap.put(TX_TWC_NAME, "999999999");
				}

			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			AssertJUnit.fail("2 Tax Agencies can have the same Tax ID :" + ex.getMessage());
		}
		validateDataModel();
	}

	/**
	 * Validate that if NULL is an allowed tax ids can be assigned for a tax agency which is not mandatory.
	 * 
	 * @throws ValidationException
	 */
	@Test
	public void testUpdateEmployerTaxIDWithNullTaxIds() throws Exception {
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(CA_EDD_NAME)) {
					employerTaxIdInfo.setTaxId(null);
					this.expectedAgencyToValueMap.put(CA_EDD_NAME, "NULL");
				}
				if (employerTaxIdInfo.getTaxAgency().name.contains(TX_TWC_NAME)) {
					employerTaxIdInfo.setTaxId(null);
					this.expectedAgencyToValueMap.put(TX_TWC_NAME, "NULL");
				}

			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			AssertJUnit.fail("Null is allowed Tax ID, while the user is still updating :" + ex.getMessage());
		}
		validateDataModel();
	}

	/**
	 * Validate that if empty is an allowed tax ids can be assigned for a tax agency which is not mandatory.
	 * 
	 * @throws ValidationException
	 */
	@Test
	public void testUpdateEmployerTaxIDWithEmptyTaxIds() throws Exception {
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains(CA_EDD_NAME)) {
					employerTaxIdInfo.setTaxId("");
					this.expectedAgencyToValueMap.put(CA_EDD_NAME, "");
				}
				if (employerTaxIdInfo.getTaxAgency().name.contains(TX_TWC_NAME)) {
					employerTaxIdInfo.setTaxId("");
					this.expectedAgencyToValueMap.put(TX_TWC_NAME, "");
				}

			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			AssertJUnit.fail("Empty is allowed Tax ID, while the user is still updating :" + ex.getMessage());
		}
		validateDataModel();
	}

	/**
	 * <P>
	 * Test getEmployerTaxIDs()
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get taxIdInfos from the system by calling getEmployerTaxIDs()
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the object returned is not null and validate that federal taxagency is not null</LI>
	 * </UL>
	 * 
	 * @throws ValidationException
	 */

	@Test
	public void testGetEmployerTaxIDs() throws Exception {
		EmployerTaxIdInfo[] employerTaxIdInfo = mEmployerTaxSettingService.getEmployerTaxIDs();
		assertEmployerTaxIdInfo(employerTaxIdInfo);
		validateEmployerTaxIDInfo(employerTaxIdInfo);
	}

	/**
	 * <P>
	 * Test getEmployerTaxIDs() for the employer without worklocation
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Remove the worklocation by calling removeWorkLocationWithWorker. This will remove the worklocation
	 * <LI>call getEmployerTaxIDs()
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the object returned is null</LI>
	 * </UL>
	 * 
	 * @throws PayrollValidationException
	 * @throws ValidationException
	 */
	// PLease note, once a worklocations are added, you can never delete all worklocation,
	// You can delete only all BUT one, this tests does the same
	@Test
	public void testGetEmployerTaxIDsWithTXWorkLocationRemoved() throws Exception {
		EmployerTaxIdInfo[] employertaxIdinfo = mEmployerTaxSettingService.getEmployerTaxIDs();
		int orgEmployerTaxIdSize = employertaxIdinfo.length;

		removeSomeWorkLocations(sMountainViewLocation);
		employertaxIdinfo = mEmployerTaxSettingService.getEmployerTaxIDs();
		this.expectedAgencyToValueMap.remove(TX_TWC_NAME);
		AssertJUnit.assertTrue("Size should be have reduced", orgEmployerTaxIdSize > employertaxIdinfo.length);
		validateDataModel();
		// minus 1, because the TX worklocation got removed, and the
		// update Tax IDs, does an assert on the size.
		numberOfJurisdiction--;
		this.updateTaxIDs();

		// Update the count so that the next test gets the clean
		// slate as the worklocation would be added.
		numberOfJurisdiction++;
	}

	/**
	 * <P>
	 * Test getJurisdictions() for the employer without worklocation
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Remove the worklocation by calling removeWorkLocationWithWorker. This will remove the worklocation
	 * <LI>call getJurisdictions()
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the object returned is null</LI>
	 * </UL>
	 * 
	 * @throws PayrollValidationException
	 * @throws
	 * @throws ServiceFailureException
	 */
	@Test
	public void testGetJurisdictionsWithCAWorkLocationRemoved() throws Exception {
		int orgJurisdictionSize = mEmployerTaxSettingService.getJurisdictions().length;
		removeSomeWorkLocations(sMesquiteLocation);
		Jurisdiction[] jurisdictions = mEmployerTaxSettingService.getJurisdictions();
		AssertJUnit.assertNotNull("employertaxIdinfo is not expected for no worklocation", jurisdictions);
		AssertJUnit.assertTrue("Size should be reduced", orgJurisdictionSize > jurisdictions.length);
	}

	private void validateEmployerTaxIDInfo(EmployerTaxIdInfo[] taxidinfo) {
		for (EmployerTaxIdInfo emptaxidinfo : taxidinfo) {
			// at the least federal taxid should not be null
			if (emptaxidinfo.getTaxAgency().name.contains(US_IRS_NAME)) {
				AssertJUnit.assertNotNull(emptaxidinfo.getTaxAgency());
			}
		}
	}

	private void assertEmployerTaxIdInfo(EmployerTaxIdInfo[] taxIdInfos) {
		AssertJUnit.assertNotNull(taxIdInfos);
		// Make sure that the number of EmployerIds are same as jurisdicions
		AssertJUnit.assertEquals("EmployerTaxId size is not same as # of Jurisdiction", numberOfJurisdiction,
				taxIdInfos.length);

	}

	/**
	 * Validate the data model based on the current set of taxes applicable for the employer.
	 */
	private void validateDataModel() throws Exception {
		java.util.Set<String> agencyKeySet = this.expectedAgencyToValueMap.keySet();
		Iterator<String> it = agencyKeySet.iterator();
		int validatedCount = 0;

		while (it.hasNext()) {
			String key = it.next();
			java.util.Set<EmployerId> empID = this.empl.getEmployerIds();
			Iterator<EmployerId> itEmpId = empID.iterator();
			while (itEmpId.hasNext()) {
				EmployerId empIDInstance = itEmpId.next();
				if (empIDInstance.getAgencyIdType().getAgencyKey().name().equals(key)) {
					String value = this.expectedAgencyToValueMap.get(key);
					if (value.equals("NULL")) {
						AssertJUnit.assertNull(empIDInstance.getValue());
					} else {
						AssertJUnit.assertEquals(value, empIDInstance.getValue());
					}
					validatedCount++;
				}
			}
			itEmpId = null;
		}
		AssertJUnit.assertEquals(validatedCount, this.expectedAgencyToValueMap.size());
	}

	/**
	 * Update the tax IDs for the current set of applicable taxes.
	 * 
	 * @throws ValidationException
	 */
	private void updateTaxIDs() throws Exception {
		int updateCount = 0;
		try {
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			assertEmployerTaxIdInfo(taxIdInfos);
			Integer tinEINValue = null;
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				AssertJUnit.assertNotNull(employerTaxIdInfo.getTaxAgency());
				if (employerTaxIdInfo.getTaxAgency().name.contains(US_IRS_NAME)) {
					tinEINValue = new Integer(this.lastTINNumberForNineDigits);
					employerTaxIdInfo.setTaxId(tinEINValue.toString());
					this.expectedAgencyToValueMap.put(US_IRS_NAME, tinEINValue.toString());
					this.lastTINNumberForNineDigits++;
				} else if (employerTaxIdInfo.getTaxAgency().name.contains(CA_EDD_NAME)) {
					tinEINValue = new Integer(this.lastEINNumberForEightDigits);
					employerTaxIdInfo.setTaxId(tinEINValue.toString());
					this.expectedAgencyToValueMap.put(CA_EDD_NAME, tinEINValue.toString());
					this.lastEINNumberForEightDigits++;
				} else if (employerTaxIdInfo.getTaxAgency().name.contains(TX_TWC_NAME)) {
					tinEINValue = new Integer(this.lastEINNumberForNineDigits);
					employerTaxIdInfo.setTaxId(tinEINValue.toString());
					this.expectedAgencyToValueMap.put(TX_TWC_NAME, tinEINValue.toString());
					this.lastTINNumberForNineDigits++;
				}
			}
			mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
			EmployerTaxIdInfo[] updatedTaxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			for (EmployerTaxIdInfo employerTaxIdInfo : updatedTaxIdInfos) {
				AssertJUnit.assertNotNull(employerTaxIdInfo.getTaxAgency());
				if (employerTaxIdInfo.getTaxAgency().name.contains(US_IRS_NAME)) {
					AssertJUnit.assertEquals(this.expectedAgencyToValueMap.get(US_IRS_NAME), employerTaxIdInfo.getTaxId());
					updateCount++;
				} else if (employerTaxIdInfo.getTaxAgency().name.contains(CA_EDD_NAME)) {
					AssertJUnit.assertEquals(this.expectedAgencyToValueMap.get(CA_EDD_NAME), employerTaxIdInfo.getTaxId());
					updateCount++;
				} else if (employerTaxIdInfo.getTaxAgency().name.contains(TX_TWC_NAME)) {
					AssertJUnit.assertEquals(this.expectedAgencyToValueMap.get(TX_TWC_NAME), employerTaxIdInfo.getTaxId());
					updateCount++;
				}
			}
			validateDataModel();
		} catch (ValidationException ex) {
			AssertJUnit.assertEquals(expectedAgencyToValueMap.size(), updateCount);
		}
	}

}
