package com.intuit.sb.payroll.qa.services.worker;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.worker.Worker;


/**
 *  <P>The <CODE>HireExistingWorkerExceptionsTest</CODE> class contains tests for testing exceptions that can be thrown
 *     by hireExistingWorker() in WorkerService</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class HireExistingWorkerExceptionsTest extends EntityUtilsTestBase
{
    /**
     * <P>Test calling hireExistingWorker() with a null terminationDate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Call IWorkerService.hireExistingWorker()with a null terminationDate</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(groups = {"worker.hireExistingWorker"}, expectedExceptions=ValidationException.class)
    public void testHiringExistingWorkerWithNullHireDate() throws Exception
    {
    	// Setup
    	Long workerId = createPayrollReadyWorker();
    	Worker worker = sWorkerService.getWorkerById(workerId);
    	worker = sWorkerService.terminateWorker(worker, DateUtil.getNowDate());
    	// Exercise
    	worker = sWorkerService.hireExistingWorker(worker.getObjectHandle().getId(), null);
    }

    /**
     * <P>Test calling hireExistingWorker() with a null workerId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IWorkerService.hireExistingWorker()with a null workerId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(groups = {"worker.hireExistingWorker"}, expectedExceptions=ValidationException.class)
    public void testHiringExistingWorkerWithNullWorkerId() throws Exception
    {
    	// Exercise
    	@SuppressWarnings("unused")
    	Worker worker = sWorkerService.hireExistingWorker(null, DateUtil.getNowDate());
    }

    /**
     * <P>Test calling hireExistingWorker() with a deleted worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Delete the worker</LI>
     * <LI>Call IWorkerService.hireExistingWorker()with the deleted worker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * </UL>
     */
    @Test(groups = {"worker.hireExistingWorker"}, expectedExceptions=ServiceFailureException.class)
    public void testHiringDeletedWorker() throws Exception
    {
    	// Setup
    	Long workerId = createPayrollReadyWorker();
    	Worker worker = sWorkerService.getWorkerById(workerId);
    	worker = sWorkerService.terminateWorker(worker, DateUtil.getNowDate());
    	cleanUpEIWorker(workerId);
    	// Exercise
    	worker = sWorkerService.hireExistingWorker(worker.getObjectHandle().getId(), DateUtil.getNowDate());
    }

}
