package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationUtils;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PaycheckStatus;
import com.intuit.sb.payroll.services.workerpayments.PaycheckSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunCreateRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunResponse;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveResponse;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunState;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class CommitPayrollRunTest extends WorkerPaymentServiceTestBase {
    private static Long sReadyForPayWorkerId = null;
    private static Long sReadyForPayWorkerId2 = null;
	private static int sNumberOfPayrollReadyWorkers = 2;
	private String mHourlyRate = "10.00";
	private String mOvertimeRate = "1.5";
	private String mBonusAmount = "1000.00";
	private double mHourlyHours = 80.0;
	private double mOvertimeHours = 8.0;

    @BeforeClass
    public static void setUpCommitPayrollRunTest() {
        try {
            getUserTransaction().begin();
            sReadyForPayWorkerId = createMVPayrollReadyWorker("Paul");
            sReadyForPayWorkerId2 = createMVPayrollReadyWorker("Pauline");
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpCommitPayrollRunTest(): " + t.getMessage());
        }
    }

    @AfterClass
    public static void tearDownCommitPayrollRunTest() {
        try {
            getUserTransaction().begin();
            if (null != sReadyForPayWorkerId) {
                cleanUpEIWorker(sReadyForPayWorkerId);
                sReadyForPayWorkerId = null;
            }
            if (null != sReadyForPayWorkerId2) {
                cleanUpEIWorker(sReadyForPayWorkerId2);
                sReadyForPayWorkerId2 = null;
            }
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownCommitPayrollRunTest(): " + t.getMessage());
        }
    }

    /**
     * <P>Test commitPayrollRun() API with a valid PayrollRunSaveRequest object using an hourly worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up an hourly worker</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a valid PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * <LI>Verify that the expected number of workers are returned</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithHourlyWorker() throws Exception {
    	PayrollRunSummary payrollSummary = null;
    	Exception exception = null;
    	try {
        	//Setup
    	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
    		workerIds[1] = sReadyForPayWorkerId2;
    	    // Verify workers have no pay items before running (his first) pay run.
    		for (Long workerId : workerIds) {
            	com.intuit.sb.payroll.services.worker.Worker worker = sWorkerService.getWorkerById(workerId);
            	AssertJUnit.assertFalse("Worker has no paycheck data", worker.isAlreadyPaid());
    		}
    		// Setup wage items for the first worker
    		CompensationItem[] wageItems = sWageService.getCompensationItems(sReadyForPayWorkerId);
    		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
    		                                                     mOvertimeHours, mBonusAmount);
    		sWageService.saveCompensationItems(sReadyForPayWorkerId,
    		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
    		                                                                    updatedWageItems));
    		// Setup wage items for the second worker
    		CompensationItem[] wageItems2 = sWageService.getCompensationItems(sReadyForPayWorkerId2);
    		CompensationItem[] updatedWageItems2 = setupWageItems(wageItems2, mHourlyRate, 72.0, mOvertimeRate,
    		                                                      7.0, mBonusAmount);
    		sWageService.saveCompensationItems(sReadyForPayWorkerId2,
    		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
    		                                                                    updatedWageItems2));
        	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
        	//Exercise
            PayrollRunSaveResponse payrollRunSaveResponse = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
            payrollSummary = payrollRunSaveResponse.getPayrollRunSummary();
            com.intuit.sb.payroll.services.worker.Worker[] workers = payrollRunSaveResponse.getWorkers();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	// Setup expected values
        	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
        	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
        	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
        	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
        	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[2];
        	paycheckSummaries[0] = createPaycheckSummary(sReadyForPayWorkerId, ""+(mHourlyHours+mOvertimeHours),
        	                                             new BigDecimal("1920.00"), new BigDecimal("1393.80"),
        	                                             new BigDecimal("526.20"), new BigDecimal("0.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	paycheckSummaries[1] = createPaycheckSummary(sReadyForPayWorkerId2, ""+(72.0+7.0),
        	                                             new BigDecimal("1825.00"), new BigDecimal("1339.23"),
        	                                             new BigDecimal("485.77"), new BigDecimal("0.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
        	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("2733.03"));
        	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("3745.00"));
        	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("1011.97"));
        	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("447.54"));
        	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
        	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
        	
        	//Verify
        	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
        	                                                                         payrollSummary));
        	AssertJUnit.assertEquals("All workers in run returned", workerIds.length, workers.length);
    	    // Verify workers have pay items after running (his first) pay run.
    		for (com.intuit.sb.payroll.services.worker.Worker worker : workers) {
            	AssertJUnit.assertTrue("Worker has paycheck data", worker.isAlreadyPaid());
    		}
    	}
    	catch (Exception e) {
			exception = e;
		}
    	finally {
    		try {
	        	//Tear down
	    		if (payrollSummary != null) {
	            	Long payrollRunId = payrollSummary.getPayrollRunId();
	            	cleanUpPayrollRunItems(payrollRunId);
	    		}
    		}
    		finally {
    			// Don't let exceptions in clean up override any initial failure
    			if (exception != null) {
    				throw exception;
    			}
    		}
    	}
    }

    /**
     * <P>Test commitPayrollRun() API using an hourly worker with a double-time compensation item</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up an hourly worker</LI>
     * <LI>Add a double-time compensation item</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a valid PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithDoubleTimeHourlyWorker() throws Exception {
    	PayrollRunSummary payrollSummary = null;
    	double doubleTimeHours = 2.0;
    	Exception exception = null;
    	try {
        	//Setup
    		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Tim", sMountainViewLocation);
    		worker = WorkerTestUtils.addDoubleTime(worker, true);
    		Long doubleTimeWorkerId = worker.getId();
    	    Long[] workerIds = new Long[1];
        	workerIds[0] = doubleTimeWorkerId;
    		// Setup wage items for the first worker
    		CompensationItem[] wageItems = sWageService.getCompensationItems(doubleTimeWorkerId);
    		CompensationItem[] updatedWageItems = setupWageItemsForAllWageTypes(wageItems, mHourlyRate, mHourlyHours,
    		                                           mOvertimeRate, mOvertimeHours, mBonusAmount, doubleTimeHours);
    		sWageService.saveCompensationItems(doubleTimeWorkerId,
    		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
    		                                                                    updatedWageItems));
        	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
        	//Exercise
        	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	// Setup expected values
        	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
        	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
        	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
        	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
        	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
        	paycheckSummaries[0] = createPaycheckSummary(doubleTimeWorkerId,
        	                                             ""+(mHourlyHours+mOvertimeHours+doubleTimeHours),
        	                                             new BigDecimal("1960.00"), new BigDecimal("1416.70"),
        	                                             new BigDecimal("543.30"), new BigDecimal("0.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
        	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("1416.70"));
        	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("1960.00"));
        	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("543.30"));
        	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("234.22"));
        	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
        	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
        	
        	//Verify
        	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
        	                                                                         payrollSummary));
    	}
    	catch (Exception e) {
			exception = e;
		}
    	finally {
    		try {
	        	//Tear down
	    		if (payrollSummary != null) {
	            	Long payrollRunId = payrollSummary.getPayrollRunId();
	            	cleanUpPayrollRunItems(payrollRunId);
	    		}
    		}
    		finally {
    			// Don't let exceptions in clean up override any initial failure
    			if (exception != null) {
    				throw exception;
    			}
    		}
    	}
    }

    /**
     * <P>Test commitPayrollRun() API with a valid PayrollRunSaveRequest object using a salary worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a salary worker</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a valid PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithSalaryWorker() throws Exception {
    	PayrollRunSummary payrollSummary = null;
    	Long salaryWorkerId = null;
    	Exception exception = null;
    	try {
        	//Setup
    	    Long[] workerIds = new Long[1];
    	    salaryWorkerId = WorkerTestUtils.createSalaryPayrollReadyWorker("Tim", sMountainViewLocation);
    	    workerIds[0] = salaryWorkerId;
        	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
        	//Exercise
        	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	// Setup expected values
    		CompensationItem[] wageItems = sWageService.getCompensationItems(salaryWorkerId);
    		// The salary item should be the only one
    		String expectedHoursValue = wageItems[0].getName();
        	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
        	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
        	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
        	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
        	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
        	paycheckSummaries[0] = createPaycheckSummary(salaryWorkerId, expectedHoursValue,
        	                                             new BigDecimal("2538.46"), new BigDecimal("1748.48"),
        	                                             new BigDecimal("789.98"), new BigDecimal("0.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
        	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("1748.48"));
        	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("2538.46"));
        	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("789.98"));
        	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("303.36"));
        	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
        	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
        	
        	//Verify
        	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
        	                                                                         payrollSummary));
    	}
    	catch (Exception e) {
			exception = e;
		}
    	finally {
    		try {
	        	//Tear down
	    		if (payrollSummary != null) {
	            	Long payrollRunId = payrollSummary.getPayrollRunId();
	            	cleanUpPayrollRunItems(payrollRunId);
	    		}
	    		if (null != salaryWorkerId) {
	    			Worker worker = entityService.findById(Worker.class, salaryWorkerId);
	    			WorkerTestUtils.cleanUpWorker(worker);
	    		}
    		}
    		finally {
    			// Don't let exceptions in clean up override any initial failure
    			if (exception != null) {
    				throw exception;
    			}
    		}
    	}
    }

    /**
     * <P>Test commitPayrollRun() API with a only a Bonus compensation item</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a worker with only a Bonus compensation item</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a valid PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithBonusOnly() throws Exception {
    	PayrollRunSummary payrollSummary = null;
    	Long bonusWorkerId = null;
    	Exception exception = null;
    	try {
        	//Setup
    		Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Tim", sMountainViewLocation);
    		worker = WorkerTestUtils.addBonus(worker, true);
    	    Long[] workerIds = new Long[1];
    	    bonusWorkerId = worker.getId();
    	    workerIds[0] = bonusWorkerId;
        	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
        	//Exercise
        	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	// Setup expected values
    		CompensationItem[] wageItems = sWageService.getCompensationItems(bonusWorkerId);
    		// The bonus item should be the only one
    		String expectedHoursValue = wageItems[0].getName();
        	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
        	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
        	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
        	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
        	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
        	paycheckSummaries[0] = createPaycheckSummary(bonusWorkerId, expectedHoursValue,
        	                                             new BigDecimal("1000.00"), new BigDecimal("799.60"),
        	                                             new BigDecimal("200.40"), new BigDecimal("0.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
        	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("799.60"));
        	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("1000.00"));
        	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("200.40"));
        	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("119.50"));
        	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
        	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
        	
        	//Verify
        	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
        	                                                                         payrollSummary));
		}
		catch (Exception e) {
			exception = e;
		}
		finally {
			try {
	        	//Tear down
	    		if (payrollSummary != null) {
	            	Long payrollRunId = payrollSummary.getPayrollRunId();
	            	cleanUpPayrollRunItems(payrollRunId);
	    		}
	    		if (null != bonusWorkerId) {
	    			Worker worker = entityService.findById(Worker.class, bonusWorkerId);
	    			WorkerTestUtils.cleanUpWorker(worker);
	    		}
			}
			finally {
				// Don't let exceptions in clean up override any initial failure
				if (exception != null) {
					throw exception;
				}
			}
		}
    }

    /**
     * <P>Test commitPayrollRun() API using a salary worker with a bonus item</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a salary worker</LI>
     * <LI>Add a bonus item to the worker</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a valid PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithSalaryAndBonus() throws Exception {
    	PayrollRunSummary payrollSummary = null;
    	Long salaryWorkerId = null;
    	Exception exception = null;
    	try {
        	//Setup
    	    Long[] workerIds = new Long[1];
    	    Worker worker = WorkerTestUtils.createASalaryPayrollReadyWorker("Tim", sMountainViewLocation);
    		worker = WorkerTestUtils.addBonus(worker, true);
    	    salaryWorkerId = worker.getId();
    	    workerIds[0] = salaryWorkerId;
        	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
        	//Exercise
        	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	// Setup expected values
    		// The value in hours should be "Other"
    		String expectedHoursValue = "Other";
        	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
        	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
        	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
        	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
        	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
        	paycheckSummaries[0] = createPaycheckSummary(salaryWorkerId, expectedHoursValue,
        	                                             new BigDecimal("3538.46"), new BigDecimal("2310.98"),
        	                                             new BigDecimal("1227.48"), new BigDecimal("0.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
        	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("2310.98"));
        	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("3538.46"));
        	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("1227.48"));
        	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("422.86"));
        	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
        	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
        	
        	//Verify
        	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
        	                                                                         payrollSummary));
    	}
    	catch (Exception e) {
			exception = e;
		}
    	finally {
    		try {
	        	//Tear down
	    		if (payrollSummary != null) {
	            	Long payrollRunId = payrollSummary.getPayrollRunId();
	            	cleanUpPayrollRunItems(payrollRunId);
	    		}
	    		if (null != salaryWorkerId) {
	    			Worker worker = entityService.findById(Worker.class, salaryWorkerId);
	    			WorkerTestUtils.cleanUpWorker(worker);
	    		}
    		}
    		finally {
    			// Don't let exceptions in clean up override any initial failure
    			if (exception != null) {
    				throw exception;
    			}
    		}
    	}
    }

    /**
     * <P>Test commitPayrollRun() API using a salary worker with a health benefit item</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a salary worker</LI>
     * <LI>Add a health benefit item to the worker</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a valid PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithSalaryAndHealthBenefit() throws Exception {
    	PayrollRunSummary payrollSummary = null;
    	Long salaryWorkerId = null;
    	Exception exception = null;
    	try {
        	//Setup
    	    Long[] workerIds = new Long[1];
    	    Worker worker = WorkerTestUtils.createASalaryPayrollReadyWorker("Tim", sMountainViewLocation);
    		worker = WorkerTestUtils.addHealthBenefit(worker);
    	    salaryWorkerId = worker.getId();
    	    workerIds[0] = salaryWorkerId;
        	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
        	//Exercise
        	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	// Setup expected values
    		CompensationItem[] wageItems = sWageService.getCompensationItems(salaryWorkerId);
    		// The salary item should be the only one
    		String expectedHoursValue = wageItems[0].getName();
        	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
        	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
        	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
        	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
        	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
        	paycheckSummaries[0] = createPaycheckSummary(salaryWorkerId, expectedHoursValue,
        	                                             new BigDecimal("2538.46"), new BigDecimal("1711.02"),
        	                                             new BigDecimal("762.44"), new BigDecimal("65.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
        	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("1711.02"));
        	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("2538.46"));
        	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("762.43"));
        	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("295.59"));
        	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("65.00"));
        	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
        	
        	//Verify
        	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
        	                                                                         payrollSummary));
    	}
    	catch (Exception e) {
			exception = e;
		}
    	finally {
    		try {
	        	//Tear down
	    		if (payrollSummary != null) {
	            	Long payrollRunId = payrollSummary.getPayrollRunId();
	            	cleanUpPayrollRunItems(payrollRunId);
	    		}
	    		if (null != salaryWorkerId) {
	    			Worker worker = entityService.findById(Worker.class, salaryWorkerId);
	    			WorkerTestUtils.cleanUpWorker(worker);
	    		}
    		}
    		finally {
    			// Don't let exceptions in clean up override any initial failure
    			if (exception != null) {
    				throw exception;
    			}
    		}
    	}
    }

    /**
     * <P>Test commitPayrollRun() for a salary worker with a partial pay period of 4 days</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a salary worker</LI>
     * <LI>Create a Payroll Run</LI>
     * <LI>Set the partial period to contain 4 work days</LI>
     * <LI>Set the payroll run type to TERMINATION</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() using Payroll Run from createPayrollRun()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * <LI>Verify that the expected number of workers are returned</LI>
     * <LI>Verify that the returned worker is terminated</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithSalaryWorkerPartialPeriodWorked4Days() throws Exception {
    	PayrollRunSummary payrollSummary = null;
    	Long salaryWorkerId = null;
    	Exception exception = null;
    	try {
        	//Setup
    	    Long[] workerIds = new Long[1];
    	    salaryWorkerId = WorkerTestUtils.createSalaryPayrollReadyWorker("Tim", sMountainViewLocation);
    	    workerIds[0] = salaryWorkerId;
    	    PayrollRunCreateRequest createRequest = new PayrollRunCreateRequest();
    		//create a pay period
    		PayrollRunPeriod period = createBiweeklyPayrollRunPeriod(DateUtil.createDate(2008, Calendar.JUNE, 1));
    		// Set the truncated periodEndDate, payDate and payRunDate
    		Date terminationDate = DateUtil.getNextDate(period.getPeriodStartDate(), 3);
    		period.setPeriodEndDate(terminationDate);
    		period.setPayrollRunDate(terminationDate);
    		period.setPayDate(terminationDate);
    		period.setPartialPeriod(true);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(period.getPayrollRunDate());
    		createRequest.setWorkerIds(workerIds);
    		createRequest.setPeriod(period);
    		createRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
    		
    		PayrollRunResponse payResponse = sWorkerPayService.createPayrollRun(createRequest);
        	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,
        			payResponse.getPayrollRunId());
        	// Set the correct values for PayrollRunPeriod and PayrollRunType 
        	payrollRunSaveRequest.setPayrollRunPeriod(period);
        	PaycheckRequest[] paycheckRequests = payrollRunSaveRequest.getPaycheckRequests();
        	paycheckRequests[0].setPayrollRunPeriod(period);
        	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
        	payrollRunSaveRequest.setTerminationDate(terminationDate);

        	//Exercise
            PayrollRunSaveResponse payrollRunSaveResponse = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
            payrollSummary = payrollRunSaveResponse.getPayrollRunSummary();
            com.intuit.sb.payroll.services.worker.Worker[] workers = payrollRunSaveResponse.getWorkers();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	// Setup expected values
    		CompensationItem[] wageItems = sWageService.getCompensationItems(salaryWorkerId);
    		// The salary item should be the only one
    		String expectedHoursValue = wageItems[0].getName();
        	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
        	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
        	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
        	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
        	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
        	paycheckSummaries[0] = createPaycheckSummary(salaryWorkerId, expectedHoursValue,
        	                                             new BigDecimal("1015.38"), new BigDecimal("811.08"),
        	                                             new BigDecimal("204.30"), new BigDecimal("0.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
        	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("811.08"));
        	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("1015.38"));
        	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("204.30"));
        	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("121.35"));
        	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
        	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
        	
        	//Verify
        	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
        	                                                                         payrollSummary));
        	AssertJUnit.assertEquals("All workers in run returned", workerIds.length, workers.length);
        	for (com.intuit.sb.payroll.services.worker.Worker worker : workers) {
        		AssertJUnit.assertEquals("Worker is terminated", EmploymentStatusEnum.TERMINATED,
        			worker.getEmploymentStatus());
        	}
    	}
    	catch (Exception e) {
			exception = e;
		}
    	finally {
    		try {
	        	//Tear down
	    		if (payrollSummary != null) {
	            	Long payrollRunId = payrollSummary.getPayrollRunId();
	            	cleanUpPayrollRunItems(payrollRunId);
	    		}
	    		if (null != salaryWorkerId) {
	    			Worker worker = entityService.findById(Worker.class, salaryWorkerId);
	    			WorkerTestUtils.cleanUpWorker(worker);
	    		}
    		}
    		finally {
    			// Don't let exceptions in clean up override any initial failure
    			if (exception != null) {
    				throw exception;
    			}
    		}
    	}
    }

    /**
     * <P>Test commitPayrollRun() using PayrollRun from createPayrollRun()</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a salary worker</LI>
     * <LI>Create a Payroll Run</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() using Payroll Run from createPayrollRun()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithSalaryWorkerAfterCreatePayrollRun() throws Exception {
    	PayrollRunSummary payrollSummary = null;
    	Long salaryWorkerId = null;
    	Exception exception = null;
    	try {
        	//Setup
    	    Long[] workerIds = new Long[1];
    	    salaryWorkerId = WorkerTestUtils.createSalaryPayrollReadyWorker("Tim", sMountainViewLocation);
    	    workerIds[0] = salaryWorkerId;
    		PayrollRunCreateRequest createRequest = new PayrollRunCreateRequest();
    		//create a pay period
    		PayrollRunPeriod period = createBiweeklyPayrollRunPeriod(DateUtil.createDate(2008, Calendar.JUNE, 1));
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(period.getPayrollRunDate());
    		createRequest.setWorkerIds(workerIds);
    		createRequest.setPeriod(period);
    		createRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
    		
    		PayrollRunResponse payResponse = sWorkerPayService.createPayrollRun(createRequest);
        	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,
        			payResponse.getPayrollRunId());
        	// Set the correct values for PayrollRunPeriod and PayrollRunType 
        	payrollRunSaveRequest.setPayrollRunPeriod(period);
        	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
        	payrollRunSaveRequest.setTerminationDate(period.getPeriodEndDate());

        	//Exercise
        	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	// Setup expected values
    		CompensationItem[] wageItems = sWageService.getCompensationItems(salaryWorkerId);
    		// The salary item should be the only one
    		String expectedHoursValue = wageItems[0].getName();
        	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
        	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
        	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
        	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
        	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
        	paycheckSummaries[0] = createPaycheckSummary(salaryWorkerId, expectedHoursValue,
        	                                             new BigDecimal("2538.46"), new BigDecimal("1748.48"),
        	                                             new BigDecimal("789.98"), new BigDecimal("0.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
        	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("1748.48"));
        	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("2538.46"));
        	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("789.98"));
        	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("303.36"));
        	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
        	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
        	
        	//Verify
        	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
        	                                                                         payrollSummary));
        	com.intuit.sb.payroll.services.worker.Worker worker = sWorkerService.getWorkerById(salaryWorkerId);
        	AssertJUnit.assertEquals("Worker is not terminated!", EmploymentStatusEnum.TERMINATED,
        			worker.getEmploymentStatus());
    	}
    	catch (Exception e) {
			exception = e;
		}
    	finally {
    		try {
	        	//Tear down
	    		if (payrollSummary != null) {
	            	Long payrollRunId = payrollSummary.getPayrollRunId();
	            	cleanUpPayrollRunItems(payrollRunId);
	    		}
	    		if (null != salaryWorkerId) {
	    			Worker worker = entityService.findById(Worker.class, salaryWorkerId);
	    			WorkerTestUtils.cleanUpWorker(worker);
	    		}
    		}
    		finally {
    			// Don't let exceptions in clean up override any initial failure
    			if (exception != null) {
    				throw exception;
    			}
    		}
    	}
    }

    /**
     * <P>Test commitPayrollRun() using PayrollRun from createPayrollRun() with an hourly worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up an hourly worker</LI>
     * <LI>Create a Payroll Run</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a valid PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithHourlyWorkerAfterCreatePayrollRun() throws Exception {
    	PayrollRunSummary payrollSummary = null;
    	Exception exception = null;
    	try {
        	//Setup
    	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
    		workerIds[1] = sReadyForPayWorkerId2;
    		PayrollRunCreateRequest createRequest = new PayrollRunCreateRequest();
    		//create a pay period
    		PayrollRunPeriod period = createBiweeklyPayrollRunPeriod(DateUtil.createDate(2008, Calendar.JUNE, 1));
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(period.getPayrollRunDate());
    		createRequest.setWorkerIds(workerIds);
    		createRequest.setPeriod(period);
    		createRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
    		
    		PayrollRunResponse payResponse = sWorkerPayService.createPayrollRun(createRequest);
    		// Setup wage items for the first worker
    		CompensationItem[] wageItems = sWageService.getCompensationItems(sReadyForPayWorkerId);
    		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
    		                                                     mOvertimeHours, mBonusAmount);
    		sWageService.saveCompensationItems(sReadyForPayWorkerId,
    		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
    		                                                                    updatedWageItems));
    		// Setup wage items for the second worker
    		CompensationItem[] wageItems2 = sWageService.getCompensationItems(sReadyForPayWorkerId2);
    		CompensationItem[] updatedWageItems2 = setupWageItems(wageItems2, mHourlyRate, 72.0, mOvertimeRate,
    		                                                      7.0, mBonusAmount);
    		sWageService.saveCompensationItems(sReadyForPayWorkerId2,
    		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
    		                                                                    updatedWageItems2));
        	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,
        			payResponse.getPayrollRunId());
        	// Set the correct values for PayrollRunPeriod and PayrollRunType 
        	payrollRunSaveRequest.setPayrollRunPeriod(period);
        	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
        	payrollRunSaveRequest.setTerminationDate(period.getPeriodEndDate());
        	//Exercise
        	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	// Setup expected values
        	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
        	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
        	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
        	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
        	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[2];
        	paycheckSummaries[0] = createPaycheckSummary(sReadyForPayWorkerId, ""+(mHourlyHours+mOvertimeHours),
        	                                             new BigDecimal("1920.00"), new BigDecimal("1393.80"),
        	                                             new BigDecimal("526.20"), new BigDecimal("0.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	paycheckSummaries[1] = createPaycheckSummary(sReadyForPayWorkerId2, ""+(72.0+7.0),
        	                                             new BigDecimal("1825.00"), new BigDecimal("1339.23"),
        	                                             new BigDecimal("485.77"), new BigDecimal("0.00"),
        	                                             PaycheckDistributionMethod.HANDWRITTEN,
        	                                             PaycheckStatus.COMMITTED, payrollSummary);
        	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
        	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("2733.03"));
        	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("3745.00"));
        	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("1011.97"));
        	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("447.54"));
        	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
        	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
        	
        	//Verify
        	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
        	                                                                         payrollSummary));
        	com.intuit.sb.payroll.services.worker.Worker worker = sWorkerService.getWorkerById(sReadyForPayWorkerId);
        	AssertJUnit.assertEquals("Worker is not terminated!", EmploymentStatusEnum.TERMINATED,
        			worker.getEmploymentStatus());
        	com.intuit.sb.payroll.services.worker.Worker worker2 = sWorkerService.getWorkerById(sReadyForPayWorkerId2);
        	AssertJUnit.assertEquals("Worker is not terminated!", EmploymentStatusEnum.TERMINATED,
        			worker2.getEmploymentStatus());
    	}
    	catch (Exception e) {
			exception = e;
		}
    	finally {
    		try {
	        	//Tear down
	    		if (payrollSummary != null) {
	            	Long payrollRunId = payrollSummary.getPayrollRunId();
	            	cleanUpPayrollRunItems(payrollRunId);
	    		}
    		}
    		finally {
    			// Don't let exceptions in clean up override any initial failure
    			if (exception != null) {
    				throw exception;
    			}
    		}
    	}
    }

}
