package com.intuit.sb.payroll.qa.services.workerpayments;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class GetPaycheckByIdExceptionsTest extends WorkerPaymentServiceTestBase {

	/**
     * <P>Test ValidationException caused by null PaycheckId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's getPaycheckById() with null PaycheckId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown since the PaycheckId is null</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullPaycheckId() throws Exception {
    	// Exercise
        sWorkerPayService.getPaycheckById(null);
    }

    /**
     * <P>Test ServiceFailureException caused by non-existent PaycheckId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's getPaycheckById() with a non-existent PaycheckId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the PaycheckId is non-existent</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNonExistentPaycheckId() throws Exception {
    	// Exercise
        sWorkerPayService.getPaycheckById(893493841L);
    }

}
