package com.intuit.sb.payroll.performance;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;

import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.taxes.TaxesTestBase;
import com.intuit.sb.payroll.services.wages.CompensationItemTestBase;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.workerpayments.PaycheckSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveResponse;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 * this servlet tests com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService.commitPayrollRun(PayrollRunSaveRequest payrollRun)
 * @author jzhong
 *
 */
public class CommitPayrollRunServlet extends PerformanceTestServlet {
	
	private static final long serialVersionUID = 1L;	
	private static WorkerPaymentService workerPayService = ServiceLocator.getInstance().getService(WorkerPaymentService.JNDI_NAME);
	
    public void init(ServletConfig config) throws ServletException {

        // Store the ServletConfig object and log the initialization
        super.init(config);

        BasicTestBase.initializeEAOs();
        CompensationItemTestBase.initializeCompensationItemTestBaseClass();
        TaxesTestBase.initializeEAOs();
        EntityUtilsTestBase.initializeEntityUtilsTestBaseClass();
        WorkerPaymentServiceTestBase.initializeEAOs();
    }

	public String performTest(String username)
	{
	  StringBuffer sb = new StringBuffer("Commit payroll run <p>");
	  PayrollRunSaveResponse payrollRunSaveResponse = null;
	  try {
    	//Setup		  
	    Long[] workerIds = new Long[1];
		Worker [] workers = PayrollDataContainer.getReadyToPayWorkers(username);
        workerIds[0] = workers[1].getObjectHandle().getId();//salaryWorkerId;
	    
    	PayrollRunSaveRequest payrollRunSaveRequest = WorkerPaymentServiceTestBase.createPayrollRunSaveRequest(workerIds,null);
                
        BasicTestBase.setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());    	
    	payrollRunSaveResponse = workerPayService.commitPayrollRun(payrollRunSaveRequest);
    	BasicTestBase.resetPayrollSystemDate();
    	PayrollRunSummary  payrollRuns = payrollRunSaveResponse.getPayrollRunSummary();
    	PaycheckSummary [] paycheckSummaries = payrollRuns.getPaycheckSummary();
    	for (PaycheckSummary paycheckSummary:paycheckSummaries)
    	{
    		sb.append(" PaycheckId: " + paycheckSummary.getPaycheckId() + " Gross: " + paycheckSummary.getGrossPay() + " Net: " + paycheckSummary.getNetPay());
    	}
    	
	  }
	  catch (Exception ex)
	  {
		  throw new RuntimeException("commit payroll run fail", ex);
	  }
	  return sb.toString();
	}    	
}
