package com.intuit.sb.payroll.performance;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.intuit.sb.payroll.performance.tools.RealmUser;
import com.intuit.sb.payroll.performance.tools.RealmsReader;
import com.intuit.sb.payroll.services.worker.Worker;

/**
 * the class stores information about the payroll domain.
 * @author jzhong
 *
 */
public class PayrollDataContainer {

	/*
	 * the usersMap maps from user name to RealmUser
	 */
	private static Map<String, RealmUser> realmUsersMap = new HashMap<String, RealmUser>();
	/*
	 * the map realmWorkersMap keep a map from the username to the array of pay ready workers in the company
	 */
	private static Map<String, Worker []> payReadyWorkerSummaryMap = new HashMap<String, Worker[]>();
	private static Map<String, Worker []> payReadyWorkersMap = new HashMap<String, Worker []>();
	private static List<RealmUser> realmUsers;

	/*
	 * This method is read by PayrollPerfInitializerServlet
	 * the method processes mock_data.xml file and store companies/realm information in
	 * realmUsersMap
	 */
	static void readUsersInRealm()
	{
		final String SECURITY_POLICY_FILE = "..//server//default//lib//bhp//conf//mock_data.xml";
		RealmsReader realmReader = new RealmsReader(SECURITY_POLICY_FILE);
		realmUsers = realmReader.getIdentities();
		for (RealmUser realmUser:realmUsers)
		{
			realmUsersMap.put(realmUser.getLoginName(), realmUser);
		}
	}

	public static List<RealmUser> getRealmUsers()
	{
		return realmUsers;
	}

	public static Map<String, RealmUser> getRealmUserMap() 
	{
		return realmUsersMap;
	}
	
	/**
	 * return the user in the realm
	 * @param realmId is the realm ID
	 * @return
	 */
	public static RealmUser getRealmUser(String username)
	{
		return realmUsersMap.get(username);
	}
	
	/*
	 * @param username
	 * @param workers ready for pay workers in company specified by "username"
	 */
	public synchronized static void setReadyToPayWorkersMap(String username, Worker [] workers)
	{
		payReadyWorkersMap.put(username, workers);
	}

	public synchronized static Worker [] getReadyToPayWorkers(String username)
	{
		return payReadyWorkersMap.get(username);
	}
		
	public static String printUsers()
	{
		StringBuffer sb = new StringBuffer();
		for (String key:realmUsersMap.keySet())
		{
			sb.append(key);
			sb.append("<br>");
		}
		return sb.toString();
	}
}
