package com.intuit.sb.payroll.worker;

import java.util.Properties;

import javax.naming.Context;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.services.TestInitializer;
import com.intuit.sb.payroll.services.common.EntityTestBaseHelper;
import com.intuit.sb.payroll.services.common.EntityTestBaseHelper.PayrollReadyWorker;
import com.intuit.sb.payroll.services.worker.GetWorkersFilter;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.util.RemoteServiceLocator;

/**
 * Test methods in class com.intuit.sb.payroll.services.worker
 * @author jzhong
 *
 */
@Test(groups = "serviceittests")
public class WorkServiceVpdTest extends TestInitializer {
	
	//@Test
	/**
	 * Test add a worker. API to test: WorkerService.saveWorker
	 */
	public void testAddWorker()
	{	
		try
		{
			//count how many workers there are in "paul"'s realm
		    Properties env = RemoteServiceLocator.getEnv();
			env.setProperty(Context.SECURITY_PRINCIPAL, "paul");
			env.setProperty(Context.SECURITY_CREDENTIALS, "paul");
			RemoteServiceLocator.setEnv(env);
			Worker [] workersBeforeAdd = getWorkerService().getWorkers(GetWorkersFilter.ALL);

			//count how many workers there are in "peter"'s realm
		    env = RemoteServiceLocator.getEnv();
			env.setProperty(Context.SECURITY_PRINCIPAL, "peter");
			env.setProperty(Context.SECURITY_CREDENTIALS, "peter");
			RemoteServiceLocator.setEnv(env);
			Worker [] workersForPeterBeforeAdd = getWorkerService().getWorkers(GetWorkersFilter.ALL);
			
			//switch back to "paul"'s realm, add a worker
			env.setProperty(Context.SECURITY_PRINCIPAL, "paul");
			env.setProperty(Context.SECURITY_CREDENTIALS, "paul");
			RemoteServiceLocator.setEnv(env);
			
			PayrollReadyWorker saveWorkerResponse = EntityTestBaseHelper.createPayrollReadyWorkerInMemory();
			String firstName = "Thomas";
			saveWorkerResponse.getWorker().setFirstName(firstName);
			
			Worker worker = getWorkerService().saveWorker(saveWorkerResponse.getWorker());
			Assert.assertEquals(firstName, worker.getFirstName(), "Hey, the worker is not what is saved");
			Worker [] workersAfterAdd = getWorkerService().getWorkers(GetWorkersFilter.ALL);
			
			Assert.assertEquals(workersBeforeAdd.length + 1, workersAfterAdd.length, 
					"there should be one more worker after adding one worker");
			
			//Now change realm to "peter". No worker is added in "peter"'s realm
		    env = RemoteServiceLocator.getEnv();
			env.setProperty(Context.SECURITY_PRINCIPAL, "peter");
			env.setProperty(Context.SECURITY_CREDENTIALS, "peter");	    
		    RemoteServiceLocator.setEnv(env);
		    
		    Worker [] workersForPeterAfterAdd = getWorkerService().getWorkers(GetWorkersFilter.ALL);
			Assert.assertEquals(workersForPeterBeforeAdd.length, workersForPeterAfterAdd.length, 
			"No worker is added in Peter's realm");
		}
		catch (Exception ex)
		{
			throw new RuntimeException(ex);
		}
	}
	
}
