/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.SimpleQuery.java
 * 
 * Created: Nov 8, 2011 10:57:35 AM
 * Author: gkesler
 */
package com.intuit.sb.services.query;

import com.intuit.sb.services.query.generator.SimpleQueryGenerator;
import com.intuit.sb.services.query.generator.SimpleQueryGeneratorDelegate;
import com.intuit.sb.services.query.parser.ParseException;
import com.intuit.sb.services.query.parser.SimpleQueryParser;
import com.intuit.sb.services.query.validator.SimpleQueryValidator;
import com.intuit.sb.services.query.validator.ValidatorException;
import com.intuit.sb.services.query.validator.context.ContextFactory;
import com.intuit.sb.services.query.validator.context.RootContext;

import java.io.Reader;

/**
 *
 * @author gkesler
 */
public class SimpleQuery extends SimpleQueryParser {
    private final SimpleQueryContext context;
    
    private final SimpleQueryValidator validator = new SimpleQueryValidator();
    private final SimpleQueryGenerator generator = new SimpleQueryGenerator();
    
    /**
     * Constructs a new SimpleQuery object ready to parse a provided stream with the
     * properties and context factory for the further validation
     * Note! This is an opportunity to specify custom defined MetadataProvider
     * 
     * @see ContextFactory
     * 
     * @param stream            stream containing SimpleQuery statement
     * @param properties        Context validation and validation properties
     * @param contextFactory    ContextFactory to use when validating the query.
     */
    SimpleQuery (Reader stream, SimpleQueryContext context) {
        super(stream);
        
        this.context = context;
    }
    
    /**
     * @return the root context
     */
    public RootContext getRootContext() {
    	return validator.getRootContext();
    }
    
    /**
     * Performs semantic validation step of the current (already parsed) statement
     */
    public void validate () {
        validator.validate(this, context.getProperties(), context.getContextFactory());
    }
    
    /**
     * Performs generation of the already validated statement
     * Note! Custom generators will be used in this step
     * 
     * @param delegate custom generator callback (observer) object
     */
    public void generate (SimpleQueryGeneratorDelegate delegate) {
        generator.generate(this, delegate);
    }
    
    /**
     * Parses and validates a SimpleQuery statement using environment properties
     * 
     * @param statement statement to parse and validate
     * @return SimpleQuery object ready for generation phase
     */
    public static SimpleQuery valueOf (String statement) {        
        try {
            SimpleQuery query = new SimpleQueryContext().newSimpleQuery(statement);
            query.parse();
            query.validate();
            
            return query;
        } catch (ParseException e) {
            throw new IllegalArgumentException(statement, e);
        } catch (ValidatorException e) {
            throw new IllegalArgumentException(statement, e);
        }
    }
}
