/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.generator.RestrictedModeDecorator.java
 * 
 * Created: Nov 16, 2011 6:40:57 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.generator;

import com.intuit.sb.services.query.generator.tuple.AndTuple;
import com.intuit.sb.services.query.generator.tuple.AscTuple;
import com.intuit.sb.services.query.generator.tuple.BetweenTuple;
import com.intuit.sb.services.query.generator.tuple.ContainsTuple;
import com.intuit.sb.services.query.generator.tuple.DescTuple;
import com.intuit.sb.services.query.generator.tuple.EqTuple;
import com.intuit.sb.services.query.generator.tuple.ExpressionTuple;
import com.intuit.sb.services.query.generator.tuple.GtTuple;
import com.intuit.sb.services.query.generator.tuple.GteTuple;
import com.intuit.sb.services.query.generator.tuple.InTuple;
import com.intuit.sb.services.query.generator.tuple.IteratorTuple;
import com.intuit.sb.services.query.generator.tuple.LikeTuple;
import com.intuit.sb.services.query.generator.tuple.MaxResultsTuple;
import com.intuit.sb.services.query.generator.tuple.LtTuple;
import com.intuit.sb.services.query.generator.tuple.LteTuple;
import com.intuit.sb.services.query.generator.tuple.NEqTuple;
import com.intuit.sb.services.query.generator.tuple.NotTuple;
import com.intuit.sb.services.query.generator.tuple.OrTuple;
import com.intuit.sb.services.query.generator.tuple.OrderByTuple;
import com.intuit.sb.services.query.generator.tuple.PaginationTuple;
import com.intuit.sb.services.query.generator.tuple.StartPositionTuple;
import com.intuit.sb.services.query.generator.tuple.TupleVisitor;
import com.intuit.sb.services.query.validator.context.Property;
import com.intuit.sb.services.query.validator.context.RootContext;

/**
 *
 * @author gkesler
 */
final class RestrictedModeDecorator implements SimpleQueryGeneratorDelegate {
    private final SimpleQueryGeneratorDelegate delegate;
    
    RestrictedModeDecorator (SimpleQueryGeneratorDelegate delegate) {
        this.delegate = delegate;
    }

    public void addToProjection(Property path) {
        delegate.addToProjection(path);
    }

    public void addRootContext(RootContext context) {
        delegate.addRootContext(context);
    }

    public void addPaginationItem(PaginationTuple tuple) {
        delegate.addPaginationItem(tuple);
    }

    public void addOrderSpec(OrderByTuple tuple) {
        delegate.addOrderSpec(tuple);
    }

    public void addFilter(ExpressionTuple tuple) {
        tuple.accept(filterVisitor, delegate);
    }
    
    private static final TupleVisitor<SimpleQueryGeneratorDelegate> filterVisitor =
            new TupleVisitor<SimpleQueryGeneratorDelegate>() {
        public SimpleQueryGeneratorDelegate visit(AndTuple tuple, SimpleQueryGeneratorDelegate data) {
            return tuple.childrenAccept(this, data);
        }

        public SimpleQueryGeneratorDelegate visit(AscTuple tuple, SimpleQueryGeneratorDelegate data) {
            throw new AssertionError("Wrong tuple: " + tuple);
        }

        public SimpleQueryGeneratorDelegate visit(BetweenTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(ContainsTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(DescTuple tuple, SimpleQueryGeneratorDelegate data) {
            throw new AssertionError("Wrong tuple: " + tuple);
        }

        public SimpleQueryGeneratorDelegate visit(EqTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(GtTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(GteTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(InTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(IteratorTuple tuple, SimpleQueryGeneratorDelegate data) {
            throw new AssertionError("Wrong tuple: " + tuple);
        }

        public SimpleQueryGeneratorDelegate visit(LikeTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(MaxResultsTuple tuple, SimpleQueryGeneratorDelegate data) {
            throw new AssertionError("Wrong tuple: " + tuple);
        }

        public SimpleQueryGeneratorDelegate visit(LtTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(LteTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(NEqTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(NotTuple tuple, SimpleQueryGeneratorDelegate data) {
            return visitFilter(tuple, data);
        }

        public SimpleQueryGeneratorDelegate visit(OrTuple tuple, SimpleQueryGeneratorDelegate data) {
            throw new AssertionError("Wrong tuple: " + tuple);
        }

        public SimpleQueryGeneratorDelegate visit(StartPositionTuple tuple, SimpleQueryGeneratorDelegate data) {
            throw new AssertionError("Wrong tuple: " + tuple);
        }
        
        private final SimpleQueryGeneratorDelegate visitFilter (ExpressionTuple tuple, SimpleQueryGeneratorDelegate delegate) {
            delegate.addFilter(tuple);            
            return delegate;
        }
    };
}
