/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.generator.SimpleQueryGenerator.java
 * 
 * Created: Aug 16, 2011 3:54:01 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.generator;

import java.util.Stack;
import java.util.UUID;

import com.intuit.sb.services.query.ASTUtils;
import com.intuit.sb.services.query.SimpleQuery;
import com.intuit.sb.services.query.generator.tuple.AndTuple;
import com.intuit.sb.services.query.generator.tuple.AscTuple;
import com.intuit.sb.services.query.generator.tuple.BetweenTuple;
import com.intuit.sb.services.query.generator.tuple.ConditionalTuple;
import com.intuit.sb.services.query.generator.tuple.ContainsTuple;
import com.intuit.sb.services.query.generator.tuple.DescTuple;
import com.intuit.sb.services.query.generator.tuple.EqTuple;
import com.intuit.sb.services.query.generator.tuple.ExpressionTuple;
import com.intuit.sb.services.query.generator.tuple.FilterTuple;
import com.intuit.sb.services.query.generator.tuple.GtTuple;
import com.intuit.sb.services.query.generator.tuple.GteTuple;
import com.intuit.sb.services.query.generator.tuple.InTuple;
import com.intuit.sb.services.query.generator.tuple.IteratorTuple;
import com.intuit.sb.services.query.generator.tuple.LikeTuple;
import com.intuit.sb.services.query.generator.tuple.MaxResultsTuple;
import com.intuit.sb.services.query.generator.tuple.LtTuple;
import com.intuit.sb.services.query.generator.tuple.LteTuple;
import com.intuit.sb.services.query.generator.tuple.NEqTuple;
import com.intuit.sb.services.query.generator.tuple.NotTuple;
import com.intuit.sb.services.query.generator.tuple.OrTuple;
import com.intuit.sb.services.query.generator.tuple.PathHolder;
import com.intuit.sb.services.query.generator.tuple.StartPositionTuple;
import com.intuit.sb.services.query.generator.tuple.ValueHolder;
import com.intuit.sb.services.query.parser.ASTAliasName;
import com.intuit.sb.services.query.parser.ASTAnd;
import com.intuit.sb.services.query.parser.ASTAnyPathElement;
import com.intuit.sb.services.query.parser.ASTAsc;
import com.intuit.sb.services.query.parser.ASTBetween;
import com.intuit.sb.services.query.parser.ASTContains;
import com.intuit.sb.services.query.parser.ASTContextDeclaration;
import com.intuit.sb.services.query.parser.ASTContextName;
import com.intuit.sb.services.query.parser.ASTCount;
import com.intuit.sb.services.query.parser.ASTCurrentDate;
import com.intuit.sb.services.query.parser.ASTCurrentTime;
import com.intuit.sb.services.query.parser.ASTCurrentTimestamp;
import com.intuit.sb.services.query.parser.ASTDateTime;
import com.intuit.sb.services.query.parser.ASTDesc;
import com.intuit.sb.services.query.parser.ASTDistance;
import com.intuit.sb.services.query.parser.ASTEq;
import com.intuit.sb.services.query.parser.ASTFalse;
import com.intuit.sb.services.query.parser.ASTFloat;
import com.intuit.sb.services.query.parser.ASTFromClause;
import com.intuit.sb.services.query.parser.ASTGt;
import com.intuit.sb.services.query.parser.ASTGte;
import com.intuit.sb.services.query.parser.ASTId;
import com.intuit.sb.services.query.parser.ASTIdPathElement;
import com.intuit.sb.services.query.parser.ASTIn;
import com.intuit.sb.services.query.parser.ASTInteger;
import com.intuit.sb.services.query.parser.ASTIterator;
import com.intuit.sb.services.query.parser.ASTLike;
import com.intuit.sb.services.query.parser.ASTMaxResults;
import com.intuit.sb.services.query.parser.ASTLt;
import com.intuit.sb.services.query.parser.ASTLte;
import com.intuit.sb.services.query.parser.ASTMe;
import com.intuit.sb.services.query.parser.ASTMyApp;
import com.intuit.sb.services.query.parser.ASTNEq;
import com.intuit.sb.services.query.parser.ASTNot;
import com.intuit.sb.services.query.parser.ASTNull;
import com.intuit.sb.services.query.parser.ASTOr;
import com.intuit.sb.services.query.parser.ASTOrderByClause;
import com.intuit.sb.services.query.parser.ASTPaginationClause;
import com.intuit.sb.services.query.parser.ASTPathElement;
import com.intuit.sb.services.query.parser.ASTPathExpression;
import com.intuit.sb.services.query.parser.ASTSelectClause;
import com.intuit.sb.services.query.parser.ASTSelectStatement;
import com.intuit.sb.services.query.parser.ASTStartPosition;
import com.intuit.sb.services.query.parser.ASTString;
import com.intuit.sb.services.query.parser.ASTStringPathElement;
import com.intuit.sb.services.query.parser.ASTTrue;
import com.intuit.sb.services.query.parser.ASTUUID;
import com.intuit.sb.services.query.parser.ASTWhereClause;
import com.intuit.sb.services.query.parser.SimpleNode;
import com.intuit.sb.services.query.parser.SimpleQueryParserTreeConstants;
import com.intuit.sb.services.query.parser.SimpleQueryParserVisitor;
import com.intuit.sb.services.query.validator.context.FirstClassContext;
import com.intuit.sb.services.query.validator.context.Property;
import com.intuit.sb.services.query.validator.context.RootContext;
import com.intuit.sb.services.query.validator.context.SingleValueProperty;

/**
 * 
 * @author gkesler
 */
public class SimpleQueryGenerator implements SimpleQueryParserVisitor,
		SimpleQueryParserTreeConstants {
	/**
	 * 
	 * @param parser
	 * @param delegate
	 */
	public void generate(SimpleQuery parser,
			SimpleQueryGeneratorDelegate delegate) {
		parser.getRootNode()
				.jjtAccept(
						this,
						parser.getSyntaxMode() == SimpleQuery.SyntaxMode.RESTRICTED ? new RestrictedModeDecorator(
								delegate) : delegate);
	}

	public Object visit(SimpleNode node, Object data) {
		// no-op
		return data;
	}

	public Object visit(ASTSelectStatement node, Object data) {
		final SimpleQueryGeneratorDelegate delegate = toDelegate(data);

		// set context on the generator delegate
		final RootContext defaultContext = (RootContext) node.jjtGetValue();
		if (defaultContext == null) {
			throw new GeneratorException("invalid AST").setProblemNode(node);
		} else if (defaultContext instanceof FirstClassContext) {
			delegate.addRootContext(defaultContext);
		}

		// cascade to children
		return node.childrenAccept(this, data);
	}

	public Object visit(ASTSelectClause node, Object data) {
		// prepare output spec
		final SimpleQueryGeneratorDelegate delegate = toDelegate(data);

		// cascade to children
		node.childrenAccept(this, new PathHolder() {
			public void setPath(Property value) {
				delegate.addToProjection(value);
			}
		});

		return data;
	}

	public Object visit(ASTFromClause node, Object data) {
		final SimpleQueryGeneratorDelegate delegate = toDelegate(data);

		// context has been already set, no-op
		node.childrenAccept(this, new ValueHolder<RootContext>() {
			public void addValue(RootContext value) {
				delegate.addRootContext(value);
			}
		});

		return data;
	}

	public Object visit(ASTWhereClause node, Object data) {
		final SimpleQueryGeneratorDelegate delegate = toDelegate(data);

		// prepare regular filter spec
		final Stack<ExpressionTuple> tuples = (Stack<ExpressionTuple>) node
				.childrenAccept(this, new Stack<ExpressionTuple>());
		// notify delegate here
		ASTUtils.trace("tuples: " + tuples);
		while (!tuples.empty()) {
			delegate.addFilter(tuples.pop());
		}

		// this may require firing open/close events to delegate
		return data;
	}

	public Object visit(ASTOrderByClause node, Object data) {
		// prepare sorting spec
		// this may require firing open/close events to delegate
		return node.childrenAccept(this, data);
	}

	public Object visit(ASTPaginationClause node, Object data) {
		// prepare pagination spec
		// this may require firing open/close events to delegate
		return node.childrenAccept(this, data);
	}

	public Object visit(ASTAnyPathElement node, Object data) {
		return data; // no-op
	}

	public Object visit(ASTStringPathElement node, Object data) {
		return data; // no-op
	}

	public Object visit(ASTIdPathElement node, Object data) {
		return data; // no-op
	}

	public Object visit(ASTPathElement node, Object data) {
		return data; // no-op
	}

	public Object visit(ASTPathExpression node, Object data) {
		return visitPathNode(node, (PathHolder) data);
	}

	public Object visit(ASTContextDeclaration node, Object data) {
		return visitValueNode(node, (ValueHolder<RootContext>) data); // no-op
	}

	public Object visit(ASTContextName node, Object data) {
		// no op
		return data;
	}

	public Object visit(ASTAliasName node, Object data) {
		// no op
		return data;
	}

	public Object visit(ASTOr node, Object data) {
		return visitConditionalNode(node, (Stack<ExpressionTuple>) data,
				new OrTuple());
	}

	public Object visit(ASTAnd node, Object data) {
		return visitConditionalNode(node, (Stack<ExpressionTuple>) data,
				new AndTuple());
	}

	public Object visit(ASTNot node, Object data) {
		return visitConditionalNode(node, (Stack<ExpressionTuple>) data,
				new NotTuple());
	}

	public Object visit(ASTLike node, Object data) {
		return visitFilterNode(node, (Stack<ExpressionTuple>) data,
				new LikeTuple());
	}

	public Object visit(ASTContains node, Object data) {
		return visitFilterNode(node, (Stack<ExpressionTuple>) data,
				new ContainsTuple());
	}

	public Object visit(ASTDistance node, Object data) {
		final ContainsTuple containsTuple = (ContainsTuple) data;

		node.childrenAccept(this, new ValueHolder<Float>() {
			public void addValue(Float value) {
				containsTuple.setDistance(value);
			}
		});

		return data;
	}

	public Object visit(ASTIn node, Object data) {
		return visitFilterNode(node, (Stack<ExpressionTuple>) data,
				new InTuple());
	}

	public Object visit(ASTBetween node, Object data) {
		return visitFilterNode(node, (Stack<ExpressionTuple>) data,
				new BetweenTuple());
	}

	public Object visit(ASTEq node, Object data) {
		return visitFilterNode(node, (Stack<ExpressionTuple>) data,
				new EqTuple());
	}

	public Object visit(ASTNEq node, Object data) {
		return visitFilterNode(node, (Stack<ExpressionTuple>) data,
				new NEqTuple());
	}

	public Object visit(ASTLt node, Object data) {
		return visitFilterNode(node, (Stack<ExpressionTuple>) data,
				new LtTuple());
	}

	public Object visit(ASTLte node, Object data) {
		return visitFilterNode(node, (Stack<ExpressionTuple>) data,
				new LteTuple());
	}

	public Object visit(ASTGt node, Object data) {
		return visitFilterNode(node, (Stack<ExpressionTuple>) data,
				new GtTuple());
	}

	public Object visit(ASTGte node, Object data) {
		return visitFilterNode(node, (Stack<ExpressionTuple>) data,
				new GteTuple());
	}

	public Object visit(ASTAsc node, Object data) {
		final SimpleQueryGeneratorDelegate delegate = toDelegate(data);

		node.childrenAccept(this, new PathHolder() {
			public void setPath(Property value) {
				delegate.addOrderSpec(new AscTuple(value));
			}
		});

		return data;
	}

	public Object visit(ASTDesc node, Object data) {
		final SimpleQueryGeneratorDelegate delegate = toDelegate(data);

		node.childrenAccept(this, new PathHolder() {
			public void setPath(Property value) {
				delegate.addOrderSpec(new DescTuple(value));
			}
		});

		return data;
	}

	public Object visit(ASTStartPosition node, Object data) {
		final SimpleQueryGeneratorDelegate delegate = toDelegate(data);

		node.childrenAccept(this, new ValueHolder<Integer>() {
			public void addValue(Integer value) {
				delegate.addPaginationItem(new StartPositionTuple(value));
			}
		});

		return data;
	}

	public Object visit(ASTMaxResults node, Object data) {
		final SimpleQueryGeneratorDelegate delegate = toDelegate(data);

		node.childrenAccept(this, new ValueHolder<Integer>() {
			public void addValue(Integer value) {
				delegate.addPaginationItem(new MaxResultsTuple(value));
			}
		});

		return data;
	}

	public Object visit(ASTIterator node, Object data) {
		final SimpleQueryGeneratorDelegate delegate = toDelegate(data);

		node.childrenAccept(this, new ValueHolder<UUID>() {
			public void addValue(UUID value) {
				delegate.addPaginationItem(new IteratorTuple(value));
			}
		});

		return data;
	}

	public Object visit(ASTCurrentDate node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	public Object visit(ASTCurrentTimestamp node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	public Object visit(ASTCurrentTime node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	public Object visit(ASTDateTime node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	public Object visit(ASTString node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	public Object visit(ASTTrue node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	public Object visit(ASTFalse node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	public Object visit(ASTMe node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	public Object visit(ASTMyApp node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	public Object visit(ASTNull node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	public Object visit(ASTInteger node, Object data) {
		return visitValueNode(node, (ValueHolder<Integer>) data);
	}

	public Object visit(ASTFloat node, Object data) {
		return visitValueNode(node, (ValueHolder<Float>) data);
	}

	public Object visit(ASTUUID node, Object data) {
		return visitValueNode(node, (ValueHolder<UUID>) data);
	}

	public Object visit(ASTId node, Object data) {
		return visitValueNode(node, (ValueHolder<Object>) data);
	}

	private static final SimpleQueryGeneratorDelegate toDelegate(Object data) {
		return (SimpleQueryGeneratorDelegate) data;
	}

	private final <T extends ConditionalTuple> Object visitConditionalNode(
			SimpleNode node, Stack<ExpressionTuple> tuples, T tuple) {
		// cascade to children
		node.childrenAccept(this, tuples);

		// collect operands and replace them with the
		// passed in node on the stack
		tuples.push(tuple.popChildren(tuples));
		ASTUtils.trace("pushed: " + tuple);

		return tuples;
	}

	private final <T extends FilterTuple> Object visitFilterNode(
			SimpleNode node, Stack<ExpressionTuple> tuples, T tuple) {
		// cascade passed pathHolder to this node children
		node.childrenAccept(this, tuple);

		// and add the constructed pathHolder to the pathHolder list provided
		tuples.push(tuple);
		ASTUtils.trace("pushed: " + tuple);

		return tuples;
	}

	private final <T> Object visitValueNode(SimpleNode node,
			ValueHolder<T> valueHolder) {
		// check if the node has value
		final T value = (T) node.jjtGetValue();
		if (value == null) {
			throw new GeneratorException("value is not set")
					.setProblemNode(node);
		}

		valueHolder.addValue(value);
		ASTUtils.trace("added value: " + value);

		return valueHolder;
	}

	private final Object visitPathNode(ASTPathExpression node,
			PathHolder pathHolder) {
		// this node must be already evaluated from during validation phase
		final Property path = (Property) node.jjtGetValue();
		if (path == null) {
			throw new GeneratorException("path is not evaluated")
					.setProblemNode(node);
		}

		pathHolder.setPath(path);
		ASTUtils.trace("set path: " + path);

		return pathHolder;
	}

	public Object visit(ASTCount node, Object data) {
		PathHolder pathHolder = (PathHolder) data;
		final Property path = new SingleValueProperty(null, node.jjtGetValue()
				.toString(), java.lang.String.class, false, false);
		pathHolder.setPath(path);
		ASTUtils.trace("set path: " + path);
		return pathHolder;
	}
}
