/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.generator.SimpleQueryGeneratorDelegate.java
 * 
 * Created: Aug 16, 2011 3:58:49 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.generator;

import com.intuit.sb.services.query.generator.tuple.ExpressionTuple;
import com.intuit.sb.services.query.generator.tuple.OrderByTuple;
import com.intuit.sb.services.query.generator.tuple.PaginationTuple;
import com.intuit.sb.services.query.validator.context.Property;
import com.intuit.sb.services.query.validator.context.RootContext;

/**
 *
 * @author gkesler
 */
public interface SimpleQueryGeneratorDelegate {
    /**
     * Called by SimpleQueryGenerator before generating anything to
     * set query context on the service delegate
     * 
     * @param context 
     */
    void addRootContext (RootContext context);
    /**
     * Called by SimpleQueryGenerator to specify projection
     * (output specification) to the service delegate
     * 
     * @param tuple set of output properties
     */
    void addToProjection (Property path);
    /**
     * Called by SimpleQueryGenerator to add a WHERE scope
     * filter to the service delegate
     * 
     * @param tuple property-operation-value tuple to be used as filter
     */
    void addFilter (ExpressionTuple tuple);
    /**
     * Called by SimplequeryGenerator to add an ORDER BY expression
     * to the service delegate
     * 
     * @param tuple orderspec-property tuple
     */
    void addOrderSpec (OrderByTuple tuple);
    /**
     * Called by SimplequeryGenerator to add a pagination item to the
     * service delegate
     * 
     * @param tuple itemname-value tuple
     */
    void addPaginationItem (PaginationTuple tuple);
}
