/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.CollectionValueProperty.java
 * 
 * Created: Jul 12, 2011 5:52:00 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;


import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Collection;

/**
 *
 * @author gkesler
 */
public class CollectionValueProperty implements Property {
    protected final Property delegate;
    
    public CollectionValueProperty (Context parent, String propertyName, ParameterizedType genericType, boolean queryable, boolean sortable) {
        Type[] typeArgs = genericType.getActualTypeArguments();
        Class elementType = (Class)typeArgs[0];
        
        // FIXME: two-dimensional collections are not supported
        this.delegate = AbstractContext.createProperty(parent, propertyName, elementType, null, queryable, sortable);
    }
    public CollectionValueProperty (Context parent, String propertyName, Class genericType, boolean queryable, boolean sortable) {               
        // FIXME: two-dimensional collections are not supported
        this.delegate = AbstractContext.createProperty(parent, propertyName, genericType, null, queryable, sortable);
    }

    public Class getType() {
        return delegate.getType();
    }

    public boolean validateValue(Object value) {
        // NULL value is allowed for the collection value property
        // this is an equivalent of empty collection
        return value == null ||
                delegate.validateValue(value);
    }

    public String getName() {
        return delegate.getName();
    }

    public Context[] getFullPath() {
        final Context[] fullPath = delegate.getFullPath();
        // replace last element to be this context
        fullPath[fullPath.length - 1] = this;
        
        return fullPath;
    }

    public RootContext getRootContext() {
        return delegate.getRootContext();
    }

    public Context getParentContext() {
        return delegate.getParentContext();
    }

    public PathBuilder createPathBuilder() {
        return delegate.createPathBuilder();
    }
    
    public Property lookupProperty(String name) {
        return delegate.lookupProperty(name);
    }

    public Property lookupIdProperty(String idValue) {
        return delegate.lookupIdProperty(idValue);
    }

    public Property lookupNameProperty(String nameValue) {
        return delegate.lookupNameProperty(nameValue);
    }

    public Collection<Property> getAllProperties() {
        return delegate.getAllProperties();
    }

    public void setPropertyFilter(PropertyFilter filter) {
        delegate.setPropertyFilter(filter);
    }

    public Collection<Property> getProperties() {
        return delegate.getProperties();
    }

    public <T> T accept(PropertyVisitor<T> visitor, T data) {
        return visitor.visit(this, data);
    }

    @Override
    public String toString() {
        return getClass().getSimpleName() + "{" + "delegate=" + delegate + '}';
    }

    public boolean isQueryable() {
        return delegate.isQueryable();
    }

    public MetadataProvider getMetadataProvider() {
        return delegate.getMetadataProvider();
    }
	public boolean isSortable() {
		return delegate.isSortable();
	}
}
