/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.validator.context.CustomFieldProperty.java
 * 
 * Created: Aug 13, 2011 3:31:14 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;

/**
 *
 * @author gkesler
 */
public class CustomFieldProperty extends PersistentProperty {
    public static final String CONTEXT_NAME = "CustomField";
    public static final String ID_PROPERTY = "definitionId";
    public static final String NAME_PROPERTY = "name";
    public static final String VALUE_PROPERTY = "value";
    
    private final Property valueProperty;
    
    public CustomFieldProperty (Context parent, String propertyName, Class propertyType, boolean queryable, boolean sortable) {
        super(parent, propertyName, propertyType, queryable, sortable);
        
        // add value property
        Property property = metadataProvider.lookupProperty(VALUE_PROPERTY);
        if (property == null) {
            property = AbstractContext.createSingleProperty(this, VALUE_PROPERTY, Object.class, queryable, sortable);
            metadataProvider.getAllProperties().put(property.getName(), property);
        }
        this.valueProperty = property;
    }

    @Override
    public boolean isQueryable() {
        return valueProperty.isQueryable();
    }


    @Override
    public boolean isSortable() {
        return valueProperty.isSortable();
    }
    
    @Override
    public Property lookupIdProperty(String idValue) {
        return createProperty(idValue, lookupProperty(ID_PROPERTY));
    }

    @Override
    public Property lookupNameProperty(String nameValue) {
        return createProperty(nameValue, lookupProperty(NAME_PROPERTY));
    }
    
    private final Property createProperty (String name, Property keyProperty) {
        return AbstractContext.createCompositeProperty(this, name, keyProperty, valueProperty, isQueryable(), isSortable());
    }

    @Override
    public <T> T accept(PropertyVisitor<T> visitor, T data) {
        return visitor.visit(this, data);
    }
}
