/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.FirstClassContext.java
 * 
 * Created: Jul 13, 2011 1:01:26 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;

import java.util.Map;

/**
 *
 * @author gkesler
 */
public class FirstClassContext extends AbstractContext implements RootContext {
    public static final String CDM_V2_PACKAGE = "com.intuit.sb.cdm.v2";
    
    private final Map<String, Object> properties;

    public FirstClassContext (String contextName, Map<String, Object> properties, MetadataProviderFactory metadataProviderFactory) {
        super(null, getContextClass(contextName, properties), metadataProviderFactory);
        
        this.properties = properties;
    }

    @Override
    public RootContext getRootContext() {
        return this;
    }

    @Override
    public Object getContextProperty(String propName) {
        return properties.get(propName);
    }

    public Property lookupIdProperty(String idValue) {
        throw generateException(idValue);
    }

    public Property lookupNameProperty(String nameValue) {
        throw generateException(nameValue);
    }

    public Class getContextClass(String contextName) {
        return getContextClass(contextName, properties);
    }
    
    public static final Class getContextClass (String contextName, Map<String, Object> properties) {
        try {
            String contextPrefix = (String)properties.get(ContextFactory.CONTEXT_PREFIX);
            if (contextPrefix == null) {
                contextPrefix = CDM_V2_PACKAGE;
            }
        
            return Class.forName(contextPrefix + "." + contextName);
        } catch (ClassNotFoundException e) {
            throw new InvalidContextException("invalid context declaration: " + contextName, e);
        }
    }
}
