/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.validator.context.ContextInfo.java
 * 
 * Created: Aug 15, 2011 4:06:08 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;

import java.util.Map;

/**
 *
 * @author gkesler
 */
public interface MetadataProvider {
    /**
     * Obtains context name
     * 
     * @return context system name
     */
    String getName ();
    /**
     * Obtains context type
     * 
     * @return context system type
     */
    Class getType ();
    /**
     * Looks up the property in the current context
     * 
     * @return property
     */
    Property lookupProperty (String name);
    /**
     * Obtains all properties defined for this context
     * 
     * @return collection of properties
     */
    Map<String, Property> getAllProperties ();
    /**
     * Creates a path builder instance that evaluates (looks up) the path expression.
     * The result will contain either single-valued property descriptor
     * or a Context object in case of collection-valued one
     * 
     * @return path builder
     */
    PathBuilder createPathBuilder ();
    /**
     * Returns ContextInfoFactory that created this context
     * 
     * @return ContextInfoFactory for this ContextInfo
     */
    MetadataProviderFactory getFactory ();
    /**
     * Initializes MetadataProvider after its creation
     * 
     */
    void init ();
}
