/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.validator.context.WildcardProperty.java
 * 
 * Created: Nov 17, 2011 10:48:56 AM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;

import com.intuit.sb.services.query.validator.ValidatorException;
import java.util.Collection;

/**
 *
 * @author gkesler
 */
public class WildcardProperty implements Property {
    private final Context parent;
    private final String contextName;

    public WildcardProperty (Context parent, String contextName) {
        this.parent = parent;
        this.contextName = contextName;
        
        parent.setPropertyFilter(null);
    }
    
    public Class getType() {
        throw generateException("type");
    }

    public boolean validateValue(Object value) {
        throw generateException("validate value=" + value);
    }

    public boolean isQueryable() {
        throw generateException("queryable");
    }

    public boolean isSortable() {
        throw generateException("sortable");
    }
    
    public <T> T accept(PropertyVisitor<T> visitor, T data) {
        return visitor.visit(this, data);
    }

    public String getName() {
        return contextName;
    }

    public Context[] getFullPath() {
        final Context[] parentPath = parent.getFullPath();
        final Context[] path = new Context[parentPath.length + 1];
        
        System.arraycopy(parentPath, 0, path, 0, parentPath.length);
        path[parentPath.length] = this;
        
        return path;
    }

    public RootContext getRootContext() {
        return parent.getRootContext();
    }

    public Context getParentContext() {
        return parent;
    }

    public PathBuilder createPathBuilder() {
        throw generateException("path");
    }

    public Property lookupProperty(String name) {
        throw generateException(name);
    }

    public Property lookupIdProperty(String idValue) {
        throw generateException(idValue);
    }

    public Property lookupNameProperty(String nameValue) {
        throw generateException(nameValue);
    }

    public Collection<Property> getAllProperties() {
        return parent.getAllProperties();
    }

    public void setPropertyFilter(PropertyFilter filter) {
        parent.setPropertyFilter(filter);
    }

    public Collection<Property> getProperties() {
        return parent.getProperties();
    }

    public MetadataProvider getMetadataProvider() {
        return parent.getMetadataProvider();
    }
    
    protected ValidatorException generateException (String expression) {
        return new ValidatorException("not valid expression:" + expression + " for projection context");
    }

    @Override
    public String toString() {
        return getClass().getSimpleName() + "{" + "parent=" + parent + ", contextName=" + contextName + '}';
    }
}
