/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.SimpleQueryGeneratorTest.java
 * 
 * Created: Aug 17, 2011 10:41:28 AM
 * Author: gkesler
 */
package com.intuit.sb.services.query;

import java.util.HashMap;
import java.util.Map;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.intuit.sb.services.query.generator.SimpleQueryGeneratorDelegate;
import com.intuit.sb.services.query.generator.tuple.ExpressionTuple;
import com.intuit.sb.services.query.generator.tuple.OrderByTuple;
import com.intuit.sb.services.query.generator.tuple.PaginationTuple;
import com.intuit.sb.services.query.validator.context.ContextFactory;
import com.intuit.sb.services.query.validator.context.FirstClassContext;
import com.intuit.sb.services.query.validator.context.Property;
import com.intuit.sb.services.query.validator.context.RootContext;

/**
 *
 * @author gkesler
 */
public class SimpleQueryGeneratorTest {
    
    public SimpleQueryGeneratorTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }
    
    
    private Map<String, Object> createProperties() {
        return new HashMap<String, Object>() {
            {
                put(ContextFactory.CONTEXT_USER_ID, System.getProperty("user.name"));
                put(ContextFactory.CONTEXT_APP_ID, "SimpleQuery-console");
                put(ContextFactory.CONTEXT_PREFIX, "com.intuit.schema.finance.v3");
                put(ContextFactory.PAGE_MAX_LIMIT, Integer.valueOf(1024));
            }
        };
    }

    private final void generatorTest (String resourceName, SimpleQuery.SyntaxMode syntaxMode) throws Exception {
        generatorTest(resourceName, syntaxMode, new DumpGeneratorDelegate());
    }
    
    private final void generatorTest (String resourceName, 
    		SimpleQuery.SyntaxMode syntaxMode,
    		SimpleQueryGeneratorDelegate delegate) throws Exception {
        DumpVisitor dumper = new DumpVisitor();

        System.out.println("resourceName=" + resourceName);
        SimpleQuery parser = Utils.parse(resourceName, createProperties(), syntaxMode);        
        dumper.dumpTree(parser.getRootNode(), "parsed -->");

        parser.validate();        
        dumper.dumpTree(parser.getRootNode(), "validated -->");
        
        parser.generate(delegate);
    }
    
    // TODO add test methods here.
    // The methods must be annotated with annotation @Test. For example:

    @Test
    public void testAggregateGenerator () throws Exception {
        System.out.println("testAggregateGenerator");
        generatorTest("/aggregate.sq", SimpleQuery.SyntaxMode.EXTENDED);
    }   
    
    @Test
    public void testBasicGenerator () throws Exception {
        System.out.println("testBasicGenerator");
        generatorTest("/basicCustomerQuery.sq", SimpleQuery.SyntaxMode.RESTRICTED);
    }  
    
    class SimpleQueryGeneratorDelegateTestHelper implements SimpleQueryGeneratorDelegate {
    	public RootContext context = null;

		public void addRootContext(RootContext context) {
			this.context = context;
			
		}

		public void addToProjection(Property path) {
			if (context == null) {
				throw new RuntimeException("Failed");
			}
			
		}

		public void addFilter(ExpressionTuple tuple) {
			// TODO Auto-generated method stub
			
		}

		public void addOrderSpec(OrderByTuple tuple) {
			// TODO Auto-generated method stub
			
		}

		public void addPaginationItem(PaginationTuple tuple) {
			// TODO Auto-generated method stub
			
		}
    	
    }
        
    @Test
    public void testDelegateHandling () throws Exception {
        System.out.println("testDelegateHandling");
        SimpleQueryGeneratorDelegateTestHelper delegate = new SimpleQueryGeneratorDelegateTestHelper();
        generatorTest("/basicAlias.sq", SimpleQuery.SyntaxMode.RESTRICTED, delegate);
        Assert.assertTrue("Context should have been set on the delegate", delegate.context instanceof FirstClassContext);
    }    
        
    @Test
    public void testValidator () throws Exception {
        System.out.println("testValidator");
        generatorTest("/customerQuery.sq", SimpleQuery.SyntaxMode.RESTRICTED);
    }
        
    @Test
    public void testValidatorWithCustomFields () throws Exception {
        System.out.println("testValidatorWithCustomFields");
        generatorTest("/customerQueryWithCustomFields.sq", SimpleQuery.SyntaxMode.RESTRICTED);
    }
        
//    @Test
//    public void testSelectFromMultipleContextsWithOr () throws Exception {
//        System.out.println("testValidatorWithCustomFields");
//        generatorTest("/search/positive/selectFromMultipleContextsWithOr.sq", SimpleQuery.SyntaxMode.EXTENDED);
//    }
}
