/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.Utils.java
 * 
 * Created: Jun 30, 2011 11:39:07 AM
 * Author: gkesler
 */
package com.intuit.sb.services.query;

import java.io.File;
import java.io.FileFilter;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 *
 * @author gkesler
 */
public class Utils {
    public static List<Object[]> findFiles (File startDir, String ext) throws Exception {
        final String suffix = ext.toLowerCase();
        final List<Object[]> positiveQueries = new ArrayList<Object[]>();
        startDir.listFiles(new FileFilter() {
            public boolean accept(File file) {
                if (file.isDirectory()) {
                    file.listFiles(this);
                } else if (file.getName().toLowerCase().endsWith(suffix)) {
                    positiveQueries.add(new Object[]{file});
                    System.out.println("added '" + file + "'");
                }

                return false;
            }
        });
        return positiveQueries;
    }
    
    private static SimpleQuery parse(String resourceName, Map<String, Object> properties, SimpleQuery.SyntaxMode syntaxMode, SimpleQueryCtor ctor) throws Exception {
        URL resource = SimpleQueryTest.class.getResource(resourceName);
        SimpleQuery parser = ctor.create(resource.openStream(), properties);
        parser.setSyntaxMode(syntaxMode);
        parser.parse();
        return parser;
    }
    
    private static SimpleQuery parse(String resourceName, Map<String, Object> properties, SimpleQueryCtor ctor) throws Exception {
        return parse(resourceName, properties, null, ctor);
    }
    
    public static SimpleQuery parse(String resourceName, Map<String, Object> properties, SimpleQuery.SyntaxMode syntaxMode) throws Exception {
        return parse(resourceName, properties, syntaxMode, customPropsCtor);
    }
    
    public static SimpleQuery parse(String resourceName, Map<String, Object> properties) throws Exception {
        return parse(resourceName, properties, customPropsCtor);
    }
    
    public static SimpleQuery parse(String resourceName, SimpleQuery.SyntaxMode syntaxMode) throws Exception {
        return parse(resourceName, null, syntaxMode, systemPropsCtor);
    }
    
    public static SimpleQuery parse(String resourceName) throws Exception {
        return parse(resourceName, null, systemPropsCtor);
    }
    
    private interface SimpleQueryCtor {
        SimpleQuery create (InputStream stream, Map<String, Object> properties);
    }
    
    private static final SimpleQueryCtor customPropsCtor = new SimpleQueryCtor() {
        public SimpleQuery create(InputStream stream, Map<String, Object> properties) {
            return new SimpleQueryContext(properties).newSimpleQuery(stream);
        }
    };
    
    private static final SimpleQueryCtor systemPropsCtor = new SimpleQueryCtor() {
        public SimpleQuery create(InputStream stream, Map<String, Object> properties) {
            return new SimpleQueryContext().newSimpleQuery(stream);
        }
    };
}
