/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.integration.IntegrationContextInfo.java
 * 
 * Created: Aug 17, 2011 11:41:53 PM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.integration.ids;

import com.intuit.dataservices.qb.query.sqlprojection.EntityMetadata;
import com.intuit.dataservices.qb.query.sqlprojection.PropertyMetadata;
import com.intuit.dataservices.qb.query.sqlprojection.registry.AllEntityMetadata;
import com.intuit.sb.services.query.parser.ASTPathExpression;
import com.intuit.sb.services.query.validator.ValidatorException;
import com.intuit.sb.services.query.validator.context.AbstractContext;
import com.intuit.sb.services.query.validator.context.Context;
import com.intuit.sb.services.query.validator.context.CustomFieldProperty;
import com.intuit.sb.services.query.validator.context.InvalidContextException;
import com.intuit.sb.services.query.validator.context.MetadataProvider;
import com.intuit.sb.services.query.validator.context.MetadataProviderFactory;
import com.intuit.sb.services.query.validator.context.PathBuilder;
import com.intuit.sb.services.query.validator.context.Property;
import com.intuit.sb.services.query.validator.context.StringPathBuilder;
import java.util.Map;

/**
 *
 * @author gkesler
 */
public class IntegrationMetadataProvider implements MetadataProvider {
    private final static AllEntityMetadata allMetadata = new AllEntityMetadata();

    private final String contextName;
    private final Class contextType;
    private final Context context;
    private /*final*/ EntityMetadata entityMetadata;
    private final MetadataProviderFactory factory;
    
    public IntegrationMetadataProvider (MetadataProviderFactory factory, Context context, Class contextType) {
        this.contextName = contextType.getSimpleName();
        this.contextType = contextType;
        this.context = context;
        this.factory = factory;
    }
    
    public void init () {
        entityMetadata = allMetadata.lookupEntityMetadata(context.getRootContext().getName());
        if (entityMetadata == null) {
            // could not construct
            throw new InvalidContextException("could not load EntityMetadata for: " + contextName);
        }
    }
    
    public String getName() {
        return contextName;
    }

    public Class getType() {
        return contextType;
    }

    public Property lookupProperty(String name) {        
        final StringBuilder nameBuilder = new StringBuilder();
        for (Context pathElem: context.getFullPath()) {
            nameBuilder.append(pathElem.getName());
            nameBuilder.append("/");
        }
        nameBuilder.append(name);
        
        name = nameBuilder.toString();
        PropertyMetadata metadata = entityMetadata.lookupPropertyMetadata(name.toLowerCase());
        if (metadata == null) {
            throw new ValidatorException("invalid path: " + name);
        }

        // FIXME: since readMethod is not provided,
        // this will fail on a Collection type property
        return AbstractContext.createProperty(context, name, metadata.getType(), null, metadata.isQueryable(), true);
    }

    public Map<String, Property> getAllProperties() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    public PathBuilder createPathBuilder() {
        return new StringPathBuilder("/") {
            Context currentContext = context;
            EvaluateStrategy strategy = evaluatePath;

            public void addAnyPathElement() {
                currentContext = AbstractContext.createWildcardProperty(currentContext, builder.toString());
            }

            public void addIdPathElement(String idValue) {
                currentContext = getCustomFieldContext().lookupIdProperty(idValue);
                strategy = evaluateComposite;
            }

            public void addNamePathElement(String nameValue) {
                currentContext = getCustomFieldContext().lookupNameProperty(nameValue);
                strategy = evaluateComposite;
            }

            @Override
            public void addPathElement(String name) {
                super.addPathElement(name);
                strategy = evaluatePath;
            }
            
            private final Context getCustomFieldContext () {
                addPathElement(CustomFieldProperty.CONTEXT_NAME);
                return AbstractContext.createPersistentProperty(currentContext, builder.toString(), 
                        currentContext.getRootContext().getContextClass(CustomFieldProperty.CONTEXT_NAME), true, true);
            }

            public Property evaluate(ASTPathExpression path) {
                return strategy.evaluate(currentContext, builder.toString());
            }
        };
    }

    public MetadataProviderFactory getFactory() {
        return factory;
    }
        
    private interface EvaluateStrategy {
        Property evaluate (Context context, String path);
    }
    
    private static final EvaluateStrategy evaluatePath = new EvaluateStrategy() {
        public Property evaluate(Context context, String path) {
            return context.lookupProperty(path);
        }
    };
    
    private static final EvaluateStrategy evaluateComposite = new EvaluateStrategy() {
        public Property evaluate(Context context, String path) {
            return (Property)context;
        }
    };
}
