/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.DumpVisitor.java
 * 
 * Created: Aug 18, 2011 2:29:17 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query;

import com.intuit.sb.services.query.parser.*;

/**
 *
 * @author gkesler
 */
public class DumpVisitor implements SimpleQueryParserVisitor {
    
    public static void dump (Node node, String prefix) {
        new DumpVisitor().dumpTree(node, prefix);
    }
    
    public void dumpTree (Node node, String prefix) {
        node.jjtAccept(new DepthFirstDecorator(this), prefix);
    }
    
    private <T extends SimpleNode> Object dumpNode (T node, Object data) {
        String prefix = (String)data;
        System.out.print(node.toString(prefix));
        Object value = node.jjtGetValue();
        if (value != null) {
            System.out.print(": value=" + value);
        }
        System.out.println();
        return prefix + " ";
    }
    
    public Object visit(SimpleNode node, Object data) {
        // no-op
        return data;
    }

    public Object visit(ASTSelectStatement node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTSelectClause node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTFromClause node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTWhereClause node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTOrderByClause node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTPaginationClause node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTAnyPathElement node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTPathExpression node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTContextDeclaration node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTContextName node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTAliasName node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTOr node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTAnd node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTNot node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTTrue node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTEq node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTLike node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTContains node, Object data) {
        return dumpNode(node, data);
    }
    
    public Object visit(ASTDistance node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTIn node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTBetween node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTNEq node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTLt node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTLte node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTGt node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTGte node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTAsc node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTDesc node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTStartPosition node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTMaxResults node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTIterator node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTStringPathElement node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTIdPathElement node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTPathElement node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTCurrentDate node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTCurrentTimestamp node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTCurrentTime node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTDateTime node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTString node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTFalse node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTMe node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTMyApp node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTNull node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTInteger node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTFloat node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTUUID node, Object data) {
        return dumpNode(node, data);
    }

    public Object visit(ASTId node, Object data) {
        return dumpNode(node, data);
    }
    public Object visit(ASTCount node, Object data) {
        return dumpNode(node, data);
    }
}
