/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.datasync.query.rest.SimpleQueryResource.java
 * 
 * Created: Nov 29, 2011 1:45:46 PM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.ws.rs;

import com.intuit.sb.services.query.DepthFirstDecorator;
import com.intuit.sb.services.query.SimpleQuery;
import com.intuit.sb.services.query.parser.ASTAliasName;
import com.intuit.sb.services.query.parser.ASTAnd;
import com.intuit.sb.services.query.parser.ASTAnyPathElement;
import com.intuit.sb.services.query.parser.ASTAsc;
import com.intuit.sb.services.query.parser.ASTBetween;
import com.intuit.sb.services.query.parser.ASTContains;
import com.intuit.sb.services.query.parser.ASTCount;
import com.intuit.sb.services.query.parser.ASTContextDeclaration;
import com.intuit.sb.services.query.parser.ASTContextName;
import com.intuit.sb.services.query.parser.ASTCurrentDate;
import com.intuit.sb.services.query.parser.ASTCurrentTime;
import com.intuit.sb.services.query.parser.ASTCurrentTimestamp;
import com.intuit.sb.services.query.parser.ASTDateTime;
import com.intuit.sb.services.query.parser.ASTDesc;
import com.intuit.sb.services.query.parser.ASTDistance;
import com.intuit.sb.services.query.parser.ASTEq;
import com.intuit.sb.services.query.parser.ASTFalse;
import com.intuit.sb.services.query.parser.ASTFloat;
import com.intuit.sb.services.query.parser.ASTFromClause;
import com.intuit.sb.services.query.parser.ASTGt;
import com.intuit.sb.services.query.parser.ASTGte;
import com.intuit.sb.services.query.parser.ASTId;
import com.intuit.sb.services.query.parser.ASTIdPathElement;
import com.intuit.sb.services.query.parser.ASTIn;
import com.intuit.sb.services.query.parser.ASTInteger;
import com.intuit.sb.services.query.parser.ASTIterator;
import com.intuit.sb.services.query.parser.ASTLike;
import com.intuit.sb.services.query.parser.ASTMaxResults;
import com.intuit.sb.services.query.parser.ASTLt;
import com.intuit.sb.services.query.parser.ASTLte;
import com.intuit.sb.services.query.parser.ASTMe;
import com.intuit.sb.services.query.parser.ASTMyApp;
import com.intuit.sb.services.query.parser.ASTNEq;
import com.intuit.sb.services.query.parser.ASTNot;
import com.intuit.sb.services.query.parser.ASTNull;
import com.intuit.sb.services.query.parser.ASTOr;
import com.intuit.sb.services.query.parser.ASTOrderByClause;
import com.intuit.sb.services.query.parser.ASTPaginationClause;
import com.intuit.sb.services.query.parser.ASTPathElement;
import com.intuit.sb.services.query.parser.ASTPathExpression;
import com.intuit.sb.services.query.parser.ASTSelectClause;
import com.intuit.sb.services.query.parser.ASTSelectStatement;
import com.intuit.sb.services.query.parser.ASTStartPosition;
import com.intuit.sb.services.query.parser.ASTString;
import com.intuit.sb.services.query.parser.ASTStringPathElement;
import com.intuit.sb.services.query.parser.ASTTrue;
import com.intuit.sb.services.query.parser.ASTUUID;
import com.intuit.sb.services.query.parser.ASTWhereClause;
import com.intuit.sb.services.query.parser.SimpleNode;
import com.intuit.sb.services.query.parser.SimpleQueryParserVisitor;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import org.apache.log4j.Logger;

/**
 *
 * @author gkesler
 */
@Path("/v2/my-realm-id")
public class SimpleQueryResource {
    private static final Logger logger = Logger.getLogger(SimpleQueryResource.class);
    
    @POST
    @Consumes(MediaType.TEXT_PLAIN)
    @Produces(MediaType.TEXT_PLAIN)
    @Path("/invoice")
    public String postInvoiceQuery (@SyntaxMode(SimpleQuery.SyntaxMode.RESTRICTED) SimpleQuery query) {
        logger.info("got query:" + query);
        SimpleQueryLogger sqLogger = new SimpleQueryLogger();
        query.getRootNode().jjtAccept(new DepthFirstDecorator(sqLogger), "-->");
        logger.info(sqLogger);
        
        return sqLogger.toString();
    }
    
    @GET
    @Produces(MediaType.TEXT_PLAIN)
    @Path("/invoice")
    public String getInvoiceQuery (@QueryParam("q") @com.intuit.sb.datasync.query.ws.rs.resteasy.SyntaxMode(SimpleQuery.SyntaxMode.EXTENDED) SimpleQuery query) {
        logger.info("got query:" + query);
        SimpleQueryLogger sqLogger = new SimpleQueryLogger();
        query.getRootNode().jjtAccept(new DepthFirstDecorator(sqLogger), "++>");
        logger.info(sqLogger);
        
        return sqLogger.toString();
    }
    
    private static class SimpleQueryLogger implements SimpleQueryParserVisitor {
        StringBuilder builder = new StringBuilder();
        
        private Object dumpNode (SimpleNode node, Object data) {
            String prefix = (String)data;
            builder.append(node.toString(prefix));
            Object value = node.jjtGetValue();
            if (value != null) {
                builder.append(": value=" + String.valueOf(value));
            }
            builder.append("\n");
            return prefix + ' ';
        }

        public Object visit(SimpleNode node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTSelectStatement node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTCount node, Object data) {
            return dumpNode(node, data);
        }
        public Object visit(ASTSelectClause node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTFromClause node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTWhereClause node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTOrderByClause node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTPaginationClause node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTAnyPathElement node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTPathExpression node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTContextDeclaration node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTAliasName node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTContextName node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTOr node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTAnd node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTNot node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTTrue node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTEq node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTLike node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTContains node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTFloat node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTDistance node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTIn node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTBetween node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTNEq node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTLt node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTLte node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTGt node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTGte node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTAsc node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTDesc node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTStartPosition node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTMaxResults node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTIterator node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTPathElement node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTStringPathElement node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTIdPathElement node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTCurrentDate node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTCurrentTimestamp node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTCurrentTime node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTDateTime node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTString node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTId node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTFalse node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTMe node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTMyApp node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTNull node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTInteger node, Object data) {
            return dumpNode(node, data);
        }

        public Object visit(ASTUUID node, Object data) {
            return dumpNode(node, data);
        }

        @Override
        public String toString() {
            return builder.toString();
        }
    }
}
